/* Generated by Cython 3.2.9 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/build/nvtx/src/nvtx/c/include/nvtx3/nvToolsExt.h",
            "/build/nvtx/src/nvtx/c/include/nvtx3/nvToolsExtCounters.h",
            "/build/nvtx/src/nvtx/c/include/nvtx3/nvToolsExtPayload.h",
            "/build/nvtx/src/nvtx/c/include/nvtx3/nvToolsExtSemanticsCounters.h",
            "/build/nvtx/src/nvtx/c/include/nvtxw3/nvtxw3.h",
            "/build/nvtx/src/nvtx/c/include/nvtxw3/nvtxw3_counter_helpers.h",
            "/build/nvtx/src/nvtx/c/include/nvtxw3/nvtxw3_event_helpers.h"
        ],
        "include_dirs": [
            "/build/nvtx/src/nvtx/c/include"
        ],
        "name": "nvtx._lib.writer",
        "sources": [
            "src/nvtx/_lib/writer.pyx"
        ]
    },
    "module_name": "nvtx._lib.writer"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API)
  #if !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef _MSC_VER
  #pragma message ("Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.")
  #else
  #warning Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_2_9"
#define CYTHON_HEX_VERSION 0x030209F0
#define CYTHON_FUTURE_DIVISION 1
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #ifndef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 1
  #endif
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
  #if defined(CYTHON_IMMORTAL_CONSTANTS) && PY_VERSION_HEX < 0x030C0000
    #undef CYTHON_IMMORTAL_CONSTANTS
    #define CYTHON_IMMORTAL_CONSTANTS 0  // definitely won't work
  #elif !defined(CYTHON_IMMORTAL_CONSTANTS)
    #define CYTHON_IMMORTAL_CONSTANTS (PY_VERSION_HEX >= 0x030C0000 && !CYTHON_USE_MODULE_STATE && CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_COMPRESS_STRINGS
  #define CYTHON_COMPRESS_STRINGS 1
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL)
#endif
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#include <stdint.h>
typedef uintptr_t  __pyx_uintptr_t;
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CInitCode */
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef Py_TPFLAGS_IMMUTABLETYPE
  #define Py_TPFLAGS_IMMUTABLETYPE (1UL << 8)
#endif
#ifndef Py_TPFLAGS_DISALLOW_INSTANTIATION
  #define Py_TPFLAGS_DISALLOW_INSTANTIATION (1UL << 7)
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if PY_VERSION_HEX < 0x03090000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) GraalPyFrame_SetLineNumber((frame), (lineno))
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && PyUnstable_Object_IsUniquelyReferenced(obj)) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#elif CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
enum __Pyx_ReferenceSharing {
  __Pyx_ReferenceSharing_DefinitelyUnique, // We created it so we know it's unshared - no need to check
  __Pyx_ReferenceSharing_OwnStrongReference,
  __Pyx_ReferenceSharing_FunctionArgument,
  __Pyx_ReferenceSharing_SharedReference, // Never trust it to be unshared because it's a global or similar
};
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && PY_VERSION_HEX >= 0x030E0000
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing)\
    (sharing == __Pyx_ReferenceSharing_DefinitelyUnique ? 1 :\
      (sharing == __Pyx_ReferenceSharing_FunctionArgument ? PyUnstable_Object_IsUniqueReferencedTemporary(o) :\
      (sharing == __Pyx_ReferenceSharing_OwnStrongReference ? PyUnstable_Object_IsUniquelyReferenced(o) : 0)))
#elif (CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING) || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)sharing), Py_REFCNT(o) == 1)
#else
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)o), ((void)sharing), 0)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyList_GetItemRef(o, i) (likely((i) >= 0) ? PySequence_GetItem(o, i) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_ITEM(o, i)
  #endif
#elif CYTHON_COMPILING_IN_LIMITED_API || !(CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE)
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_XNewRef(PyList_GetItem(o, i))
#else
  #define __Pyx_PyList_GetItemRef(o, i) (likely(__Pyx_is_valid_index(i, PyList_GET_SIZE(o))) ?\
    __Pyx_NewRef(PyList_GET_ITEM(o, i)) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
#endif
#if CYTHON_AVOID_BORROWED_REFS || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared)  ((void)(unsafe_shared),\
      __Pyx_PyList_GetItemRef(o, i))
#elif CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared) (\
      __Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
      __Pyx_NewRef(PyList_GET_ITEM(o, i)) : __Pyx_PyList_GetItemRef(o, i))
  #else
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared) (\
      __Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
      __Pyx_XNewRef(PyList_GetItem(o, i)) : __Pyx_PyList_GetItemRef(o, i))
  #endif
#elif CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared)  ((void)(unsafe_shared),\
      __Pyx_NewRef(PyList_GET_ITEM(o, i)))
#else
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared)  ((void)(unsafe_shared),\
      __Pyx_XNewRef(PyList_GetItem(o, i)))
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__nvtx___lib__writer
#define __PYX_HAVE_API__nvtx___lib__writer
/* Early includes */
#include <stddef.h>
#include <stdint.h>
#include "nvtx3/nvToolsExt.h"
#include "nvtx3/nvToolsExtPayload.h"
#include "nvtx3/nvToolsExtCounters.h"
#include "nvtx3/nvToolsExtSemanticsCounters.h"
#include "nvtxw3/nvtxw3.h"
#include "nvtxw3/nvtxw3_counter_helpers.h"
#include "nvtxw3/nvtxw3_event_helpers.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifdef CYTHON_FREETHREADING_COMPATIBLE
#if CYTHON_FREETHREADING_COMPATIBLE
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_USED
#endif
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#endif
#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#if __cplusplus > 201103L
#include <type_traits>
#endif
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "src/nvtx/_lib/writer.pyx",
  "(tree fragment)",
  "src/nvtx/_lib/lib.pxd",
  "src/nvtx/_lib/counters.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* Atomics.proto (used by UnpackUnboundCMethod) */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_nonatomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_nonatomic_ptr_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #define __pyx_nonatomic_ptr_type void*
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_atomic_ptr_type old = _InterlockedCompareExchangePointer(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif

/* CriticalSectionsDefinition.proto (used by CriticalSections) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_End(cs)
#define __Pyx_PyCriticalSection2_End(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_End PyCriticalSection_End
#define __Pyx_PyCriticalSection2_End PyCriticalSection2_End
#endif

/* CriticalSections.proto (used by ParseKeywordsImpl) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection_Begin(cs, arg) (void)(cs)
#define __Pyx_PyCriticalSection2_Begin(cs, arg1, arg2) (void)(cs)
#else
#define __Pyx_PyCriticalSection_Begin PyCriticalSection_Begin
#define __Pyx_PyCriticalSection2_Begin PyCriticalSection2_Begin
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* IncludeStructmemberH.proto (used by FixUpExtensionType) */
#include <structmember.h>

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes;
struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar;
struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle;
struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle;
struct __pyx_obj_4nvtx_4_lib_8counters_Counter;
struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule;
struct __pyx_obj_4nvtx_4_lib_6writer_Backend;
struct __pyx_obj_4nvtx_4_lib_6writer_Session;
struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString;
struct __pyx_obj_4nvtx_4_lib_6writer_Scope;
struct __pyx_obj_4nvtx_4_lib_6writer_Domain;
struct __pyx_obj_4nvtx_4_lib_6writer_Counter;
struct __pyx_obj_4nvtx_4_lib_6writer_Schema;
struct __pyx_obj_4nvtx_4_lib_6writer_Stream;
struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar;

/* "nvtx/_lib/writer.pxd":437
 * # (single timestamp, begin/end span), so the Stream._emit_* helpers can
 * # dispatch to a function pair instead of duplicating each call.
 * ctypedef nvtxwResultCode_t (*_at_bin_fn)(             # <<<<<<<<<<<<<<
 *     const nvtxwEventWriter_t*, int64_t, nvtxwEventAttributes_t)
 * ctypedef nvtxwResultCode_t (*_at_utf8_fn)(
*/
typedef nvtxwResultCode_t (*__pyx_t_4nvtx_4_lib_6writer__at_bin_fn)(nvtxwEventWriter_t const *, int64_t, nvtxwEventAttributes_t);

/* "nvtx/_lib/writer.pxd":439
 * ctypedef nvtxwResultCode_t (*_at_bin_fn)(
 *     const nvtxwEventWriter_t*, int64_t, nvtxwEventAttributes_t)
 * ctypedef nvtxwResultCode_t (*_at_utf8_fn)(             # <<<<<<<<<<<<<<
 *     const nvtxwEventWriter_t*, int64_t, nvtxwEventAttributesUtf8_t)
 * ctypedef nvtxwResultCode_t (*_span_bin_fn)(
*/
typedef nvtxwResultCode_t (*__pyx_t_4nvtx_4_lib_6writer__at_utf8_fn)(nvtxwEventWriter_t const *, int64_t, nvtxwEventAttributesUtf8_t);

/* "nvtx/_lib/writer.pxd":441
 * ctypedef nvtxwResultCode_t (*_at_utf8_fn)(
 *     const nvtxwEventWriter_t*, int64_t, nvtxwEventAttributesUtf8_t)
 * ctypedef nvtxwResultCode_t (*_span_bin_fn)(             # <<<<<<<<<<<<<<
 *     const nvtxwEventWriter_t*, int64_t, int64_t, nvtxwEventAttributes_t)
 * ctypedef nvtxwResultCode_t (*_span_utf8_fn)(
*/
typedef nvtxwResultCode_t (*__pyx_t_4nvtx_4_lib_6writer__span_bin_fn)(nvtxwEventWriter_t const *, int64_t, int64_t, nvtxwEventAttributes_t);

/* "nvtx/_lib/writer.pxd":443
 * ctypedef nvtxwResultCode_t (*_span_bin_fn)(
 *     const nvtxwEventWriter_t*, int64_t, int64_t, nvtxwEventAttributes_t)
 * ctypedef nvtxwResultCode_t (*_span_utf8_fn)(             # <<<<<<<<<<<<<<
 *     const nvtxwEventWriter_t*, int64_t, int64_t, nvtxwEventAttributesUtf8_t)
 * 
*/
typedef nvtxwResultCode_t (*__pyx_t_4nvtx_4_lib_6writer__span_utf8_fn)(nvtxwEventWriter_t const *, int64_t, int64_t, nvtxwEventAttributesUtf8_t);

/* "nvtx/_lib/lib.pxd":250
 *     )
 * 
 * cdef class EventAttributes:             # <<<<<<<<<<<<<<
 *     cdef object domain
 *     cdef object _message
*/
struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes {
  PyObject_HEAD
  struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes *__pyx_vtab;
  PyObject *domain;
  PyObject *_message;
  PyObject *_color;
  uint32_t _category;
  PyObject *_payload;
  nvtxStringHandle_t string_handle;
  nvtxEventAttributes_t c_obj;
  nvtxPayloadData_t _payload_data;
  PyObject *_payload_object;
  void *_allocated_payload;
};


/* "nvtx/_lib/lib.pxd":269
 * 
 * 
 * cdef class SchemaRegistrar:             # <<<<<<<<<<<<<<
 *     cdef dict _dtype_cache
 *     cdef dict _array_cache
*/
struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar {
  PyObject_HEAD
  struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_vtab;
  PyObject *_dtype_cache;
  PyObject *_array_cache;
  PyObject *_fixed_cache;
};


/* "nvtx/_lib/lib.pxd":278
 * 
 * 
 * cdef class DomainHandle:             # <<<<<<<<<<<<<<
 *     cdef bytes _name
 *     cdef nvtxDomainHandle_t c_obj
*/
struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle {
  PyObject_HEAD
  PyObject *_name;
  nvtxDomainHandle_t c_obj;
};


/* "nvtx/_lib/lib.pxd":282
 *     cdef nvtxDomainHandle_t c_obj
 * 
 * cdef class StringHandle:             # <<<<<<<<<<<<<<
 *     cdef bytes _string
 *     cdef nvtxStringHandle_t c_obj
*/
struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle {
  PyObject_HEAD
  PyObject *_string;
  nvtxStringHandle_t c_obj;
};


/* "nvtx/_lib/counters.pxd":112
 *         nvtxCounterLimit_t max
 * 
 * cdef class Counter:             # <<<<<<<<<<<<<<
 *     cdef readonly object domain
 *     cdef readonly object name
*/
struct __pyx_obj_4nvtx_4_lib_8counters_Counter {
  PyObject_HEAD
  PyObject *domain;
  PyObject *name;
  PyObject *dtype;
  PyObject *description;
  PyObject *scope;
  PyObject *semantics;
  uint64_t time_domain;
  uint64_t _counter_id;
  uint64_t _schema_id;
};


/* "nvtx/_lib/writer.pxd":355
 * 
 * 
 * cdef class _BackendModule:             # <<<<<<<<<<<<<<
 *     cdef const nvtxwInterface_v2_t* _iface
 *     cdef object _lib
*/
struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule {
  PyObject_HEAD
  struct __pyx_vtabstruct_4nvtx_4_lib_6writer__BackendModule *__pyx_vtab;
  nvtxwInterface_v2_t const *_iface;
  PyObject *_lib;
  Py_ssize_t _refcount;
};


/* "nvtx/_lib/writer.pxd":363
 * 
 * 
 * cdef class Backend:             # <<<<<<<<<<<<<<
 *     cdef const nvtxwInterface_v2_t* _iface
 *     cdef object _module
*/
struct __pyx_obj_4nvtx_4_lib_6writer_Backend {
  PyObject_HEAD
  struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Backend *__pyx_vtab;
  nvtxwInterface_v2_t const *_iface;
  PyObject *_module;
  PyObject *_lock;
  PyObject *_active_sessions;
};


/* "nvtx/_lib/writer.pxd":371
 * 
 * 
 * cdef class Session:             # <<<<<<<<<<<<<<
 *     cdef Backend _backend
 *     cdef bytes _name
*/
struct __pyx_obj_4nvtx_4_lib_6writer_Session {
  PyObject_HEAD
  struct __pyx_obj_4nvtx_4_lib_6writer_Backend *_backend;
  PyObject *_name;
  PyObject *_config;
  nvtxwSessionHandle_t _handle;
  PyObject *_lock;
  PyObject *_open_streams;
  PyObject *_domains;
};


/* "nvtx/_lib/writer.pxd":381
 * 
 * 
 * cdef class RegisteredString:             # <<<<<<<<<<<<<<
 *     cdef nvtxStringHandle_t _handle
 *     cdef str _string
*/
struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString {
  PyObject_HEAD
  nvtxStringHandle_t _handle;
  PyObject *_string;
  PyObject *_domain;
};


/* "nvtx/_lib/writer.pxd":387
 * 
 * 
 * cdef class Scope:             # <<<<<<<<<<<<<<
 *     cdef uint64_t _scope_id
 *     cdef object _path
*/
struct __pyx_obj_4nvtx_4_lib_6writer_Scope {
  PyObject_HEAD
  uint64_t _scope_id;
  PyObject *_path;
  PyObject *_domain;
};


/* "nvtx/_lib/writer.pxd":393
 * 
 * 
 * cdef class Domain:             # <<<<<<<<<<<<<<
 *     cdef Backend _backend
 *     cdef Session _session
*/
struct __pyx_obj_4nvtx_4_lib_6writer_Domain {
  PyObject_HEAD
  struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *__pyx_vtab;
  struct __pyx_obj_4nvtx_4_lib_6writer_Backend *_backend;
  struct __pyx_obj_4nvtx_4_lib_6writer_Session *_session;
  int _valid;
  nvtxDomainHandle_t _handle;
  int _event_schemas_registered;
  nvtxwEventHelperSchemaIds_t _event_schema_ids;
  PyObject *_get_string_cached;
  PyObject *_get_scope_cached;
  PyObject *_categories;
  PyObject *_schemas;
  PyObject *_get_counter_cached;
  PyObject *_category_ids;
  PyObject *_user_category_ids;
  struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *_schema_registrar;
};


/* "nvtx/_lib/writer.pxd":415
 * 
 * 
 * cdef class Counter:             # <<<<<<<<<<<<<<
 *     cdef Domain _domain
 *     cdef object _name
*/
struct __pyx_obj_4nvtx_4_lib_6writer_Counter {
  PyObject_HEAD
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *_domain;
  PyObject *_name;
  PyObject *_dtype;
  PyObject *_description;
  PyObject *_semantics;
  uint64_t _scope_id;
  uint64_t _counter_id;
  uint64_t _schema_id;
  PyObject *_timestamp_field;
};


/* "nvtx/_lib/writer.pxd":427
 * 
 * 
 * cdef class Schema:             # <<<<<<<<<<<<<<
 *     cdef Domain _domain
 *     cdef uint64_t _schema_id
*/
struct __pyx_obj_4nvtx_4_lib_6writer_Schema {
  PyObject_HEAD
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *_domain;
  uint64_t _schema_id;
  PyObject *_dtype;
  PyObject *_kind;
};


/* "nvtx/_lib/writer.pxd":447
 * 
 * 
 * cdef class Stream:             # <<<<<<<<<<<<<<
 *     cdef Backend _backend
 *     cdef Session _session
*/
struct __pyx_obj_4nvtx_4_lib_6writer_Stream {
  PyObject_HEAD
  struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *__pyx_vtab;
  struct __pyx_obj_4nvtx_4_lib_6writer_Backend *_backend;
  struct __pyx_obj_4nvtx_4_lib_6writer_Session *_session;
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *_domain;
  PyObject *_name;
  uint64_t _scope_id;
  uint64_t _time_domain_id;
  int16_t _order_interleaving;
  int16_t _ordering_type;
  int32_t _ordering_skid;
  int64_t _ordering_skid_amount;
  int _writer_ready;
  nvtxwEventWriter_t _writer;
};


/* "nvtx/_lib/writer.pyx":135
 * 
 * 
 * cdef class _WriterSchemaRegistrar(SchemaRegistrar):             # <<<<<<<<<<<<<<
 *     """Registers schemas via the backend interface's ``SchemaRegister``."""
 * 
*/
struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar {
  struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar __pyx_base;
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *_domain;
};



/* "nvtx/_lib/lib.pxd":250
 *     )
 * 
 * cdef class EventAttributes:             # <<<<<<<<<<<<<<
 *     cdef object domain
 *     cdef object _message
*/

struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes {
  PyObject *(*_set_binary_payload)(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *, void *, uint64_t, size_t, size_t);
  PyObject *(*_clear_payload)(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *);
};
static struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes *__pyx_vtabptr_4nvtx_4_lib_3lib_EventAttributes;


/* "nvtx/_lib/lib.pxd":269
 * 
 * 
 * cdef class SchemaRegistrar:             # <<<<<<<<<<<<<<
 *     cdef dict _dtype_cache
 *     cdef dict _array_cache
*/

struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar {
  uint64_t (*_do_register)(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *, nvtxPayloadSchemaAttr_t const *);
};
static struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_vtabptr_4nvtx_4_lib_3lib_SchemaRegistrar;


/* "nvtx/_lib/writer.pyx":398
 * 
 * 
 * cdef class _BackendModule:             # <<<<<<<<<<<<<<
 *     """One canonical backend module shared by all public wrappers."""
 * 
*/

struct __pyx_vtabstruct_4nvtx_4_lib_6writer__BackendModule {
  void (*_initialize)(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *);
  void (*_finalize)(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *);
};
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer__BackendModule *__pyx_vtabptr_4nvtx_4_lib_6writer__BackendModule;


/* "nvtx/_lib/writer.pyx":448
 * 
 * 
 * cdef class Backend:             # <<<<<<<<<<<<<<
 *     """
 *     A loaded NVTXW backend: the module handle plus its interface table.
*/

struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Backend {
  void (*_ensure_open)(struct __pyx_obj_4nvtx_4_lib_6writer_Backend *);
};
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Backend *__pyx_vtabptr_4nvtx_4_lib_6writer_Backend;


/* "nvtx/_lib/writer.pyx":987
 * 
 * 
 * cdef class Domain:             # <<<<<<<<<<<<<<
 *     """
 *     Session-owned NVTX domain. Created by :meth:`Session.get_domain`.
*/

struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain {
  PyObject *(*_invalidate)(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *);
  PyObject *(*_ensure_valid)(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *);
  PyObject *(*_get_event_schema_ids)(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *, nvtxwEventHelperSchemaIds_t *);
};
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *__pyx_vtabptr_4nvtx_4_lib_6writer_Domain;


/* "nvtx/_lib/writer.pyx":1402
 *         )
 * 
 * cdef class Stream:             # <<<<<<<<<<<<<<
 *     """
 *     NVTXW stream: the object events and counter samples are written to.
*/

struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream {
  void (*_ensure_writable)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *);
  void (*_ensure_counter)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *);
  nvtxwEventWriter_t *(*_get_writer)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *);
  PyObject *(*_resolve_event_attrs)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *, PyObject *, PyObject *, PyObject *, nvtxwEventAttributes_t *, nvtxwEventAttributesUtf8_t *);
  void (*_emit_at)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *, int64_t, PyObject *, PyObject *, PyObject *, __pyx_t_4nvtx_4_lib_6writer__at_bin_fn, __pyx_t_4nvtx_4_lib_6writer__at_utf8_fn, PyObject *, PyObject *);
  void (*_emit_span)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *, int64_t, int64_t, PyObject *, PyObject *, PyObject *, __pyx_t_4nvtx_4_lib_6writer__span_bin_fn, __pyx_t_4nvtx_4_lib_6writer__span_utf8_fn, PyObject *, PyObject *);
};
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *__pyx_vtabptr_4nvtx_4_lib_6writer_Stream;


/* "nvtx/_lib/writer.pyx":135
 * 
 * 
 * cdef class _WriterSchemaRegistrar(SchemaRegistrar):             # <<<<<<<<<<<<<<
 *     """Registers schemas via the backend interface's ``SchemaRegister``."""
 * 
*/

struct __pyx_vtabstruct_4nvtx_4_lib_6writer__WriterSchemaRegistrar {
  struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar __pyx_base;
};
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer__WriterSchemaRegistrar *__pyx_vtabptr_4nvtx_4_lib_6writer__WriterSchemaRegistrar;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto (used by PyErrFetchRestore) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto (used by GetBuiltinName) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* TupleAndListFromArray.proto (used by fastcall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto (used by BytesEquals) */
#include <string.h>

/* BytesEquals.proto (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL __Pyx_ArgsSlice_VARARGS
#endif

/* py_dict_items.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* CallCFunction.proto (used by CallUnboundCMethod0) */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* PyObjectCall.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto (used by PyObjectCallOneArg) */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* PyObjectCallOneArg.proto (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* UnpackUnboundCMethod.proto (used by CallUnboundCMethod0) */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod0.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* py_dict_values.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* OwnedDictNext.proto (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue);
#else
CYTHON_INLINE
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue);
#endif

/* RaiseDoubleKeywords.proto (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywordsImpl.export */
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name
);
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectDelAttr.proto (used by PyObjectSetAttrStr) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
#define __Pyx_PyObject_DelAttr(o, n) PyObject_SetAttr(o, n, NULL)
#else
#define __Pyx_PyObject_DelAttr(o, n) PyObject_DelAttr(o, n)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   __Pyx_PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyDictVersioning.proto (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __Pyx_XNewRef(__pyx_dict_cached_value);\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* BuildPyUnicode.proto (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* COrdinalToPyUnicode.proto (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value);
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t width, char padding_char);

/* GCCDiagnostics.proto (used by CIntToPyUnicode) */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* IncludeStdlibH.proto (used by CIntToPyUnicode) */
#include <stdlib.h>

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_int(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_int(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_int(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char);

/* JoinPyUnicode.export */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* RaiseException.export */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* GetTopmostException.proto (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* RejectKeywords.export */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PyTypeError_Check.proto */
#define __Pyx_PyExc_TypeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_TypeError)

/* decode_c_string_utf16.proto (used by decode_c_bytes) */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_bytes.proto (used by decode_bytes) */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* decode_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_bytes(
         PyObject* string, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    char* as_c_string;
    Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    as_c_string = PyBytes_AS_STRING(string);
    size = PyBytes_GET_SIZE(string);
#else
    if (PyBytes_AsStringAndSize(string, &as_c_string, &size) < 0) {
        return NULL;
    }
#endif
    return __Pyx_decode_c_bytes(
        as_c_string, size,
        start, stop, encoding, errors, decode_func);
}

/* PyValueError_Check.proto */
#define __Pyx_PyExc_ValueError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ValueError)

/* PyObjectCallNoArg.proto (used by pyfrozenset_new) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* pyfrozenset_new.proto (used by PySetContains) */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it);

/* PySetContains.proto */
static CYTHON_INLINE int __Pyx_PySet_ContainsTF(PyObject* key, PyObject* set, int eq);

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_uint64_t(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_uint64_t(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_uint64_t(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_uint64_t(uint64_t value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_uint64_t(uint64_t value, Py_ssize_t width, char padding_char, char format_char);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck, unsafe_shared) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck, int unsafe_shared);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyObjectVectorCallKwBuilder.proto */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* PyKeyError_Check.proto */
#define __Pyx_PyExc_KeyError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_KeyError)

/* PyObjectVectorCallMethodKwBuilder.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_VectorcallMethod_CallFromBuilder PyObject_VectorcallMethod
#else
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#endif

/* UnicodeConcatInPlace.proto */
# if CYTHON_COMPILING_IN_CPYTHON
    #if CYTHON_REFNANNY
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared, __pyx_refnanny)
    #else
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared)
    #endif
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_OwnStrongReference)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_FunctionArgument)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_SharedReference)
    static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    );
#else
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace __Pyx_PyUnicode_Concat
#endif
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right))

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyObjectCall2Args.proto (used by PyObjectCallMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectGetMethod.proto (used by PyObjectCallMethod1) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);
#endif

/* PyObjectCallMethod1.proto (used by append) */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* DictGetItem.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* IterFinish.proto (used by dict_iter) */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectCallMethod0.proto (used by dict_iter) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto (used by UnpackTuple2) */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto (used by UnpackItemEndCheck) */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto (used by UnpackTuple2) */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto (used by UnpackTupleError) */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto (used by UnpackTuple2) */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto (used by dict_iter) */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
    PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple);
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyRuntimeError_Check.proto */
#define __Pyx_PyExc_RuntimeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_RuntimeError)

/* RaiseKeywordRequired.proto */
static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name);

/* py_set_discard.proto */
static CYTHON_INLINE int __Pyx_PySet_Discard(PyObject *set, PyObject *key);

/* py_dict_clear.proto */
#define __Pyx_PyDict_Clear(d) (PyDict_Clear(d), 0)

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* CallUnboundCMethod1.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* ArgTypeTestFunc.export */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))

/* IterNextPlain.proto (used by IterNext) */
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator);
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void);
#endif

/* IterNext.proto */
#define __Pyx_PyIter_Next(obj) __Pyx_PyIter_Next2(obj, NULL)
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject *, PyObject *);

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* AllocateExtensionType.proto */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final);

/* CallTypeTraverse.proto */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#define __Pyx_call_type_traverse(o, always_call, visit, arg) 0
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg);
#endif

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(PyObject *api_dict, const char *name, void (*f)(void), const char *sig);

/* GetApiDict.proto */
static PyObject *__Pyx_ApiExport_GetApiDict(void);

/* LimitedApiGetTypeDict.proto (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp);
#endif

/* SetItemOnTypeDict.proto (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v);
#define __Pyx_SetItemOnTypeDict(tp, k, v) __Pyx__SetItemOnTypeDict((PyTypeObject*)tp, k, v)

/* FixUpExtensionType.proto */
static CYTHON_INLINE int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);

/* ValidateBasesTuple.proto (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
static int __Pyx_MergeVtables(PyTypeObject *type);

/* DelItemOnTypeDict.proto (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k);
#define __Pyx_DelItemOnTypeDict(tp, k) __Pyx__DelItemOnTypeDict((PyTypeObject*)tp, k)

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_2_9
#define __PYX_HAVE_RT_ImportType_proto_3_2_9
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_9(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_9(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_2_9 {
   __Pyx_ImportType_CheckSize_Error_3_2_9 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_2_9 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_2_9 = 2
};
static PyTypeObject *__Pyx_ImportType_3_2_9(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_9 check_size);
#endif

/* FunctionImport.proto */
static int __Pyx_ImportFunction_3_2_9(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* HasAttr.proto (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* ImportImpl.export */
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level);

/* Import.proto */
static CYTHON_INLINE PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* PyImportError_Check.proto */
#define __Pyx_PyExc_ImportError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ImportError)

/* decode_c_string.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* Py3UpdateBases.proto */
static PyObject* __Pyx_PEP560_update_bases(PyObject *bases);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* dict_setdefault.proto (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value);

/* AddModuleRef.proto (used by FetchSharedCythonModule) */
#if ((CYTHON_COMPILING_IN_CPYTHON_FREETHREADING ) ||\
     __PYX_LIMITED_VERSION_HEX < 0x030d0000)
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name);
#else
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#endif

/* FetchSharedCythonModule.proto (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto (used by CommonTypesMetaclass) */
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases);

/* CommonTypesMetaclass.proto (used by CythonFunctionShared) */
static int __pyx_CommonTypesMetaclass_init(PyObject *module);
#define __Pyx_CommonTypesMetaclass_USED

/* PyMethodNew.proto (used by CythonFunctionShared) */
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ);

/* PyVectorcallFastCallDict.proto (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto (used by CythonFunction) */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && CYTHON_METH_FASTCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_dict;
#endif
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    PyObject *defaults;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_mstate_global->__pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_mstate_global->__pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void));
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func,
                                                         PyTypeObject *defaults_type);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* CyFunctionClassCell.proto */
static int __Pyx_CyFunction_InitClassCell(PyObject *cyfunctions, PyObject *classobj);

/* CLineInTraceback.proto (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto (used by AddTraceback) */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CheckUnpickleChecksum.proto */
static CYTHON_INLINE int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint64_t(uint64_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint64_t __Pyx_PyLong_As_uint64_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int64_t __Pyx_PyLong_As_int64_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int32_t(int32_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyLong_As_uint32_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint32_t(uint32_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE int16_t __Pyx_PyLong_As_int16_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int32_t __Pyx_PyLong_As_int32_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint8_t __Pyx_PyLong_As_uint8_t(PyObject *);

/* UpdateUnpickledDict.proto */
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

/* GetRuntimeVersion.proto */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
static unsigned long __Pyx_cached_runtime_version = 0;
static void __Pyx_init_runtime_version(void);
#else
#define __Pyx_init_runtime_version()
#endif
static unsigned long __Pyx_get_runtime_version(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* DecompressString.proto */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

static uint64_t __pyx_f_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar__do_register(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *__pyx_v_self, nvtxPayloadSchemaAttr_t const *__pyx_v_attr); /* proto*/
static void __pyx_f_4nvtx_4_lib_6writer_14_BackendModule__initialize(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self); /* proto*/
static void __pyx_f_4nvtx_4_lib_6writer_14_BackendModule__finalize(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self); /* proto*/
static void __pyx_f_4nvtx_4_lib_6writer_7Backend__ensure_open(struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_4nvtx_4_lib_6writer_6Domain__invalidate(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_4nvtx_4_lib_6writer_6Domain__ensure_valid(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_4nvtx_4_lib_6writer_6Domain__get_event_schema_ids(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, nvtxwEventHelperSchemaIds_t *__pyx_v_schema_ids_out); /* proto*/
static void __pyx_f_4nvtx_4_lib_6writer_6Stream__ensure_writable(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self); /* proto*/
static void __pyx_f_4nvtx_4_lib_6writer_6Stream__ensure_counter(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter); /* proto*/
static nvtxwEventWriter_t *__pyx_f_4nvtx_4_lib_6writer_6Stream__get_writer(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_4nvtx_4_lib_6writer_6Stream__resolve_event_attrs(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, nvtxwEventAttributes_t *__pyx_v_attr, nvtxwEventAttributesUtf8_t *__pyx_v_uattr); /* proto*/
static void __pyx_f_4nvtx_4_lib_6writer_6Stream__emit_at(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, int64_t __pyx_v_timestamp, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, __pyx_t_4nvtx_4_lib_6writer__at_bin_fn __pyx_v_bin_fn, __pyx_t_4nvtx_4_lib_6writer__at_utf8_fn __pyx_v_utf8_fn, PyObject *__pyx_v_op, PyObject *__pyx_v_utf8_op); /* proto*/
static void __pyx_f_4nvtx_4_lib_6writer_6Stream__emit_span(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, int64_t __pyx_v_begin, int64_t __pyx_v_end, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, __pyx_t_4nvtx_4_lib_6writer__span_bin_fn __pyx_v_bin_fn, __pyx_t_4nvtx_4_lib_6writer__span_utf8_fn __pyx_v_utf8_fn, PyObject *__pyx_v_op, PyObject *__pyx_v_utf8_op); /* proto*/

/* Module declarations from "libc.stddef" */

/* Module declarations from "libc.stdint" */

/* Module declarations from "nvtx._lib.lib" */

/* Module declarations from "nvtx._lib.counters" */
static void (*__pyx_f_4nvtx_4_lib_8counters__fill_counter_semantics)(nvtxSemanticsCounter_t *, PyObject *); /*proto*/

/* Module declarations from "nvtx._lib.writer" */
static PyObject *__pyx_f_4nvtx_4_lib_6writer__as_bytes(PyObject *); /*proto*/
static PyObject *__pyx_f_4nvtx_4_lib_6writer__as_str(PyObject *); /*proto*/
static void __pyx_f_4nvtx_4_lib_6writer__check(nvtxwResultCode_t, PyObject *); /*proto*/
static uint32_t __pyx_f_4nvtx_4_lib_6writer__resolve_color(PyObject *); /*proto*/
static uint32_t __pyx_f_4nvtx_4_lib_6writer__resolve_category(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *, PyObject *); /*proto*/
static uint64_t __pyx_f_4nvtx_4_lib_6writer__resolve_scope_id(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *, PyObject *); /*proto*/
static uint64_t __pyx_f_4nvtx_4_lib_6writer__resolve_stream_scope_id(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *, PyObject *); /*proto*/
static PyObject *__pyx_f_4nvtx_4_lib_6writer___pyx_unpickle__WriterSchemaRegistrar__set_state(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *, PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "nvtx._lib.writer"
extern int __pyx_module_is_main_nvtx___lib__writer;
int __pyx_module_is_main_nvtx___lib__writer = 0;

/* Implementation of "nvtx._lib.writer" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_super;
/* #### Code section: string_decls ### */
static const char __pyx_k_array_cache__domain__dtype_cach[] = "_array_cache, _domain, _dtype_cache, _fixed_cache";
/* #### Code section: decls ### */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_11WriterError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, int __pyx_v_result_code, PyObject *__pyx_v_operation); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar___reduce_cython__(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_2__setstate_cython__(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer__validate_counter_layout(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_2_batch_rows(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data, PyObject *__pyx_v_dt); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_4_validate_native_byte_order(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dt, PyObject *__pyx_v_operation); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6_validate_event_dtype(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_kind, PyObject *__pyx_v_dt, PyObject *__pyx_v_operation); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_8_canonical_backend_path(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path); /* proto */
static int __pyx_pf_4nvtx_4_lib_6writer_14_BackendModule___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self, PyObject *__pyx_v_lib); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_14_BackendModule_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_14_BackendModule_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_10_release_backend_module(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_module); /* proto */
static int __pyx_pf_4nvtx_4_lib_6writer_7Backend___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self, PyObject *__pyx_v_module); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Backend_2close(struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Backend_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Backend_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_12load_backend(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path); /* proto */
static int __pyx_pf_4nvtx_4_lib_6writer_7Session___init__(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_backend, PyObject *__pyx_v_config); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_2begin(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_4end(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_6_register_domain(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_8get_domain(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_10create_stream(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_domain, PyObject *__pyx_v_scope, PyObject *__pyx_v_time_domain_id, PyObject *__pyx_v_interleaving, PyObject *__pyx_v_ordering, PyObject *__pyx_v_skid, PyObject *__pyx_v_skid_amount); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_12__enter__(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_14__exit__(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_16__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_18__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4nvtx_4_lib_6writer_16RegisteredString___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, PyObject *__pyx_v_string, PyObject *__pyx_v_handle_addr); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_2__repr__(struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_6handle___get__(struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4nvtx_4_lib_6writer_5Scope___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self, uint64_t __pyx_v_scope_id, PyObject *__pyx_v_path, PyObject *__pyx_v_domain); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_2__repr__(struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_8scope_id___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_4path___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4nvtx_4_lib_6writer_7Counter___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, PyObject *__pyx_v_description, PyObject *__pyx_v_semantics, uint64_t __pyx_v_scope_id, uint64_t __pyx_v_counter_id, uint64_t __pyx_v_schema_id, PyObject *__pyx_v_timestamp_field); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_2__repr__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_10counter_id___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_4name___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_5dtype___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_11description___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_9semantics___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4nvtx_4_lib_6writer_6Schema___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, uint64_t __pyx_v_schema_id, PyObject *__pyx_v_dtype, PyObject *__pyx_v_kind); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_2__repr__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_9schema_id___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_5dtype___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_4kind___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4nvtx_4_lib_6writer_6Domain___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_session, PyObject *__pyx_v_handle_addr); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_2get_registered_string(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_string); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_4_register_string(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_string); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_6get_category_id(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_8_register_category(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_10get_scope(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_path, PyObject *__pyx_v_parent); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_12_register_scope(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_path, uint64_t __pyx_v_parent_id); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_14get_schema(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_dtype, PyObject *__pyx_v_kind); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_16get_counter(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, PyObject *__pyx_v_description, PyObject *__pyx_v_scope, PyObject *__pyx_v_semantics); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_18_register_counter(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_schema_key, PyObject *__pyx_v_description, uint64_t __pyx_v_scope_id, PyObject *__pyx_v_semantics); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4nvtx_4_lib_6writer_6Stream___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_session, PyObject *__pyx_v_name, struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, PyObject *__pyx_v_scope, PyObject *__pyx_v_time_domain_id, PyObject *__pyx_v_interleaving, PyObject *__pyx_v_ordering, PyObject *__pyx_v_skid, PyObject *__pyx_v_skid_amount); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_2open(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_4close(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_6__enter__(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_8__exit__(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_10write_mark(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_12write_pushpop(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_end, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_14write_startend(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_end, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_16write_push(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_18write_pop(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_20write_start(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_range_id, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_22write_end(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_range_id); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_24write_event(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_schema, PyObject *__pyx_v_row); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_26write_event_batch(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_schema, PyObject *__pyx_v_rows, PyObject *__pyx_v_ordering); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_28write_counter_sample(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter, int64_t __pyx_v_timestamp, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_30write_counter_sample_no_value(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter, int64_t __pyx_v_timestamp, PyObject *__pyx_v_reason); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_32write_counter_batch(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter, PyObject *__pyx_v_data, PyObject *__pyx_v_timestamps, PyObject *__pyx_v_ordering); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_34__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_36__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_6writer_14__pyx_unpickle__WriterSchemaRegistrar(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer__BackendModule(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Backend(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Session(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_RegisteredString(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Scope(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Domain(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Counter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Schema(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Stream(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer__WriterSchemaRegistrar(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib_StringHandle;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_8counters_Counter;
  PyObject *__pyx_type_4nvtx_4_lib_6writer__BackendModule;
  PyObject *__pyx_type_4nvtx_4_lib_6writer_Backend;
  PyObject *__pyx_type_4nvtx_4_lib_6writer_Session;
  PyObject *__pyx_type_4nvtx_4_lib_6writer_RegisteredString;
  PyObject *__pyx_type_4nvtx_4_lib_6writer_Scope;
  PyObject *__pyx_type_4nvtx_4_lib_6writer_Domain;
  PyObject *__pyx_type_4nvtx_4_lib_6writer_Counter;
  PyObject *__pyx_type_4nvtx_4_lib_6writer_Schema;
  PyObject *__pyx_type_4nvtx_4_lib_6writer_Stream;
  PyObject *__pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer__BackendModule;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer_Backend;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer_Session;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer_Scope;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer_Domain;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer_Counter;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer_Schema;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer_Stream;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values;
  PyObject *__pyx_k__3;
  PyObject *__pyx_k__4;
  PyObject *__pyx_k__5;
  PyObject *__pyx_k__6;
  PyObject *__pyx_k__8;
  PyObject *__pyx_k__9;
  PyObject *__pyx_tuple[2];
  PyObject *__pyx_codeobj_tab[62];
  PyObject *__pyx_string_tab[518];
  PyObject *__pyx_number_tab[5];
/* #### Code section: module_state_contents ### */
/* IterNextPlain.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
PyObject *__Pyx_GetBuiltinNext_LimitedAPI_cache;
#endif

/* CommonTypesMetaclass.module_state_decls */
PyTypeObject *__pyx_CommonTypesMetaclassType;

/* CachedMethodType.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedMethodType;
#endif

/* CythonFunctionShared.module_state_decls */
PyTypeObject *__pyx_CyFunctionType;

/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_ __pyx_string_tab[0]
#define __pyx_kp_u_Counter_name __pyx_string_tab[1]
#define __pyx_kp_u_EntryKind_2 __pyx_string_tab[2]
#define __pyx_kp_u_How_fully_events_are_sorted_in_t __pyx_string_tab[3]
#define __pyx_kp_u_How_partial_sort_skid_is_quantif __pyx_string_tab[4]
#define __pyx_kp_u_Install_numpy_to_use_write_count __pyx_string_tab[5]
#define __pyx_kp_u_Note_that_Cython_is_deliberately __pyx_string_tab[6]
#define __pyx_kp_u_PayloadEntryType_RANGE_ID __pyx_string_tab[7]
#define __pyx_kp_u_Raised_when_an_NVTXW_backend_ca __pyx_string_tab[8]
#define __pyx_kp_u_RegisteredString __pyx_string_tab[9]
#define __pyx_kp_u_Schema_schema_id __pyx_string_tab[10]
#define __pyx_kp_u_Scope_scope_id __pyx_string_tab[11]
#define __pyx_kp_u_Top_level_fixed_size_array_count __pyx_string_tab[12]
#define __pyx_kp_u_Whether_ordering_guarantees_appl __pyx_string_tab[13]
#define __pyx_kp_u__10 __pyx_string_tab[14]
#define __pyx_kp_u__11 __pyx_string_tab[15]
#define __pyx_kp_u__2 __pyx_string_tab[16]
#define __pyx_kp_u_a_counter_layout __pyx_string_tab[17]
#define __pyx_kp_u_a_counter_layout_may_embed_at_mo __pyx_string_tab[18]
#define __pyx_kp_u_add_note __pyx_string_tab[19]
#define __pyx_kp_u_an_event_schema __pyx_string_tab[20]
#define __pyx_kp_u_backend_is_closed __pyx_string_tab[21]
#define __pyx_kp_u_backend_module_reference_count_u __pyx_string_tab[22]
#define __pyx_kp_u_backend_must_be_an_nvtx_writer_B __pyx_string_tab[23]
#define __pyx_kp_u_batch_data_is_missing_a_column_f __pyx_string_tab[24]
#define __pyx_kp_u_batch_data_must_be_one_dimension __pyx_string_tab[25]
#define __pyx_kp_u_batch_data_must_contain_at_least __pyx_string_tab[26]
#define __pyx_kp_u_batch_rows_must_contain_at_least __pyx_string_tab[27]
#define __pyx_kp_u_cannot_combine_the_EntryKind __pyx_string_tab[28]
#define __pyx_kp_u_category_must_be_a_str_bytes_int __pyx_string_tab[29]
#define __pyx_kp_u_category_must_be_in_0_2_32_1 __pyx_string_tab[30]
#define __pyx_kp_u_category_name_must_be_a_str_or_b __pyx_string_tab[31]
#define __pyx_kp_u_code __pyx_string_tab[32]
#define __pyx_kp_u_counter_id_2 __pyx_string_tab[33]
#define __pyx_kp_u_counter_is_not_registered_in_thi __pyx_string_tab[34]
#define __pyx_kp_u_data_and_timestamps_must_have_th __pyx_string_tab[35]
#define __pyx_kp_u_disable __pyx_string_tab[36]
#define __pyx_kp_u_domain_2 __pyx_string_tab[37]
#define __pyx_kp_u_domain_belongs_to_a_different_se __pyx_string_tab[38]
#define __pyx_kp_u_domain_is_no_longer_valid_its_ow __pyx_string_tab[39]
#define __pyx_kp_u_dtype_2 __pyx_string_tab[40]
#define __pyx_kp_u_dtype_for __pyx_string_tab[41]
#define __pyx_kp_u_enable __pyx_string_tab[42]
#define __pyx_kp_u_event_helpers_require_a_register __pyx_string_tab[43]
#define __pyx_kp_u_expected_a_str_or_bytes __pyx_string_tab[44]
#define __pyx_kp_u_failed __pyx_string_tab[45]
#define __pyx_kp_u_field __pyx_string_tab[46]
#define __pyx_kp_u_field_2 __pyx_string_tab[47]
#define __pyx_kp_u_gc __pyx_string_tab[48]
#define __pyx_kp_u_got __pyx_string_tab[49]
#define __pyx_kp_u_integer_scope_must_be_in_the_NVT __pyx_string_tab[50]
#define __pyx_kp_u_interface_version_not_supported __pyx_string_tab[51]
#define __pyx_kp_u_interleaving_must_be_an_nvtx_wri __pyx_string_tab[52]
#define __pyx_kp_u_invalid_argument __pyx_string_tab[53]
#define __pyx_kp_u_is_not_registered_in_this_sessi __pyx_string_tab[54]
#define __pyx_kp_u_is_not_valid_for __pyx_string_tab[55]
#define __pyx_kp_u_is_not_valid_in_a_counter_layou __pyx_string_tab[56]
#define __pyx_kp_u_isenabled __pyx_string_tab[57]
#define __pyx_kp_u_kind_2 __pyx_string_tab[58]
#define __pyx_kp_u_kind_must_be_an_nvtx_EventKind_o __pyx_string_tab[59]
#define __pyx_kp_u_library_load_failed __pyx_string_tab[60]
#define __pyx_kp_u_library_not_found __pyx_string_tab[61]
#define __pyx_kp_u_library_symbol_missing __pyx_string_tab[62]
#define __pyx_kp_u_message_is_a_RegisteredString_fr __pyx_string_tab[63]
#define __pyx_kp_u_message_must_be_a_str_bytes_Regi __pyx_string_tab[64]
#define __pyx_kp_u_module_must_be_a_loaded_backend __pyx_string_tab[65]
#define __pyx_kp_u_must_contain_exactly_one __pyx_string_tab[66]
#define __pyx_kp_u_no_default___reduce___due_to_non __pyx_string_tab[67]
#define __pyx_kp_u_not_supported_by_this_backend __pyx_string_tab[68]
#define __pyx_kp_u_ordering_must_be_an_nvtx_BatchOr __pyx_string_tab[69]
#define __pyx_kp_u_ordering_must_be_an_nvtx_writer __pyx_string_tab[70]
#define __pyx_kp_u_parent_must_be_a_PredefinedScope __pyx_string_tab[71]
#define __pyx_kp_u_parent_scope_is_registered_in_a __pyx_string_tab[72]
#define __pyx_kp_u_path_2 __pyx_string_tab[73]
#define __pyx_kp_u_predefined_parent_scope_must_be __pyx_string_tab[74]
#define __pyx_kp_u_range_id_must_be_non_zero __pyx_string_tab[75]
#define __pyx_kp_u_reason_must_be_an_nvtx_CounterNo __pyx_string_tab[76]
#define __pyx_kp_u_requires_a_structured_dtype __pyx_string_tab[77]
#define __pyx_kp_u_requires_fields_with_self_conta __pyx_string_tab[78]
#define __pyx_kp_u_requires_native_byte_order_got __pyx_string_tab[79]
#define __pyx_kp_u_runtime_resolved_scopes_are_not __pyx_string_tab[80]
#define __pyx_kp_u_schema_is_not_registered_in_this __pyx_string_tab[81]
#define __pyx_kp_u_scope_is_not_registered_in_this __pyx_string_tab[82]
#define __pyx_kp_u_scope_must_be_None_a_PredefinedS __pyx_string_tab[83]
#define __pyx_kp_u_scope_requires_a_registered_stre __pyx_string_tab[84]
#define __pyx_kp_u_self__handle_cannot_be_converted __pyx_string_tab[85]
#define __pyx_kp_u_session_is_already_active __pyx_string_tab[86]
#define __pyx_kp_u_session_is_not_active __pyx_string_tab[87]
#define __pyx_kp_u_skid_must_be_an_nvtx_writer_Stre __pyx_string_tab[88]
#define __pyx_kp_u_src_nvtx__lib_writer_pyx __pyx_string_tab[89]
#define __pyx_kp_u_str_bytes __pyx_string_tab[90]
#define __pyx_kp_u_stream_is_already_open __pyx_string_tab[91]
#define __pyx_kp_u_stream_is_not_open __pyx_string_tab[92]
#define __pyx_kp_u_string_must_be_a_str_or_bytes __pyx_string_tab[93]
#define __pyx_kp_u_time_domain_id_must_be_an_nvtx_T __pyx_string_tab[94]
#define __pyx_kp_u_timestamp_role_with_PayloadEntr __pyx_string_tab[95]
#define __pyx_kp_u_timestamps_is_mutually_exclusive __pyx_string_tab[96]
#define __pyx_kp_u_timestamps_must_be_one_dimension __pyx_string_tab[97]
#define __pyx_kp_u_tree_fragment __pyx_string_tab[98]
#define __pyx_kp_u_unknown_error __pyx_string_tab[99]
#define __pyx_kp_u_uses_managed_storage __pyx_string_tab[100]
#define __pyx_kp_u_write_counter_batch_requires_tim __pyx_string_tab[101]
#define __pyx_kp_u_write_counter_sample_does_not_su __pyx_string_tab[102]
#define __pyx_kp_u_write_counter_sample_expects_one __pyx_string_tab[103]
#define __pyx_kp_u_write_event_batch_accepts_only_c __pyx_string_tab[104]
#define __pyx_kp_u_write_event_expects_one_row_of_d __pyx_string_tab[105]
#define __pyx_kp_u_write_event_requires_an_event_sc __pyx_string_tab[106]
#define __pyx_n_u_BACKEND_MODULES __pyx_string_tab[107]
#define __pyx_n_u_BACKEND_MODULES_LOCK __pyx_string_tab[108]
#define __pyx_n_u_Backend __pyx_string_tab[109]
#define __pyx_n_u_BackendModule __pyx_string_tab[110]
#define __pyx_n_u_BackendModule___reduce_cython __pyx_string_tab[111]
#define __pyx_n_u_BackendModule___setstate_cython __pyx_string_tab[112]
#define __pyx_n_u_Backend___reduce_cython __pyx_string_tab[113]
#define __pyx_n_u_Backend___setstate_cython __pyx_string_tab[114]
#define __pyx_n_u_Backend_close __pyx_string_tab[115]
#define __pyx_n_u_BatchOrdering __pyx_string_tab[116]
#define __pyx_n_u_CDLL __pyx_string_tab[117]
#define __pyx_n_u_COMPLETE_EVENT_KINDS __pyx_string_tab[118]
#define __pyx_n_u_COUNTER_TIMESTAMP __pyx_string_tab[119]
#define __pyx_n_u_CURRENT_HW_MACHINE __pyx_string_tab[120]
#define __pyx_n_u_CURRENT_VM __pyx_string_tab[121]
#define __pyx_n_u_CategoryRegister __pyx_string_tab[122]
#define __pyx_n_u_Counter __pyx_string_tab[123]
#define __pyx_n_u_CounterBatchWrite __pyx_string_tab[124]
#define __pyx_n_u_CounterNoValueReason __pyx_string_tab[125]
#define __pyx_n_u_CounterNoValueWrite __pyx_string_tab[126]
#define __pyx_n_u_CounterRegister __pyx_string_tab[127]
#define __pyx_n_u_CounterWrite __pyx_string_tab[128]
#define __pyx_n_u_Counter___reduce_cython __pyx_string_tab[129]
#define __pyx_n_u_Counter___setstate_cython __pyx_string_tab[130]
#define __pyx_n_u_DataFrame __pyx_string_tab[131]
#define __pyx_n_u_Domain __pyx_string_tab[132]
#define __pyx_n_u_DomainRegister __pyx_string_tab[133]
#define __pyx_n_u_Domain___reduce_cython __pyx_string_tab[134]
#define __pyx_n_u_Domain___setstate_cython __pyx_string_tab[135]
#define __pyx_n_u_Domain__register_category __pyx_string_tab[136]
#define __pyx_n_u_Domain__register_counter __pyx_string_tab[137]
#define __pyx_n_u_Domain__register_scope __pyx_string_tab[138]
#define __pyx_n_u_Domain__register_string __pyx_string_tab[139]
#define __pyx_n_u_Domain_get_category_id __pyx_string_tab[140]
#define __pyx_n_u_Domain_get_counter __pyx_string_tab[141]
#define __pyx_n_u_Domain_get_registered_string __pyx_string_tab[142]
#define __pyx_n_u_Domain_get_schema __pyx_string_tab[143]
#define __pyx_n_u_Domain_get_scope __pyx_string_tab[144]
#define __pyx_n_u_EVENT_COUNT __pyx_string_tab[145]
#define __pyx_n_u_EntryKind __pyx_string_tab[146]
#define __pyx_n_u_Enum __pyx_string_tab[147]
#define __pyx_n_u_EventBatchWrite __pyx_string_tab[148]
#define __pyx_n_u_EventKind __pyx_string_tab[149]
#define __pyx_n_u_EventWrite __pyx_string_tab[150]
#define __pyx_n_u_FINALIZE_SYMBOL_NAME __pyx_string_tab[151]
#define __pyx_n_u_GET_INTERFACE_SYMBOL_NAME __pyx_string_tab[152]
#define __pyx_n_u_MARK __pyx_string_tab[153]
#define __pyx_n_u_NONE __pyx_string_tab[154]
#define __pyx_n_u_PACKED_RANGE_END __pyx_string_tab[155]
#define __pyx_n_u_PACKED_RANGE_START __pyx_string_tab[156]
#define __pyx_n_u_Path __pyx_string_tab[157]
#define __pyx_n_u_PayloadEntryType __pyx_string_tab[158]
#define __pyx_n_u_PayloadSchemaKey __pyx_string_tab[159]
#define __pyx_n_u_PredefinedScope __pyx_string_tab[160]
#define __pyx_n_u_Pyx_PyDict_NextRef __pyx_string_tab[161]
#define __pyx_n_u_RANGE_BEGIN __pyx_string_tab[162]
#define __pyx_n_u_RANGE_END __pyx_string_tab[163]
#define __pyx_n_u_RANGE_ID __pyx_string_tab[164]
#define __pyx_n_u_RANGE_POP __pyx_string_tab[165]
#define __pyx_n_u_RANGE_PUSH __pyx_string_tab[166]
#define __pyx_n_u_RANGE_PUSHPOP __pyx_string_tab[167]
#define __pyx_n_u_RANGE_START __pyx_string_tab[168]
#define __pyx_n_u_RANGE_STARTEND __pyx_string_tab[169]
#define __pyx_n_u_RESULT_MESSAGES __pyx_string_tab[170]
#define __pyx_n_u_RLock __pyx_string_tab[171]
#define __pyx_n_u_ROOT __pyx_string_tab[172]
#define __pyx_n_u_RegisteredString_2 __pyx_string_tab[173]
#define __pyx_n_u_RegisteredString___reduce_cython __pyx_string_tab[174]
#define __pyx_n_u_RegisteredString___setstate_cyth __pyx_string_tab[175]
#define __pyx_n_u_SCOPE __pyx_string_tab[176]
#define __pyx_n_u_SORTED __pyx_string_tab[177]
#define __pyx_n_u_STRICT __pyx_string_tab[178]
#define __pyx_n_u_Schema __pyx_string_tab[179]
#define __pyx_n_u_SchemaRegister __pyx_string_tab[180]
#define __pyx_n_u_Schema___reduce_cython __pyx_string_tab[181]
#define __pyx_n_u_Schema___setstate_cython __pyx_string_tab[182]
#define __pyx_n_u_Scope __pyx_string_tab[183]
#define __pyx_n_u_ScopeRegister __pyx_string_tab[184]
#define __pyx_n_u_Scope___reduce_cython __pyx_string_tab[185]
#define __pyx_n_u_Scope___setstate_cython __pyx_string_tab[186]
#define __pyx_n_u_Session __pyx_string_tab[187]
#define __pyx_n_u_SessionBegin __pyx_string_tab[188]
#define __pyx_n_u_SessionEnd __pyx_string_tab[189]
#define __pyx_n_u_Session___enter __pyx_string_tab[190]
#define __pyx_n_u_Session___exit __pyx_string_tab[191]
#define __pyx_n_u_Session___reduce_cython __pyx_string_tab[192]
#define __pyx_n_u_Session___setstate_cython __pyx_string_tab[193]
#define __pyx_n_u_Session__register_domain __pyx_string_tab[194]
#define __pyx_n_u_Session_begin __pyx_string_tab[195]
#define __pyx_n_u_Session_create_stream __pyx_string_tab[196]
#define __pyx_n_u_Session_end __pyx_string_tab[197]
#define __pyx_n_u_Session_get_domain __pyx_string_tab[198]
#define __pyx_n_u_Stream __pyx_string_tab[199]
#define __pyx_n_u_StreamClose __pyx_string_tab[200]
#define __pyx_n_u_StreamInterleaving __pyx_string_tab[201]
#define __pyx_n_u_StreamOpen __pyx_string_tab[202]
#define __pyx_n_u_StreamOrdering __pyx_string_tab[203]
#define __pyx_n_u_StreamSkid __pyx_string_tab[204]
#define __pyx_n_u_Stream___enter __pyx_string_tab[205]
#define __pyx_n_u_Stream___exit __pyx_string_tab[206]
#define __pyx_n_u_Stream___reduce_cython __pyx_string_tab[207]
#define __pyx_n_u_Stream___setstate_cython __pyx_string_tab[208]
#define __pyx_n_u_Stream_close __pyx_string_tab[209]
#define __pyx_n_u_Stream_open __pyx_string_tab[210]
#define __pyx_n_u_Stream_write_counter_batch __pyx_string_tab[211]
#define __pyx_n_u_Stream_write_counter_sample __pyx_string_tab[212]
#define __pyx_n_u_Stream_write_counter_sample_no_v __pyx_string_tab[213]
#define __pyx_n_u_Stream_write_end __pyx_string_tab[214]
#define __pyx_n_u_Stream_write_event __pyx_string_tab[215]
#define __pyx_n_u_Stream_write_event_batch __pyx_string_tab[216]
#define __pyx_n_u_Stream_write_mark __pyx_string_tab[217]
#define __pyx_n_u_Stream_write_pop __pyx_string_tab[218]
#define __pyx_n_u_Stream_write_push __pyx_string_tab[219]
#define __pyx_n_u_Stream_write_pushpop __pyx_string_tab[220]
#define __pyx_n_u_Stream_write_start __pyx_string_tab[221]
#define __pyx_n_u_Stream_write_startend __pyx_string_tab[222]
#define __pyx_n_u_StringRegister __pyx_string_tab[223]
#define __pyx_n_u_TIME_NS __pyx_string_tab[224]
#define __pyx_n_u_TimestampType __pyx_string_tab[225]
#define __pyx_n_u_UNKNOWN __pyx_string_tab[226]
#define __pyx_n_u_WriterError __pyx_string_tab[227]
#define __pyx_n_u_WriterError___init __pyx_string_tab[228]
#define __pyx_n_u_WriterSchemaRegistrar __pyx_string_tab[229]
#define __pyx_n_u_WriterSchemaRegistrar___reduce __pyx_string_tab[230]
#define __pyx_n_u_WriterSchemaRegistrar___setstat __pyx_string_tab[231]
#define __pyx_n_u__12 __pyx_string_tab[232]
#define __pyx_n_u_annotate __pyx_string_tab[233]
#define __pyx_n_u_append __pyx_string_tab[234]
#define __pyx_n_u_ascontiguousarray __pyx_string_tab[235]
#define __pyx_n_u_asyncio_coroutines __pyx_string_tab[236]
#define __pyx_n_u_attr __pyx_string_tab[237]
#define __pyx_n_u_backend __pyx_string_tab[238]
#define __pyx_n_u_batch __pyx_string_tab[239]
#define __pyx_n_u_batch_rows __pyx_string_tab[240]
#define __pyx_n_u_begin __pyx_string_tab[241]
#define __pyx_n_u_c_name __pyx_string_tab[242]
#define __pyx_n_u_c_path __pyx_string_tab[243]
#define __pyx_n_u_c_string __pyx_string_tab[244]
#define __pyx_n_u_c_void_p __pyx_string_tab[245]
#define __pyx_n_u_cache __pyx_string_tab[246]
#define __pyx_n_u_cached __pyx_string_tab[247]
#define __pyx_n_u_canonical_backend_path __pyx_string_tab[248]
#define __pyx_n_u_canonical_path __pyx_string_tab[249]
#define __pyx_n_u_cast __pyx_string_tab[250]
#define __pyx_n_u_category __pyx_string_tab[251]
#define __pyx_n_u_category_id __pyx_string_tab[252]
#define __pyx_n_u_class_getitem __pyx_string_tab[253]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[254]
#define __pyx_n_u_close __pyx_string_tab[255]
#define __pyx_n_u_color __pyx_string_tab[256]
#define __pyx_n_u_color_to_hex __pyx_string_tab[257]
#define __pyx_n_u_column __pyx_string_tab[258]
#define __pyx_n_u_config __pyx_string_tab[259]
#define __pyx_n_u_copy __pyx_string_tab[260]
#define __pyx_n_u_count __pyx_string_tab[261]
#define __pyx_n_u_counter __pyx_string_tab[262]
#define __pyx_n_u_counter_group __pyx_string_tab[263]
#define __pyx_n_u_counter_id __pyx_string_tab[264]
#define __pyx_n_u_counter_semantics __pyx_string_tab[265]
#define __pyx_n_u_create_stream __pyx_string_tab[266]
#define __pyx_n_u_ctypes __pyx_string_tab[267]
#define __pyx_n_u_data __pyx_string_tab[268]
#define __pyx_n_u_description __pyx_string_tab[269]
#define __pyx_n_u_description_bytes __pyx_string_tab[270]
#define __pyx_n_u_description_ptr __pyx_string_tab[271]
#define __pyx_n_u_dict __pyx_string_tab[272]
#define __pyx_n_u_dict_2 __pyx_string_tab[273]
#define __pyx_n_u_doc __pyx_string_tab[274]
#define __pyx_n_u_domain __pyx_string_tab[275]
#define __pyx_n_u_domain_handle __pyx_string_tab[276]
#define __pyx_n_u_dt __pyx_string_tab[277]
#define __pyx_n_u_dtype __pyx_string_tab[278]
#define __pyx_n_u_empty __pyx_string_tab[279]
#define __pyx_n_u_encoded __pyx_string_tab[280]
#define __pyx_n_u_end __pyx_string_tab[281]
#define __pyx_n_u_enter __pyx_string_tab[282]
#define __pyx_n_u_entry_kind __pyx_string_tab[283]
#define __pyx_n_u_entry_type __pyx_string_tab[284]
#define __pyx_n_u_enum __pyx_string_tab[285]
#define __pyx_n_u_error __pyx_string_tab[286]
#define __pyx_n_u_event_data __pyx_string_tab[287]
#define __pyx_n_u_events __pyx_string_tab[288]
#define __pyx_n_u_exc __pyx_string_tab[289]
#define __pyx_n_u_exit __pyx_string_tab[290]
#define __pyx_n_u_expanduser __pyx_string_tab[291]
#define __pyx_n_u_expected_count __pyx_string_tab[292]
#define __pyx_n_u_expected_range_ids __pyx_string_tab[293]
#define __pyx_n_u_f __pyx_string_tab[294]
#define __pyx_n_u_f64 __pyx_string_tab[295]
#define __pyx_n_u_failed_2 __pyx_string_tab[296]
#define __pyx_n_u_field_dtype __pyx_string_tab[297]
#define __pyx_n_u_field_name __pyx_string_tab[298]
#define __pyx_n_u_fields __pyx_string_tab[299]
#define __pyx_n_u_fromiter __pyx_string_tab[300]
#define __pyx_n_u_fsdecode __pyx_string_tab[301]
#define __pyx_n_u_fspath __pyx_string_tab[302]
#define __pyx_n_u_func __pyx_string_tab[303]
#define __pyx_n_u_functools __pyx_string_tab[304]
#define __pyx_n_u_get __pyx_string_tab[305]
#define __pyx_n_u_get_category_id __pyx_string_tab[306]
#define __pyx_n_u_get_counter __pyx_string_tab[307]
#define __pyx_n_u_get_domain __pyx_string_tab[308]
#define __pyx_n_u_get_numpy_dtype_schema __pyx_string_tab[309]
#define __pyx_n_u_get_registered_string __pyx_string_tab[310]
#define __pyx_n_u_get_schema __pyx_string_tab[311]
#define __pyx_n_u_get_scope __pyx_string_tab[312]
#define __pyx_n_u_getstate __pyx_string_tab[313]
#define __pyx_n_u_handle __pyx_string_tab[314]
#define __pyx_n_u_handle_addr __pyx_string_tab[315]
#define __pyx_n_u_hasobject __pyx_string_tab[316]
#define __pyx_n_u_i __pyx_string_tab[317]
#define __pyx_n_u_i64 __pyx_string_tab[318]
#define __pyx_n_u_iface __pyx_string_tab[319]
#define __pyx_n_u_init __pyx_string_tab[320]
#define __pyx_n_u_int64 __pyx_string_tab[321]
#define __pyx_n_u_interleaving __pyx_string_tab[322]
#define __pyx_n_u_is_coroutine __pyx_string_tab[323]
#define __pyx_n_u_isnative __pyx_string_tab[324]
#define __pyx_n_u_items __pyx_string_tab[325]
#define __pyx_n_u_itemsize __pyx_string_tab[326]
#define __pyx_n_u_itertools __pyx_string_tab[327]
#define __pyx_n_u_kind __pyx_string_tab[328]
#define __pyx_n_u_lib __pyx_string_tab[329]
#define __pyx_n_u_load_backend __pyx_string_tab[330]
#define __pyx_n_u_loaded __pyx_string_tab[331]
#define __pyx_n_u_main __pyx_string_tab[332]
#define __pyx_n_u_message __pyx_string_tab[333]
#define __pyx_n_u_metaclass __pyx_string_tab[334]
#define __pyx_n_u_metadata __pyx_string_tab[335]
#define __pyx_n_u_module __pyx_string_tab[336]
#define __pyx_n_u_module_2 __pyx_string_tab[337]
#define __pyx_n_u_mro_entries __pyx_string_tab[338]
#define __pyx_n_u_name __pyx_string_tab[339]
#define __pyx_n_u_name_2 __pyx_string_tab[340]
#define __pyx_n_u_name_bytes __pyx_string_tab[341]
#define __pyx_n_u_names __pyx_string_tab[342]
#define __pyx_n_u_nbytes __pyx_string_tab[343]
#define __pyx_n_u_ndarray __pyx_string_tab[344]
#define __pyx_n_u_ndim __pyx_string_tab[345]
#define __pyx_n_u_new __pyx_string_tab[346]
#define __pyx_n_u_next __pyx_string_tab[347]
#define __pyx_n_u_normalize_counter_dtype __pyx_string_tab[348]
#define __pyx_n_u_normcase __pyx_string_tab[349]
#define __pyx_n_u_np __pyx_string_tab[350]
#define __pyx_n_u_numpy __pyx_string_tab[351]
#define __pyx_n_u_numpy_dtype __pyx_string_tab[352]
#define __pyx_n_u_nvtx __pyx_string_tab[353]
#define __pyx_n_u_nvtx__lib_lib __pyx_string_tab[354]
#define __pyx_n_u_nvtx__lib_writer __pyx_string_tab[355]
#define __pyx_n_u_nvtx__metadata __pyx_string_tab[356]
#define __pyx_n_u_nvtx_colors __pyx_string_tab[357]
#define __pyx_n_u_nvtx_metadata_from_dtype __pyx_string_tab[358]
#define __pyx_n_u_nvtxwEventSchemasRegister __pyx_string_tab[359]
#define __pyx_n_u_nvtxwMarkWrite __pyx_string_tab[360]
#define __pyx_n_u_nvtxwMarkWriteUtf8 __pyx_string_tab[361]
#define __pyx_n_u_nvtxwRangeEndWrite __pyx_string_tab[362]
#define __pyx_n_u_nvtxwRangePopWrite __pyx_string_tab[363]
#define __pyx_n_u_nvtxwRangePushPopWrite __pyx_string_tab[364]
#define __pyx_n_u_nvtxwRangePushPopWriteUtf8 __pyx_string_tab[365]
#define __pyx_n_u_nvtxwRangePushWrite __pyx_string_tab[366]
#define __pyx_n_u_nvtxwRangePushWriteUtf8 __pyx_string_tab[367]
#define __pyx_n_u_nvtxwRangeStartEndWrite __pyx_string_tab[368]
#define __pyx_n_u_nvtxwRangeStartEndWriteUtf8 __pyx_string_tab[369]
#define __pyx_n_u_nvtxwRangeStartWrite __pyx_string_tab[370]
#define __pyx_n_u_nvtxwRangeStartWriteUtf8 __pyx_string_tab[371]
#define __pyx_n_u_open __pyx_string_tab[372]
#define __pyx_n_u_operation __pyx_string_tab[373]
#define __pyx_n_u_ordering __pyx_string_tab[374]
#define __pyx_n_u_os __pyx_string_tab[375]
#define __pyx_n_u_owner __pyx_string_tab[376]
#define __pyx_n_u_pandas __pyx_string_tab[377]
#define __pyx_n_u_parent __pyx_string_tab[378]
#define __pyx_n_u_parent_id __pyx_string_tab[379]
#define __pyx_n_u_path __pyx_string_tab[380]
#define __pyx_n_u_pathlib __pyx_string_tab[381]
#define __pyx_n_u_payload __pyx_string_tab[382]
#define __pyx_n_u_pd __pyx_string_tab[383]
#define __pyx_n_u_pop __pyx_string_tab[384]
#define __pyx_n_u_prepare __pyx_string_tab[385]
#define __pyx_n_u_pyx_capi __pyx_string_tab[386]
#define __pyx_n_u_pyx_checksum __pyx_string_tab[387]
#define __pyx_n_u_pyx_result __pyx_string_tab[388]
#define __pyx_n_u_pyx_state __pyx_string_tab[389]
#define __pyx_n_u_pyx_type __pyx_string_tab[390]
#define __pyx_n_u_pyx_unpickle__WriterSchemaRegi __pyx_string_tab[391]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[392]
#define __pyx_n_u_qualname __pyx_string_tab[393]
#define __pyx_n_u_range_id __pyx_string_tab[394]
#define __pyx_n_u_range_id_fields __pyx_string_tab[395]
#define __pyx_n_u_rc __pyx_string_tab[396]
#define __pyx_n_u_reason __pyx_string_tab[397]
#define __pyx_n_u_reduce __pyx_string_tab[398]
#define __pyx_n_u_reduce_cython __pyx_string_tab[399]
#define __pyx_n_u_reduce_ex __pyx_string_tab[400]
#define __pyx_n_u_register_category __pyx_string_tab[401]
#define __pyx_n_u_register_counter __pyx_string_tab[402]
#define __pyx_n_u_register_domain __pyx_string_tab[403]
#define __pyx_n_u_register_scope __pyx_string_tab[404]
#define __pyx_n_u_register_string __pyx_string_tab[405]
#define __pyx_n_u_release_backend_module __pyx_string_tab[406]
#define __pyx_n_u_required_roles __pyx_string_tab[407]
#define __pyx_n_u_resolve __pyx_string_tab[408]
#define __pyx_n_u_restype __pyx_string_tab[409]
#define __pyx_n_u_result_code __pyx_string_tab[410]
#define __pyx_n_u_rid __pyx_string_tab[411]
#define __pyx_n_u_role __pyx_string_tab[412]
#define __pyx_n_u_role_fields __pyx_string_tab[413]
#define __pyx_n_u_row __pyx_string_tab[414]
#define __pyx_n_u_rows __pyx_string_tab[415]
#define __pyx_n_u_samples __pyx_string_tab[416]
#define __pyx_n_u_schema __pyx_string_tab[417]
#define __pyx_n_u_schema_flags __pyx_string_tab[418]
#define __pyx_n_u_schema_id __pyx_string_tab[419]
#define __pyx_n_u_schema_key __pyx_string_tab[420]
#define __pyx_n_u_scope __pyx_string_tab[421]
#define __pyx_n_u_scope_id __pyx_string_tab[422]
#define __pyx_n_u_self __pyx_string_tab[423]
#define __pyx_n_u_semantics __pyx_string_tab[424]
#define __pyx_n_u_session __pyx_string_tab[425]
#define __pyx_n_u_set_name __pyx_string_tab[426]
#define __pyx_n_u_setdefault __pyx_string_tab[427]
#define __pyx_n_u_setstate __pyx_string_tab[428]
#define __pyx_n_u_setstate_cython __pyx_string_tab[429]
#define __pyx_n_u_size __pyx_string_tab[430]
#define __pyx_n_u_skid __pyx_string_tab[431]
#define __pyx_n_u_skid_amount __pyx_string_tab[432]
#define __pyx_n_u_start __pyx_string_tab[433]
#define __pyx_n_u_state __pyx_string_tab[434]
#define __pyx_n_u_stream __pyx_string_tab[435]
#define __pyx_n_u_string __pyx_string_tab[436]
#define __pyx_n_u_subdtype __pyx_string_tab[437]
#define __pyx_n_u_success __pyx_string_tab[438]
#define __pyx_n_u_super __pyx_string_tab[439]
#define __pyx_n_u_test __pyx_string_tab[440]
#define __pyx_n_u_threading __pyx_string_tab[441]
#define __pyx_n_u_time_domain_id __pyx_string_tab[442]
#define __pyx_n_u_timestamp __pyx_string_tab[443]
#define __pyx_n_u_timestamp_field __pyx_string_tab[444]
#define __pyx_n_u_timestamp_fields __pyx_string_tab[445]
#define __pyx_n_u_timestamps __pyx_string_tab[446]
#define __pyx_n_u_timestamps_np __pyx_string_tab[447]
#define __pyx_n_u_to_numpy __pyx_string_tab[448]
#define __pyx_n_u_uattr __pyx_string_tab[449]
#define __pyx_n_u_update __pyx_string_tab[450]
#define __pyx_n_u_use_setstate __pyx_string_tab[451]
#define __pyx_n_u_validate_counter_layout __pyx_string_tab[452]
#define __pyx_n_u_validate_event_dtype __pyx_string_tab[453]
#define __pyx_n_u_validate_native_byte_order __pyx_string_tab[454]
#define __pyx_n_u_value __pyx_string_tab[455]
#define __pyx_n_u_values __pyx_string_tab[456]
#define __pyx_n_u_write_counter_batch __pyx_string_tab[457]
#define __pyx_n_u_write_counter_sample __pyx_string_tab[458]
#define __pyx_n_u_write_counter_sample_no_value __pyx_string_tab[459]
#define __pyx_n_u_write_end __pyx_string_tab[460]
#define __pyx_n_u_write_event __pyx_string_tab[461]
#define __pyx_n_u_write_event_batch __pyx_string_tab[462]
#define __pyx_n_u_write_mark __pyx_string_tab[463]
#define __pyx_n_u_write_pop __pyx_string_tab[464]
#define __pyx_n_u_write_push __pyx_string_tab[465]
#define __pyx_n_u_write_pushpop __pyx_string_tab[466]
#define __pyx_n_u_write_start __pyx_string_tab[467]
#define __pyx_n_u_write_startend __pyx_string_tab[468]
#define __pyx_n_u_writer __pyx_string_tab[469]
#define __pyx_kp_b_PyObject_PyObject__as_bytes__as __pyx_string_tab[470]
#define __pyx_kp_b__7 __pyx_string_tab[471]
#define __pyx_kp_b_iso88591_2_AQ_Qa_7_Q_Qa_1A_A_q_6_Cq_l_j __pyx_string_tab[472]
#define __pyx_kp_b_iso88591_4AV1 __pyx_string_tab[473]
#define __pyx_kp_b_iso88591_4q_1F_Rxq_2U_1BgQa __pyx_string_tab[474]
#define __pyx_kp_b_iso88591_6_Cq_aq_nA_6_Cq_A __pyx_string_tab[475]
#define __pyx_kp_b_iso88591_A0_4z_5_1A_a_4_7_1 __pyx_string_tab[476]
#define __pyx_kp_b_iso88591_A0_Q_4y_1_aq_I_Jd_4q_fA_6_A_A_N __pyx_string_tab[477]
#define __pyx_kp_b_iso88591_A2_4z_a_1A_N_waq_d_d_1_wa_1_t_a __pyx_string_tab[478]
#define __pyx_kp_b_iso88591_A8_AQ_4z_1A_Q_a_4A_HI_q_q __pyx_string_tab[479]
#define __pyx_kp_b_iso88591_AD_6_Q_A_6_1_A_Q_auF_7_1_A_7q_a __pyx_string_tab[480]
#define __pyx_kp_b_iso88591_AF_AQ_Q_7_G1_A_1_IQ_7_c_q_t89A_Q __pyx_string_tab[481]
#define __pyx_kp_b_iso88591_A_0_q8_IQk_a __pyx_string_tab[482]
#define __pyx_kp_b_iso88591_A_0_q_IQk_1_Q __pyx_string_tab[483]
#define __pyx_kp_b_iso88591_A_0_t883a_l_1_t9IS_l_1_t9G1_HN_H __pyx_string_tab[484]
#define __pyx_kp_b_iso88591_A_1_1_2_1_6_A_3a_A_Zs_V3a_Zs_A_Z __pyx_string_tab[485]
#define __pyx_kp_b_iso88591_A_1_Kq_uIWA_0 __pyx_string_tab[486]
#define __pyx_kp_b_iso88591_A_1_Kq_uIWA_1 __pyx_string_tab[487]
#define __pyx_kp_b_iso88591_A_9Cq_AQ_a_at_t_a __pyx_string_tab[488]
#define __pyx_kp_b_iso88591_A_E_q __pyx_string_tab[489]
#define __pyx_kp_b_iso88591_A_F_q __pyx_string_tab[490]
#define __pyx_kp_b_iso88591_A_L_9Cq_AQ_a_4_4A_WJavQa_6_A_q_5 __pyx_string_tab[491]
#define __pyx_kp_b_iso88591_A_Q_4y_1_A __pyx_string_tab[492]
#define __pyx_kp_b_iso88591_A_Q_4y_1_aq_I_4wgQ_4q_4y_q_T_N_H __pyx_string_tab[493]
#define __pyx_kp_b_iso88591_A_Yaq_87_Q_I_N_HA_T_Zq_aq_at1_vQ __pyx_string_tab[494]
#define __pyx_kp_b_iso88591_A_Yaq_87_Q_N_HA_O1_Kq_T_q_JavQa __pyx_string_tab[495]
#define __pyx_kp_b_iso88591_A_Yaq_A_D_q_AQ_at1_q_hk __pyx_string_tab[496]
#define __pyx_kp_b_iso88591_A_a_4z_A_1A_6_Q_A_6_q_A_Q_AV6_6 __pyx_string_tab[497]
#define __pyx_kp_b_iso88591_A_a_T_T_wgQ_q_at1_c_Q_d_4q_iq_Q __pyx_string_tab[498]
#define __pyx_kp_b_iso88591_A_a_at_t1 __pyx_string_tab[499]
#define __pyx_kp_b_iso88591_A_b_a_q_gQgZvWA_4_1_q_7 __pyx_string_tab[500]
#define __pyx_kp_b_iso88591_A_q_4z_A_1A_AQ_Q_3c_aq_WA_7_G1_j __pyx_string_tab[501]
#define __pyx_kp_b_iso88591_A_q_q_q_f_4z_1_4z_A_1A_4z_1A_4z __pyx_string_tab[502]
#define __pyx_kp_b_iso88591_A_t883a_F __pyx_string_tab[503]
#define __pyx_kp_b_iso88591_A_t883a_l_1_a_T_xq_Yg_4q __pyx_string_tab[504]
#define __pyx_kp_b_iso88591_A_t9Cq_4q_A_hd_A_vS_vS_Q_Ja_Kq_6 __pyx_string_tab[505]
#define __pyx_kp_b_iso88591_F_7_Q_q_wa_q_q_q_q_j_q_wiwa_j_q __pyx_string_tab[506]
#define __pyx_kp_b_iso88591_O1_M_a_A_1_b_A_AQ_Ry __pyx_string_tab[507]
#define __pyx_kp_b_iso88591_Q __pyx_string_tab[508]
#define __pyx_kp_b_iso88591_T_5_uD_JavQ_1A_a_AQ_M_e3j_A_T_a __pyx_string_tab[509]
#define __pyx_kp_b_iso88591_T_Jd_Q_G1F_a_vWE_Q_q_t_S_IWEQTT __pyx_string_tab[510]
#define __pyx_kp_b_iso88591_a4_t9Cq_l_1_7_6_A_T_aq_wc_aq_IQ __pyx_string_tab[511]
#define __pyx_kp_b_iso88591_q_0_kQR_7_8_9RR_a_1 __pyx_string_tab[512]
#define __pyx_kp_b_iso88591_r_A_iq_1_q_A_A_N_Q_Q_Q_Q_a_b_b __pyx_string_tab[513]
#define __pyx_kp_b_iso88591_r_y_q_1HAT_r_N_A_q_Qd_t2Q_iq_a __pyx_string_tab[514]
#define __pyx_kp_b_iso88591_vS_CvS_q_uJgQ_iq_q_q_wawj_c_q_q __pyx_string_tab[515]
#define __pyx_kp_b_iso88591_z_4vS_AQ_r_1F_s_d_AV2Q_r_q_1G6 __pyx_string_tab[516]
#define __pyx_kp_b_void_nvtxSemanticsCounter_t_PyOb __pyx_string_tab[517]
#define __pyx_int_0 __pyx_number_tab[0]
#define __pyx_int_1 __pyx_number_tab[1]
#define __pyx_int_8 __pyx_number_tab[2]
#define __pyx_int_129641307 __pyx_number_tab[3]
#define __pyx_int_4294967295 __pyx_number_tab[4]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_8counters_Counter);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer__BackendModule);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Backend);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer_Backend);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Session);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer_Session);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer_RegisteredString);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Scope);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer_Scope);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Domain);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer_Domain);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Counter);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer_Counter);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Schema);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer_Schema);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Stream);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer_Stream);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar);
  Py_CLEAR(clear_module_state->__pyx_k__3);
  Py_CLEAR(clear_module_state->__pyx_k__4);
  Py_CLEAR(clear_module_state->__pyx_k__5);
  Py_CLEAR(clear_module_state->__pyx_k__6);
  Py_CLEAR(clear_module_state->__pyx_k__8);
  Py_CLEAR(clear_module_state->__pyx_k__9);
  for (int i=0; i<2; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<62; ++i) { Py_CLEAR(clear_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<518; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<5; ++i) { Py_CLEAR(clear_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_clear_contents ### */
/* CommonTypesMetaclass.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CyFunctionType);

/* #### Code section: module_state_clear_end ### */
return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_8counters_Counter);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer__BackendModule);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Backend);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer_Backend);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Session);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer_Session);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer_RegisteredString);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Scope);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer_Scope);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Domain);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer_Domain);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Counter);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer_Counter);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Schema);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer_Schema);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer_Stream);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer_Stream);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar);
  Py_VISIT(traverse_module_state->__pyx_k__3);
  Py_VISIT(traverse_module_state->__pyx_k__4);
  Py_VISIT(traverse_module_state->__pyx_k__5);
  Py_VISIT(traverse_module_state->__pyx_k__6);
  Py_VISIT(traverse_module_state->__pyx_k__8);
  Py_VISIT(traverse_module_state->__pyx_k__9);
  for (int i=0; i<2; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<62; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<518; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<5; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_traverse_contents ### */
/* CommonTypesMetaclass.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CyFunctionType);

/* #### Code section: module_state_traverse_end ### */
return 0;
}
#endif
/* #### Code section: module_code ### */

/* "nvtx/_lib/writer.pyx":121
 *     """
 * 
 *     def __init__(self, int result_code, operation=None):             # <<<<<<<<<<<<<<
 *         self.result_code = result_code
 *         self.operation = operation
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_11WriterError_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_11WriterError___init__, "WriterError.__init__(self, int result_code, operation=None)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_11WriterError_1__init__ = {"__init__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_11WriterError_1__init__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_11WriterError___init__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_11WriterError_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  int __pyx_v_result_code;
  PyObject *__pyx_v_operation = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_result_code,&__pyx_mstate_global->__pyx_n_u_operation,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 121, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 121, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 121, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 121, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 121, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, i); __PYX_ERR(0, 121, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 121, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 121, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 121, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_result_code = __Pyx_PyLong_As_int(values[1]); if (unlikely((__pyx_v_result_code == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 121, __pyx_L3_error)
    __pyx_v_operation = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 121, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.WriterError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_11WriterError___init__(__pyx_self, __pyx_v_self, __pyx_v_result_code, __pyx_v_operation);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_11WriterError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, int __pyx_v_result_code, PyObject *__pyx_v_operation) {
  PyObject *__pyx_v_message = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7[6];
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "nvtx/_lib/writer.pyx":122
 * 
 *     def __init__(self, int result_code, operation=None):
 *         self.result_code = result_code             # <<<<<<<<<<<<<<
 *         self.operation = operation
 *         message = _RESULT_MESSAGES.get(result_code, "unknown error")
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(__pyx_v_result_code); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_result_code, __pyx_t_1) < (0)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":123
 *     def __init__(self, int result_code, operation=None):
 *         self.result_code = result_code
 *         self.operation = operation             # <<<<<<<<<<<<<<
 *         message = _RESULT_MESSAGES.get(result_code, "unknown error")
 *         if operation:
*/
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_operation, __pyx_v_operation) < (0)) __PYX_ERR(0, 123, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":124
 *         self.result_code = result_code
 *         self.operation = operation
 *         message = _RESULT_MESSAGES.get(result_code, "unknown error")             # <<<<<<<<<<<<<<
 *         if operation:
 *             message = f"{operation} failed: {message} (code {result_code})"
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_RESULT_MESSAGES); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyLong_From_int(__pyx_v_result_code); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_3, __pyx_mstate_global->__pyx_kp_u_unknown_error};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_message = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":125
 *         self.operation = operation
 *         message = _RESULT_MESSAGES.get(result_code, "unknown error")
 *         if operation:             # <<<<<<<<<<<<<<
 *             message = f"{operation} failed: {message} (code {result_code})"
 *         super().__init__(message)
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_operation); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 125, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "nvtx/_lib/writer.pyx":126
 *         message = _RESULT_MESSAGES.get(result_code, "unknown error")
 *         if operation:
 *             message = f"{operation} failed: {message} (code {result_code})"             # <<<<<<<<<<<<<<
 *         super().__init__(message)
 * 
*/
    __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_operation, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_message, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyUnicode_From_int(__pyx_v_result_code, 0, ' ', 'd'); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7[0] = __pyx_t_1;
    __pyx_t_7[1] = __pyx_mstate_global->__pyx_kp_u_failed;
    __pyx_t_7[2] = __pyx_t_4;
    __pyx_t_7[3] = __pyx_mstate_global->__pyx_kp_u_code;
    __pyx_t_7[4] = __pyx_t_3;
    __pyx_t_7[5] = __pyx_mstate_global->__pyx_kp_u_;
    __pyx_t_2 = __Pyx_PyUnicode_Join(__pyx_t_7, 6, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 9 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4) + 7 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4));
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_message, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "nvtx/_lib/writer.pyx":125
 *         self.operation = operation
 *         message = _RESULT_MESSAGES.get(result_code, "unknown error")
 *         if operation:             # <<<<<<<<<<<<<<
 *             message = f"{operation} failed: {message} (code {result_code})"
 *         super().__init__(message)
*/
  }

  /* "nvtx/_lib/writer.pyx":127
 *         if operation:
 *             message = f"{operation} failed: {message} (code {result_code})"
 *         super().__init__(message)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_1 = NULL;
  __pyx_t_8 = __Pyx_CyFunction_GetClassObj(__pyx_self);
  if (!__pyx_t_8) { PyErr_SetString(PyExc_RuntimeError, "super(): empty __class__ cell"); __PYX_ERR(0, 127, __pyx_L1_error) }
  __Pyx_INCREF(__pyx_t_8);
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_t_8, __pyx_v_self};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_super, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_t_3 = __pyx_t_4;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_message};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_init, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":121
 *     """
 * 
 *     def __init__(self, int result_code, operation=None):             # <<<<<<<<<<<<<<
 *         self.result_code = result_code
 *         self.operation = operation
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("nvtx._lib.writer.WriterError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_message);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":130
 * 
 * 
 * cdef void _check(nvtxwResultCode_t rc, str op) except *:             # <<<<<<<<<<<<<<
 *     if rc != NVTXW_RESULT_SUCCESS:
 *         raise WriterError(rc, op)
*/

static void __pyx_f_4nvtx_4_lib_6writer__check(nvtxwResultCode_t __pyx_v_rc, PyObject *__pyx_v_op) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_check", 0);

  /* "nvtx/_lib/writer.pyx":131
 * 
 * cdef void _check(nvtxwResultCode_t rc, str op) except *:
 *     if rc != NVTXW_RESULT_SUCCESS:             # <<<<<<<<<<<<<<
 *         raise WriterError(rc, op)
 * 
*/
  __pyx_t_1 = (__pyx_v_rc != NVTXW_RESULT_SUCCESS);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":132
 * cdef void _check(nvtxwResultCode_t rc, str op) except *:
 *     if rc != NVTXW_RESULT_SUCCESS:
 *         raise WriterError(rc, op)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_WriterError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyLong_From_int32_t(__pyx_v_rc); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_5, __pyx_v_op};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 132, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":131
 * 
 * cdef void _check(nvtxwResultCode_t rc, str op) except *:
 *     if rc != NVTXW_RESULT_SUCCESS:             # <<<<<<<<<<<<<<
 *         raise WriterError(rc, op)
 * 
*/
  }

  /* "nvtx/_lib/writer.pyx":130
 * 
 * 
 * cdef void _check(nvtxwResultCode_t rc, str op) except *:             # <<<<<<<<<<<<<<
 *     if rc != NVTXW_RESULT_SUCCESS:
 *         raise WriterError(rc, op)
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.writer._check", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "nvtx/_lib/writer.pyx":140
 *     cdef Domain _domain
 * 
 *     cdef uint64_t _do_register(             # <<<<<<<<<<<<<<
 *         self, const nvtxPayloadSchemaAttr_t* attr
 *     ) except *:
*/

static uint64_t __pyx_f_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar__do_register(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *__pyx_v_self, nvtxPayloadSchemaAttr_t const *__pyx_v_attr) {
  uint64_t __pyx_v_schema_id;
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain = 0;
  nvtxwResultCode_t __pyx_v_rc;
  uint64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_do_register", 0);

  /* "nvtx/_lib/writer.pyx":143
 *         self, const nvtxPayloadSchemaAttr_t* attr
 *     ) except *:
 *         cdef uint64_t schema_id = 0             # <<<<<<<<<<<<<<
 *         cdef Domain domain = self._domain
 *         cdef nvtxwResultCode_t rc
*/
  __pyx_v_schema_id = 0;

  /* "nvtx/_lib/writer.pyx":144
 *     ) except *:
 *         cdef uint64_t schema_id = 0
 *         cdef Domain domain = self._domain             # <<<<<<<<<<<<<<
 *         cdef nvtxwResultCode_t rc
 *         with domain._session._lock:
*/
  __pyx_t_1 = ((PyObject *)__pyx_v_self->_domain);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":146
 *         cdef Domain domain = self._domain
 *         cdef nvtxwResultCode_t rc
 *         with domain._session._lock:             # <<<<<<<<<<<<<<
 *             domain._ensure_valid()
 *             rc = domain._backend._iface.SchemaRegister(
*/
  /*with:*/ {
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_domain->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_domain->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 146, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":147
 *         cdef nvtxwResultCode_t rc
 *         with domain._session._lock:
 *             domain._ensure_valid()             # <<<<<<<<<<<<<<
 *             rc = domain._backend._iface.SchemaRegister(
 *                 domain._handle, attr, &schema_id)
*/
          __pyx_t_1 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_domain->__pyx_vtab)->_ensure_valid(__pyx_v_domain); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 147, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "nvtx/_lib/writer.pyx":148
 *         with domain._session._lock:
 *             domain._ensure_valid()
 *             rc = domain._backend._iface.SchemaRegister(             # <<<<<<<<<<<<<<
 *                 domain._handle, attr, &schema_id)
 *             _check(rc, "SchemaRegister")
*/
          __pyx_v_rc = __pyx_v_domain->_backend->_iface->SchemaRegister(__pyx_v_domain->_handle, __pyx_v_attr, (&__pyx_v_schema_id));

          /* "nvtx/_lib/writer.pyx":150
 *             rc = domain._backend._iface.SchemaRegister(
 *                 domain._handle, attr, &schema_id)
 *             _check(rc, "SchemaRegister")             # <<<<<<<<<<<<<<
 *         return schema_id
 * 
*/
          __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_SchemaRegister); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 150, __pyx_L7_error)

          /* "nvtx/_lib/writer.pyx":146
 *         cdef Domain domain = self._domain
 *         cdef nvtxwResultCode_t rc
 *         with domain._session._lock:             # <<<<<<<<<<<<<<
 *             domain._ensure_valid()
 *             rc = domain._backend._iface.SchemaRegister(
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer._WriterSchemaRegistrar._do_register", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 146, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_1);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_3);
          __pyx_t_9 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 146, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_9, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 146, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (__pyx_t_11 < (0)) __PYX_ERR(0, 146, __pyx_L9_except_error)
          __pyx_t_12 = (!__pyx_t_11);
          if (unlikely(__pyx_t_12)) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_4, __pyx_t_3);
            __pyx_t_1 = 0;  __pyx_t_4 = 0;  __pyx_t_3 = 0; 
            __PYX_ERR(0, 146, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_2) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 146, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L16;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L1_error;
    __pyx_L16:;
  }

  /* "nvtx/_lib/writer.pyx":151
 *                 domain._handle, attr, &schema_id)
 *             _check(rc, "SchemaRegister")
 *         return schema_id             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = __pyx_v_schema_id;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":140
 *     cdef Domain _domain
 * 
 *     cdef uint64_t _do_register(             # <<<<<<<<<<<<<<
 *         self, const nvtxPayloadSchemaAttr_t* attr
 *     ) except *:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("nvtx._lib.writer._WriterSchemaRegistrar._do_register", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_domain);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar___reduce_cython__, "_WriterSchemaRegistrar.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_1__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_1__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar___reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_1__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar___reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar___reduce_cython__(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._array_cache, self._domain, self._dtype_cache, self._fixed_cache)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = PyTuple_New(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._array_cache);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._array_cache);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->__pyx_base._array_cache) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_self->_domain);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->_domain);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self->_domain)) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._dtype_cache);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._dtype_cache);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_self->__pyx_base._dtype_cache) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._fixed_cache);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._fixed_cache);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_v_self->__pyx_base._fixed_cache) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._array_cache, self._domain, self._dtype_cache, self._fixed_cache)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self._array_cache, self._domain, self._dtype_cache, self._fixed_cache)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict) != (0)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._array_cache is not None or self._domain is not None or self._dtype_cache is not None or self._fixed_cache is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._array_cache, self._domain, self._dtype_cache, self._fixed_cache)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._array_cache is not None or self._domain is not None or self._dtype_cache is not None or self._fixed_cache is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, None), state
*/
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->__pyx_base._array_cache != ((PyObject*)Py_None));
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (((PyObject *)__pyx_v_self->_domain) != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->__pyx_base._dtype_cache != ((PyObject*)Py_None));
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->__pyx_base._fixed_cache != ((PyObject*)Py_None));
    __pyx_t_2 = __pyx_t_3;
    __pyx_L6_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._array_cache is not None or self._domain is not None or self._dtype_cache is not None or self._fixed_cache is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._array_cache is not None or self._domain is not None or self._dtype_cache is not None or self._fixed_cache is not None
 *     if use_setstate:
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle__WriterSchemaRegi); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_129641307);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_129641307);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_129641307) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._array_cache is not None or self._domain is not None or self._dtype_cache is not None or self._fixed_cache is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, None), state
 *     else:
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__WriterSchemaRegistrar__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle__WriterSchemaRegi); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_129641307);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_129641307);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_129641307) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.writer._WriterSchemaRegistrar.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__WriterSchemaRegistrar__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_2__setstate_cython__, "_WriterSchemaRegistrar.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_3__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_3__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_2__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_3__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer._WriterSchemaRegistrar.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_2__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_2__setstate_cython__(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__WriterSchemaRegistrar__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(1, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(1, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_4nvtx_4_lib_6writer___pyx_unpickle__WriterSchemaRegistrar__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__WriterSchemaRegistrar__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.writer._WriterSchemaRegistrar.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":154
 * 
 * 
 * cdef bytes _as_bytes(object s):             # <<<<<<<<<<<<<<
 *     if s is None:
 *         return None
*/

static PyObject *__pyx_f_4nvtx_4_lib_6writer__as_bytes(PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_as_bytes", 0);

  /* "nvtx/_lib/writer.pyx":155
 * 
 * cdef bytes _as_bytes(object s):
 *     if s is None:             # <<<<<<<<<<<<<<
 *         return None
 *     if isinstance(s, bytes):
*/
  __pyx_t_1 = (__pyx_v_s == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":156
 * cdef bytes _as_bytes(object s):
 *     if s is None:
 *         return None             # <<<<<<<<<<<<<<
 *     if isinstance(s, bytes):
 *         return <bytes>s
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = ((PyObject*)Py_None); __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":155
 * 
 * cdef bytes _as_bytes(object s):
 *     if s is None:             # <<<<<<<<<<<<<<
 *         return None
 *     if isinstance(s, bytes):
*/
  }

  /* "nvtx/_lib/writer.pyx":157
 *     if s is None:
 *         return None
 *     if isinstance(s, bytes):             # <<<<<<<<<<<<<<
 *         return <bytes>s
 *     if isinstance(s, str):
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_s); 
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":158
 *         return None
 *     if isinstance(s, bytes):
 *         return <bytes>s             # <<<<<<<<<<<<<<
 *     if isinstance(s, str):
 *         return (<str>s).encode("utf-8")
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject*)__pyx_v_s));
    __pyx_r = ((PyObject*)__pyx_v_s);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":157
 *     if s is None:
 *         return None
 *     if isinstance(s, bytes):             # <<<<<<<<<<<<<<
 *         return <bytes>s
 *     if isinstance(s, str):
*/
  }

  /* "nvtx/_lib/writer.pyx":159
 *     if isinstance(s, bytes):
 *         return <bytes>s
 *     if isinstance(s, str):             # <<<<<<<<<<<<<<
 *         return (<str>s).encode("utf-8")
 *     raise TypeError("expected a str or bytes")
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_s); 
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":160
 *         return <bytes>s
 *     if isinstance(s, str):
 *         return (<str>s).encode("utf-8")             # <<<<<<<<<<<<<<
 *     raise TypeError("expected a str or bytes")
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_s == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
      __PYX_ERR(0, 160, __pyx_L1_error)
    }
    __pyx_t_2 = PyUnicode_AsUTF8String(((PyObject*)__pyx_v_s)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":159
 *     if isinstance(s, bytes):
 *         return <bytes>s
 *     if isinstance(s, str):             # <<<<<<<<<<<<<<
 *         return (<str>s).encode("utf-8")
 *     raise TypeError("expected a str or bytes")
*/
  }

  /* "nvtx/_lib/writer.pyx":161
 *     if isinstance(s, str):
 *         return (<str>s).encode("utf-8")
 *     raise TypeError("expected a str or bytes")             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_3 = NULL;
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_expected_a_str_or_bytes};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 161, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 161, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":154
 * 
 * 
 * cdef bytes _as_bytes(object s):             # <<<<<<<<<<<<<<
 *     if s is None:
 *         return None
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer._as_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":164
 * 
 * 
 * cdef object _as_str(object s):             # <<<<<<<<<<<<<<
 *     if s is None or isinstance(s, str):
 *         return s
*/

static PyObject *__pyx_f_4nvtx_4_lib_6writer__as_str(PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_as_str", 0);

  /* "nvtx/_lib/writer.pyx":165
 * 
 * cdef object _as_str(object s):
 *     if s is None or isinstance(s, str):             # <<<<<<<<<<<<<<
 *         return s
 *     if isinstance(s, bytes):
*/
  __pyx_t_2 = (__pyx_v_s == Py_None);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyUnicode_Check(__pyx_v_s); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":166
 * cdef object _as_str(object s):
 *     if s is None or isinstance(s, str):
 *         return s             # <<<<<<<<<<<<<<
 *     if isinstance(s, bytes):
 *         return (<bytes>s).decode("utf-8")
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_s);
    __pyx_r = __pyx_v_s;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":165
 * 
 * cdef object _as_str(object s):
 *     if s is None or isinstance(s, str):             # <<<<<<<<<<<<<<
 *         return s
 *     if isinstance(s, bytes):
*/
  }

  /* "nvtx/_lib/writer.pyx":167
 *     if s is None or isinstance(s, str):
 *         return s
 *     if isinstance(s, bytes):             # <<<<<<<<<<<<<<
 *         return (<bytes>s).decode("utf-8")
 *     raise TypeError("expected a str or bytes")
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_s); 
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":168
 *         return s
 *     if isinstance(s, bytes):
 *         return (<bytes>s).decode("utf-8")             # <<<<<<<<<<<<<<
 *     raise TypeError("expected a str or bytes")
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_s == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "decode");
      __PYX_ERR(0, 168, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_decode_bytes(((PyObject*)__pyx_v_s), 0, PY_SSIZE_T_MAX, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":167
 *     if s is None or isinstance(s, str):
 *         return s
 *     if isinstance(s, bytes):             # <<<<<<<<<<<<<<
 *         return (<bytes>s).decode("utf-8")
 *     raise TypeError("expected a str or bytes")
*/
  }

  /* "nvtx/_lib/writer.pyx":169
 *     if isinstance(s, bytes):
 *         return (<bytes>s).decode("utf-8")
 *     raise TypeError("expected a str or bytes")             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_4 = NULL;
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_expected_a_str_or_bytes};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_Raise(__pyx_t_3, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __PYX_ERR(0, 169, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":164
 * 
 * 
 * cdef object _as_str(object s):             # <<<<<<<<<<<<<<
 *     if s is None or isinstance(s, str):
 *         return s
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.writer._as_str", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":172
 * 
 * 
 * cdef uint32_t _resolve_color(object color) except? 0:             # <<<<<<<<<<<<<<
 *     # NVTXW event attributes use 0 for "no explicit color", so ``None`` maps to
 *     # 0 rather than the instrumentation default (blue).
*/

static uint32_t __pyx_f_4nvtx_4_lib_6writer__resolve_color(PyObject *__pyx_v_color) {
  uint32_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  uint32_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_resolve_color", 0);

  /* "nvtx/_lib/writer.pyx":175
 *     # NVTXW event attributes use 0 for "no explicit color", so ``None`` maps to
 *     # 0 rather than the instrumentation default (blue).
 *     if color is None:             # <<<<<<<<<<<<<<
 *         return 0
 *     return <uint32_t>color_to_hex(color)
*/
  __pyx_t_1 = (__pyx_v_color == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":176
 *     # 0 rather than the instrumentation default (blue).
 *     if color is None:
 *         return 0             # <<<<<<<<<<<<<<
 *     return <uint32_t>color_to_hex(color)
 * 
*/
    __pyx_r = 0;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":175
 *     # NVTXW event attributes use 0 for "no explicit color", so ``None`` maps to
 *     # 0 rather than the instrumentation default (blue).
 *     if color is None:             # <<<<<<<<<<<<<<
 *         return 0
 *     return <uint32_t>color_to_hex(color)
*/
  }

  /* "nvtx/_lib/writer.pyx":177
 *     if color is None:
 *         return 0
 *     return <uint32_t>color_to_hex(color)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_color_to_hex); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_color};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_t_6 = __Pyx_PyLong_As_uint32_t(__pyx_t_2); if (unlikely((__pyx_t_6 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = ((uint32_t)__pyx_t_6);
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":172
 * 
 * 
 * cdef uint32_t _resolve_color(object color) except? 0:             # <<<<<<<<<<<<<<
 *     # NVTXW event attributes use 0 for "no explicit color", so ``None`` maps to
 *     # 0 rather than the instrumentation default (blue).
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.writer._resolve_color", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":180
 * 
 * 
 * cdef uint32_t _resolve_category(Domain domain, object category) except? 0:             # <<<<<<<<<<<<<<
 *     # None -> 0 (no category); str/bytes are named via the domain; ints pass
 *     # through and are reserved so the named-category allocator skips them.
*/

static uint32_t __pyx_f_4nvtx_4_lib_6writer__resolve_category(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, PyObject *__pyx_v_category) {
  uint32_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  uint32_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_resolve_category", 0);

  /* "nvtx/_lib/writer.pyx":183
 *     # None -> 0 (no category); str/bytes are named via the domain; ints pass
 *     # through and are reserved so the named-category allocator skips them.
 *     if category is None:             # <<<<<<<<<<<<<<
 *         return 0
 *     if isinstance(category, (str, bytes)):
*/
  __pyx_t_1 = (__pyx_v_category == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":184
 *     # through and are reserved so the named-category allocator skips them.
 *     if category is None:
 *         return 0             # <<<<<<<<<<<<<<
 *     if isinstance(category, (str, bytes)):
 *         return <uint32_t>domain.get_category_id(category)
*/
    __pyx_r = 0;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":183
 *     # None -> 0 (no category); str/bytes are named via the domain; ints pass
 *     # through and are reserved so the named-category allocator skips them.
 *     if category is None:             # <<<<<<<<<<<<<<
 *         return 0
 *     if isinstance(category, (str, bytes)):
*/
  }

  /* "nvtx/_lib/writer.pyx":185
 *     if category is None:
 *         return 0
 *     if isinstance(category, (str, bytes)):             # <<<<<<<<<<<<<<
 *         return <uint32_t>domain.get_category_id(category)
 *     if isinstance(category, int):
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_category); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_category); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":186
 *         return 0
 *     if isinstance(category, (str, bytes)):
 *         return <uint32_t>domain.get_category_id(category)             # <<<<<<<<<<<<<<
 *     if isinstance(category, int):
 *         if category < 0 or category > 0xFFFFFFFF:
*/
    __pyx_t_4 = ((PyObject *)__pyx_v_domain);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_category};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_category_id, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_t_6 = __Pyx_PyLong_As_uint32_t(__pyx_t_3); if (unlikely((__pyx_t_6 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = ((uint32_t)__pyx_t_6);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":185
 *     if category is None:
 *         return 0
 *     if isinstance(category, (str, bytes)):             # <<<<<<<<<<<<<<
 *         return <uint32_t>domain.get_category_id(category)
 *     if isinstance(category, int):
*/
  }

  /* "nvtx/_lib/writer.pyx":187
 *     if isinstance(category, (str, bytes)):
 *         return <uint32_t>domain.get_category_id(category)
 *     if isinstance(category, int):             # <<<<<<<<<<<<<<
 *         if category < 0 or category > 0xFFFFFFFF:
 *             raise ValueError("category must be in [0, 2**32 - 1]")
*/
  __pyx_t_1 = PyLong_Check(__pyx_v_category); 
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":188
 *         return <uint32_t>domain.get_category_id(category)
 *     if isinstance(category, int):
 *         if category < 0 or category > 0xFFFFFFFF:             # <<<<<<<<<<<<<<
 *             raise ValueError("category must be in [0, 2**32 - 1]")
 *         # Reserving an integer category only needs the lifecycle lock once.
*/
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_category, __pyx_mstate_global->__pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_3 = PyObject_RichCompare(__pyx_v_category, __pyx_mstate_global->__pyx_int_4294967295, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __pyx_t_2;
    __pyx_L9_bool_binop_done:;
    if (unlikely(__pyx_t_1)) {

      /* "nvtx/_lib/writer.pyx":189
 *     if isinstance(category, int):
 *         if category < 0 or category > 0xFFFFFFFF:
 *             raise ValueError("category must be in [0, 2**32 - 1]")             # <<<<<<<<<<<<<<
 *         # Reserving an integer category only needs the lifecycle lock once.
 *         # The second check handles another thread reserving it first.
*/
      __pyx_t_4 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_category_must_be_in_0_2_32_1};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 189, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":188
 *         return <uint32_t>domain.get_category_id(category)
 *     if isinstance(category, int):
 *         if category < 0 or category > 0xFFFFFFFF:             # <<<<<<<<<<<<<<
 *             raise ValueError("category must be in [0, 2**32 - 1]")
 *         # Reserving an integer category only needs the lifecycle lock once.
*/
    }

    /* "nvtx/_lib/writer.pyx":192
 *         # Reserving an integer category only needs the lifecycle lock once.
 *         # The second check handles another thread reserving it first.
 *         if category not in domain._user_category_ids:             # <<<<<<<<<<<<<<
 *             with domain._session._lock:
 *                 domain._ensure_valid()
*/
    if (unlikely(__pyx_v_domain->_user_category_ids == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 192, __pyx_L1_error)
    }
    __pyx_t_1 = (__Pyx_PySet_ContainsTF(__pyx_v_category, __pyx_v_domain->_user_category_ids, Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 192, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "nvtx/_lib/writer.pyx":193
 *         # The second check handles another thread reserving it first.
 *         if category not in domain._user_category_ids:
 *             with domain._session._lock:             # <<<<<<<<<<<<<<
 *                 domain._ensure_valid()
 *                 if category not in domain._user_category_ids:
*/
      /*with:*/ {
        __pyx_t_7 = __Pyx_PyObject_LookupSpecial(__pyx_v_domain->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 193, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_4 = NULL;
        __pyx_t_8 = __Pyx_PyObject_LookupSpecial(__pyx_v_domain->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 193, __pyx_L12_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
          assert(__pyx_t_4);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
          __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L12_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        /*try:*/ {
          {
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
            __Pyx_XGOTREF(__pyx_t_9);
            __Pyx_XGOTREF(__pyx_t_10);
            __Pyx_XGOTREF(__pyx_t_11);
            /*try:*/ {

              /* "nvtx/_lib/writer.pyx":194
 *         if category not in domain._user_category_ids:
 *             with domain._session._lock:
 *                 domain._ensure_valid()             # <<<<<<<<<<<<<<
 *                 if category not in domain._user_category_ids:
 *                     domain._user_category_ids.add(category)
*/
              __pyx_t_3 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_domain->__pyx_vtab)->_ensure_valid(__pyx_v_domain); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 194, __pyx_L16_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "nvtx/_lib/writer.pyx":195
 *             with domain._session._lock:
 *                 domain._ensure_valid()
 *                 if category not in domain._user_category_ids:             # <<<<<<<<<<<<<<
 *                     domain._user_category_ids.add(category)
 *         return <uint32_t>category
*/
              if (unlikely(__pyx_v_domain->_user_category_ids == Py_None)) {
                PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
                __PYX_ERR(0, 195, __pyx_L16_error)
              }
              __pyx_t_1 = (__Pyx_PySet_ContainsTF(__pyx_v_category, __pyx_v_domain->_user_category_ids, Py_NE)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 195, __pyx_L16_error)
              if (__pyx_t_1) {

                /* "nvtx/_lib/writer.pyx":196
 *                 domain._ensure_valid()
 *                 if category not in domain._user_category_ids:
 *                     domain._user_category_ids.add(category)             # <<<<<<<<<<<<<<
 *         return <uint32_t>category
 *     raise TypeError("category must be a str, bytes, int, or None")
*/
                if (unlikely(__pyx_v_domain->_user_category_ids == Py_None)) {
                  PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
                  __PYX_ERR(0, 196, __pyx_L16_error)
                }
                __pyx_t_12 = PySet_Add(__pyx_v_domain->_user_category_ids, __pyx_v_category); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 196, __pyx_L16_error)

                /* "nvtx/_lib/writer.pyx":195
 *             with domain._session._lock:
 *                 domain._ensure_valid()
 *                 if category not in domain._user_category_ids:             # <<<<<<<<<<<<<<
 *                     domain._user_category_ids.add(category)
 *         return <uint32_t>category
*/
              }

              /* "nvtx/_lib/writer.pyx":193
 *         # The second check handles another thread reserving it first.
 *         if category not in domain._user_category_ids:
 *             with domain._session._lock:             # <<<<<<<<<<<<<<
 *                 domain._ensure_valid()
 *                 if category not in domain._user_category_ids:
*/
            }
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            goto __pyx_L21_try_end;
            __pyx_L16_error:;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            /*except:*/ {
              __Pyx_AddTraceback("nvtx._lib.writer._resolve_category", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_8, &__pyx_t_4) < 0) __PYX_ERR(0, 193, __pyx_L18_except_error)
              __Pyx_XGOTREF(__pyx_t_3);
              __Pyx_XGOTREF(__pyx_t_8);
              __Pyx_XGOTREF(__pyx_t_4);
              __pyx_t_13 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_8, __pyx_t_4); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 193, __pyx_L18_except_error)
              __Pyx_GOTREF(__pyx_t_13);
              __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_13, NULL);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
              if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 193, __pyx_L18_except_error)
              __Pyx_GOTREF(__pyx_t_14);
              __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_14);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              if (__pyx_t_1 < (0)) __PYX_ERR(0, 193, __pyx_L18_except_error)
              __pyx_t_2 = (!__pyx_t_1);
              if (unlikely(__pyx_t_2)) {
                __Pyx_GIVEREF(__pyx_t_3);
                __Pyx_GIVEREF(__pyx_t_8);
                __Pyx_XGIVEREF(__pyx_t_4);
                __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_8, __pyx_t_4);
                __pyx_t_3 = 0;  __pyx_t_8 = 0;  __pyx_t_4 = 0; 
                __PYX_ERR(0, 193, __pyx_L18_except_error)
              }
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              goto __pyx_L17_exception_handled;
            }
            __pyx_L18_except_error:;
            __Pyx_XGIVEREF(__pyx_t_9);
            __Pyx_XGIVEREF(__pyx_t_10);
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
            goto __pyx_L1_error;
            __pyx_L17_exception_handled:;
            __Pyx_XGIVEREF(__pyx_t_9);
            __Pyx_XGIVEREF(__pyx_t_10);
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
            __pyx_L21_try_end:;
          }
        }
        /*finally:*/ {
          /*normal exit:*/{
            if (__pyx_t_7) {
              __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_mstate_global->__pyx_tuple[0], NULL);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 193, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_11);
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            }
            goto __pyx_L15;
          }
          __pyx_L15:;
        }
        goto __pyx_L26;
        __pyx_L12_error:;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L1_error;
        __pyx_L26:;
      }

      /* "nvtx/_lib/writer.pyx":192
 *         # Reserving an integer category only needs the lifecycle lock once.
 *         # The second check handles another thread reserving it first.
 *         if category not in domain._user_category_ids:             # <<<<<<<<<<<<<<
 *             with domain._session._lock:
 *                 domain._ensure_valid()
*/
    }

    /* "nvtx/_lib/writer.pyx":197
 *                 if category not in domain._user_category_ids:
 *                     domain._user_category_ids.add(category)
 *         return <uint32_t>category             # <<<<<<<<<<<<<<
 *     raise TypeError("category must be a str, bytes, int, or None")
 * 
*/
    __pyx_t_6 = __Pyx_PyLong_As_uint32_t(__pyx_v_category); if (unlikely((__pyx_t_6 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 197, __pyx_L1_error)
    __pyx_r = ((uint32_t)__pyx_t_6);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":187
 *     if isinstance(category, (str, bytes)):
 *         return <uint32_t>domain.get_category_id(category)
 *     if isinstance(category, int):             # <<<<<<<<<<<<<<
 *         if category < 0 or category > 0xFFFFFFFF:
 *             raise ValueError("category must be in [0, 2**32 - 1]")
*/
  }

  /* "nvtx/_lib/writer.pyx":198
 *                     domain._user_category_ids.add(category)
 *         return <uint32_t>category
 *     raise TypeError("category must be a str, bytes, int, or None")             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_8 = NULL;
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_mstate_global->__pyx_kp_u_category_must_be_a_str_bytes_int};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 198, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __Pyx_Raise(__pyx_t_4, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(0, 198, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":180
 * 
 * 
 * cdef uint32_t _resolve_category(Domain domain, object category) except? 0:             # <<<<<<<<<<<<<<
 *     # None -> 0 (no category); str/bytes are named via the domain; ints pass
 *     # through and are reserved so the named-category allocator skips them.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("nvtx._lib.writer._resolve_category", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":201
 * 
 * 
 * cdef uint64_t _resolve_scope_id(Domain domain, object scope) except? 0:             # <<<<<<<<<<<<<<
 *     if scope is None:
 *         return NVTX_SCOPE_NONE
*/

static uint64_t __pyx_f_4nvtx_4_lib_6writer__resolve_scope_id(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, PyObject *__pyx_v_scope) {
  uint64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  uint64_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[5];
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_resolve_scope_id", 0);

  /* "nvtx/_lib/writer.pyx":202
 * 
 * cdef uint64_t _resolve_scope_id(Domain domain, object scope) except? 0:
 *     if scope is None:             # <<<<<<<<<<<<<<
 *         return NVTX_SCOPE_NONE
 *     if isinstance(scope, PredefinedScope):
*/
  __pyx_t_1 = (__pyx_v_scope == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":203
 * cdef uint64_t _resolve_scope_id(Domain domain, object scope) except? 0:
 *     if scope is None:
 *         return NVTX_SCOPE_NONE             # <<<<<<<<<<<<<<
 *     if isinstance(scope, PredefinedScope):
 *         return <uint64_t>scope.value
*/
    __pyx_r = NVTX_SCOPE_NONE;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":202
 * 
 * cdef uint64_t _resolve_scope_id(Domain domain, object scope) except? 0:
 *     if scope is None:             # <<<<<<<<<<<<<<
 *         return NVTX_SCOPE_NONE
 *     if isinstance(scope, PredefinedScope):
*/
  }

  /* "nvtx/_lib/writer.pyx":204
 *     if scope is None:
 *         return NVTX_SCOPE_NONE
 *     if isinstance(scope, PredefinedScope):             # <<<<<<<<<<<<<<
 *         return <uint64_t>scope.value
 *     if isinstance(scope, Scope):
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_PredefinedScope); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_IsInstance(__pyx_v_scope, __pyx_t_2); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":205
 *         return NVTX_SCOPE_NONE
 *     if isinstance(scope, PredefinedScope):
 *         return <uint64_t>scope.value             # <<<<<<<<<<<<<<
 *     if isinstance(scope, Scope):
 *         if domain is None:
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_scope, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyLong_As_uint64_t(__pyx_t_2); if (unlikely((__pyx_t_3 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = ((uint64_t)__pyx_t_3);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":204
 *     if scope is None:
 *         return NVTX_SCOPE_NONE
 *     if isinstance(scope, PredefinedScope):             # <<<<<<<<<<<<<<
 *         return <uint64_t>scope.value
 *     if isinstance(scope, Scope):
*/
  }

  /* "nvtx/_lib/writer.pyx":206
 *     if isinstance(scope, PredefinedScope):
 *         return <uint64_t>scope.value
 *     if isinstance(scope, Scope):             # <<<<<<<<<<<<<<
 *         if domain is None:
 *             raise ValueError("scope requires a registered stream domain")
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_scope, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Scope); 
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":207
 *         return <uint64_t>scope.value
 *     if isinstance(scope, Scope):
 *         if domain is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("scope requires a registered stream domain")
 *         if (<Scope>scope)._domain is not domain:
*/
    __pyx_t_1 = (((PyObject *)__pyx_v_domain) == Py_None);
    if (unlikely(__pyx_t_1)) {

      /* "nvtx/_lib/writer.pyx":208
 *     if isinstance(scope, Scope):
 *         if domain is None:
 *             raise ValueError("scope requires a registered stream domain")             # <<<<<<<<<<<<<<
 *         if (<Scope>scope)._domain is not domain:
 *             raise ValueError("scope is not registered in this domain")
*/
      __pyx_t_4 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_scope_requires_a_registered_stre};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 208, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 208, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":207
 *         return <uint64_t>scope.value
 *     if isinstance(scope, Scope):
 *         if domain is None:             # <<<<<<<<<<<<<<
 *             raise ValueError("scope requires a registered stream domain")
 *         if (<Scope>scope)._domain is not domain:
*/
    }

    /* "nvtx/_lib/writer.pyx":209
 *         if domain is None:
 *             raise ValueError("scope requires a registered stream domain")
 *         if (<Scope>scope)._domain is not domain:             # <<<<<<<<<<<<<<
 *             raise ValueError("scope is not registered in this domain")
 *         return (<Scope>scope)._scope_id
*/
    __pyx_t_1 = (((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_scope)->_domain != ((PyObject *)__pyx_v_domain));
    if (unlikely(__pyx_t_1)) {

      /* "nvtx/_lib/writer.pyx":210
 *             raise ValueError("scope requires a registered stream domain")
 *         if (<Scope>scope)._domain is not domain:
 *             raise ValueError("scope is not registered in this domain")             # <<<<<<<<<<<<<<
 *         return (<Scope>scope)._scope_id
 *     if isinstance(scope, int) and not isinstance(scope, bool):
*/
      __pyx_t_4 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_scope_is_not_registered_in_this};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 210, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":209
 *         if domain is None:
 *             raise ValueError("scope requires a registered stream domain")
 *         if (<Scope>scope)._domain is not domain:             # <<<<<<<<<<<<<<
 *             raise ValueError("scope is not registered in this domain")
 *         return (<Scope>scope)._scope_id
*/
    }

    /* "nvtx/_lib/writer.pyx":211
 *         if (<Scope>scope)._domain is not domain:
 *             raise ValueError("scope is not registered in this domain")
 *         return (<Scope>scope)._scope_id             # <<<<<<<<<<<<<<
 *     if isinstance(scope, int) and not isinstance(scope, bool):
 *         if not (
*/
    __pyx_r = ((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_scope)->_scope_id;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":206
 *     if isinstance(scope, PredefinedScope):
 *         return <uint64_t>scope.value
 *     if isinstance(scope, Scope):             # <<<<<<<<<<<<<<
 *         if domain is None:
 *             raise ValueError("scope requires a registered stream domain")
*/
  }

  /* "nvtx/_lib/writer.pyx":212
 *             raise ValueError("scope is not registered in this domain")
 *         return (<Scope>scope)._scope_id
 *     if isinstance(scope, int) and not isinstance(scope, bool):             # <<<<<<<<<<<<<<
 *         if not (
 *             NVTX_SCOPE_ID_STATIC_START
*/
  __pyx_t_6 = PyLong_Check(__pyx_v_scope); 
  if (__pyx_t_6) {
  } else {
    __pyx_t_1 = __pyx_t_6;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_6 = PyBool_Check(__pyx_v_scope); 
  __pyx_t_7 = (!__pyx_t_6);
  __pyx_t_1 = __pyx_t_7;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":214
 *     if isinstance(scope, int) and not isinstance(scope, bool):
 *         if not (
 *             NVTX_SCOPE_ID_STATIC_START             # <<<<<<<<<<<<<<
 *             <= scope
 *             < NVTX_SCOPE_ID_DYNAMIC_START
*/
    __pyx_t_2 = __Pyx_PyLong_From_uint64_t(NVTX_SCOPE_ID_STATIC_START); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "nvtx/_lib/writer.pyx":215
 *         if not (
 *             NVTX_SCOPE_ID_STATIC_START
 *             <= scope             # <<<<<<<<<<<<<<
 *             < NVTX_SCOPE_ID_DYNAMIC_START
 *         ):
*/
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_v_scope, Py_LE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 215, __pyx_L1_error)
    if (__Pyx_PyObject_IsTrue(__pyx_t_4)) {
      __Pyx_DECREF(__pyx_t_4);

      /* "nvtx/_lib/writer.pyx":216
 *             NVTX_SCOPE_ID_STATIC_START
 *             <= scope
 *             < NVTX_SCOPE_ID_DYNAMIC_START             # <<<<<<<<<<<<<<
 *         ):
 *             raise ValueError(
*/
      __pyx_t_8 = __Pyx_PyLong_From_uint64_t(NVTX_SCOPE_ID_DYNAMIC_START); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_scope, __pyx_t_8, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "nvtx/_lib/writer.pyx":215
 *         if not (
 *             NVTX_SCOPE_ID_STATIC_START
 *             <= scope             # <<<<<<<<<<<<<<
 *             < NVTX_SCOPE_ID_DYNAMIC_START
 *         ):
*/
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/writer.pyx":213
 *         return (<Scope>scope)._scope_id
 *     if isinstance(scope, int) and not isinstance(scope, bool):
 *         if not (             # <<<<<<<<<<<<<<
 *             NVTX_SCOPE_ID_STATIC_START
 *             <= scope
*/
    __pyx_t_7 = (!__pyx_t_1);
    if (unlikely(__pyx_t_7)) {

      /* "nvtx/_lib/writer.pyx":218
 *             < NVTX_SCOPE_ID_DYNAMIC_START
 *         ):
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "integer scope must be in the NVTX static scope ID range "
 *                 f"[{NVTX_SCOPE_ID_STATIC_START}, "
*/
      __pyx_t_2 = NULL;

      /* "nvtx/_lib/writer.pyx":220
 *             raise ValueError(
 *                 "integer scope must be in the NVTX static scope ID range "
 *                 f"[{NVTX_SCOPE_ID_STATIC_START}, "             # <<<<<<<<<<<<<<
 *                 f"{NVTX_SCOPE_ID_DYNAMIC_START})"
 *             )
*/
      __pyx_t_8 = __Pyx_PyUnicode_From_uint64_t(NVTX_SCOPE_ID_STATIC_START, 0, ' ', 'd'); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 220, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);

      /* "nvtx/_lib/writer.pyx":221
 *                 "integer scope must be in the NVTX static scope ID range "
 *                 f"[{NVTX_SCOPE_ID_STATIC_START}, "
 *                 f"{NVTX_SCOPE_ID_DYNAMIC_START})"             # <<<<<<<<<<<<<<
 *             )
 *         return <uint64_t>scope
*/
      __pyx_t_9 = __Pyx_PyUnicode_From_uint64_t(NVTX_SCOPE_ID_DYNAMIC_START, 0, ' ', 'd'); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 221, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_integer_scope_must_be_in_the_NVT;
      __pyx_t_10[1] = __pyx_t_8;
      __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u__2;
      __pyx_t_10[3] = __pyx_t_9;
      __pyx_t_10[4] = __pyx_mstate_global->__pyx_kp_u_;

      /* "nvtx/_lib/writer.pyx":219
 *         ):
 *             raise ValueError(
 *                 "integer scope must be in the NVTX static scope ID range "             # <<<<<<<<<<<<<<
 *                 f"[{NVTX_SCOPE_ID_STATIC_START}, "
 *                 f"{NVTX_SCOPE_ID_DYNAMIC_START})"
*/
      __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 5, 57 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9) + 1, 127);
      if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_11};
        __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 218, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __PYX_ERR(0, 218, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":213
 *         return (<Scope>scope)._scope_id
 *     if isinstance(scope, int) and not isinstance(scope, bool):
 *         if not (             # <<<<<<<<<<<<<<
 *             NVTX_SCOPE_ID_STATIC_START
 *             <= scope
*/
    }

    /* "nvtx/_lib/writer.pyx":223
 *                 f"{NVTX_SCOPE_ID_DYNAMIC_START})"
 *             )
 *         return <uint64_t>scope             # <<<<<<<<<<<<<<
 *     raise TypeError(
 *         "scope must be None, a PredefinedScope, a Scope, or an int"
*/
    __pyx_t_3 = __Pyx_PyLong_As_uint64_t(__pyx_v_scope); if (unlikely((__pyx_t_3 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 223, __pyx_L1_error)
    __pyx_r = ((uint64_t)__pyx_t_3);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":212
 *             raise ValueError("scope is not registered in this domain")
 *         return (<Scope>scope)._scope_id
 *     if isinstance(scope, int) and not isinstance(scope, bool):             # <<<<<<<<<<<<<<
 *         if not (
 *             NVTX_SCOPE_ID_STATIC_START
*/
  }

  /* "nvtx/_lib/writer.pyx":224
 *             )
 *         return <uint64_t>scope
 *     raise TypeError(             # <<<<<<<<<<<<<<
 *         "scope must be None, a PredefinedScope, a Scope, or an int"
 *     )
*/
  __pyx_t_11 = NULL;
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_mstate_global->__pyx_kp_u_scope_must_be_None_a_PredefinedS};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __Pyx_Raise(__pyx_t_4, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(0, 224, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":201
 * 
 * 
 * cdef uint64_t _resolve_scope_id(Domain domain, object scope) except? 0:             # <<<<<<<<<<<<<<
 *     if scope is None:
 *         return NVTX_SCOPE_NONE
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("nvtx._lib.writer._resolve_scope_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":229
 * 
 * 
 * cdef uint64_t _resolve_stream_scope_id(             # <<<<<<<<<<<<<<
 *     Domain domain, object scope
 * ) except? 0:
*/

static uint64_t __pyx_f_4nvtx_4_lib_6writer__resolve_stream_scope_id(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, PyObject *__pyx_v_scope) {
  uint64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  uint64_t __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_resolve_stream_scope_id", 0);

  /* "nvtx/_lib/writer.pyx":232
 *     Domain domain, object scope
 * ) except? 0:
 *     if isinstance(scope, PredefinedScope):             # <<<<<<<<<<<<<<
 *         if scope is PredefinedScope.NONE or scope is PredefinedScope.ROOT:
 *             return <uint64_t>scope.value
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_PredefinedScope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_scope, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 232, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/writer.pyx":233
 * ) except? 0:
 *     if isinstance(scope, PredefinedScope):
 *         if scope is PredefinedScope.NONE or scope is PredefinedScope.ROOT:             # <<<<<<<<<<<<<<
 *             return <uint64_t>scope.value
 *         raise ValueError(
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_PredefinedScope); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_NONE); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = (__pyx_v_scope == __pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L5_bool_binop_done;
    }
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PredefinedScope); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_ROOT); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_4 = (__pyx_v_scope == __pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_t_4;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_2) {

      /* "nvtx/_lib/writer.pyx":234
 *     if isinstance(scope, PredefinedScope):
 *         if scope is PredefinedScope.NONE or scope is PredefinedScope.ROOT:
 *             return <uint64_t>scope.value             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "runtime-resolved scopes are not valid as stream defaults"
*/
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_scope, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 234, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyLong_As_uint64_t(__pyx_t_1); if (unlikely((__pyx_t_5 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 234, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_r = ((uint64_t)__pyx_t_5);
      goto __pyx_L0;

      /* "nvtx/_lib/writer.pyx":233
 * ) except? 0:
 *     if isinstance(scope, PredefinedScope):
 *         if scope is PredefinedScope.NONE or scope is PredefinedScope.ROOT:             # <<<<<<<<<<<<<<
 *             return <uint64_t>scope.value
 *         raise ValueError(
*/
    }

    /* "nvtx/_lib/writer.pyx":235
 *         if scope is PredefinedScope.NONE or scope is PredefinedScope.ROOT:
 *             return <uint64_t>scope.value
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "runtime-resolved scopes are not valid as stream defaults"
 *         )
*/
    __pyx_t_3 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_runtime_resolved_scopes_are_not};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 235, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":232
 *     Domain domain, object scope
 * ) except? 0:
 *     if isinstance(scope, PredefinedScope):             # <<<<<<<<<<<<<<
 *         if scope is PredefinedScope.NONE or scope is PredefinedScope.ROOT:
 *             return <uint64_t>scope.value
*/
  }

  /* "nvtx/_lib/writer.pyx":238
 *             "runtime-resolved scopes are not valid as stream defaults"
 *         )
 *     return _resolve_scope_id(domain, scope)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_5 = __pyx_f_4nvtx_4_lib_6writer__resolve_scope_id(__pyx_v_domain, __pyx_v_scope); if (unlikely(__pyx_t_5 == ((uint64_t)0) && PyErr_Occurred())) __PYX_ERR(0, 238, __pyx_L1_error)
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":229
 * 
 * 
 * cdef uint64_t _resolve_stream_scope_id(             # <<<<<<<<<<<<<<
 *     Domain domain, object scope
 * ) except? 0:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer._resolve_stream_scope_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":241
 * 
 * 
 * def _validate_counter_layout(dtype):             # <<<<<<<<<<<<<<
 *     """
 *     Reject event entry roles in a counter layout; return the name of the
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_1_validate_counter_layout(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer__validate_counter_layout, "_validate_counter_layout(dtype)\n\nReject event entry roles in a counter layout; return the name of the\nembedded ``EntryKind.COUNTER_TIMESTAMP`` field, or ``None``.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_1_validate_counter_layout = {"_validate_counter_layout", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_1_validate_counter_layout, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer__validate_counter_layout};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_1_validate_counter_layout(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_dtype = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_validate_counter_layout (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_dtype,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 241, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 241, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_validate_counter_layout", 0) < (0)) __PYX_ERR(0, 241, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_validate_counter_layout", 1, 1, 1, i); __PYX_ERR(0, 241, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 241, __pyx_L3_error)
    }
    __pyx_v_dtype = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_validate_counter_layout", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 241, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer._validate_counter_layout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer__validate_counter_layout(__pyx_self, __pyx_v_dtype);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer__validate_counter_layout(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_v_timestamp_fields = NULL;
  PyObject *__pyx_v_field_name = NULL;
  PyObject *__pyx_v_metadata = NULL;
  PyObject *__pyx_v_entry_kind = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13[5];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_validate_counter_layout", 0);

  /* "nvtx/_lib/writer.pyx":246
 *     embedded ``EntryKind.COUNTER_TIMESTAMP`` field, or ``None``.
 *     """
 *     if dtype is int or dtype is float:             # <<<<<<<<<<<<<<
 *         return None
 *     if dtype.subdtype is not None:
*/
  __pyx_t_2 = (__pyx_v_dtype == ((PyObject *)(&PyLong_Type)));
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_dtype == ((PyObject *)(&PyFloat_Type)));
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":247
 *     """
 *     if dtype is int or dtype is float:
 *         return None             # <<<<<<<<<<<<<<
 *     if dtype.subdtype is not None:
 *         raise TypeError(
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":246
 *     embedded ``EntryKind.COUNTER_TIMESTAMP`` field, or ``None``.
 *     """
 *     if dtype is int or dtype is float:             # <<<<<<<<<<<<<<
 *         return None
 *     if dtype.subdtype is not None:
*/
  }

  /* "nvtx/_lib/writer.pyx":248
 *     if dtype is int or dtype is float:
 *         return None
 *     if dtype.subdtype is not None:             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             "Top-level fixed-size array counter dtypes are not supported."
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = (__pyx_t_3 != Py_None);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":249
 *         return None
 *     if dtype.subdtype is not None:
 *         raise TypeError(             # <<<<<<<<<<<<<<
 *             "Top-level fixed-size array counter dtypes are not supported."
 *         )
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Top_level_fixed_size_array_count};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 249, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 249, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":248
 *     if dtype is int or dtype is float:
 *         return None
 *     if dtype.subdtype is not None:             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             "Top-level fixed-size array counter dtypes are not supported."
*/
  }

  /* "nvtx/_lib/writer.pyx":252
 *             "Top-level fixed-size array counter dtypes are not supported."
 *         )
 *     _validate_native_byte_order(dtype, "a counter layout")             # <<<<<<<<<<<<<<
 *     if getattr(dtype, "fields", None) is None:
 *         return None
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_validate_native_byte_order); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_v_dtype, __pyx_mstate_global->__pyx_kp_u_a_counter_layout};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":253
 *         )
 *     _validate_native_byte_order(dtype, "a counter layout")
 *     if getattr(dtype, "fields", None) is None:             # <<<<<<<<<<<<<<
 *         return None
 *     timestamp_fields = []
*/
  __pyx_t_3 = __Pyx_GetAttr3(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_fields, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = (__pyx_t_3 == Py_None);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":254
 *     _validate_native_byte_order(dtype, "a counter layout")
 *     if getattr(dtype, "fields", None) is None:
 *         return None             # <<<<<<<<<<<<<<
 *     timestamp_fields = []
 *     for field_name in dtype.names:
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":253
 *         )
 *     _validate_native_byte_order(dtype, "a counter layout")
 *     if getattr(dtype, "fields", None) is None:             # <<<<<<<<<<<<<<
 *         return None
 *     timestamp_fields = []
*/
  }

  /* "nvtx/_lib/writer.pyx":255
 *     if getattr(dtype, "fields", None) is None:
 *         return None
 *     timestamp_fields = []             # <<<<<<<<<<<<<<
 *     for field_name in dtype.names:
 *         metadata = _nvtx_metadata_from_dtype(dtype.fields[field_name][0])
*/
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_timestamp_fields = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":256
 *         return None
 *     timestamp_fields = []
 *     for field_name in dtype.names:             # <<<<<<<<<<<<<<
 *         metadata = _nvtx_metadata_from_dtype(dtype.fields[field_name][0])
 *         entry_kind = getattr(metadata, "entry_kind", None)
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_names); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_6 = __pyx_t_3; __Pyx_INCREF(__pyx_t_6);
    __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 256, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_6))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_6);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 256, __pyx_L1_error)
          #endif
          if (__pyx_t_7 >= __pyx_temp) break;
        }
        __pyx_t_3 = __Pyx_PyList_GET_ITEM_REF(__pyx_t_6, __pyx_t_7, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_7;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_6);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 256, __pyx_L1_error)
          #endif
          if (__pyx_t_7 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_7));
        #else
        __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_6, __pyx_t_7);
        #endif
        ++__pyx_t_7;
      }
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 256, __pyx_L1_error)
    } else {
      __pyx_t_3 = __pyx_t_8(__pyx_t_6);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 256, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_field_name, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":257
 *     timestamp_fields = []
 *     for field_name in dtype.names:
 *         metadata = _nvtx_metadata_from_dtype(dtype.fields[field_name][0])             # <<<<<<<<<<<<<<
 *         entry_kind = getattr(metadata, "entry_kind", None)
 *         if entry_kind is None:
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_nvtx_metadata_from_dtype); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = __Pyx_PyObject_GetItem(__pyx_t_10, __pyx_v_field_name); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = __Pyx_GetItemInt(__pyx_t_11, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_9);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_10};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_metadata, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":258
 *     for field_name in dtype.names:
 *         metadata = _nvtx_metadata_from_dtype(dtype.fields[field_name][0])
 *         entry_kind = getattr(metadata, "entry_kind", None)             # <<<<<<<<<<<<<<
 *         if entry_kind is None:
 *             continue
*/
    __pyx_t_3 = __Pyx_GetAttr3(__pyx_v_metadata, __pyx_mstate_global->__pyx_n_u_entry_kind, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_entry_kind, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":259
 *         metadata = _nvtx_metadata_from_dtype(dtype.fields[field_name][0])
 *         entry_kind = getattr(metadata, "entry_kind", None)
 *         if entry_kind is None:             # <<<<<<<<<<<<<<
 *             continue
 *         if entry_kind is EntryKind.COUNTER_TIMESTAMP:
*/
    __pyx_t_1 = (__pyx_v_entry_kind == Py_None);
    if (__pyx_t_1) {

      /* "nvtx/_lib/writer.pyx":260
 *         entry_kind = getattr(metadata, "entry_kind", None)
 *         if entry_kind is None:
 *             continue             # <<<<<<<<<<<<<<
 *         if entry_kind is EntryKind.COUNTER_TIMESTAMP:
 *             timestamp_fields.append(field_name)
*/
      goto __pyx_L8_continue;

      /* "nvtx/_lib/writer.pyx":259
 *         metadata = _nvtx_metadata_from_dtype(dtype.fields[field_name][0])
 *         entry_kind = getattr(metadata, "entry_kind", None)
 *         if entry_kind is None:             # <<<<<<<<<<<<<<
 *             continue
 *         if entry_kind is EntryKind.COUNTER_TIMESTAMP:
*/
    }

    /* "nvtx/_lib/writer.pyx":261
 *         if entry_kind is None:
 *             continue
 *         if entry_kind is EntryKind.COUNTER_TIMESTAMP:             # <<<<<<<<<<<<<<
 *             timestamp_fields.append(field_name)
 *         else:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_COUNTER_TIMESTAMP); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = (__pyx_v_entry_kind == __pyx_t_9);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(__pyx_t_1)) {

      /* "nvtx/_lib/writer.pyx":262
 *             continue
 *         if entry_kind is EntryKind.COUNTER_TIMESTAMP:
 *             timestamp_fields.append(field_name)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError(
*/
      __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_timestamp_fields, __pyx_v_field_name); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 262, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":261
 *         if entry_kind is None:
 *             continue
 *         if entry_kind is EntryKind.COUNTER_TIMESTAMP:             # <<<<<<<<<<<<<<
 *             timestamp_fields.append(field_name)
 *         else:
*/
      goto __pyx_L11;
    }

    /* "nvtx/_lib/writer.pyx":264
 *             timestamp_fields.append(field_name)
 *         else:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 f"EntryKind.{entry_kind.name} is not valid in a counter "
 *                 f"layout (field {field_name!r})"
*/
    /*else*/ {
      __pyx_t_3 = NULL;

      /* "nvtx/_lib/writer.pyx":265
 *         else:
 *             raise ValueError(
 *                 f"EntryKind.{entry_kind.name} is not valid in a counter "             # <<<<<<<<<<<<<<
 *                 f"layout (field {field_name!r})"
 *             )
*/
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_entry_kind, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 265, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_t_10, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 265, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "nvtx/_lib/writer.pyx":266
 *             raise ValueError(
 *                 f"EntryKind.{entry_kind.name} is not valid in a counter "
 *                 f"layout (field {field_name!r})"             # <<<<<<<<<<<<<<
 *             )
 *     if len(timestamp_fields) > 1:
*/
      __pyx_t_10 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_field_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 266, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_13[0] = __pyx_mstate_global->__pyx_kp_u_EntryKind_2;
      __pyx_t_13[1] = __pyx_t_4;
      __pyx_t_13[2] = __pyx_mstate_global->__pyx_kp_u_is_not_valid_in_a_counter_layou;
      __pyx_t_13[3] = __pyx_t_10;
      __pyx_t_13[4] = __pyx_mstate_global->__pyx_kp_u_;

      /* "nvtx/_lib/writer.pyx":265
 *         else:
 *             raise ValueError(
 *                 f"EntryKind.{entry_kind.name} is not valid in a counter "             # <<<<<<<<<<<<<<
 *                 f"layout (field {field_name!r})"
 *             )
*/
      __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_13, 5, 10 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4) + 41 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10));
      if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 265, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_11};
        __pyx_t_9 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 264, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
      }
      __Pyx_Raise(__pyx_t_9, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __PYX_ERR(0, 264, __pyx_L1_error)
    }
    __pyx_L11:;

    /* "nvtx/_lib/writer.pyx":256
 *         return None
 *     timestamp_fields = []
 *     for field_name in dtype.names:             # <<<<<<<<<<<<<<
 *         metadata = _nvtx_metadata_from_dtype(dtype.fields[field_name][0])
 *         entry_kind = getattr(metadata, "entry_kind", None)
*/
    __pyx_L8_continue:;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "nvtx/_lib/writer.pyx":268
 *                 f"layout (field {field_name!r})"
 *             )
 *     if len(timestamp_fields) > 1:             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "a counter layout may embed at most one "
*/
  __pyx_t_7 = __Pyx_PyList_GET_SIZE(__pyx_v_timestamp_fields); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 268, __pyx_L1_error)
  __pyx_t_1 = (__pyx_t_7 > 1);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":269
 *             )
 *     if len(timestamp_fields) > 1:
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "a counter layout may embed at most one "
 *             "EntryKind.COUNTER_TIMESTAMP field"
*/
    __pyx_t_9 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_a_counter_layout_may_embed_at_mo};
      __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 269, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 269, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":268
 *                 f"layout (field {field_name!r})"
 *             )
 *     if len(timestamp_fields) > 1:             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "a counter layout may embed at most one "
*/
  }

  /* "nvtx/_lib/writer.pyx":273
 *             "EntryKind.COUNTER_TIMESTAMP field"
 *         )
 *     return timestamp_fields[0] if timestamp_fields else None             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  {
    Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_v_timestamp_fields);
    if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 273, __pyx_L1_error)
    __pyx_t_1 = (__pyx_temp != 0);
  }

  if (__pyx_t_1) {
    __pyx_t_9 = __Pyx_GetItemInt_List(__pyx_v_timestamp_fields, 0, long, 1, __Pyx_PyLong_From_long, 1, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_6 = __pyx_t_9;
    __pyx_t_9 = 0;
  } else {
    __Pyx_INCREF(Py_None);
    __pyx_t_6 = Py_None;
  }
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":241
 * 
 * 
 * def _validate_counter_layout(dtype):             # <<<<<<<<<<<<<<
 *     """
 *     Reject event entry roles in a counter layout; return the name of the
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("nvtx._lib.writer._validate_counter_layout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_timestamp_fields);
  __Pyx_XDECREF(__pyx_v_field_name);
  __Pyx_XDECREF(__pyx_v_metadata);
  __Pyx_XDECREF(__pyx_v_entry_kind);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":276
 * 
 * 
 * def _batch_rows(data, dt):             # <<<<<<<<<<<<<<
 *     """Return batch data as a flat, contiguous array of ``dt`` rows."""
 *     if isinstance(data, np.ndarray):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_3_batch_rows(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_2_batch_rows, "_batch_rows(data, dt)\n\nReturn batch data as a flat, contiguous array of ``dt`` rows.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_3_batch_rows = {"_batch_rows", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_3_batch_rows, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_2_batch_rows};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_3_batch_rows(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_dt = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_batch_rows (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_data,&__pyx_mstate_global->__pyx_n_u_dt,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 276, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 276, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 276, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_batch_rows", 0) < (0)) __PYX_ERR(0, 276, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_batch_rows", 1, 2, 2, i); __PYX_ERR(0, 276, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 276, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 276, __pyx_L3_error)
    }
    __pyx_v_data = values[0];
    __pyx_v_dt = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_batch_rows", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 276, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer._batch_rows", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_2_batch_rows(__pyx_self, __pyx_v_data, __pyx_v_dt);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_2_batch_rows(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data, PyObject *__pyx_v_dt) {
  PyObject *__pyx_v_rows = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_column = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_batch_rows", 0);

  /* "nvtx/_lib/writer.pyx":278
 * def _batch_rows(data, dt):
 *     """Return batch data as a flat, contiguous array of ``dt`` rows."""
 *     if isinstance(data, np.ndarray):             # <<<<<<<<<<<<<<
 *         if data.ndim != 1:
 *             raise ValueError("batch data must be one-dimensional")
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = PyObject_IsInstance(__pyx_v_data, __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":279
 *     """Return batch data as a flat, contiguous array of ``dt`` rows."""
 *     if isinstance(data, np.ndarray):
 *         if data.ndim != 1:             # <<<<<<<<<<<<<<
 *             raise ValueError("batch data must be one-dimensional")
 *         return np.ascontiguousarray(data, dtype=dt)
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_data, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_2, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(__pyx_t_3)) {

      /* "nvtx/_lib/writer.pyx":280
 *     if isinstance(data, np.ndarray):
 *         if data.ndim != 1:
 *             raise ValueError("batch data must be one-dimensional")             # <<<<<<<<<<<<<<
 *         return np.ascontiguousarray(data, dtype=dt)
 * 
*/
      __pyx_t_1 = NULL;
      __pyx_t_4 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_batch_data_must_be_one_dimension};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 280, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 280, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":279
 *     """Return batch data as a flat, contiguous array of ``dt`` rows."""
 *     if isinstance(data, np.ndarray):
 *         if data.ndim != 1:             # <<<<<<<<<<<<<<
 *             raise ValueError("batch data must be one-dimensional")
 *         return np.ascontiguousarray(data, dtype=dt)
*/
    }

    /* "nvtx/_lib/writer.pyx":281
 *         if data.ndim != 1:
 *             raise ValueError("batch data must be one-dimensional")
 *         return np.ascontiguousarray(data, dtype=dt)             # <<<<<<<<<<<<<<
 * 
 *     if pd is not None and isinstance(data, pd.DataFrame):
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 281, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_ascontiguousarray); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 281, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_1, __pyx_v_data};
      __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_dt, __pyx_t_5, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 281, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":278
 * def _batch_rows(data, dt):
 *     """Return batch data as a flat, contiguous array of ``dt`` rows."""
 *     if isinstance(data, np.ndarray):             # <<<<<<<<<<<<<<
 *         if data.ndim != 1:
 *             raise ValueError("batch data must be one-dimensional")
*/
  }

  /* "nvtx/_lib/writer.pyx":283
 *         return np.ascontiguousarray(data, dtype=dt)
 * 
 *     if pd is not None and isinstance(data, pd.DataFrame):             # <<<<<<<<<<<<<<
 *         rows = np.empty(len(data), dtype=dt)
 *         for name in dt.names:
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pd); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = (__pyx_t_2 != Py_None);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_7) {
  } else {
    __pyx_t_3 = __pyx_t_7;
    goto __pyx_L6_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pd); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_DataFrame); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = PyObject_IsInstance(__pyx_v_data, __pyx_t_6); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_3 = __pyx_t_7;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":284
 * 
 *     if pd is not None and isinstance(data, pd.DataFrame):
 *         rows = np.empty(len(data), dtype=dt)             # <<<<<<<<<<<<<<
 *         for name in dt.names:
 *             try:
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_empty); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_8 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1))) __PYX_ERR(0, 284, __pyx_L1_error)
    __pyx_t_5 = PyLong_FromSsize_t(__pyx_t_8); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_t_5};
      __pyx_t_9 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 284, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_dt, __pyx_t_9, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 284, __pyx_L1_error)
      __pyx_t_6 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_9);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 284, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __pyx_v_rows = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "nvtx/_lib/writer.pyx":285
 *     if pd is not None and isinstance(data, pd.DataFrame):
 *         rows = np.empty(len(data), dtype=dt)
 *         for name in dt.names:             # <<<<<<<<<<<<<<
 *             try:
 *                 column = data[name]
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_names); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
      __pyx_t_1 = __pyx_t_6; __Pyx_INCREF(__pyx_t_1);
      __pyx_t_8 = 0;
      __pyx_t_10 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 285, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 285, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    for (;;) {
      if (likely(!__pyx_t_10)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 285, __pyx_L1_error)
            #endif
            if (__pyx_t_8 >= __pyx_temp) break;
          }
          __pyx_t_6 = __Pyx_PyList_GET_ITEM_REF(__pyx_t_1, __pyx_t_8, __Pyx_ReferenceSharing_OwnStrongReference);
          ++__pyx_t_8;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 285, __pyx_L1_error)
            #endif
            if (__pyx_t_8 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_8));
          #else
          __pyx_t_6 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_8);
          #endif
          ++__pyx_t_8;
        }
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 285, __pyx_L1_error)
      } else {
        __pyx_t_6 = __pyx_t_10(__pyx_t_1);
        if (unlikely(!__pyx_t_6)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 285, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "nvtx/_lib/writer.pyx":286
 *         rows = np.empty(len(data), dtype=dt)
 *         for name in dt.names:
 *             try:             # <<<<<<<<<<<<<<
 *                 column = data[name]
 *             except KeyError:
*/
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_13);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":287
 *         for name in dt.names:
 *             try:
 *                 column = data[name]             # <<<<<<<<<<<<<<
 *             except KeyError:
 *                 raise ValueError(
*/
          __pyx_t_6 = __Pyx_PyObject_GetItem(__pyx_v_data, __pyx_v_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 287, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_XDECREF_SET(__pyx_v_column, __pyx_t_6);
          __pyx_t_6 = 0;

          /* "nvtx/_lib/writer.pyx":286
 *         rows = np.empty(len(data), dtype=dt)
 *         for name in dt.names:
 *             try:             # <<<<<<<<<<<<<<
 *                 column = data[name]
 *             except KeyError:
*/
        }
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L17_try_end;
        __pyx_L10_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

        /* "nvtx/_lib/writer.pyx":288
 *             try:
 *                 column = data[name]
 *             except KeyError:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     f"batch data is missing a column for dtype field {name!r}"
*/
        __pyx_t_14 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_KeyError))));
        if (__pyx_t_14) {
          __Pyx_AddTraceback("nvtx._lib.writer._batch_rows", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_9, &__pyx_t_5) < 0) __PYX_ERR(0, 288, __pyx_L12_except_error)
          __Pyx_XGOTREF(__pyx_t_6);
          __Pyx_XGOTREF(__pyx_t_9);
          __Pyx_XGOTREF(__pyx_t_5);

          /* "nvtx/_lib/writer.pyx":289
 *                 column = data[name]
 *             except KeyError:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     f"batch data is missing a column for dtype field {name!r}"
 *                 )
*/
          __pyx_t_15 = NULL;

          /* "nvtx/_lib/writer.pyx":290
 *             except KeyError:
 *                 raise ValueError(
 *                     f"batch data is missing a column for dtype field {name!r}"             # <<<<<<<<<<<<<<
 *                 )
 *             rows[name] = column.to_numpy(copy=False)
*/
          __pyx_t_16 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 290, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_16);
          __pyx_t_17 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_batch_data_is_missing_a_column_f, __pyx_t_16); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 290, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_17);
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          __pyx_t_4 = 1;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_t_17};
            __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 289, __pyx_L12_except_error)
            __Pyx_GOTREF(__pyx_t_2);
          }
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 289, __pyx_L12_except_error)
        }
        goto __pyx_L12_except_error;

        /* "nvtx/_lib/writer.pyx":286
 *         rows = np.empty(len(data), dtype=dt)
 *         for name in dt.names:
 *             try:             # <<<<<<<<<<<<<<
 *                 column = data[name]
 *             except KeyError:
*/
        __pyx_L12_except_error:;
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_13);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
        goto __pyx_L1_error;
        __pyx_L17_try_end:;
      }

      /* "nvtx/_lib/writer.pyx":292
 *                     f"batch data is missing a column for dtype field {name!r}"
 *                 )
 *             rows[name] = column.to_numpy(copy=False)             # <<<<<<<<<<<<<<
 *         return rows
 * 
*/
      __pyx_t_9 = __pyx_v_column;
      __Pyx_INCREF(__pyx_t_9);
      __pyx_t_4 = 0;
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_9, NULL};
        __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 292, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_copy, Py_False, __pyx_t_6, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 292, __pyx_L1_error)
        __pyx_t_5 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_to_numpy, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 292, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      if (unlikely((PyObject_SetItem(__pyx_v_rows, __pyx_v_name, __pyx_t_5) < 0))) __PYX_ERR(0, 292, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "nvtx/_lib/writer.pyx":285
 *     if pd is not None and isinstance(data, pd.DataFrame):
 *         rows = np.empty(len(data), dtype=dt)
 *         for name in dt.names:             # <<<<<<<<<<<<<<
 *             try:
 *                 column = data[name]
*/
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "nvtx/_lib/writer.pyx":293
 *                 )
 *             rows[name] = column.to_numpy(copy=False)
 *         return rows             # <<<<<<<<<<<<<<
 * 
 *     return np.fromiter(data, dtype=dt)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_rows);
    __pyx_r = __pyx_v_rows;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":283
 *         return np.ascontiguousarray(data, dtype=dt)
 * 
 *     if pd is not None and isinstance(data, pd.DataFrame):             # <<<<<<<<<<<<<<
 *         rows = np.empty(len(data), dtype=dt)
 *         for name in dt.names:
*/
  }

  /* "nvtx/_lib/writer.pyx":295
 *         return rows
 * 
 *     return np.fromiter(data, dtype=dt)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_fromiter); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_9);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_5, __pyx_v_data};
    __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_dt, __pyx_t_6, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 295, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":276
 * 
 * 
 * def _batch_rows(data, dt):             # <<<<<<<<<<<<<<
 *     """Return batch data as a flat, contiguous array of ``dt`` rows."""
 *     if isinstance(data, np.ndarray):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("nvtx._lib.writer._batch_rows", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rows);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_column);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":305
 * 
 * 
 * def _validate_native_byte_order(dt, operation):             # <<<<<<<<<<<<<<
 *     subdtype = dt.subdtype
 *     if subdtype is not None:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5_validate_native_byte_order(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_4_validate_native_byte_order, "_validate_native_byte_order(dt, operation)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_5_validate_native_byte_order = {"_validate_native_byte_order", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_5_validate_native_byte_order, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_4_validate_native_byte_order};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5_validate_native_byte_order(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_dt = 0;
  PyObject *__pyx_v_operation = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_validate_native_byte_order (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_dt,&__pyx_mstate_global->__pyx_n_u_operation,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 305, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 305, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 305, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_validate_native_byte_order", 0) < (0)) __PYX_ERR(0, 305, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_validate_native_byte_order", 1, 2, 2, i); __PYX_ERR(0, 305, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 305, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 305, __pyx_L3_error)
    }
    __pyx_v_dt = values[0];
    __pyx_v_operation = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_validate_native_byte_order", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 305, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer._validate_native_byte_order", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_4_validate_native_byte_order(__pyx_self, __pyx_v_dt, __pyx_v_operation);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_4_validate_native_byte_order(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dt, PyObject *__pyx_v_operation) {
  PyObject *__pyx_v_subdtype = NULL;
  PyObject *__pyx_v_field_name = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12[3];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_validate_native_byte_order", 0);

  /* "nvtx/_lib/writer.pyx":306
 * 
 * def _validate_native_byte_order(dt, operation):
 *     subdtype = dt.subdtype             # <<<<<<<<<<<<<<
 *     if subdtype is not None:
 *         _validate_native_byte_order(subdtype[0], operation)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_subdtype = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":307
 * def _validate_native_byte_order(dt, operation):
 *     subdtype = dt.subdtype
 *     if subdtype is not None:             # <<<<<<<<<<<<<<
 *         _validate_native_byte_order(subdtype[0], operation)
 *         return
*/
  __pyx_t_2 = (__pyx_v_subdtype != Py_None);
  if (__pyx_t_2) {

    /* "nvtx/_lib/writer.pyx":308
 *     subdtype = dt.subdtype
 *     if subdtype is not None:
 *         _validate_native_byte_order(subdtype[0], operation)             # <<<<<<<<<<<<<<
 *         return
 *     if dt.fields is not None:
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_validate_native_byte_order); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_subdtype, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_5, __pyx_v_operation};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "nvtx/_lib/writer.pyx":309
 *     if subdtype is not None:
 *         _validate_native_byte_order(subdtype[0], operation)
 *         return             # <<<<<<<<<<<<<<
 *     if dt.fields is not None:
 *         for field_name in dt.names:
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":307
 * def _validate_native_byte_order(dt, operation):
 *     subdtype = dt.subdtype
 *     if subdtype is not None:             # <<<<<<<<<<<<<<
 *         _validate_native_byte_order(subdtype[0], operation)
 *         return
*/
  }

  /* "nvtx/_lib/writer.pyx":310
 *         _validate_native_byte_order(subdtype[0], operation)
 *         return
 *     if dt.fields is not None:             # <<<<<<<<<<<<<<
 *         for field_name in dt.names:
 *             _validate_native_byte_order(dt.fields[field_name][0], operation)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/writer.pyx":311
 *         return
 *     if dt.fields is not None:
 *         for field_name in dt.names:             # <<<<<<<<<<<<<<
 *             _validate_native_byte_order(dt.fields[field_name][0], operation)
 *         return
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_4 = __pyx_t_1; __Pyx_INCREF(__pyx_t_4);
      __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_7 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 311, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 311, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 311, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          __pyx_t_1 = __Pyx_PyList_GET_ITEM_REF(__pyx_t_4, __pyx_t_7, __Pyx_ReferenceSharing_OwnStrongReference);
          ++__pyx_t_7;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 311, __pyx_L1_error)
            #endif
            if (__pyx_t_7 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_7));
          #else
          __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_7);
          #endif
          ++__pyx_t_7;
        }
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
      } else {
        __pyx_t_1 = __pyx_t_8(__pyx_t_4);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 311, __pyx_L1_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_field_name, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "nvtx/_lib/writer.pyx":312
 *     if dt.fields is not None:
 *         for field_name in dt.names:
 *             _validate_native_byte_order(dt.fields[field_name][0], operation)             # <<<<<<<<<<<<<<
 *         return
 *     if not dt.isnative:
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_validate_native_byte_order); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 312, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 312, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_t_9, __pyx_v_field_name); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 312, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_GetItemInt(__pyx_t_10, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 312, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_t_9, __pyx_v_operation};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "nvtx/_lib/writer.pyx":311
 *         return
 *     if dt.fields is not None:
 *         for field_name in dt.names:             # <<<<<<<<<<<<<<
 *             _validate_native_byte_order(dt.fields[field_name][0], operation)
 *         return
*/
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/writer.pyx":313
 *         for field_name in dt.names:
 *             _validate_native_byte_order(dt.fields[field_name][0], operation)
 *         return             # <<<<<<<<<<<<<<
 *     if not dt.isnative:
 *         raise TypeError(
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":310
 *         _validate_native_byte_order(subdtype[0], operation)
 *         return
 *     if dt.fields is not None:             # <<<<<<<<<<<<<<
 *         for field_name in dt.names:
 *             _validate_native_byte_order(dt.fields[field_name][0], operation)
*/
  }

  /* "nvtx/_lib/writer.pyx":314
 *             _validate_native_byte_order(dt.fields[field_name][0], operation)
 *         return
 *     if not dt.isnative:             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             f"{operation} requires native byte order; got dtype {dt}"
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_isnative); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = (!__pyx_t_2);
  if (unlikely(__pyx_t_11)) {

    /* "nvtx/_lib/writer.pyx":315
 *         return
 *     if not dt.isnative:
 *         raise TypeError(             # <<<<<<<<<<<<<<
 *             f"{operation} requires native byte order; got dtype {dt}"
 *         )
*/
    __pyx_t_1 = NULL;

    /* "nvtx/_lib/writer.pyx":316
 *     if not dt.isnative:
 *         raise TypeError(
 *             f"{operation} requires native byte order; got dtype {dt}"             # <<<<<<<<<<<<<<
 *         )
 * 
*/
    __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_operation, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyObject_FormatSimple(__pyx_v_dt, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_12[0] = __pyx_t_3;
    __pyx_t_12[1] = __pyx_mstate_global->__pyx_kp_u_requires_native_byte_order_got;
    __pyx_t_12[2] = __pyx_t_9;
    __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_12, 3, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 39 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_5};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 315, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 315, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":314
 *             _validate_native_byte_order(dt.fields[field_name][0], operation)
 *         return
 *     if not dt.isnative:             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             f"{operation} requires native byte order; got dtype {dt}"
*/
  }

  /* "nvtx/_lib/writer.pyx":305
 * 
 * 
 * def _validate_native_byte_order(dt, operation):             # <<<<<<<<<<<<<<
 *     subdtype = dt.subdtype
 *     if subdtype is not None:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("nvtx._lib.writer._validate_native_byte_order", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_subdtype);
  __Pyx_XDECREF(__pyx_v_field_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":320
 * 
 * 
 * def _validate_event_dtype(kind, dt, operation):             # <<<<<<<<<<<<<<
 *     """Validate the static layout and event roles of an event dtype."""
 *     if dt.fields is None:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7_validate_event_dtype(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6_validate_event_dtype, "_validate_event_dtype(kind, dt, operation)\n\nValidate the static layout and event roles of an event dtype.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7_validate_event_dtype = {"_validate_event_dtype", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7_validate_event_dtype, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6_validate_event_dtype};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7_validate_event_dtype(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_kind = 0;
  PyObject *__pyx_v_dt = 0;
  PyObject *__pyx_v_operation = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_validate_event_dtype (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_kind,&__pyx_mstate_global->__pyx_n_u_dt,&__pyx_mstate_global->__pyx_n_u_operation,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 320, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 320, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 320, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 320, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_validate_event_dtype", 0) < (0)) __PYX_ERR(0, 320, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_validate_event_dtype", 1, 3, 3, i); __PYX_ERR(0, 320, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 320, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 320, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 320, __pyx_L3_error)
    }
    __pyx_v_kind = values[0];
    __pyx_v_dt = values[1];
    __pyx_v_operation = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_validate_event_dtype", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 320, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer._validate_event_dtype", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6_validate_event_dtype(__pyx_self, __pyx_v_kind, __pyx_v_dt, __pyx_v_operation);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6_validate_event_dtype(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_kind, PyObject *__pyx_v_dt, PyObject *__pyx_v_operation) {
  PyObject *__pyx_v_role_fields = NULL;
  PyObject *__pyx_v_range_id_fields = NULL;
  PyObject *__pyx_v_field_name = NULL;
  PyObject *__pyx_v_field_dtype = NULL;
  PyObject *__pyx_v_metadata = NULL;
  PyObject *__pyx_v_entry_kind = NULL;
  PyObject *__pyx_v_entry_type = NULL;
  PyObject *__pyx_v_required_roles = NULL;
  PyObject *__pyx_v_fields = NULL;
  long __pyx_v_expected_count;
  PyObject *__pyx_v_role = NULL;
  long __pyx_v_expected_range_ids;
  PyObject *__pyx_7genexpr__pyx_v_entry_kind = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11[4];
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15[7];
  int __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_t_18;
  int __pyx_t_19;
  long __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  PyObject *__pyx_t_22[6];
  PyObject *__pyx_t_23[3];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_validate_event_dtype", 0);

  /* "nvtx/_lib/writer.pyx":322
 * def _validate_event_dtype(kind, dt, operation):
 *     """Validate the static layout and event roles of an event dtype."""
 *     if dt.fields is None:             # <<<<<<<<<<<<<<
 *         raise TypeError(f"{operation} requires a structured dtype")
 *     _validate_native_byte_order(dt, operation)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/writer.pyx":323
 *     """Validate the static layout and event roles of an event dtype."""
 *     if dt.fields is None:
 *         raise TypeError(f"{operation} requires a structured dtype")             # <<<<<<<<<<<<<<
 *     _validate_native_byte_order(dt, operation)
 * 
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_v_operation, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 323, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_requires_a_structured_dtype); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 323, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 323, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 323, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":322
 * def _validate_event_dtype(kind, dt, operation):
 *     """Validate the static layout and event roles of an event dtype."""
 *     if dt.fields is None:             # <<<<<<<<<<<<<<
 *         raise TypeError(f"{operation} requires a structured dtype")
 *     _validate_native_byte_order(dt, operation)
*/
  }

  /* "nvtx/_lib/writer.pyx":324
 *     if dt.fields is None:
 *         raise TypeError(f"{operation} requires a structured dtype")
 *     _validate_native_byte_order(dt, operation)             # <<<<<<<<<<<<<<
 * 
 *     role_fields = {
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_validate_native_byte_order); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 324, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_v_dt, __pyx_v_operation};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":326
 *     _validate_native_byte_order(dt, operation)
 * 
 *     role_fields = {             # <<<<<<<<<<<<<<
 *         entry_kind: []
 *         for entry_kind in (
*/
  { /* enter inner scope */
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 326, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "nvtx/_lib/writer.pyx":329
 *         entry_kind: []
 *         for entry_kind in (
 *             EntryKind.RANGE_BEGIN,             # <<<<<<<<<<<<<<
 *             EntryKind.RANGE_END,
 *             EntryKind.MARK,
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 329, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_RANGE_BEGIN); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 329, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":330
 *         for entry_kind in (
 *             EntryKind.RANGE_BEGIN,
 *             EntryKind.RANGE_END,             # <<<<<<<<<<<<<<
 *             EntryKind.MARK,
 *             EntryKind.COUNTER_TIMESTAMP,
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 330, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_RANGE_END); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 330, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":331
 *             EntryKind.RANGE_BEGIN,
 *             EntryKind.RANGE_END,
 *             EntryKind.MARK,             # <<<<<<<<<<<<<<
 *             EntryKind.COUNTER_TIMESTAMP,
 *         )
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 331, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_MARK); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 331, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":332
 *             EntryKind.RANGE_END,
 *             EntryKind.MARK,
 *             EntryKind.COUNTER_TIMESTAMP,             # <<<<<<<<<<<<<<
 *         )
 *     }
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_COUNTER_TIMESTAMP); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 332, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":329
 *         entry_kind: []
 *         for entry_kind in (
 *             EntryKind.RANGE_BEGIN,             # <<<<<<<<<<<<<<
 *             EntryKind.RANGE_END,
 *             EntryKind.MARK,
*/
    __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 329, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5) != (0)) __PYX_ERR(0, 329, __pyx_L6_error);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4) != (0)) __PYX_ERR(0, 329, __pyx_L6_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_7) != (0)) __PYX_ERR(0, 329, __pyx_L6_error);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_t_8) != (0)) __PYX_ERR(0, 329, __pyx_L6_error);
    __pyx_t_5 = 0;
    __pyx_t_4 = 0;
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;

    /* "nvtx/_lib/writer.pyx":328
 *     role_fields = {
 *         entry_kind: []
 *         for entry_kind in (             # <<<<<<<<<<<<<<
 *             EntryKind.RANGE_BEGIN,
 *             EntryKind.RANGE_END,
*/
    __pyx_t_8 = __pyx_t_3; __Pyx_INCREF(__pyx_t_8);
    __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (__pyx_t_9 >= 4) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_9));
      #else
      __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_9);
      #endif
      ++__pyx_t_9;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 328, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_entry_kind, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "nvtx/_lib/writer.pyx":327
 * 
 *     role_fields = {
 *         entry_kind: []             # <<<<<<<<<<<<<<
 *         for entry_kind in (
 *             EntryKind.RANGE_BEGIN,
*/
      __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 327, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (unlikely(PyDict_SetItem(__pyx_t_1, (PyObject*)__pyx_7genexpr__pyx_v_entry_kind, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 327, __pyx_L6_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "nvtx/_lib/writer.pyx":328
 *     role_fields = {
 *         entry_kind: []
 *         for entry_kind in (             # <<<<<<<<<<<<<<
 *             EntryKind.RANGE_BEGIN,
 *             EntryKind.RANGE_END,
*/
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_entry_kind); __pyx_7genexpr__pyx_v_entry_kind = 0;
    goto __pyx_L10_exit_scope;
    __pyx_L6_error:;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_entry_kind); __pyx_7genexpr__pyx_v_entry_kind = 0;
    goto __pyx_L1_error;
    __pyx_L10_exit_scope:;
  } /* exit inner scope */
  __pyx_v_role_fields = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":335
 *         )
 *     }
 *     range_id_fields = []             # <<<<<<<<<<<<<<
 * 
 *     for field_name in dt.names:
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_range_id_fields = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":337
 *     range_id_fields = []
 * 
 *     for field_name in dt.names:             # <<<<<<<<<<<<<<
 *         field_dtype = dt.fields[field_name][0]
 *         if field_dtype.hasobject:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_8 = __pyx_t_1; __Pyx_INCREF(__pyx_t_8);
    __pyx_t_9 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_9 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 337, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 337, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_8))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 337, __pyx_L1_error)
          #endif
          if (__pyx_t_9 >= __pyx_temp) break;
        }
        __pyx_t_1 = __Pyx_PyList_GET_ITEM_REF(__pyx_t_8, __pyx_t_9, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_9;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_8);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 337, __pyx_L1_error)
          #endif
          if (__pyx_t_9 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_9));
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_9);
        #endif
        ++__pyx_t_9;
      }
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
    } else {
      __pyx_t_1 = __pyx_t_10(__pyx_t_8);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 337, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_field_name, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/writer.pyx":338
 * 
 *     for field_name in dt.names:
 *         field_dtype = dt.fields[field_name][0]             # <<<<<<<<<<<<<<
 *         if field_dtype.hasobject:
 *             raise TypeError(
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_field_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_field_dtype, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/writer.pyx":339
 *     for field_name in dt.names:
 *         field_dtype = dt.fields[field_name][0]
 *         if field_dtype.hasobject:             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 f"{operation} requires fields with self-contained, "
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_dtype, __pyx_mstate_global->__pyx_n_u_hasobject); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 339, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(__pyx_t_2)) {

      /* "nvtx/_lib/writer.pyx":340
 *         field_dtype = dt.fields[field_name][0]
 *         if field_dtype.hasobject:
 *             raise TypeError(             # <<<<<<<<<<<<<<
 *                 f"{operation} requires fields with self-contained, "
 *                 f"fixed-width storage; field {field_name!r} uses managed storage"
*/
      __pyx_t_3 = NULL;

      /* "nvtx/_lib/writer.pyx":341
 *         if field_dtype.hasobject:
 *             raise TypeError(
 *                 f"{operation} requires fields with self-contained, "             # <<<<<<<<<<<<<<
 *                 f"fixed-width storage; field {field_name!r} uses managed storage"
 *             )
*/
      __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_v_operation, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 341, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);

      /* "nvtx/_lib/writer.pyx":342
 *             raise TypeError(
 *                 f"{operation} requires fields with self-contained, "
 *                 f"fixed-width storage; field {field_name!r} uses managed storage"             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_4 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_field_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 342, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_11[0] = __pyx_t_7;
      __pyx_t_11[1] = __pyx_mstate_global->__pyx_kp_u_requires_fields_with_self_conta;
      __pyx_t_11[2] = __pyx_t_4;
      __pyx_t_11[3] = __pyx_mstate_global->__pyx_kp_u_uses_managed_storage;

      /* "nvtx/_lib/writer.pyx":341
 *         if field_dtype.hasobject:
 *             raise TypeError(
 *                 f"{operation} requires fields with self-contained, "             # <<<<<<<<<<<<<<
 *                 f"fixed-width storage; field {field_name!r} uses managed storage"
 *             )
*/
      __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_11, 4, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 65 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4) + 21, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4));
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 341, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 340, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":339
 *     for field_name in dt.names:
 *         field_dtype = dt.fields[field_name][0]
 *         if field_dtype.hasobject:             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 f"{operation} requires fields with self-contained, "
*/
    }

    /* "nvtx/_lib/writer.pyx":345
 *             )
 * 
 *         metadata = _nvtx_metadata_from_dtype(field_dtype)             # <<<<<<<<<<<<<<
 *         entry_kind = getattr(metadata, "entry_kind", None)
 *         entry_type = getattr(metadata, "entry_type", None)
*/
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_nvtx_metadata_from_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_field_dtype};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_metadata, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/writer.pyx":346
 * 
 *         metadata = _nvtx_metadata_from_dtype(field_dtype)
 *         entry_kind = getattr(metadata, "entry_kind", None)             # <<<<<<<<<<<<<<
 *         entry_type = getattr(metadata, "entry_type", None)
 *         if entry_type is PayloadEntryType.RANGE_ID:
*/
    __pyx_t_1 = __Pyx_GetAttr3(__pyx_v_metadata, __pyx_mstate_global->__pyx_n_u_entry_kind, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 346, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_entry_kind, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/writer.pyx":347
 *         metadata = _nvtx_metadata_from_dtype(field_dtype)
 *         entry_kind = getattr(metadata, "entry_kind", None)
 *         entry_type = getattr(metadata, "entry_type", None)             # <<<<<<<<<<<<<<
 *         if entry_type is PayloadEntryType.RANGE_ID:
 *             range_id_fields.append(field_name)
*/
    __pyx_t_1 = __Pyx_GetAttr3(__pyx_v_metadata, __pyx_mstate_global->__pyx_n_u_entry_type, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 347, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_entry_type, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/writer.pyx":348
 *         entry_kind = getattr(metadata, "entry_kind", None)
 *         entry_type = getattr(metadata, "entry_type", None)
 *         if entry_type is PayloadEntryType.RANGE_ID:             # <<<<<<<<<<<<<<
 *             range_id_fields.append(field_name)
 *         if entry_kind in role_fields:
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_PayloadEntryType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_RANGE_ID); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = (__pyx_v_entry_type == __pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_2) {

      /* "nvtx/_lib/writer.pyx":349
 *         entry_type = getattr(metadata, "entry_type", None)
 *         if entry_type is PayloadEntryType.RANGE_ID:
 *             range_id_fields.append(field_name)             # <<<<<<<<<<<<<<
 *         if entry_kind in role_fields:
 *             if entry_type is not None:
*/
      __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_range_id_fields, __pyx_v_field_name); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 349, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":348
 *         entry_kind = getattr(metadata, "entry_kind", None)
 *         entry_type = getattr(metadata, "entry_type", None)
 *         if entry_type is PayloadEntryType.RANGE_ID:             # <<<<<<<<<<<<<<
 *             range_id_fields.append(field_name)
 *         if entry_kind in role_fields:
*/
    }

    /* "nvtx/_lib/writer.pyx":350
 *         if entry_type is PayloadEntryType.RANGE_ID:
 *             range_id_fields.append(field_name)
 *         if entry_kind in role_fields:             # <<<<<<<<<<<<<<
 *             if entry_type is not None:
 *                 raise ValueError(
*/
    __pyx_t_2 = (__Pyx_PyDict_ContainsTF(__pyx_v_entry_kind, __pyx_v_role_fields, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 350, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "nvtx/_lib/writer.pyx":351
 *             range_id_fields.append(field_name)
 *         if entry_kind in role_fields:
 *             if entry_type is not None:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     f"{operation} field {field_name!r} cannot combine the "
*/
      __pyx_t_2 = (__pyx_v_entry_type != Py_None);
      if (unlikely(__pyx_t_2)) {

        /* "nvtx/_lib/writer.pyx":352
 *         if entry_kind in role_fields:
 *             if entry_type is not None:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     f"{operation} field {field_name!r} cannot combine the "
 *                     f"EntryKind.{entry_kind.name} timestamp role with "
*/
        __pyx_t_1 = NULL;

        /* "nvtx/_lib/writer.pyx":353
 *             if entry_type is not None:
 *                 raise ValueError(
 *                     f"{operation} field {field_name!r} cannot combine the "             # <<<<<<<<<<<<<<
 *                     f"EntryKind.{entry_kind.name} timestamp role with "
 *                     f"PayloadEntryType.{entry_type.name}"
*/
        __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_v_operation, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 353, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_field_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 353, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);

        /* "nvtx/_lib/writer.pyx":354
 *                 raise ValueError(
 *                     f"{operation} field {field_name!r} cannot combine the "
 *                     f"EntryKind.{entry_kind.name} timestamp role with "             # <<<<<<<<<<<<<<
 *                     f"PayloadEntryType.{entry_type.name}"
 *                 )
*/
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_entry_kind, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 354, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_13 = __Pyx_PyObject_FormatSimple(__pyx_t_7, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 354, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "nvtx/_lib/writer.pyx":355
 *                     f"{operation} field {field_name!r} cannot combine the "
 *                     f"EntryKind.{entry_kind.name} timestamp role with "
 *                     f"PayloadEntryType.{entry_type.name}"             # <<<<<<<<<<<<<<
 *                 )
 *             role_fields[entry_kind].append(field_name)
*/
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_entry_type, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 355, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_14 = __Pyx_PyObject_FormatSimple(__pyx_t_7, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 355, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_15[0] = __pyx_t_5;
        __pyx_t_15[1] = __pyx_mstate_global->__pyx_kp_u_field;
        __pyx_t_15[2] = __pyx_t_4;
        __pyx_t_15[3] = __pyx_mstate_global->__pyx_kp_u_cannot_combine_the_EntryKind;
        __pyx_t_15[4] = __pyx_t_13;
        __pyx_t_15[5] = __pyx_mstate_global->__pyx_kp_u_timestamp_role_with_PayloadEntr;
        __pyx_t_15[6] = __pyx_t_14;

        /* "nvtx/_lib/writer.pyx":353
 *             if entry_type is not None:
 *                 raise ValueError(
 *                     f"{operation} field {field_name!r} cannot combine the "             # <<<<<<<<<<<<<<
 *                     f"EntryKind.{entry_kind.name} timestamp role with "
 *                     f"PayloadEntryType.{entry_type.name}"
*/
        __pyx_t_7 = __Pyx_PyUnicode_Join(__pyx_t_15, 7, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5) + 7 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4) + 30 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13) + 38 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14));
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 353, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_6 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 352, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
        }
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 352, __pyx_L1_error)

        /* "nvtx/_lib/writer.pyx":351
 *             range_id_fields.append(field_name)
 *         if entry_kind in role_fields:
 *             if entry_type is not None:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     f"{operation} field {field_name!r} cannot combine the "
*/
      }

      /* "nvtx/_lib/writer.pyx":357
 *                     f"PayloadEntryType.{entry_type.name}"
 *                 )
 *             role_fields[entry_kind].append(field_name)             # <<<<<<<<<<<<<<
 * 
 *     if kind is EventKind.MARK:
*/
      __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_role_fields, __pyx_v_entry_kind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_12 = __Pyx_PyObject_Append(__pyx_t_3, __pyx_v_field_name); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 357, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "nvtx/_lib/writer.pyx":350
 *         if entry_type is PayloadEntryType.RANGE_ID:
 *             range_id_fields.append(field_name)
 *         if entry_kind in role_fields:             # <<<<<<<<<<<<<<
 *             if entry_type is not None:
 *                 raise ValueError(
*/
    }

    /* "nvtx/_lib/writer.pyx":337
 *     range_id_fields = []
 * 
 *     for field_name in dt.names:             # <<<<<<<<<<<<<<
 *         field_dtype = dt.fields[field_name][0]
 *         if field_dtype.hasobject:
*/
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/writer.pyx":359
 *             role_fields[entry_kind].append(field_name)
 * 
 *     if kind is EventKind.MARK:             # <<<<<<<<<<<<<<
 *         required_roles = (EntryKind.MARK,)
 *     elif kind in (EventKind.RANGE_PUSH, EventKind.RANGE_START):
*/
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 359, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_MARK); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 359, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_2 = (__pyx_v_kind == __pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/writer.pyx":360
 * 
 *     if kind is EventKind.MARK:
 *         required_roles = (EntryKind.MARK,)             # <<<<<<<<<<<<<<
 *     elif kind in (EventKind.RANGE_PUSH, EventKind.RANGE_START):
 *         required_roles = (EntryKind.RANGE_BEGIN,)
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_MARK); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_8) != (0)) __PYX_ERR(0, 360, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_v_required_roles = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":359
 *             role_fields[entry_kind].append(field_name)
 * 
 *     if kind is EventKind.MARK:             # <<<<<<<<<<<<<<
 *         required_roles = (EntryKind.MARK,)
 *     elif kind in (EventKind.RANGE_PUSH, EventKind.RANGE_START):
*/
    goto __pyx_L18;
  }

  /* "nvtx/_lib/writer.pyx":361
 *     if kind is EventKind.MARK:
 *         required_roles = (EntryKind.MARK,)
 *     elif kind in (EventKind.RANGE_PUSH, EventKind.RANGE_START):             # <<<<<<<<<<<<<<
 *         required_roles = (EntryKind.RANGE_BEGIN,)
 *     elif kind in (EventKind.RANGE_POP, EventKind.RANGE_END):
*/
  __Pyx_INCREF(__pyx_v_kind);
  __pyx_t_3 = __pyx_v_kind;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_RANGE_PUSH); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyObject_RichCompare(__pyx_t_3, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (!__pyx_t_16) {
  } else {
    __pyx_t_2 = __pyx_t_16;
    goto __pyx_L19_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_RANGE_START); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyObject_RichCompare(__pyx_t_3, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_2 = __pyx_t_16;
  __pyx_L19_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_16 = __pyx_t_2;
  if (__pyx_t_16) {

    /* "nvtx/_lib/writer.pyx":362
 *         required_roles = (EntryKind.MARK,)
 *     elif kind in (EventKind.RANGE_PUSH, EventKind.RANGE_START):
 *         required_roles = (EntryKind.RANGE_BEGIN,)             # <<<<<<<<<<<<<<
 *     elif kind in (EventKind.RANGE_POP, EventKind.RANGE_END):
 *         required_roles = (EntryKind.RANGE_END,)
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_RANGE_BEGIN); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_8) != (0)) __PYX_ERR(0, 362, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_v_required_roles = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":361
 *     if kind is EventKind.MARK:
 *         required_roles = (EntryKind.MARK,)
 *     elif kind in (EventKind.RANGE_PUSH, EventKind.RANGE_START):             # <<<<<<<<<<<<<<
 *         required_roles = (EntryKind.RANGE_BEGIN,)
 *     elif kind in (EventKind.RANGE_POP, EventKind.RANGE_END):
*/
    goto __pyx_L18;
  }

  /* "nvtx/_lib/writer.pyx":363
 *     elif kind in (EventKind.RANGE_PUSH, EventKind.RANGE_START):
 *         required_roles = (EntryKind.RANGE_BEGIN,)
 *     elif kind in (EventKind.RANGE_POP, EventKind.RANGE_END):             # <<<<<<<<<<<<<<
 *         required_roles = (EntryKind.RANGE_END,)
 *     else:
*/
  __Pyx_INCREF(__pyx_v_kind);
  __pyx_t_3 = __pyx_v_kind;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_RANGE_POP); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyObject_RichCompare(__pyx_t_3, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (!__pyx_t_2) {
  } else {
    __pyx_t_16 = __pyx_t_2;
    goto __pyx_L21_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_RANGE_END); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyObject_RichCompare(__pyx_t_3, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 363, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_16 = __pyx_t_2;
  __pyx_L21_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_2 = __pyx_t_16;
  if (__pyx_t_2) {

    /* "nvtx/_lib/writer.pyx":364
 *         required_roles = (EntryKind.RANGE_BEGIN,)
 *     elif kind in (EventKind.RANGE_POP, EventKind.RANGE_END):
 *         required_roles = (EntryKind.RANGE_END,)             # <<<<<<<<<<<<<<
 *     else:
 *         required_roles = (EntryKind.RANGE_BEGIN, EntryKind.RANGE_END)
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_RANGE_END); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_8) != (0)) __PYX_ERR(0, 364, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_v_required_roles = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":363
 *     elif kind in (EventKind.RANGE_PUSH, EventKind.RANGE_START):
 *         required_roles = (EntryKind.RANGE_BEGIN,)
 *     elif kind in (EventKind.RANGE_POP, EventKind.RANGE_END):             # <<<<<<<<<<<<<<
 *         required_roles = (EntryKind.RANGE_END,)
 *     else:
*/
    goto __pyx_L18;
  }

  /* "nvtx/_lib/writer.pyx":366
 *         required_roles = (EntryKind.RANGE_END,)
 *     else:
 *         required_roles = (EntryKind.RANGE_BEGIN, EntryKind.RANGE_END)             # <<<<<<<<<<<<<<
 * 
 *     for entry_kind, fields in role_fields.items():
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 366, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_RANGE_BEGIN); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 366, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EntryKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 366, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_RANGE_END); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 366, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 366, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_8) != (0)) __PYX_ERR(0, 366, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_7) != (0)) __PYX_ERR(0, 366, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_t_7 = 0;
    __pyx_v_required_roles = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L18:;

  /* "nvtx/_lib/writer.pyx":368
 *         required_roles = (EntryKind.RANGE_BEGIN, EntryKind.RANGE_END)
 * 
 *     for entry_kind, fields in role_fields.items():             # <<<<<<<<<<<<<<
 *         expected_count = 1 if entry_kind in required_roles else 0
 *         if len(fields) != expected_count:
*/
  __pyx_t_9 = 0;
  __pyx_t_7 = __Pyx_dict_iterator(__pyx_v_role_fields, 1, __pyx_mstate_global->__pyx_n_u_items, (&__pyx_t_17), (&__pyx_t_18)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_3);
  __pyx_t_3 = __pyx_t_7;
  __pyx_t_7 = 0;
  while (1) {
    __pyx_t_19 = __Pyx_dict_iter_next(__pyx_t_3, __pyx_t_17, &__pyx_t_9, &__pyx_t_7, &__pyx_t_8, NULL, __pyx_t_18);
    if (unlikely(__pyx_t_19 == 0)) break;
    if (unlikely(__pyx_t_19 == -1)) __PYX_ERR(0, 368, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_XDECREF_SET(__pyx_v_entry_kind, __pyx_t_7);
    __pyx_t_7 = 0;
    __Pyx_XDECREF_SET(__pyx_v_fields, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "nvtx/_lib/writer.pyx":369
 * 
 *     for entry_kind, fields in role_fields.items():
 *         expected_count = 1 if entry_kind in required_roles else 0             # <<<<<<<<<<<<<<
 *         if len(fields) != expected_count:
 *             role = f"EntryKind.{entry_kind.name}"
*/
    __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_v_entry_kind, __pyx_v_required_roles, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 369, __pyx_L1_error)
    if (__pyx_t_2) {
      __pyx_t_20 = 1;
    } else {
      __pyx_t_20 = 0;
    }
    __pyx_v_expected_count = __pyx_t_20;

    /* "nvtx/_lib/writer.pyx":370
 *     for entry_kind, fields in role_fields.items():
 *         expected_count = 1 if entry_kind in required_roles else 0
 *         if len(fields) != expected_count:             # <<<<<<<<<<<<<<
 *             role = f"EntryKind.{entry_kind.name}"
 *             if expected_count:
*/
    __pyx_t_21 = PyObject_Length(__pyx_v_fields); if (unlikely(__pyx_t_21 == ((Py_ssize_t)-1))) __PYX_ERR(0, 370, __pyx_L1_error)
    __pyx_t_2 = (__pyx_t_21 != __pyx_v_expected_count);
    if (__pyx_t_2) {

      /* "nvtx/_lib/writer.pyx":371
 *         expected_count = 1 if entry_kind in required_roles else 0
 *         if len(fields) != expected_count:
 *             role = f"EntryKind.{entry_kind.name}"             # <<<<<<<<<<<<<<
 *             if expected_count:
 *                 raise ValueError(
*/
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_entry_kind, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_t_8, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_EntryKind_2, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_v_role = ((PyObject*)__pyx_t_8);
      __pyx_t_8 = 0;

      /* "nvtx/_lib/writer.pyx":372
 *         if len(fields) != expected_count:
 *             role = f"EntryKind.{entry_kind.name}"
 *             if expected_count:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     f"{operation} dtype for {kind!s} must contain exactly one "
*/
      __pyx_t_2 = (__pyx_v_expected_count != 0);
      if (unlikely(__pyx_t_2)) {

        /* "nvtx/_lib/writer.pyx":373
 *             role = f"EntryKind.{entry_kind.name}"
 *             if expected_count:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     f"{operation} dtype for {kind!s} must contain exactly one "
 *                     f"{role} field"
*/
        __pyx_t_7 = NULL;

        /* "nvtx/_lib/writer.pyx":374
 *             if expected_count:
 *                 raise ValueError(
 *                     f"{operation} dtype for {kind!s} must contain exactly one "             # <<<<<<<<<<<<<<
 *                     f"{role} field"
 *                 )
*/
        __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_operation, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 374, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_14 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_kind), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 374, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);

        /* "nvtx/_lib/writer.pyx":375
 *                 raise ValueError(
 *                     f"{operation} dtype for {kind!s} must contain exactly one "
 *                     f"{role} field"             # <<<<<<<<<<<<<<
 *                 )
 *             raise ValueError(f"{role} is not valid for {kind!s}")
*/
        __pyx_t_22[0] = __pyx_t_1;
        __pyx_t_22[1] = __pyx_mstate_global->__pyx_kp_u_dtype_for;
        __pyx_t_22[2] = __pyx_t_14;
        __pyx_t_22[3] = __pyx_mstate_global->__pyx_kp_u_must_contain_exactly_one;
        __pyx_t_22[4] = __pyx_v_role;
        __pyx_t_22[5] = __pyx_mstate_global->__pyx_kp_u_field_2;

        /* "nvtx/_lib/writer.pyx":374
 *             if expected_count:
 *                 raise ValueError(
 *                     f"{operation} dtype for {kind!s} must contain exactly one "             # <<<<<<<<<<<<<<
 *                     f"{role} field"
 *                 )
*/
        __pyx_t_13 = __Pyx_PyUnicode_Join(__pyx_t_22, 6, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14) + 26 + __Pyx_PyUnicode_GET_LENGTH(__pyx_v_role) + 6, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_role));
        if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 374, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_6 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_13};
          __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 373, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
        }
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 373, __pyx_L1_error)

        /* "nvtx/_lib/writer.pyx":372
 *         if len(fields) != expected_count:
 *             role = f"EntryKind.{entry_kind.name}"
 *             if expected_count:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     f"{operation} dtype for {kind!s} must contain exactly one "
*/
      }

      /* "nvtx/_lib/writer.pyx":377
 *                     f"{role} field"
 *                 )
 *             raise ValueError(f"{role} is not valid for {kind!s}")             # <<<<<<<<<<<<<<
 * 
 *     expected_range_ids = (
*/
      __pyx_t_13 = NULL;
      __pyx_t_7 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_kind), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 377, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_23[0] = __pyx_v_role;
      __pyx_t_23[1] = __pyx_mstate_global->__pyx_kp_u_is_not_valid_for;
      __pyx_t_23[2] = __pyx_t_7;
      __pyx_t_14 = __Pyx_PyUnicode_Join(__pyx_t_23, 3, __Pyx_PyUnicode_GET_LENGTH(__pyx_v_role) + 18 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_role) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7));
      if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 377, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_14};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 377, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }
      __Pyx_Raise(__pyx_t_8, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __PYX_ERR(0, 377, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":370
 *     for entry_kind, fields in role_fields.items():
 *         expected_count = 1 if entry_kind in required_roles else 0
 *         if len(fields) != expected_count:             # <<<<<<<<<<<<<<
 *             role = f"EntryKind.{entry_kind.name}"
 *             if expected_count:
*/
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":380
 * 
 *     expected_range_ids = (
 *         1 if kind in (EventKind.RANGE_START, EventKind.RANGE_END) else 0             # <<<<<<<<<<<<<<
 *     )
 *     if len(range_id_fields) != expected_range_ids:
*/
  __Pyx_INCREF(__pyx_v_kind);
  __pyx_t_3 = __pyx_v_kind;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_RANGE_START); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyObject_RichCompare(__pyx_t_3, __pyx_t_14, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (!__pyx_t_16) {
  } else {
    __pyx_t_2 = __pyx_t_16;
    goto __pyx_L27_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_RANGE_END); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyObject_RichCompare(__pyx_t_3, __pyx_t_14, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_16 < 0))) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_2 = __pyx_t_16;
  __pyx_L27_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_16 = __pyx_t_2;
  if (__pyx_t_16) {
    __pyx_t_20 = 1;
  } else {
    __pyx_t_20 = 0;
  }
  __pyx_v_expected_range_ids = __pyx_t_20;

  /* "nvtx/_lib/writer.pyx":382
 *         1 if kind in (EventKind.RANGE_START, EventKind.RANGE_END) else 0
 *     )
 *     if len(range_id_fields) != expected_range_ids:             # <<<<<<<<<<<<<<
 *         role = "PayloadEntryType.RANGE_ID"
 *         if expected_range_ids:
*/
  __pyx_t_17 = __Pyx_PyList_GET_SIZE(__pyx_v_range_id_fields); if (unlikely(__pyx_t_17 == ((Py_ssize_t)-1))) __PYX_ERR(0, 382, __pyx_L1_error)
  __pyx_t_16 = (__pyx_t_17 != __pyx_v_expected_range_ids);
  if (__pyx_t_16) {

    /* "nvtx/_lib/writer.pyx":383
 *     )
 *     if len(range_id_fields) != expected_range_ids:
 *         role = "PayloadEntryType.RANGE_ID"             # <<<<<<<<<<<<<<
 *         if expected_range_ids:
 *             raise ValueError(
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_PayloadEntryType_RANGE_ID);
    __pyx_v_role = __pyx_mstate_global->__pyx_kp_u_PayloadEntryType_RANGE_ID;

    /* "nvtx/_lib/writer.pyx":384
 *     if len(range_id_fields) != expected_range_ids:
 *         role = "PayloadEntryType.RANGE_ID"
 *         if expected_range_ids:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 f"{operation} dtype for {kind!s} must contain exactly one "
*/
    __pyx_t_16 = (__pyx_v_expected_range_ids != 0);
    if (unlikely(__pyx_t_16)) {

      /* "nvtx/_lib/writer.pyx":385
 *         role = "PayloadEntryType.RANGE_ID"
 *         if expected_range_ids:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 f"{operation} dtype for {kind!s} must contain exactly one "
 *                 f"{role} field"
*/
      __pyx_t_8 = NULL;

      /* "nvtx/_lib/writer.pyx":386
 *         if expected_range_ids:
 *             raise ValueError(
 *                 f"{operation} dtype for {kind!s} must contain exactly one "             # <<<<<<<<<<<<<<
 *                 f"{role} field"
 *             )
*/
      __pyx_t_14 = __Pyx_PyObject_FormatSimple(__pyx_v_operation, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_13 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_kind), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);

      /* "nvtx/_lib/writer.pyx":387
 *             raise ValueError(
 *                 f"{operation} dtype for {kind!s} must contain exactly one "
 *                 f"{role} field"             # <<<<<<<<<<<<<<
 *             )
 *         raise ValueError(f"{role} is not valid for {kind!s}")
*/
      __pyx_t_22[0] = __pyx_t_14;
      __pyx_t_22[1] = __pyx_mstate_global->__pyx_kp_u_dtype_for;
      __pyx_t_22[2] = __pyx_t_13;
      __pyx_t_22[3] = __pyx_mstate_global->__pyx_kp_u_must_contain_exactly_one;
      __pyx_t_22[4] = __pyx_v_role;
      __pyx_t_22[5] = __pyx_mstate_global->__pyx_kp_u_field_2;

      /* "nvtx/_lib/writer.pyx":386
 *         if expected_range_ids:
 *             raise ValueError(
 *                 f"{operation} dtype for {kind!s} must contain exactly one "             # <<<<<<<<<<<<<<
 *                 f"{role} field"
 *             )
*/
      __pyx_t_7 = __Pyx_PyUnicode_Join(__pyx_t_22, 6, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14) + 11 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13) + 26 + __Pyx_PyUnicode_GET_LENGTH(__pyx_v_role) + 6, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_role));
      if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_7};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 385, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 385, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":384
 *     if len(range_id_fields) != expected_range_ids:
 *         role = "PayloadEntryType.RANGE_ID"
 *         if expected_range_ids:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 f"{operation} dtype for {kind!s} must contain exactly one "
*/
    }

    /* "nvtx/_lib/writer.pyx":389
 *                 f"{role} field"
 *             )
 *         raise ValueError(f"{role} is not valid for {kind!s}")             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_7 = NULL;
    __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Str(__pyx_v_kind), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_23[0] = __pyx_v_role;
    __pyx_t_23[1] = __pyx_mstate_global->__pyx_kp_u_is_not_valid_for;
    __pyx_t_23[2] = __pyx_t_8;
    __pyx_t_13 = __Pyx_PyUnicode_Join(__pyx_t_23, 3, __Pyx_PyUnicode_GET_LENGTH(__pyx_v_role) + 18 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_role) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8));
    if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_13};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 389, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":382
 *         1 if kind in (EventKind.RANGE_START, EventKind.RANGE_END) else 0
 *     )
 *     if len(range_id_fields) != expected_range_ids:             # <<<<<<<<<<<<<<
 *         role = "PayloadEntryType.RANGE_ID"
 *         if expected_range_ids:
*/
  }

  /* "nvtx/_lib/writer.pyx":320
 * 
 * 
 * def _validate_event_dtype(kind, dt, operation):             # <<<<<<<<<<<<<<
 *     """Validate the static layout and event roles of an event dtype."""
 *     if dt.fields is None:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("nvtx._lib.writer._validate_event_dtype", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_role_fields);
  __Pyx_XDECREF(__pyx_v_range_id_fields);
  __Pyx_XDECREF(__pyx_v_field_name);
  __Pyx_XDECREF(__pyx_v_field_dtype);
  __Pyx_XDECREF(__pyx_v_metadata);
  __Pyx_XDECREF(__pyx_v_entry_kind);
  __Pyx_XDECREF(__pyx_v_entry_type);
  __Pyx_XDECREF(__pyx_v_required_roles);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_role);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_entry_kind);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":392
 * 
 * 
 * def _canonical_backend_path(path):             # <<<<<<<<<<<<<<
 *     """Return the canonical registry key and filename for a backend path."""
 *     path = Path(os.fsdecode(path)).expanduser().resolve()
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_9_canonical_backend_path(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_8_canonical_backend_path, "_canonical_backend_path(path)\n\nReturn the canonical registry key and filename for a backend path.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_9_canonical_backend_path = {"_canonical_backend_path", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_9_canonical_backend_path, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_8_canonical_backend_path};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_9_canonical_backend_path(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_canonical_backend_path (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 392, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 392, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_canonical_backend_path", 0) < (0)) __PYX_ERR(0, 392, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_canonical_backend_path", 1, 1, 1, i); __PYX_ERR(0, 392, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 392, __pyx_L3_error)
    }
    __pyx_v_path = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_canonical_backend_path", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 392, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer._canonical_backend_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_8_canonical_backend_path(__pyx_self, __pyx_v_path);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_8_canonical_backend_path(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_canonical_backend_path", 0);
  __Pyx_INCREF(__pyx_v_path);

  /* "nvtx/_lib/writer.pyx":394
 * def _canonical_backend_path(path):
 *     """Return the canonical registry key and filename for a backend path."""
 *     path = Path(os.fsdecode(path)).expanduser().resolve()             # <<<<<<<<<<<<<<
 *     return os.path.normcase(os.fspath(path))
 * 
*/
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_Path); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_fsdecode); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_12 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_11);
    assert(__pyx_t_9);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
    __Pyx_INCREF(__pyx_t_9);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
    __pyx_t_12 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_v_path};
    __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_t_12 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_6);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_12 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_8};
    __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
  }
  __pyx_t_4 = __pyx_t_5;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_12 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_expanduser, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_12 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_resolve, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF_SET(__pyx_v_path, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":395
 *     """Return the canonical registry key and filename for a backend path."""
 *     path = Path(os.fsdecode(path)).expanduser().resolve()
 *     return os.path.normcase(os.fspath(path))             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_path); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_t_5;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_os); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fspath); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_12 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_12 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_path};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_t_12 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_2};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_normcase, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":392
 * 
 * 
 * def _canonical_backend_path(path):             # <<<<<<<<<<<<<<
 *     """Return the canonical registry key and filename for a backend path."""
 *     path = Path(os.fsdecode(path)).expanduser().resolve()
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("nvtx._lib.writer._canonical_backend_path", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":401
 *     """One canonical backend module shared by all public wrappers."""
 * 
 *     def __cinit__(self, lib):             # <<<<<<<<<<<<<<
 *         self._iface = NULL
 *         self._lib = lib
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_lib = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_lib,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 401, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 401, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(0, 401, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, i); __PYX_ERR(0, 401, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 401, __pyx_L3_error)
    }
    __pyx_v_lib = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 401, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer._BackendModule.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_14_BackendModule___cinit__(((struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)__pyx_v_self), __pyx_v_lib);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_6writer_14_BackendModule___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self, PyObject *__pyx_v_lib) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "nvtx/_lib/writer.pyx":402
 * 
 *     def __cinit__(self, lib):
 *         self._iface = NULL             # <<<<<<<<<<<<<<
 *         self._lib = lib
 *         self._refcount = 0
*/
  __pyx_v_self->_iface = NULL;

  /* "nvtx/_lib/writer.pyx":403
 *     def __cinit__(self, lib):
 *         self._iface = NULL
 *         self._lib = lib             # <<<<<<<<<<<<<<
 *         self._refcount = 0
 * 
*/
  __Pyx_INCREF(__pyx_v_lib);
  __Pyx_GIVEREF(__pyx_v_lib);
  __Pyx_GOTREF(__pyx_v_self->_lib);
  __Pyx_DECREF(__pyx_v_self->_lib);
  __pyx_v_self->_lib = __pyx_v_lib;

  /* "nvtx/_lib/writer.pyx":404
 *         self._iface = NULL
 *         self._lib = lib
 *         self._refcount = 0             # <<<<<<<<<<<<<<
 * 
 *     cdef void _initialize(self) except *:
*/
  __pyx_v_self->_refcount = 0;

  /* "nvtx/_lib/writer.pyx":401
 *     """One canonical backend module shared by all public wrappers."""
 * 
 *     def __cinit__(self, lib):             # <<<<<<<<<<<<<<
 *         self._iface = NULL
 *         self._lib = lib
*/

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":406
 *         self._refcount = 0
 * 
 *     cdef void _initialize(self) except *:             # <<<<<<<<<<<<<<
 *         cdef nvtxwGetInterface_t get_interface
 *         cdef const nvtxwInterface_v2_t* iface = NULL
*/

static void __pyx_f_4nvtx_4_lib_6writer_14_BackendModule__initialize(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self) {
  nvtxwGetInterface_t __pyx_v_get_interface;
  nvtxwInterface_v2_t const *__pyx_v_iface;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_v_get_interface_obj = NULL;
  PyObject *__pyx_v_get_interface_addr = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  uintptr_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_initialize", 0);

  /* "nvtx/_lib/writer.pyx":408
 *     cdef void _initialize(self) except *:
 *         cdef nvtxwGetInterface_t get_interface
 *         cdef const nvtxwInterface_v2_t* iface = NULL             # <<<<<<<<<<<<<<
 *         cdef nvtxwResultCode_t rc
 *         get_interface_obj = getattr(
*/
  __pyx_v_iface = NULL;

  /* "nvtx/_lib/writer.pyx":411
 *         cdef nvtxwResultCode_t rc
 *         get_interface_obj = getattr(
 *             self._lib, _GET_INTERFACE_SYMBOL_NAME, None)             # <<<<<<<<<<<<<<
 *         if get_interface_obj is None:
 *             raise WriterError(
*/
  __pyx_t_1 = __pyx_v_self->_lib;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_GET_INTERFACE_SYMBOL_NAME); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 411, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "nvtx/_lib/writer.pyx":410
 *         cdef const nvtxwInterface_v2_t* iface = NULL
 *         cdef nvtxwResultCode_t rc
 *         get_interface_obj = getattr(             # <<<<<<<<<<<<<<
 *             self._lib, _GET_INTERFACE_SYMBOL_NAME, None)
 *         if get_interface_obj is None:
*/
  __pyx_t_3 = __Pyx_GetAttr3(__pyx_t_1, __pyx_t_2, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_get_interface_obj = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":412
 *         get_interface_obj = getattr(
 *             self._lib, _GET_INTERFACE_SYMBOL_NAME, None)
 *         if get_interface_obj is None:             # <<<<<<<<<<<<<<
 *             raise WriterError(
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")
*/
  __pyx_t_4 = (__pyx_v_get_interface_obj == Py_None);
  if (unlikely(__pyx_t_4)) {

    /* "nvtx/_lib/writer.pyx":413
 *             self._lib, _GET_INTERFACE_SYMBOL_NAME, None)
 *         if get_interface_obj is None:
 *             raise WriterError(             # <<<<<<<<<<<<<<
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")
 *         get_interface_addr = ctypes.cast(
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_WriterError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 413, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "nvtx/_lib/writer.pyx":414
 *         if get_interface_obj is None:
 *             raise WriterError(
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")             # <<<<<<<<<<<<<<
 *         get_interface_addr = ctypes.cast(
 *             get_interface_obj, ctypes.c_void_p).value
*/
    __pyx_t_5 = __Pyx_PyLong_From_int(NVTXW_RESULT_LIBRARY_SYMBOL_MISSING); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 414, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_5, __pyx_mstate_global->__pyx_n_u_load_backend};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 413, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 413, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":412
 *         get_interface_obj = getattr(
 *             self._lib, _GET_INTERFACE_SYMBOL_NAME, None)
 *         if get_interface_obj is None:             # <<<<<<<<<<<<<<
 *             raise WriterError(
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")
*/
  }

  /* "nvtx/_lib/writer.pyx":415
 *             raise WriterError(
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")
 *         get_interface_addr = ctypes.cast(             # <<<<<<<<<<<<<<
 *             get_interface_obj, ctypes.c_void_p).value
 *         if not get_interface_addr:
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_cast); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "nvtx/_lib/writer.pyx":416
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")
 *         get_interface_addr = ctypes.cast(
 *             get_interface_obj, ctypes.c_void_p).value             # <<<<<<<<<<<<<<
 *         if not get_interface_addr:
 *             raise WriterError(
*/
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_c_void_p); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_v_get_interface_obj, __pyx_t_7};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 415, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_get_interface_addr = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":417
 *         get_interface_addr = ctypes.cast(
 *             get_interface_obj, ctypes.c_void_p).value
 *         if not get_interface_addr:             # <<<<<<<<<<<<<<
 *             raise WriterError(
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")
*/
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_get_interface_addr); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 417, __pyx_L1_error)
  __pyx_t_8 = (!__pyx_t_4);
  if (unlikely(__pyx_t_8)) {

    /* "nvtx/_lib/writer.pyx":418
 *             get_interface_obj, ctypes.c_void_p).value
 *         if not get_interface_addr:
 *             raise WriterError(             # <<<<<<<<<<<<<<
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")
 *         get_interface = \
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_WriterError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);

    /* "nvtx/_lib/writer.pyx":419
 *         if not get_interface_addr:
 *             raise WriterError(
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")             # <<<<<<<<<<<<<<
 *         get_interface = \
 *             <nvtxwGetInterface_t><void*><uintptr_t>get_interface_addr
*/
    __pyx_t_1 = __Pyx_PyLong_From_int(NVTXW_RESULT_LIBRARY_SYMBOL_MISSING); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_1, __pyx_mstate_global->__pyx_n_u_load_backend};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 418, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 418, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":417
 *         get_interface_addr = ctypes.cast(
 *             get_interface_obj, ctypes.c_void_p).value
 *         if not get_interface_addr:             # <<<<<<<<<<<<<<
 *             raise WriterError(
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")
*/
  }

  /* "nvtx/_lib/writer.pyx":421
 *                 NVTXW_RESULT_LIBRARY_SYMBOL_MISSING, "load_backend")
 *         get_interface = \
 *             <nvtxwGetInterface_t><void*><uintptr_t>get_interface_addr             # <<<<<<<<<<<<<<
 *         rc = get_interface(
 *             NVTXW_INTERFACE_VERSION, <const void**>&iface)
*/
  __pyx_t_9 = __Pyx_PyLong_As_size_t(__pyx_v_get_interface_addr); if (unlikely((__pyx_t_9 == ((uintptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 421, __pyx_L1_error)
  __pyx_v_get_interface = ((nvtxwGetInterface_t)((void *)((uintptr_t)__pyx_t_9)));

  /* "nvtx/_lib/writer.pyx":422
 *         get_interface = \
 *             <nvtxwGetInterface_t><void*><uintptr_t>get_interface_addr
 *         rc = get_interface(             # <<<<<<<<<<<<<<
 *             NVTXW_INTERFACE_VERSION, <const void**>&iface)
 *         _check(rc, _GET_INTERFACE_SYMBOL_NAME)
*/
  __pyx_v_rc = __pyx_v_get_interface(NVTXW_INTERFACE_VERSION, ((void const **)(&__pyx_v_iface)));

  /* "nvtx/_lib/writer.pyx":424
 *         rc = get_interface(
 *             NVTXW_INTERFACE_VERSION, <const void**>&iface)
 *         _check(rc, _GET_INTERFACE_SYMBOL_NAME)             # <<<<<<<<<<<<<<
 *         if iface == NULL:
 *             raise WriterError(NVTXW_RESULT_FAILED, _GET_INTERFACE_SYMBOL_NAME)
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_GET_INTERFACE_SYMBOL_NAME); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 424, __pyx_L1_error)
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, ((PyObject*)__pyx_t_2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":425
 *             NVTXW_INTERFACE_VERSION, <const void**>&iface)
 *         _check(rc, _GET_INTERFACE_SYMBOL_NAME)
 *         if iface == NULL:             # <<<<<<<<<<<<<<
 *             raise WriterError(NVTXW_RESULT_FAILED, _GET_INTERFACE_SYMBOL_NAME)
 *         self._iface = iface
*/
  __pyx_t_8 = (__pyx_v_iface == NULL);
  if (unlikely(__pyx_t_8)) {

    /* "nvtx/_lib/writer.pyx":426
 *         _check(rc, _GET_INTERFACE_SYMBOL_NAME)
 *         if iface == NULL:
 *             raise WriterError(NVTXW_RESULT_FAILED, _GET_INTERFACE_SYMBOL_NAME)             # <<<<<<<<<<<<<<
 *         self._iface = iface
 * 
*/
    __pyx_t_7 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_WriterError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyLong_From_int(NVTXW_RESULT_FAILED); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_GET_INTERFACE_SYMBOL_NAME); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 426, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
      assert(__pyx_t_7);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_7, __pyx_t_3, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 426, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":425
 *             NVTXW_INTERFACE_VERSION, <const void**>&iface)
 *         _check(rc, _GET_INTERFACE_SYMBOL_NAME)
 *         if iface == NULL:             # <<<<<<<<<<<<<<
 *             raise WriterError(NVTXW_RESULT_FAILED, _GET_INTERFACE_SYMBOL_NAME)
 *         self._iface = iface
*/
  }

  /* "nvtx/_lib/writer.pyx":427
 *         if iface == NULL:
 *             raise WriterError(NVTXW_RESULT_FAILED, _GET_INTERFACE_SYMBOL_NAME)
 *         self._iface = iface             # <<<<<<<<<<<<<<
 * 
 *     cdef void _finalize(self) except *:
*/
  __pyx_v_self->_iface = __pyx_v_iface;

  /* "nvtx/_lib/writer.pyx":406
 *         self._refcount = 0
 * 
 *     cdef void _initialize(self) except *:             # <<<<<<<<<<<<<<
 *         cdef nvtxwGetInterface_t get_interface
 *         cdef const nvtxwInterface_v2_t* iface = NULL
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("nvtx._lib.writer._BackendModule._initialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_get_interface_obj);
  __Pyx_XDECREF(__pyx_v_get_interface_addr);
  __Pyx_RefNannyFinishContext();
}

/* "nvtx/_lib/writer.pyx":429
 *         self._iface = iface
 * 
 *     cdef void _finalize(self) except *:             # <<<<<<<<<<<<<<
 *         self._iface = NULL
 *         finalize = getattr(self._lib, _FINALIZE_SYMBOL_NAME, None)
*/

static void __pyx_f_4nvtx_4_lib_6writer_14_BackendModule__finalize(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self) {
  PyObject *__pyx_v_finalize = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_finalize", 0);

  /* "nvtx/_lib/writer.pyx":430
 * 
 *     cdef void _finalize(self) except *:
 *         self._iface = NULL             # <<<<<<<<<<<<<<
 *         finalize = getattr(self._lib, _FINALIZE_SYMBOL_NAME, None)
 *         if finalize is not None:
*/
  __pyx_v_self->_iface = NULL;

  /* "nvtx/_lib/writer.pyx":431
 *     cdef void _finalize(self) except *:
 *         self._iface = NULL
 *         finalize = getattr(self._lib, _FINALIZE_SYMBOL_NAME, None)             # <<<<<<<<<<<<<<
 *         if finalize is not None:
 *             finalize.restype = None
*/
  __pyx_t_1 = __pyx_v_self->_lib;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_FINALIZE_SYMBOL_NAME); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetAttr3(__pyx_t_1, __pyx_t_2, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_finalize = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":432
 *         self._iface = NULL
 *         finalize = getattr(self._lib, _FINALIZE_SYMBOL_NAME, None)
 *         if finalize is not None:             # <<<<<<<<<<<<<<
 *             finalize.restype = None
 *             finalize()
*/
  __pyx_t_4 = (__pyx_v_finalize != Py_None);
  if (__pyx_t_4) {

    /* "nvtx/_lib/writer.pyx":433
 *         finalize = getattr(self._lib, _FINALIZE_SYMBOL_NAME, None)
 *         if finalize is not None:
 *             finalize.restype = None             # <<<<<<<<<<<<<<
 *             finalize()
 * 
*/
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_finalize, __pyx_mstate_global->__pyx_n_u_restype, Py_None) < (0)) __PYX_ERR(0, 433, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":434
 *         if finalize is not None:
 *             finalize.restype = None
 *             finalize()             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_2 = NULL;
    __Pyx_INCREF(__pyx_v_finalize);
    __pyx_t_1 = __pyx_v_finalize; 
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 434, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "nvtx/_lib/writer.pyx":432
 *         self._iface = NULL
 *         finalize = getattr(self._lib, _FINALIZE_SYMBOL_NAME, None)
 *         if finalize is not None:             # <<<<<<<<<<<<<<
 *             finalize.restype = None
 *             finalize()
*/
  }

  /* "nvtx/_lib/writer.pyx":429
 *         self._iface = iface
 * 
 *     cdef void _finalize(self) except *:             # <<<<<<<<<<<<<<
 *         self._iface = NULL
 *         finalize = getattr(self._lib, _FINALIZE_SYMBOL_NAME, None)
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer._BackendModule._finalize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_finalize);
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_14_BackendModule_2__reduce_cython__, "_BackendModule.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_14_BackendModule_3__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_14_BackendModule_2__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_14_BackendModule_2__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_14_BackendModule_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer._BackendModule.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_14_BackendModule_4__setstate_cython__, "_BackendModule.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_14_BackendModule_5__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_14_BackendModule_4__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer._BackendModule.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_14_BackendModule_4__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_14_BackendModule_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer._BackendModule.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":437
 * 
 * 
 * def _release_backend_module(module):             # <<<<<<<<<<<<<<
 *     """Drop one wrapper reference and finalize the module at zero."""
 *     cdef _BackendModule loaded = module
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_11_release_backend_module(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_10_release_backend_module, "_release_backend_module(module)\n\nDrop one wrapper reference and finalize the module at zero.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_11_release_backend_module = {"_release_backend_module", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_11_release_backend_module, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_10_release_backend_module};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_11_release_backend_module(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_module = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_release_backend_module (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_module,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 437, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 437, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_release_backend_module", 0) < (0)) __PYX_ERR(0, 437, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_release_backend_module", 1, 1, 1, i); __PYX_ERR(0, 437, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 437, __pyx_L3_error)
    }
    __pyx_v_module = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_release_backend_module", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 437, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer._release_backend_module", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_10_release_backend_module(__pyx_self, __pyx_v_module);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_10_release_backend_module(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_module) {
  struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_loaded = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_release_backend_module", 0);

  /* "nvtx/_lib/writer.pyx":439
 * def _release_backend_module(module):
 *     """Drop one wrapper reference and finalize the module at zero."""
 *     cdef _BackendModule loaded = module             # <<<<<<<<<<<<<<
 *     with _BACKEND_MODULES_LOCK:
 *         if loaded._refcount <= 0:
*/
  __pyx_t_1 = __pyx_v_module;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule))))) __PYX_ERR(0, 439, __pyx_L1_error)
  __pyx_v_loaded = ((struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":440
 *     """Drop one wrapper reference and finalize the module at zero."""
 *     cdef _BackendModule loaded = module
 *     with _BACKEND_MODULES_LOCK:             # <<<<<<<<<<<<<<
 *         if loaded._refcount <= 0:
 *             raise RuntimeError("backend module reference count underflow")
*/
  /*with:*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_BACKEND_MODULES_LOCK); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 440, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 440, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 440, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 440, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":441
 *     cdef _BackendModule loaded = module
 *     with _BACKEND_MODULES_LOCK:
 *         if loaded._refcount <= 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("backend module reference count underflow")
 *         loaded._refcount -= 1
*/
          __pyx_t_10 = (__pyx_v_loaded->_refcount <= 0);
          if (unlikely(__pyx_t_10)) {

            /* "nvtx/_lib/writer.pyx":442
 *     with _BACKEND_MODULES_LOCK:
 *         if loaded._refcount <= 0:
 *             raise RuntimeError("backend module reference count underflow")             # <<<<<<<<<<<<<<
 *         loaded._refcount -= 1
 *         if loaded._refcount == 0:
*/
            __pyx_t_3 = NULL;
            __pyx_t_6 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_backend_module_reference_count_u};
              __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 442, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_1);
            }
            __Pyx_Raise(__pyx_t_1, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __PYX_ERR(0, 442, __pyx_L7_error)

            /* "nvtx/_lib/writer.pyx":441
 *     cdef _BackendModule loaded = module
 *     with _BACKEND_MODULES_LOCK:
 *         if loaded._refcount <= 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("backend module reference count underflow")
 *         loaded._refcount -= 1
*/
          }

          /* "nvtx/_lib/writer.pyx":443
 *         if loaded._refcount <= 0:
 *             raise RuntimeError("backend module reference count underflow")
 *         loaded._refcount -= 1             # <<<<<<<<<<<<<<
 *         if loaded._refcount == 0:
 *             loaded._finalize()
*/
          __pyx_v_loaded->_refcount = (__pyx_v_loaded->_refcount - 1);

          /* "nvtx/_lib/writer.pyx":444
 *             raise RuntimeError("backend module reference count underflow")
 *         loaded._refcount -= 1
 *         if loaded._refcount == 0:             # <<<<<<<<<<<<<<
 *             loaded._finalize()
 * 
*/
          __pyx_t_10 = (__pyx_v_loaded->_refcount == 0);
          if (__pyx_t_10) {

            /* "nvtx/_lib/writer.pyx":445
 *         loaded._refcount -= 1
 *         if loaded._refcount == 0:
 *             loaded._finalize()             # <<<<<<<<<<<<<<
 * 
 * 
*/
            ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer__BackendModule *)__pyx_v_loaded->__pyx_vtab)->_finalize(__pyx_v_loaded); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 445, __pyx_L7_error)

            /* "nvtx/_lib/writer.pyx":444
 *             raise RuntimeError("backend module reference count underflow")
 *         loaded._refcount -= 1
 *         if loaded._refcount == 0:             # <<<<<<<<<<<<<<
 *             loaded._finalize()
 * 
*/
          }

          /* "nvtx/_lib/writer.pyx":440
 *     """Drop one wrapper reference and finalize the module at zero."""
 *     cdef _BackendModule loaded = module
 *     with _BACKEND_MODULES_LOCK:             # <<<<<<<<<<<<<<
 *         if loaded._refcount <= 0:
 *             raise RuntimeError("backend module reference count underflow")
*/
        }
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer._release_backend_module", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_3, &__pyx_t_5) < 0) __PYX_ERR(0, 440, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_1);
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_t_5);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 440, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 440, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (__pyx_t_10 < (0)) __PYX_ERR(0, 440, __pyx_L9_except_error)
          __pyx_t_12 = (!__pyx_t_10);
          if (unlikely(__pyx_t_12)) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_5);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_3, __pyx_t_5);
            __pyx_t_1 = 0;  __pyx_t_3 = 0;  __pyx_t_5 = 0; 
            __PYX_ERR(0, 440, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_2) {
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 440, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L18;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L1_error;
    __pyx_L18:;
  }

  /* "nvtx/_lib/writer.pyx":437
 * 
 * 
 * def _release_backend_module(module):             # <<<<<<<<<<<<<<
 *     """Drop one wrapper reference and finalize the module at zero."""
 *     cdef _BackendModule loaded = module
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.writer._release_backend_module", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_loaded);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":455
 *     """
 * 
 *     def __cinit__(self, module):             # <<<<<<<<<<<<<<
 *         if not isinstance(module, _BackendModule):
 *             raise ValueError(
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_6writer_7Backend_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_6writer_7Backend_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_module = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_module,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 455, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 455, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(0, 455, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, i); __PYX_ERR(0, 455, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 455, __pyx_L3_error)
    }
    __pyx_v_module = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 455, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Backend.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Backend___cinit__(((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)__pyx_v_self), __pyx_v_module);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_6writer_7Backend___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self, PyObject *__pyx_v_module) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  nvtxwInterface_v2_t const *__pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "nvtx/_lib/writer.pyx":456
 * 
 *     def __cinit__(self, module):
 *         if not isinstance(module, _BackendModule):             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "module must be a loaded backend module; "
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_module, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule); 
  __pyx_t_2 = (!__pyx_t_1);
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/writer.pyx":457
 *     def __cinit__(self, module):
 *         if not isinstance(module, _BackendModule):
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "module must be a loaded backend module; "
 *                 "use nvtx.writer.load_backend to create a Backend"
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_module_must_be_a_loaded_backend};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 457, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 457, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":456
 * 
 *     def __cinit__(self, module):
 *         if not isinstance(module, _BackendModule):             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "module must be a loaded backend module; "
*/
  }

  /* "nvtx/_lib/writer.pyx":461
 *                 "use nvtx.writer.load_backend to create a Backend"
 *             )
 *         self._iface = (<_BackendModule>module)._iface             # <<<<<<<<<<<<<<
 *         self._module = module
 *         self._lock = threading.RLock()
*/
  __pyx_t_6 = ((struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)__pyx_v_module)->_iface;
  __pyx_v_self->_iface = __pyx_t_6;

  /* "nvtx/_lib/writer.pyx":462
 *             )
 *         self._iface = (<_BackendModule>module)._iface
 *         self._module = module             # <<<<<<<<<<<<<<
 *         self._lock = threading.RLock()
 *         self._active_sessions = set()
*/
  __Pyx_INCREF(__pyx_v_module);
  __Pyx_GIVEREF(__pyx_v_module);
  __Pyx_GOTREF(__pyx_v_self->_module);
  __Pyx_DECREF(__pyx_v_self->_module);
  __pyx_v_self->_module = __pyx_v_module;

  /* "nvtx/_lib/writer.pyx":463
 *         self._iface = (<_BackendModule>module)._iface
 *         self._module = module
 *         self._lock = threading.RLock()             # <<<<<<<<<<<<<<
 *         self._active_sessions = set()
 * 
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_threading); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_RLock); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 463, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_lock);
  __Pyx_DECREF(__pyx_v_self->_lock);
  __pyx_v_self->_lock = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":464
 *         self._module = module
 *         self._lock = threading.RLock()
 *         self._active_sessions = set()             # <<<<<<<<<<<<<<
 * 
 *     cdef void _ensure_open(self) except *:
*/
  __pyx_t_3 = PySet_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_active_sessions);
  __Pyx_DECREF(__pyx_v_self->_active_sessions);
  __pyx_v_self->_active_sessions = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":455
 *     """
 * 
 *     def __cinit__(self, module):             # <<<<<<<<<<<<<<
 *         if not isinstance(module, _BackendModule):
 *             raise ValueError(
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("nvtx._lib.writer.Backend.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":466
 *         self._active_sessions = set()
 * 
 *     cdef void _ensure_open(self) except *:             # <<<<<<<<<<<<<<
 *         if self._iface == NULL or self._module is None:
 *             raise RuntimeError("backend is closed")
*/

static void __pyx_f_4nvtx_4_lib_6writer_7Backend__ensure_open(struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_ensure_open", 0);

  /* "nvtx/_lib/writer.pyx":467
 * 
 *     cdef void _ensure_open(self) except *:
 *         if self._iface == NULL or self._module is None:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("backend is closed")
 * 
*/
  __pyx_t_2 = (__pyx_v_self->_iface == NULL);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->_module == Py_None);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":468
 *     cdef void _ensure_open(self) except *:
 *         if self._iface == NULL or self._module is None:
 *             raise RuntimeError("backend is closed")             # <<<<<<<<<<<<<<
 * 
 *     def close(self):
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_backend_is_closed};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 468, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 468, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":467
 * 
 *     cdef void _ensure_open(self) except *:
 *         if self._iface == NULL or self._module is None:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("backend is closed")
 * 
*/
  }

  /* "nvtx/_lib/writer.pyx":466
 *         self._active_sessions = set()
 * 
 *     cdef void _ensure_open(self) except *:             # <<<<<<<<<<<<<<
 *         if self._iface == NULL or self._module is None:
 *             raise RuntimeError("backend is closed")
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.writer.Backend._ensure_open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "nvtx/_lib/writer.pyx":470
 *             raise RuntimeError("backend is closed")
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """
 *         Finalize and release the backend.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Backend_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Backend_2close, "Backend.close(self)\n\nFinalize and release the backend.\n\nActive sessions, and their open streams, are ended first.\n\nCalling this method is optional because backends normally remain\nloaded until the process exits. Calling it more than once has no\neffect. When multiple backends were loaded from the same library, the\nlibrary is finalized only after every wrapper is closed. Its module\nremains loaded until process exit.\n\nRaises\n------\nWriterError\n    If the backend fails to end an active session.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Backend_3close = {"close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Backend_3close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Backend_2close};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Backend_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("close", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("close", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Backend_2close(((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Backend_2close(struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self) {
  PyObject *__pyx_v_error = NULL;
  PyObject *__pyx_v_module = NULL;
  PyObject *__pyx_v_session = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);

  /* "nvtx/_lib/writer.pyx":489
 *         # Best-effort teardown: mark the backend closed and re-raise the first
 *         # session failure at the end. Only finalize after every session ends.
 *         error = None             # <<<<<<<<<<<<<<
 *         module = None
 *         with self._lock:
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_error = ((PyObject*)Py_None);

  /* "nvtx/_lib/writer.pyx":490
 *         # session failure at the end. Only finalize after every session ends.
 *         error = None
 *         module = None             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             if self._module is None:
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_module = Py_None;

  /* "nvtx/_lib/writer.pyx":491
 *         error = None
 *         module = None
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if self._module is None:
 *                 return
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 491, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 491, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 491, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":492
 *         module = None
 *         with self._lock:
 *             if self._module is None:             # <<<<<<<<<<<<<<
 *                 return
 *             for session in list(self._active_sessions):
*/
          __pyx_t_9 = (__pyx_v_self->_module == Py_None);
          if (__pyx_t_9) {

            /* "nvtx/_lib/writer.pyx":493
 *         with self._lock:
 *             if self._module is None:
 *                 return             # <<<<<<<<<<<<<<
 *             for session in list(self._active_sessions):
 *                 try:
*/
            __Pyx_XDECREF(__pyx_r);
            __pyx_r = Py_None; __Pyx_INCREF(Py_None);
            goto __pyx_L11_try_return;

            /* "nvtx/_lib/writer.pyx":492
 *         module = None
 *         with self._lock:
 *             if self._module is None:             # <<<<<<<<<<<<<<
 *                 return
 *             for session in list(self._active_sessions):
*/
          }

          /* "nvtx/_lib/writer.pyx":494
 *             if self._module is None:
 *                 return
 *             for session in list(self._active_sessions):             # <<<<<<<<<<<<<<
 *                 try:
 *                     (<Session>session).end()
*/
          __pyx_t_2 = PySequence_List(__pyx_v_self->_active_sessions); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 494, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_4 = __pyx_t_2; __Pyx_INCREF(__pyx_t_4);
          __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          for (;;) {
            {
              Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
              #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 494, __pyx_L7_error)
              #endif
              if (__pyx_t_10 >= __pyx_temp) break;
            }
            __pyx_t_2 = __Pyx_PyList_GET_ITEM_REF(__pyx_t_4, __pyx_t_10, __Pyx_ReferenceSharing_OwnStrongReference);
            ++__pyx_t_10;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 494, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_XDECREF_SET(__pyx_v_session, __pyx_t_2);
            __pyx_t_2 = 0;

            /* "nvtx/_lib/writer.pyx":495
 *                 return
 *             for session in list(self._active_sessions):
 *                 try:             # <<<<<<<<<<<<<<
 *                     (<Session>session).end()
 *                 except Exception as exc:
*/
            {
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
              __Pyx_XGOTREF(__pyx_t_11);
              __Pyx_XGOTREF(__pyx_t_12);
              __Pyx_XGOTREF(__pyx_t_13);
              /*try:*/ {

                /* "nvtx/_lib/writer.pyx":496
 *             for session in list(self._active_sessions):
 *                 try:
 *                     (<Session>session).end()             # <<<<<<<<<<<<<<
 *                 except Exception as exc:
 *                     if error is None:
*/
                __pyx_t_3 = __pyx_v_session;
                __Pyx_INCREF(__pyx_t_3);
                __pyx_t_5 = 0;
                {
                  PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
                  __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_end, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 496, __pyx_L16_error)
                  __Pyx_GOTREF(__pyx_t_2);
                }
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                /* "nvtx/_lib/writer.pyx":495
 *                 return
 *             for session in list(self._active_sessions):
 *                 try:             # <<<<<<<<<<<<<<
 *                     (<Session>session).end()
 *                 except Exception as exc:
*/
              }
              __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              goto __pyx_L23_try_end;
              __pyx_L16_error:;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "nvtx/_lib/writer.pyx":497
 *                 try:
 *                     (<Session>session).end()
 *                 except Exception as exc:             # <<<<<<<<<<<<<<
 *                     if error is None:
 *                         error = exc
*/
              __pyx_t_14 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
              if (__pyx_t_14) {
                __Pyx_AddTraceback("nvtx._lib.writer.Backend.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
                if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_15) < 0) __PYX_ERR(0, 497, __pyx_L18_except_error)
                __Pyx_XGOTREF(__pyx_t_2);
                __Pyx_XGOTREF(__pyx_t_3);
                __Pyx_XGOTREF(__pyx_t_15);
                __Pyx_INCREF(__pyx_t_3);
                __pyx_v_exc = __pyx_t_3;
                /*try:*/ {

                  /* "nvtx/_lib/writer.pyx":498
 *                     (<Session>session).end()
 *                 except Exception as exc:
 *                     if error is None:             # <<<<<<<<<<<<<<
 *                         error = exc
 *             if error is None:
*/
                  __pyx_t_9 = (__pyx_v_error == ((PyObject*)Py_None));
                  if (__pyx_t_9) {

                    /* "nvtx/_lib/writer.pyx":499
 *                 except Exception as exc:
 *                     if error is None:
 *                         error = exc             # <<<<<<<<<<<<<<
 *             if error is None:
 *                 module = self._module
*/
                    __Pyx_INCREF(__pyx_v_exc);
                    __Pyx_DECREF_SET(__pyx_v_error, __pyx_v_exc);

                    /* "nvtx/_lib/writer.pyx":498
 *                     (<Session>session).end()
 *                 except Exception as exc:
 *                     if error is None:             # <<<<<<<<<<<<<<
 *                         error = exc
 *             if error is None:
*/
                  }
                }

                /* "nvtx/_lib/writer.pyx":497
 *                 try:
 *                     (<Session>session).end()
 *                 except Exception as exc:             # <<<<<<<<<<<<<<
 *                     if error is None:
 *                         error = exc
*/
                /*finally:*/ {
                  /*normal exit:*/{
                    __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
                    goto __pyx_L30;
                  }
                  __pyx_L30:;
                }
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                goto __pyx_L17_exception_handled;
              }
              goto __pyx_L18_except_error;

              /* "nvtx/_lib/writer.pyx":495
 *                 return
 *             for session in list(self._active_sessions):
 *                 try:             # <<<<<<<<<<<<<<
 *                     (<Session>session).end()
 *                 except Exception as exc:
*/
              __pyx_L18_except_error:;
              __Pyx_XGIVEREF(__pyx_t_11);
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
              goto __pyx_L7_error;
              __pyx_L17_exception_handled:;
              __Pyx_XGIVEREF(__pyx_t_11);
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
              __pyx_L23_try_end:;
            }

            /* "nvtx/_lib/writer.pyx":494
 *             if self._module is None:
 *                 return
 *             for session in list(self._active_sessions):             # <<<<<<<<<<<<<<
 *                 try:
 *                     (<Session>session).end()
*/
          }
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

          /* "nvtx/_lib/writer.pyx":500
 *                     if error is None:
 *                         error = exc
 *             if error is None:             # <<<<<<<<<<<<<<
 *                 module = self._module
 *                 self._iface = NULL
*/
          __pyx_t_9 = (__pyx_v_error == ((PyObject*)Py_None));
          if (__pyx_t_9) {

            /* "nvtx/_lib/writer.pyx":501
 *                         error = exc
 *             if error is None:
 *                 module = self._module             # <<<<<<<<<<<<<<
 *                 self._iface = NULL
 *                 self._module = None
*/
            __pyx_t_4 = __pyx_v_self->_module;
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_DECREF_SET(__pyx_v_module, __pyx_t_4);
            __pyx_t_4 = 0;

            /* "nvtx/_lib/writer.pyx":502
 *             if error is None:
 *                 module = self._module
 *                 self._iface = NULL             # <<<<<<<<<<<<<<
 *                 self._module = None
 *         if error is not None:
*/
            __pyx_v_self->_iface = NULL;

            /* "nvtx/_lib/writer.pyx":503
 *                 module = self._module
 *                 self._iface = NULL
 *                 self._module = None             # <<<<<<<<<<<<<<
 *         if error is not None:
 *             raise error
*/
            __Pyx_INCREF(Py_None);
            __Pyx_GIVEREF(Py_None);
            __Pyx_GOTREF(__pyx_v_self->_module);
            __Pyx_DECREF(__pyx_v_self->_module);
            __pyx_v_self->_module = Py_None;

            /* "nvtx/_lib/writer.pyx":500
 *                     if error is None:
 *                         error = exc
 *             if error is None:             # <<<<<<<<<<<<<<
 *                 module = self._module
 *                 self._iface = NULL
*/
          }

          /* "nvtx/_lib/writer.pyx":491
 *         error = None
 *         module = None
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if self._module is None:
 *                 return
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Backend.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_15, &__pyx_t_3) < 0) __PYX_ERR(0, 491, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_3);
          __pyx_t_2 = PyTuple_Pack(3, __pyx_t_4, __pyx_t_15, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 491, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 491, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (__pyx_t_9 < (0)) __PYX_ERR(0, 491, __pyx_L9_except_error)
          __pyx_t_16 = (!__pyx_t_9);
          if (unlikely(__pyx_t_16)) {
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_15);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_4, __pyx_t_15, __pyx_t_3);
            __pyx_t_4 = 0;  __pyx_t_15 = 0;  __pyx_t_3 = 0; 
            __PYX_ERR(0, 491, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 491, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 491, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L37;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L37:;
  }

  /* "nvtx/_lib/writer.pyx":504
 *                 self._iface = NULL
 *                 self._module = None
 *         if error is not None:             # <<<<<<<<<<<<<<
 *             raise error
 *         _release_backend_module(module)
*/
  __pyx_t_16 = (__pyx_v_error != ((PyObject*)Py_None));
  if (unlikely(__pyx_t_16)) {

    /* "nvtx/_lib/writer.pyx":505
 *                 self._module = None
 *         if error is not None:
 *             raise error             # <<<<<<<<<<<<<<
 *         _release_backend_module(module)
 * 
*/
    __Pyx_Raise(__pyx_v_error, 0, 0, 0);
    __PYX_ERR(0, 505, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":504
 *                 self._iface = NULL
 *                 self._module = None
 *         if error is not None:             # <<<<<<<<<<<<<<
 *             raise error
 *         _release_backend_module(module)
*/
  }

  /* "nvtx/_lib/writer.pyx":506
 *         if error is not None:
 *             raise error
 *         _release_backend_module(module)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_15 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_release_backend_module); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_15);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_15);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_v_module};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 506, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":470
 *             raise RuntimeError("backend is closed")
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """
 *         Finalize and release the backend.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("nvtx._lib.writer.Backend.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_XDECREF(__pyx_v_module);
  __Pyx_XDECREF(__pyx_v_session);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Backend_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Backend_4__reduce_cython__, "Backend.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Backend_5__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Backend_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Backend_4__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Backend_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Backend_4__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Backend_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Backend.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Backend_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Backend_6__setstate_cython__, "Backend.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Backend_7__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Backend_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Backend_6__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Backend_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Backend.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Backend_6__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Backend_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Backend *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Backend.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":509
 * 
 * 
 * def load_backend(path):             # <<<<<<<<<<<<<<
 *     """
 *     Load and initialize an NVTXW backend from an explicit library path.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_13load_backend(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_12load_backend, "load_backend(path)\n\nLoad and initialize an NVTXW backend from an explicit library path.\n\nParameters\n----------\npath : pathlib.Path or str\n    Path to the backend shared library.\n\nReturns\n-------\nBackend\n    The loaded backend.\n\nRaises\n------\nOSError\n    If the library cannot be loaded.\nWriterError\n    If the backend does not export ``nvtxwGetInterface`` or does not\n    provide the requested interface version.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_13load_backend = {"load_backend", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_13load_backend, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_12load_backend};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_13load_backend(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("load_backend (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 509, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 509, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "load_backend", 0) < (0)) __PYX_ERR(0, 509, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("load_backend", 1, 1, 1, i); __PYX_ERR(0, 509, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 509, __pyx_L3_error)
    }
    __pyx_v_path = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("load_backend", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 509, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.load_backend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_12load_backend(__pyx_self, __pyx_v_path);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_12load_backend(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_path) {
  struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *__pyx_v_loaded = 0;
  PyObject *__pyx_v_canonical_path = NULL;
  PyObject *__pyx_v_module = NULL;
  PyObject *__pyx_v_lib = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("load_backend", 0);

  /* "nvtx/_lib/writer.pyx":534
 *     cdef _BackendModule loaded
 * 
 *     canonical_path = _canonical_backend_path(path)             # <<<<<<<<<<<<<<
 *     with _BACKEND_MODULES_LOCK:
 *         module = _BACKEND_MODULES.get(canonical_path)
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_canonical_backend_path); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 534, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_path};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 534, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_canonical_path = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":535
 * 
 *     canonical_path = _canonical_backend_path(path)
 *     with _BACKEND_MODULES_LOCK:             # <<<<<<<<<<<<<<
 *         module = _BACKEND_MODULES.get(canonical_path)
 *         if module is None:
*/
  /*with:*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_BACKEND_MODULES_LOCK); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 535, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = NULL;
    __pyx_t_6 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 535, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 535, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":536
 *     canonical_path = _canonical_backend_path(path)
 *     with _BACKEND_MODULES_LOCK:
 *         module = _BACKEND_MODULES.get(canonical_path)             # <<<<<<<<<<<<<<
 *         if module is None:
 *             lib = ctypes.CDLL(canonical_path)
*/
          __pyx_t_3 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_BACKEND_MODULES); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 536, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 536, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_4 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_2))) {
            __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
            assert(__pyx_t_3);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
            __pyx_t_4 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_canonical_path};
            __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 536, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
          }
          __pyx_v_module = __pyx_t_1;
          __pyx_t_1 = 0;

          /* "nvtx/_lib/writer.pyx":537
 *     with _BACKEND_MODULES_LOCK:
 *         module = _BACKEND_MODULES.get(canonical_path)
 *         if module is None:             # <<<<<<<<<<<<<<
 *             lib = ctypes.CDLL(canonical_path)
 *             module = _BackendModule(lib)
*/
          __pyx_t_10 = (__pyx_v_module == Py_None);
          if (__pyx_t_10) {

            /* "nvtx/_lib/writer.pyx":538
 *         module = _BACKEND_MODULES.get(canonical_path)
 *         if module is None:
 *             lib = ctypes.CDLL(canonical_path)             # <<<<<<<<<<<<<<
 *             module = _BackendModule(lib)
 *             _BACKEND_MODULES[canonical_path] = module
*/
            __pyx_t_2 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 538, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_CDLL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 538, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_4 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_6))) {
              __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
              assert(__pyx_t_2);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
              __Pyx_INCREF(__pyx_t_2);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
              __pyx_t_4 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_canonical_path};
              __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
              if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 538, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_1);
            }
            __pyx_v_lib = __pyx_t_1;
            __pyx_t_1 = 0;

            /* "nvtx/_lib/writer.pyx":539
 *         if module is None:
 *             lib = ctypes.CDLL(canonical_path)
 *             module = _BackendModule(lib)             # <<<<<<<<<<<<<<
 *             _BACKEND_MODULES[canonical_path] = module
 * 
*/
            __pyx_t_6 = NULL;
            __pyx_t_4 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_lib};
              __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L7_error)
              __Pyx_GOTREF((PyObject *)__pyx_t_1);
            }
            __Pyx_DECREF_SET(__pyx_v_module, ((PyObject *)__pyx_t_1));
            __pyx_t_1 = 0;

            /* "nvtx/_lib/writer.pyx":540
 *             lib = ctypes.CDLL(canonical_path)
 *             module = _BackendModule(lib)
 *             _BACKEND_MODULES[canonical_path] = module             # <<<<<<<<<<<<<<
 * 
 *         loaded = <_BackendModule>module
*/
            __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_BACKEND_MODULES); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 540, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
            if (unlikely((PyObject_SetItem(__pyx_t_1, __pyx_v_canonical_path, __pyx_v_module) < 0))) __PYX_ERR(0, 540, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "nvtx/_lib/writer.pyx":537
 *     with _BACKEND_MODULES_LOCK:
 *         module = _BACKEND_MODULES.get(canonical_path)
 *         if module is None:             # <<<<<<<<<<<<<<
 *             lib = ctypes.CDLL(canonical_path)
 *             module = _BackendModule(lib)
*/
          }

          /* "nvtx/_lib/writer.pyx":542
 *             _BACKEND_MODULES[canonical_path] = module
 * 
 *         loaded = <_BackendModule>module             # <<<<<<<<<<<<<<
 *         if loaded._refcount == 0:
 *             try:
*/
          __pyx_t_1 = __pyx_v_module;
          __Pyx_INCREF(__pyx_t_1);
          __pyx_v_loaded = ((struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)__pyx_t_1);
          __pyx_t_1 = 0;

          /* "nvtx/_lib/writer.pyx":543
 * 
 *         loaded = <_BackendModule>module
 *         if loaded._refcount == 0:             # <<<<<<<<<<<<<<
 *             try:
 *                 loaded._initialize()
*/
          __pyx_t_10 = (__pyx_v_loaded->_refcount == 0);
          if (__pyx_t_10) {

            /* "nvtx/_lib/writer.pyx":544
 *         loaded = <_BackendModule>module
 *         if loaded._refcount == 0:
 *             try:             # <<<<<<<<<<<<<<
 *                 loaded._initialize()
 *             except Exception:
*/
            {
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
              __Pyx_XGOTREF(__pyx_t_11);
              __Pyx_XGOTREF(__pyx_t_12);
              __Pyx_XGOTREF(__pyx_t_13);
              /*try:*/ {

                /* "nvtx/_lib/writer.pyx":545
 *         if loaded._refcount == 0:
 *             try:
 *                 loaded._initialize()             # <<<<<<<<<<<<<<
 *             except Exception:
 *                 loaded._finalize()
*/
                ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer__BackendModule *)__pyx_v_loaded->__pyx_vtab)->_initialize(__pyx_v_loaded); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 545, __pyx_L15_error)

                /* "nvtx/_lib/writer.pyx":544
 *         loaded = <_BackendModule>module
 *         if loaded._refcount == 0:
 *             try:             # <<<<<<<<<<<<<<
 *                 loaded._initialize()
 *             except Exception:
*/
              }
              __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              goto __pyx_L20_try_end;
              __pyx_L15_error:;
              __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

              /* "nvtx/_lib/writer.pyx":546
 *             try:
 *                 loaded._initialize()
 *             except Exception:             # <<<<<<<<<<<<<<
 *                 loaded._finalize()
 *                 raise
*/
              __pyx_t_14 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
              if (__pyx_t_14) {
                __Pyx_AddTraceback("nvtx._lib.writer.load_backend", __pyx_clineno, __pyx_lineno, __pyx_filename);
                if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_6, &__pyx_t_2) < 0) __PYX_ERR(0, 546, __pyx_L17_except_error)
                __Pyx_XGOTREF(__pyx_t_1);
                __Pyx_XGOTREF(__pyx_t_6);
                __Pyx_XGOTREF(__pyx_t_2);

                /* "nvtx/_lib/writer.pyx":547
 *                 loaded._initialize()
 *             except Exception:
 *                 loaded._finalize()             # <<<<<<<<<<<<<<
 *                 raise
 *         loaded._refcount += 1
*/
                ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer__BackendModule *)__pyx_v_loaded->__pyx_vtab)->_finalize(__pyx_v_loaded); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 547, __pyx_L17_except_error)

                /* "nvtx/_lib/writer.pyx":548
 *             except Exception:
 *                 loaded._finalize()
 *                 raise             # <<<<<<<<<<<<<<
 *         loaded._refcount += 1
 *         try:
*/
                __Pyx_GIVEREF(__pyx_t_1);
                __Pyx_GIVEREF(__pyx_t_6);
                __Pyx_XGIVEREF(__pyx_t_2);
                __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_6, __pyx_t_2);
                __pyx_t_1 = 0;  __pyx_t_6 = 0;  __pyx_t_2 = 0; 
                __PYX_ERR(0, 548, __pyx_L17_except_error)
              }
              goto __pyx_L17_except_error;

              /* "nvtx/_lib/writer.pyx":544
 *         loaded = <_BackendModule>module
 *         if loaded._refcount == 0:
 *             try:             # <<<<<<<<<<<<<<
 *                 loaded._initialize()
 *             except Exception:
*/
              __pyx_L17_except_error:;
              __Pyx_XGIVEREF(__pyx_t_11);
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
              goto __pyx_L7_error;
              __pyx_L20_try_end:;
            }

            /* "nvtx/_lib/writer.pyx":543
 * 
 *         loaded = <_BackendModule>module
 *         if loaded._refcount == 0:             # <<<<<<<<<<<<<<
 *             try:
 *                 loaded._initialize()
*/
          }

          /* "nvtx/_lib/writer.pyx":549
 *                 loaded._finalize()
 *                 raise
 *         loaded._refcount += 1             # <<<<<<<<<<<<<<
 *         try:
 *             return Backend(module)
*/
          __pyx_v_loaded->_refcount = (__pyx_v_loaded->_refcount + 1);

          /* "nvtx/_lib/writer.pyx":550
 *                 raise
 *         loaded._refcount += 1
 *         try:             # <<<<<<<<<<<<<<
 *             return Backend(module)
 *         except Exception:
*/
          {
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __Pyx_ExceptionSave(&__pyx_t_13, &__pyx_t_12, &__pyx_t_11);
            __Pyx_XGOTREF(__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_12);
            __Pyx_XGOTREF(__pyx_t_11);
            /*try:*/ {

              /* "nvtx/_lib/writer.pyx":551
 *         loaded._refcount += 1
 *         try:
 *             return Backend(module)             # <<<<<<<<<<<<<<
 *         except Exception:
 *             _release_backend_module(module)
*/
              __Pyx_XDECREF(__pyx_r);
              __pyx_t_6 = NULL;
              __pyx_t_4 = 1;
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_module};
                __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Backend, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
                if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 551, __pyx_L23_error)
                __Pyx_GOTREF((PyObject *)__pyx_t_2);
              }
              __pyx_r = ((PyObject *)__pyx_t_2);
              __pyx_t_2 = 0;
              goto __pyx_L27_try_return;

              /* "nvtx/_lib/writer.pyx":550
 *                 raise
 *         loaded._refcount += 1
 *         try:             # <<<<<<<<<<<<<<
 *             return Backend(module)
 *         except Exception:
*/
            }
            __pyx_L23_error:;
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

            /* "nvtx/_lib/writer.pyx":552
 *         try:
 *             return Backend(module)
 *         except Exception:             # <<<<<<<<<<<<<<
 *             _release_backend_module(module)
 *             raise
*/
            __pyx_t_14 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
            if (__pyx_t_14) {
              __Pyx_AddTraceback("nvtx._lib.writer.load_backend", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_6, &__pyx_t_1) < 0) __PYX_ERR(0, 552, __pyx_L25_except_error)
              __Pyx_XGOTREF(__pyx_t_2);
              __Pyx_XGOTREF(__pyx_t_6);
              __Pyx_XGOTREF(__pyx_t_1);

              /* "nvtx/_lib/writer.pyx":553
 *             return Backend(module)
 *         except Exception:
 *             _release_backend_module(module)             # <<<<<<<<<<<<<<
 *             raise
 * 
*/
              __pyx_t_15 = NULL;
              __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_release_backend_module); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 553, __pyx_L25_except_error)
              __Pyx_GOTREF(__pyx_t_16);
              __pyx_t_4 = 1;
              #if CYTHON_UNPACK_METHODS
              if (unlikely(PyMethod_Check(__pyx_t_16))) {
                __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_16);
                assert(__pyx_t_15);
                PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_16);
                __Pyx_INCREF(__pyx_t_15);
                __Pyx_INCREF(__pyx__function);
                __Pyx_DECREF_SET(__pyx_t_16, __pyx__function);
                __pyx_t_4 = 0;
              }
              #endif
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_v_module};
                __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_16, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
                if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 553, __pyx_L25_except_error)
                __Pyx_GOTREF(__pyx_t_3);
              }
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "nvtx/_lib/writer.pyx":554
 *         except Exception:
 *             _release_backend_module(module)
 *             raise             # <<<<<<<<<<<<<<
 * 
 * 
*/
              __Pyx_GIVEREF(__pyx_t_2);
              __Pyx_GIVEREF(__pyx_t_6);
              __Pyx_XGIVEREF(__pyx_t_1);
              __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_6, __pyx_t_1);
              __pyx_t_2 = 0;  __pyx_t_6 = 0;  __pyx_t_1 = 0; 
              __PYX_ERR(0, 554, __pyx_L25_except_error)
            }
            goto __pyx_L25_except_error;

            /* "nvtx/_lib/writer.pyx":550
 *                 raise
 *         loaded._refcount += 1
 *         try:             # <<<<<<<<<<<<<<
 *             return Backend(module)
 *         except Exception:
*/
            __pyx_L25_except_error:;
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_ExceptionReset(__pyx_t_13, __pyx_t_12, __pyx_t_11);
            goto __pyx_L7_error;
            __pyx_L27_try_return:;
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_ExceptionReset(__pyx_t_13, __pyx_t_12, __pyx_t_11);
            goto __pyx_L11_try_return;
          }

          /* "nvtx/_lib/writer.pyx":535
 * 
 *     canonical_path = _canonical_backend_path(path)
 *     with _BACKEND_MODULES_LOCK:             # <<<<<<<<<<<<<<
 *         module = _BACKEND_MODULES.get(canonical_path)
 *         if module is None:
*/
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.load_backend", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_6, &__pyx_t_2) < 0) __PYX_ERR(0, 535, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_1);
          __Pyx_XGOTREF(__pyx_t_6);
          __Pyx_XGOTREF(__pyx_t_2);
          __pyx_t_3 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_6, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 535, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 535, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (__pyx_t_10 < (0)) __PYX_ERR(0, 535, __pyx_L9_except_error)
          __pyx_t_17 = (!__pyx_t_10);
          if (unlikely(__pyx_t_17)) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_6);
            __Pyx_XGIVEREF(__pyx_t_2);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_6, __pyx_t_2);
            __pyx_t_1 = 0;  __pyx_t_6 = 0;  __pyx_t_2 = 0; 
            __PYX_ERR(0, 535, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_5) {
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 535, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_9 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_5) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 535, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __pyx_r = __pyx_t_9;
        __pyx_t_9 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L34;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L1_error;
    __pyx_L34:;
  }

  /* "nvtx/_lib/writer.pyx":509
 * 
 * 
 * def load_backend(path):             # <<<<<<<<<<<<<<
 *     """
 *     Load and initialize an NVTXW backend from an explicit library path.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("nvtx._lib.writer.load_backend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_loaded);
  __Pyx_XDECREF(__pyx_v_canonical_path);
  __Pyx_XDECREF(__pyx_v_module);
  __Pyx_XDECREF(__pyx_v_lib);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":585
 *     """
 * 
 *     def __init__(self, name, *, backend, config=None):             # <<<<<<<<<<<<<<
 *         if not isinstance(backend, Backend):
 *             raise TypeError("backend must be an nvtx.writer.Backend")
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_6writer_7Session_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_6writer_7Session_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_backend = 0;
  PyObject *__pyx_v_config = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_backend,&__pyx_mstate_global->__pyx_n_u_config,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 585, __pyx_L3_error)
    if (likely(__pyx_kwds_len > 0)) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 585, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 585, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, i); __PYX_ERR(0, 585, __pyx_L3_error) }
      }
      for (Py_ssize_t i = 1; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseKeywordRequired("__init__", *(__pyx_pyargnames[i - 0])); __PYX_ERR(0, 585, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("__init__", __pyx_mstate_global->__pyx_n_u_backend); __PYX_ERR(0, 585, __pyx_L3_error)
    }
    __pyx_v_name = values[0];
    __pyx_v_backend = values[1];
    __pyx_v_config = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 585, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Session.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session___init__(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self), __pyx_v_name, __pyx_v_backend, __pyx_v_config);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_6writer_7Session___init__(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_backend, PyObject *__pyx_v_config) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "nvtx/_lib/writer.pyx":586
 * 
 *     def __init__(self, name, *, backend, config=None):
 *         if not isinstance(backend, Backend):             # <<<<<<<<<<<<<<
 *             raise TypeError("backend must be an nvtx.writer.Backend")
 *         self._backend = backend
*/
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_backend, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Backend); 
  __pyx_t_2 = (!__pyx_t_1);
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/writer.pyx":587
 *     def __init__(self, name, *, backend, config=None):
 *         if not isinstance(backend, Backend):
 *             raise TypeError("backend must be an nvtx.writer.Backend")             # <<<<<<<<<<<<<<
 *         self._backend = backend
 *         self._name = _as_bytes(name)
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_backend_must_be_an_nvtx_writer_B};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 587, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 587, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":586
 * 
 *     def __init__(self, name, *, backend, config=None):
 *         if not isinstance(backend, Backend):             # <<<<<<<<<<<<<<
 *             raise TypeError("backend must be an nvtx.writer.Backend")
 *         self._backend = backend
*/
  }

  /* "nvtx/_lib/writer.pyx":588
 *         if not isinstance(backend, Backend):
 *             raise TypeError("backend must be an nvtx.writer.Backend")
 *         self._backend = backend             # <<<<<<<<<<<<<<
 *         self._name = _as_bytes(name)
 *         self._config = _as_bytes(config)
*/
  __pyx_t_3 = __pyx_v_backend;
  __Pyx_INCREF(__pyx_t_3);
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Backend))))) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_backend);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_backend);
  __pyx_v_self->_backend = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":589
 *             raise TypeError("backend must be an nvtx.writer.Backend")
 *         self._backend = backend
 *         self._name = _as_bytes(name)             # <<<<<<<<<<<<<<
 *         self._config = _as_bytes(config)
 *         self._handle = NULL
*/
  __pyx_t_3 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 589, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_name);
  __Pyx_DECREF(__pyx_v_self->_name);
  __pyx_v_self->_name = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":590
 *         self._backend = backend
 *         self._name = _as_bytes(name)
 *         self._config = _as_bytes(config)             # <<<<<<<<<<<<<<
 *         self._handle = NULL
 *         self._lock = threading.RLock()
*/
  __pyx_t_3 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_config); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_config);
  __Pyx_DECREF(__pyx_v_self->_config);
  __pyx_v_self->_config = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":591
 *         self._name = _as_bytes(name)
 *         self._config = _as_bytes(config)
 *         self._handle = NULL             # <<<<<<<<<<<<<<
 *         self._lock = threading.RLock()
 *         self._open_streams = set()
*/
  __pyx_v_self->_handle = NULL;

  /* "nvtx/_lib/writer.pyx":592
 *         self._config = _as_bytes(config)
 *         self._handle = NULL
 *         self._lock = threading.RLock()             # <<<<<<<<<<<<<<
 *         self._open_streams = set()
 *         self._domains = {}
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_threading); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_RLock); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 592, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_lock);
  __Pyx_DECREF(__pyx_v_self->_lock);
  __pyx_v_self->_lock = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":593
 *         self._handle = NULL
 *         self._lock = threading.RLock()
 *         self._open_streams = set()             # <<<<<<<<<<<<<<
 *         self._domains = {}
 * 
*/
  __pyx_t_3 = PySet_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_open_streams);
  __Pyx_DECREF(__pyx_v_self->_open_streams);
  __pyx_v_self->_open_streams = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":594
 *         self._lock = threading.RLock()
 *         self._open_streams = set()
 *         self._domains = {}             # <<<<<<<<<<<<<<
 * 
 *     def begin(self):
*/
  __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_domains);
  __Pyx_DECREF(__pyx_v_self->_domains);
  __pyx_v_self->_domains = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":585
 *     """
 * 
 *     def __init__(self, name, *, backend, config=None):             # <<<<<<<<<<<<<<
 *         if not isinstance(backend, Backend):
 *             raise TypeError("backend must be an nvtx.writer.Backend")
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("nvtx._lib.writer.Session.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":596
 *         self._domains = {}
 * 
 *     def begin(self):             # <<<<<<<<<<<<<<
 *         """
 *         Start the session.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_3begin(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Session_2begin, "Session.begin(self)\n\nStart the session.\n\nWhen applicable, prefer using a context manager\n(``with Session(\"my session\", backend=backend) as session:``) over\ncalling this method explicitly.\n\nRaises\n------\nRuntimeError\n    If the session is already active or the backend is closed.\nWriterError\n    If the backend fails to begin the session.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Session_3begin = {"begin", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_3begin, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_2begin};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_3begin(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("begin (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("begin", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("begin", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session_2begin(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_2begin(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self) {
  nvtxwSessionAttributes_t __pyx_v_attr;
  char const *__pyx_v_name;
  char const *__pyx_v_config;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  char const *__pyx_t_14;
  char const *__pyx_t_15;
  int __pyx_t_16;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("begin", 0);

  /* "nvtx/_lib/writer.pyx":612
 *         """
 *         cdef nvtxwSessionAttributes_t attr
 *         cdef const char* name = NULL             # <<<<<<<<<<<<<<
 *         cdef const char* config = NULL
 *         cdef nvtxwResultCode_t rc
*/
  __pyx_v_name = NULL;

  /* "nvtx/_lib/writer.pyx":613
 *         cdef nvtxwSessionAttributes_t attr
 *         cdef const char* name = NULL
 *         cdef const char* config = NULL             # <<<<<<<<<<<<<<
 *         cdef nvtxwResultCode_t rc
 *         # When both are needed, always acquire the backend lock first.
*/
  __pyx_v_config = NULL;

  /* "nvtx/_lib/writer.pyx":616
 *         cdef nvtxwResultCode_t rc
 *         # When both are needed, always acquire the backend lock first.
 *         with self._backend._lock:             # <<<<<<<<<<<<<<
 *             with self._lock:
 *                 if self._handle != NULL:
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_backend->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 616, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_backend->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 616, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 616, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":617
 *         # When both are needed, always acquire the backend lock first.
 *         with self._backend._lock:
 *             with self._lock:             # <<<<<<<<<<<<<<
 *                 if self._handle != NULL:
 *                     raise RuntimeError("session is already active")
*/
          /*with:*/ {
            __pyx_t_9 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 617, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_4 = NULL;
            __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 617, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_5 = 1;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_3))) {
              __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
              assert(__pyx_t_4);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
              __pyx_t_5 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 617, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            /*try:*/ {
              {
                __Pyx_PyThreadState_declare
                __Pyx_PyThreadState_assign
                __Pyx_ExceptionSave(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
                __Pyx_XGOTREF(__pyx_t_10);
                __Pyx_XGOTREF(__pyx_t_11);
                __Pyx_XGOTREF(__pyx_t_12);
                /*try:*/ {

                  /* "nvtx/_lib/writer.pyx":618
 *         with self._backend._lock:
 *             with self._lock:
 *                 if self._handle != NULL:             # <<<<<<<<<<<<<<
 *                     raise RuntimeError("session is already active")
 *                 self._backend._ensure_open()
*/
                  __pyx_t_13 = (__pyx_v_self->_handle != NULL);
                  if (unlikely(__pyx_t_13)) {

                    /* "nvtx/_lib/writer.pyx":619
 *             with self._lock:
 *                 if self._handle != NULL:
 *                     raise RuntimeError("session is already active")             # <<<<<<<<<<<<<<
 *                 self._backend._ensure_open()
 *                 if self._name is not None:
*/
                    __pyx_t_3 = NULL;
                    __pyx_t_5 = 1;
                    {
                      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_session_is_already_active};
                      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 619, __pyx_L17_error)
                      __Pyx_GOTREF(__pyx_t_2);
                    }
                    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
                    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                    __PYX_ERR(0, 619, __pyx_L17_error)

                    /* "nvtx/_lib/writer.pyx":618
 *         with self._backend._lock:
 *             with self._lock:
 *                 if self._handle != NULL:             # <<<<<<<<<<<<<<
 *                     raise RuntimeError("session is already active")
 *                 self._backend._ensure_open()
*/
                  }

                  /* "nvtx/_lib/writer.pyx":620
 *                 if self._handle != NULL:
 *                     raise RuntimeError("session is already active")
 *                 self._backend._ensure_open()             # <<<<<<<<<<<<<<
 *                 if self._name is not None:
 *                     name = self._name
*/
                  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Backend *)__pyx_v_self->_backend->__pyx_vtab)->_ensure_open(__pyx_v_self->_backend); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 620, __pyx_L17_error)

                  /* "nvtx/_lib/writer.pyx":621
 *                     raise RuntimeError("session is already active")
 *                 self._backend._ensure_open()
 *                 if self._name is not None:             # <<<<<<<<<<<<<<
 *                     name = self._name
 *                 if self._config is not None:
*/
                  __pyx_t_13 = (__pyx_v_self->_name != ((PyObject*)Py_None));
                  if (__pyx_t_13) {

                    /* "nvtx/_lib/writer.pyx":622
 *                 self._backend._ensure_open()
 *                 if self._name is not None:
 *                     name = self._name             # <<<<<<<<<<<<<<
 *                 if self._config is not None:
 *                     config = self._config
*/
                    if (unlikely(__pyx_v_self->_name == Py_None)) {
                      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
                      __PYX_ERR(0, 622, __pyx_L17_error)
                    }
                    __pyx_t_14 = __Pyx_PyBytes_AsString(__pyx_v_self->_name); if (unlikely((!__pyx_t_14) && PyErr_Occurred())) __PYX_ERR(0, 622, __pyx_L17_error)
                    __pyx_v_name = __pyx_t_14;

                    /* "nvtx/_lib/writer.pyx":621
 *                     raise RuntimeError("session is already active")
 *                 self._backend._ensure_open()
 *                 if self._name is not None:             # <<<<<<<<<<<<<<
 *                     name = self._name
 *                 if self._config is not None:
*/
                  }

                  /* "nvtx/_lib/writer.pyx":623
 *                 if self._name is not None:
 *                     name = self._name
 *                 if self._config is not None:             # <<<<<<<<<<<<<<
 *                     config = self._config
 *                 attr.structSize = sizeof(nvtxwSessionAttributes_t)
*/
                  __pyx_t_13 = (__pyx_v_self->_config != ((PyObject*)Py_None));
                  if (__pyx_t_13) {

                    /* "nvtx/_lib/writer.pyx":624
 *                     name = self._name
 *                 if self._config is not None:
 *                     config = self._config             # <<<<<<<<<<<<<<
 *                 attr.structSize = sizeof(nvtxwSessionAttributes_t)
 *                 attr.name = name
*/
                    if (unlikely(__pyx_v_self->_config == Py_None)) {
                      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
                      __PYX_ERR(0, 624, __pyx_L17_error)
                    }
                    __pyx_t_15 = __Pyx_PyBytes_AsString(__pyx_v_self->_config); if (unlikely((!__pyx_t_15) && PyErr_Occurred())) __PYX_ERR(0, 624, __pyx_L17_error)
                    __pyx_v_config = __pyx_t_15;

                    /* "nvtx/_lib/writer.pyx":623
 *                 if self._name is not None:
 *                     name = self._name
 *                 if self._config is not None:             # <<<<<<<<<<<<<<
 *                     config = self._config
 *                 attr.structSize = sizeof(nvtxwSessionAttributes_t)
*/
                  }

                  /* "nvtx/_lib/writer.pyx":625
 *                 if self._config is not None:
 *                     config = self._config
 *                 attr.structSize = sizeof(nvtxwSessionAttributes_t)             # <<<<<<<<<<<<<<
 *                 attr.name = name
 *                 attr.configString = config
*/
                  __pyx_v_attr.structSize = (sizeof(nvtxwSessionAttributes_t));

                  /* "nvtx/_lib/writer.pyx":626
 *                     config = self._config
 *                 attr.structSize = sizeof(nvtxwSessionAttributes_t)
 *                 attr.name = name             # <<<<<<<<<<<<<<
 *                 attr.configString = config
 *                 self._backend._active_sessions.add(self)
*/
                  __pyx_v_attr.name = __pyx_v_name;

                  /* "nvtx/_lib/writer.pyx":627
 *                 attr.structSize = sizeof(nvtxwSessionAttributes_t)
 *                 attr.name = name
 *                 attr.configString = config             # <<<<<<<<<<<<<<
 *                 self._backend._active_sessions.add(self)
 *                 rc = self._backend._iface.SessionBegin(
*/
                  __pyx_v_attr.configString = __pyx_v_config;

                  /* "nvtx/_lib/writer.pyx":628
 *                 attr.name = name
 *                 attr.configString = config
 *                 self._backend._active_sessions.add(self)             # <<<<<<<<<<<<<<
 *                 rc = self._backend._iface.SessionBegin(
 *                     &attr, &self._handle)
*/
                  if (unlikely(__pyx_v_self->_backend->_active_sessions == Py_None)) {
                    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
                    __PYX_ERR(0, 628, __pyx_L17_error)
                  }
                  __pyx_t_16 = PySet_Add(__pyx_v_self->_backend->_active_sessions, ((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 628, __pyx_L17_error)

                  /* "nvtx/_lib/writer.pyx":629
 *                 attr.configString = config
 *                 self._backend._active_sessions.add(self)
 *                 rc = self._backend._iface.SessionBegin(             # <<<<<<<<<<<<<<
 *                     &attr, &self._handle)
 *                 if rc != NVTXW_RESULT_SUCCESS:
*/
                  __pyx_v_rc = __pyx_v_self->_backend->_iface->SessionBegin((&__pyx_v_attr), (&__pyx_v_self->_handle));

                  /* "nvtx/_lib/writer.pyx":631
 *                 rc = self._backend._iface.SessionBegin(
 *                     &attr, &self._handle)
 *                 if rc != NVTXW_RESULT_SUCCESS:             # <<<<<<<<<<<<<<
 *                     self._backend._active_sessions.discard(self)
 *                     self._handle = NULL
*/
                  __pyx_t_13 = (__pyx_v_rc != NVTXW_RESULT_SUCCESS);
                  if (unlikely(__pyx_t_13)) {

                    /* "nvtx/_lib/writer.pyx":632
 *                     &attr, &self._handle)
 *                 if rc != NVTXW_RESULT_SUCCESS:
 *                     self._backend._active_sessions.discard(self)             # <<<<<<<<<<<<<<
 *                     self._handle = NULL
 *                     raise WriterError(rc, "SessionBegin")
*/
                    if (unlikely(__pyx_v_self->_backend->_active_sessions == Py_None)) {
                      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "discard");
                      __PYX_ERR(0, 632, __pyx_L17_error)
                    }
                    __pyx_t_16 = __Pyx_PySet_Discard(__pyx_v_self->_backend->_active_sessions, ((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 632, __pyx_L17_error)

                    /* "nvtx/_lib/writer.pyx":633
 *                 if rc != NVTXW_RESULT_SUCCESS:
 *                     self._backend._active_sessions.discard(self)
 *                     self._handle = NULL             # <<<<<<<<<<<<<<
 *                     raise WriterError(rc, "SessionBegin")
 * 
*/
                    __pyx_v_self->_handle = NULL;

                    /* "nvtx/_lib/writer.pyx":634
 *                     self._backend._active_sessions.discard(self)
 *                     self._handle = NULL
 *                     raise WriterError(rc, "SessionBegin")             # <<<<<<<<<<<<<<
 * 
 *     def end(self):
*/
                    __pyx_t_3 = NULL;
                    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_WriterError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 634, __pyx_L17_error)
                    __Pyx_GOTREF(__pyx_t_4);
                    __pyx_t_17 = __Pyx_PyLong_From_int32_t(__pyx_v_rc); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 634, __pyx_L17_error)
                    __Pyx_GOTREF(__pyx_t_17);
                    __pyx_t_5 = 1;
                    #if CYTHON_UNPACK_METHODS
                    if (unlikely(PyMethod_Check(__pyx_t_4))) {
                      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
                      assert(__pyx_t_3);
                      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
                      __Pyx_INCREF(__pyx_t_3);
                      __Pyx_INCREF(__pyx__function);
                      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
                      __pyx_t_5 = 0;
                    }
                    #endif
                    {
                      PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_17, __pyx_mstate_global->__pyx_n_u_SessionBegin};
                      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
                      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L17_error)
                      __Pyx_GOTREF(__pyx_t_2);
                    }
                    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
                    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                    __PYX_ERR(0, 634, __pyx_L17_error)

                    /* "nvtx/_lib/writer.pyx":631
 *                 rc = self._backend._iface.SessionBegin(
 *                     &attr, &self._handle)
 *                 if rc != NVTXW_RESULT_SUCCESS:             # <<<<<<<<<<<<<<
 *                     self._backend._active_sessions.discard(self)
 *                     self._handle = NULL
*/
                  }

                  /* "nvtx/_lib/writer.pyx":617
 *         # When both are needed, always acquire the backend lock first.
 *         with self._backend._lock:
 *             with self._lock:             # <<<<<<<<<<<<<<
 *                 if self._handle != NULL:
 *                     raise RuntimeError("session is already active")
*/
                }
                __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
                __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
                goto __pyx_L22_try_end;
                __pyx_L17_error:;
                __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                /*except:*/ {
                  __Pyx_AddTraceback("nvtx._lib.writer.Session.begin", __pyx_clineno, __pyx_lineno, __pyx_filename);
                  if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_17) < 0) __PYX_ERR(0, 617, __pyx_L19_except_error)
                  __Pyx_XGOTREF(__pyx_t_2);
                  __Pyx_XGOTREF(__pyx_t_4);
                  __Pyx_XGOTREF(__pyx_t_17);
                  __pyx_t_3 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_4, __pyx_t_17); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 617, __pyx_L19_except_error)
                  __Pyx_GOTREF(__pyx_t_3);
                  __pyx_t_18 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_3, NULL);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                  if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 617, __pyx_L19_except_error)
                  __Pyx_GOTREF(__pyx_t_18);
                  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_18);
                  __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
                  if (__pyx_t_13 < (0)) __PYX_ERR(0, 617, __pyx_L19_except_error)
                  __pyx_t_19 = (!__pyx_t_13);
                  if (unlikely(__pyx_t_19)) {
                    __Pyx_GIVEREF(__pyx_t_2);
                    __Pyx_GIVEREF(__pyx_t_4);
                    __Pyx_XGIVEREF(__pyx_t_17);
                    __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_4, __pyx_t_17);
                    __pyx_t_2 = 0;  __pyx_t_4 = 0;  __pyx_t_17 = 0; 
                    __PYX_ERR(0, 617, __pyx_L19_except_error)
                  }
                  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                  __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
                  goto __pyx_L18_exception_handled;
                }
                __pyx_L19_except_error:;
                __Pyx_XGIVEREF(__pyx_t_10);
                __Pyx_XGIVEREF(__pyx_t_11);
                __Pyx_XGIVEREF(__pyx_t_12);
                __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
                goto __pyx_L7_error;
                __pyx_L18_exception_handled:;
                __Pyx_XGIVEREF(__pyx_t_10);
                __Pyx_XGIVEREF(__pyx_t_11);
                __Pyx_XGIVEREF(__pyx_t_12);
                __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
                __pyx_L22_try_end:;
              }
            }
            /*finally:*/ {
              /*normal exit:*/{
                if (__pyx_t_9) {
                  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_mstate_global->__pyx_tuple[0], NULL);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 617, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_12);
                  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
                }
                goto __pyx_L16;
              }
              __pyx_L16:;
            }
            goto __pyx_L30;
            __pyx_L13_error:;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            goto __pyx_L7_error;
            __pyx_L30:;
          }

          /* "nvtx/_lib/writer.pyx":616
 *         cdef nvtxwResultCode_t rc
 *         # When both are needed, always acquire the backend lock first.
 *         with self._backend._lock:             # <<<<<<<<<<<<<<
 *             with self._lock:
 *                 if self._handle != NULL:
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Session.begin", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_17, &__pyx_t_4, &__pyx_t_2) < 0) __PYX_ERR(0, 616, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_2);
          __pyx_t_3 = PyTuple_Pack(3, __pyx_t_17, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 616, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 616, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_19 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_19 < (0)) __PYX_ERR(0, 616, __pyx_L9_except_error)
          __pyx_t_13 = (!__pyx_t_19);
          if (unlikely(__pyx_t_13)) {
            __Pyx_GIVEREF(__pyx_t_17);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_2);
            __Pyx_ErrRestoreWithState(__pyx_t_17, __pyx_t_4, __pyx_t_2);
            __pyx_t_17 = 0;  __pyx_t_4 = 0;  __pyx_t_2 = 0; 
            __PYX_ERR(0, 616, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 616, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L34;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L34:;
  }

  /* "nvtx/_lib/writer.pyx":596
 *         self._domains = {}
 * 
 *     def begin(self):             # <<<<<<<<<<<<<<
 *         """
 *         Start the session.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("nvtx._lib.writer.Session.begin", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":636
 *                     raise WriterError(rc, "SessionBegin")
 * 
 *     def end(self):             # <<<<<<<<<<<<<<
 *         """
 *         End the session.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_5end(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Session_4end, "Session.end(self)\n\nEnd the session.\n\nStreams still open in this session are closed first. Objects created\nfrom the session (domains, streams, registered strings, scopes, and\ncounters) become invalid when it ends.\n\nWhen applicable, prefer using a context manager\n(``with Session(\"my session\", backend=backend) as session:``) over\ncalling this method explicitly.\n\nRaises\n------\nRuntimeError\n    If the session is not active or the backend is closed.\nWriterError\n    If the backend fails to close a stream or end the session.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Session_5end = {"end", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_5end, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_4end};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_5end(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("end (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("end", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("end", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session_4end(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_4end(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self) {
  nvtxwSessionHandle_t __pyx_v_handle;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_v_error = NULL;
  PyObject *__pyx_v_stream = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_v_domain = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_t_20;
  nvtxwSessionHandle_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  int __pyx_t_23;
  int __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("end", 0);

  /* "nvtx/_lib/writer.pyx":660
 *         # in a consistent terminal state, re-raising the first failure at the
 *         # end.
 *         error = None             # <<<<<<<<<<<<<<
 *         with self._backend._lock:
 *             with self._lock:
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_error = ((PyObject*)Py_None);

  /* "nvtx/_lib/writer.pyx":661
 *         # end.
 *         error = None
 *         with self._backend._lock:             # <<<<<<<<<<<<<<
 *             with self._lock:
 *                 if self._handle == NULL:
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_backend->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 661, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_backend->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 661, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 661, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":662
 *         error = None
 *         with self._backend._lock:
 *             with self._lock:             # <<<<<<<<<<<<<<
 *                 if self._handle == NULL:
 *                     raise RuntimeError("session is not active")
*/
          /*with:*/ {
            __pyx_t_9 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 662, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_4 = NULL;
            __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 662, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_5 = 1;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_3))) {
              __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
              assert(__pyx_t_4);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
              __pyx_t_5 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 662, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            /*try:*/ {
              {
                __Pyx_PyThreadState_declare
                __Pyx_PyThreadState_assign
                __Pyx_ExceptionSave(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
                __Pyx_XGOTREF(__pyx_t_10);
                __Pyx_XGOTREF(__pyx_t_11);
                __Pyx_XGOTREF(__pyx_t_12);
                /*try:*/ {

                  /* "nvtx/_lib/writer.pyx":663
 *         with self._backend._lock:
 *             with self._lock:
 *                 if self._handle == NULL:             # <<<<<<<<<<<<<<
 *                     raise RuntimeError("session is not active")
 *                 self._backend._ensure_open()
*/
                  __pyx_t_13 = (__pyx_v_self->_handle == NULL);
                  if (unlikely(__pyx_t_13)) {

                    /* "nvtx/_lib/writer.pyx":664
 *             with self._lock:
 *                 if self._handle == NULL:
 *                     raise RuntimeError("session is not active")             # <<<<<<<<<<<<<<
 *                 self._backend._ensure_open()
 *                 for stream in list(self._open_streams):
*/
                    __pyx_t_3 = NULL;
                    __pyx_t_5 = 1;
                    {
                      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_session_is_not_active};
                      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 664, __pyx_L17_error)
                      __Pyx_GOTREF(__pyx_t_2);
                    }
                    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
                    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                    __PYX_ERR(0, 664, __pyx_L17_error)

                    /* "nvtx/_lib/writer.pyx":663
 *         with self._backend._lock:
 *             with self._lock:
 *                 if self._handle == NULL:             # <<<<<<<<<<<<<<
 *                     raise RuntimeError("session is not active")
 *                 self._backend._ensure_open()
*/
                  }

                  /* "nvtx/_lib/writer.pyx":665
 *                 if self._handle == NULL:
 *                     raise RuntimeError("session is not active")
 *                 self._backend._ensure_open()             # <<<<<<<<<<<<<<
 *                 for stream in list(self._open_streams):
 *                     try:
*/
                  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Backend *)__pyx_v_self->_backend->__pyx_vtab)->_ensure_open(__pyx_v_self->_backend); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 665, __pyx_L17_error)

                  /* "nvtx/_lib/writer.pyx":666
 *                     raise RuntimeError("session is not active")
 *                 self._backend._ensure_open()
 *                 for stream in list(self._open_streams):             # <<<<<<<<<<<<<<
 *                     try:
 *                         stream.close()
*/
                  __pyx_t_2 = PySequence_List(__pyx_v_self->_open_streams); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 666, __pyx_L17_error)
                  __Pyx_GOTREF(__pyx_t_2);
                  __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3);
                  __pyx_t_14 = 0;
                  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                  for (;;) {
                    {
                      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_3);
                      #if !CYTHON_ASSUME_SAFE_SIZE
                      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 666, __pyx_L17_error)
                      #endif
                      if (__pyx_t_14 >= __pyx_temp) break;
                    }
                    __pyx_t_2 = __Pyx_PyList_GET_ITEM_REF(__pyx_t_3, __pyx_t_14, __Pyx_ReferenceSharing_OwnStrongReference);
                    ++__pyx_t_14;
                    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 666, __pyx_L17_error)
                    __Pyx_GOTREF(__pyx_t_2);
                    __Pyx_XDECREF_SET(__pyx_v_stream, __pyx_t_2);
                    __pyx_t_2 = 0;

                    /* "nvtx/_lib/writer.pyx":667
 *                 self._backend._ensure_open()
 *                 for stream in list(self._open_streams):
 *                     try:             # <<<<<<<<<<<<<<
 *                         stream.close()
 *                     except Exception as exc:
*/
                    {
                      __Pyx_PyThreadState_declare
                      __Pyx_PyThreadState_assign
                      __Pyx_ExceptionSave(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
                      __Pyx_XGOTREF(__pyx_t_15);
                      __Pyx_XGOTREF(__pyx_t_16);
                      __Pyx_XGOTREF(__pyx_t_17);
                      /*try:*/ {

                        /* "nvtx/_lib/writer.pyx":668
 *                 for stream in list(self._open_streams):
 *                     try:
 *                         stream.close()             # <<<<<<<<<<<<<<
 *                     except Exception as exc:
 *                         if error is None:
*/
                        __pyx_t_4 = __pyx_v_stream;
                        __Pyx_INCREF(__pyx_t_4);
                        __pyx_t_5 = 0;
                        {
                          PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
                          __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_close, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 668, __pyx_L26_error)
                          __Pyx_GOTREF(__pyx_t_2);
                        }
                        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                        /* "nvtx/_lib/writer.pyx":667
 *                 self._backend._ensure_open()
 *                 for stream in list(self._open_streams):
 *                     try:             # <<<<<<<<<<<<<<
 *                         stream.close()
 *                     except Exception as exc:
*/
                      }
                      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
                      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
                      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
                      goto __pyx_L33_try_end;
                      __pyx_L26_error:;
                      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

                      /* "nvtx/_lib/writer.pyx":669
 *                     try:
 *                         stream.close()
 *                     except Exception as exc:             # <<<<<<<<<<<<<<
 *                         if error is None:
 *                             error = exc
*/
                      __pyx_t_18 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
                      if (__pyx_t_18) {
                        __Pyx_AddTraceback("nvtx._lib.writer.Session.end", __pyx_clineno, __pyx_lineno, __pyx_filename);
                        if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_19) < 0) __PYX_ERR(0, 669, __pyx_L28_except_error)
                        __Pyx_XGOTREF(__pyx_t_2);
                        __Pyx_XGOTREF(__pyx_t_4);
                        __Pyx_XGOTREF(__pyx_t_19);
                        __Pyx_INCREF(__pyx_t_4);
                        __pyx_v_exc = __pyx_t_4;
                        /*try:*/ {

                          /* "nvtx/_lib/writer.pyx":670
 *                         stream.close()
 *                     except Exception as exc:
 *                         if error is None:             # <<<<<<<<<<<<<<
 *                             error = exc
 *                 self._open_streams.clear()
*/
                          __pyx_t_13 = (__pyx_v_error == ((PyObject*)Py_None));
                          if (__pyx_t_13) {

                            /* "nvtx/_lib/writer.pyx":671
 *                     except Exception as exc:
 *                         if error is None:
 *                             error = exc             # <<<<<<<<<<<<<<
 *                 self._open_streams.clear()
 *                 handle = self._handle
*/
                            __Pyx_INCREF(__pyx_v_exc);
                            __Pyx_DECREF_SET(__pyx_v_error, __pyx_v_exc);

                            /* "nvtx/_lib/writer.pyx":670
 *                         stream.close()
 *                     except Exception as exc:
 *                         if error is None:             # <<<<<<<<<<<<<<
 *                             error = exc
 *                 self._open_streams.clear()
*/
                          }
                        }

                        /* "nvtx/_lib/writer.pyx":669
 *                     try:
 *                         stream.close()
 *                     except Exception as exc:             # <<<<<<<<<<<<<<
 *                         if error is None:
 *                             error = exc
*/
                        /*finally:*/ {
                          /*normal exit:*/{
                            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
                            goto __pyx_L40;
                          }
                          __pyx_L40:;
                        }
                        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                        __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                        goto __pyx_L27_exception_handled;
                      }
                      goto __pyx_L28_except_error;

                      /* "nvtx/_lib/writer.pyx":667
 *                 self._backend._ensure_open()
 *                 for stream in list(self._open_streams):
 *                     try:             # <<<<<<<<<<<<<<
 *                         stream.close()
 *                     except Exception as exc:
*/
                      __pyx_L28_except_error:;
                      __Pyx_XGIVEREF(__pyx_t_15);
                      __Pyx_XGIVEREF(__pyx_t_16);
                      __Pyx_XGIVEREF(__pyx_t_17);
                      __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
                      goto __pyx_L17_error;
                      __pyx_L27_exception_handled:;
                      __Pyx_XGIVEREF(__pyx_t_15);
                      __Pyx_XGIVEREF(__pyx_t_16);
                      __Pyx_XGIVEREF(__pyx_t_17);
                      __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
                      __pyx_L33_try_end:;
                    }

                    /* "nvtx/_lib/writer.pyx":666
 *                     raise RuntimeError("session is not active")
 *                 self._backend._ensure_open()
 *                 for stream in list(self._open_streams):             # <<<<<<<<<<<<<<
 *                     try:
 *                         stream.close()
*/
                  }
                  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

                  /* "nvtx/_lib/writer.pyx":672
 *                         if error is None:
 *                             error = exc
 *                 self._open_streams.clear()             # <<<<<<<<<<<<<<
 *                 handle = self._handle
 *                 self._handle = NULL
*/
                  if (unlikely(__pyx_v_self->_open_streams == Py_None)) {
                    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "clear");
                    __PYX_ERR(0, 672, __pyx_L17_error)
                  }
                  __pyx_t_20 = PySet_Clear(__pyx_v_self->_open_streams); if (unlikely(__pyx_t_20 == ((int)-1))) __PYX_ERR(0, 672, __pyx_L17_error)

                  /* "nvtx/_lib/writer.pyx":673
 *                             error = exc
 *                 self._open_streams.clear()
 *                 handle = self._handle             # <<<<<<<<<<<<<<
 *                 self._handle = NULL
 *                 for domain in self._domains.values():
*/
                  __pyx_t_21 = __pyx_v_self->_handle;
                  __pyx_v_handle = __pyx_t_21;

                  /* "nvtx/_lib/writer.pyx":674
 *                 self._open_streams.clear()
 *                 handle = self._handle
 *                 self._handle = NULL             # <<<<<<<<<<<<<<
 *                 for domain in self._domains.values():
 *                     (<Domain>domain)._invalidate()
*/
                  __pyx_v_self->_handle = NULL;

                  /* "nvtx/_lib/writer.pyx":675
 *                 handle = self._handle
 *                 self._handle = NULL
 *                 for domain in self._domains.values():             # <<<<<<<<<<<<<<
 *                     (<Domain>domain)._invalidate()
 *                 self._domains.clear()
*/
                  __pyx_t_14 = 0;
                  if (unlikely(__pyx_v_self->_domains == Py_None)) {
                    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
                    __PYX_ERR(0, 675, __pyx_L17_error)
                  }
                  __pyx_t_19 = __Pyx_dict_iterator(__pyx_v_self->_domains, 1, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_22), (&__pyx_t_18)); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 675, __pyx_L17_error)
                  __Pyx_GOTREF(__pyx_t_19);
                  __Pyx_XDECREF(__pyx_t_3);
                  __pyx_t_3 = __pyx_t_19;
                  __pyx_t_19 = 0;
                  while (1) {
                    __pyx_t_23 = __Pyx_dict_iter_next(__pyx_t_3, __pyx_t_22, &__pyx_t_14, NULL, &__pyx_t_19, NULL, __pyx_t_18);
                    if (unlikely(__pyx_t_23 == 0)) break;
                    if (unlikely(__pyx_t_23 == -1)) __PYX_ERR(0, 675, __pyx_L17_error)
                    __Pyx_GOTREF(__pyx_t_19);
                    __Pyx_XDECREF_SET(__pyx_v_domain, __pyx_t_19);
                    __pyx_t_19 = 0;

                    /* "nvtx/_lib/writer.pyx":676
 *                 self._handle = NULL
 *                 for domain in self._domains.values():
 *                     (<Domain>domain)._invalidate()             # <<<<<<<<<<<<<<
 *                 self._domains.clear()
 *                 self._backend._active_sessions.discard(self)
*/
                    __pyx_t_19 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_domain)->__pyx_vtab)->_invalidate(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_domain)); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 676, __pyx_L17_error)
                    __Pyx_GOTREF(__pyx_t_19);
                    __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
                  }
                  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

                  /* "nvtx/_lib/writer.pyx":677
 *                 for domain in self._domains.values():
 *                     (<Domain>domain)._invalidate()
 *                 self._domains.clear()             # <<<<<<<<<<<<<<
 *                 self._backend._active_sessions.discard(self)
 *                 rc = self._backend._iface.SessionEnd(handle)
*/
                  if (unlikely(__pyx_v_self->_domains == Py_None)) {
                    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "clear");
                    __PYX_ERR(0, 677, __pyx_L17_error)
                  }
                  __pyx_t_20 = __Pyx_PyDict_Clear(__pyx_v_self->_domains); if (unlikely(__pyx_t_20 == ((int)-1))) __PYX_ERR(0, 677, __pyx_L17_error)

                  /* "nvtx/_lib/writer.pyx":678
 *                     (<Domain>domain)._invalidate()
 *                 self._domains.clear()
 *                 self._backend._active_sessions.discard(self)             # <<<<<<<<<<<<<<
 *                 rc = self._backend._iface.SessionEnd(handle)
 *                 if rc != NVTXW_RESULT_SUCCESS:
*/
                  if (unlikely(__pyx_v_self->_backend->_active_sessions == Py_None)) {
                    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "discard");
                    __PYX_ERR(0, 678, __pyx_L17_error)
                  }
                  __pyx_t_20 = __Pyx_PySet_Discard(__pyx_v_self->_backend->_active_sessions, ((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_20 == ((int)-1))) __PYX_ERR(0, 678, __pyx_L17_error)

                  /* "nvtx/_lib/writer.pyx":679
 *                 self._domains.clear()
 *                 self._backend._active_sessions.discard(self)
 *                 rc = self._backend._iface.SessionEnd(handle)             # <<<<<<<<<<<<<<
 *                 if rc != NVTXW_RESULT_SUCCESS:
 *                     raise WriterError(rc, "SessionEnd") from error
*/
                  __pyx_v_rc = __pyx_v_self->_backend->_iface->SessionEnd(__pyx_v_handle);

                  /* "nvtx/_lib/writer.pyx":680
 *                 self._backend._active_sessions.discard(self)
 *                 rc = self._backend._iface.SessionEnd(handle)
 *                 if rc != NVTXW_RESULT_SUCCESS:             # <<<<<<<<<<<<<<
 *                     raise WriterError(rc, "SessionEnd") from error
 *         if error is not None:
*/
                  __pyx_t_13 = (__pyx_v_rc != NVTXW_RESULT_SUCCESS);
                  if (unlikely(__pyx_t_13)) {

                    /* "nvtx/_lib/writer.pyx":681
 *                 rc = self._backend._iface.SessionEnd(handle)
 *                 if rc != NVTXW_RESULT_SUCCESS:
 *                     raise WriterError(rc, "SessionEnd") from error             # <<<<<<<<<<<<<<
 *         if error is not None:
 *             raise error
*/
                    __pyx_t_19 = NULL;
                    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_WriterError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 681, __pyx_L17_error)
                    __Pyx_GOTREF(__pyx_t_4);
                    __pyx_t_2 = __Pyx_PyLong_From_int32_t(__pyx_v_rc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 681, __pyx_L17_error)
                    __Pyx_GOTREF(__pyx_t_2);
                    __pyx_t_5 = 1;
                    #if CYTHON_UNPACK_METHODS
                    if (unlikely(PyMethod_Check(__pyx_t_4))) {
                      __pyx_t_19 = PyMethod_GET_SELF(__pyx_t_4);
                      assert(__pyx_t_19);
                      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
                      __Pyx_INCREF(__pyx_t_19);
                      __Pyx_INCREF(__pyx__function);
                      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
                      __pyx_t_5 = 0;
                    }
                    #endif
                    {
                      PyObject *__pyx_callargs[3] = {__pyx_t_19, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_SessionEnd};
                      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                      __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 681, __pyx_L17_error)
                      __Pyx_GOTREF(__pyx_t_3);
                    }
                    __Pyx_Raise(__pyx_t_3, 0, 0, __pyx_v_error);
                    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                    __PYX_ERR(0, 681, __pyx_L17_error)

                    /* "nvtx/_lib/writer.pyx":680
 *                 self._backend._active_sessions.discard(self)
 *                 rc = self._backend._iface.SessionEnd(handle)
 *                 if rc != NVTXW_RESULT_SUCCESS:             # <<<<<<<<<<<<<<
 *                     raise WriterError(rc, "SessionEnd") from error
 *         if error is not None:
*/
                  }

                  /* "nvtx/_lib/writer.pyx":662
 *         error = None
 *         with self._backend._lock:
 *             with self._lock:             # <<<<<<<<<<<<<<
 *                 if self._handle == NULL:
 *                     raise RuntimeError("session is not active")
*/
                }
                __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
                __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
                goto __pyx_L22_try_end;
                __pyx_L17_error:;
                __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                /*except:*/ {
                  __Pyx_AddTraceback("nvtx._lib.writer.Session.end", __pyx_clineno, __pyx_lineno, __pyx_filename);
                  if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_4, &__pyx_t_2) < 0) __PYX_ERR(0, 662, __pyx_L19_except_error)
                  __Pyx_XGOTREF(__pyx_t_3);
                  __Pyx_XGOTREF(__pyx_t_4);
                  __Pyx_XGOTREF(__pyx_t_2);
                  __pyx_t_19 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 662, __pyx_L19_except_error)
                  __Pyx_GOTREF(__pyx_t_19);
                  __pyx_t_17 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_19, NULL);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
                  if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 662, __pyx_L19_except_error)
                  __Pyx_GOTREF(__pyx_t_17);
                  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_17);
                  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
                  if (__pyx_t_13 < (0)) __PYX_ERR(0, 662, __pyx_L19_except_error)
                  __pyx_t_24 = (!__pyx_t_13);
                  if (unlikely(__pyx_t_24)) {
                    __Pyx_GIVEREF(__pyx_t_3);
                    __Pyx_GIVEREF(__pyx_t_4);
                    __Pyx_XGIVEREF(__pyx_t_2);
                    __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_4, __pyx_t_2);
                    __pyx_t_3 = 0;  __pyx_t_4 = 0;  __pyx_t_2 = 0; 
                    __PYX_ERR(0, 662, __pyx_L19_except_error)
                  }
                  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                  goto __pyx_L18_exception_handled;
                }
                __pyx_L19_except_error:;
                __Pyx_XGIVEREF(__pyx_t_10);
                __Pyx_XGIVEREF(__pyx_t_11);
                __Pyx_XGIVEREF(__pyx_t_12);
                __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
                goto __pyx_L7_error;
                __pyx_L18_exception_handled:;
                __Pyx_XGIVEREF(__pyx_t_10);
                __Pyx_XGIVEREF(__pyx_t_11);
                __Pyx_XGIVEREF(__pyx_t_12);
                __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
                __pyx_L22_try_end:;
              }
            }
            /*finally:*/ {
              /*normal exit:*/{
                if (__pyx_t_9) {
                  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_mstate_global->__pyx_tuple[0], NULL);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 662, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_12);
                  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
                }
                goto __pyx_L16;
              }
              __pyx_L16:;
            }
            goto __pyx_L49;
            __pyx_L13_error:;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            goto __pyx_L7_error;
            __pyx_L49:;
          }

          /* "nvtx/_lib/writer.pyx":661
 *         # end.
 *         error = None
 *         with self._backend._lock:             # <<<<<<<<<<<<<<
 *             with self._lock:
 *                 if self._handle == NULL:
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Session.end", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 661, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_3);
          __pyx_t_19 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 661, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_19);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_19, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 661, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_24 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_24 < (0)) __PYX_ERR(0, 661, __pyx_L9_except_error)
          __pyx_t_13 = (!__pyx_t_24);
          if (unlikely(__pyx_t_13)) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_4, __pyx_t_3);
            __pyx_t_2 = 0;  __pyx_t_4 = 0;  __pyx_t_3 = 0; 
            __PYX_ERR(0, 661, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 661, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L53;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L53:;
  }

  /* "nvtx/_lib/writer.pyx":682
 *                 if rc != NVTXW_RESULT_SUCCESS:
 *                     raise WriterError(rc, "SessionEnd") from error
 *         if error is not None:             # <<<<<<<<<<<<<<
 *             raise error
 * 
*/
  __pyx_t_13 = (__pyx_v_error != ((PyObject*)Py_None));
  if (unlikely(__pyx_t_13)) {

    /* "nvtx/_lib/writer.pyx":683
 *                     raise WriterError(rc, "SessionEnd") from error
 *         if error is not None:
 *             raise error             # <<<<<<<<<<<<<<
 * 
 *     def _register_domain(self, name):
*/
    __Pyx_Raise(__pyx_v_error, 0, 0, 0);
    __PYX_ERR(0, 683, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":682
 *                 if rc != NVTXW_RESULT_SUCCESS:
 *                     raise WriterError(rc, "SessionEnd") from error
 *         if error is not None:             # <<<<<<<<<<<<<<
 *             raise error
 * 
*/
  }

  /* "nvtx/_lib/writer.pyx":636
 *                     raise WriterError(rc, "SessionBegin")
 * 
 *     def end(self):             # <<<<<<<<<<<<<<
 *         """
 *         End the session.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_AddTraceback("nvtx._lib.writer.Session.end", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_XDECREF(__pyx_v_stream);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v_domain);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":685
 *             raise error
 * 
 *     def _register_domain(self, name):             # <<<<<<<<<<<<<<
 *         cdef nvtxwDomainAttributes_t attr
 *         cdef nvtxDomainHandle_t handle = NULL
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_7_register_domain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Session_6_register_domain, "Session._register_domain(self, name)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Session_7_register_domain = {"_register_domain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_7_register_domain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_6_register_domain};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_7_register_domain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_register_domain (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 685, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 685, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_register_domain", 0) < (0)) __PYX_ERR(0, 685, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_register_domain", 1, 1, 1, i); __PYX_ERR(0, 685, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 685, __pyx_L3_error)
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_register_domain", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 685, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Session._register_domain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session_6_register_domain(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_6_register_domain(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, PyObject *__pyx_v_name) {
  nvtxwDomainAttributes_t __pyx_v_attr;
  nvtxDomainHandle_t __pyx_v_handle;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_v_encoded = 0;
  char const *__pyx_v_c_name;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  char const *__pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_register_domain", 0);

  /* "nvtx/_lib/writer.pyx":687
 *     def _register_domain(self, name):
 *         cdef nvtxwDomainAttributes_t attr
 *         cdef nvtxDomainHandle_t handle = NULL             # <<<<<<<<<<<<<<
 *         cdef nvtxwResultCode_t rc
 *         cdef bytes encoded = _as_bytes(name)
*/
  __pyx_v_handle = NULL;

  /* "nvtx/_lib/writer.pyx":689
 *         cdef nvtxDomainHandle_t handle = NULL
 *         cdef nvtxwResultCode_t rc
 *         cdef bytes encoded = _as_bytes(name)             # <<<<<<<<<<<<<<
 *         cdef const char* c_name = NULL
 *         if encoded is not None:
*/
  __pyx_t_1 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_encoded = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":690
 *         cdef nvtxwResultCode_t rc
 *         cdef bytes encoded = _as_bytes(name)
 *         cdef const char* c_name = NULL             # <<<<<<<<<<<<<<
 *         if encoded is not None:
 *             c_name = encoded
*/
  __pyx_v_c_name = NULL;

  /* "nvtx/_lib/writer.pyx":691
 *         cdef bytes encoded = _as_bytes(name)
 *         cdef const char* c_name = NULL
 *         if encoded is not None:             # <<<<<<<<<<<<<<
 *             c_name = encoded
 *         self._backend._ensure_open()
*/
  __pyx_t_2 = (__pyx_v_encoded != ((PyObject*)Py_None));
  if (__pyx_t_2) {

    /* "nvtx/_lib/writer.pyx":692
 *         cdef const char* c_name = NULL
 *         if encoded is not None:
 *             c_name = encoded             # <<<<<<<<<<<<<<
 *         self._backend._ensure_open()
 *         attr.structSize = sizeof(nvtxwDomainAttributes_t)
*/
    if (unlikely(__pyx_v_encoded == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 692, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyBytes_AsString(__pyx_v_encoded); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(0, 692, __pyx_L1_error)
    __pyx_v_c_name = __pyx_t_3;

    /* "nvtx/_lib/writer.pyx":691
 *         cdef bytes encoded = _as_bytes(name)
 *         cdef const char* c_name = NULL
 *         if encoded is not None:             # <<<<<<<<<<<<<<
 *             c_name = encoded
 *         self._backend._ensure_open()
*/
  }

  /* "nvtx/_lib/writer.pyx":693
 *         if encoded is not None:
 *             c_name = encoded
 *         self._backend._ensure_open()             # <<<<<<<<<<<<<<
 *         attr.structSize = sizeof(nvtxwDomainAttributes_t)
 *         attr.name = c_name
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Backend *)__pyx_v_self->_backend->__pyx_vtab)->_ensure_open(__pyx_v_self->_backend); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 693, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":694
 *             c_name = encoded
 *         self._backend._ensure_open()
 *         attr.structSize = sizeof(nvtxwDomainAttributes_t)             # <<<<<<<<<<<<<<
 *         attr.name = c_name
 *         rc = self._backend._iface.DomainRegister(self._handle, &attr, &handle)
*/
  __pyx_v_attr.structSize = (sizeof(nvtxwDomainAttributes_t));

  /* "nvtx/_lib/writer.pyx":695
 *         self._backend._ensure_open()
 *         attr.structSize = sizeof(nvtxwDomainAttributes_t)
 *         attr.name = c_name             # <<<<<<<<<<<<<<
 *         rc = self._backend._iface.DomainRegister(self._handle, &attr, &handle)
 *         _check(rc, "DomainRegister")
*/
  __pyx_v_attr.name = __pyx_v_c_name;

  /* "nvtx/_lib/writer.pyx":696
 *         attr.structSize = sizeof(nvtxwDomainAttributes_t)
 *         attr.name = c_name
 *         rc = self._backend._iface.DomainRegister(self._handle, &attr, &handle)             # <<<<<<<<<<<<<<
 *         _check(rc, "DomainRegister")
 *         return Domain(self, <uintptr_t>handle)
*/
  __pyx_v_rc = __pyx_v_self->_backend->_iface->DomainRegister(__pyx_v_self->_handle, (&__pyx_v_attr), (&__pyx_v_handle));

  /* "nvtx/_lib/writer.pyx":697
 *         attr.name = c_name
 *         rc = self._backend._iface.DomainRegister(self._handle, &attr, &handle)
 *         _check(rc, "DomainRegister")             # <<<<<<<<<<<<<<
 *         return Domain(self, <uintptr_t>handle)
 * 
*/
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_DomainRegister); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 697, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":698
 *         rc = self._backend._iface.DomainRegister(self._handle, &attr, &handle)
 *         _check(rc, "DomainRegister")
 *         return Domain(self, <uintptr_t>handle)             # <<<<<<<<<<<<<<
 * 
 *     def get_domain(self, name=None):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = NULL;
  __pyx_t_5 = __Pyx_PyLong_FromSize_t(((uintptr_t)__pyx_v_handle)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_4, ((PyObject *)__pyx_v_self), __pyx_t_5};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 698, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":685
 *             raise error
 * 
 *     def _register_domain(self, name):             # <<<<<<<<<<<<<<
 *         cdef nvtxwDomainAttributes_t attr
 *         cdef nvtxDomainHandle_t handle = NULL
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.writer.Session._register_domain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":700
 *         return Domain(self, <uintptr_t>handle)
 * 
 *     def get_domain(self, name=None):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a domain within this session.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_9get_domain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Session_8get_domain, "Session.get_domain(self, name=None)\n\nGet or create a domain within this session.\n\nParameters\n----------\nname : str or bytes, optional\n    Domain name. ``None`` or an empty name selects the session's\n    default domain.\n\nReturns\n-------\nDomain\n    The requested domain. Repeated calls with the same name return\n    the same object.\n\nRaises\n------\nRuntimeError\n    If the session is not active.\nTypeError\n    If ``name`` is not a str, bytes, or None.\nWriterError\n    If the backend fails to register the domain.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Session_9get_domain = {"get_domain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_9get_domain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_8get_domain};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_9get_domain(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_domain (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 700, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 700, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_domain", 0) < (0)) __PYX_ERR(0, 700, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 700, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_domain", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 700, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Session.get_domain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session_8get_domain(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_8get_domain(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, PyObject *__pyx_v_name) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_domain", 0);
  __Pyx_INCREF(__pyx_v_name);

  /* "nvtx/_lib/writer.pyx":726
 *         """
 *         cdef Domain domain
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if self._handle == NULL:
 *                 raise RuntimeError("session is not active")
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 726, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":727
 *         cdef Domain domain
 *         with self._lock:
 *             if self._handle == NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("session is not active")
 *             name = _as_str(name) or None
*/
          __pyx_t_9 = (__pyx_v_self->_handle == NULL);
          if (unlikely(__pyx_t_9)) {

            /* "nvtx/_lib/writer.pyx":728
 *         with self._lock:
 *             if self._handle == NULL:
 *                 raise RuntimeError("session is not active")             # <<<<<<<<<<<<<<
 *             name = _as_str(name) or None
 *             domain = self._domains.get(name)
*/
            __pyx_t_4 = NULL;
            __pyx_t_5 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_session_is_not_active};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 728, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_Raise(__pyx_t_2, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __PYX_ERR(0, 728, __pyx_L7_error)

            /* "nvtx/_lib/writer.pyx":727
 *         cdef Domain domain
 *         with self._lock:
 *             if self._handle == NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("session is not active")
 *             name = _as_str(name) or None
*/
          }

          /* "nvtx/_lib/writer.pyx":729
 *             if self._handle == NULL:
 *                 raise RuntimeError("session is not active")
 *             name = _as_str(name) or None             # <<<<<<<<<<<<<<
 *             domain = self._domains.get(name)
 *             if domain is None:
*/
          __pyx_t_4 = __pyx_f_4nvtx_4_lib_6writer__as_str(__pyx_v_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 729, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 729, __pyx_L7_error)
          if (!__pyx_t_9) {
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          } else {
            __Pyx_INCREF(__pyx_t_4);
            __pyx_t_2 = __pyx_t_4;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            goto __pyx_L14_bool_binop_done;
          }
          __Pyx_INCREF(Py_None);
          __pyx_t_2 = Py_None;
          __pyx_L14_bool_binop_done:;
          __Pyx_DECREF_SET(__pyx_v_name, __pyx_t_2);
          __pyx_t_2 = 0;

          /* "nvtx/_lib/writer.pyx":730
 *                 raise RuntimeError("session is not active")
 *             name = _as_str(name) or None
 *             domain = self._domains.get(name)             # <<<<<<<<<<<<<<
 *             if domain is None:
 *                 domain = self._register_domain(name)
*/
          if (unlikely(__pyx_v_self->_domains == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
            __PYX_ERR(0, 730, __pyx_L7_error)
          }
          __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_domains, __pyx_v_name, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain))))) __PYX_ERR(0, 730, __pyx_L7_error)
          __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_t_2);
          __pyx_t_2 = 0;

          /* "nvtx/_lib/writer.pyx":731
 *             name = _as_str(name) or None
 *             domain = self._domains.get(name)
 *             if domain is None:             # <<<<<<<<<<<<<<
 *                 domain = self._register_domain(name)
 *                 self._domains[name] = domain
*/
          __pyx_t_9 = (((PyObject *)__pyx_v_domain) == Py_None);
          if (__pyx_t_9) {

            /* "nvtx/_lib/writer.pyx":732
 *             domain = self._domains.get(name)
 *             if domain is None:
 *                 domain = self._register_domain(name)             # <<<<<<<<<<<<<<
 *                 self._domains[name] = domain
 *             return domain
*/
            __pyx_t_4 = ((PyObject *)__pyx_v_self);
            __Pyx_INCREF(__pyx_t_4);
            __pyx_t_5 = 0;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_name};
              __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_register_domain, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 732, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain))))) __PYX_ERR(0, 732, __pyx_L7_error)
            __Pyx_DECREF_SET(__pyx_v_domain, ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_t_2));
            __pyx_t_2 = 0;

            /* "nvtx/_lib/writer.pyx":733
 *             if domain is None:
 *                 domain = self._register_domain(name)
 *                 self._domains[name] = domain             # <<<<<<<<<<<<<<
 *             return domain
 * 
*/
            if (unlikely(__pyx_v_self->_domains == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
              __PYX_ERR(0, 733, __pyx_L7_error)
            }
            if (unlikely((PyDict_SetItem(__pyx_v_self->_domains, __pyx_v_name, ((PyObject *)__pyx_v_domain)) < 0))) __PYX_ERR(0, 733, __pyx_L7_error)

            /* "nvtx/_lib/writer.pyx":731
 *             name = _as_str(name) or None
 *             domain = self._domains.get(name)
 *             if domain is None:             # <<<<<<<<<<<<<<
 *                 domain = self._register_domain(name)
 *                 self._domains[name] = domain
*/
          }

          /* "nvtx/_lib/writer.pyx":734
 *                 domain = self._register_domain(name)
 *                 self._domains[name] = domain
 *             return domain             # <<<<<<<<<<<<<<
 * 
 *     def create_stream(
*/
          __Pyx_XDECREF(__pyx_r);
          __Pyx_INCREF((PyObject *)__pyx_v_domain);
          __pyx_r = ((PyObject *)__pyx_v_domain);
          goto __pyx_L11_try_return;

          /* "nvtx/_lib/writer.pyx":726
 *         """
 *         cdef Domain domain
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if self._handle == NULL:
 *                 raise RuntimeError("session is not active")
*/
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Session.get_domain", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 726, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_3);
          __pyx_t_10 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 726, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 726, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (__pyx_t_9 < (0)) __PYX_ERR(0, 726, __pyx_L9_except_error)
          __pyx_t_12 = (!__pyx_t_9);
          if (unlikely(__pyx_t_12)) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_4, __pyx_t_3);
            __pyx_t_2 = 0;  __pyx_t_4 = 0;  __pyx_t_3 = 0; 
            __PYX_ERR(0, 726, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 726, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 726, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L20;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L20:;
  }

  /* "nvtx/_lib/writer.pyx":700
 *         return Domain(self, <uintptr_t>handle)
 * 
 *     def get_domain(self, name=None):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a domain within this session.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("nvtx._lib.writer.Session.get_domain", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_domain);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":736
 *             return domain
 * 
 *     def create_stream(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_11create_stream(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Session_10create_stream, "Session.create_stream(self, name, *, domain=None, scope=None, time_domain_id=NVTX_TIME_DOMAIN_ID_NONE, interleaving=StreamInterleaving.NONE, ordering=StreamOrdering.UNKNOWN, skid=StreamSkid.NONE, skid_amount=0)\n\nCreate a stream associated with this session.\n\nThis method configures the stream but does not open it. Use the stream\nas a context manager or call :meth:`Stream.open` before writing events.\n\nParameters\n----------\nname : str or bytes\n    Stream name.\ndomain : Domain, str, bytes, optional\n    Domain object or domain name. ``None`` selects the session default\n    domain. Names and ``None`` are resolved with :meth:`get_domain`.\nscope : PredefinedScope, Scope, int, or None, optional\n    Default scope associated with the stream. Pass\n    :attr:`PredefinedScope.NONE`, :attr:`PredefinedScope.ROOT`, or a\n    dynamic scope returned by :meth:`Domain.get_scope` for this\n    stream's domain. An integer must be in the NVTX static scope ID\n    range.\ntime_domain_id : TimestampType or int, optional\n    Time domain used by event timestamps.\ninterleaving : StreamInterleaving, optional\n    Whether ordering guarantees apply across the stream or per scope.\nordering : StreamOrdering, optional\n    Ordering guarantee for events in the stream.\nskid : StreamSkid, optional\n    Unit used to express the partial-sort skid.\nskid_amount : int, optional\n    Maximum partial-sort skid in the unit selected by ``skid``.\n\nReturns\n-------\nStream\n    A configured, unopened stream.\n\nRaises\n------\nRuntimeError\n    If the session is not active or the domain is no longer valid.\nTypeError\n    If ``interleaving``, ``ordering``, ``skid``, or\n    ``time_domain_id`` has an unsupported type, or ``scope`` is not a\n    supported stream scope.\nValueError\n    If ``domain`` belongs to a different session, ``scope`` is\n    registered in a different domain, or a runtime-resolved\n    predefined scope or an integer outside the static scope ID range\n    is provided.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Session_11create_stream = {"create_stream", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_11create_stream, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_10create_stream};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_11create_stream(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_domain = 0;
  PyObject *__pyx_v_scope = 0;
  PyObject *__pyx_v_time_domain_id = 0;
  PyObject *__pyx_v_interleaving = 0;
  PyObject *__pyx_v_ordering = 0;
  PyObject *__pyx_v_skid = 0;
  PyObject *__pyx_v_skid_amount = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[8] = {0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_stream (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_domain,&__pyx_mstate_global->__pyx_n_u_scope,&__pyx_mstate_global->__pyx_n_u_time_domain_id,&__pyx_mstate_global->__pyx_n_u_interleaving,&__pyx_mstate_global->__pyx_n_u_ordering,&__pyx_mstate_global->__pyx_n_u_skid,&__pyx_mstate_global->__pyx_n_u_skid_amount,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 736, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "create_stream", 0) < (0)) __PYX_ERR(0, 736, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":740
 *         name,
 *         *,
 *         domain=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":741
 *         *,
 *         domain=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
 *         interleaving=StreamInterleaving.NONE,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__3);
      if (!values[4]) values[4] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__4);
      if (!values[5]) values[5] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__5);
      if (!values[6]) values[6] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__6);
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_int_0));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("create_stream", 1, 1, 1, i); __PYX_ERR(0, 736, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 736, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":740
 *         name,
 *         *,
 *         domain=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":741
 *         *,
 *         domain=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
 *         interleaving=StreamInterleaving.NONE,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__3);
      if (!values[4]) values[4] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__4);
      if (!values[5]) values[5] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__5);
      if (!values[6]) values[6] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__6);
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_int_0));
    }
    __pyx_v_name = values[0];
    __pyx_v_domain = values[1];
    __pyx_v_scope = values[2];
    __pyx_v_time_domain_id = values[3];
    __pyx_v_interleaving = values[4];
    __pyx_v_ordering = values[5];
    __pyx_v_skid = values[6];
    __pyx_v_skid_amount = values[7];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_stream", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 736, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Session.create_stream", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session_10create_stream(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self), __pyx_v_name, __pyx_v_domain, __pyx_v_scope, __pyx_v_time_domain_id, __pyx_v_interleaving, __pyx_v_ordering, __pyx_v_skid, __pyx_v_skid_amount);

  /* "nvtx/_lib/writer.pyx":736
 *             return domain
 * 
 *     def create_stream(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_10create_stream(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_domain, PyObject *__pyx_v_scope, PyObject *__pyx_v_time_domain_id, PyObject *__pyx_v_interleaving, PyObject *__pyx_v_ordering, PyObject *__pyx_v_skid, PyObject *__pyx_v_skid_amount) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_stream", 0);
  __Pyx_INCREF(__pyx_v_domain);

  /* "nvtx/_lib/writer.pyx":797
 *             is provided.
 *         """
 *         if not isinstance(interleaving, StreamInterleaving):             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 "interleaving must be an nvtx.writer.StreamInterleaving")
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_StreamInterleaving); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_interleaving, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (!__pyx_t_2);
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":798
 *         """
 *         if not isinstance(interleaving, StreamInterleaving):
 *             raise TypeError(             # <<<<<<<<<<<<<<
 *                 "interleaving must be an nvtx.writer.StreamInterleaving")
 *         if not isinstance(ordering, StreamOrdering):
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_interleaving_must_be_an_nvtx_wri};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 798, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 798, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":797
 *             is provided.
 *         """
 *         if not isinstance(interleaving, StreamInterleaving):             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 "interleaving must be an nvtx.writer.StreamInterleaving")
*/
  }

  /* "nvtx/_lib/writer.pyx":800
 *             raise TypeError(
 *                 "interleaving must be an nvtx.writer.StreamInterleaving")
 *         if not isinstance(ordering, StreamOrdering):             # <<<<<<<<<<<<<<
 *             raise TypeError("ordering must be an nvtx.writer.StreamOrdering")
 *         if not isinstance(skid, StreamSkid):
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_StreamOrdering); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyObject_IsInstance(__pyx_v_ordering, __pyx_t_1); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = (!__pyx_t_3);
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/writer.pyx":801
 *                 "interleaving must be an nvtx.writer.StreamInterleaving")
 *         if not isinstance(ordering, StreamOrdering):
 *             raise TypeError("ordering must be an nvtx.writer.StreamOrdering")             # <<<<<<<<<<<<<<
 *         if not isinstance(skid, StreamSkid):
 *             raise TypeError("skid must be an nvtx.writer.StreamSkid")
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_ordering_must_be_an_nvtx_writer};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 801, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 801, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":800
 *             raise TypeError(
 *                 "interleaving must be an nvtx.writer.StreamInterleaving")
 *         if not isinstance(ordering, StreamOrdering):             # <<<<<<<<<<<<<<
 *             raise TypeError("ordering must be an nvtx.writer.StreamOrdering")
 *         if not isinstance(skid, StreamSkid):
*/
  }

  /* "nvtx/_lib/writer.pyx":802
 *         if not isinstance(ordering, StreamOrdering):
 *             raise TypeError("ordering must be an nvtx.writer.StreamOrdering")
 *         if not isinstance(skid, StreamSkid):             # <<<<<<<<<<<<<<
 *             raise TypeError("skid must be an nvtx.writer.StreamSkid")
 *         if not isinstance(time_domain_id, (TimestampType, int)):
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_StreamSkid); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 802, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_skid, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 802, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (!__pyx_t_2);
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":803
 *             raise TypeError("ordering must be an nvtx.writer.StreamOrdering")
 *         if not isinstance(skid, StreamSkid):
 *             raise TypeError("skid must be an nvtx.writer.StreamSkid")             # <<<<<<<<<<<<<<
 *         if not isinstance(time_domain_id, (TimestampType, int)):
 *             raise TypeError(
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_skid_must_be_an_nvtx_writer_Stre};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 803, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 803, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":802
 *         if not isinstance(ordering, StreamOrdering):
 *             raise TypeError("ordering must be an nvtx.writer.StreamOrdering")
 *         if not isinstance(skid, StreamSkid):             # <<<<<<<<<<<<<<
 *             raise TypeError("skid must be an nvtx.writer.StreamSkid")
 *         if not isinstance(time_domain_id, (TimestampType, int)):
*/
  }

  /* "nvtx/_lib/writer.pyx":804
 *         if not isinstance(skid, StreamSkid):
 *             raise TypeError("skid must be an nvtx.writer.StreamSkid")
 *         if not isinstance(time_domain_id, (TimestampType, int)):             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 "time_domain_id must be an nvtx.TimestampType or an int")
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_TimestampType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_time_domain_id, __pyx_t_1); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = PyLong_Check(__pyx_v_time_domain_id); 
  __pyx_t_3 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = (!__pyx_t_3);
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/writer.pyx":805
 *             raise TypeError("skid must be an nvtx.writer.StreamSkid")
 *         if not isinstance(time_domain_id, (TimestampType, int)):
 *             raise TypeError(             # <<<<<<<<<<<<<<
 *                 "time_domain_id must be an nvtx.TimestampType or an int")
 *         with self._lock:
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_time_domain_id_must_be_an_nvtx_T};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 805, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 805, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":804
 *         if not isinstance(skid, StreamSkid):
 *             raise TypeError("skid must be an nvtx.writer.StreamSkid")
 *         if not isinstance(time_domain_id, (TimestampType, int)):             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 "time_domain_id must be an nvtx.TimestampType or an int")
*/
  }

  /* "nvtx/_lib/writer.pyx":807
 *             raise TypeError(
 *                 "time_domain_id must be an nvtx.TimestampType or an int")
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if self._handle == NULL:
 *                 raise RuntimeError("session is not active")
*/
  /*with:*/ {
    __pyx_t_6 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 807, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = NULL;
    __pyx_t_7 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 807, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 807, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":808
 *                 "time_domain_id must be an nvtx.TimestampType or an int")
 *         with self._lock:
 *             if self._handle == NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("session is not active")
 *             if not isinstance(domain, Domain):
*/
          __pyx_t_2 = (__pyx_v_self->_handle == NULL);
          if (unlikely(__pyx_t_2)) {

            /* "nvtx/_lib/writer.pyx":809
 *         with self._lock:
 *             if self._handle == NULL:
 *                 raise RuntimeError("session is not active")             # <<<<<<<<<<<<<<
 *             if not isinstance(domain, Domain):
 *                 domain = self.get_domain(domain)
*/
            __pyx_t_7 = NULL;
            __pyx_t_5 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_session_is_not_active};
              __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
              if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 809, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_1);
            }
            __Pyx_Raise(__pyx_t_1, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __PYX_ERR(0, 809, __pyx_L13_error)

            /* "nvtx/_lib/writer.pyx":808
 *                 "time_domain_id must be an nvtx.TimestampType or an int")
 *         with self._lock:
 *             if self._handle == NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("session is not active")
 *             if not isinstance(domain, Domain):
*/
          }

          /* "nvtx/_lib/writer.pyx":810
 *             if self._handle == NULL:
 *                 raise RuntimeError("session is not active")
 *             if not isinstance(domain, Domain):             # <<<<<<<<<<<<<<
 *                 domain = self.get_domain(domain)
 *             elif (<Domain>domain)._session is not self:
*/
          __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_domain, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain); 
          __pyx_t_3 = (!__pyx_t_2);
          if (__pyx_t_3) {

            /* "nvtx/_lib/writer.pyx":811
 *                 raise RuntimeError("session is not active")
 *             if not isinstance(domain, Domain):
 *                 domain = self.get_domain(domain)             # <<<<<<<<<<<<<<
 *             elif (<Domain>domain)._session is not self:
 *                 raise ValueError("domain belongs to a different session")
*/
            __pyx_t_7 = ((PyObject *)__pyx_v_self);
            __Pyx_INCREF(__pyx_t_7);
            __pyx_t_5 = 0;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_v_domain};
              __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_domain, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
              if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 811, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_1);
            }
            __Pyx_DECREF_SET(__pyx_v_domain, __pyx_t_1);
            __pyx_t_1 = 0;

            /* "nvtx/_lib/writer.pyx":810
 *             if self._handle == NULL:
 *                 raise RuntimeError("session is not active")
 *             if not isinstance(domain, Domain):             # <<<<<<<<<<<<<<
 *                 domain = self.get_domain(domain)
 *             elif (<Domain>domain)._session is not self:
*/
            goto __pyx_L20;
          }

          /* "nvtx/_lib/writer.pyx":812
 *             if not isinstance(domain, Domain):
 *                 domain = self.get_domain(domain)
 *             elif (<Domain>domain)._session is not self:             # <<<<<<<<<<<<<<
 *                 raise ValueError("domain belongs to a different session")
 *             else:
*/
          __pyx_t_3 = (((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_domain)->_session != __pyx_v_self);
          if (unlikely(__pyx_t_3)) {

            /* "nvtx/_lib/writer.pyx":813
 *                 domain = self.get_domain(domain)
 *             elif (<Domain>domain)._session is not self:
 *                 raise ValueError("domain belongs to a different session")             # <<<<<<<<<<<<<<
 *             else:
 *                 (<Domain>domain)._ensure_valid()
*/
            __pyx_t_7 = NULL;
            __pyx_t_5 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_domain_belongs_to_a_different_se};
              __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
              if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 813, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_1);
            }
            __Pyx_Raise(__pyx_t_1, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __PYX_ERR(0, 813, __pyx_L13_error)

            /* "nvtx/_lib/writer.pyx":812
 *             if not isinstance(domain, Domain):
 *                 domain = self.get_domain(domain)
 *             elif (<Domain>domain)._session is not self:             # <<<<<<<<<<<<<<
 *                 raise ValueError("domain belongs to a different session")
 *             else:
*/
          }

          /* "nvtx/_lib/writer.pyx":815
 *                 raise ValueError("domain belongs to a different session")
 *             else:
 *                 (<Domain>domain)._ensure_valid()             # <<<<<<<<<<<<<<
 *             return Stream(
 *                 self,
*/
          /*else*/ {
            __pyx_t_1 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_domain)->__pyx_vtab)->_ensure_valid(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_domain)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 815, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          }
          __pyx_L20:;

          /* "nvtx/_lib/writer.pyx":816
 *             else:
 *                 (<Domain>domain)._ensure_valid()
 *             return Stream(             # <<<<<<<<<<<<<<
 *                 self,
 *                 name,
*/
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_7 = NULL;

          /* "nvtx/_lib/writer.pyx":825
 *                 ordering,
 *                 skid,
 *                 skid_amount,             # <<<<<<<<<<<<<<
 *             )
 * 
*/
          __pyx_t_5 = 1;
          {
            PyObject *__pyx_callargs[10] = {__pyx_t_7, ((PyObject *)__pyx_v_self), __pyx_v_name, __pyx_v_domain, __pyx_v_scope, __pyx_v_time_domain_id, __pyx_v_interleaving, __pyx_v_ordering, __pyx_v_skid, __pyx_v_skid_amount};
            __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_callargs+__pyx_t_5, (10-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 816, __pyx_L13_error)
            __Pyx_GOTREF((PyObject *)__pyx_t_1);
          }
          __pyx_r = ((PyObject *)__pyx_t_1);
          __pyx_t_1 = 0;
          goto __pyx_L17_try_return;

          /* "nvtx/_lib/writer.pyx":807
 *             raise TypeError(
 *                 "time_domain_id must be an nvtx.TimestampType or an int")
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if self._handle == NULL:
 *                 raise RuntimeError("session is not active")
*/
        }
        __pyx_L13_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Session.create_stream", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_7, &__pyx_t_4) < 0) __PYX_ERR(0, 807, __pyx_L15_except_error)
          __Pyx_XGOTREF(__pyx_t_1);
          __Pyx_XGOTREF(__pyx_t_7);
          __Pyx_XGOTREF(__pyx_t_4);
          __pyx_t_11 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_7, __pyx_t_4); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 807, __pyx_L15_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_11, NULL);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 807, __pyx_L15_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (__pyx_t_3 < (0)) __PYX_ERR(0, 807, __pyx_L15_except_error)
          __pyx_t_2 = (!__pyx_t_3);
          if (unlikely(__pyx_t_2)) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_7);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_7, __pyx_t_4);
            __pyx_t_1 = 0;  __pyx_t_7 = 0;  __pyx_t_4 = 0; 
            __PYX_ERR(0, 807, __pyx_L15_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L14_exception_handled;
        }
        __pyx_L15_except_error:;
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
        goto __pyx_L1_error;
        __pyx_L17_try_return:;
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
        goto __pyx_L10_return;
        __pyx_L14_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_6) {
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 807, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        goto __pyx_L12;
      }
      __pyx_L10_return: {
        __pyx_t_10 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_6) {
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 807, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __pyx_r = __pyx_t_10;
        __pyx_t_10 = 0;
        goto __pyx_L0;
      }
      __pyx_L12:;
    }
    goto __pyx_L24;
    __pyx_L9_error:;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L1_error;
    __pyx_L24:;
  }

  /* "nvtx/_lib/writer.pyx":736
 *             return domain
 * 
 *     def create_stream(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("nvtx._lib.writer.Session.create_stream", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_domain);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":828
 *             )
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.begin()
 *         return self
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_13__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Session_12__enter__, "Session.__enter__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Session_13__enter__ = {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_13__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_12__enter__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_13__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__enter__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__enter__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session_12__enter__(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_12__enter__(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "nvtx/_lib/writer.pyx":829
 * 
 *     def __enter__(self):
 *         self.begin()             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_begin, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":830
 *     def __enter__(self):
 *         self.begin()
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, *_):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":828
 *             )
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.begin()
 *         return self
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.writer.Session.__enter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":832
 *         return self
 * 
 *     def __exit__(self, *_):             # <<<<<<<<<<<<<<
 *         # Backend.close() may already have ended this session.
 *         with self._backend._lock:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_15__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Session_14__exit__, "Session.__exit__(self, *_)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Session_15__exit__ = {"__exit__", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_15__exit__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_14__exit__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_15__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v__ = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__exit__", __pyx_kwds); return NULL;}
  __Pyx_INCREF(__pyx_args);
  __pyx_v__ = __pyx_args;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session_14__exit__(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self), __pyx_v__);

  /* function exit code */
  __Pyx_DECREF(__pyx_v__);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_14__exit__(struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "nvtx/_lib/writer.pyx":834
 *     def __exit__(self, *_):
 *         # Backend.close() may already have ended this session.
 *         with self._backend._lock:             # <<<<<<<<<<<<<<
 *             with self._lock:
 *                 if self._handle != NULL:
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_backend->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 834, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_backend->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 834, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 834, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":835
 *         # Backend.close() may already have ended this session.
 *         with self._backend._lock:
 *             with self._lock:             # <<<<<<<<<<<<<<
 *                 if self._handle != NULL:
 *                     self.end()
*/
          /*with:*/ {
            __pyx_t_9 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 835, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_4 = NULL;
            __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 835, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_5 = 1;
            #if CYTHON_UNPACK_METHODS
            if (likely(PyMethod_Check(__pyx_t_3))) {
              __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
              assert(__pyx_t_4);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
              __pyx_t_5 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 835, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            /*try:*/ {
              {
                __Pyx_PyThreadState_declare
                __Pyx_PyThreadState_assign
                __Pyx_ExceptionSave(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
                __Pyx_XGOTREF(__pyx_t_10);
                __Pyx_XGOTREF(__pyx_t_11);
                __Pyx_XGOTREF(__pyx_t_12);
                /*try:*/ {

                  /* "nvtx/_lib/writer.pyx":836
 *         with self._backend._lock:
 *             with self._lock:
 *                 if self._handle != NULL:             # <<<<<<<<<<<<<<
 *                     self.end()
 * 
*/
                  __pyx_t_13 = (__pyx_v_self->_handle != NULL);
                  if (__pyx_t_13) {

                    /* "nvtx/_lib/writer.pyx":837
 *             with self._lock:
 *                 if self._handle != NULL:
 *                     self.end()             # <<<<<<<<<<<<<<
 * 
 * 
*/
                    __pyx_t_3 = ((PyObject *)__pyx_v_self);
                    __Pyx_INCREF(__pyx_t_3);
                    __pyx_t_5 = 0;
                    {
                      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
                      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_end, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 837, __pyx_L17_error)
                      __Pyx_GOTREF(__pyx_t_2);
                    }
                    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                    /* "nvtx/_lib/writer.pyx":836
 *         with self._backend._lock:
 *             with self._lock:
 *                 if self._handle != NULL:             # <<<<<<<<<<<<<<
 *                     self.end()
 * 
*/
                  }

                  /* "nvtx/_lib/writer.pyx":835
 *         # Backend.close() may already have ended this session.
 *         with self._backend._lock:
 *             with self._lock:             # <<<<<<<<<<<<<<
 *                 if self._handle != NULL:
 *                     self.end()
*/
                }
                __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
                __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
                goto __pyx_L22_try_end;
                __pyx_L17_error:;
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                /*except:*/ {
                  __Pyx_AddTraceback("nvtx._lib.writer.Session.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
                  if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4) < 0) __PYX_ERR(0, 835, __pyx_L19_except_error)
                  __Pyx_XGOTREF(__pyx_t_2);
                  __Pyx_XGOTREF(__pyx_t_3);
                  __Pyx_XGOTREF(__pyx_t_4);
                  __pyx_t_14 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 835, __pyx_L19_except_error)
                  __Pyx_GOTREF(__pyx_t_14);
                  __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_14, NULL);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                  if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 835, __pyx_L19_except_error)
                  __Pyx_GOTREF(__pyx_t_15);
                  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_15);
                  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                  if (__pyx_t_13 < (0)) __PYX_ERR(0, 835, __pyx_L19_except_error)
                  __pyx_t_16 = (!__pyx_t_13);
                  if (unlikely(__pyx_t_16)) {
                    __Pyx_GIVEREF(__pyx_t_2);
                    __Pyx_GIVEREF(__pyx_t_3);
                    __Pyx_XGIVEREF(__pyx_t_4);
                    __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_3, __pyx_t_4);
                    __pyx_t_2 = 0;  __pyx_t_3 = 0;  __pyx_t_4 = 0; 
                    __PYX_ERR(0, 835, __pyx_L19_except_error)
                  }
                  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                  goto __pyx_L18_exception_handled;
                }
                __pyx_L19_except_error:;
                __Pyx_XGIVEREF(__pyx_t_10);
                __Pyx_XGIVEREF(__pyx_t_11);
                __Pyx_XGIVEREF(__pyx_t_12);
                __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
                goto __pyx_L7_error;
                __pyx_L18_exception_handled:;
                __Pyx_XGIVEREF(__pyx_t_10);
                __Pyx_XGIVEREF(__pyx_t_11);
                __Pyx_XGIVEREF(__pyx_t_12);
                __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
                __pyx_L22_try_end:;
              }
            }
            /*finally:*/ {
              /*normal exit:*/{
                if (__pyx_t_9) {
                  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_mstate_global->__pyx_tuple[0], NULL);
                  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 835, __pyx_L7_error)
                  __Pyx_GOTREF(__pyx_t_12);
                  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
                }
                goto __pyx_L16;
              }
              __pyx_L16:;
            }
            goto __pyx_L27;
            __pyx_L13_error:;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            goto __pyx_L7_error;
            __pyx_L27:;
          }

          /* "nvtx/_lib/writer.pyx":834
 *     def __exit__(self, *_):
 *         # Backend.close() may already have ended this session.
 *         with self._backend._lock:             # <<<<<<<<<<<<<<
 *             with self._lock:
 *                 if self._handle != NULL:
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Session.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_3, &__pyx_t_2) < 0) __PYX_ERR(0, 834, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_t_2);
          __pyx_t_14 = PyTuple_Pack(3, __pyx_t_4, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 834, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_14, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 834, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_16 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_16 < (0)) __PYX_ERR(0, 834, __pyx_L9_except_error)
          __pyx_t_13 = (!__pyx_t_16);
          if (unlikely(__pyx_t_13)) {
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_2);
            __Pyx_ErrRestoreWithState(__pyx_t_4, __pyx_t_3, __pyx_t_2);
            __pyx_t_4 = 0;  __pyx_t_3 = 0;  __pyx_t_2 = 0; 
            __PYX_ERR(0, 834, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 834, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L31;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L31:;
  }

  /* "nvtx/_lib/writer.pyx":832
 *         return self
 * 
 *     def __exit__(self, *_):             # <<<<<<<<<<<<<<
 *         # Backend.close() may already have ended this session.
 *         with self._backend._lock:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("nvtx._lib.writer.Session.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_17__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Session_16__reduce_cython__, "Session.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Session_17__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_17__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_16__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_17__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session_16__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_16__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__handle_cannot_be_converted, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Session.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_19__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Session_18__setstate_cython__, "Session.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Session_19__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_19__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_18__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Session_19__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Session.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Session_18__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Session_18__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__handle_cannot_be_converted, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Session.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":852
 *     """
 * 
 *     def __cinit__(self, Domain domain, string, handle_addr):             # <<<<<<<<<<<<<<
 *         self._domain = domain
 *         self._string = string
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain = 0;
  PyObject *__pyx_v_string = 0;
  PyObject *__pyx_v_handle_addr = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_domain,&__pyx_mstate_global->__pyx_n_u_string,&__pyx_mstate_global->__pyx_n_u_handle_addr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 852, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 852, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 852, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 852, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(0, 852, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, i); __PYX_ERR(0, 852, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 852, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 852, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 852, __pyx_L3_error)
    }
    __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)values[0]);
    __pyx_v_string = values[1];
    __pyx_v_handle_addr = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 852, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.RegisteredString.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, 1, "domain", 0))) __PYX_ERR(0, 852, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_16RegisteredString___cinit__(((struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)__pyx_v_self), __pyx_v_domain, __pyx_v_string, __pyx_v_handle_addr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_6writer_16RegisteredString___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, PyObject *__pyx_v_string, PyObject *__pyx_v_handle_addr) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  uintptr_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "nvtx/_lib/writer.pyx":853
 * 
 *     def __cinit__(self, Domain domain, string, handle_addr):
 *         self._domain = domain             # <<<<<<<<<<<<<<
 *         self._string = string
 *         self._handle = \
*/
  __Pyx_INCREF((PyObject *)__pyx_v_domain);
  __Pyx_GIVEREF((PyObject *)__pyx_v_domain);
  __Pyx_GOTREF(__pyx_v_self->_domain);
  __Pyx_DECREF(__pyx_v_self->_domain);
  __pyx_v_self->_domain = ((PyObject *)__pyx_v_domain);

  /* "nvtx/_lib/writer.pyx":854
 *     def __cinit__(self, Domain domain, string, handle_addr):
 *         self._domain = domain
 *         self._string = string             # <<<<<<<<<<<<<<
 *         self._handle = \
 *             <nvtxStringHandle_t><void*><uintptr_t>handle_addr
*/
  __pyx_t_1 = __pyx_v_string;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 854, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_string);
  __Pyx_DECREF(__pyx_v_self->_string);
  __pyx_v_self->_string = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":856
 *         self._string = string
 *         self._handle = \
 *             <nvtxStringHandle_t><void*><uintptr_t>handle_addr             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __pyx_t_2 = __Pyx_PyLong_As_size_t(__pyx_v_handle_addr); if (unlikely((__pyx_t_2 == ((uintptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 856, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":855
 *         self._domain = domain
 *         self._string = string
 *         self._handle = \             # <<<<<<<<<<<<<<
 *             <nvtxStringHandle_t><void*><uintptr_t>handle_addr
 * 
*/
  __pyx_v_self->_handle = ((nvtxStringHandle_t)((void *)((uintptr_t)__pyx_t_2)));

  /* "nvtx/_lib/writer.pyx":852
 *     """
 * 
 *     def __cinit__(self, Domain domain, string, handle_addr):             # <<<<<<<<<<<<<<
 *         self._domain = domain
 *         self._string = string
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.writer.RegisteredString.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":858
 *             <nvtxStringHandle_t><void*><uintptr_t>handle_addr
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return f"RegisteredString({self._string!r})"
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_3__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_2__repr__(((struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_2__repr__(struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2[3];
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "nvtx/_lib/writer.pyx":859
 * 
 *     def __repr__(self):
 *         return f"RegisteredString({self._string!r})"             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->_string), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2[0] = __pyx_mstate_global->__pyx_kp_u_RegisteredString;
  __pyx_t_2[1] = __pyx_t_1;
  __pyx_t_2[2] = __pyx_mstate_global->__pyx_kp_u_;
  __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_2, 3, 17 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1));
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":858
 *             <nvtxStringHandle_t><void*><uintptr_t>handle_addr
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return f"RegisteredString({self._string!r})"
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer.RegisteredString.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":861
 *         return f"RegisteredString({self._string!r})"
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def handle(self):
 *         """Opaque numeric handle for a registered-string payload field."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_6handle_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_6handle_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_6handle___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_6handle___get__(struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":864
 *     def handle(self):
 *         """Opaque numeric handle for a registered-string payload field."""
 *         return <uintptr_t>self._handle             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_FromSize_t(((uintptr_t)__pyx_v_self->_handle)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":861
 *         return f"RegisteredString({self._string!r})"
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def handle(self):
 *         """Opaque numeric handle for a registered-string payload field."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.writer.RegisteredString.handle.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_16RegisteredString_4__reduce_cython__, "RegisteredString.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_16RegisteredString_5__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_16RegisteredString_4__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_4__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.RegisteredString.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_16RegisteredString_6__setstate_cython__, "RegisteredString.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_16RegisteredString_7__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_16RegisteredString_6__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.RegisteredString.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_6__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_16RegisteredString_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.RegisteredString.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":874
 *     """
 * 
 *     def __cinit__(self, uint64_t scope_id, path=None, domain=None):             # <<<<<<<<<<<<<<
 *         self._scope_id = scope_id
 *         self._path = path
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_6writer_5Scope_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_6writer_5Scope_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  uint64_t __pyx_v_scope_id;
  PyObject *__pyx_v_path = 0;
  PyObject *__pyx_v_domain = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_scope_id,&__pyx_mstate_global->__pyx_n_u_path,&__pyx_mstate_global->__pyx_n_u_domain,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 874, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 874, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 874, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 874, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(0, 874, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 1, 3, i); __PYX_ERR(0, 874, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 874, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 874, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 874, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_scope_id = __Pyx_PyLong_As_uint64_t(values[0]); if (unlikely((__pyx_v_scope_id == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 874, __pyx_L3_error)
    __pyx_v_path = values[1];
    __pyx_v_domain = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 1, 3, __pyx_nargs); __PYX_ERR(0, 874, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Scope.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_5Scope___cinit__(((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_self), __pyx_v_scope_id, __pyx_v_path, __pyx_v_domain);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_6writer_5Scope___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self, uint64_t __pyx_v_scope_id, PyObject *__pyx_v_path, PyObject *__pyx_v_domain) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "nvtx/_lib/writer.pyx":875
 * 
 *     def __cinit__(self, uint64_t scope_id, path=None, domain=None):
 *         self._scope_id = scope_id             # <<<<<<<<<<<<<<
 *         self._path = path
 *         self._domain = domain
*/
  __pyx_v_self->_scope_id = __pyx_v_scope_id;

  /* "nvtx/_lib/writer.pyx":876
 *     def __cinit__(self, uint64_t scope_id, path=None, domain=None):
 *         self._scope_id = scope_id
 *         self._path = path             # <<<<<<<<<<<<<<
 *         self._domain = domain
 * 
*/
  __Pyx_INCREF(__pyx_v_path);
  __Pyx_GIVEREF(__pyx_v_path);
  __Pyx_GOTREF(__pyx_v_self->_path);
  __Pyx_DECREF(__pyx_v_self->_path);
  __pyx_v_self->_path = __pyx_v_path;

  /* "nvtx/_lib/writer.pyx":877
 *         self._scope_id = scope_id
 *         self._path = path
 *         self._domain = domain             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __Pyx_INCREF(__pyx_v_domain);
  __Pyx_GIVEREF(__pyx_v_domain);
  __Pyx_GOTREF(__pyx_v_self->_domain);
  __Pyx_DECREF(__pyx_v_self->_domain);
  __pyx_v_self->_domain = __pyx_v_domain;

  /* "nvtx/_lib/writer.pyx":874
 *     """
 * 
 *     def __cinit__(self, uint64_t scope_id, path=None, domain=None):             # <<<<<<<<<<<<<<
 *         self._scope_id = scope_id
 *         self._path = path
*/

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":879
 *         self._domain = domain
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return f"Scope(scope_id={self._scope_id}, path={self._path!r})"
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_3__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_5Scope_2__repr__(((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_2__repr__(struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3[5];
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "nvtx/_lib/writer.pyx":880
 * 
 *     def __repr__(self):
 *         return f"Scope(scope_id={self._scope_id}, path={self._path!r})"             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyUnicode_From_uint64_t(__pyx_v_self->_scope_id, 0, ' ', 'd'); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->_path), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3[0] = __pyx_mstate_global->__pyx_kp_u_Scope_scope_id;
  __pyx_t_3[1] = __pyx_t_1;
  __pyx_t_3[2] = __pyx_mstate_global->__pyx_kp_u_path_2;
  __pyx_t_3[3] = __pyx_t_2;
  __pyx_t_3[4] = __pyx_mstate_global->__pyx_kp_u_;
  __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_3, 5, 15 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 7 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2));
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":879
 *         self._domain = domain
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return f"Scope(scope_id={self._scope_id}, path={self._path!r})"
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.writer.Scope.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":882
 *         return f"Scope(scope_id={self._scope_id}, path={self._path!r})"
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def scope_id(self):
 *         """Registered scope ID within the domain."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_8scope_id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_8scope_id_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_5Scope_8scope_id___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_8scope_id___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":885
 *     def scope_id(self):
 *         """Registered scope ID within the domain."""
 *         return self._scope_id             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_uint64_t(__pyx_v_self->_scope_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 885, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":882
 *         return f"Scope(scope_id={self._scope_id}, path={self._path!r})"
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def scope_id(self):
 *         """Registered scope ID within the domain."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.writer.Scope.scope_id.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":887
 *         return self._scope_id
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def path(self):
 *         """The path the scope was registered with (``None`` if unnamed)."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_4path_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_4path_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_5Scope_4path___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_4path___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":890
 *     def path(self):
 *         """The path the scope was registered with (``None`` if unnamed)."""
 *         return self._path             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_path);
  __pyx_r = __pyx_v_self->_path;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":887
 *         return self._scope_id
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def path(self):
 *         """The path the scope was registered with (``None`` if unnamed)."""
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_5Scope_4__reduce_cython__, "Scope.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_5Scope_5__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_5Scope_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_5Scope_4__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_5Scope_4__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Scope.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_5Scope_6__setstate_cython__, "Scope.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_5Scope_7__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_5Scope_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_5Scope_6__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_5Scope_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Scope.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_5Scope_6__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_5Scope_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Scope *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Scope.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":900
 *     """
 * 
 *     def __cinit__(             # <<<<<<<<<<<<<<
 *         self,
 *         Domain domain,
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_6writer_7Counter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_6writer_7Counter_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_v_description = 0;
  PyObject *__pyx_v_semantics = 0;
  uint64_t __pyx_v_scope_id;
  uint64_t __pyx_v_counter_id;
  uint64_t __pyx_v_schema_id;
  PyObject *__pyx_v_timestamp_field = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[9] = {0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_domain,&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_dtype,&__pyx_mstate_global->__pyx_n_u_description,&__pyx_mstate_global->__pyx_n_u_semantics,&__pyx_mstate_global->__pyx_n_u_scope_id,&__pyx_mstate_global->__pyx_n_u_counter_id,&__pyx_mstate_global->__pyx_n_u_schema_id,&__pyx_mstate_global->__pyx_n_u_timestamp_field,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 900, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  9:
        values[8] = __Pyx_ArgRef_VARARGS(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 900, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_VARARGS(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 900, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 900, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 900, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 900, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 900, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 900, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 900, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 900, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(0, 900, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 9; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 9, 9, i); __PYX_ERR(0, 900, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 9)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 900, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 900, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 900, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 900, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 900, __pyx_L3_error)
      values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 900, __pyx_L3_error)
      values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 900, __pyx_L3_error)
      values[7] = __Pyx_ArgRef_VARARGS(__pyx_args, 7);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 900, __pyx_L3_error)
      values[8] = __Pyx_ArgRef_VARARGS(__pyx_args, 8);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 900, __pyx_L3_error)
    }
    __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)values[0]);
    __pyx_v_name = values[1];
    __pyx_v_dtype = values[2];
    __pyx_v_description = values[3];
    __pyx_v_semantics = values[4];
    __pyx_v_scope_id = __Pyx_PyLong_As_uint64_t(values[5]); if (unlikely((__pyx_v_scope_id == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 907, __pyx_L3_error)
    __pyx_v_counter_id = __Pyx_PyLong_As_uint64_t(values[6]); if (unlikely((__pyx_v_counter_id == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 908, __pyx_L3_error)
    __pyx_v_schema_id = __Pyx_PyLong_As_uint64_t(values[7]); if (unlikely((__pyx_v_schema_id == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 909, __pyx_L3_error)
    __pyx_v_timestamp_field = values[8];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 9, 9, __pyx_nargs); __PYX_ERR(0, 900, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Counter.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, 1, "domain", 0))) __PYX_ERR(0, 902, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Counter___cinit__(((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)__pyx_v_self), __pyx_v_domain, __pyx_v_name, __pyx_v_dtype, __pyx_v_description, __pyx_v_semantics, __pyx_v_scope_id, __pyx_v_counter_id, __pyx_v_schema_id, __pyx_v_timestamp_field);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_6writer_7Counter___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, PyObject *__pyx_v_description, PyObject *__pyx_v_semantics, uint64_t __pyx_v_scope_id, uint64_t __pyx_v_counter_id, uint64_t __pyx_v_schema_id, PyObject *__pyx_v_timestamp_field) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "nvtx/_lib/writer.pyx":912
 *         timestamp_field,
 *     ):
 *         self._domain = domain             # <<<<<<<<<<<<<<
 *         self._name = name
 *         self._dtype = dtype
*/
  __Pyx_INCREF((PyObject *)__pyx_v_domain);
  __Pyx_GIVEREF((PyObject *)__pyx_v_domain);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_domain);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_domain);
  __pyx_v_self->_domain = __pyx_v_domain;

  /* "nvtx/_lib/writer.pyx":913
 *     ):
 *         self._domain = domain
 *         self._name = name             # <<<<<<<<<<<<<<
 *         self._dtype = dtype
 *         self._description = description
*/
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->_name);
  __Pyx_DECREF(__pyx_v_self->_name);
  __pyx_v_self->_name = __pyx_v_name;

  /* "nvtx/_lib/writer.pyx":914
 *         self._domain = domain
 *         self._name = name
 *         self._dtype = dtype             # <<<<<<<<<<<<<<
 *         self._description = description
 *         self._semantics = semantics
*/
  __Pyx_INCREF(__pyx_v_dtype);
  __Pyx_GIVEREF(__pyx_v_dtype);
  __Pyx_GOTREF(__pyx_v_self->_dtype);
  __Pyx_DECREF(__pyx_v_self->_dtype);
  __pyx_v_self->_dtype = __pyx_v_dtype;

  /* "nvtx/_lib/writer.pyx":915
 *         self._name = name
 *         self._dtype = dtype
 *         self._description = description             # <<<<<<<<<<<<<<
 *         self._semantics = semantics
 *         self._scope_id = scope_id
*/
  __Pyx_INCREF(__pyx_v_description);
  __Pyx_GIVEREF(__pyx_v_description);
  __Pyx_GOTREF(__pyx_v_self->_description);
  __Pyx_DECREF(__pyx_v_self->_description);
  __pyx_v_self->_description = __pyx_v_description;

  /* "nvtx/_lib/writer.pyx":916
 *         self._dtype = dtype
 *         self._description = description
 *         self._semantics = semantics             # <<<<<<<<<<<<<<
 *         self._scope_id = scope_id
 *         self._counter_id = counter_id
*/
  __Pyx_INCREF(__pyx_v_semantics);
  __Pyx_GIVEREF(__pyx_v_semantics);
  __Pyx_GOTREF(__pyx_v_self->_semantics);
  __Pyx_DECREF(__pyx_v_self->_semantics);
  __pyx_v_self->_semantics = __pyx_v_semantics;

  /* "nvtx/_lib/writer.pyx":917
 *         self._description = description
 *         self._semantics = semantics
 *         self._scope_id = scope_id             # <<<<<<<<<<<<<<
 *         self._counter_id = counter_id
 *         self._schema_id = schema_id
*/
  __pyx_v_self->_scope_id = __pyx_v_scope_id;

  /* "nvtx/_lib/writer.pyx":918
 *         self._semantics = semantics
 *         self._scope_id = scope_id
 *         self._counter_id = counter_id             # <<<<<<<<<<<<<<
 *         self._schema_id = schema_id
 *         self._timestamp_field = timestamp_field
*/
  __pyx_v_self->_counter_id = __pyx_v_counter_id;

  /* "nvtx/_lib/writer.pyx":919
 *         self._scope_id = scope_id
 *         self._counter_id = counter_id
 *         self._schema_id = schema_id             # <<<<<<<<<<<<<<
 *         self._timestamp_field = timestamp_field
 * 
*/
  __pyx_v_self->_schema_id = __pyx_v_schema_id;

  /* "nvtx/_lib/writer.pyx":920
 *         self._counter_id = counter_id
 *         self._schema_id = schema_id
 *         self._timestamp_field = timestamp_field             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __Pyx_INCREF(__pyx_v_timestamp_field);
  __Pyx_GIVEREF(__pyx_v_timestamp_field);
  __Pyx_GOTREF(__pyx_v_self->_timestamp_field);
  __Pyx_DECREF(__pyx_v_self->_timestamp_field);
  __pyx_v_self->_timestamp_field = __pyx_v_timestamp_field;

  /* "nvtx/_lib/writer.pyx":900
 *     """
 * 
 *     def __cinit__(             # <<<<<<<<<<<<<<
 *         self,
 *         Domain domain,
*/

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":922
 *         self._timestamp_field = timestamp_field
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return f"Counter(name={self._name!r}, counter_id={self._counter_id})"
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_3__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Counter_2__repr__(((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_2__repr__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3[5];
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "nvtx/_lib/writer.pyx":923
 * 
 *     def __repr__(self):
 *         return f"Counter(name={self._name!r}, counter_id={self._counter_id})"             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 923, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyUnicode_From_uint64_t(__pyx_v_self->_counter_id, 0, ' ', 'd'); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 923, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3[0] = __pyx_mstate_global->__pyx_kp_u_Counter_name;
  __pyx_t_3[1] = __pyx_t_1;
  __pyx_t_3[2] = __pyx_mstate_global->__pyx_kp_u_counter_id_2;
  __pyx_t_3[3] = __pyx_t_2;
  __pyx_t_3[4] = __pyx_mstate_global->__pyx_kp_u_;
  __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_3, 5, 13 * 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1));
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 923, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":922
 *         self._timestamp_field = timestamp_field
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return f"Counter(name={self._name!r}, counter_id={self._counter_id})"
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.writer.Counter.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":925
 *         return f"Counter(name={self._name!r}, counter_id={self._counter_id})"
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def counter_id(self):
 *         """Registered counter ID within the domain."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_10counter_id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_10counter_id_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Counter_10counter_id___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_10counter_id___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":928
 *     def counter_id(self):
 *         """Registered counter ID within the domain."""
 *         return self._counter_id             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_uint64_t(__pyx_v_self->_counter_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 928, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":925
 *         return f"Counter(name={self._name!r}, counter_id={self._counter_id})"
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def counter_id(self):
 *         """Registered counter ID within the domain."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.writer.Counter.counter_id.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":930
 *         return self._counter_id
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def name(self):
 *         """The counter's display name."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_4name_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Counter_4name___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_4name___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":933
 *     def name(self):
 *         """The counter's display name."""
 *         return self._name             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_name);
  __pyx_r = __pyx_v_self->_name;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":930
 *         return self._counter_id
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def name(self):
 *         """The counter's display name."""
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":935
 *         return self._name
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def dtype(self):
 *         """The normalized dtype the counter was registered with."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_5dtype_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_5dtype_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Counter_5dtype___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_5dtype___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":938
 *     def dtype(self):
 *         """The normalized dtype the counter was registered with."""
 *         return self._dtype             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_dtype);
  __pyx_r = __pyx_v_self->_dtype;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":935
 *         return self._name
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def dtype(self):
 *         """The normalized dtype the counter was registered with."""
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":940
 *         return self._dtype
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def description(self):
 *         """The counter's description, or ``None``."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_11description_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_11description_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Counter_11description___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_11description___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":943
 *     def description(self):
 *         """The counter's description, or ``None``."""
 *         return self._description             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_description);
  __pyx_r = __pyx_v_self->_description;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":940
 *         return self._dtype
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def description(self):
 *         """The counter's description, or ``None``."""
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":945
 *         return self._description
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def semantics(self):
 *         """The counter's :class:`nvtx.CounterSemantics`, or ``None``."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_9semantics_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_9semantics_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Counter_9semantics___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_9semantics___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":948
 *     def semantics(self):
 *         """The counter's :class:`nvtx.CounterSemantics`, or ``None``."""
 *         return self._semantics             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_semantics);
  __pyx_r = __pyx_v_self->_semantics;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":945
 *         return self._description
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def semantics(self):
 *         """The counter's :class:`nvtx.CounterSemantics`, or ``None``."""
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Counter_4__reduce_cython__, "Counter.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Counter_5__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Counter_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Counter_4__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Counter_4__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Counter.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_7Counter_6__setstate_cython__, "Counter.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_7Counter_7__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Counter_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Counter_6__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_7Counter_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Counter.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_7Counter_6__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_7Counter_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Counter.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":959
 *     """
 * 
 *     def __cinit__(self, Domain domain, uint64_t schema_id, dtype, kind):             # <<<<<<<<<<<<<<
 *         self._domain = domain
 *         self._schema_id = schema_id
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_6writer_6Schema_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_6writer_6Schema_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain = 0;
  uint64_t __pyx_v_schema_id;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_v_kind = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_domain,&__pyx_mstate_global->__pyx_n_u_schema_id,&__pyx_mstate_global->__pyx_n_u_dtype,&__pyx_mstate_global->__pyx_n_u_kind,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 959, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 959, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 959, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 959, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 959, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(0, 959, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, i); __PYX_ERR(0, 959, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 4)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 959, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 959, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 959, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 959, __pyx_L3_error)
    }
    __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)values[0]);
    __pyx_v_schema_id = __Pyx_PyLong_As_uint64_t(values[1]); if (unlikely((__pyx_v_schema_id == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 959, __pyx_L3_error)
    __pyx_v_dtype = values[2];
    __pyx_v_kind = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 4, 4, __pyx_nargs); __PYX_ERR(0, 959, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Schema.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, 1, "domain", 0))) __PYX_ERR(0, 959, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Schema___cinit__(((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)__pyx_v_self), __pyx_v_domain, __pyx_v_schema_id, __pyx_v_dtype, __pyx_v_kind);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_6writer_6Schema___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, uint64_t __pyx_v_schema_id, PyObject *__pyx_v_dtype, PyObject *__pyx_v_kind) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "nvtx/_lib/writer.pyx":960
 * 
 *     def __cinit__(self, Domain domain, uint64_t schema_id, dtype, kind):
 *         self._domain = domain             # <<<<<<<<<<<<<<
 *         self._schema_id = schema_id
 *         self._dtype = dtype
*/
  __Pyx_INCREF((PyObject *)__pyx_v_domain);
  __Pyx_GIVEREF((PyObject *)__pyx_v_domain);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_domain);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_domain);
  __pyx_v_self->_domain = __pyx_v_domain;

  /* "nvtx/_lib/writer.pyx":961
 *     def __cinit__(self, Domain domain, uint64_t schema_id, dtype, kind):
 *         self._domain = domain
 *         self._schema_id = schema_id             # <<<<<<<<<<<<<<
 *         self._dtype = dtype
 *         self._kind = kind
*/
  __pyx_v_self->_schema_id = __pyx_v_schema_id;

  /* "nvtx/_lib/writer.pyx":962
 *         self._domain = domain
 *         self._schema_id = schema_id
 *         self._dtype = dtype             # <<<<<<<<<<<<<<
 *         self._kind = kind
 * 
*/
  __Pyx_INCREF(__pyx_v_dtype);
  __Pyx_GIVEREF(__pyx_v_dtype);
  __Pyx_GOTREF(__pyx_v_self->_dtype);
  __Pyx_DECREF(__pyx_v_self->_dtype);
  __pyx_v_self->_dtype = __pyx_v_dtype;

  /* "nvtx/_lib/writer.pyx":963
 *         self._schema_id = schema_id
 *         self._dtype = dtype
 *         self._kind = kind             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __Pyx_INCREF(__pyx_v_kind);
  __Pyx_GIVEREF(__pyx_v_kind);
  __Pyx_GOTREF(__pyx_v_self->_kind);
  __Pyx_DECREF(__pyx_v_self->_kind);
  __pyx_v_self->_kind = __pyx_v_kind;

  /* "nvtx/_lib/writer.pyx":959
 *     """
 * 
 *     def __cinit__(self, Domain domain, uint64_t schema_id, dtype, kind):             # <<<<<<<<<<<<<<
 *         self._domain = domain
 *         self._schema_id = schema_id
*/

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":965
 *         self._kind = kind
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return (
 *             f"Schema(schema_id={self._schema_id}, dtype={self._dtype!r}, "
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_3__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Schema_2__repr__(((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_2__repr__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4[7];
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "nvtx/_lib/writer.pyx":966
 * 
 *     def __repr__(self):
 *         return (             # <<<<<<<<<<<<<<
 *             f"Schema(schema_id={self._schema_id}, dtype={self._dtype!r}, "
 *             f"kind={self._kind})"
*/
  __Pyx_XDECREF(__pyx_r);

  /* "nvtx/_lib/writer.pyx":967
 *     def __repr__(self):
 *         return (
 *             f"Schema(schema_id={self._schema_id}, dtype={self._dtype!r}, "             # <<<<<<<<<<<<<<
 *             f"kind={self._kind})"
 *         )
*/
  __pyx_t_1 = __Pyx_PyUnicode_From_uint64_t(__pyx_v_self->_schema_id, 0, ' ', 'd'); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 967, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->_dtype), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 967, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "nvtx/_lib/writer.pyx":968
 *         return (
 *             f"Schema(schema_id={self._schema_id}, dtype={self._dtype!r}, "
 *             f"kind={self._kind})"             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_self->_kind, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4[0] = __pyx_mstate_global->__pyx_kp_u_Schema_schema_id;
  __pyx_t_4[1] = __pyx_t_1;
  __pyx_t_4[2] = __pyx_mstate_global->__pyx_kp_u_dtype_2;
  __pyx_t_4[3] = __pyx_t_2;
  __pyx_t_4[4] = __pyx_mstate_global->__pyx_kp_u_kind_2;
  __pyx_t_4[5] = __pyx_t_3;
  __pyx_t_4[6] = __pyx_mstate_global->__pyx_kp_u_;

  /* "nvtx/_lib/writer.pyx":967
 *     def __repr__(self):
 *         return (
 *             f"Schema(schema_id={self._schema_id}, dtype={self._dtype!r}, "             # <<<<<<<<<<<<<<
 *             f"kind={self._kind})"
 *         )
*/
  __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_4, 7, 17 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 8 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 7 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3));
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 967, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":965
 *         self._kind = kind
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return (
 *             f"Schema(schema_id={self._schema_id}, dtype={self._dtype!r}, "
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.writer.Schema.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":971
 *         )
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def schema_id(self):
 *         """Registered schema ID within the domain."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_9schema_id_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_9schema_id_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Schema_9schema_id___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_9schema_id___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":974
 *     def schema_id(self):
 *         """Registered schema ID within the domain."""
 *         return self._schema_id             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_uint64_t(__pyx_v_self->_schema_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":971
 *         )
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def schema_id(self):
 *         """Registered schema ID within the domain."""
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.writer.Schema.schema_id.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":976
 *         return self._schema_id
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def dtype(self):
 *         """The NumPy dtype the schema was registered with."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_5dtype_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_5dtype_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Schema_5dtype___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_5dtype___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":979
 *     def dtype(self):
 *         """The NumPy dtype the schema was registered with."""
 *         return self._dtype             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_dtype);
  __pyx_r = __pyx_v_self->_dtype;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":976
 *         return self._schema_id
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def dtype(self):
 *         """The NumPy dtype the schema was registered with."""
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":981
 *         return self._dtype
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def kind(self):
 *         """The schema's :class:`EventKind`, or ``None`` if generic."""
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_4kind_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_4kind_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Schema_4kind___get__(((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_4kind___get__(struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/writer.pyx":984
 *     def kind(self):
 *         """The schema's :class:`EventKind`, or ``None`` if generic."""
 *         return self._kind             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_kind);
  __pyx_r = __pyx_v_self->_kind;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":981
 *         return self._dtype
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def kind(self):
 *         """The schema's :class:`EventKind`, or ``None`` if generic."""
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Schema_4__reduce_cython__, "Schema.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Schema_5__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Schema_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Schema_4__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_5__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Schema_4__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Schema.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Schema_6__setstate_cython__, "Schema.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Schema_7__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Schema_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Schema_6__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Schema_7__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Schema.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Schema_6__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Schema_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Schema.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":993
 *     """
 * 
 *     def __cinit__(self, Session session, handle_addr):             # <<<<<<<<<<<<<<
 *         self._session = session
 *         self._backend = session._backend
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_6writer_6Domain_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_6writer_6Domain_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_session = 0;
  PyObject *__pyx_v_handle_addr = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_session,&__pyx_mstate_global->__pyx_n_u_handle_addr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 993, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 993, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 993, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(0, 993, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, i); __PYX_ERR(0, 993, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 993, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 993, __pyx_L3_error)
    }
    __pyx_v_session = ((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)values[0]);
    __pyx_v_handle_addr = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 993, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_session), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Session, 1, "session", 0))) __PYX_ERR(0, 993, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain___cinit__(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_session, __pyx_v_handle_addr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_6writer_6Domain___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_session, PyObject *__pyx_v_handle_addr) {
  struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *__pyx_v_registrar = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  uintptr_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "nvtx/_lib/writer.pyx":994
 * 
 *     def __cinit__(self, Session session, handle_addr):
 *         self._session = session             # <<<<<<<<<<<<<<
 *         self._backend = session._backend
 *         self._valid = True
*/
  __Pyx_INCREF((PyObject *)__pyx_v_session);
  __Pyx_GIVEREF((PyObject *)__pyx_v_session);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_session);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_session);
  __pyx_v_self->_session = __pyx_v_session;

  /* "nvtx/_lib/writer.pyx":995
 *     def __cinit__(self, Session session, handle_addr):
 *         self._session = session
 *         self._backend = session._backend             # <<<<<<<<<<<<<<
 *         self._valid = True
 *         self._handle = <nvtxDomainHandle_t><void*><uintptr_t>handle_addr
*/
  __pyx_t_1 = ((PyObject *)__pyx_v_session->_backend);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_backend);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_backend);
  __pyx_v_self->_backend = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":996
 *         self._session = session
 *         self._backend = session._backend
 *         self._valid = True             # <<<<<<<<<<<<<<
 *         self._handle = <nvtxDomainHandle_t><void*><uintptr_t>handle_addr
 *         self._get_string_cached = functools.cache(self._register_string)
*/
  __pyx_v_self->_valid = 1;

  /* "nvtx/_lib/writer.pyx":997
 *         self._backend = session._backend
 *         self._valid = True
 *         self._handle = <nvtxDomainHandle_t><void*><uintptr_t>handle_addr             # <<<<<<<<<<<<<<
 *         self._get_string_cached = functools.cache(self._register_string)
 *         self._get_scope_cached = functools.cache(self._register_scope)
*/
  __pyx_t_2 = __Pyx_PyLong_As_size_t(__pyx_v_handle_addr); if (unlikely((__pyx_t_2 == ((uintptr_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 997, __pyx_L1_error)
  __pyx_v_self->_handle = ((nvtxDomainHandle_t)((void *)((uintptr_t)__pyx_t_2)));

  /* "nvtx/_lib/writer.pyx":998
 *         self._valid = True
 *         self._handle = <nvtxDomainHandle_t><void*><uintptr_t>handle_addr
 *         self._get_string_cached = functools.cache(self._register_string)             # <<<<<<<<<<<<<<
 *         self._get_scope_cached = functools.cache(self._register_scope)
 *         self._categories = {}
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_functools); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 998, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cache); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 998, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_register_string); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 998, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 998, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_get_string_cached);
  __Pyx_DECREF(__pyx_v_self->_get_string_cached);
  __pyx_v_self->_get_string_cached = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":999
 *         self._handle = <nvtxDomainHandle_t><void*><uintptr_t>handle_addr
 *         self._get_string_cached = functools.cache(self._register_string)
 *         self._get_scope_cached = functools.cache(self._register_scope)             # <<<<<<<<<<<<<<
 *         self._categories = {}
 *         self._schemas = {}
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_functools); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cache); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_register_scope); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 999, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_get_scope_cached);
  __Pyx_DECREF(__pyx_v_self->_get_scope_cached);
  __pyx_v_self->_get_scope_cached = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1000
 *         self._get_string_cached = functools.cache(self._register_string)
 *         self._get_scope_cached = functools.cache(self._register_scope)
 *         self._categories = {}             # <<<<<<<<<<<<<<
 *         self._schemas = {}
 *         self._get_counter_cached = functools.cache(self._register_counter)
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1000, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_categories);
  __Pyx_DECREF(__pyx_v_self->_categories);
  __pyx_v_self->_categories = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1001
 *         self._get_scope_cached = functools.cache(self._register_scope)
 *         self._categories = {}
 *         self._schemas = {}             # <<<<<<<<<<<<<<
 *         self._get_counter_cached = functools.cache(self._register_counter)
 *         # 0 is reserved for "no category", so IDs start at 1.
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1001, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_schemas);
  __Pyx_DECREF(__pyx_v_self->_schemas);
  __pyx_v_self->_schemas = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1002
 *         self._categories = {}
 *         self._schemas = {}
 *         self._get_counter_cached = functools.cache(self._register_counter)             # <<<<<<<<<<<<<<
 *         # 0 is reserved for "no category", so IDs start at 1.
 *         self._category_ids = itertools.count(1)
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_functools); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1002, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_cache); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1002, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_register_counter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1002, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1002, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_get_counter_cached);
  __Pyx_DECREF(__pyx_v_self->_get_counter_cached);
  __pyx_v_self->_get_counter_cached = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1004
 *         self._get_counter_cached = functools.cache(self._register_counter)
 *         # 0 is reserved for "no category", so IDs start at 1.
 *         self._category_ids = itertools.count(1)             # <<<<<<<<<<<<<<
 *         self._user_category_ids = set()
 *         cdef _WriterSchemaRegistrar registrar = _WriterSchemaRegistrar()
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_itertools); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1004, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1004, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_int_1};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1004, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_category_ids);
  __Pyx_DECREF(__pyx_v_self->_category_ids);
  __pyx_v_self->_category_ids = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1005
 *         # 0 is reserved for "no category", so IDs start at 1.
 *         self._category_ids = itertools.count(1)
 *         self._user_category_ids = set()             # <<<<<<<<<<<<<<
 *         cdef _WriterSchemaRegistrar registrar = _WriterSchemaRegistrar()
 *         registrar._domain = self
*/
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1005, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_user_category_ids);
  __Pyx_DECREF(__pyx_v_self->_user_category_ids);
  __pyx_v_self->_user_category_ids = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1006
 *         self._category_ids = itertools.count(1)
 *         self._user_category_ids = set()
 *         cdef _WriterSchemaRegistrar registrar = _WriterSchemaRegistrar()             # <<<<<<<<<<<<<<
 *         registrar._domain = self
 *         self._schema_registrar = registrar
*/
  __pyx_t_3 = NULL;
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1006, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_registrar = ((struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1007
 *         self._user_category_ids = set()
 *         cdef _WriterSchemaRegistrar registrar = _WriterSchemaRegistrar()
 *         registrar._domain = self             # <<<<<<<<<<<<<<
 *         self._schema_registrar = registrar
 * 
*/
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self);
  __Pyx_GOTREF((PyObject *)__pyx_v_registrar->_domain);
  __Pyx_DECREF((PyObject *)__pyx_v_registrar->_domain);
  __pyx_v_registrar->_domain = __pyx_v_self;

  /* "nvtx/_lib/writer.pyx":1008
 *         cdef _WriterSchemaRegistrar registrar = _WriterSchemaRegistrar()
 *         registrar._domain = self
 *         self._schema_registrar = registrar             # <<<<<<<<<<<<<<
 * 
 *     cdef _invalidate(self):
*/
  __Pyx_INCREF((PyObject *)__pyx_v_registrar);
  __Pyx_GIVEREF((PyObject *)__pyx_v_registrar);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_schema_registrar);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_schema_registrar);
  __pyx_v_self->_schema_registrar = ((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_registrar);

  /* "nvtx/_lib/writer.pyx":993
 *     """
 * 
 *     def __cinit__(self, Session session, handle_addr):             # <<<<<<<<<<<<<<
 *         self._session = session
 *         self._backend = session._backend
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_registrar);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1010
 *         self._schema_registrar = registrar
 * 
 *     cdef _invalidate(self):             # <<<<<<<<<<<<<<
 *         self._valid = False
 * 
*/

static PyObject *__pyx_f_4nvtx_4_lib_6writer_6Domain__invalidate(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_invalidate", 0);

  /* "nvtx/_lib/writer.pyx":1011
 * 
 *     cdef _invalidate(self):
 *         self._valid = False             # <<<<<<<<<<<<<<
 * 
 *     cdef _ensure_valid(self):
*/
  __pyx_v_self->_valid = 0;

  /* "nvtx/_lib/writer.pyx":1010
 *         self._schema_registrar = registrar
 * 
 *     cdef _invalidate(self):             # <<<<<<<<<<<<<<
 *         self._valid = False
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1013
 *         self._valid = False
 * 
 *     cdef _ensure_valid(self):             # <<<<<<<<<<<<<<
 *         if not self._valid:
 *             raise RuntimeError(
*/

static PyObject *__pyx_f_4nvtx_4_lib_6writer_6Domain__ensure_valid(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_ensure_valid", 0);

  /* "nvtx/_lib/writer.pyx":1014
 * 
 *     cdef _ensure_valid(self):
 *         if not self._valid:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 "domain is no longer valid: its owning session has ended")
*/
  __pyx_t_1 = (!__pyx_v_self->_valid);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1015
 *     cdef _ensure_valid(self):
 *         if not self._valid:
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 "domain is no longer valid: its owning session has ended")
 * 
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_domain_is_no_longer_valid_its_ow};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1015, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1015, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1014
 * 
 *     cdef _ensure_valid(self):
 *         if not self._valid:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 "domain is no longer valid: its owning session has ended")
*/
  }

  /* "nvtx/_lib/writer.pyx":1013
 *         self._valid = False
 * 
 *     cdef _ensure_valid(self):             # <<<<<<<<<<<<<<
 *         if not self._valid:
 *             raise RuntimeError(
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._ensure_valid", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1018
 *                 "domain is no longer valid: its owning session has ended")
 * 
 *     cdef _get_event_schema_ids(             # <<<<<<<<<<<<<<
 *         self, nvtxwEventHelperSchemaIds_t* schema_ids_out
 *     ):
*/

static PyObject *__pyx_f_4nvtx_4_lib_6writer_6Domain__get_event_schema_ids(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, nvtxwEventHelperSchemaIds_t *__pyx_v_schema_ids_out) {
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  nvtxwEventHelperSchemaIds_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_event_schema_ids", 0);

  /* "nvtx/_lib/writer.pyx":1022
 *     ):
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             if not self._event_schemas_registered:
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1022, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1022, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1022, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":1023
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:
 *             self._ensure_valid()             # <<<<<<<<<<<<<<
 *             if not self._event_schemas_registered:
 *                 rc = nvtxwEventSchemasRegister(
*/
          __pyx_t_2 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_self->__pyx_vtab)->_ensure_valid(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1023, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "nvtx/_lib/writer.pyx":1024
 *         with self._session._lock:
 *             self._ensure_valid()
 *             if not self._event_schemas_registered:             # <<<<<<<<<<<<<<
 *                 rc = nvtxwEventSchemasRegister(
 *                     self._backend._iface,
*/
          __pyx_t_9 = (!__pyx_v_self->_event_schemas_registered);
          if (__pyx_t_9) {

            /* "nvtx/_lib/writer.pyx":1025
 *             self._ensure_valid()
 *             if not self._event_schemas_registered:
 *                 rc = nvtxwEventSchemasRegister(             # <<<<<<<<<<<<<<
 *                     self._backend._iface,
 *                     self._handle,
*/
            __pyx_v_rc = nvtxwEventSchemasRegister(__pyx_v_self->_backend->_iface, __pyx_v_self->_handle, NVTXW_EVENT_HELPER_SCHEMA_ALL, (&__pyx_v_self->_event_schema_ids));

            /* "nvtx/_lib/writer.pyx":1031
 *                     &self._event_schema_ids,
 *                 )
 *                 _check(rc, "nvtxwEventSchemasRegister")             # <<<<<<<<<<<<<<
 *                 self._event_schemas_registered = True
 *             schema_ids_out[0] = self._event_schema_ids
*/
            __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_nvtxwEventSchemasRegister); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1031, __pyx_L7_error)

            /* "nvtx/_lib/writer.pyx":1032
 *                 )
 *                 _check(rc, "nvtxwEventSchemasRegister")
 *                 self._event_schemas_registered = True             # <<<<<<<<<<<<<<
 *             schema_ids_out[0] = self._event_schema_ids
 * 
*/
            __pyx_v_self->_event_schemas_registered = 1;

            /* "nvtx/_lib/writer.pyx":1024
 *         with self._session._lock:
 *             self._ensure_valid()
 *             if not self._event_schemas_registered:             # <<<<<<<<<<<<<<
 *                 rc = nvtxwEventSchemasRegister(
 *                     self._backend._iface,
*/
          }

          /* "nvtx/_lib/writer.pyx":1033
 *                 _check(rc, "nvtxwEventSchemasRegister")
 *                 self._event_schemas_registered = True
 *             schema_ids_out[0] = self._event_schema_ids             # <<<<<<<<<<<<<<
 * 
 *     def get_registered_string(self, string):
*/
          __pyx_t_10 = __pyx_v_self->_event_schema_ids;
          (__pyx_v_schema_ids_out[0]) = __pyx_t_10;

          /* "nvtx/_lib/writer.pyx":1022
 *     ):
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             if not self._event_schemas_registered:
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Domain._get_event_schema_ids", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 1022, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_3);
          __pyx_t_11 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1022, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_11, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1022, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (__pyx_t_9 < (0)) __PYX_ERR(0, 1022, __pyx_L9_except_error)
          __pyx_t_13 = (!__pyx_t_9);
          if (unlikely(__pyx_t_13)) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_4, __pyx_t_3);
            __pyx_t_2 = 0;  __pyx_t_4 = 0;  __pyx_t_3 = 0; 
            __PYX_ERR(0, 1022, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1022, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "nvtx/_lib/writer.pyx":1018
 *                 "domain is no longer valid: its owning session has ended")
 * 
 *     cdef _get_event_schema_ids(             # <<<<<<<<<<<<<<
 *         self, nvtxwEventHelperSchemaIds_t* schema_ids_out
 *     ):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._get_event_schema_ids", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1035
 *             schema_ids_out[0] = self._event_schema_ids
 * 
 *     def get_registered_string(self, string):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a registered string in this domain.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_3get_registered_string(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_2get_registered_string, "Domain.get_registered_string(self, string)\n\nGet or create a registered string in this domain.\n\nParameters\n----------\nstring : str or bytes\n    String to register.\n\nReturns\n-------\nRegisteredString\n    The registered string. Results are cached per domain; str and\n    bytes spellings of the same string share one registration.\n\nRaises\n------\nRuntimeError\n    If the domain is no longer valid.\nTypeError\n    If ``string`` is not a string or bytes object.\nWriterError\n    If the backend fails to register the string.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_3get_registered_string = {"get_registered_string", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_3get_registered_string, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_2get_registered_string};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_3get_registered_string(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_string = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_registered_string (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1035, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1035, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_registered_string", 0) < (0)) __PYX_ERR(0, 1035, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_registered_string", 1, 1, 1, i); __PYX_ERR(0, 1035, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1035, __pyx_L3_error)
    }
    __pyx_v_string = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_registered_string", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1035, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_registered_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_2get_registered_string(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_string);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_2get_registered_string(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_string) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_registered_string", 0);

  /* "nvtx/_lib/writer.pyx":1059
 *             If the backend fails to register the string.
 *         """
 *         if not isinstance(string, (str, bytes)):             # <<<<<<<<<<<<<<
 *             raise TypeError("string must be a str or bytes")
 *         with self._session._lock:
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_string); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_string); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = (!__pyx_t_1);
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/writer.pyx":1060
 *         """
 *         if not isinstance(string, (str, bytes)):
 *             raise TypeError("string must be a str or bytes")             # <<<<<<<<<<<<<<
 *         with self._session._lock:
 *             self._ensure_valid()
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_string_must_be_a_str_or_bytes};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1060, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1059
 *             If the backend fails to register the string.
 *         """
 *         if not isinstance(string, (str, bytes)):             # <<<<<<<<<<<<<<
 *             raise TypeError("string must be a str or bytes")
 *         with self._session._lock:
*/
  }

  /* "nvtx/_lib/writer.pyx":1061
 *         if not isinstance(string, (str, bytes)):
 *             raise TypeError("string must be a str or bytes")
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             return self._get_string_cached(_as_str(string))
*/
  /*with:*/ {
    __pyx_t_6 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1061, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = NULL;
    __pyx_t_7 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1061, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1061, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":1062
 *             raise TypeError("string must be a str or bytes")
 *         with self._session._lock:
 *             self._ensure_valid()             # <<<<<<<<<<<<<<
 *             return self._get_string_cached(_as_str(string))
 * 
*/
          __pyx_t_3 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_self->__pyx_vtab)->_ensure_valid(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1062, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "nvtx/_lib/writer.pyx":1063
 *         with self._session._lock:
 *             self._ensure_valid()
 *             return self._get_string_cached(_as_str(string))             # <<<<<<<<<<<<<<
 * 
 *     def _register_string(self, string):
*/
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_7 = NULL;
          __Pyx_INCREF(__pyx_v_self->_get_string_cached);
          __pyx_t_4 = __pyx_v_self->_get_string_cached; 
          __pyx_t_11 = __pyx_f_4nvtx_4_lib_6writer__as_str(__pyx_v_string); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1063, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_5 = 1;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
            assert(__pyx_t_7);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
            __pyx_t_5 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_11};
            __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1063, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_3);
          }
          __pyx_r = __pyx_t_3;
          __pyx_t_3 = 0;
          goto __pyx_L14_try_return;

          /* "nvtx/_lib/writer.pyx":1061
 *         if not isinstance(string, (str, bytes)):
 *             raise TypeError("string must be a str or bytes")
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             return self._get_string_cached(_as_str(string))
*/
        }
        __pyx_L10_error:;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_registered_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_4, &__pyx_t_11) < 0) __PYX_ERR(0, 1061, __pyx_L12_except_error)
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_11);
          __pyx_t_7 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_4, __pyx_t_11); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1061, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_7, NULL);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1061, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (__pyx_t_2 < (0)) __PYX_ERR(0, 1061, __pyx_L12_except_error)
          __pyx_t_1 = (!__pyx_t_2);
          if (unlikely(__pyx_t_1)) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_4, __pyx_t_11);
            __pyx_t_3 = 0;  __pyx_t_4 = 0;  __pyx_t_11 = 0; 
            __PYX_ERR(0, 1061, __pyx_L12_except_error)
          }
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          goto __pyx_L11_exception_handled;
        }
        __pyx_L12_except_error:;
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
        goto __pyx_L1_error;
        __pyx_L14_try_return:;
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
        goto __pyx_L7_return;
        __pyx_L11_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_6) {
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1061, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        goto __pyx_L9;
      }
      __pyx_L7_return: {
        __pyx_t_10 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_6) {
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1061, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __pyx_r = __pyx_t_10;
        __pyx_t_10 = 0;
        goto __pyx_L0;
      }
      __pyx_L9:;
    }
    goto __pyx_L19;
    __pyx_L6_error:;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L1_error;
    __pyx_L19:;
  }

  /* "nvtx/_lib/writer.pyx":1035
 *             schema_ids_out[0] = self._event_schema_ids
 * 
 *     def get_registered_string(self, string):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a registered string in this domain.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_registered_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1065
 *             return self._get_string_cached(_as_str(string))
 * 
 *     def _register_string(self, string):             # <<<<<<<<<<<<<<
 *         cdef bytes encoded = _as_bytes(string)
 *         cdef const char* c_string = encoded
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_5_register_string(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_4_register_string, "Domain._register_string(self, string)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_5_register_string = {"_register_string", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_5_register_string, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_4_register_string};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_5_register_string(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_string = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_register_string (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1065, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1065, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_register_string", 0) < (0)) __PYX_ERR(0, 1065, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_register_string", 1, 1, 1, i); __PYX_ERR(0, 1065, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1065, __pyx_L3_error)
    }
    __pyx_v_string = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_register_string", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1065, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._register_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_4_register_string(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_string);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_4_register_string(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_string) {
  PyObject *__pyx_v_encoded = 0;
  char const *__pyx_v_c_string;
  nvtxStringHandle_t __pyx_v_handle;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char const *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_register_string", 0);

  /* "nvtx/_lib/writer.pyx":1066
 * 
 *     def _register_string(self, string):
 *         cdef bytes encoded = _as_bytes(string)             # <<<<<<<<<<<<<<
 *         cdef const char* c_string = encoded
 *         cdef nvtxStringHandle_t handle = NULL
*/
  __pyx_t_1 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_string); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1066, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_encoded = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1067
 *     def _register_string(self, string):
 *         cdef bytes encoded = _as_bytes(string)
 *         cdef const char* c_string = encoded             # <<<<<<<<<<<<<<
 *         cdef nvtxStringHandle_t handle = NULL
 *         cdef nvtxwResultCode_t rc = self._backend._iface.StringRegister(
*/
  if (unlikely(__pyx_v_encoded == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1067, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyBytes_AsString(__pyx_v_encoded); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 1067, __pyx_L1_error)
  __pyx_v_c_string = __pyx_t_2;

  /* "nvtx/_lib/writer.pyx":1068
 *         cdef bytes encoded = _as_bytes(string)
 *         cdef const char* c_string = encoded
 *         cdef nvtxStringHandle_t handle = NULL             # <<<<<<<<<<<<<<
 *         cdef nvtxwResultCode_t rc = self._backend._iface.StringRegister(
 *             self._handle, c_string, &handle)
*/
  __pyx_v_handle = NULL;

  /* "nvtx/_lib/writer.pyx":1069
 *         cdef const char* c_string = encoded
 *         cdef nvtxStringHandle_t handle = NULL
 *         cdef nvtxwResultCode_t rc = self._backend._iface.StringRegister(             # <<<<<<<<<<<<<<
 *             self._handle, c_string, &handle)
 *         _check(rc, "StringRegister")
*/
  __pyx_v_rc = __pyx_v_self->_backend->_iface->StringRegister(__pyx_v_self->_handle, __pyx_v_c_string, (&__pyx_v_handle));

  /* "nvtx/_lib/writer.pyx":1071
 *         cdef nvtxwResultCode_t rc = self._backend._iface.StringRegister(
 *             self._handle, c_string, &handle)
 *         _check(rc, "StringRegister")             # <<<<<<<<<<<<<<
 *         return RegisteredString(self, string, <uintptr_t>handle)
 * 
*/
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_StringRegister); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1071, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1072
 *             self._handle, c_string, &handle)
 *         _check(rc, "StringRegister")
 *         return RegisteredString(self, string, <uintptr_t>handle)             # <<<<<<<<<<<<<<
 * 
 *     def get_category_id(self, name: str | bytes):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = NULL;
  __pyx_t_4 = __Pyx_PyLong_FromSize_t(((uintptr_t)__pyx_v_handle)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1072, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_3, ((PyObject *)__pyx_v_self), __pyx_v_string, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString, __pyx_callargs+__pyx_t_5, (4-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1072, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":1065
 *             return self._get_string_cached(_as_str(string))
 * 
 *     def _register_string(self, string):             # <<<<<<<<<<<<<<
 *         cdef bytes encoded = _as_bytes(string)
 *         cdef const char* c_string = encoded
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._register_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1074
 *         return RegisteredString(self, string, <uintptr_t>handle)
 * 
 *     def get_category_id(self, name: str | bytes):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a named category in this domain.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_7get_category_id(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_6get_category_id, "Domain.get_category_id(self, name: str | bytes)\n\nGet or create a named category in this domain.\n\nParameters\n----------\nname : str or bytes\n    Category name.\n\nReturns\n-------\nint\n    The category ID. IDs start at 1 because 0 represents no category,\n    and skip integer category values already passed to this domain's\n    event write methods. Results are cached per domain.\n\nRaises\n------\nRuntimeError\n    If the domain is no longer valid.\nTypeError\n    If ``name`` is not a string or bytes object.\nWriterError\n    If the backend fails to register the category.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_7get_category_id = {"get_category_id", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_7get_category_id, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_6get_category_id};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_7get_category_id(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_category_id (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1074, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1074, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_category_id", 0) < (0)) __PYX_ERR(0, 1074, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_category_id", 1, 1, 1, i); __PYX_ERR(0, 1074, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1074, __pyx_L3_error)
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_category_id", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1074, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_category_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_6get_category_id(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_6get_category_id(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_v_category_id = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_category_id", 0);
  __Pyx_INCREF(__pyx_v_name);

  /* "nvtx/_lib/writer.pyx":1099
 *             If the backend fails to register the category.
 *         """
 *         if not isinstance(name, (str, bytes)):             # <<<<<<<<<<<<<<
 *             raise TypeError("category name must be a str or bytes")
 *         self._ensure_valid()
*/
  __pyx_t_2 = PyUnicode_Check(__pyx_v_name); 
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyBytes_Check(__pyx_v_name); 
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  __pyx_t_2 = (!__pyx_t_1);
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/writer.pyx":1100
 *         """
 *         if not isinstance(name, (str, bytes)):
 *             raise TypeError("category name must be a str or bytes")             # <<<<<<<<<<<<<<
 *         self._ensure_valid()
 *         name = _as_str(name)
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_category_name_must_be_a_str_or_b};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1100, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1100, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1099
 *             If the backend fails to register the category.
 *         """
 *         if not isinstance(name, (str, bytes)):             # <<<<<<<<<<<<<<
 *             raise TypeError("category name must be a str or bytes")
 *         self._ensure_valid()
*/
  }

  /* "nvtx/_lib/writer.pyx":1101
 *         if not isinstance(name, (str, bytes)):
 *             raise TypeError("category name must be a str or bytes")
 *         self._ensure_valid()             # <<<<<<<<<<<<<<
 *         name = _as_str(name)
 *         category_id = self._categories.get(name)
*/
  __pyx_t_3 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_self->__pyx_vtab)->_ensure_valid(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":1102
 *             raise TypeError("category name must be a str or bytes")
 *         self._ensure_valid()
 *         name = _as_str(name)             # <<<<<<<<<<<<<<
 *         category_id = self._categories.get(name)
 *         if category_id is not None:
*/
  __pyx_t_3 = __pyx_f_4nvtx_4_lib_6writer__as_str(__pyx_v_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF_SET(__pyx_v_name, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":1103
 *         self._ensure_valid()
 *         name = _as_str(name)
 *         category_id = self._categories.get(name)             # <<<<<<<<<<<<<<
 *         if category_id is not None:
 *             return category_id
*/
  if (unlikely(__pyx_v_self->_categories == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 1103, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_categories, __pyx_v_name, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_category_id = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "nvtx/_lib/writer.pyx":1104
 *         name = _as_str(name)
 *         category_id = self._categories.get(name)
 *         if category_id is not None:             # <<<<<<<<<<<<<<
 *             return category_id
 *         return self._register_category(name)
*/
  __pyx_t_2 = (__pyx_v_category_id != Py_None);
  if (__pyx_t_2) {

    /* "nvtx/_lib/writer.pyx":1105
 *         category_id = self._categories.get(name)
 *         if category_id is not None:
 *             return category_id             # <<<<<<<<<<<<<<
 *         return self._register_category(name)
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_category_id);
    __pyx_r = __pyx_v_category_id;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":1104
 *         name = _as_str(name)
 *         category_id = self._categories.get(name)
 *         if category_id is not None:             # <<<<<<<<<<<<<<
 *             return category_id
 *         return self._register_category(name)
*/
  }

  /* "nvtx/_lib/writer.pyx":1106
 *         if category_id is not None:
 *             return category_id
 *         return self._register_category(name)             # <<<<<<<<<<<<<<
 * 
 *     def _register_category(self, name):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_name};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_register_category, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":1074
 *         return RegisteredString(self, string, <uintptr_t>handle)
 * 
 *     def get_category_id(self, name: str | bytes):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a named category in this domain.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_category_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_category_id);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1108
 *         return self._register_category(name)
 * 
 *     def _register_category(self, name):             # <<<<<<<<<<<<<<
 *         cdef uint32_t category_id
 *         cdef bytes encoded
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_9_register_category(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_8_register_category, "Domain._register_category(self, name)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_9_register_category = {"_register_category", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_9_register_category, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_8_register_category};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_9_register_category(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_register_category (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1108, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1108, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_register_category", 0) < (0)) __PYX_ERR(0, 1108, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_register_category", 1, 1, 1, i); __PYX_ERR(0, 1108, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1108, __pyx_L3_error)
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_register_category", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1108, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._register_category", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_8_register_category(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_8_register_category(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_name) {
  uint32_t __pyx_v_category_id;
  PyObject *__pyx_v_encoded = 0;
  char const *__pyx_v_c_name;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_v_cached = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  uint32_t __pyx_t_10;
  char const *__pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_register_category", 0);

  /* "nvtx/_lib/writer.pyx":1113
 *         cdef const char* c_name
 *         cdef nvtxwResultCode_t rc
 *         cached = None             # <<<<<<<<<<<<<<
 *         with self._session._lock:
 *             self._ensure_valid()
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_cached = Py_None;

  /* "nvtx/_lib/writer.pyx":1114
 *         cdef nvtxwResultCode_t rc
 *         cached = None
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             cached = self._categories.get(name)
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1114, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1114, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":1115
 *         cached = None
 *         with self._session._lock:
 *             self._ensure_valid()             # <<<<<<<<<<<<<<
 *             cached = self._categories.get(name)
 *             if cached is not None:
*/
          __pyx_t_2 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_self->__pyx_vtab)->_ensure_valid(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1115, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "nvtx/_lib/writer.pyx":1116
 *         with self._session._lock:
 *             self._ensure_valid()
 *             cached = self._categories.get(name)             # <<<<<<<<<<<<<<
 *             if cached is not None:
 *                 return cached
*/
          if (unlikely(__pyx_v_self->_categories == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
            __PYX_ERR(0, 1116, __pyx_L7_error)
          }
          __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_categories, __pyx_v_name, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1116, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF_SET(__pyx_v_cached, __pyx_t_2);
          __pyx_t_2 = 0;

          /* "nvtx/_lib/writer.pyx":1117
 *             self._ensure_valid()
 *             cached = self._categories.get(name)
 *             if cached is not None:             # <<<<<<<<<<<<<<
 *                 return cached
 *             category_id = next(self._category_ids)
*/
          __pyx_t_9 = (__pyx_v_cached != Py_None);
          if (__pyx_t_9) {

            /* "nvtx/_lib/writer.pyx":1118
 *             cached = self._categories.get(name)
 *             if cached is not None:
 *                 return cached             # <<<<<<<<<<<<<<
 *             category_id = next(self._category_ids)
 *             # Skip IDs the user already claimed by passing raw integer
*/
            __Pyx_XDECREF(__pyx_r);
            __Pyx_INCREF(__pyx_v_cached);
            __pyx_r = __pyx_v_cached;
            goto __pyx_L11_try_return;

            /* "nvtx/_lib/writer.pyx":1117
 *             self._ensure_valid()
 *             cached = self._categories.get(name)
 *             if cached is not None:             # <<<<<<<<<<<<<<
 *                 return cached
 *             category_id = next(self._category_ids)
*/
          }

          /* "nvtx/_lib/writer.pyx":1119
 *             if cached is not None:
 *                 return cached
 *             category_id = next(self._category_ids)             # <<<<<<<<<<<<<<
 *             # Skip IDs the user already claimed by passing raw integer
 *             # categories to event writes.
*/
          __pyx_t_2 = __pyx_v_self->_category_ids;
          __Pyx_INCREF(__pyx_t_2);
          __pyx_t_4 = __Pyx_PyIter_Next(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1119, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_10 = __Pyx_PyLong_As_uint32_t(__pyx_t_4); if (unlikely((__pyx_t_10 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1119, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_v_category_id = __pyx_t_10;

          /* "nvtx/_lib/writer.pyx":1122
 *             # Skip IDs the user already claimed by passing raw integer
 *             # categories to event writes.
 *             while category_id in self._user_category_ids:             # <<<<<<<<<<<<<<
 *                 category_id = next(self._category_ids)
 *             encoded = _as_bytes(name)
*/
          while (1) {
            __pyx_t_4 = __Pyx_PyLong_From_uint32_t(__pyx_v_category_id); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1122, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_4);
            if (unlikely(__pyx_v_self->_user_category_ids == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
              __PYX_ERR(0, 1122, __pyx_L7_error)
            }
            __pyx_t_9 = (__Pyx_PySet_ContainsTF(__pyx_t_4, __pyx_v_self->_user_category_ids, Py_EQ)); if (unlikely((__pyx_t_9 < 0))) __PYX_ERR(0, 1122, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (!__pyx_t_9) break;

            /* "nvtx/_lib/writer.pyx":1123
 *             # categories to event writes.
 *             while category_id in self._user_category_ids:
 *                 category_id = next(self._category_ids)             # <<<<<<<<<<<<<<
 *             encoded = _as_bytes(name)
 *             c_name = encoded
*/
            __pyx_t_4 = __pyx_v_self->_category_ids;
            __Pyx_INCREF(__pyx_t_4);
            __pyx_t_2 = __Pyx_PyIter_Next(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1123, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __pyx_t_10 = __Pyx_PyLong_As_uint32_t(__pyx_t_2); if (unlikely((__pyx_t_10 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1123, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __pyx_v_category_id = __pyx_t_10;
          }

          /* "nvtx/_lib/writer.pyx":1124
 *             while category_id in self._user_category_ids:
 *                 category_id = next(self._category_ids)
 *             encoded = _as_bytes(name)             # <<<<<<<<<<<<<<
 *             c_name = encoded
 *             rc = self._backend._iface.CategoryRegister(
*/
          __pyx_t_2 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1124, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_v_encoded = ((PyObject*)__pyx_t_2);
          __pyx_t_2 = 0;

          /* "nvtx/_lib/writer.pyx":1125
 *                 category_id = next(self._category_ids)
 *             encoded = _as_bytes(name)
 *             c_name = encoded             # <<<<<<<<<<<<<<
 *             rc = self._backend._iface.CategoryRegister(
 *                 self._handle, category_id, c_name)
*/
          if (unlikely(__pyx_v_encoded == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
            __PYX_ERR(0, 1125, __pyx_L7_error)
          }
          __pyx_t_11 = __Pyx_PyBytes_AsString(__pyx_v_encoded); if (unlikely((!__pyx_t_11) && PyErr_Occurred())) __PYX_ERR(0, 1125, __pyx_L7_error)
          __pyx_v_c_name = __pyx_t_11;

          /* "nvtx/_lib/writer.pyx":1126
 *             encoded = _as_bytes(name)
 *             c_name = encoded
 *             rc = self._backend._iface.CategoryRegister(             # <<<<<<<<<<<<<<
 *                 self._handle, category_id, c_name)
 *             _check(rc, "CategoryRegister")
*/
          __pyx_v_rc = __pyx_v_self->_backend->_iface->CategoryRegister(__pyx_v_self->_handle, __pyx_v_category_id, __pyx_v_c_name);

          /* "nvtx/_lib/writer.pyx":1128
 *             rc = self._backend._iface.CategoryRegister(
 *                 self._handle, category_id, c_name)
 *             _check(rc, "CategoryRegister")             # <<<<<<<<<<<<<<
 *             self._categories[name] = category_id
 *             return category_id
*/
          __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_CategoryRegister); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1128, __pyx_L7_error)

          /* "nvtx/_lib/writer.pyx":1129
 *                 self._handle, category_id, c_name)
 *             _check(rc, "CategoryRegister")
 *             self._categories[name] = category_id             # <<<<<<<<<<<<<<
 *             return category_id
 * 
*/
          __pyx_t_2 = __Pyx_PyLong_From_uint32_t(__pyx_v_category_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1129, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          if (unlikely(__pyx_v_self->_categories == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 1129, __pyx_L7_error)
          }
          if (unlikely((PyDict_SetItem(__pyx_v_self->_categories, __pyx_v_name, __pyx_t_2) < 0))) __PYX_ERR(0, 1129, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "nvtx/_lib/writer.pyx":1130
 *             _check(rc, "CategoryRegister")
 *             self._categories[name] = category_id
 *             return category_id             # <<<<<<<<<<<<<<
 * 
 *     def get_scope(self, path, *, parent=None):
*/
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_2 = __Pyx_PyLong_From_uint32_t(__pyx_v_category_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1130, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_r = __pyx_t_2;
          __pyx_t_2 = 0;
          goto __pyx_L11_try_return;

          /* "nvtx/_lib/writer.pyx":1114
 *         cdef nvtxwResultCode_t rc
 *         cached = None
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             cached = self._categories.get(name)
*/
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Domain._register_category", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 1114, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_3);
          __pyx_t_12 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1114, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_12, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1114, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (__pyx_t_9 < (0)) __PYX_ERR(0, 1114, __pyx_L9_except_error)
          __pyx_t_14 = (!__pyx_t_9);
          if (unlikely(__pyx_t_14)) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_4, __pyx_t_3);
            __pyx_t_2 = 0;  __pyx_t_4 = 0;  __pyx_t_3 = 0; 
            __PYX_ERR(0, 1114, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1114, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1114, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L19;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L19:;
  }

  /* "nvtx/_lib/writer.pyx":1108
 *         return self._register_category(name)
 * 
 *     def _register_category(self, name):             # <<<<<<<<<<<<<<
 *         cdef uint32_t category_id
 *         cdef bytes encoded
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._register_category", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded);
  __Pyx_XDECREF(__pyx_v_cached);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1132
 *             return category_id
 * 
 *     def get_scope(self, path, *, parent=None):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a scope in this domain.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_11get_scope(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_10get_scope, "Domain.get_scope(self, path, *, parent=None)\n\nGet or create a scope in this domain.\n\nParameters\n----------\npath : str or bytes\n    Path of the scope relative to ``parent``.\nparent : PredefinedScope or Scope, optional\n    Parent scope. ``None`` places the scope at the domain root. The\n    predefined parents may be :attr:`PredefinedScope.NONE`,\n    :attr:`PredefinedScope.ROOT`,\n    :attr:`PredefinedScope.CURRENT_HW_MACHINE`, or\n    :attr:`PredefinedScope.CURRENT_VM`.\n\nReturns\n-------\nScope\n    The registered scope. Results are cached for each ``path`` and\n    parent pair.\n\nRaises\n------\nRuntimeError\n    If the domain is no longer valid.\nTypeError\n    If ``parent`` is not a :class:`PredefinedScope`, a\n    :class:`Scope`, or ``None``.\nValueError\n    If ``parent`` is registered in a different domain or is not a\n    supported predefined parent scope.\nWriterError\n    If the backend fails to register the scope.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_11get_scope = {"get_scope", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_11get_scope, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_10get_scope};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_11get_scope(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path = 0;
  PyObject *__pyx_v_parent = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_scope (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path,&__pyx_mstate_global->__pyx_n_u_parent,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1132, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1132, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_scope", 0) < (0)) __PYX_ERR(0, 1132, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_scope", 1, 1, 1, i); __PYX_ERR(0, 1132, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1132, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_path = values[0];
    __pyx_v_parent = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_scope", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1132, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_scope", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_10get_scope(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_path, __pyx_v_parent);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_10get_scope(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_path, PyObject *__pyx_v_parent) {
  uint64_t __pyx_v_parent_id;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  uint64_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_scope", 0);

  /* "nvtx/_lib/writer.pyx":1167
 *         """
 *         cdef uint64_t parent_id
 *         if parent is None:             # <<<<<<<<<<<<<<
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):
*/
  __pyx_t_1 = (__pyx_v_parent == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":1168
 *         cdef uint64_t parent_id
 *         if parent is None:
 *             parent_id = NVTX_SCOPE_ROOT             # <<<<<<<<<<<<<<
 *         elif isinstance(parent, PredefinedScope):
 *             if parent not in (
*/
    __pyx_v_parent_id = NVTX_SCOPE_ROOT;

    /* "nvtx/_lib/writer.pyx":1167
 *         """
 *         cdef uint64_t parent_id
 *         if parent is None:             # <<<<<<<<<<<<<<
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/writer.pyx":1169
 *         if parent is None:
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):             # <<<<<<<<<<<<<<
 *             if parent not in (
 *                 PredefinedScope.NONE,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_PredefinedScope); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_IsInstance(__pyx_v_parent, __pyx_t_2); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1169, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":1170
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):
 *             if parent not in (             # <<<<<<<<<<<<<<
 *                 PredefinedScope.NONE,
 *                 PredefinedScope.ROOT,
*/
    __Pyx_INCREF(__pyx_v_parent);
    __pyx_t_2 = __pyx_v_parent;

    /* "nvtx/_lib/writer.pyx":1171
 *         elif isinstance(parent, PredefinedScope):
 *             if parent not in (
 *                 PredefinedScope.NONE,             # <<<<<<<<<<<<<<
 *                 PredefinedScope.ROOT,
 *                 PredefinedScope.CURRENT_HW_MACHINE,
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PredefinedScope); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_NONE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/writer.pyx":1170
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):
 *             if parent not in (             # <<<<<<<<<<<<<<
 *                 PredefinedScope.NONE,
 *                 PredefinedScope.ROOT,
*/
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L5_bool_binop_done;
    }

    /* "nvtx/_lib/writer.pyx":1172
 *             if parent not in (
 *                 PredefinedScope.NONE,
 *                 PredefinedScope.ROOT,             # <<<<<<<<<<<<<<
 *                 PredefinedScope.CURRENT_HW_MACHINE,
 *                 PredefinedScope.CURRENT_VM,
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PredefinedScope); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_ROOT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/writer.pyx":1170
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):
 *             if parent not in (             # <<<<<<<<<<<<<<
 *                 PredefinedScope.NONE,
 *                 PredefinedScope.ROOT,
*/
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L5_bool_binop_done;
    }

    /* "nvtx/_lib/writer.pyx":1173
 *                 PredefinedScope.NONE,
 *                 PredefinedScope.ROOT,
 *                 PredefinedScope.CURRENT_HW_MACHINE,             # <<<<<<<<<<<<<<
 *                 PredefinedScope.CURRENT_VM,
 *             ):
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PredefinedScope); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_CURRENT_HW_MACHINE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/writer.pyx":1170
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):
 *             if parent not in (             # <<<<<<<<<<<<<<
 *                 PredefinedScope.NONE,
 *                 PredefinedScope.ROOT,
*/
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L5_bool_binop_done;
    }

    /* "nvtx/_lib/writer.pyx":1174
 *                 PredefinedScope.ROOT,
 *                 PredefinedScope.CURRENT_HW_MACHINE,
 *                 PredefinedScope.CURRENT_VM,             # <<<<<<<<<<<<<<
 *             ):
 *                 raise ValueError(
*/
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PredefinedScope); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_CURRENT_VM); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/writer.pyx":1170
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):
 *             if parent not in (             # <<<<<<<<<<<<<<
 *                 PredefinedScope.NONE,
 *                 PredefinedScope.ROOT,
*/
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __pyx_t_5;
    __pyx_L5_bool_binop_done:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __pyx_t_1;
    if (unlikely(__pyx_t_5)) {

      /* "nvtx/_lib/writer.pyx":1176
 *                 PredefinedScope.CURRENT_VM,
 *             ):
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     "predefined parent scope must be NONE, ROOT, "
 *                     "CURRENT_HW_MACHINE, or CURRENT_VM"
*/
      __pyx_t_3 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_predefined_parent_scope_must_be};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1176, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 1176, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":1170
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):
 *             if parent not in (             # <<<<<<<<<<<<<<
 *                 PredefinedScope.NONE,
 *                 PredefinedScope.ROOT,
*/
    }

    /* "nvtx/_lib/writer.pyx":1180
 *                     "CURRENT_HW_MACHINE, or CURRENT_VM"
 *                 )
 *             parent_id = <uint64_t>parent.value             # <<<<<<<<<<<<<<
 *         elif isinstance(parent, Scope):
 *             if (<Scope>parent)._domain is not self:
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_parent, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = __Pyx_PyLong_As_uint64_t(__pyx_t_2); if (unlikely((__pyx_t_7 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1180, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_parent_id = ((uint64_t)__pyx_t_7);

    /* "nvtx/_lib/writer.pyx":1169
 *         if parent is None:
 *             parent_id = NVTX_SCOPE_ROOT
 *         elif isinstance(parent, PredefinedScope):             # <<<<<<<<<<<<<<
 *             if parent not in (
 *                 PredefinedScope.NONE,
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/writer.pyx":1181
 *                 )
 *             parent_id = <uint64_t>parent.value
 *         elif isinstance(parent, Scope):             # <<<<<<<<<<<<<<
 *             if (<Scope>parent)._domain is not self:
 *                 raise ValueError(
*/
  __pyx_t_5 = __Pyx_TypeCheck(__pyx_v_parent, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Scope); 
  if (likely(__pyx_t_5)) {

    /* "nvtx/_lib/writer.pyx":1182
 *             parent_id = <uint64_t>parent.value
 *         elif isinstance(parent, Scope):
 *             if (<Scope>parent)._domain is not self:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "parent scope is registered in a different domain")
*/
    __pyx_t_5 = (((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_parent)->_domain != ((PyObject *)__pyx_v_self));
    if (unlikely(__pyx_t_5)) {

      /* "nvtx/_lib/writer.pyx":1183
 *         elif isinstance(parent, Scope):
 *             if (<Scope>parent)._domain is not self:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     "parent scope is registered in a different domain")
 *             parent_id = (<Scope>parent)._scope_id
*/
      __pyx_t_3 = NULL;
      __pyx_t_6 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_parent_scope_is_registered_in_a};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1183, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 1183, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":1182
 *             parent_id = <uint64_t>parent.value
 *         elif isinstance(parent, Scope):
 *             if (<Scope>parent)._domain is not self:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "parent scope is registered in a different domain")
*/
    }

    /* "nvtx/_lib/writer.pyx":1185
 *                 raise ValueError(
 *                     "parent scope is registered in a different domain")
 *             parent_id = (<Scope>parent)._scope_id             # <<<<<<<<<<<<<<
 *         else:
 *             raise TypeError(
*/
    __pyx_t_7 = ((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)__pyx_v_parent)->_scope_id;
    __pyx_v_parent_id = __pyx_t_7;

    /* "nvtx/_lib/writer.pyx":1181
 *                 )
 *             parent_id = <uint64_t>parent.value
 *         elif isinstance(parent, Scope):             # <<<<<<<<<<<<<<
 *             if (<Scope>parent)._domain is not self:
 *                 raise ValueError(
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/writer.pyx":1187
 *             parent_id = (<Scope>parent)._scope_id
 *         else:
 *             raise TypeError(             # <<<<<<<<<<<<<<
 *                 "parent must be a PredefinedScope, a Scope, or None"
 *             )
*/
  /*else*/ {
    __pyx_t_3 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_parent_must_be_a_PredefinedScope};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1187, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1187, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "nvtx/_lib/writer.pyx":1190
 *                 "parent must be a PredefinedScope, a Scope, or None"
 *             )
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             return self._get_scope_cached(_as_str(path), parent_id)
*/
  /*with:*/ {
    __pyx_t_8 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1190, __pyx_L10_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1190, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":1191
 *             )
 *         with self._session._lock:
 *             self._ensure_valid()             # <<<<<<<<<<<<<<
 *             return self._get_scope_cached(_as_str(path), parent_id)
 * 
*/
          __pyx_t_2 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_self->__pyx_vtab)->_ensure_valid(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1191, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "nvtx/_lib/writer.pyx":1192
 *         with self._session._lock:
 *             self._ensure_valid()
 *             return self._get_scope_cached(_as_str(path), parent_id)             # <<<<<<<<<<<<<<
 * 
 *     def _register_scope(self, path, uint64_t parent_id):
*/
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_self->_get_scope_cached);
          __pyx_t_3 = __pyx_v_self->_get_scope_cached; 
          __pyx_t_12 = __pyx_f_4nvtx_4_lib_6writer__as_str(__pyx_v_path); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1192, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_13 = __Pyx_PyLong_From_uint64_t(__pyx_v_parent_id); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1192, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
            assert(__pyx_t_4);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_t_12, __pyx_t_13};
            __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1192, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_2);
          }
          __pyx_r = __pyx_t_2;
          __pyx_t_2 = 0;
          goto __pyx_L18_try_return;

          /* "nvtx/_lib/writer.pyx":1190
 *                 "parent must be a PredefinedScope, a Scope, or None"
 *             )
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             return self._get_scope_cached(_as_str(path), parent_id)
*/
        }
        __pyx_L14_error:;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_scope", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_13) < 0) __PYX_ERR(0, 1190, __pyx_L16_except_error)
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_t_13);
          __pyx_t_12 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_13); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1190, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_12, NULL);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1190, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (__pyx_t_5 < (0)) __PYX_ERR(0, 1190, __pyx_L16_except_error)
          __pyx_t_1 = (!__pyx_t_5);
          if (unlikely(__pyx_t_1)) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_3, __pyx_t_13);
            __pyx_t_2 = 0;  __pyx_t_3 = 0;  __pyx_t_13 = 0; 
            __PYX_ERR(0, 1190, __pyx_L16_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          goto __pyx_L15_exception_handled;
        }
        __pyx_L16_except_error:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L1_error;
        __pyx_L18_try_return:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L11_return;
        __pyx_L15_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_8) {
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1190, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        goto __pyx_L13;
      }
      __pyx_L11_return: {
        __pyx_t_11 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_8) {
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1190, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __pyx_r = __pyx_t_11;
        __pyx_t_11 = 0;
        goto __pyx_L0;
      }
      __pyx_L13:;
    }
    goto __pyx_L23;
    __pyx_L10_error:;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    goto __pyx_L1_error;
    __pyx_L23:;
  }

  /* "nvtx/_lib/writer.pyx":1132
 *             return category_id
 * 
 *     def get_scope(self, path, *, parent=None):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a scope in this domain.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_scope", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1194
 *             return self._get_scope_cached(_as_str(path), parent_id)
 * 
 *     def _register_scope(self, path, uint64_t parent_id):             # <<<<<<<<<<<<<<
 *         cdef bytes encoded = _as_bytes(path)
 *         cdef const char* c_path = NULL
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_13_register_scope(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_12_register_scope, "Domain._register_scope(self, path, uint64_t parent_id)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_13_register_scope = {"_register_scope", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_13_register_scope, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_12_register_scope};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_13_register_scope(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_path = 0;
  uint64_t __pyx_v_parent_id;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_register_scope (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_path,&__pyx_mstate_global->__pyx_n_u_parent_id,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1194, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1194, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1194, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_register_scope", 0) < (0)) __PYX_ERR(0, 1194, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_register_scope", 1, 2, 2, i); __PYX_ERR(0, 1194, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1194, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1194, __pyx_L3_error)
    }
    __pyx_v_path = values[0];
    __pyx_v_parent_id = __Pyx_PyLong_As_uint64_t(values[1]); if (unlikely((__pyx_v_parent_id == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1194, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_register_scope", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1194, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._register_scope", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_12_register_scope(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_path, __pyx_v_parent_id);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_12_register_scope(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_path, uint64_t __pyx_v_parent_id) {
  PyObject *__pyx_v_encoded = 0;
  char const *__pyx_v_c_path;
  nvtxScopeAttr_t __pyx_v_attr;
  uint64_t __pyx_v_scope_id;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  char const *__pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_register_scope", 0);

  /* "nvtx/_lib/writer.pyx":1195
 * 
 *     def _register_scope(self, path, uint64_t parent_id):
 *         cdef bytes encoded = _as_bytes(path)             # <<<<<<<<<<<<<<
 *         cdef const char* c_path = NULL
 *         cdef nvtxScopeAttr_t attr
*/
  __pyx_t_1 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_path); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_encoded = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1196
 *     def _register_scope(self, path, uint64_t parent_id):
 *         cdef bytes encoded = _as_bytes(path)
 *         cdef const char* c_path = NULL             # <<<<<<<<<<<<<<
 *         cdef nvtxScopeAttr_t attr
 *         cdef uint64_t scope_id = 0
*/
  __pyx_v_c_path = NULL;

  /* "nvtx/_lib/writer.pyx":1198
 *         cdef const char* c_path = NULL
 *         cdef nvtxScopeAttr_t attr
 *         cdef uint64_t scope_id = 0             # <<<<<<<<<<<<<<
 *         cdef nvtxwResultCode_t rc
 *         if encoded is not None:
*/
  __pyx_v_scope_id = 0;

  /* "nvtx/_lib/writer.pyx":1200
 *         cdef uint64_t scope_id = 0
 *         cdef nvtxwResultCode_t rc
 *         if encoded is not None:             # <<<<<<<<<<<<<<
 *             c_path = encoded
 *         attr.structSize = sizeof(nvtxScopeAttr_t)
*/
  __pyx_t_2 = (__pyx_v_encoded != ((PyObject*)Py_None));
  if (__pyx_t_2) {

    /* "nvtx/_lib/writer.pyx":1201
 *         cdef nvtxwResultCode_t rc
 *         if encoded is not None:
 *             c_path = encoded             # <<<<<<<<<<<<<<
 *         attr.structSize = sizeof(nvtxScopeAttr_t)
 *         attr.path = c_path
*/
    if (unlikely(__pyx_v_encoded == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1201, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyBytes_AsString(__pyx_v_encoded); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(0, 1201, __pyx_L1_error)
    __pyx_v_c_path = __pyx_t_3;

    /* "nvtx/_lib/writer.pyx":1200
 *         cdef uint64_t scope_id = 0
 *         cdef nvtxwResultCode_t rc
 *         if encoded is not None:             # <<<<<<<<<<<<<<
 *             c_path = encoded
 *         attr.structSize = sizeof(nvtxScopeAttr_t)
*/
  }

  /* "nvtx/_lib/writer.pyx":1202
 *         if encoded is not None:
 *             c_path = encoded
 *         attr.structSize = sizeof(nvtxScopeAttr_t)             # <<<<<<<<<<<<<<
 *         attr.path = c_path
 *         attr.parentScope = parent_id
*/
  __pyx_v_attr.structSize = (sizeof(nvtxScopeAttr_t));

  /* "nvtx/_lib/writer.pyx":1203
 *             c_path = encoded
 *         attr.structSize = sizeof(nvtxScopeAttr_t)
 *         attr.path = c_path             # <<<<<<<<<<<<<<
 *         attr.parentScope = parent_id
 *         attr.scopeId = NVTX_SCOPE_NONE
*/
  __pyx_v_attr.path = __pyx_v_c_path;

  /* "nvtx/_lib/writer.pyx":1204
 *         attr.structSize = sizeof(nvtxScopeAttr_t)
 *         attr.path = c_path
 *         attr.parentScope = parent_id             # <<<<<<<<<<<<<<
 *         attr.scopeId = NVTX_SCOPE_NONE
 *         rc = self._backend._iface.ScopeRegister(self._handle, &attr, &scope_id)
*/
  __pyx_v_attr.parentScope = __pyx_v_parent_id;

  /* "nvtx/_lib/writer.pyx":1205
 *         attr.path = c_path
 *         attr.parentScope = parent_id
 *         attr.scopeId = NVTX_SCOPE_NONE             # <<<<<<<<<<<<<<
 *         rc = self._backend._iface.ScopeRegister(self._handle, &attr, &scope_id)
 *         _check(rc, "ScopeRegister")
*/
  __pyx_v_attr.scopeId = NVTX_SCOPE_NONE;

  /* "nvtx/_lib/writer.pyx":1206
 *         attr.parentScope = parent_id
 *         attr.scopeId = NVTX_SCOPE_NONE
 *         rc = self._backend._iface.ScopeRegister(self._handle, &attr, &scope_id)             # <<<<<<<<<<<<<<
 *         _check(rc, "ScopeRegister")
 *         return Scope(scope_id, path, self)
*/
  __pyx_v_rc = __pyx_v_self->_backend->_iface->ScopeRegister(__pyx_v_self->_handle, (&__pyx_v_attr), (&__pyx_v_scope_id));

  /* "nvtx/_lib/writer.pyx":1207
 *         attr.scopeId = NVTX_SCOPE_NONE
 *         rc = self._backend._iface.ScopeRegister(self._handle, &attr, &scope_id)
 *         _check(rc, "ScopeRegister")             # <<<<<<<<<<<<<<
 *         return Scope(scope_id, path, self)
 * 
*/
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_ScopeRegister); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1207, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1208
 *         rc = self._backend._iface.ScopeRegister(self._handle, &attr, &scope_id)
 *         _check(rc, "ScopeRegister")
 *         return Scope(scope_id, path, self)             # <<<<<<<<<<<<<<
 * 
 *     def get_schema(self, dtype, *, kind=None):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = NULL;
  __pyx_t_5 = __Pyx_PyLong_From_uint64_t(__pyx_v_scope_id); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = 1;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_t_5, __pyx_v_path, ((PyObject *)__pyx_v_self)};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Scope, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1208, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":1194
 *             return self._get_scope_cached(_as_str(path), parent_id)
 * 
 *     def _register_scope(self, path, uint64_t parent_id):             # <<<<<<<<<<<<<<
 *         cdef bytes encoded = _as_bytes(path)
 *         cdef const char* c_path = NULL
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._register_scope", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1210
 *         return Scope(scope_id, path, self)
 * 
 *     def get_schema(self, dtype, *, kind=None):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a payload schema in this domain.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_15get_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_14get_schema, "Domain.get_schema(self, dtype, *, kind=None)\n\nGet or create a payload schema in this domain.\n\nParameters\n----------\ndtype : dtype-like\n    NumPy dtype describing the payload layout. In an event schema,\n    fields define their roles with :func:`nvtx.numpy_dtype` and its\n    ``entry_kind`` argument, and may define specially interpreted\n    integer fields with its ``entry_type`` argument.\nkind : EventKind, optional\n    Event family described by the schema; required for use with\n    :meth:`Stream.write_event` and :meth:`Stream.write_event_batch`.\n    The ID-correlated :attr:`EventKind.RANGE_START` and\n    :attr:`EventKind.RANGE_END` families require exactly one field\n    annotated with :attr:`PayloadEntryType.RANGE_ID`. ``None`` creates\n    a generic payload schema.\n\nReturns\n-------\nSchema\n    The registered schema. Results are cached per domain, dtype,\n    and kind.\n\nRaises\n------\nRuntimeError\n    If numpy is not installed or the domain is no longer valid.\nTypeError\n    If ``dtype`` is not dtype-like, or ``kind`` is not an\n    :class:`nvtx.EventKind` or ``None``. For event schemas, if the\n    dtype is not structured, a field uses managed storage or a\n    non-native byte order, or a timestamp role field is not a\n    64-bit integer.\nValueError\n    If ``kind`` cannot be described by a dtype, or the dtype's\n    timestamp roles do not match ``kind``.\nWriterError\n    If the backend fails to register the schema.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_15get_schema = {"get_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_15get_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_14get_schema};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_15get_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_v_kind = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_schema (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_dtype,&__pyx_mstate_global->__pyx_n_u_kind,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1210, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1210, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_schema", 0) < (0)) __PYX_ERR(0, 1210, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_schema", 1, 1, 1, i); __PYX_ERR(0, 1210, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1210, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_dtype = values[0];
    __pyx_v_kind = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_schema", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1210, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_14get_schema(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_dtype, __pyx_v_kind);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_14get_schema(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_dtype, PyObject *__pyx_v_kind) {
  uint64_t __pyx_v_schema_id;
  PyObject *__pyx_v_dt = NULL;
  PyObject *__pyx_v_schema_key = NULL;
  PyObject *__pyx_v_schema = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  uint64_t __pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_schema", 0);

  /* "nvtx/_lib/writer.pyx":1252
 *         """
 *         cdef uint64_t schema_id
 *         if kind is not None and not isinstance(kind, EventKind):             # <<<<<<<<<<<<<<
 *             raise TypeError("kind must be an nvtx.EventKind or None")
 *         with self._session._lock:
*/
  __pyx_t_2 = (__pyx_v_kind != Py_None);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1252, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_kind, __pyx_t_3); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1252, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = (!__pyx_t_2);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1253
 *         cdef uint64_t schema_id
 *         if kind is not None and not isinstance(kind, EventKind):
 *             raise TypeError("kind must be an nvtx.EventKind or None")             # <<<<<<<<<<<<<<
 *         with self._session._lock:
 *             self._ensure_valid()
*/
    __pyx_t_5 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_kind_must_be_an_nvtx_EventKind_o};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1253, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1253, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1252
 *         """
 *         cdef uint64_t schema_id
 *         if kind is not None and not isinstance(kind, EventKind):             # <<<<<<<<<<<<<<
 *             raise TypeError("kind must be an nvtx.EventKind or None")
 *         with self._session._lock:
*/
  }

  /* "nvtx/_lib/writer.pyx":1254
 *         if kind is not None and not isinstance(kind, EventKind):
 *             raise TypeError("kind must be an nvtx.EventKind or None")
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             # numpy_dtype normalizes the input and raises if numpy is missing.
*/
  /*with:*/ {
    __pyx_t_7 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = NULL;
    __pyx_t_8 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1254, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1254, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":1255
 *             raise TypeError("kind must be an nvtx.EventKind or None")
 *         with self._session._lock:
 *             self._ensure_valid()             # <<<<<<<<<<<<<<
 *             # numpy_dtype normalizes the input and raises if numpy is missing.
 *             dt = numpy_dtype(dtype)
*/
          __pyx_t_3 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_self->__pyx_vtab)->_ensure_valid(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1255, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "nvtx/_lib/writer.pyx":1257
 *             self._ensure_valid()
 *             # numpy_dtype normalizes the input and raises if numpy is missing.
 *             dt = numpy_dtype(dtype)             # <<<<<<<<<<<<<<
 *             schema_key = PayloadSchemaKey(
 *                 dt, schema_flags=0 if kind is None else kind.value
*/
          __pyx_t_8 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_numpy_dtype); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1257, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
            assert(__pyx_t_8);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_v_dtype};
            __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1257, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_3);
          }
          __pyx_v_dt = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "nvtx/_lib/writer.pyx":1258
 *             # numpy_dtype normalizes the input and raises if numpy is missing.
 *             dt = numpy_dtype(dtype)
 *             schema_key = PayloadSchemaKey(             # <<<<<<<<<<<<<<
 *                 dt, schema_flags=0 if kind is None else kind.value
 *             )
*/
          __pyx_t_5 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_PayloadSchemaKey); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1258, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_8);

          /* "nvtx/_lib/writer.pyx":1259
 *             dt = numpy_dtype(dtype)
 *             schema_key = PayloadSchemaKey(
 *                 dt, schema_flags=0 if kind is None else kind.value             # <<<<<<<<<<<<<<
 *             )
 *             schema = self._schemas.get(schema_key)
*/
          __pyx_t_1 = (__pyx_v_kind == Py_None);
          if (__pyx_t_1) {
            __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
            __pyx_t_12 = __pyx_mstate_global->__pyx_int_0;
          } else {
            __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_kind, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1259, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_13);
            __pyx_t_12 = __pyx_t_13;
            __pyx_t_13 = 0;
          }
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_8))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
            assert(__pyx_t_5);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_5, __pyx_v_dt};
            __pyx_t_13 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1258, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_13);
            if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_schema_flags, __pyx_t_12, __pyx_t_13, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1258, __pyx_L10_error)
            __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_13);
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1258, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_3);
          }
          __pyx_v_schema_key = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "nvtx/_lib/writer.pyx":1261
 *                 dt, schema_flags=0 if kind is None else kind.value
 *             )
 *             schema = self._schemas.get(schema_key)             # <<<<<<<<<<<<<<
 *             if schema is None:
 *                 if kind is not None:
*/
          if (unlikely(__pyx_v_self->_schemas == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
            __PYX_ERR(0, 1261, __pyx_L10_error)
          }
          __pyx_t_3 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_schemas, __pyx_v_schema_key, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1261, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_v_schema = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "nvtx/_lib/writer.pyx":1262
 *             )
 *             schema = self._schemas.get(schema_key)
 *             if schema is None:             # <<<<<<<<<<<<<<
 *                 if kind is not None:
 *                     _validate_event_dtype(kind, dt, "an event schema")
*/
          __pyx_t_1 = (__pyx_v_schema == Py_None);
          if (__pyx_t_1) {

            /* "nvtx/_lib/writer.pyx":1263
 *             schema = self._schemas.get(schema_key)
 *             if schema is None:
 *                 if kind is not None:             # <<<<<<<<<<<<<<
 *                     _validate_event_dtype(kind, dt, "an event schema")
 *                 schema_id = self._schema_registrar._get_numpy_dtype_schema(
*/
            __pyx_t_1 = (__pyx_v_kind != Py_None);
            if (__pyx_t_1) {

              /* "nvtx/_lib/writer.pyx":1264
 *             if schema is None:
 *                 if kind is not None:
 *                     _validate_event_dtype(kind, dt, "an event schema")             # <<<<<<<<<<<<<<
 *                 schema_id = self._schema_registrar._get_numpy_dtype_schema(
 *                     schema_key
*/
              __pyx_t_8 = NULL;
              __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_validate_event_dtype); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1264, __pyx_L10_error)
              __Pyx_GOTREF(__pyx_t_13);
              __pyx_t_6 = 1;
              #if CYTHON_UNPACK_METHODS
              if (unlikely(PyMethod_Check(__pyx_t_13))) {
                __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_13);
                assert(__pyx_t_8);
                PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
                __Pyx_INCREF(__pyx_t_8);
                __Pyx_INCREF(__pyx__function);
                __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
                __pyx_t_6 = 0;
              }
              #endif
              {
                PyObject *__pyx_callargs[4] = {__pyx_t_8, __pyx_v_kind, __pyx_v_dt, __pyx_mstate_global->__pyx_kp_u_an_event_schema};
                __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
                if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1264, __pyx_L10_error)
                __Pyx_GOTREF(__pyx_t_3);
              }
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

              /* "nvtx/_lib/writer.pyx":1263
 *             schema = self._schemas.get(schema_key)
 *             if schema is None:
 *                 if kind is not None:             # <<<<<<<<<<<<<<
 *                     _validate_event_dtype(kind, dt, "an event schema")
 *                 schema_id = self._schema_registrar._get_numpy_dtype_schema(
*/
            }

            /* "nvtx/_lib/writer.pyx":1265
 *                 if kind is not None:
 *                     _validate_event_dtype(kind, dt, "an event schema")
 *                 schema_id = self._schema_registrar._get_numpy_dtype_schema(             # <<<<<<<<<<<<<<
 *                     schema_key
 *                 )
*/
            __pyx_t_13 = ((PyObject *)__pyx_v_self->_schema_registrar);
            __Pyx_INCREF(__pyx_t_13);

            /* "nvtx/_lib/writer.pyx":1266
 *                     _validate_event_dtype(kind, dt, "an event schema")
 *                 schema_id = self._schema_registrar._get_numpy_dtype_schema(
 *                     schema_key             # <<<<<<<<<<<<<<
 *                 )
 *                 schema = Schema(self, schema_id, dt, kind)
*/
            __pyx_t_6 = 0;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_v_schema_key};
              __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1265, __pyx_L10_error)
              __Pyx_GOTREF(__pyx_t_3);
            }

            /* "nvtx/_lib/writer.pyx":1265
 *                 if kind is not None:
 *                     _validate_event_dtype(kind, dt, "an event schema")
 *                 schema_id = self._schema_registrar._get_numpy_dtype_schema(             # <<<<<<<<<<<<<<
 *                     schema_key
 *                 )
*/
            __pyx_t_14 = __Pyx_PyLong_As_uint64_t(__pyx_t_3); if (unlikely((__pyx_t_14 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1265, __pyx_L10_error)
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_v_schema_id = __pyx_t_14;

            /* "nvtx/_lib/writer.pyx":1268
 *                     schema_key
 *                 )
 *                 schema = Schema(self, schema_id, dt, kind)             # <<<<<<<<<<<<<<
 *                 self._schemas[schema_key] = schema
 *             return schema
*/
            __pyx_t_13 = NULL;
            __pyx_t_8 = __Pyx_PyLong_From_uint64_t(__pyx_v_schema_id); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1268, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_8);
            __pyx_t_6 = 1;
            {
              PyObject *__pyx_callargs[5] = {__pyx_t_13, ((PyObject *)__pyx_v_self), __pyx_t_8, __pyx_v_dt, __pyx_v_kind};
              __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Schema, __pyx_callargs+__pyx_t_6, (5-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1268, __pyx_L10_error)
              __Pyx_GOTREF((PyObject *)__pyx_t_3);
            }
            __Pyx_DECREF_SET(__pyx_v_schema, ((PyObject *)__pyx_t_3));
            __pyx_t_3 = 0;

            /* "nvtx/_lib/writer.pyx":1269
 *                 )
 *                 schema = Schema(self, schema_id, dt, kind)
 *                 self._schemas[schema_key] = schema             # <<<<<<<<<<<<<<
 *             return schema
 * 
*/
            if (unlikely(__pyx_v_self->_schemas == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
              __PYX_ERR(0, 1269, __pyx_L10_error)
            }
            if (unlikely((PyDict_SetItem(__pyx_v_self->_schemas, __pyx_v_schema_key, __pyx_v_schema) < 0))) __PYX_ERR(0, 1269, __pyx_L10_error)

            /* "nvtx/_lib/writer.pyx":1262
 *             )
 *             schema = self._schemas.get(schema_key)
 *             if schema is None:             # <<<<<<<<<<<<<<
 *                 if kind is not None:
 *                     _validate_event_dtype(kind, dt, "an event schema")
*/
          }

          /* "nvtx/_lib/writer.pyx":1270
 *                 schema = Schema(self, schema_id, dt, kind)
 *                 self._schemas[schema_key] = schema
 *             return schema             # <<<<<<<<<<<<<<
 * 
 *     def get_counter(
*/
          __Pyx_XDECREF(__pyx_r);
          __Pyx_INCREF(__pyx_v_schema);
          __pyx_r = __pyx_v_schema;
          goto __pyx_L14_try_return;

          /* "nvtx/_lib/writer.pyx":1254
 *         if kind is not None and not isinstance(kind, EventKind):
 *             raise TypeError("kind must be an nvtx.EventKind or None")
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             # numpy_dtype normalizes the input and raises if numpy is missing.
*/
        }
        __pyx_L10_error:;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_8, &__pyx_t_13) < 0) __PYX_ERR(0, 1254, __pyx_L12_except_error)
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_t_8);
          __Pyx_XGOTREF(__pyx_t_13);
          __pyx_t_12 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_8, __pyx_t_13); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1254, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_12, NULL);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1254, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_15);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          if (__pyx_t_1 < (0)) __PYX_ERR(0, 1254, __pyx_L12_except_error)
          __pyx_t_4 = (!__pyx_t_1);
          if (unlikely(__pyx_t_4)) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_8);
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_8, __pyx_t_13);
            __pyx_t_3 = 0;  __pyx_t_8 = 0;  __pyx_t_13 = 0; 
            __PYX_ERR(0, 1254, __pyx_L12_except_error)
          }
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          goto __pyx_L11_exception_handled;
        }
        __pyx_L12_except_error:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L1_error;
        __pyx_L14_try_return:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L7_return;
        __pyx_L11_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_7) {
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1254, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        goto __pyx_L9;
      }
      __pyx_L7_return: {
        __pyx_t_11 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_7) {
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1254, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
        __pyx_r = __pyx_t_11;
        __pyx_t_11 = 0;
        goto __pyx_L0;
      }
      __pyx_L9:;
    }
    goto __pyx_L21;
    __pyx_L6_error:;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L1_error;
    __pyx_L21:;
  }

  /* "nvtx/_lib/writer.pyx":1210
 *         return Scope(scope_id, path, self)
 * 
 *     def get_schema(self, dtype, *, kind=None):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a payload schema in this domain.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dt);
  __Pyx_XDECREF(__pyx_v_schema_key);
  __Pyx_XDECREF(__pyx_v_schema);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1272
 *             return schema
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_17get_counter(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_16get_counter, "Domain.get_counter(self, name, dtype, *, description=None, scope=None, semantics=None)\n\nGet or create a counter or counter group in this domain.\n\nParameters\n----------\nname : str or bytes\n    Display name for the counter.\ndtype : int, float, or dtype-like\n    Counter value type. ``int`` records signed 64-bit integer samples,\n    ``float`` records double-precision samples, and a NumPy dtype-like\n    object records samples with that dtype. Top-level fixed-size array\n    dtypes are not supported. A structured dtype describes a flat\n    counter group. Its fields may define per-field\n    semantics with :func:`nvtx.numpy_dtype` and its\n    ``counter_semantics`` argument, and at most one field may have the\n    :attr:`nvtx.EntryKind.COUNTER_TIMESTAMP` role.\ndescription : str or bytes, optional\n    Longer description for the counter.\nscope : PredefinedScope, Scope, int, or None, optional\n    Scope associated with the counter. Pass a predefined scope, a\n    dynamic scope returned by :meth:`get_scope`, or an integer in the\n    NVTX static scope ID range.\nsemantics : CounterSemantics, optional\n    Semantics for the counter as a whole. For per-field semantics in a\n    structured dtype, use :func:`nvtx.numpy_dtype`.\n\nReturns\n-------\nCounter\n    The requested counter. Results are cached per domain.\n\nRaises\n------\nRuntimeError\n    If NumPy is required but not installed, or the domain is no\n    longer valid.\nTypeError\n    If ``dtype`` is not dtype-like, is a top-level fixed-size array,\n    or contains nested or array fields, or if ``scope`` has an\n    unsupported type.\nValueError\n    If the counter layout contains an unsupported field role, more\n    than one embedded counter timestamp, or ``scope`` is registered\n    in a different domain.\nWriterError\n    If the backend fails to register the counter or its schema.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_17get_counter = {"get_counter", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_17get_counter, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_16get_counter};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_17get_counter(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_v_description = 0;
  PyObject *__pyx_v_scope = 0;
  PyObject *__pyx_v_semantics = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_counter (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_dtype,&__pyx_mstate_global->__pyx_n_u_description,&__pyx_mstate_global->__pyx_n_u_scope,&__pyx_mstate_global->__pyx_n_u_semantics,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1272, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1272, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1272, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_counter", 0) < (0)) __PYX_ERR(0, 1272, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":1277
 *         dtype,
 *         *,
 *         description=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         semantics=None,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":1278
 *         *,
 *         description=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         semantics=None,
 *     ):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":1279
 *         description=None,
 *         scope=None,
 *         semantics=None,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_counter", 1, 2, 2, i); __PYX_ERR(0, 1272, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1272, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1272, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":1277
 *         dtype,
 *         *,
 *         description=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         semantics=None,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":1278
 *         *,
 *         description=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         semantics=None,
 *     ):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":1279
 *         description=None,
 *         scope=None,
 *         semantics=None,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_name = values[0];
    __pyx_v_dtype = values[1];
    __pyx_v_description = values[2];
    __pyx_v_scope = values[3];
    __pyx_v_semantics = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_counter", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1272, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_16get_counter(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_name, __pyx_v_dtype, __pyx_v_description, __pyx_v_scope, __pyx_v_semantics);

  /* "nvtx/_lib/writer.pyx":1272
 *             return schema
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_16get_counter(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, PyObject *__pyx_v_description, PyObject *__pyx_v_scope, PyObject *__pyx_v_semantics) {
  uint64_t __pyx_v_scope_id;
  PyObject *__pyx_v_schema_key = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  uint64_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_counter", 0);
  __Pyx_INCREF(__pyx_v_dtype);

  /* "nvtx/_lib/writer.pyx":1328
 *             If the backend fails to register the counter or its schema.
 *         """
 *         dtype = _normalize_counter_dtype(dtype)             # <<<<<<<<<<<<<<
 *         cdef uint64_t scope_id
 *         with self._session._lock:
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_normalize_counter_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_dtype};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1328, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF_SET(__pyx_v_dtype, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1330
 *         dtype = _normalize_counter_dtype(dtype)
 *         cdef uint64_t scope_id
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             scope_id = _resolve_scope_id(self, scope)
*/
  /*with:*/ {
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = NULL;
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1330, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1330, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":1331
 *         cdef uint64_t scope_id
 *         with self._session._lock:
 *             self._ensure_valid()             # <<<<<<<<<<<<<<
 *             scope_id = _resolve_scope_id(self, scope)
 *             schema_key = PayloadSchemaKey(
*/
          __pyx_t_1 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_self->__pyx_vtab)->_ensure_valid(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1331, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "nvtx/_lib/writer.pyx":1332
 *         with self._session._lock:
 *             self._ensure_valid()
 *             scope_id = _resolve_scope_id(self, scope)             # <<<<<<<<<<<<<<
 *             schema_key = PayloadSchemaKey(
 *                 dtype,
*/
          __pyx_t_9 = __pyx_f_4nvtx_4_lib_6writer__resolve_scope_id(__pyx_v_self, __pyx_v_scope); if (unlikely(__pyx_t_9 == ((uint64_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1332, __pyx_L7_error)
          __pyx_v_scope_id = __pyx_t_9;

          /* "nvtx/_lib/writer.pyx":1333
 *             self._ensure_valid()
 *             scope_id = _resolve_scope_id(self, scope)
 *             schema_key = PayloadSchemaKey(             # <<<<<<<<<<<<<<
 *                 dtype,
 *                 counter_group=getattr(dtype, "fields", None) is not None,
*/
          __pyx_t_2 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_PayloadSchemaKey); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1333, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_3);

          /* "nvtx/_lib/writer.pyx":1335
 *             schema_key = PayloadSchemaKey(
 *                 dtype,
 *                 counter_group=getattr(dtype, "fields", None) is not None,             # <<<<<<<<<<<<<<
 *             )
 *             return self._get_counter_cached(
*/
          __pyx_t_10 = __Pyx_GetAttr3(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_fields, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1335, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = (__pyx_t_10 != Py_None);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __pyx_t_10 = __Pyx_PyBool_FromLong(__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1335, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_4 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
            assert(__pyx_t_2);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_2);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
            __pyx_t_4 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_v_dtype};
            __pyx_t_12 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1333, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_12);
            if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_counter_group, __pyx_t_10, __pyx_t_12, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1333, __pyx_L7_error)
            __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_12);
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1333, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
          }
          __pyx_v_schema_key = __pyx_t_1;
          __pyx_t_1 = 0;

          /* "nvtx/_lib/writer.pyx":1337
 *                 counter_group=getattr(dtype, "fields", None) is not None,
 *             )
 *             return self._get_counter_cached(             # <<<<<<<<<<<<<<
 *                 _as_str(name), schema_key, _as_str(description), scope_id,
 *                 semantics,
*/
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_3 = NULL;
          __Pyx_INCREF(__pyx_v_self->_get_counter_cached);
          __pyx_t_12 = __pyx_v_self->_get_counter_cached; 

          /* "nvtx/_lib/writer.pyx":1338
 *             )
 *             return self._get_counter_cached(
 *                 _as_str(name), schema_key, _as_str(description), scope_id,             # <<<<<<<<<<<<<<
 *                 semantics,
 *             )
*/
          __pyx_t_10 = __pyx_f_4nvtx_4_lib_6writer__as_str(__pyx_v_name); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1338, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_2 = __pyx_f_4nvtx_4_lib_6writer__as_str(__pyx_v_description); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1338, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_13 = __Pyx_PyLong_From_uint64_t(__pyx_v_scope_id); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1338, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_13);

          /* "nvtx/_lib/writer.pyx":1339
 *             return self._get_counter_cached(
 *                 _as_str(name), schema_key, _as_str(description), scope_id,
 *                 semantics,             # <<<<<<<<<<<<<<
 *             )
 * 
*/
          __pyx_t_4 = 1;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_12))) {
            __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_12);
            assert(__pyx_t_3);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
            __pyx_t_4 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[6] = {__pyx_t_3, __pyx_t_10, __pyx_v_schema_key, __pyx_t_2, __pyx_t_13, __pyx_v_semantics};
            __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_4, (6-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1337, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_1);
          }
          __pyx_r = __pyx_t_1;
          __pyx_t_1 = 0;
          goto __pyx_L11_try_return;

          /* "nvtx/_lib/writer.pyx":1330
 *         dtype = _normalize_counter_dtype(dtype)
 *         cdef uint64_t scope_id
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             self._ensure_valid()
 *             scope_id = _resolve_scope_id(self, scope)
*/
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_12, &__pyx_t_13) < 0) __PYX_ERR(0, 1330, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_1);
          __Pyx_XGOTREF(__pyx_t_12);
          __Pyx_XGOTREF(__pyx_t_13);
          __pyx_t_2 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_12, __pyx_t_13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1330, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1330, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          if (__pyx_t_11 < (0)) __PYX_ERR(0, 1330, __pyx_L9_except_error)
          __pyx_t_15 = (!__pyx_t_11);
          if (unlikely(__pyx_t_15)) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_12, __pyx_t_13);
            __pyx_t_1 = 0;  __pyx_t_12 = 0;  __pyx_t_13 = 0; 
            __PYX_ERR(0, 1330, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_5) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1330, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_5) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1330, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L16;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L1_error;
    __pyx_L16:;
  }

  /* "nvtx/_lib/writer.pyx":1272
 *             return schema
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.get_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_schema_key);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1342
 *             )
 * 
 *     def _register_counter(             # <<<<<<<<<<<<<<
 *         self, name, schema_key, description, uint64_t scope_id, semantics
 *     ):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_19_register_counter(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_18_register_counter, "Domain._register_counter(self, name, schema_key, description, uint64_t scope_id, semantics)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_19_register_counter = {"_register_counter", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_19_register_counter, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_18_register_counter};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_19_register_counter(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_schema_key = 0;
  PyObject *__pyx_v_description = 0;
  uint64_t __pyx_v_scope_id;
  PyObject *__pyx_v_semantics = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_register_counter (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_schema_key,&__pyx_mstate_global->__pyx_n_u_description,&__pyx_mstate_global->__pyx_n_u_scope_id,&__pyx_mstate_global->__pyx_n_u_semantics,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1342, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1342, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1342, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1342, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1342, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1342, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_register_counter", 0) < (0)) __PYX_ERR(0, 1342, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 5; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_register_counter", 1, 5, 5, i); __PYX_ERR(0, 1342, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 5)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1342, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1342, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1342, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1342, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1342, __pyx_L3_error)
    }
    __pyx_v_name = values[0];
    __pyx_v_schema_key = values[1];
    __pyx_v_description = values[2];
    __pyx_v_scope_id = __Pyx_PyLong_As_uint64_t(values[3]); if (unlikely((__pyx_v_scope_id == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1343, __pyx_L3_error)
    __pyx_v_semantics = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_register_counter", 1, 5, 5, __pyx_nargs); __PYX_ERR(0, 1342, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._register_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_18_register_counter(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v_name, __pyx_v_schema_key, __pyx_v_description, __pyx_v_scope_id, __pyx_v_semantics);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_18_register_counter(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_schema_key, PyObject *__pyx_v_description, uint64_t __pyx_v_scope_id, PyObject *__pyx_v_semantics) {
  nvtxCounterAttr_t __pyx_v_attr;
  nvtxSemanticsCounter_t __pyx_v_counter_semantics;
  uint64_t __pyx_v_schema_id;
  uint64_t __pyx_v_counter_id;
  PyObject *__pyx_v_name_bytes = 0;
  PyObject *__pyx_v_description_bytes = 0;
  char const *__pyx_v_description_ptr;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_timestamp_field = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  uint64_t __pyx_t_7;
  char const *__pyx_t_8;
  char const *__pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_register_counter", 0);

  /* "nvtx/_lib/writer.pyx":1348
 *         cdef nvtxSemanticsCounter_t counter_semantics
 *         cdef uint64_t schema_id
 *         cdef uint64_t counter_id = 0             # <<<<<<<<<<<<<<
 *         cdef bytes name_bytes = _as_bytes(name)
 *         cdef bytes description_bytes
*/
  __pyx_v_counter_id = 0;

  /* "nvtx/_lib/writer.pyx":1349
 *         cdef uint64_t schema_id
 *         cdef uint64_t counter_id = 0
 *         cdef bytes name_bytes = _as_bytes(name)             # <<<<<<<<<<<<<<
 *         cdef bytes description_bytes
 *         cdef const char* description_ptr = NULL
*/
  __pyx_t_1 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_name_bytes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1351
 *         cdef bytes name_bytes = _as_bytes(name)
 *         cdef bytes description_bytes
 *         cdef const char* description_ptr = NULL             # <<<<<<<<<<<<<<
 *         cdef nvtxwResultCode_t rc
 * 
*/
  __pyx_v_description_ptr = NULL;

  /* "nvtx/_lib/writer.pyx":1354
 *         cdef nvtxwResultCode_t rc
 * 
 *         dtype = schema_key.dtype             # <<<<<<<<<<<<<<
 *         timestamp_field = _validate_counter_layout(dtype)
 *         if dtype is int:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_schema_key, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_dtype = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1355
 * 
 *         dtype = schema_key.dtype
 *         timestamp_field = _validate_counter_layout(dtype)             # <<<<<<<<<<<<<<
 *         if dtype is int:
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_validate_counter_layout); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1355, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_dtype};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_timestamp_field = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1356
 *         dtype = schema_key.dtype
 *         timestamp_field = _validate_counter_layout(dtype)
 *         if dtype is int:             # <<<<<<<<<<<<<<
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
 *         elif dtype is float:
*/
  __pyx_t_5 = (__pyx_v_dtype == ((PyObject *)(&PyLong_Type)));
  if (__pyx_t_5) {

    /* "nvtx/_lib/writer.pyx":1357
 *         timestamp_field = _validate_counter_layout(dtype)
 *         if dtype is int:
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64             # <<<<<<<<<<<<<<
 *         elif dtype is float:
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64
*/
    __pyx_v_schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64;

    /* "nvtx/_lib/writer.pyx":1356
 *         dtype = schema_key.dtype
 *         timestamp_field = _validate_counter_layout(dtype)
 *         if dtype is int:             # <<<<<<<<<<<<<<
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
 *         elif dtype is float:
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/writer.pyx":1358
 *         if dtype is int:
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
 *         elif dtype is float:             # <<<<<<<<<<<<<<
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64
 *         elif (
*/
  __pyx_t_5 = (__pyx_v_dtype == ((PyObject *)(&PyFloat_Type)));
  if (__pyx_t_5) {

    /* "nvtx/_lib/writer.pyx":1359
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
 *         elif dtype is float:
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64             # <<<<<<<<<<<<<<
 *         elif (
 *             dtype.fields is None
*/
    __pyx_v_schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64;

    /* "nvtx/_lib/writer.pyx":1358
 *         if dtype is int:
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
 *         elif dtype is float:             # <<<<<<<<<<<<<<
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64
 *         elif (
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/writer.pyx":1361
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64
 *         elif (
 *             dtype.fields is None             # <<<<<<<<<<<<<<
 *             and dtype.subdtype is None
 *             and dtype.kind == "i"
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }

  /* "nvtx/_lib/writer.pyx":1362
 *         elif (
 *             dtype.fields is None
 *             and dtype.subdtype is None             # <<<<<<<<<<<<<<
 *             and dtype.kind == "i"
 *             and dtype.itemsize == 8
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }

  /* "nvtx/_lib/writer.pyx":1363
 *             dtype.fields is None
 *             and dtype.subdtype is None
 *             and dtype.kind == "i"             # <<<<<<<<<<<<<<
 *             and dtype.itemsize == 8
 *         ):
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1363, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_i, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1363, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }

  /* "nvtx/_lib/writer.pyx":1364
 *             and dtype.subdtype is None
 *             and dtype.kind == "i"
 *             and dtype.itemsize == 8             # <<<<<<<<<<<<<<
 *         ):
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_8, 8, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1364, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;

  /* "nvtx/_lib/writer.pyx":1360
 *         elif dtype is float:
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64
 *         elif (             # <<<<<<<<<<<<<<
 *             dtype.fields is None
 *             and dtype.subdtype is None
*/
  if (__pyx_t_5) {

    /* "nvtx/_lib/writer.pyx":1366
 *             and dtype.itemsize == 8
 *         ):
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64             # <<<<<<<<<<<<<<
 *         elif (
 *             dtype.fields is None
*/
    __pyx_v_schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64;

    /* "nvtx/_lib/writer.pyx":1360
 *         elif dtype is float:
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64
 *         elif (             # <<<<<<<<<<<<<<
 *             dtype.fields is None
 *             and dtype.subdtype is None
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/writer.pyx":1368
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
 *         elif (
 *             dtype.fields is None             # <<<<<<<<<<<<<<
 *             and dtype.subdtype is None
 *             and dtype.kind == "f"
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L8_bool_binop_done;
  }

  /* "nvtx/_lib/writer.pyx":1369
 *         elif (
 *             dtype.fields is None
 *             and dtype.subdtype is None             # <<<<<<<<<<<<<<
 *             and dtype.kind == "f"
 *             and dtype.itemsize == 8
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L8_bool_binop_done;
  }

  /* "nvtx/_lib/writer.pyx":1370
 *             dtype.fields is None
 *             and dtype.subdtype is None
 *             and dtype.kind == "f"             # <<<<<<<<<<<<<<
 *             and dtype.itemsize == 8
 *         ):
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_kind); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyUnicode_Equals(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_f, Py_EQ)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1370, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L8_bool_binop_done;
  }

  /* "nvtx/_lib/writer.pyx":1371
 *             and dtype.subdtype is None
 *             and dtype.kind == "f"
 *             and dtype.itemsize == 8             # <<<<<<<<<<<<<<
 *         ):
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1371, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_8, 8, 0)); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1371, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __pyx_t_6;
  __pyx_L8_bool_binop_done:;

  /* "nvtx/_lib/writer.pyx":1367
 *         ):
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
 *         elif (             # <<<<<<<<<<<<<<
 *             dtype.fields is None
 *             and dtype.subdtype is None
*/
  if (__pyx_t_5) {

    /* "nvtx/_lib/writer.pyx":1373
 *             and dtype.itemsize == 8
 *         ):
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64             # <<<<<<<<<<<<<<
 *         else:
 *             schema_id = self._schema_registrar._get_numpy_dtype_schema(
*/
    __pyx_v_schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64;

    /* "nvtx/_lib/writer.pyx":1367
 *         ):
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_INT64
 *         elif (             # <<<<<<<<<<<<<<
 *             dtype.fields is None
 *             and dtype.subdtype is None
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/writer.pyx":1375
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64
 *         else:
 *             schema_id = self._schema_registrar._get_numpy_dtype_schema(             # <<<<<<<<<<<<<<
 *                 schema_key
 *             )
*/
  /*else*/ {
    __pyx_t_3 = ((PyObject *)__pyx_v_self->_schema_registrar);
    __Pyx_INCREF(__pyx_t_3);

    /* "nvtx/_lib/writer.pyx":1376
 *         else:
 *             schema_id = self._schema_registrar._get_numpy_dtype_schema(
 *                 schema_key             # <<<<<<<<<<<<<<
 *             )
 * 
*/
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_schema_key};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1375, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }

    /* "nvtx/_lib/writer.pyx":1375
 *             schema_id = NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64
 *         else:
 *             schema_id = self._schema_registrar._get_numpy_dtype_schema(             # <<<<<<<<<<<<<<
 *                 schema_key
 *             )
*/
    __pyx_t_7 = __Pyx_PyLong_As_uint64_t(__pyx_t_1); if (unlikely((__pyx_t_7 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1375, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_schema_id = __pyx_t_7;
  }
  __pyx_L3:;

  /* "nvtx/_lib/writer.pyx":1379
 *             )
 * 
 *         if description is not None:             # <<<<<<<<<<<<<<
 *             description_bytes = _as_bytes(description)
 *             description_ptr = description_bytes
*/
  __pyx_t_5 = (__pyx_v_description != Py_None);
  if (__pyx_t_5) {

    /* "nvtx/_lib/writer.pyx":1380
 * 
 *         if description is not None:
 *             description_bytes = _as_bytes(description)             # <<<<<<<<<<<<<<
 *             description_ptr = description_bytes
 * 
*/
    __pyx_t_1 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_description); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_description_bytes = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/writer.pyx":1381
 *         if description is not None:
 *             description_bytes = _as_bytes(description)
 *             description_ptr = description_bytes             # <<<<<<<<<<<<<<
 * 
 *         attr.structSize = sizeof(nvtxCounterAttr_t)
*/
    if (unlikely(__pyx_v_description_bytes == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1381, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_PyBytes_AsString(__pyx_v_description_bytes); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 1381, __pyx_L1_error)
    __pyx_v_description_ptr = __pyx_t_8;

    /* "nvtx/_lib/writer.pyx":1379
 *             )
 * 
 *         if description is not None:             # <<<<<<<<<<<<<<
 *             description_bytes = _as_bytes(description)
 *             description_ptr = description_bytes
*/
  }

  /* "nvtx/_lib/writer.pyx":1383
 *             description_ptr = description_bytes
 * 
 *         attr.structSize = sizeof(nvtxCounterAttr_t)             # <<<<<<<<<<<<<<
 *         attr.schemaId = schema_id
 *         attr.name = name_bytes
*/
  __pyx_v_attr.structSize = (sizeof(nvtxCounterAttr_t));

  /* "nvtx/_lib/writer.pyx":1384
 * 
 *         attr.structSize = sizeof(nvtxCounterAttr_t)
 *         attr.schemaId = schema_id             # <<<<<<<<<<<<<<
 *         attr.name = name_bytes
 *         attr.description = description_ptr
*/
  __pyx_v_attr.schemaId = __pyx_v_schema_id;

  /* "nvtx/_lib/writer.pyx":1385
 *         attr.structSize = sizeof(nvtxCounterAttr_t)
 *         attr.schemaId = schema_id
 *         attr.name = name_bytes             # <<<<<<<<<<<<<<
 *         attr.description = description_ptr
 *         attr.scopeId = scope_id
*/
  if (unlikely(__pyx_v_name_bytes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1385, __pyx_L1_error)
  }
  __pyx_t_9 = __Pyx_PyBytes_AsString(__pyx_v_name_bytes); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(0, 1385, __pyx_L1_error)
  __pyx_v_attr.name = __pyx_t_9;

  /* "nvtx/_lib/writer.pyx":1386
 *         attr.schemaId = schema_id
 *         attr.name = name_bytes
 *         attr.description = description_ptr             # <<<<<<<<<<<<<<
 *         attr.scopeId = scope_id
 *         attr.semantics = NULL
*/
  __pyx_v_attr.description = __pyx_v_description_ptr;

  /* "nvtx/_lib/writer.pyx":1387
 *         attr.name = name_bytes
 *         attr.description = description_ptr
 *         attr.scopeId = scope_id             # <<<<<<<<<<<<<<
 *         attr.semantics = NULL
 *         if semantics is not None:
*/
  __pyx_v_attr.scopeId = __pyx_v_scope_id;

  /* "nvtx/_lib/writer.pyx":1388
 *         attr.description = description_ptr
 *         attr.scopeId = scope_id
 *         attr.semantics = NULL             # <<<<<<<<<<<<<<
 *         if semantics is not None:
 *             _fill_counter_semantics(&counter_semantics, semantics)
*/
  __pyx_v_attr.semantics = NULL;

  /* "nvtx/_lib/writer.pyx":1389
 *         attr.scopeId = scope_id
 *         attr.semantics = NULL
 *         if semantics is not None:             # <<<<<<<<<<<<<<
 *             _fill_counter_semantics(&counter_semantics, semantics)
 *             attr.semantics = &counter_semantics.header
*/
  __pyx_t_5 = (__pyx_v_semantics != Py_None);
  if (__pyx_t_5) {

    /* "nvtx/_lib/writer.pyx":1390
 *         attr.semantics = NULL
 *         if semantics is not None:
 *             _fill_counter_semantics(&counter_semantics, semantics)             # <<<<<<<<<<<<<<
 *             attr.semantics = &counter_semantics.header
 *         attr.counterId = NVTX_COUNTER_ID_NONE
*/
    __pyx_f_4nvtx_4_lib_8counters__fill_counter_semantics((&__pyx_v_counter_semantics), __pyx_v_semantics); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1390, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1391
 *         if semantics is not None:
 *             _fill_counter_semantics(&counter_semantics, semantics)
 *             attr.semantics = &counter_semantics.header             # <<<<<<<<<<<<<<
 *         attr.counterId = NVTX_COUNTER_ID_NONE
 *         rc = self._backend._iface.CounterRegister(
*/
    __pyx_v_attr.semantics = (&__pyx_v_counter_semantics.header);

    /* "nvtx/_lib/writer.pyx":1389
 *         attr.scopeId = scope_id
 *         attr.semantics = NULL
 *         if semantics is not None:             # <<<<<<<<<<<<<<
 *             _fill_counter_semantics(&counter_semantics, semantics)
 *             attr.semantics = &counter_semantics.header
*/
  }

  /* "nvtx/_lib/writer.pyx":1392
 *             _fill_counter_semantics(&counter_semantics, semantics)
 *             attr.semantics = &counter_semantics.header
 *         attr.counterId = NVTX_COUNTER_ID_NONE             # <<<<<<<<<<<<<<
 *         rc = self._backend._iface.CounterRegister(
 *             self._handle, &attr, &counter_id)
*/
  __pyx_v_attr.counterId = NVTX_COUNTER_ID_NONE;

  /* "nvtx/_lib/writer.pyx":1393
 *             attr.semantics = &counter_semantics.header
 *         attr.counterId = NVTX_COUNTER_ID_NONE
 *         rc = self._backend._iface.CounterRegister(             # <<<<<<<<<<<<<<
 *             self._handle, &attr, &counter_id)
 *         _check(rc, "CounterRegister")
*/
  __pyx_v_rc = __pyx_v_self->_backend->_iface->CounterRegister(__pyx_v_self->_handle, (&__pyx_v_attr), (&__pyx_v_counter_id));

  /* "nvtx/_lib/writer.pyx":1395
 *         rc = self._backend._iface.CounterRegister(
 *             self._handle, &attr, &counter_id)
 *         _check(rc, "CounterRegister")             # <<<<<<<<<<<<<<
 * 
 *         return Counter(
*/
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_CounterRegister); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1395, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1397
 *         _check(rc, "CounterRegister")
 * 
 *         return Counter(             # <<<<<<<<<<<<<<
 *             self, name, dtype, description, semantics,
 *             scope_id, counter_id, schema_id, timestamp_field,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = NULL;

  /* "nvtx/_lib/writer.pyx":1399
 *         return Counter(
 *             self, name, dtype, description, semantics,
 *             scope_id, counter_id, schema_id, timestamp_field,             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_From_uint64_t(__pyx_v_scope_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyLong_From_uint64_t(__pyx_v_counter_id); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyLong_From_uint64_t(__pyx_v_schema_id); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[10] = {__pyx_t_3, ((PyObject *)__pyx_v_self), __pyx_v_name, __pyx_v_dtype, __pyx_v_description, __pyx_v_semantics, __pyx_t_2, __pyx_t_10, __pyx_t_11, __pyx_v_timestamp_field};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Counter, __pyx_callargs+__pyx_t_4, (10-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1397, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":1342
 *             )
 * 
 *     def _register_counter(             # <<<<<<<<<<<<<<
 *         self, name, schema_key, description, uint64_t scope_id, semantics
 *     ):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("nvtx._lib.writer.Domain._register_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name_bytes);
  __Pyx_XDECREF(__pyx_v_description_bytes);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_timestamp_field);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_20__reduce_cython__, "Domain.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_21__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_21__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_20__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_20__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Domain_22__setstate_cython__, "Domain.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Domain_23__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_23__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_22__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Domain_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Domain_22__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Domain_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Domain.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1411
 *     """
 * 
 *     def __cinit__(             # <<<<<<<<<<<<<<
 *         self,
 *         Session session,
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_6writer_6Stream_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_6writer_6Stream_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_session = 0;
  PyObject *__pyx_v_name = 0;
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain = 0;
  PyObject *__pyx_v_scope = 0;
  PyObject *__pyx_v_time_domain_id = 0;
  PyObject *__pyx_v_interleaving = 0;
  PyObject *__pyx_v_ordering = 0;
  PyObject *__pyx_v_skid = 0;
  PyObject *__pyx_v_skid_amount = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[9] = {0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_session,&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_domain,&__pyx_mstate_global->__pyx_n_u_scope,&__pyx_mstate_global->__pyx_n_u_time_domain_id,&__pyx_mstate_global->__pyx_n_u_interleaving,&__pyx_mstate_global->__pyx_n_u_ordering,&__pyx_mstate_global->__pyx_n_u_skid,&__pyx_mstate_global->__pyx_n_u_skid_amount,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1411, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  9:
        values[8] = __Pyx_ArgRef_VARARGS(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 1411, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_VARARGS(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 1411, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1411, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1411, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1411, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1411, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1411, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1411, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1411, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__cinit__", 0) < (0)) __PYX_ERR(0, 1411, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 9; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 9, 9, i); __PYX_ERR(0, 1411, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 9)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1411, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1411, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1411, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1411, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1411, __pyx_L3_error)
      values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1411, __pyx_L3_error)
      values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1411, __pyx_L3_error)
      values[7] = __Pyx_ArgRef_VARARGS(__pyx_args, 7);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 1411, __pyx_L3_error)
      values[8] = __Pyx_ArgRef_VARARGS(__pyx_args, 8);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 1411, __pyx_L3_error)
    }
    __pyx_v_session = ((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)values[0]);
    __pyx_v_name = values[1];
    __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)values[2]);
    __pyx_v_scope = values[3];
    __pyx_v_time_domain_id = values[4];
    __pyx_v_interleaving = values[5];
    __pyx_v_ordering = values[6];
    __pyx_v_skid = values[7];
    __pyx_v_skid_amount = values[8];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 9, 9, __pyx_nargs); __PYX_ERR(0, 1411, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_session), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Session, 1, "session", 0))) __PYX_ERR(0, 1413, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, 1, "domain", 0))) __PYX_ERR(0, 1415, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream___cinit__(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_session, __pyx_v_name, __pyx_v_domain, __pyx_v_scope, __pyx_v_time_domain_id, __pyx_v_interleaving, __pyx_v_ordering, __pyx_v_skid, __pyx_v_skid_amount);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_6writer_6Stream___cinit__(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Session *__pyx_v_session, PyObject *__pyx_v_name, struct __pyx_obj_4nvtx_4_lib_6writer_Domain *__pyx_v_domain, PyObject *__pyx_v_scope, PyObject *__pyx_v_time_domain_id, PyObject *__pyx_v_interleaving, PyObject *__pyx_v_ordering, PyObject *__pyx_v_skid, PyObject *__pyx_v_skid_amount) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5[3];
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  uint64_t __pyx_t_8;
  int16_t __pyx_t_9;
  int32_t __pyx_t_10;
  int64_t __pyx_t_11;
  nvtxwInterface_v2_t const *__pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);
  __Pyx_INCREF(__pyx_v_time_domain_id);

  /* "nvtx/_lib/writer.pyx":1423
 *         skid_amount,
 *     ):
 *         if domain._session is not session:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 f"domain '{domain}' is not registered in this session. "
*/
  __pyx_t_1 = (__pyx_v_domain->_session != __pyx_v_session);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1424
 *     ):
 *         if domain._session is not session:
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 f"domain '{domain}' is not registered in this session. "
 *                 "Do not construct streams directly; "
*/
    __pyx_t_3 = NULL;

    /* "nvtx/_lib/writer.pyx":1425
 *         if domain._session is not session:
 *             raise RuntimeError(
 *                 f"domain '{domain}' is not registered in this session. "             # <<<<<<<<<<<<<<
 *                 "Do not construct streams directly; "
 *                 "use Session.create_stream()."
*/
    __pyx_t_4 = __Pyx_PyObject_FormatSimple(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5[0] = __pyx_mstate_global->__pyx_kp_u_domain_2;
    __pyx_t_5[1] = __pyx_t_4;
    __pyx_t_5[2] = __pyx_mstate_global->__pyx_kp_u_is_not_registered_in_this_sessi;
    __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_5, 3, 8 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4) + 100, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4));
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_7 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_6};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1424, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1424, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1423
 *         skid_amount,
 *     ):
 *         if domain._session is not session:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 f"domain '{domain}' is not registered in this session. "
*/
  }

  /* "nvtx/_lib/writer.pyx":1429
 *                 "use Session.create_stream()."
 *             )
 *         self._session = session             # <<<<<<<<<<<<<<
 *         self._backend = session._backend
 *         self._domain = domain
*/
  __Pyx_INCREF((PyObject *)__pyx_v_session);
  __Pyx_GIVEREF((PyObject *)__pyx_v_session);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_session);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_session);
  __pyx_v_self->_session = __pyx_v_session;

  /* "nvtx/_lib/writer.pyx":1430
 *             )
 *         self._session = session
 *         self._backend = session._backend             # <<<<<<<<<<<<<<
 *         self._domain = domain
 *         self._name = _as_bytes(name)
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_session->_backend);
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_backend);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_backend);
  __pyx_v_self->_backend = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":1431
 *         self._session = session
 *         self._backend = session._backend
 *         self._domain = domain             # <<<<<<<<<<<<<<
 *         self._name = _as_bytes(name)
 *         self._scope_id = _resolve_stream_scope_id(domain, scope)
*/
  __Pyx_INCREF((PyObject *)__pyx_v_domain);
  __Pyx_GIVEREF((PyObject *)__pyx_v_domain);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_domain);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_domain);
  __pyx_v_self->_domain = __pyx_v_domain;

  /* "nvtx/_lib/writer.pyx":1432
 *         self._backend = session._backend
 *         self._domain = domain
 *         self._name = _as_bytes(name)             # <<<<<<<<<<<<<<
 *         self._scope_id = _resolve_stream_scope_id(domain, scope)
 *         if isinstance(time_domain_id, TimestampType):
*/
  __pyx_t_2 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_name);
  __Pyx_DECREF(__pyx_v_self->_name);
  __pyx_v_self->_name = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":1433
 *         self._domain = domain
 *         self._name = _as_bytes(name)
 *         self._scope_id = _resolve_stream_scope_id(domain, scope)             # <<<<<<<<<<<<<<
 *         if isinstance(time_domain_id, TimestampType):
 *             time_domain_id = time_domain_id.value
*/
  __pyx_t_8 = __pyx_f_4nvtx_4_lib_6writer__resolve_stream_scope_id(__pyx_v_domain, __pyx_v_scope); if (unlikely(__pyx_t_8 == ((uint64_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1433, __pyx_L1_error)
  __pyx_v_self->_scope_id = __pyx_t_8;

  /* "nvtx/_lib/writer.pyx":1434
 *         self._name = _as_bytes(name)
 *         self._scope_id = _resolve_stream_scope_id(domain, scope)
 *         if isinstance(time_domain_id, TimestampType):             # <<<<<<<<<<<<<<
 *             time_domain_id = time_domain_id.value
 *         self._time_domain_id = time_domain_id
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_TimestampType); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_IsInstance(__pyx_v_time_domain_id, __pyx_t_2); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 1434, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":1435
 *         self._scope_id = _resolve_stream_scope_id(domain, scope)
 *         if isinstance(time_domain_id, TimestampType):
 *             time_domain_id = time_domain_id.value             # <<<<<<<<<<<<<<
 *         self._time_domain_id = time_domain_id
 *         self._order_interleaving = interleaving.value
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_time_domain_id, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_time_domain_id, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "nvtx/_lib/writer.pyx":1434
 *         self._name = _as_bytes(name)
 *         self._scope_id = _resolve_stream_scope_id(domain, scope)
 *         if isinstance(time_domain_id, TimestampType):             # <<<<<<<<<<<<<<
 *             time_domain_id = time_domain_id.value
 *         self._time_domain_id = time_domain_id
*/
  }

  /* "nvtx/_lib/writer.pyx":1436
 *         if isinstance(time_domain_id, TimestampType):
 *             time_domain_id = time_domain_id.value
 *         self._time_domain_id = time_domain_id             # <<<<<<<<<<<<<<
 *         self._order_interleaving = interleaving.value
 *         self._ordering_type = ordering.value
*/
  __pyx_t_8 = __Pyx_PyLong_As_uint64_t(__pyx_v_time_domain_id); if (unlikely((__pyx_t_8 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1436, __pyx_L1_error)
  __pyx_v_self->_time_domain_id = __pyx_t_8;

  /* "nvtx/_lib/writer.pyx":1437
 *             time_domain_id = time_domain_id.value
 *         self._time_domain_id = time_domain_id
 *         self._order_interleaving = interleaving.value             # <<<<<<<<<<<<<<
 *         self._ordering_type = ordering.value
 *         self._ordering_skid = skid.value
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_interleaving, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = __Pyx_PyLong_As_int16_t(__pyx_t_2); if (unlikely((__pyx_t_9 == ((int16_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1437, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->_order_interleaving = __pyx_t_9;

  /* "nvtx/_lib/writer.pyx":1438
 *         self._time_domain_id = time_domain_id
 *         self._order_interleaving = interleaving.value
 *         self._ordering_type = ordering.value             # <<<<<<<<<<<<<<
 *         self._ordering_skid = skid.value
 *         self._ordering_skid_amount = skid_amount
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_ordering, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = __Pyx_PyLong_As_int16_t(__pyx_t_2); if (unlikely((__pyx_t_9 == ((int16_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1438, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->_ordering_type = __pyx_t_9;

  /* "nvtx/_lib/writer.pyx":1439
 *         self._order_interleaving = interleaving.value
 *         self._ordering_type = ordering.value
 *         self._ordering_skid = skid.value             # <<<<<<<<<<<<<<
 *         self._ordering_skid_amount = skid_amount
 *         # ``iface`` is constant for the stream's lifetime; the stream handle is
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_skid, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyLong_As_int32_t(__pyx_t_2); if (unlikely((__pyx_t_10 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1439, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->_ordering_skid = __pyx_t_10;

  /* "nvtx/_lib/writer.pyx":1440
 *         self._ordering_type = ordering.value
 *         self._ordering_skid = skid.value
 *         self._ordering_skid_amount = skid_amount             # <<<<<<<<<<<<<<
 *         # ``iface`` is constant for the stream's lifetime; the stream handle is
 *         # set in open()/close() and the schema IDs lazily on the first write.
*/
  __pyx_t_11 = __Pyx_PyLong_As_int64_t(__pyx_v_skid_amount); if (unlikely((__pyx_t_11 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1440, __pyx_L1_error)
  __pyx_v_self->_ordering_skid_amount = __pyx_t_11;

  /* "nvtx/_lib/writer.pyx":1443
 *         # ``iface`` is constant for the stream's lifetime; the stream handle is
 *         # set in open()/close() and the schema IDs lazily on the first write.
 *         self._writer.iface = self._backend._iface             # <<<<<<<<<<<<<<
 * 
 *     def open(self):
*/
  __pyx_t_12 = __pyx_v_self->_backend->_iface;
  __pyx_v_self->_writer.iface = __pyx_t_12;

  /* "nvtx/_lib/writer.pyx":1411
 *     """
 * 
 *     def __cinit__(             # <<<<<<<<<<<<<<
 *         self,
 *         Session session,
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_time_domain_id);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1445
 *         self._writer.iface = self._backend._iface
 * 
 *     def open(self):             # <<<<<<<<<<<<<<
 *         """
 *         Open the stream for writing.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_3open(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_2open, "Stream.open(self)\n\nOpen the stream for writing.\n\nRaises\n------\nRuntimeError\n    If the stream is already open or its domain is no longer valid.\nWriterError\n    If the backend fails to open the stream.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_3open = {"open", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_3open, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_2open};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_3open(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("open (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("open", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("open", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_2open(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_2open(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self) {
  nvtxwStreamAttributes_t __pyx_v_attr;
  char const *__pyx_v_name;
  nvtxDomainHandle_t __pyx_v_domain_handle;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  nvtxDomainHandle_t __pyx_t_10;
  char const *__pyx_t_11;
  uint64_t __pyx_t_12;
  int16_t __pyx_t_13;
  int32_t __pyx_t_14;
  int64_t __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("open", 0);

  /* "nvtx/_lib/writer.pyx":1457
 *         """
 *         cdef nvtxwStreamAttributes_t attr
 *         cdef const char* name = NULL             # <<<<<<<<<<<<<<
 *         cdef nvtxDomainHandle_t domain_handle = NULL
 *         cdef nvtxwResultCode_t rc
*/
  __pyx_v_name = NULL;

  /* "nvtx/_lib/writer.pyx":1458
 *         cdef nvtxwStreamAttributes_t attr
 *         cdef const char* name = NULL
 *         cdef nvtxDomainHandle_t domain_handle = NULL             # <<<<<<<<<<<<<<
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:
*/
  __pyx_v_domain_handle = NULL;

  /* "nvtx/_lib/writer.pyx":1460
 *         cdef nvtxDomainHandle_t domain_handle = NULL
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             if self._writer.stream != NULL:
 *                 raise RuntimeError("stream is already open")
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1460, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1460, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1460, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":1461
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:
 *             if self._writer.stream != NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("stream is already open")
 *             if self._session._handle == NULL:
*/
          __pyx_t_9 = (__pyx_v_self->_writer.stream != NULL);
          if (unlikely(__pyx_t_9)) {

            /* "nvtx/_lib/writer.pyx":1462
 *         with self._session._lock:
 *             if self._writer.stream != NULL:
 *                 raise RuntimeError("stream is already open")             # <<<<<<<<<<<<<<
 *             if self._session._handle == NULL:
 *                 raise RuntimeError("session is not active")
*/
            __pyx_t_4 = NULL;
            __pyx_t_5 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_stream_is_already_open};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1462, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_Raise(__pyx_t_2, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __PYX_ERR(0, 1462, __pyx_L7_error)

            /* "nvtx/_lib/writer.pyx":1461
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:
 *             if self._writer.stream != NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("stream is already open")
 *             if self._session._handle == NULL:
*/
          }

          /* "nvtx/_lib/writer.pyx":1463
 *             if self._writer.stream != NULL:
 *                 raise RuntimeError("stream is already open")
 *             if self._session._handle == NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("session is not active")
 *             if self._domain is not None:
*/
          __pyx_t_9 = (__pyx_v_self->_session->_handle == NULL);
          if (unlikely(__pyx_t_9)) {

            /* "nvtx/_lib/writer.pyx":1464
 *                 raise RuntimeError("stream is already open")
 *             if self._session._handle == NULL:
 *                 raise RuntimeError("session is not active")             # <<<<<<<<<<<<<<
 *             if self._domain is not None:
 *                 self._domain._ensure_valid()
*/
            __pyx_t_4 = NULL;
            __pyx_t_5 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_session_is_not_active};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1464, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_Raise(__pyx_t_2, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __PYX_ERR(0, 1464, __pyx_L7_error)

            /* "nvtx/_lib/writer.pyx":1463
 *             if self._writer.stream != NULL:
 *                 raise RuntimeError("stream is already open")
 *             if self._session._handle == NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("session is not active")
 *             if self._domain is not None:
*/
          }

          /* "nvtx/_lib/writer.pyx":1465
 *             if self._session._handle == NULL:
 *                 raise RuntimeError("session is not active")
 *             if self._domain is not None:             # <<<<<<<<<<<<<<
 *                 self._domain._ensure_valid()
 *                 domain_handle = self._domain._handle
*/
          __pyx_t_9 = (((PyObject *)__pyx_v_self->_domain) != Py_None);
          if (__pyx_t_9) {

            /* "nvtx/_lib/writer.pyx":1466
 *                 raise RuntimeError("session is not active")
 *             if self._domain is not None:
 *                 self._domain._ensure_valid()             # <<<<<<<<<<<<<<
 *                 domain_handle = self._domain._handle
 *             if self._name is not None:
*/
            __pyx_t_2 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_self->_domain->__pyx_vtab)->_ensure_valid(__pyx_v_self->_domain); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1466, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

            /* "nvtx/_lib/writer.pyx":1467
 *             if self._domain is not None:
 *                 self._domain._ensure_valid()
 *                 domain_handle = self._domain._handle             # <<<<<<<<<<<<<<
 *             if self._name is not None:
 *                 name = self._name
*/
            __pyx_t_10 = __pyx_v_self->_domain->_handle;
            __pyx_v_domain_handle = __pyx_t_10;

            /* "nvtx/_lib/writer.pyx":1465
 *             if self._session._handle == NULL:
 *                 raise RuntimeError("session is not active")
 *             if self._domain is not None:             # <<<<<<<<<<<<<<
 *                 self._domain._ensure_valid()
 *                 domain_handle = self._domain._handle
*/
          }

          /* "nvtx/_lib/writer.pyx":1468
 *                 self._domain._ensure_valid()
 *                 domain_handle = self._domain._handle
 *             if self._name is not None:             # <<<<<<<<<<<<<<
 *                 name = self._name
 *             attr.structSize = sizeof(nvtxwStreamAttributes_t)
*/
          __pyx_t_9 = (__pyx_v_self->_name != ((PyObject*)Py_None));
          if (__pyx_t_9) {

            /* "nvtx/_lib/writer.pyx":1469
 *                 domain_handle = self._domain._handle
 *             if self._name is not None:
 *                 name = self._name             # <<<<<<<<<<<<<<
 *             attr.structSize = sizeof(nvtxwStreamAttributes_t)
 *             attr.name = name
*/
            if (unlikely(__pyx_v_self->_name == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
              __PYX_ERR(0, 1469, __pyx_L7_error)
            }
            __pyx_t_11 = __Pyx_PyBytes_AsString(__pyx_v_self->_name); if (unlikely((!__pyx_t_11) && PyErr_Occurred())) __PYX_ERR(0, 1469, __pyx_L7_error)
            __pyx_v_name = __pyx_t_11;

            /* "nvtx/_lib/writer.pyx":1468
 *                 self._domain._ensure_valid()
 *                 domain_handle = self._domain._handle
 *             if self._name is not None:             # <<<<<<<<<<<<<<
 *                 name = self._name
 *             attr.structSize = sizeof(nvtxwStreamAttributes_t)
*/
          }

          /* "nvtx/_lib/writer.pyx":1470
 *             if self._name is not None:
 *                 name = self._name
 *             attr.structSize = sizeof(nvtxwStreamAttributes_t)             # <<<<<<<<<<<<<<
 *             attr.name = name
 *             attr.domain = domain_handle
*/
          __pyx_v_attr.structSize = (sizeof(nvtxwStreamAttributes_t));

          /* "nvtx/_lib/writer.pyx":1471
 *                 name = self._name
 *             attr.structSize = sizeof(nvtxwStreamAttributes_t)
 *             attr.name = name             # <<<<<<<<<<<<<<
 *             attr.domain = domain_handle
 *             attr.scopeId = self._scope_id
*/
          __pyx_v_attr.name = __pyx_v_name;

          /* "nvtx/_lib/writer.pyx":1472
 *             attr.structSize = sizeof(nvtxwStreamAttributes_t)
 *             attr.name = name
 *             attr.domain = domain_handle             # <<<<<<<<<<<<<<
 *             attr.scopeId = self._scope_id
 *             attr.timeDomainId = self._time_domain_id
*/
          __pyx_v_attr.domain = __pyx_v_domain_handle;

          /* "nvtx/_lib/writer.pyx":1473
 *             attr.name = name
 *             attr.domain = domain_handle
 *             attr.scopeId = self._scope_id             # <<<<<<<<<<<<<<
 *             attr.timeDomainId = self._time_domain_id
 *             attr.orderInterleaving = self._order_interleaving
*/
          __pyx_t_12 = __pyx_v_self->_scope_id;
          __pyx_v_attr.scopeId = __pyx_t_12;

          /* "nvtx/_lib/writer.pyx":1474
 *             attr.domain = domain_handle
 *             attr.scopeId = self._scope_id
 *             attr.timeDomainId = self._time_domain_id             # <<<<<<<<<<<<<<
 *             attr.orderInterleaving = self._order_interleaving
 *             attr.orderingType = self._ordering_type
*/
          __pyx_t_12 = __pyx_v_self->_time_domain_id;
          __pyx_v_attr.timeDomainId = __pyx_t_12;

          /* "nvtx/_lib/writer.pyx":1475
 *             attr.scopeId = self._scope_id
 *             attr.timeDomainId = self._time_domain_id
 *             attr.orderInterleaving = self._order_interleaving             # <<<<<<<<<<<<<<
 *             attr.orderingType = self._ordering_type
 *             attr.orderingSkid = self._ordering_skid
*/
          __pyx_t_13 = __pyx_v_self->_order_interleaving;
          __pyx_v_attr.orderInterleaving = __pyx_t_13;

          /* "nvtx/_lib/writer.pyx":1476
 *             attr.timeDomainId = self._time_domain_id
 *             attr.orderInterleaving = self._order_interleaving
 *             attr.orderingType = self._ordering_type             # <<<<<<<<<<<<<<
 *             attr.orderingSkid = self._ordering_skid
 *             attr.orderingSkidAmount = self._ordering_skid_amount
*/
          __pyx_t_13 = __pyx_v_self->_ordering_type;
          __pyx_v_attr.orderingType = __pyx_t_13;

          /* "nvtx/_lib/writer.pyx":1477
 *             attr.orderInterleaving = self._order_interleaving
 *             attr.orderingType = self._ordering_type
 *             attr.orderingSkid = self._ordering_skid             # <<<<<<<<<<<<<<
 *             attr.orderingSkidAmount = self._ordering_skid_amount
 *             rc = self._backend._iface.StreamOpen(
*/
          __pyx_t_14 = __pyx_v_self->_ordering_skid;
          __pyx_v_attr.orderingSkid = __pyx_t_14;

          /* "nvtx/_lib/writer.pyx":1478
 *             attr.orderingType = self._ordering_type
 *             attr.orderingSkid = self._ordering_skid
 *             attr.orderingSkidAmount = self._ordering_skid_amount             # <<<<<<<<<<<<<<
 *             rc = self._backend._iface.StreamOpen(
 *                 self._session._handle, &attr, &self._writer.stream)
*/
          __pyx_t_15 = __pyx_v_self->_ordering_skid_amount;
          __pyx_v_attr.orderingSkidAmount = __pyx_t_15;

          /* "nvtx/_lib/writer.pyx":1479
 *             attr.orderingSkid = self._ordering_skid
 *             attr.orderingSkidAmount = self._ordering_skid_amount
 *             rc = self._backend._iface.StreamOpen(             # <<<<<<<<<<<<<<
 *                 self._session._handle, &attr, &self._writer.stream)
 *             if rc != NVTXW_RESULT_SUCCESS:
*/
          __pyx_v_rc = __pyx_v_self->_backend->_iface->StreamOpen(__pyx_v_self->_session->_handle, (&__pyx_v_attr), (&__pyx_v_self->_writer.stream));

          /* "nvtx/_lib/writer.pyx":1481
 *             rc = self._backend._iface.StreamOpen(
 *                 self._session._handle, &attr, &self._writer.stream)
 *             if rc != NVTXW_RESULT_SUCCESS:             # <<<<<<<<<<<<<<
 *                 self._writer.stream = NULL
 *                 raise WriterError(rc, "StreamOpen")
*/
          __pyx_t_9 = (__pyx_v_rc != NVTXW_RESULT_SUCCESS);
          if (unlikely(__pyx_t_9)) {

            /* "nvtx/_lib/writer.pyx":1482
 *                 self._session._handle, &attr, &self._writer.stream)
 *             if rc != NVTXW_RESULT_SUCCESS:
 *                 self._writer.stream = NULL             # <<<<<<<<<<<<<<
 *                 raise WriterError(rc, "StreamOpen")
 *             self._session._open_streams.add(self)
*/
            __pyx_v_self->_writer.stream = NULL;

            /* "nvtx/_lib/writer.pyx":1483
 *             if rc != NVTXW_RESULT_SUCCESS:
 *                 self._writer.stream = NULL
 *                 raise WriterError(rc, "StreamOpen")             # <<<<<<<<<<<<<<
 *             self._session._open_streams.add(self)
 * 
*/
            __pyx_t_4 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_WriterError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1483, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_16 = __Pyx_PyLong_From_int32_t(__pyx_v_rc); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1483, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_16);
            __pyx_t_5 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_3))) {
              __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
              assert(__pyx_t_4);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_4);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
              __pyx_t_5 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_t_16, __pyx_mstate_global->__pyx_n_u_StreamOpen};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1483, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_Raise(__pyx_t_2, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __PYX_ERR(0, 1483, __pyx_L7_error)

            /* "nvtx/_lib/writer.pyx":1481
 *             rc = self._backend._iface.StreamOpen(
 *                 self._session._handle, &attr, &self._writer.stream)
 *             if rc != NVTXW_RESULT_SUCCESS:             # <<<<<<<<<<<<<<
 *                 self._writer.stream = NULL
 *                 raise WriterError(rc, "StreamOpen")
*/
          }

          /* "nvtx/_lib/writer.pyx":1484
 *                 self._writer.stream = NULL
 *                 raise WriterError(rc, "StreamOpen")
 *             self._session._open_streams.add(self)             # <<<<<<<<<<<<<<
 * 
 *     def close(self):
*/
          if (unlikely(__pyx_v_self->_session->_open_streams == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
            __PYX_ERR(0, 1484, __pyx_L7_error)
          }
          __pyx_t_17 = PySet_Add(__pyx_v_self->_session->_open_streams, ((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(0, 1484, __pyx_L7_error)

          /* "nvtx/_lib/writer.pyx":1460
 *         cdef nvtxDomainHandle_t domain_handle = NULL
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             if self._writer.stream != NULL:
 *                 raise RuntimeError("stream is already open")
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Stream.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_16) < 0) __PYX_ERR(0, 1460, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_t_16);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1460, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_18 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1460, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_18);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_18);
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
          if (__pyx_t_9 < (0)) __PYX_ERR(0, 1460, __pyx_L9_except_error)
          __pyx_t_19 = (!__pyx_t_9);
          if (unlikely(__pyx_t_19)) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_3, __pyx_t_16);
            __pyx_t_2 = 0;  __pyx_t_3 = 0;  __pyx_t_16 = 0; 
            __PYX_ERR(0, 1460, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1460, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L21;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L21:;
  }

  /* "nvtx/_lib/writer.pyx":1445
 *         self._writer.iface = self._backend._iface
 * 
 *     def open(self):             # <<<<<<<<<<<<<<
 *         """
 *         Open the stream for writing.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.open", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1486
 *             self._session._open_streams.add(self)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """
 *         Close the stream without ending its session.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_5close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_4close, "Stream.close(self)\n\nClose the stream without ending its session.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nWriterError\n    If the backend fails to close the stream.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_5close = {"close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_5close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_4close};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_5close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("close", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("close", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_4close(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_4close(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self) {
  nvtxwStreamHandle_t __pyx_v_handle;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  nvtxwStreamHandle_t __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);

  /* "nvtx/_lib/writer.pyx":1499
 *         cdef nvtxwStreamHandle_t handle
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             if self._writer.stream == NULL:
 *                 raise RuntimeError("stream is not open")
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1499, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1499, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1499, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":1500
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:
 *             if self._writer.stream == NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("stream is not open")
 *             self._backend._ensure_open()
*/
          __pyx_t_9 = (__pyx_v_self->_writer.stream == NULL);
          if (unlikely(__pyx_t_9)) {

            /* "nvtx/_lib/writer.pyx":1501
 *         with self._session._lock:
 *             if self._writer.stream == NULL:
 *                 raise RuntimeError("stream is not open")             # <<<<<<<<<<<<<<
 *             self._backend._ensure_open()
 *             handle = self._writer.stream
*/
            __pyx_t_4 = NULL;
            __pyx_t_5 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_stream_is_not_open};
              __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1501, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_Raise(__pyx_t_2, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __PYX_ERR(0, 1501, __pyx_L7_error)

            /* "nvtx/_lib/writer.pyx":1500
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:
 *             if self._writer.stream == NULL:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("stream is not open")
 *             self._backend._ensure_open()
*/
          }

          /* "nvtx/_lib/writer.pyx":1502
 *             if self._writer.stream == NULL:
 *                 raise RuntimeError("stream is not open")
 *             self._backend._ensure_open()             # <<<<<<<<<<<<<<
 *             handle = self._writer.stream
 *             self._writer.stream = NULL
*/
          ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Backend *)__pyx_v_self->_backend->__pyx_vtab)->_ensure_open(__pyx_v_self->_backend); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1502, __pyx_L7_error)

          /* "nvtx/_lib/writer.pyx":1503
 *                 raise RuntimeError("stream is not open")
 *             self._backend._ensure_open()
 *             handle = self._writer.stream             # <<<<<<<<<<<<<<
 *             self._writer.stream = NULL
 *             self._session._open_streams.discard(self)
*/
          __pyx_t_10 = __pyx_v_self->_writer.stream;
          __pyx_v_handle = __pyx_t_10;

          /* "nvtx/_lib/writer.pyx":1504
 *             self._backend._ensure_open()
 *             handle = self._writer.stream
 *             self._writer.stream = NULL             # <<<<<<<<<<<<<<
 *             self._session._open_streams.discard(self)
 *             rc = self._backend._iface.StreamClose(handle)
*/
          __pyx_v_self->_writer.stream = NULL;

          /* "nvtx/_lib/writer.pyx":1505
 *             handle = self._writer.stream
 *             self._writer.stream = NULL
 *             self._session._open_streams.discard(self)             # <<<<<<<<<<<<<<
 *             rc = self._backend._iface.StreamClose(handle)
 *             _check(rc, "StreamClose")
*/
          if (unlikely(__pyx_v_self->_session->_open_streams == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "discard");
            __PYX_ERR(0, 1505, __pyx_L7_error)
          }
          __pyx_t_11 = __Pyx_PySet_Discard(__pyx_v_self->_session->_open_streams, ((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 1505, __pyx_L7_error)

          /* "nvtx/_lib/writer.pyx":1506
 *             self._writer.stream = NULL
 *             self._session._open_streams.discard(self)
 *             rc = self._backend._iface.StreamClose(handle)             # <<<<<<<<<<<<<<
 *             _check(rc, "StreamClose")
 * 
*/
          __pyx_v_rc = __pyx_v_self->_backend->_iface->StreamClose(__pyx_v_handle);

          /* "nvtx/_lib/writer.pyx":1507
 *             self._session._open_streams.discard(self)
 *             rc = self._backend._iface.StreamClose(handle)
 *             _check(rc, "StreamClose")             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
*/
          __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_StreamClose); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1507, __pyx_L7_error)

          /* "nvtx/_lib/writer.pyx":1499
 *         cdef nvtxwStreamHandle_t handle
 *         cdef nvtxwResultCode_t rc
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             if self._writer.stream == NULL:
 *                 raise RuntimeError("stream is not open")
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Stream.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 1499, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_3);
          __pyx_t_12 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1499, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_12, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1499, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (__pyx_t_9 < (0)) __PYX_ERR(0, 1499, __pyx_L9_except_error)
          __pyx_t_14 = (!__pyx_t_9);
          if (unlikely(__pyx_t_14)) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_4, __pyx_t_3);
            __pyx_t_2 = 0;  __pyx_t_4 = 0;  __pyx_t_3 = 0; 
            __PYX_ERR(0, 1499, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1499, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "nvtx/_lib/writer.pyx":1486
 *             self._session._open_streams.add(self)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """
 *         Close the stream without ending its session.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1509
 *             _check(rc, "StreamClose")
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.open()
 *         return self
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_7__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_6__enter__, "Stream.__enter__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_7__enter__ = {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_7__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_6__enter__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_7__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__enter__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__enter__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_6__enter__(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_6__enter__(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "nvtx/_lib/writer.pyx":1510
 * 
 *     def __enter__(self):
 *         self.open()             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_open, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1511
 *     def __enter__(self):
 *         self.open()
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, *_):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":1509
 *             _check(rc, "StreamClose")
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.open()
 *         return self
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.__enter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1513
 *         return self
 * 
 *     def __exit__(self, *_):             # <<<<<<<<<<<<<<
 *         with self._session._lock:
 *             if self._writer.stream != NULL:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_9__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_8__exit__, "Stream.__exit__(self, *_)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_9__exit__ = {"__exit__", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_9__exit__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_8__exit__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_9__exit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v__ = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__exit__", __pyx_kwds); return NULL;}
  __Pyx_INCREF(__pyx_args);
  __pyx_v__ = __pyx_args;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_8__exit__(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v__);

  /* function exit code */
  __Pyx_DECREF(__pyx_v__);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_8__exit__(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "nvtx/_lib/writer.pyx":1514
 * 
 *     def __exit__(self, *_):
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             if self._writer.stream != NULL:
 *                 self.close()
*/
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1514, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_session->_lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1514, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1514, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "nvtx/_lib/writer.pyx":1515
 *     def __exit__(self, *_):
 *         with self._session._lock:
 *             if self._writer.stream != NULL:             # <<<<<<<<<<<<<<
 *                 self.close()
 * 
*/
          __pyx_t_9 = (__pyx_v_self->_writer.stream != NULL);
          if (__pyx_t_9) {

            /* "nvtx/_lib/writer.pyx":1516
 *         with self._session._lock:
 *             if self._writer.stream != NULL:
 *                 self.close()             # <<<<<<<<<<<<<<
 * 
 *     cdef void _ensure_writable(self) except *:
*/
            __pyx_t_4 = ((PyObject *)__pyx_v_self);
            __Pyx_INCREF(__pyx_t_4);
            __pyx_t_5 = 0;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
              __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_close, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1516, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_2);
            }
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

            /* "nvtx/_lib/writer.pyx":1515
 *     def __exit__(self, *_):
 *         with self._session._lock:
 *             if self._writer.stream != NULL:             # <<<<<<<<<<<<<<
 *                 self.close()
 * 
*/
          }

          /* "nvtx/_lib/writer.pyx":1514
 * 
 *     def __exit__(self, *_):
 *         with self._session._lock:             # <<<<<<<<<<<<<<
 *             if self._writer.stream != NULL:
 *                 self.close()
*/
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("nvtx._lib.writer.Stream.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 1514, __pyx_L9_except_error)
          __Pyx_XGOTREF(__pyx_t_2);
          __Pyx_XGOTREF(__pyx_t_4);
          __Pyx_XGOTREF(__pyx_t_3);
          __pyx_t_10 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1514, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1514, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (__pyx_t_9 < (0)) __PYX_ERR(0, 1514, __pyx_L9_except_error)
          __pyx_t_12 = (!__pyx_t_9);
          if (unlikely(__pyx_t_12)) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_4, __pyx_t_3);
            __pyx_t_2 = 0;  __pyx_t_4 = 0;  __pyx_t_3 = 0; 
            __PYX_ERR(0, 1514, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_mstate_global->__pyx_tuple[0], NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1514, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "nvtx/_lib/writer.pyx":1513
 *         return self
 * 
 *     def __exit__(self, *_):             # <<<<<<<<<<<<<<
 *         with self._session._lock:
 *             if self._writer.stream != NULL:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1518
 *                 self.close()
 * 
 *     cdef void _ensure_writable(self) except *:             # <<<<<<<<<<<<<<
 *         if self._writer.stream == NULL:
 *             raise RuntimeError("stream is not open")
*/

static void __pyx_f_4nvtx_4_lib_6writer_6Stream__ensure_writable(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_ensure_writable", 0);

  /* "nvtx/_lib/writer.pyx":1519
 * 
 *     cdef void _ensure_writable(self) except *:
 *         if self._writer.stream == NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("stream is not open")
 *         self._backend._ensure_open()
*/
  __pyx_t_1 = (__pyx_v_self->_writer.stream == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1520
 *     cdef void _ensure_writable(self) except *:
 *         if self._writer.stream == NULL:
 *             raise RuntimeError("stream is not open")             # <<<<<<<<<<<<<<
 *         self._backend._ensure_open()
 * 
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_stream_is_not_open};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1520, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1520, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1519
 * 
 *     cdef void _ensure_writable(self) except *:
 *         if self._writer.stream == NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("stream is not open")
 *         self._backend._ensure_open()
*/
  }

  /* "nvtx/_lib/writer.pyx":1521
 *         if self._writer.stream == NULL:
 *             raise RuntimeError("stream is not open")
 *         self._backend._ensure_open()             # <<<<<<<<<<<<<<
 * 
 *     cdef void _ensure_counter(self, Counter counter) except *:
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Backend *)__pyx_v_self->_backend->__pyx_vtab)->_ensure_open(__pyx_v_self->_backend); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1521, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1518
 *                 self.close()
 * 
 *     cdef void _ensure_writable(self) except *:             # <<<<<<<<<<<<<<
 *         if self._writer.stream == NULL:
 *             raise RuntimeError("stream is not open")
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream._ensure_writable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "nvtx/_lib/writer.pyx":1523
 *         self._backend._ensure_open()
 * 
 *     cdef void _ensure_counter(self, Counter counter) except *:             # <<<<<<<<<<<<<<
 *         if counter._domain is not self._domain:
 *             raise ValueError(
*/

static void __pyx_f_4nvtx_4_lib_6writer_6Stream__ensure_counter(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_ensure_counter", 0);

  /* "nvtx/_lib/writer.pyx":1524
 * 
 *     cdef void _ensure_counter(self, Counter counter) except *:
 *         if counter._domain is not self._domain:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "counter is not registered in this stream's domain")
*/
  __pyx_t_1 = (__pyx_v_counter->_domain != __pyx_v_self->_domain);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1525
 *     cdef void _ensure_counter(self, Counter counter) except *:
 *         if counter._domain is not self._domain:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "counter is not registered in this stream's domain")
 * 
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_counter_is_not_registered_in_thi};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1525, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1525, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1524
 * 
 *     cdef void _ensure_counter(self, Counter counter) except *:
 *         if counter._domain is not self._domain:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "counter is not registered in this stream's domain")
*/
  }

  /* "nvtx/_lib/writer.pyx":1523
 *         self._backend._ensure_open()
 * 
 *     cdef void _ensure_counter(self, Counter counter) except *:             # <<<<<<<<<<<<<<
 *         if counter._domain is not self._domain:
 *             raise ValueError(
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream._ensure_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "nvtx/_lib/writer.pyx":1528
 *                 "counter is not registered in this stream's domain")
 * 
 *     cdef nvtxwEventWriter_t* _get_writer(self) except NULL:             # <<<<<<<<<<<<<<
 *         # ``iface`` is set at construction and the stream handle in
 *         # open()/close().  The domain's event schema IDs are registered lazily,
*/

static nvtxwEventWriter_t *__pyx_f_4nvtx_4_lib_6writer_6Stream__get_writer(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self) {
  nvtxwEventWriter_t *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_writer", 0);

  /* "nvtx/_lib/writer.pyx":1535
 *         # the writer once.  They are stable for the rest of the session, so the
 *         # copy never needs repeating or invalidating on close.
 *         self._ensure_writable()             # <<<<<<<<<<<<<<
 *         if self._domain is None:
 *             raise ValueError(
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_ensure_writable(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1535, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1536
 *         # copy never needs repeating or invalidating on close.
 *         self._ensure_writable()
 *         if self._domain is None:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "event helpers require a registered stream domain"
*/
  __pyx_t_1 = (((PyObject *)__pyx_v_self->_domain) == Py_None);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1537
 *         self._ensure_writable()
 *         if self._domain is None:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "event helpers require a registered stream domain"
 *             )
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_event_helpers_require_a_register};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1537, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1537, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1536
 *         # copy never needs repeating or invalidating on close.
 *         self._ensure_writable()
 *         if self._domain is None:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "event helpers require a registered stream domain"
*/
  }

  /* "nvtx/_lib/writer.pyx":1540
 *                 "event helpers require a registered stream domain"
 *             )
 *         if not self._writer_ready:             # <<<<<<<<<<<<<<
 *             self._domain._get_event_schema_ids(&self._writer.schemaIds)
 *             self._writer_ready = True
*/
  __pyx_t_1 = (!__pyx_v_self->_writer_ready);
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":1541
 *             )
 *         if not self._writer_ready:
 *             self._domain._get_event_schema_ids(&self._writer.schemaIds)             # <<<<<<<<<<<<<<
 *             self._writer_ready = True
 *         return &self._writer
*/
    __pyx_t_2 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain *)__pyx_v_self->_domain->__pyx_vtab)->_get_event_schema_ids(__pyx_v_self->_domain, (&__pyx_v_self->_writer.schemaIds)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1541, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "nvtx/_lib/writer.pyx":1542
 *         if not self._writer_ready:
 *             self._domain._get_event_schema_ids(&self._writer.schemaIds)
 *             self._writer_ready = True             # <<<<<<<<<<<<<<
 *         return &self._writer
 * 
*/
    __pyx_v_self->_writer_ready = 1;

    /* "nvtx/_lib/writer.pyx":1540
 *                 "event helpers require a registered stream domain"
 *             )
 *         if not self._writer_ready:             # <<<<<<<<<<<<<<
 *             self._domain._get_event_schema_ids(&self._writer.schemaIds)
 *             self._writer_ready = True
*/
  }

  /* "nvtx/_lib/writer.pyx":1543
 *             self._domain._get_event_schema_ids(&self._writer.schemaIds)
 *             self._writer_ready = True
 *         return &self._writer             # <<<<<<<<<<<<<<
 * 
 *     cdef object _resolve_event_attrs(
*/
  __pyx_r = (&__pyx_v_self->_writer);
  goto __pyx_L0;

  /* "nvtx/_lib/writer.pyx":1528
 *                 "counter is not registered in this stream's domain")
 * 
 *     cdef nvtxwEventWriter_t* _get_writer(self) except NULL:             # <<<<<<<<<<<<<<
 *         # ``iface`` is set at construction and the stream handle in
 *         # open()/close().  The domain's event schema IDs are registered lazily,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream._get_writer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1545
 *         return &self._writer
 * 
 *     cdef object _resolve_event_attrs(             # <<<<<<<<<<<<<<
 *         self,
 *         object message,
*/

static PyObject *__pyx_f_4nvtx_4_lib_6writer_6Stream__resolve_event_attrs(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, nvtxwEventAttributes_t *__pyx_v_attr, nvtxwEventAttributesUtf8_t *__pyx_v_uattr) {
  uint32_t __pyx_v_color_argb;
  uint32_t __pyx_v_category_id;
  PyObject *__pyx_v_message_bytes = 0;
  char const *__pyx_v_message_ptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint32_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  nvtxStringHandle_t __pyx_t_6;
  int __pyx_t_7;
  char const *__pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_resolve_event_attrs", 0);
  __Pyx_INCREF(__pyx_v_message);

  /* "nvtx/_lib/writer.pyx":1553
 *         nvtxwEventAttributesUtf8_t* uattr,
 *     ):
 *         cdef uint32_t color_argb = _resolve_color(color)             # <<<<<<<<<<<<<<
 *         cdef uint32_t category_id = _resolve_category(self._domain, category)
 *         cdef bytes message_bytes
*/
  __pyx_t_1 = __pyx_f_4nvtx_4_lib_6writer__resolve_color(__pyx_v_color); if (unlikely(__pyx_t_1 == ((uint32_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1553, __pyx_L1_error)
  __pyx_v_color_argb = __pyx_t_1;

  /* "nvtx/_lib/writer.pyx":1554
 *     ):
 *         cdef uint32_t color_argb = _resolve_color(color)
 *         cdef uint32_t category_id = _resolve_category(self._domain, category)             # <<<<<<<<<<<<<<
 *         cdef bytes message_bytes
 *         cdef const char* message_ptr
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self->_domain);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_1 = __pyx_f_4nvtx_4_lib_6writer__resolve_category(((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_t_2), __pyx_v_category); if (unlikely(__pyx_t_1 == ((uint32_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1554, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_category_id = __pyx_t_1;

  /* "nvtx/_lib/writer.pyx":1557
 *         cdef bytes message_bytes
 *         cdef const char* message_ptr
 *         if message is None:             # <<<<<<<<<<<<<<
 *             message = b""
 *         if isinstance(message, RegisteredString):
*/
  __pyx_t_3 = (__pyx_v_message == Py_None);
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":1558
 *         cdef const char* message_ptr
 *         if message is None:
 *             message = b""             # <<<<<<<<<<<<<<
 *         if isinstance(message, RegisteredString):
 *             # Registered-string form: the handle is owned by the domain, so
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_b__7);
    __Pyx_DECREF_SET(__pyx_v_message, __pyx_mstate_global->__pyx_kp_b__7);

    /* "nvtx/_lib/writer.pyx":1557
 *         cdef bytes message_bytes
 *         cdef const char* message_ptr
 *         if message is None:             # <<<<<<<<<<<<<<
 *             message = b""
 *         if isinstance(message, RegisteredString):
*/
  }

  /* "nvtx/_lib/writer.pyx":1559
 *         if message is None:
 *             message = b""
 *         if isinstance(message, RegisteredString):             # <<<<<<<<<<<<<<
 *             # Registered-string form: the handle is owned by the domain, so
 *             # nothing needs to be kept alive past the call.
*/
  __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_message, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString); 
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":1562
 *             # Registered-string form: the handle is owned by the domain, so
 *             # nothing needs to be kept alive past the call.
 *             if (<RegisteredString>message)._domain is not self._domain:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "message is a RegisteredString from a different domain")
*/
    __pyx_t_3 = (((struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)__pyx_v_message)->_domain != ((PyObject *)__pyx_v_self->_domain));
    if (unlikely(__pyx_t_3)) {

      /* "nvtx/_lib/writer.pyx":1563
 *             # nothing needs to be kept alive past the call.
 *             if (<RegisteredString>message)._domain is not self._domain:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     "message is a RegisteredString from a different domain")
 *             attr.color = color_argb
*/
      __pyx_t_4 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_message_is_a_RegisteredString_fr};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1563, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 1563, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":1562
 *             # Registered-string form: the handle is owned by the domain, so
 *             # nothing needs to be kept alive past the call.
 *             if (<RegisteredString>message)._domain is not self._domain:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "message is a RegisteredString from a different domain")
*/
    }

    /* "nvtx/_lib/writer.pyx":1565
 *                 raise ValueError(
 *                     "message is a RegisteredString from a different domain")
 *             attr.color = color_argb             # <<<<<<<<<<<<<<
 *             attr.category = category_id
 *             attr.message = (<RegisteredString>message)._handle
*/
    __pyx_v_attr->color = __pyx_v_color_argb;

    /* "nvtx/_lib/writer.pyx":1566
 *                     "message is a RegisteredString from a different domain")
 *             attr.color = color_argb
 *             attr.category = category_id             # <<<<<<<<<<<<<<
 *             attr.message = (<RegisteredString>message)._handle
 *             return None
*/
    __pyx_v_attr->category = __pyx_v_category_id;

    /* "nvtx/_lib/writer.pyx":1567
 *             attr.color = color_argb
 *             attr.category = category_id
 *             attr.message = (<RegisteredString>message)._handle             # <<<<<<<<<<<<<<
 *             return None
 *         if isinstance(message, (str, bytes)):
*/
    __pyx_t_6 = ((struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)__pyx_v_message)->_handle;
    __pyx_v_attr->message = __pyx_t_6;

    /* "nvtx/_lib/writer.pyx":1568
 *             attr.category = category_id
 *             attr.message = (<RegisteredString>message)._handle
 *             return None             # <<<<<<<<<<<<<<
 *         if isinstance(message, (str, bytes)):
 *             message_bytes = _as_bytes(message)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":1559
 *         if message is None:
 *             message = b""
 *         if isinstance(message, RegisteredString):             # <<<<<<<<<<<<<<
 *             # Registered-string form: the handle is owned by the domain, so
 *             # nothing needs to be kept alive past the call.
*/
  }

  /* "nvtx/_lib/writer.pyx":1569
 *             attr.message = (<RegisteredString>message)._handle
 *             return None
 *         if isinstance(message, (str, bytes)):             # <<<<<<<<<<<<<<
 *             message_bytes = _as_bytes(message)
 *             message_ptr = message_bytes
*/
  __pyx_t_7 = PyUnicode_Check(__pyx_v_message); 
  if (!__pyx_t_7) {
  } else {
    __pyx_t_3 = __pyx_t_7;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_7 = PyBytes_Check(__pyx_v_message); 
  __pyx_t_3 = __pyx_t_7;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":1570
 *             return None
 *         if isinstance(message, (str, bytes)):
 *             message_bytes = _as_bytes(message)             # <<<<<<<<<<<<<<
 *             message_ptr = message_bytes
 *             uattr.color = color_argb
*/
    __pyx_t_2 = __pyx_f_4nvtx_4_lib_6writer__as_bytes(__pyx_v_message); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1570, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_message_bytes = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "nvtx/_lib/writer.pyx":1571
 *         if isinstance(message, (str, bytes)):
 *             message_bytes = _as_bytes(message)
 *             message_ptr = message_bytes             # <<<<<<<<<<<<<<
 *             uattr.color = color_argb
 *             uattr.category = category_id
*/
    if (unlikely(__pyx_v_message_bytes == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1571, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_PyBytes_AsString(__pyx_v_message_bytes); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 1571, __pyx_L1_error)
    __pyx_v_message_ptr = __pyx_t_8;

    /* "nvtx/_lib/writer.pyx":1572
 *             message_bytes = _as_bytes(message)
 *             message_ptr = message_bytes
 *             uattr.color = color_argb             # <<<<<<<<<<<<<<
 *             uattr.category = category_id
 *             uattr.messageLength = <uint32_t>len(message_bytes)
*/
    __pyx_v_uattr->color = __pyx_v_color_argb;

    /* "nvtx/_lib/writer.pyx":1573
 *             message_ptr = message_bytes
 *             uattr.color = color_argb
 *             uattr.category = category_id             # <<<<<<<<<<<<<<
 *             uattr.messageLength = <uint32_t>len(message_bytes)
 *             uattr.message = message_ptr
*/
    __pyx_v_uattr->category = __pyx_v_category_id;

    /* "nvtx/_lib/writer.pyx":1574
 *             uattr.color = color_argb
 *             uattr.category = category_id
 *             uattr.messageLength = <uint32_t>len(message_bytes)             # <<<<<<<<<<<<<<
 *             uattr.message = message_ptr
 *             return message_bytes
*/
    if (unlikely(__pyx_v_message_bytes == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 1574, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyBytes_GET_SIZE(__pyx_v_message_bytes); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1574, __pyx_L1_error)
    __pyx_v_uattr->messageLength = ((uint32_t)__pyx_t_9);

    /* "nvtx/_lib/writer.pyx":1575
 *             uattr.category = category_id
 *             uattr.messageLength = <uint32_t>len(message_bytes)
 *             uattr.message = message_ptr             # <<<<<<<<<<<<<<
 *             return message_bytes
 *         raise TypeError(
*/
    __pyx_v_uattr->message = __pyx_v_message_ptr;

    /* "nvtx/_lib/writer.pyx":1576
 *             uattr.messageLength = <uint32_t>len(message_bytes)
 *             uattr.message = message_ptr
 *             return message_bytes             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             "message must be a str, bytes, RegisteredString, or None")
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_message_bytes);
    __pyx_r = __pyx_v_message_bytes;
    goto __pyx_L0;

    /* "nvtx/_lib/writer.pyx":1569
 *             attr.message = (<RegisteredString>message)._handle
 *             return None
 *         if isinstance(message, (str, bytes)):             # <<<<<<<<<<<<<<
 *             message_bytes = _as_bytes(message)
 *             message_ptr = message_bytes
*/
  }

  /* "nvtx/_lib/writer.pyx":1577
 *             uattr.message = message_ptr
 *             return message_bytes
 *         raise TypeError(             # <<<<<<<<<<<<<<
 *             "message must be a str, bytes, RegisteredString, or None")
 * 
*/
  __pyx_t_4 = NULL;
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_message_must_be_a_str_bytes_Regi};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1577, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 1577, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1545
 *         return &self._writer
 * 
 *     cdef object _resolve_event_attrs(             # <<<<<<<<<<<<<<
 *         self,
 *         object message,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream._resolve_event_attrs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_message_bytes);
  __Pyx_XDECREF(__pyx_v_message);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1580
 *             "message must be a str, bytes, RegisteredString, or None")
 * 
 *     cdef void _emit_at(             # <<<<<<<<<<<<<<
 *         self, int64_t timestamp, object message, object color, object category,
 *         _at_bin_fn bin_fn, _at_utf8_fn utf8_fn, str op, str utf8_op,
*/

static void __pyx_f_4nvtx_4_lib_6writer_6Stream__emit_at(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, int64_t __pyx_v_timestamp, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, __pyx_t_4nvtx_4_lib_6writer__at_bin_fn __pyx_v_bin_fn, __pyx_t_4nvtx_4_lib_6writer__at_utf8_fn __pyx_v_utf8_fn, PyObject *__pyx_v_op, PyObject *__pyx_v_utf8_op) {
  nvtxwEventWriter_t *__pyx_v_writer;
  nvtxwEventAttributes_t __pyx_v_attr;
  nvtxwEventAttributesUtf8_t __pyx_v_uattr;
  PyObject *__pyx_v_owner = 0;
  __Pyx_RefNannyDeclarations
  nvtxwEventWriter_t *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  nvtxwResultCode_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_emit_at", 0);

  /* "nvtx/_lib/writer.pyx":1584
 *         _at_bin_fn bin_fn, _at_utf8_fn utf8_fn, str op, str utf8_op,
 *     ) except *:
 *         cdef nvtxwEventWriter_t* writer = self._get_writer()             # <<<<<<<<<<<<<<
 *         cdef nvtxwEventAttributes_t attr
 *         cdef nvtxwEventAttributesUtf8_t uattr
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_get_writer(__pyx_v_self); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(0, 1584, __pyx_L1_error)
  __pyx_v_writer = __pyx_t_1;

  /* "nvtx/_lib/writer.pyx":1587
 *         cdef nvtxwEventAttributes_t attr
 *         cdef nvtxwEventAttributesUtf8_t uattr
 *         cdef bytes owner = self._resolve_event_attrs(             # <<<<<<<<<<<<<<
 *             message, color, category, &attr, &uattr)
 *         if owner is None:
*/
  __pyx_t_2 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_resolve_event_attrs(__pyx_v_self, __pyx_v_message, __pyx_v_color, __pyx_v_category, (&__pyx_v_attr), (&__pyx_v_uattr)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(PyBytes_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_2))) __PYX_ERR(0, 1587, __pyx_L1_error)
  __pyx_v_owner = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":1589
 *         cdef bytes owner = self._resolve_event_attrs(
 *             message, color, category, &attr, &uattr)
 *         if owner is None:             # <<<<<<<<<<<<<<
 *             _check(bin_fn(writer, timestamp, attr), op)
 *         else:
*/
  __pyx_t_3 = (__pyx_v_owner == ((PyObject*)Py_None));
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":1590
 *             message, color, category, &attr, &uattr)
 *         if owner is None:
 *             _check(bin_fn(writer, timestamp, attr), op)             # <<<<<<<<<<<<<<
 *         else:
 *             _check(utf8_fn(writer, timestamp, uattr), utf8_op)
*/
    __pyx_t_4 = __pyx_v_bin_fn(__pyx_v_writer, __pyx_v_timestamp, __pyx_v_attr); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1590, __pyx_L1_error)
    __pyx_f_4nvtx_4_lib_6writer__check(__pyx_t_4, __pyx_v_op); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1590, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1589
 *         cdef bytes owner = self._resolve_event_attrs(
 *             message, color, category, &attr, &uattr)
 *         if owner is None:             # <<<<<<<<<<<<<<
 *             _check(bin_fn(writer, timestamp, attr), op)
 *         else:
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/writer.pyx":1592
 *             _check(bin_fn(writer, timestamp, attr), op)
 *         else:
 *             _check(utf8_fn(writer, timestamp, uattr), utf8_op)             # <<<<<<<<<<<<<<
 * 
 *     cdef void _emit_span(
*/
  /*else*/ {
    __pyx_t_4 = __pyx_v_utf8_fn(__pyx_v_writer, __pyx_v_timestamp, __pyx_v_uattr); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1592, __pyx_L1_error)
    __pyx_f_4nvtx_4_lib_6writer__check(__pyx_t_4, __pyx_v_utf8_op); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1592, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "nvtx/_lib/writer.pyx":1580
 *             "message must be a str, bytes, RegisteredString, or None")
 * 
 *     cdef void _emit_at(             # <<<<<<<<<<<<<<
 *         self, int64_t timestamp, object message, object color, object category,
 *         _at_bin_fn bin_fn, _at_utf8_fn utf8_fn, str op, str utf8_op,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream._emit_at", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_owner);
  __Pyx_RefNannyFinishContext();
}

/* "nvtx/_lib/writer.pyx":1594
 *             _check(utf8_fn(writer, timestamp, uattr), utf8_op)
 * 
 *     cdef void _emit_span(             # <<<<<<<<<<<<<<
 *         self, int64_t begin, int64_t end, object message, object color,
 *         object category, _span_bin_fn bin_fn, _span_utf8_fn utf8_fn,
*/

static void __pyx_f_4nvtx_4_lib_6writer_6Stream__emit_span(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, int64_t __pyx_v_begin, int64_t __pyx_v_end, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, __pyx_t_4nvtx_4_lib_6writer__span_bin_fn __pyx_v_bin_fn, __pyx_t_4nvtx_4_lib_6writer__span_utf8_fn __pyx_v_utf8_fn, PyObject *__pyx_v_op, PyObject *__pyx_v_utf8_op) {
  nvtxwEventWriter_t *__pyx_v_writer;
  nvtxwEventAttributes_t __pyx_v_attr;
  nvtxwEventAttributesUtf8_t __pyx_v_uattr;
  PyObject *__pyx_v_owner = 0;
  __Pyx_RefNannyDeclarations
  nvtxwEventWriter_t *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  nvtxwResultCode_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_emit_span", 0);

  /* "nvtx/_lib/writer.pyx":1599
 *         str op, str utf8_op,
 *     ) except *:
 *         cdef nvtxwEventWriter_t* writer = self._get_writer()             # <<<<<<<<<<<<<<
 *         cdef nvtxwEventAttributes_t attr
 *         cdef nvtxwEventAttributesUtf8_t uattr
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_get_writer(__pyx_v_self); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(0, 1599, __pyx_L1_error)
  __pyx_v_writer = __pyx_t_1;

  /* "nvtx/_lib/writer.pyx":1602
 *         cdef nvtxwEventAttributes_t attr
 *         cdef nvtxwEventAttributesUtf8_t uattr
 *         cdef bytes owner = self._resolve_event_attrs(             # <<<<<<<<<<<<<<
 *             message, color, category, &attr, &uattr)
 *         if owner is None:
*/
  __pyx_t_2 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_resolve_event_attrs(__pyx_v_self, __pyx_v_message, __pyx_v_color, __pyx_v_category, (&__pyx_v_attr), (&__pyx_v_uattr)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(PyBytes_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_2))) __PYX_ERR(0, 1602, __pyx_L1_error)
  __pyx_v_owner = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":1604
 *         cdef bytes owner = self._resolve_event_attrs(
 *             message, color, category, &attr, &uattr)
 *         if owner is None:             # <<<<<<<<<<<<<<
 *             _check(bin_fn(writer, begin, end, attr), op)
 *         else:
*/
  __pyx_t_3 = (__pyx_v_owner == ((PyObject*)Py_None));
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":1605
 *             message, color, category, &attr, &uattr)
 *         if owner is None:
 *             _check(bin_fn(writer, begin, end, attr), op)             # <<<<<<<<<<<<<<
 *         else:
 *             _check(utf8_fn(writer, begin, end, uattr), utf8_op)
*/
    __pyx_t_4 = __pyx_v_bin_fn(__pyx_v_writer, __pyx_v_begin, __pyx_v_end, __pyx_v_attr); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1605, __pyx_L1_error)
    __pyx_f_4nvtx_4_lib_6writer__check(__pyx_t_4, __pyx_v_op); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1605, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1604
 *         cdef bytes owner = self._resolve_event_attrs(
 *             message, color, category, &attr, &uattr)
 *         if owner is None:             # <<<<<<<<<<<<<<
 *             _check(bin_fn(writer, begin, end, attr), op)
 *         else:
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/writer.pyx":1607
 *             _check(bin_fn(writer, begin, end, attr), op)
 *         else:
 *             _check(utf8_fn(writer, begin, end, uattr), utf8_op)             # <<<<<<<<<<<<<<
 * 
 *     def write_mark(
*/
  /*else*/ {
    __pyx_t_4 = __pyx_v_utf8_fn(__pyx_v_writer, __pyx_v_begin, __pyx_v_end, __pyx_v_uattr); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1607, __pyx_L1_error)
    __pyx_f_4nvtx_4_lib_6writer__check(__pyx_t_4, __pyx_v_utf8_op); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1607, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "nvtx/_lib/writer.pyx":1594
 *             _check(utf8_fn(writer, timestamp, uattr), utf8_op)
 * 
 *     cdef void _emit_span(             # <<<<<<<<<<<<<<
 *         self, int64_t begin, int64_t end, object message, object color,
 *         object category, _span_bin_fn bin_fn, _span_utf8_fn utf8_fn,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream._emit_span", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_owner);
  __Pyx_RefNannyFinishContext();
}

/* "nvtx/_lib/writer.pyx":1609
 *             _check(utf8_fn(writer, begin, end, uattr), utf8_op)
 * 
 *     def write_mark(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_11write_mark(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_10write_mark, "Stream.write_mark(self, timestamp, *, message=None, color=None, category=None)\n\nWrite a mark event.\n\nParameters\n----------\ntimestamp : int\n    Event timestamp in the stream's time domain.\nmessage : str, bytes, or RegisteredString, optional\n    Message associated with the event.\ncolor : int or color-like, optional\n    Event color. Integers are interpreted as ARGB values.\ncategory : str, bytes, or int, optional\n    Event category. A name is registered in the stream's domain.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nTypeError\n    If ``message`` or ``category`` has an unsupported type.\nValueError\n    If an integer ``category`` is outside ``[0, 2**32 - 1]``, or\n    ``message`` is a :class:`RegisteredString` from a different domain.\nWriterError\n    If the backend fails to write the event.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_11write_mark = {"write_mark", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_11write_mark, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_10write_mark};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_11write_mark(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_timestamp = 0;
  PyObject *__pyx_v_message = 0;
  PyObject *__pyx_v_color = 0;
  PyObject *__pyx_v_category = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_mark (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_timestamp,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1609, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1609, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_mark", 0) < (0)) __PYX_ERR(0, 1609, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":1610
 * 
 *     def write_mark(
 *         self, timestamp, *, message=None, color=None, category=None             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_mark", 1, 1, 1, i); __PYX_ERR(0, 1609, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1609, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_timestamp = values[0];
    __pyx_v_message = values[1];
    __pyx_v_color = values[2];
    __pyx_v_category = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_mark", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1609, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_mark", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_10write_mark(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_timestamp, __pyx_v_message, __pyx_v_color, __pyx_v_category);

  /* "nvtx/_lib/writer.pyx":1609
 *             _check(utf8_fn(writer, begin, end, uattr), utf8_op)
 * 
 *     def write_mark(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_10write_mark(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int64_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_mark", 0);

  /* "nvtx/_lib/writer.pyx":1638
 *             If the backend fails to write the event.
 *         """
 *         self._emit_at(timestamp, message, color, category,             # <<<<<<<<<<<<<<
 *                       nvtxwMarkWrite, nvtxwMarkWriteUtf8,
 *                       "nvtxwMarkWrite", "nvtxwMarkWriteUtf8")
*/
  __pyx_t_1 = __Pyx_PyLong_As_int64_t(__pyx_v_timestamp); if (unlikely((__pyx_t_1 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1638, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1639
 *         """
 *         self._emit_at(timestamp, message, color, category,
 *                       nvtxwMarkWrite, nvtxwMarkWriteUtf8,             # <<<<<<<<<<<<<<
 *                       "nvtxwMarkWrite", "nvtxwMarkWriteUtf8")
 * 
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_emit_at(__pyx_v_self, __pyx_t_1, __pyx_v_message, __pyx_v_color, __pyx_v_category, nvtxwMarkWrite, nvtxwMarkWriteUtf8, __pyx_mstate_global->__pyx_n_u_nvtxwMarkWrite, __pyx_mstate_global->__pyx_n_u_nvtxwMarkWriteUtf8); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1638, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1609
 *             _check(utf8_fn(writer, begin, end, uattr), utf8_op)
 * 
 *     def write_mark(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_mark", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1642
 *                       "nvtxwMarkWrite", "nvtxwMarkWriteUtf8")
 * 
 *     def write_pushpop(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_13write_pushpop(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_12write_pushpop, "Stream.write_pushpop(self, start, end, *, message=None, color=None, category=None)\n\nWrite a complete push/pop range event.\n\nParameters\n----------\nstart : int\n    Range start timestamp in the stream's time domain.\nend : int\n    Range end timestamp in the stream's time domain.\nmessage : str, bytes, or RegisteredString, optional\n    Message associated with the range.\ncolor : int or color-like, optional\n    Range color. Integers are interpreted as ARGB values.\ncategory : str, bytes, or int, optional\n    Range category. A name is registered in the stream's domain.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nTypeError\n    If ``message`` or ``category`` has an unsupported type.\nValueError\n    If an integer ``category`` is outside ``[0, 2**32 - 1]``, or\n    ``message`` is a :class:`RegisteredString` from a different domain.\nWriterError\n    If the backend fails to write the event.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_13write_pushpop = {"write_pushpop", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_13write_pushpop, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_12write_pushpop};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_13write_pushpop(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_end = 0;
  PyObject *__pyx_v_message = 0;
  PyObject *__pyx_v_color = 0;
  PyObject *__pyx_v_category = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_pushpop (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_end,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1642, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1642, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1642, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_pushpop", 0) < (0)) __PYX_ERR(0, 1642, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":1643
 * 
 *     def write_pushpop(
 *         self, start, end, *, message=None, color=None, category=None             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_pushpop", 1, 2, 2, i); __PYX_ERR(0, 1642, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1642, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1642, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_start = values[0];
    __pyx_v_end = values[1];
    __pyx_v_message = values[2];
    __pyx_v_color = values[3];
    __pyx_v_category = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_pushpop", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1642, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_pushpop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_12write_pushpop(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_start, __pyx_v_end, __pyx_v_message, __pyx_v_color, __pyx_v_category);

  /* "nvtx/_lib/writer.pyx":1642
 *                       "nvtxwMarkWrite", "nvtxwMarkWriteUtf8")
 * 
 *     def write_pushpop(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_12write_pushpop(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_end, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int64_t __pyx_t_1;
  int64_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_pushpop", 0);

  /* "nvtx/_lib/writer.pyx":1673
 *             If the backend fails to write the event.
 *         """
 *         self._emit_span(start, end, message, color, category,             # <<<<<<<<<<<<<<
 *                         nvtxwRangePushPopWrite, nvtxwRangePushPopWriteUtf8,
 *                         "nvtxwRangePushPopWrite",
*/
  __pyx_t_1 = __Pyx_PyLong_As_int64_t(__pyx_v_start); if (unlikely((__pyx_t_1 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1673, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_As_int64_t(__pyx_v_end); if (unlikely((__pyx_t_2 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1673, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1674
 *         """
 *         self._emit_span(start, end, message, color, category,
 *                         nvtxwRangePushPopWrite, nvtxwRangePushPopWriteUtf8,             # <<<<<<<<<<<<<<
 *                         "nvtxwRangePushPopWrite",
 *                         "nvtxwRangePushPopWriteUtf8")
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_emit_span(__pyx_v_self, __pyx_t_1, __pyx_t_2, __pyx_v_message, __pyx_v_color, __pyx_v_category, nvtxwRangePushPopWrite, nvtxwRangePushPopWriteUtf8, __pyx_mstate_global->__pyx_n_u_nvtxwRangePushPopWrite, __pyx_mstate_global->__pyx_n_u_nvtxwRangePushPopWriteUtf8); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1673, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1642
 *                       "nvtxwMarkWrite", "nvtxwMarkWriteUtf8")
 * 
 *     def write_pushpop(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_pushpop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1678
 *                         "nvtxwRangePushPopWriteUtf8")
 * 
 *     def write_startend(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_15write_startend(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_14write_startend, "Stream.write_startend(self, start, end, *, message=None, color=None, category=None)\n\nWrite a complete start/end range event.\n\nParameters\n----------\nstart : int\n    Range start timestamp in the stream's time domain.\nend : int\n    Range end timestamp in the stream's time domain.\nmessage : str, bytes, or RegisteredString, optional\n    Message associated with the range.\ncolor : int or color-like, optional\n    Range color. Integers are interpreted as ARGB values.\ncategory : str, bytes, or int, optional\n    Range category. A name is registered in the stream's domain.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nTypeError\n    If ``message`` or ``category`` has an unsupported type.\nValueError\n    If an integer ``category`` is outside ``[0, 2**32 - 1]``, or\n    ``message`` is a :class:`RegisteredString` from a different domain.\nWriterError\n    If the backend fails to write the event.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_15write_startend = {"write_startend", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_15write_startend, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_14write_startend};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_15write_startend(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_end = 0;
  PyObject *__pyx_v_message = 0;
  PyObject *__pyx_v_color = 0;
  PyObject *__pyx_v_category = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_startend (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_end,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1678, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1678, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1678, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_startend", 0) < (0)) __PYX_ERR(0, 1678, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":1679
 * 
 *     def write_startend(
 *         self, start, end, *, message=None, color=None, category=None             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_startend", 1, 2, 2, i); __PYX_ERR(0, 1678, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1678, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1678, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_start = values[0];
    __pyx_v_end = values[1];
    __pyx_v_message = values[2];
    __pyx_v_color = values[3];
    __pyx_v_category = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_startend", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1678, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_startend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_14write_startend(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_start, __pyx_v_end, __pyx_v_message, __pyx_v_color, __pyx_v_category);

  /* "nvtx/_lib/writer.pyx":1678
 *                         "nvtxwRangePushPopWriteUtf8")
 * 
 *     def write_startend(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_14write_startend(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_end, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int64_t __pyx_t_1;
  int64_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_startend", 0);

  /* "nvtx/_lib/writer.pyx":1709
 *             If the backend fails to write the event.
 *         """
 *         self._emit_span(start, end, message, color, category,             # <<<<<<<<<<<<<<
 *                         nvtxwRangeStartEndWrite, nvtxwRangeStartEndWriteUtf8,
 *                         "nvtxwRangeStartEndWrite",
*/
  __pyx_t_1 = __Pyx_PyLong_As_int64_t(__pyx_v_start); if (unlikely((__pyx_t_1 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1709, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_As_int64_t(__pyx_v_end); if (unlikely((__pyx_t_2 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1709, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1710
 *         """
 *         self._emit_span(start, end, message, color, category,
 *                         nvtxwRangeStartEndWrite, nvtxwRangeStartEndWriteUtf8,             # <<<<<<<<<<<<<<
 *                         "nvtxwRangeStartEndWrite",
 *                         "nvtxwRangeStartEndWriteUtf8")
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_emit_span(__pyx_v_self, __pyx_t_1, __pyx_t_2, __pyx_v_message, __pyx_v_color, __pyx_v_category, nvtxwRangeStartEndWrite, nvtxwRangeStartEndWriteUtf8, __pyx_mstate_global->__pyx_n_u_nvtxwRangeStartEndWrite, __pyx_mstate_global->__pyx_n_u_nvtxwRangeStartEndWriteUtf8); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1709, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1678
 *                         "nvtxwRangePushPopWriteUtf8")
 * 
 *     def write_startend(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_startend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1714
 *                         "nvtxwRangeStartEndWriteUtf8")
 * 
 *     def write_push(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_17write_push(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_16write_push, "Stream.write_push(self, timestamp, *, message=None, color=None, category=None)\n\nWrite the beginning of a push/pop range.\n\nPush/pop ranges are nested and pair in last-in, first-out order.\n\nParameters\n----------\ntimestamp : int\n    Range start timestamp in the stream's time domain.\nmessage : str, bytes, or RegisteredString, optional\n    Message associated with the range.\ncolor : int or color-like, optional\n    Range color. Integers are interpreted as ARGB values.\ncategory : str, bytes, or int, optional\n    Range category. A name is registered in the stream's domain.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nTypeError\n    If ``message`` or ``category`` has an unsupported type.\nValueError\n    If an integer ``category`` is outside ``[0, 2**32 - 1]``, or\n    ``message`` is a :class:`RegisteredString` from a different domain.\nWriterError\n    If the backend fails to write the event.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_17write_push = {"write_push", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_17write_push, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_16write_push};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_17write_push(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_timestamp = 0;
  PyObject *__pyx_v_message = 0;
  PyObject *__pyx_v_color = 0;
  PyObject *__pyx_v_category = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_push (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_timestamp,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1714, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1714, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_push", 0) < (0)) __PYX_ERR(0, 1714, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":1715
 * 
 *     def write_push(
 *         self, timestamp, *, message=None, color=None, category=None             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_push", 1, 1, 1, i); __PYX_ERR(0, 1714, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1714, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_timestamp = values[0];
    __pyx_v_message = values[1];
    __pyx_v_color = values[2];
    __pyx_v_category = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_push", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1714, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_16write_push(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_timestamp, __pyx_v_message, __pyx_v_color, __pyx_v_category);

  /* "nvtx/_lib/writer.pyx":1714
 *                         "nvtxwRangeStartEndWriteUtf8")
 * 
 *     def write_push(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_16write_push(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int64_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_push", 0);

  /* "nvtx/_lib/writer.pyx":1745
 *             If the backend fails to write the event.
 *         """
 *         self._emit_at(timestamp, message, color, category,             # <<<<<<<<<<<<<<
 *                       nvtxwRangePushWrite, nvtxwRangePushWriteUtf8,
 *                       "nvtxwRangePushWrite", "nvtxwRangePushWriteUtf8")
*/
  __pyx_t_1 = __Pyx_PyLong_As_int64_t(__pyx_v_timestamp); if (unlikely((__pyx_t_1 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1745, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1746
 *         """
 *         self._emit_at(timestamp, message, color, category,
 *                       nvtxwRangePushWrite, nvtxwRangePushWriteUtf8,             # <<<<<<<<<<<<<<
 *                       "nvtxwRangePushWrite", "nvtxwRangePushWriteUtf8")
 * 
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_emit_at(__pyx_v_self, __pyx_t_1, __pyx_v_message, __pyx_v_color, __pyx_v_category, nvtxwRangePushWrite, nvtxwRangePushWriteUtf8, __pyx_mstate_global->__pyx_n_u_nvtxwRangePushWrite, __pyx_mstate_global->__pyx_n_u_nvtxwRangePushWriteUtf8); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1745, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1714
 *                         "nvtxwRangeStartEndWriteUtf8")
 * 
 *     def write_push(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_push", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1749
 *                       "nvtxwRangePushWrite", "nvtxwRangePushWriteUtf8")
 * 
 *     def write_pop(self, timestamp):             # <<<<<<<<<<<<<<
 *         """
 *         Write the end of the most recently pushed range.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_19write_pop(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_18write_pop, "Stream.write_pop(self, timestamp)\n\nWrite the end of the most recently pushed range.\n\nParameters\n----------\ntimestamp : int\n    Range end timestamp in the stream's time domain.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nWriterError\n    If the backend fails to write the event.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_19write_pop = {"write_pop", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_19write_pop, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_18write_pop};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_19write_pop(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_timestamp = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_pop (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_timestamp,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1749, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1749, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_pop", 0) < (0)) __PYX_ERR(0, 1749, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_pop", 1, 1, 1, i); __PYX_ERR(0, 1749, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1749, __pyx_L3_error)
    }
    __pyx_v_timestamp = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_pop", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1749, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_pop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_18write_pop(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_timestamp);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_18write_pop(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  nvtxwEventWriter_t *__pyx_t_1;
  int64_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_pop", 0);

  /* "nvtx/_lib/writer.pyx":1766
 *         """
 *         _check(
 *             nvtxwRangePopWrite(self._get_writer(), timestamp),             # <<<<<<<<<<<<<<
 *             "nvtxwRangePopWrite")
 * 
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_get_writer(__pyx_v_self); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(0, 1766, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyLong_As_int64_t(__pyx_v_timestamp); if (unlikely((__pyx_t_2 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1766, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1765
 *             If the backend fails to write the event.
 *         """
 *         _check(             # <<<<<<<<<<<<<<
 *             nvtxwRangePopWrite(self._get_writer(), timestamp),
 *             "nvtxwRangePopWrite")
*/
  __pyx_f_4nvtx_4_lib_6writer__check(nvtxwRangePopWrite(__pyx_t_1, __pyx_t_2), __pyx_mstate_global->__pyx_n_u_nvtxwRangePopWrite); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1765, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1749
 *                       "nvtxwRangePushWrite", "nvtxwRangePushWriteUtf8")
 * 
 *     def write_pop(self, timestamp):             # <<<<<<<<<<<<<<
 *         """
 *         Write the end of the most recently pushed range.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_pop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1769
 *             "nvtxwRangePopWrite")
 * 
 *     def write_start(             # <<<<<<<<<<<<<<
 *         self,
 *         timestamp,
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_21write_start(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_20write_start, "Stream.write_start(self, timestamp, range_id, *, message=None, color=None, category=None)\n\nWrite the beginning of a start/end range.\n\nThe range is paired with a subsequent :meth:`write_end` call that\nuses the same ``range_id``.\n\nParameters\n----------\ntimestamp : int\n    Range start timestamp in the stream's time domain.\nrange_id : int\n    Nonzero identifier used to pair the start with its end.\nmessage : str, bytes, or RegisteredString, optional\n    Message associated with the range.\ncolor : int or color-like, optional\n    Range color. Integers are interpreted as ARGB values.\ncategory : str, bytes, or int, optional\n    Range category. A name is registered in the stream's domain.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nTypeError\n    If ``message`` or ``category`` has an unsupported type.\nValueError\n    If ``range_id`` is zero, an integer ``category`` is outside\n    ``[0, 2**32 - 1]``, or ``message`` is a :class:`RegisteredString`\n    from a different domain.\nWriterError\n    If the backend fails to write the event.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_21write_start = {"write_start", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_21write_start, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_20write_start};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_21write_start(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_timestamp = 0;
  PyObject *__pyx_v_range_id = 0;
  PyObject *__pyx_v_message = 0;
  PyObject *__pyx_v_color = 0;
  PyObject *__pyx_v_category = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_start (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_timestamp,&__pyx_mstate_global->__pyx_n_u_range_id,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1769, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1769, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1769, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_start", 0) < (0)) __PYX_ERR(0, 1769, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":1774
 *         range_id,
 *         *,
 *         message=None,             # <<<<<<<<<<<<<<
 *         color=None,
 *         category=None,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":1775
 *         *,
 *         message=None,
 *         color=None,             # <<<<<<<<<<<<<<
 *         category=None,
 *     ):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":1776
 *         message=None,
 *         color=None,
 *         category=None,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_start", 1, 2, 2, i); __PYX_ERR(0, 1769, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1769, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1769, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":1774
 *         range_id,
 *         *,
 *         message=None,             # <<<<<<<<<<<<<<
 *         color=None,
 *         category=None,
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":1775
 *         *,
 *         message=None,
 *         color=None,             # <<<<<<<<<<<<<<
 *         category=None,
 *     ):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/writer.pyx":1776
 *         message=None,
 *         color=None,
 *         category=None,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_timestamp = values[0];
    __pyx_v_range_id = values[1];
    __pyx_v_message = values[2];
    __pyx_v_color = values[3];
    __pyx_v_category = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_start", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1769, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_20write_start(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_timestamp, __pyx_v_range_id, __pyx_v_message, __pyx_v_color, __pyx_v_category);

  /* "nvtx/_lib/writer.pyx":1769
 *             "nvtxwRangePopWrite")
 * 
 *     def write_start(             # <<<<<<<<<<<<<<
 *         self,
 *         timestamp,
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_20write_start(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_range_id, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category) {
  nvtxwEventWriter_t *__pyx_v_writer;
  nvtxRangeId_t __pyx_v_rid;
  nvtxwEventAttributes_t __pyx_v_attr;
  nvtxwEventAttributesUtf8_t __pyx_v_uattr;
  PyObject *__pyx_v_owner = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  nvtxRangeId_t __pyx_t_5;
  nvtxwEventWriter_t *__pyx_t_6;
  int64_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_start", 0);

  /* "nvtx/_lib/writer.pyx":1814
 *         cdef nvtxwEventAttributes_t attr
 *         cdef nvtxwEventAttributesUtf8_t uattr
 *         if range_id == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("range_id must be non-zero")
 *         rid = range_id
*/
  __pyx_t_1 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_range_id, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1814, __pyx_L1_error)
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1815
 *         cdef nvtxwEventAttributesUtf8_t uattr
 *         if range_id == 0:
 *             raise ValueError("range_id must be non-zero")             # <<<<<<<<<<<<<<
 *         rid = range_id
 *         writer = self._get_writer()
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_range_id_must_be_non_zero};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1815, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1815, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1814
 *         cdef nvtxwEventAttributes_t attr
 *         cdef nvtxwEventAttributesUtf8_t uattr
 *         if range_id == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("range_id must be non-zero")
 *         rid = range_id
*/
  }

  /* "nvtx/_lib/writer.pyx":1816
 *         if range_id == 0:
 *             raise ValueError("range_id must be non-zero")
 *         rid = range_id             # <<<<<<<<<<<<<<
 *         writer = self._get_writer()
 *         cdef bytes owner = self._resolve_event_attrs(
*/
  __pyx_t_5 = __Pyx_PyLong_As_uint64_t(__pyx_v_range_id); if (unlikely((__pyx_t_5 == ((nvtxRangeId_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1816, __pyx_L1_error)
  __pyx_v_rid = __pyx_t_5;

  /* "nvtx/_lib/writer.pyx":1817
 *             raise ValueError("range_id must be non-zero")
 *         rid = range_id
 *         writer = self._get_writer()             # <<<<<<<<<<<<<<
 *         cdef bytes owner = self._resolve_event_attrs(
 *             message, color, category, &attr, &uattr)
*/
  __pyx_t_6 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_get_writer(__pyx_v_self); if (unlikely(__pyx_t_6 == ((void *)NULL))) __PYX_ERR(0, 1817, __pyx_L1_error)
  __pyx_v_writer = __pyx_t_6;

  /* "nvtx/_lib/writer.pyx":1818
 *         rid = range_id
 *         writer = self._get_writer()
 *         cdef bytes owner = self._resolve_event_attrs(             # <<<<<<<<<<<<<<
 *             message, color, category, &attr, &uattr)
 *         if owner is None:
*/
  __pyx_t_2 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_resolve_event_attrs(__pyx_v_self, __pyx_v_message, __pyx_v_color, __pyx_v_category, (&__pyx_v_attr), (&__pyx_v_uattr)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(PyBytes_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_2))) __PYX_ERR(0, 1818, __pyx_L1_error)
  __pyx_v_owner = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":1820
 *         cdef bytes owner = self._resolve_event_attrs(
 *             message, color, category, &attr, &uattr)
 *         if owner is None:             # <<<<<<<<<<<<<<
 *             _check(nvtxwRangeStartWrite(writer, timestamp, rid, attr),
 *                    "nvtxwRangeStartWrite")
*/
  __pyx_t_1 = (__pyx_v_owner == ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":1821
 *             message, color, category, &attr, &uattr)
 *         if owner is None:
 *             _check(nvtxwRangeStartWrite(writer, timestamp, rid, attr),             # <<<<<<<<<<<<<<
 *                    "nvtxwRangeStartWrite")
 *         else:
*/
    __pyx_t_7 = __Pyx_PyLong_As_int64_t(__pyx_v_timestamp); if (unlikely((__pyx_t_7 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1821, __pyx_L1_error)
    __pyx_f_4nvtx_4_lib_6writer__check(nvtxwRangeStartWrite(__pyx_v_writer, __pyx_t_7, __pyx_v_rid, __pyx_v_attr), __pyx_mstate_global->__pyx_n_u_nvtxwRangeStartWrite); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1821, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1820
 *         cdef bytes owner = self._resolve_event_attrs(
 *             message, color, category, &attr, &uattr)
 *         if owner is None:             # <<<<<<<<<<<<<<
 *             _check(nvtxwRangeStartWrite(writer, timestamp, rid, attr),
 *                    "nvtxwRangeStartWrite")
*/
    goto __pyx_L4;
  }

  /* "nvtx/_lib/writer.pyx":1824
 *                    "nvtxwRangeStartWrite")
 *         else:
 *             _check(nvtxwRangeStartWriteUtf8(writer, timestamp, rid, uattr),             # <<<<<<<<<<<<<<
 *                    "nvtxwRangeStartWriteUtf8")
 * 
*/
  /*else*/ {
    __pyx_t_7 = __Pyx_PyLong_As_int64_t(__pyx_v_timestamp); if (unlikely((__pyx_t_7 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1824, __pyx_L1_error)
    __pyx_f_4nvtx_4_lib_6writer__check(nvtxwRangeStartWriteUtf8(__pyx_v_writer, __pyx_t_7, __pyx_v_rid, __pyx_v_uattr), __pyx_mstate_global->__pyx_n_u_nvtxwRangeStartWriteUtf8); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1824, __pyx_L1_error)
  }
  __pyx_L4:;

  /* "nvtx/_lib/writer.pyx":1769
 *             "nvtxwRangePopWrite")
 * 
 *     def write_start(             # <<<<<<<<<<<<<<
 *         self,
 *         timestamp,
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_owner);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1827
 *                    "nvtxwRangeStartWriteUtf8")
 * 
 *     def write_end(self, timestamp, range_id):             # <<<<<<<<<<<<<<
 *         """
 *         Write the end of a start/end range.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_23write_end(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_22write_end, "Stream.write_end(self, timestamp, range_id)\n\nWrite the end of a start/end range.\n\nParameters\n----------\ntimestamp : int\n    Range end timestamp in the stream's time domain.\nrange_id : int\n    Identifier passed to the matching :meth:`write_start` call.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nWriterError\n    If the backend fails to write the event.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_23write_end = {"write_end", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_23write_end, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_22write_end};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_23write_end(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_timestamp = 0;
  PyObject *__pyx_v_range_id = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_end (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_timestamp,&__pyx_mstate_global->__pyx_n_u_range_id,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1827, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1827, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1827, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_end", 0) < (0)) __PYX_ERR(0, 1827, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_end", 1, 2, 2, i); __PYX_ERR(0, 1827, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1827, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1827, __pyx_L3_error)
    }
    __pyx_v_timestamp = values[0];
    __pyx_v_range_id = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_end", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1827, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_end", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_22write_end(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_timestamp, __pyx_v_range_id);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_22write_end(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, PyObject *__pyx_v_timestamp, PyObject *__pyx_v_range_id) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  nvtxwEventWriter_t *__pyx_t_5;
  int64_t __pyx_t_6;
  nvtxRangeId_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_end", 0);

  /* "nvtx/_lib/writer.pyx":1845
 *             If the backend fails to write the event.
 *         """
 *         if range_id == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("range_id must be non-zero")
 *         _check(
*/
  __pyx_t_1 = (__Pyx_PyLong_BoolEqObjC(__pyx_v_range_id, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1845, __pyx_L1_error)
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1846
 *         """
 *         if range_id == 0:
 *             raise ValueError("range_id must be non-zero")             # <<<<<<<<<<<<<<
 *         _check(
 *             nvtxwRangeEndWrite(self._get_writer(), timestamp, range_id),
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_range_id_must_be_non_zero};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1846, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1846, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1845
 *             If the backend fails to write the event.
 *         """
 *         if range_id == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("range_id must be non-zero")
 *         _check(
*/
  }

  /* "nvtx/_lib/writer.pyx":1848
 *             raise ValueError("range_id must be non-zero")
 *         _check(
 *             nvtxwRangeEndWrite(self._get_writer(), timestamp, range_id),             # <<<<<<<<<<<<<<
 *             "nvtxwRangeEndWrite")
 * 
*/
  __pyx_t_5 = ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_get_writer(__pyx_v_self); if (unlikely(__pyx_t_5 == ((void *)NULL))) __PYX_ERR(0, 1848, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyLong_As_int64_t(__pyx_v_timestamp); if (unlikely((__pyx_t_6 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1848, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyLong_As_uint64_t(__pyx_v_range_id); if (unlikely((__pyx_t_7 == ((nvtxRangeId_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1848, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1847
 *         if range_id == 0:
 *             raise ValueError("range_id must be non-zero")
 *         _check(             # <<<<<<<<<<<<<<
 *             nvtxwRangeEndWrite(self._get_writer(), timestamp, range_id),
 *             "nvtxwRangeEndWrite")
*/
  __pyx_f_4nvtx_4_lib_6writer__check(nvtxwRangeEndWrite(__pyx_t_5, __pyx_t_6, __pyx_t_7), __pyx_mstate_global->__pyx_n_u_nvtxwRangeEndWrite); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1847, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1827
 *                    "nvtxwRangeStartWriteUtf8")
 * 
 *     def write_end(self, timestamp, range_id):             # <<<<<<<<<<<<<<
 *         """
 *         Write the end of a start/end range.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_end", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1851
 *             "nvtxwRangeEndWrite")
 * 
 *     def write_event(self, Schema schema not None, row):             # <<<<<<<<<<<<<<
 *         """
 *         Write an event described by a role-annotated dtype schema.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_25write_event(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_24write_event, "Stream.write_event(self, Schema schema, row)\n\nWrite an event described by a role-annotated dtype schema.\n\nAll event families are supported. The ID-correlated\n:attr:`EventKind.RANGE_START` and :attr:`EventKind.RANGE_END` schemas\ninclude a :attr:`PayloadEntryType.RANGE_ID` field.\n\nParameters\n----------\nschema : Schema\n    Event schema returned by :meth:`Domain.get_schema` with a\n    ``kind``, for this stream's domain.\nrow : array-like\n    Values for one row of the schema's dtype, such as a tuple or a\n    NumPy scalar.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nTypeError\n    If ``schema`` is not a :class:`Schema`.\nValueError\n    If ``schema`` belongs to another domain or is not an event\n    schema, or ``row`` does not contain exactly one event.\nWriterError\n    If the backend fails to write the event.\n\nNotes\n-----\nEach call converts ``row`` to a contiguous array. For high write\nrates with a complete-event schema, prefer :meth:`write_event_batch`.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_25write_event = {"write_event", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_25write_event, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_24write_event};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_25write_event(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_schema = 0;
  PyObject *__pyx_v_row = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_event (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_schema,&__pyx_mstate_global->__pyx_n_u_row,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1851, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1851, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1851, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_event", 0) < (0)) __PYX_ERR(0, 1851, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_event", 1, 2, 2, i); __PYX_ERR(0, 1851, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1851, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1851, __pyx_L3_error)
    }
    __pyx_v_schema = ((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)values[0]);
    __pyx_v_row = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_event", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1851, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_schema), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Schema, 0, "schema", 0))) __PYX_ERR(0, 1851, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_24write_event(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_schema, __pyx_v_row);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_24write_event(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_schema, PyObject *__pyx_v_row) {
  PyObject *__pyx_v_payload = NULL;
  nvtxPayloadData_t __pyx_v_data;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8[4];
  uint64_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_event", 0);

  /* "nvtx/_lib/writer.pyx":1885
 *         rates with a complete-event schema, prefer :meth:`write_event_batch`.
 *         """
 *         if schema._domain is not self._domain:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "schema is not registered in this stream's domain")
*/
  __pyx_t_1 = (__pyx_v_schema->_domain != __pyx_v_self->_domain);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1886
 *         """
 *         if schema._domain is not self._domain:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "schema is not registered in this stream's domain")
 *         if schema._kind is None:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_schema_is_not_registered_in_this};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1886, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1886, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1885
 *         rates with a complete-event schema, prefer :meth:`write_event_batch`.
 *         """
 *         if schema._domain is not self._domain:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "schema is not registered in this stream's domain")
*/
  }

  /* "nvtx/_lib/writer.pyx":1888
 *             raise ValueError(
 *                 "schema is not registered in this stream's domain")
 *         if schema._kind is None:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "write_event requires an event schema; pass kind= to "
*/
  __pyx_t_1 = (__pyx_v_schema->_kind == Py_None);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1889
 *                 "schema is not registered in this stream's domain")
 *         if schema._kind is None:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "write_event requires an event schema; pass kind= to "
 *                 "Domain.get_schema"
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_write_event_requires_an_event_sc};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1889, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1889, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1888
 *             raise ValueError(
 *                 "schema is not registered in this stream's domain")
 *         if schema._kind is None:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "write_event requires an event schema; pass kind= to "
*/
  }

  /* "nvtx/_lib/writer.pyx":1893
 *                 "Domain.get_schema"
 *             )
 *         self._ensure_writable()             # <<<<<<<<<<<<<<
 *         payload = np.ascontiguousarray(row, schema._dtype)
 *         if payload.size != 1:
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_ensure_writable(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1893, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1894
 *             )
 *         self._ensure_writable()
 *         payload = np.ascontiguousarray(row, schema._dtype)             # <<<<<<<<<<<<<<
 *         if payload.size != 1:
 *             raise ValueError(
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_ascontiguousarray); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_row, __pyx_v_schema->_dtype};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1894, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v_payload = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":1895
 *         self._ensure_writable()
 *         payload = np.ascontiguousarray(row, schema._dtype)
 *         if payload.size != 1:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 f"write_event expects one row of dtype {schema._dtype}; "
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_2, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1895, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":1896
 *         payload = np.ascontiguousarray(row, schema._dtype)
 *         if payload.size != 1:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 f"write_event expects one row of dtype {schema._dtype}; "
 *                 f"got {payload.size}"
*/
    __pyx_t_6 = NULL;

    /* "nvtx/_lib/writer.pyx":1897
 *         if payload.size != 1:
 *             raise ValueError(
 *                 f"write_event expects one row of dtype {schema._dtype}; "             # <<<<<<<<<<<<<<
 *                 f"got {payload.size}"
 *             )
*/
    __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_schema->_dtype, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1897, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "nvtx/_lib/writer.pyx":1898
 *             raise ValueError(
 *                 f"write_event expects one row of dtype {schema._dtype}; "
 *                 f"got {payload.size}"             # <<<<<<<<<<<<<<
 *             )
 * 
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1898, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = __Pyx_PyObject_FormatSimple(__pyx_t_5, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1898, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_8[0] = __pyx_mstate_global->__pyx_kp_u_write_event_expects_one_row_of_d;
    __pyx_t_8[1] = __pyx_t_3;
    __pyx_t_8[2] = __pyx_mstate_global->__pyx_kp_u_got;
    __pyx_t_8[3] = __pyx_t_7;

    /* "nvtx/_lib/writer.pyx":1897
 *         if payload.size != 1:
 *             raise ValueError(
 *                 f"write_event expects one row of dtype {schema._dtype}; "             # <<<<<<<<<<<<<<
 *                 f"got {payload.size}"
 *             )
*/
    __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_8, 4, 37 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 6 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7));
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1897, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1896, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1896, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1895
 *         self._ensure_writable()
 *         payload = np.ascontiguousarray(row, schema._dtype)
 *         if payload.size != 1:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 f"write_event expects one row of dtype {schema._dtype}; "
*/
  }

  /* "nvtx/_lib/writer.pyx":1902
 * 
 *         cdef nvtxPayloadData_t data
 *         data.schemaId = schema._schema_id             # <<<<<<<<<<<<<<
 *         data.size = payload.nbytes
 *         data.payload = <const void*><size_t>payload.ctypes.data
*/
  __pyx_t_9 = __pyx_v_schema->_schema_id;
  __pyx_v_data.schemaId = __pyx_t_9;

  /* "nvtx/_lib/writer.pyx":1903
 *         cdef nvtxPayloadData_t data
 *         data.schemaId = schema._schema_id
 *         data.size = payload.nbytes             # <<<<<<<<<<<<<<
 *         data.payload = <const void*><size_t>payload.ctypes.data
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_nbytes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1903, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1903, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_data.size = __pyx_t_4;

  /* "nvtx/_lib/writer.pyx":1904
 *         data.schemaId = schema._schema_id
 *         data.size = payload.nbytes
 *         data.payload = <const void*><size_t>payload.ctypes.data             # <<<<<<<<<<<<<<
 * 
 *         cdef nvtxwResultCode_t rc = self._backend._iface.EventWrite(
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1904, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1904, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_5); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1904, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_data.payload = ((void const *)((size_t)__pyx_t_4));

  /* "nvtx/_lib/writer.pyx":1906
 *         data.payload = <const void*><size_t>payload.ctypes.data
 * 
 *         cdef nvtxwResultCode_t rc = self._backend._iface.EventWrite(             # <<<<<<<<<<<<<<
 *             self._writer.stream, &data, 1)
 *         _check(rc, "EventWrite")
*/
  __pyx_v_rc = __pyx_v_self->_backend->_iface->EventWrite(__pyx_v_self->_writer.stream, (&__pyx_v_data), 1);

  /* "nvtx/_lib/writer.pyx":1908
 *         cdef nvtxwResultCode_t rc = self._backend._iface.EventWrite(
 *             self._writer.stream, &data, 1)
 *         _check(rc, "EventWrite")             # <<<<<<<<<<<<<<
 * 
 *     def write_event_batch(
*/
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_EventWrite); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1908, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1851
 *             "nvtxwRangeEndWrite")
 * 
 *     def write_event(self, Schema schema not None, row):             # <<<<<<<<<<<<<<
 *         """
 *         Write an event described by a role-annotated dtype schema.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_event", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_payload);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1910
 *         _check(rc, "EventWrite")
 * 
 *     def write_event_batch(             # <<<<<<<<<<<<<<
 *         self, Schema schema not None, rows, *,
 *         ordering=BatchOrdering.SORTED,
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_27write_event_batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_26write_event_batch, "Stream.write_event_batch(self, Schema schema, rows, *, ordering=BatchOrdering.SORTED)\n\nWrite a batch of complete events.\n\nParameters\n----------\nschema : Schema\n    Complete-event schema returned by :meth:`Domain.get_schema`\n    with a ``kind`` of :attr:`EventKind.MARK`,\n    :attr:`EventKind.RANGE_PUSHPOP`, or\n    :attr:`EventKind.RANGE_STARTEND`, for this stream's domain.\nrows : numpy.ndarray, pandas.DataFrame, or iterable\n    Event rows. DataFrame columns are matched to fields by name.\nordering : BatchOrdering, optional\n    Timestamp ordering of the rows. The default is\n    :attr:`BatchOrdering.SORTED`.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nTypeError\n    If ``schema`` is not a :class:`Schema` or ``ordering`` is not a\n    :class:`BatchOrdering`.\nValueError\n    If ``schema`` belongs to another domain or is not a\n    complete-event schema, or ``rows`` contains no events.\nWriterError\n    If the backend fails to write the batch.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_27write_event_batch = {"write_event_batch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_27write_event_batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_26write_event_batch};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_27write_event_batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_schema = 0;
  PyObject *__pyx_v_rows = 0;
  PyObject *__pyx_v_ordering = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_event_batch (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_schema,&__pyx_mstate_global->__pyx_n_u_rows,&__pyx_mstate_global->__pyx_n_u_ordering,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1910, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1910, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1910, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_event_batch", 0) < (0)) __PYX_ERR(0, 1910, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__8);
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_event_batch", 1, 2, 2, i); __PYX_ERR(0, 1910, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1910, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1910, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__8);
    }
    __pyx_v_schema = ((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)values[0]);
    __pyx_v_rows = values[1];
    __pyx_v_ordering = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_event_batch", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1910, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_event_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_schema), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Schema, 0, "schema", 0))) __PYX_ERR(0, 1911, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_26write_event_batch(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_schema, __pyx_v_rows, __pyx_v_ordering);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_26write_event_batch(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Schema *__pyx_v_schema, PyObject *__pyx_v_rows, PyObject *__pyx_v_ordering) {
  PyObject *__pyx_v_events = NULL;
  void const *__pyx_v_event_data;
  nvtxEventBatch_t __pyx_v_batch;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  uint64_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_event_batch", 0);

  /* "nvtx/_lib/writer.pyx":1943
 *             If the backend fails to write the batch.
 *         """
 *         if not isinstance(ordering, BatchOrdering):             # <<<<<<<<<<<<<<
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")
 *         if schema._domain is not self._domain:
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_BatchOrdering); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1943, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_ordering, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 1943, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (!__pyx_t_2);
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":1944
 *         """
 *         if not isinstance(ordering, BatchOrdering):
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")             # <<<<<<<<<<<<<<
 *         if schema._domain is not self._domain:
 *             raise ValueError(
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_ordering_must_be_an_nvtx_BatchOr};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1944, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1944, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1943
 *             If the backend fails to write the batch.
 *         """
 *         if not isinstance(ordering, BatchOrdering):             # <<<<<<<<<<<<<<
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")
 *         if schema._domain is not self._domain:
*/
  }

  /* "nvtx/_lib/writer.pyx":1945
 *         if not isinstance(ordering, BatchOrdering):
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")
 *         if schema._domain is not self._domain:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "schema is not registered in this stream's domain")
*/
  __pyx_t_3 = (__pyx_v_schema->_domain != __pyx_v_self->_domain);
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":1946
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")
 *         if schema._domain is not self._domain:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "schema is not registered in this stream's domain")
 *         if schema._kind not in _COMPLETE_EVENT_KINDS:
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_schema_is_not_registered_in_this};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1946, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1946, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1945
 *         if not isinstance(ordering, BatchOrdering):
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")
 *         if schema._domain is not self._domain:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "schema is not registered in this stream's domain")
*/
  }

  /* "nvtx/_lib/writer.pyx":1948
 *             raise ValueError(
 *                 "schema is not registered in this stream's domain")
 *         if schema._kind not in _COMPLETE_EVENT_KINDS:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "write_event_batch accepts only complete-event schemas; "
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_COMPLETE_EVENT_KINDS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1948, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__Pyx_PySequence_ContainsTF(__pyx_v_schema->_kind, __pyx_t_1, Py_NE)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1948, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":1949
 *                 "schema is not registered in this stream's domain")
 *         if schema._kind not in _COMPLETE_EVENT_KINDS:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "write_event_batch accepts only complete-event schemas; "
 *                 "pass kind=EventKind.MARK, EventKind.RANGE_PUSHPOP, or "
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_write_event_batch_accepts_only_c};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1949, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1949, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1948
 *             raise ValueError(
 *                 "schema is not registered in this stream's domain")
 *         if schema._kind not in _COMPLETE_EVENT_KINDS:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "write_event_batch accepts only complete-event schemas; "
*/
  }

  /* "nvtx/_lib/writer.pyx":1954
 *                 "EventKind.RANGE_STARTEND to Domain.get_schema"
 *             )
 *         self._ensure_writable()             # <<<<<<<<<<<<<<
 *         events = _batch_rows(rows, schema._dtype)
 *         if events.size == 0:
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_ensure_writable(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1954, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1955
 *             )
 *         self._ensure_writable()
 *         events = _batch_rows(rows, schema._dtype)             # <<<<<<<<<<<<<<
 *         if events.size == 0:
 *             raise ValueError("batch rows must contain at least one event")
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_batch_rows); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1955, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_v_rows, __pyx_v_schema->_dtype};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1955, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_events = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":1956
 *         self._ensure_writable()
 *         events = _batch_rows(rows, schema._dtype)
 *         if events.size == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("batch rows must contain at least one event")
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_events, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1956, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 1956, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":1957
 *         events = _batch_rows(rows, schema._dtype)
 *         if events.size == 0:
 *             raise ValueError("batch rows must contain at least one event")             # <<<<<<<<<<<<<<
 * 
 *         cdef const void* event_data = <const void*><size_t>events.ctypes.data
*/
    __pyx_t_6 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_batch_rows_must_contain_at_least};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1957, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 1957, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":1956
 *         self._ensure_writable()
 *         events = _batch_rows(rows, schema._dtype)
 *         if events.size == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("batch rows must contain at least one event")
 * 
*/
  }

  /* "nvtx/_lib/writer.pyx":1959
 *             raise ValueError("batch rows must contain at least one event")
 * 
 *         cdef const void* event_data = <const void*><size_t>events.ctypes.data             # <<<<<<<<<<<<<<
 *         cdef nvtxEventBatch_t batch
 *         batch.eventSchemaId = schema._schema_id
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_events, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1959, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1959, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1959, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_event_data = ((void const *)((size_t)__pyx_t_5));

  /* "nvtx/_lib/writer.pyx":1961
 *         cdef const void* event_data = <const void*><size_t>events.ctypes.data
 *         cdef nvtxEventBatch_t batch
 *         batch.eventSchemaId = schema._schema_id             # <<<<<<<<<<<<<<
 *         batch.size = events.nbytes
 *         batch.events = event_data
*/
  __pyx_t_7 = __pyx_v_schema->_schema_id;
  __pyx_v_batch.eventSchemaId = __pyx_t_7;

  /* "nvtx/_lib/writer.pyx":1962
 *         cdef nvtxEventBatch_t batch
 *         batch.eventSchemaId = schema._schema_id
 *         batch.size = events.nbytes             # <<<<<<<<<<<<<<
 *         batch.events = event_data
 *         batch.scope = self._scope_id
*/
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_events, __pyx_mstate_global->__pyx_n_u_nbytes); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_6); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 1962, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_batch.size = __pyx_t_5;

  /* "nvtx/_lib/writer.pyx":1963
 *         batch.eventSchemaId = schema._schema_id
 *         batch.size = events.nbytes
 *         batch.events = event_data             # <<<<<<<<<<<<<<
 *         batch.scope = self._scope_id
 *         batch.flags = ordering.value
*/
  __pyx_v_batch.events = __pyx_v_event_data;

  /* "nvtx/_lib/writer.pyx":1964
 *         batch.size = events.nbytes
 *         batch.events = event_data
 *         batch.scope = self._scope_id             # <<<<<<<<<<<<<<
 *         batch.flags = ordering.value
 *         batch.flexData = NULL
*/
  __pyx_t_7 = __pyx_v_self->_scope_id;
  __pyx_v_batch.scope = __pyx_t_7;

  /* "nvtx/_lib/writer.pyx":1965
 *         batch.events = event_data
 *         batch.scope = self._scope_id
 *         batch.flags = ordering.value             # <<<<<<<<<<<<<<
 *         batch.flexData = NULL
 *         batch.flexDataSize = 0
*/
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_ordering, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1965, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyLong_As_uint64_t(__pyx_t_6); if (unlikely((__pyx_t_7 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1965, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_batch.flags = __pyx_t_7;

  /* "nvtx/_lib/writer.pyx":1966
 *         batch.scope = self._scope_id
 *         batch.flags = ordering.value
 *         batch.flexData = NULL             # <<<<<<<<<<<<<<
 *         batch.flexDataSize = 0
 *         batch.flexDataOffset = 0
*/
  __pyx_v_batch.flexData = NULL;

  /* "nvtx/_lib/writer.pyx":1967
 *         batch.flags = ordering.value
 *         batch.flexData = NULL
 *         batch.flexDataSize = 0             # <<<<<<<<<<<<<<
 *         batch.flexDataOffset = 0
 *         cdef nvtxwResultCode_t rc = self._backend._iface.EventBatchWrite(
*/
  __pyx_v_batch.flexDataSize = 0;

  /* "nvtx/_lib/writer.pyx":1968
 *         batch.flexData = NULL
 *         batch.flexDataSize = 0
 *         batch.flexDataOffset = 0             # <<<<<<<<<<<<<<
 *         cdef nvtxwResultCode_t rc = self._backend._iface.EventBatchWrite(
 *             self._writer.stream, &batch)
*/
  __pyx_v_batch.flexDataOffset = 0;

  /* "nvtx/_lib/writer.pyx":1969
 *         batch.flexDataSize = 0
 *         batch.flexDataOffset = 0
 *         cdef nvtxwResultCode_t rc = self._backend._iface.EventBatchWrite(             # <<<<<<<<<<<<<<
 *             self._writer.stream, &batch)
 *         _check(rc, "EventBatchWrite")
*/
  __pyx_v_rc = __pyx_v_self->_backend->_iface->EventBatchWrite(__pyx_v_self->_writer.stream, (&__pyx_v_batch));

  /* "nvtx/_lib/writer.pyx":1971
 *         cdef nvtxwResultCode_t rc = self._backend._iface.EventBatchWrite(
 *             self._writer.stream, &batch)
 *         _check(rc, "EventBatchWrite")             # <<<<<<<<<<<<<<
 * 
 *     def write_counter_sample(
*/
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_EventBatchWrite); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1971, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1910
 *         _check(rc, "EventWrite")
 * 
 *     def write_event_batch(             # <<<<<<<<<<<<<<
 *         self, Schema schema not None, rows, *,
 *         ordering=BatchOrdering.SORTED,
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_event_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_events);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":1973
 *         _check(rc, "EventBatchWrite")
 * 
 *     def write_counter_sample(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, int64_t timestamp, value
 *     ):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_29write_counter_sample(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_28write_counter_sample, "Stream.write_counter_sample(self, Counter counter, int64_t timestamp, value)\n\nWrite a single counter sample.\n\nParameters\n----------\ncounter : Counter\n    Counter returned by :meth:`Domain.get_counter` for this stream's\n    domain.\ntimestamp : int\n    Sample timestamp in the stream's time domain.\nvalue\n    Sample value matching the counter's registered layout. For a\n    counter group, this contains one row of field values.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nTypeError\n    If ``counter`` is not a :class:`Counter`.\nValueError\n    If ``counter`` belongs to another domain, its layout embeds a\n    counter timestamp, or ``value`` does not contain exactly one\n    sample.\nWriterError\n    If the backend fails to write the sample.\n\nNotes\n-----\nA structured sample is converted to a contiguous array on each call.\nFor high write rates, prefer :meth:`write_counter_batch`.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_29write_counter_sample = {"write_counter_sample", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_29write_counter_sample, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_28write_counter_sample};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_29write_counter_sample(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter = 0;
  int64_t __pyx_v_timestamp;
  PyObject *__pyx_v_value = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_counter_sample (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_counter,&__pyx_mstate_global->__pyx_n_u_timestamp,&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1973, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1973, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1973, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1973, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_counter_sample", 0) < (0)) __PYX_ERR(0, 1973, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_counter_sample", 1, 3, 3, i); __PYX_ERR(0, 1973, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1973, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1973, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1973, __pyx_L3_error)
    }
    __pyx_v_counter = ((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)values[0]);
    __pyx_v_timestamp = __Pyx_PyLong_As_int64_t(values[1]); if (unlikely((__pyx_v_timestamp == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1974, __pyx_L3_error)
    __pyx_v_value = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_counter_sample", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 1973, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_counter_sample", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_counter), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Counter, 0, "counter", 0))) __PYX_ERR(0, 1974, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_28write_counter_sample(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_counter, __pyx_v_timestamp, __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_28write_counter_sample(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter, int64_t __pyx_v_timestamp, PyObject *__pyx_v_value) {
  nvtxwInterface_v2_t const *__pyx_v_iface;
  int64_t __pyx_v_i64;
  double __pyx_v_f64;
  nvtxwResultCode_t __pyx_v_rc;
  PyObject *__pyx_v_payload = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  nvtxwInterface_v2_t const *__pyx_t_5;
  int64_t __pyx_t_6;
  double __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11[4];
  size_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_counter_sample", 0);

  /* "nvtx/_lib/writer.pyx":2008
 *         For high write rates, prefer :meth:`write_counter_batch`.
 *         """
 *         self._ensure_counter(counter)             # <<<<<<<<<<<<<<
 *         self._ensure_writable()
 *         if counter._timestamp_field is not None:
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_ensure_counter(__pyx_v_self, __pyx_v_counter); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2008, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":2009
 *         """
 *         self._ensure_counter(counter)
 *         self._ensure_writable()             # <<<<<<<<<<<<<<
 *         if counter._timestamp_field is not None:
 *             raise ValueError(
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_ensure_writable(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2009, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":2010
 *         self._ensure_counter(counter)
 *         self._ensure_writable()
 *         if counter._timestamp_field is not None:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "write_counter_sample does not support a counter layout that "
*/
  __pyx_t_1 = (__pyx_v_counter->_timestamp_field != Py_None);
  if (unlikely(__pyx_t_1)) {

    /* "nvtx/_lib/writer.pyx":2011
 *         self._ensure_writable()
 *         if counter._timestamp_field is not None:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "write_counter_sample does not support a counter layout that "
 *                 "embeds an EntryKind.COUNTER_TIMESTAMP field; use "
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_write_counter_sample_does_not_su};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2011, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 2011, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":2010
 *         self._ensure_counter(counter)
 *         self._ensure_writable()
 *         if counter._timestamp_field is not None:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "write_counter_sample does not support a counter layout that "
*/
  }

  /* "nvtx/_lib/writer.pyx":2016
 *                 "write_counter_batch instead"
 *             )
 *         cdef const nvtxwInterface_v2_t* iface = self._backend._iface             # <<<<<<<<<<<<<<
 *         cdef int64_t i64
 *         cdef double f64
*/
  __pyx_t_5 = __pyx_v_self->_backend->_iface;
  __pyx_v_iface = __pyx_t_5;

  /* "nvtx/_lib/writer.pyx":2020
 *         cdef double f64
 *         cdef nvtxwResultCode_t rc
 *         if counter._schema_id == NVTX_PAYLOAD_ENTRY_TYPE_INT64:             # <<<<<<<<<<<<<<
 *             i64 = value
 *             rc = nvtxwCounterInt64Write(
*/
  __pyx_t_1 = (__pyx_v_counter->_schema_id == NVTX_PAYLOAD_ENTRY_TYPE_INT64);
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":2021
 *         cdef nvtxwResultCode_t rc
 *         if counter._schema_id == NVTX_PAYLOAD_ENTRY_TYPE_INT64:
 *             i64 = value             # <<<<<<<<<<<<<<
 *             rc = nvtxwCounterInt64Write(
 *                 iface, self._writer.stream, timestamp,
*/
    __pyx_t_6 = __Pyx_PyLong_As_int64_t(__pyx_v_value); if (unlikely((__pyx_t_6 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2021, __pyx_L1_error)
    __pyx_v_i64 = __pyx_t_6;

    /* "nvtx/_lib/writer.pyx":2022
 *         if counter._schema_id == NVTX_PAYLOAD_ENTRY_TYPE_INT64:
 *             i64 = value
 *             rc = nvtxwCounterInt64Write(             # <<<<<<<<<<<<<<
 *                 iface, self._writer.stream, timestamp,
 *                 counter._counter_id, i64)
*/
    __pyx_v_rc = nvtxwCounterInt64Write(__pyx_v_iface, __pyx_v_self->_writer.stream, __pyx_v_timestamp, __pyx_v_counter->_counter_id, __pyx_v_i64);

    /* "nvtx/_lib/writer.pyx":2020
 *         cdef double f64
 *         cdef nvtxwResultCode_t rc
 *         if counter._schema_id == NVTX_PAYLOAD_ENTRY_TYPE_INT64:             # <<<<<<<<<<<<<<
 *             i64 = value
 *             rc = nvtxwCounterInt64Write(
*/
    goto __pyx_L4;
  }

  /* "nvtx/_lib/writer.pyx":2025
 *                 iface, self._writer.stream, timestamp,
 *                 counter._counter_id, i64)
 *         elif counter._schema_id == NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64:             # <<<<<<<<<<<<<<
 *             f64 = value
 *             rc = nvtxwCounterFloat64Write(
*/
  __pyx_t_1 = (__pyx_v_counter->_schema_id == NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64);
  if (__pyx_t_1) {

    /* "nvtx/_lib/writer.pyx":2026
 *                 counter._counter_id, i64)
 *         elif counter._schema_id == NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64:
 *             f64 = value             # <<<<<<<<<<<<<<
 *             rc = nvtxwCounterFloat64Write(
 *                 iface, self._writer.stream, timestamp,
*/
    __pyx_t_7 = __Pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_7 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 2026, __pyx_L1_error)
    __pyx_v_f64 = __pyx_t_7;

    /* "nvtx/_lib/writer.pyx":2027
 *         elif counter._schema_id == NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64:
 *             f64 = value
 *             rc = nvtxwCounterFloat64Write(             # <<<<<<<<<<<<<<
 *                 iface, self._writer.stream, timestamp,
 *                 counter._counter_id, f64)
*/
    __pyx_v_rc = nvtxwCounterFloat64Write(__pyx_v_iface, __pyx_v_self->_writer.stream, __pyx_v_timestamp, __pyx_v_counter->_counter_id, __pyx_v_f64);

    /* "nvtx/_lib/writer.pyx":2025
 *                 iface, self._writer.stream, timestamp,
 *                 counter._counter_id, i64)
 *         elif counter._schema_id == NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64:             # <<<<<<<<<<<<<<
 *             f64 = value
 *             rc = nvtxwCounterFloat64Write(
*/
    goto __pyx_L4;
  }

  /* "nvtx/_lib/writer.pyx":2031
 *                 counter._counter_id, f64)
 *         else:
 *             payload = np.ascontiguousarray(value, counter._dtype)             # <<<<<<<<<<<<<<
 *             if payload.size != 1:
 *                 raise ValueError(
*/
  /*else*/ {
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2031, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_ascontiguousarray); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2031, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_9);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_value, __pyx_v_counter->_dtype};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2031, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_v_payload = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "nvtx/_lib/writer.pyx":2032
 *         else:
 *             payload = np.ascontiguousarray(value, counter._dtype)
 *             if payload.size != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     f"write_counter_sample expects one sample of dtype "
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2032, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_2, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 2032, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(__pyx_t_1)) {

      /* "nvtx/_lib/writer.pyx":2033
 *             payload = np.ascontiguousarray(value, counter._dtype)
 *             if payload.size != 1:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     f"write_counter_sample expects one sample of dtype "
 *                     f"{counter._dtype}; "
*/
      __pyx_t_9 = NULL;

      /* "nvtx/_lib/writer.pyx":2035
 *                 raise ValueError(
 *                     f"write_counter_sample expects one sample of dtype "
 *                     f"{counter._dtype}; "             # <<<<<<<<<<<<<<
 *                     f"got {payload.size}"
 *                 )
*/
      __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_counter->_dtype, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 2035, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);

      /* "nvtx/_lib/writer.pyx":2036
 *                     f"write_counter_sample expects one sample of dtype "
 *                     f"{counter._dtype}; "
 *                     f"got {payload.size}"             # <<<<<<<<<<<<<<
 *                 )
 *             rc = iface.CounterWrite(
*/
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2036, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_10 = __Pyx_PyObject_FormatSimple(__pyx_t_8, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 2036, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_11[0] = __pyx_mstate_global->__pyx_kp_u_write_counter_sample_expects_one;
      __pyx_t_11[1] = __pyx_t_3;
      __pyx_t_11[2] = __pyx_mstate_global->__pyx_kp_u_got;
      __pyx_t_11[3] = __pyx_t_10;

      /* "nvtx/_lib/writer.pyx":2034
 *             if payload.size != 1:
 *                 raise ValueError(
 *                     f"write_counter_sample expects one sample of dtype "             # <<<<<<<<<<<<<<
 *                     f"{counter._dtype}; "
 *                     f"got {payload.size}"
*/
      __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_11, 4, 49 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 6 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10));
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2034, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_4 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_8};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2033, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 2033, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":2032
 *         else:
 *             payload = np.ascontiguousarray(value, counter._dtype)
 *             if payload.size != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     f"write_counter_sample expects one sample of dtype "
*/
    }

    /* "nvtx/_lib/writer.pyx":2040
 *             rc = iface.CounterWrite(
 *                 self._writer.stream, timestamp, counter._counter_id,
 *                 <const void*><size_t>payload.ctypes.data, payload.nbytes)             # <<<<<<<<<<<<<<
 *         _check(rc, "CounterWrite")
 * 
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2040, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2040, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_8); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2040, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_nbytes); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 2040, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_12 = __Pyx_PyLong_As_size_t(__pyx_t_8); if (unlikely((__pyx_t_12 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2040, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "nvtx/_lib/writer.pyx":2038
 *                     f"got {payload.size}"
 *                 )
 *             rc = iface.CounterWrite(             # <<<<<<<<<<<<<<
 *                 self._writer.stream, timestamp, counter._counter_id,
 *                 <const void*><size_t>payload.ctypes.data, payload.nbytes)
*/
    __pyx_v_rc = __pyx_v_iface->CounterWrite(__pyx_v_self->_writer.stream, __pyx_v_timestamp, __pyx_v_counter->_counter_id, ((void const *)((size_t)__pyx_t_4)), __pyx_t_12);
  }
  __pyx_L4:;

  /* "nvtx/_lib/writer.pyx":2041
 *                 self._writer.stream, timestamp, counter._counter_id,
 *                 <const void*><size_t>payload.ctypes.data, payload.nbytes)
 *         _check(rc, "CounterWrite")             # <<<<<<<<<<<<<<
 * 
 *     def write_counter_sample_no_value(
*/
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_rc, __pyx_mstate_global->__pyx_n_u_CounterWrite); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2041, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1973
 *         _check(rc, "EventBatchWrite")
 * 
 *     def write_counter_sample(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, int64_t timestamp, value
 *     ):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_counter_sample", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_payload);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":2043
 *         _check(rc, "CounterWrite")
 * 
 *     def write_counter_sample_no_value(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, int64_t timestamp, reason
 *     ):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_31write_counter_sample_no_value(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_30write_counter_sample_no_value, "Stream.write_counter_sample_no_value(self, Counter counter, int64_t timestamp, reason)\n\nWrite a counter sample with no explicit value.\n\nParameters\n----------\ncounter : Counter\n    Counter returned by :meth:`Domain.get_counter` for this stream's\n    domain.\ntimestamp : int\n    Sample timestamp in the stream's time domain.\nreason : CounterNoValueReason\n    Reason the sample has no explicit value.\n\nRaises\n------\nRuntimeError\n    If the stream is not open or the backend is closed.\nTypeError\n    If ``counter`` is not a :class:`Counter` or ``reason`` is not a\n    :class:`nvtx.CounterNoValueReason`.\nValueError\n    If ``counter`` belongs to another domain.\nWriterError\n    If the backend fails to write the sample.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_31write_counter_sample_no_value = {"write_counter_sample_no_value", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_31write_counter_sample_no_value, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_30write_counter_sample_no_value};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_31write_counter_sample_no_value(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter = 0;
  int64_t __pyx_v_timestamp;
  PyObject *__pyx_v_reason = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_counter_sample_no_value (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_counter,&__pyx_mstate_global->__pyx_n_u_timestamp,&__pyx_mstate_global->__pyx_n_u_reason,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2043, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2043, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2043, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2043, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_counter_sample_no_value", 0) < (0)) __PYX_ERR(0, 2043, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_counter_sample_no_value", 1, 3, 3, i); __PYX_ERR(0, 2043, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2043, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2043, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 2043, __pyx_L3_error)
    }
    __pyx_v_counter = ((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)values[0]);
    __pyx_v_timestamp = __Pyx_PyLong_As_int64_t(values[1]); if (unlikely((__pyx_v_timestamp == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2044, __pyx_L3_error)
    __pyx_v_reason = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_counter_sample_no_value", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 2043, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_counter_sample_no_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_counter), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Counter, 0, "counter", 0))) __PYX_ERR(0, 2044, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_30write_counter_sample_no_value(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_counter, __pyx_v_timestamp, __pyx_v_reason);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_30write_counter_sample_no_value(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter, int64_t __pyx_v_timestamp, PyObject *__pyx_v_reason) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  uint8_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_counter_sample_no_value", 0);

  /* "nvtx/_lib/writer.pyx":2071
 *             If the backend fails to write the sample.
 *         """
 *         self._ensure_counter(counter)             # <<<<<<<<<<<<<<
 *         if not isinstance(reason, CounterNoValueReason):
 *             raise TypeError("reason must be an nvtx.CounterNoValueReason")
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_ensure_counter(__pyx_v_self, __pyx_v_counter); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2071, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":2072
 *         """
 *         self._ensure_counter(counter)
 *         if not isinstance(reason, CounterNoValueReason):             # <<<<<<<<<<<<<<
 *             raise TypeError("reason must be an nvtx.CounterNoValueReason")
 *         self._ensure_writable()
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_CounterNoValueReason); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2072, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_reason, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 2072, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (!__pyx_t_2);
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":2073
 *         self._ensure_counter(counter)
 *         if not isinstance(reason, CounterNoValueReason):
 *             raise TypeError("reason must be an nvtx.CounterNoValueReason")             # <<<<<<<<<<<<<<
 *         self._ensure_writable()
 *         _check(
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_reason_must_be_an_nvtx_CounterNo};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2073, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2073, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":2072
 *         """
 *         self._ensure_counter(counter)
 *         if not isinstance(reason, CounterNoValueReason):             # <<<<<<<<<<<<<<
 *             raise TypeError("reason must be an nvtx.CounterNoValueReason")
 *         self._ensure_writable()
*/
  }

  /* "nvtx/_lib/writer.pyx":2074
 *         if not isinstance(reason, CounterNoValueReason):
 *             raise TypeError("reason must be an nvtx.CounterNoValueReason")
 *         self._ensure_writable()             # <<<<<<<<<<<<<<
 *         _check(
 *             self._backend._iface.CounterNoValueWrite(
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_ensure_writable(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2074, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":2078
 *             self._backend._iface.CounterNoValueWrite(
 *                 self._writer.stream, timestamp, counter._counter_id,
 *                 <uint8_t>reason.value),             # <<<<<<<<<<<<<<
 *             "CounterNoValueWrite")
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_reason, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2078, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyLong_As_uint8_t(__pyx_t_1); if (unlikely((__pyx_t_6 == ((uint8_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2078, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":2075
 *             raise TypeError("reason must be an nvtx.CounterNoValueReason")
 *         self._ensure_writable()
 *         _check(             # <<<<<<<<<<<<<<
 *             self._backend._iface.CounterNoValueWrite(
 *                 self._writer.stream, timestamp, counter._counter_id,
*/
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_self->_backend->_iface->CounterNoValueWrite(__pyx_v_self->_writer.stream, __pyx_v_timestamp, __pyx_v_counter->_counter_id, ((uint8_t)__pyx_t_6)), __pyx_mstate_global->__pyx_n_u_CounterNoValueWrite); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2075, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":2043
 *         _check(rc, "CounterWrite")
 * 
 *     def write_counter_sample_no_value(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, int64_t timestamp, reason
 *     ):
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_counter_sample_no_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/writer.pyx":2081
 *             "CounterNoValueWrite")
 * 
 *     def write_counter_batch(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, data, *,
 *         timestamps=None, ordering=BatchOrdering.SORTED,
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_33write_counter_batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_32write_counter_batch, "Stream.write_counter_batch(self, Counter counter, data, *, timestamps=None, ordering=BatchOrdering.SORTED)\n\nWrite a batch of counter samples.\n\nTimestamps come from one of two sources, selected by the counter's\nregistered layout:\n\n* **Separate**: ``timestamps`` is an iterable of per-sample\n  timestamps, same length as ``data``, and ``data`` carries the\n  values only.  Required unless the layout embeds a timestamp field.\n* **Embedded**: the counter-group layout has an\n  ``entry_kind=nvtx.EntryKind.COUNTER_TIMESTAMP`` field and each row\n  in ``data`` carries it; ``timestamps`` must be omitted.\n\nParameters\n----------\ncounter : Counter\n    Counter returned by :meth:`Domain.get_counter` for this stream's\n    domain.\ndata : numpy.ndarray, pandas.DataFrame, or iterable\n    Sample rows. For scalar counters, this is an iterable of values.\n    DataFrame columns are matched to counter fields by name.\ntimestamps : iterable of int, optional\n    Timestamp for each sample. Required unless the counter layout\n    contains an embedded counter timestamp and otherwise must be\n    omitted.\nordering : BatchOrdering, optional\n    Timestamp ordering of the samples. The default is\n    :attr:`BatchOrdering.SORTED`.\n\nRaises\n------\nRuntimeError\n    If the stream is not open, the backend is closed, or NumPy is not\n    installed.\nTypeError\n    If ``counter`` is not a :class:`Counter` or ``ordering`` is not a\n    :class:`BatchOrdering`.\nValueError\n    If ``counter`` belongs to another domain, the timestamp source is\n    invalid, ``data`` is empty, ``data`` or ``timestamps`` is not\n    one-dimensional, or their lengths differ.\nWriterError\n    If the backend fails to write the batch.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_33write_counter_batch = {"write_counter_batch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_33write_counter_batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_32write_counter_batch};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_33write_counter_batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_timestamps = 0;
  PyObject *__pyx_v_ordering = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_counter_batch (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_counter,&__pyx_mstate_global->__pyx_n_u_data,&__pyx_mstate_global->__pyx_n_u_timestamps,&__pyx_mstate_global->__pyx_n_u_ordering,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 2081, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2081, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2081, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "write_counter_batch", 0) < (0)) __PYX_ERR(0, 2081, __pyx_L3_error)

      /* "nvtx/_lib/writer.pyx":2083
 *     def write_counter_batch(
 *         self, Counter counter not None, data, *,
 *         timestamps=None, ordering=BatchOrdering.SORTED,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__9);
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("write_counter_batch", 1, 2, 2, i); __PYX_ERR(0, 2081, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 2081, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 2081, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_mstate_global->__pyx_k__9);
    }
    __pyx_v_counter = ((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)values[0]);
    __pyx_v_data = values[1];
    __pyx_v_timestamps = values[2];
    __pyx_v_ordering = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_counter_batch", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 2081, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_counter_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_counter), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Counter, 0, "counter", 0))) __PYX_ERR(0, 2082, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_32write_counter_batch(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v_counter, __pyx_v_data, __pyx_v_timestamps, __pyx_v_ordering);

  /* "nvtx/_lib/writer.pyx":2081
 *             "CounterNoValueWrite")
 * 
 *     def write_counter_batch(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, data, *,
 *         timestamps=None, ordering=BatchOrdering.SORTED,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_32write_counter_batch(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *__pyx_v_counter, PyObject *__pyx_v_data, PyObject *__pyx_v_timestamps, PyObject *__pyx_v_ordering) {
  PyObject *__pyx_v_dt = NULL;
  PyObject *__pyx_v_samples = NULL;
  PyObject *__pyx_v_timestamps_np = 0;
  nvtxCounterBatch_t __pyx_v_batch;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  uint64_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_counter_batch", 0);

  /* "nvtx/_lib/writer.pyx":2129
 *             If the backend fails to write the batch.
 *         """
 *         if not isinstance(ordering, BatchOrdering):             # <<<<<<<<<<<<<<
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")
 *         self._ensure_counter(counter)
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_BatchOrdering); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_ordering, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 2129, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (!__pyx_t_2);
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":2130
 *         """
 *         if not isinstance(ordering, BatchOrdering):
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")             # <<<<<<<<<<<<<<
 *         self._ensure_counter(counter)
 *         self._ensure_writable()
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_ordering_must_be_an_nvtx_BatchOr};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2130, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2130, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":2129
 *             If the backend fails to write the batch.
 *         """
 *         if not isinstance(ordering, BatchOrdering):             # <<<<<<<<<<<<<<
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")
 *         self._ensure_counter(counter)
*/
  }

  /* "nvtx/_lib/writer.pyx":2131
 *         if not isinstance(ordering, BatchOrdering):
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")
 *         self._ensure_counter(counter)             # <<<<<<<<<<<<<<
 *         self._ensure_writable()
 *         if np is None:
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_ensure_counter(__pyx_v_self, __pyx_v_counter); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2131, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":2132
 *             raise TypeError("ordering must be an nvtx.BatchOrdering")
 *         self._ensure_counter(counter)
 *         self._ensure_writable()             # <<<<<<<<<<<<<<
 *         if np is None:
 *             raise RuntimeError("Install numpy to use write_counter_batch.")
*/
  ((struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream *)__pyx_v_self->__pyx_vtab)->_ensure_writable(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2132, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":2133
 *         self._ensure_counter(counter)
 *         self._ensure_writable()
 *         if np is None:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Install numpy to use write_counter_batch.")
 *         dt = counter._dtype
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":2134
 *         self._ensure_writable()
 *         if np is None:
 *             raise RuntimeError("Install numpy to use write_counter_batch.")             # <<<<<<<<<<<<<<
 *         dt = counter._dtype
 *         if counter._timestamp_field is not None:
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Install_numpy_to_use_write_count};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2134, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2134, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":2133
 *         self._ensure_counter(counter)
 *         self._ensure_writable()
 *         if np is None:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Install numpy to use write_counter_batch.")
 *         dt = counter._dtype
*/
  }

  /* "nvtx/_lib/writer.pyx":2135
 *         if np is None:
 *             raise RuntimeError("Install numpy to use write_counter_batch.")
 *         dt = counter._dtype             # <<<<<<<<<<<<<<
 *         if counter._timestamp_field is not None:
 *             if timestamps is not None:
*/
  __pyx_t_1 = __pyx_v_counter->_dtype;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_dt = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/writer.pyx":2136
 *             raise RuntimeError("Install numpy to use write_counter_batch.")
 *         dt = counter._dtype
 *         if counter._timestamp_field is not None:             # <<<<<<<<<<<<<<
 *             if timestamps is not None:
 *                 raise ValueError(
*/
  __pyx_t_3 = (__pyx_v_counter->_timestamp_field != Py_None);
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":2137
 *         dt = counter._dtype
 *         if counter._timestamp_field is not None:
 *             if timestamps is not None:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "timestamps is mutually exclusive with a counter layout "
*/
    __pyx_t_3 = (__pyx_v_timestamps != Py_None);
    if (unlikely(__pyx_t_3)) {

      /* "nvtx/_lib/writer.pyx":2138
 *         if counter._timestamp_field is not None:
 *             if timestamps is not None:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     "timestamps is mutually exclusive with a counter layout "
 *                     "that embeds an EntryKind.COUNTER_TIMESTAMP field"
*/
      __pyx_t_4 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_timestamps_is_mutually_exclusive};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2138, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 2138, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":2137
 *         dt = counter._dtype
 *         if counter._timestamp_field is not None:
 *             if timestamps is not None:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "timestamps is mutually exclusive with a counter layout "
*/
    }

    /* "nvtx/_lib/writer.pyx":2136
 *             raise RuntimeError("Install numpy to use write_counter_batch.")
 *         dt = counter._dtype
 *         if counter._timestamp_field is not None:             # <<<<<<<<<<<<<<
 *             if timestamps is not None:
 *                 raise ValueError(
*/
    goto __pyx_L5;
  }

  /* "nvtx/_lib/writer.pyx":2142
 *                     "that embeds an EntryKind.COUNTER_TIMESTAMP field"
 *                 )
 *         elif timestamps is None:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "write_counter_batch requires timestamps unless the counter "
*/
  __pyx_t_3 = (__pyx_v_timestamps == Py_None);
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":2143
 *                 )
 *         elif timestamps is None:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "write_counter_batch requires timestamps unless the counter "
 *                 "layout embeds an EntryKind.COUNTER_TIMESTAMP field"
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_write_counter_batch_requires_tim};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2143, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 2143, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":2142
 *                     "that embeds an EntryKind.COUNTER_TIMESTAMP field"
 *                 )
 *         elif timestamps is None:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "write_counter_batch requires timestamps unless the counter "
*/
  }
  __pyx_L5:;

  /* "nvtx/_lib/writer.pyx":2148
 *             )
 * 
 *         if dt.fields is not None:             # <<<<<<<<<<<<<<
 *             samples = _batch_rows(data, dt)
 *         else:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":2149
 * 
 *         if dt.fields is not None:
 *             samples = _batch_rows(data, dt)             # <<<<<<<<<<<<<<
 *         else:
 *             if isinstance(data, np.ndarray):
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_batch_rows); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_v_data, __pyx_v_dt};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2149, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_samples = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "nvtx/_lib/writer.pyx":2148
 *             )
 * 
 *         if dt.fields is not None:             # <<<<<<<<<<<<<<
 *             samples = _batch_rows(data, dt)
 *         else:
*/
    goto __pyx_L7;
  }

  /* "nvtx/_lib/writer.pyx":2151
 *             samples = _batch_rows(data, dt)
 *         else:
 *             if isinstance(data, np.ndarray):             # <<<<<<<<<<<<<<
 *                 samples = np.ascontiguousarray(data, dt)
 *             else:
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_3 = PyObject_IsInstance(__pyx_v_data, __pyx_t_6); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 2151, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_3) {

      /* "nvtx/_lib/writer.pyx":2152
 *         else:
 *             if isinstance(data, np.ndarray):
 *                 samples = np.ascontiguousarray(data, dt)             # <<<<<<<<<<<<<<
 *             else:
 *                 samples = np.fromiter(data, dtype=dt)
*/
      __pyx_t_1 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2152, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ascontiguousarray); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2152, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_5 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
        assert(__pyx_t_1);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
        __pyx_t_5 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_v_data, __pyx_v_dt};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2152, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __pyx_v_samples = __pyx_t_6;
      __pyx_t_6 = 0;

      /* "nvtx/_lib/writer.pyx":2151
 *             samples = _batch_rows(data, dt)
 *         else:
 *             if isinstance(data, np.ndarray):             # <<<<<<<<<<<<<<
 *                 samples = np.ascontiguousarray(data, dt)
 *             else:
*/
      goto __pyx_L8;
    }

    /* "nvtx/_lib/writer.pyx":2154
 *                 samples = np.ascontiguousarray(data, dt)
 *             else:
 *                 samples = np.fromiter(data, dtype=dt)             # <<<<<<<<<<<<<<
 *             if samples.ndim != 1:
 *                 raise ValueError("batch data must be one-dimensional")
*/
    /*else*/ {
      __pyx_t_7 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2154, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_fromiter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2154, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_5 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_5 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_7, __pyx_v_data};
        __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2154, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_v_dt, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 2154, __pyx_L1_error)
        __pyx_t_6 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2154, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __pyx_v_samples = __pyx_t_6;
      __pyx_t_6 = 0;
    }
    __pyx_L8:;

    /* "nvtx/_lib/writer.pyx":2155
 *             else:
 *                 samples = np.fromiter(data, dtype=dt)
 *             if samples.ndim != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError("batch data must be one-dimensional")
 *         if samples.size == 0:
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_samples, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2155, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_6, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2155, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_t_3)) {

      /* "nvtx/_lib/writer.pyx":2156
 *                 samples = np.fromiter(data, dtype=dt)
 *             if samples.ndim != 1:
 *                 raise ValueError("batch data must be one-dimensional")             # <<<<<<<<<<<<<<
 *         if samples.size == 0:
 *             raise ValueError("batch data must contain at least one sample")
*/
      __pyx_t_4 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_batch_data_must_be_one_dimension};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2156, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 2156, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":2155
 *             else:
 *                 samples = np.fromiter(data, dtype=dt)
 *             if samples.ndim != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError("batch data must be one-dimensional")
 *         if samples.size == 0:
*/
    }
  }
  __pyx_L7:;

  /* "nvtx/_lib/writer.pyx":2157
 *             if samples.ndim != 1:
 *                 raise ValueError("batch data must be one-dimensional")
 *         if samples.size == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("batch data must contain at least one sample")
 * 
*/
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_samples, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_6, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2157, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "nvtx/_lib/writer.pyx":2158
 *                 raise ValueError("batch data must be one-dimensional")
 *         if samples.size == 0:
 *             raise ValueError("batch data must contain at least one sample")             # <<<<<<<<<<<<<<
 * 
 *         cdef object timestamps_np = None
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_batch_data_must_contain_at_least};
      __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2158, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 2158, __pyx_L1_error)

    /* "nvtx/_lib/writer.pyx":2157
 *             if samples.ndim != 1:
 *                 raise ValueError("batch data must be one-dimensional")
 *         if samples.size == 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("batch data must contain at least one sample")
 * 
*/
  }

  /* "nvtx/_lib/writer.pyx":2160
 *             raise ValueError("batch data must contain at least one sample")
 * 
 *         cdef object timestamps_np = None             # <<<<<<<<<<<<<<
 *         cdef nvtxCounterBatch_t batch
 *         batch.counterId = counter._counter_id
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_timestamps_np = Py_None;

  /* "nvtx/_lib/writer.pyx":2162
 *         cdef object timestamps_np = None
 *         cdef nvtxCounterBatch_t batch
 *         batch.counterId = counter._counter_id             # <<<<<<<<<<<<<<
 *         batch.counters = <const void*><size_t>samples.ctypes.data
 *         batch.countersSize = samples.nbytes
*/
  __pyx_t_8 = __pyx_v_counter->_counter_id;
  __pyx_v_batch.counterId = __pyx_t_8;

  /* "nvtx/_lib/writer.pyx":2163
 *         cdef nvtxCounterBatch_t batch
 *         batch.counterId = counter._counter_id
 *         batch.counters = <const void*><size_t>samples.ctypes.data             # <<<<<<<<<<<<<<
 *         batch.countersSize = samples.nbytes
 *         batch.flags = ordering.value
*/
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_samples, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_4); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2163, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_batch.counters = ((void const *)((size_t)__pyx_t_5));

  /* "nvtx/_lib/writer.pyx":2164
 *         batch.counterId = counter._counter_id
 *         batch.counters = <const void*><size_t>samples.ctypes.data
 *         batch.countersSize = samples.nbytes             # <<<<<<<<<<<<<<
 *         batch.flags = ordering.value
 *         batch.timestamps = NULL
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_samples, __pyx_mstate_global->__pyx_n_u_nbytes); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_4); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2164, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_batch.countersSize = __pyx_t_5;

  /* "nvtx/_lib/writer.pyx":2165
 *         batch.counters = <const void*><size_t>samples.ctypes.data
 *         batch.countersSize = samples.nbytes
 *         batch.flags = ordering.value             # <<<<<<<<<<<<<<
 *         batch.timestamps = NULL
 *         batch.timestampsSize = 0
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_ordering, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyLong_As_uint64_t(__pyx_t_4); if (unlikely((__pyx_t_8 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 2165, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_batch.flags = __pyx_t_8;

  /* "nvtx/_lib/writer.pyx":2166
 *         batch.countersSize = samples.nbytes
 *         batch.flags = ordering.value
 *         batch.timestamps = NULL             # <<<<<<<<<<<<<<
 *         batch.timestampsSize = 0
 *         if timestamps is not None:
*/
  __pyx_v_batch.timestamps = NULL;

  /* "nvtx/_lib/writer.pyx":2167
 *         batch.flags = ordering.value
 *         batch.timestamps = NULL
 *         batch.timestampsSize = 0             # <<<<<<<<<<<<<<
 *         if timestamps is not None:
 *             if isinstance(timestamps, np.ndarray):
*/
  __pyx_v_batch.timestampsSize = 0;

  /* "nvtx/_lib/writer.pyx":2168
 *         batch.timestamps = NULL
 *         batch.timestampsSize = 0
 *         if timestamps is not None:             # <<<<<<<<<<<<<<
 *             if isinstance(timestamps, np.ndarray):
 *                 timestamps_np = np.ascontiguousarray(
*/
  __pyx_t_3 = (__pyx_v_timestamps != Py_None);
  if (__pyx_t_3) {

    /* "nvtx/_lib/writer.pyx":2169
 *         batch.timestampsSize = 0
 *         if timestamps is not None:
 *             if isinstance(timestamps, np.ndarray):             # <<<<<<<<<<<<<<
 *                 timestamps_np = np.ascontiguousarray(
 *                     timestamps, dtype=np.int64)
*/
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_3 = PyObject_IsInstance(__pyx_v_timestamps, __pyx_t_6); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 2169, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_3) {

      /* "nvtx/_lib/writer.pyx":2170
 *         if timestamps is not None:
 *             if isinstance(timestamps, np.ndarray):
 *                 timestamps_np = np.ascontiguousarray(             # <<<<<<<<<<<<<<
 *                     timestamps, dtype=np.int64)
 *             else:
*/
      __pyx_t_4 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2170, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ascontiguousarray); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 2170, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "nvtx/_lib/writer.pyx":2171
 *             if isinstance(timestamps, np.ndarray):
 *                 timestamps_np = np.ascontiguousarray(
 *                     timestamps, dtype=np.int64)             # <<<<<<<<<<<<<<
 *             else:
 *                 timestamps_np = np.fromiter(timestamps, dtype=np.int64)
*/
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2171, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_int64); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2171, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_5 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_7);
        assert(__pyx_t_4);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
        __pyx_t_5 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_4, __pyx_v_timestamps};
        __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_9, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 2170, __pyx_L1_error)
        __pyx_t_6 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_DECREF_SET(__pyx_v_timestamps_np, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "nvtx/_lib/writer.pyx":2169
 *         batch.timestampsSize = 0
 *         if timestamps is not None:
 *             if isinstance(timestamps, np.ndarray):             # <<<<<<<<<<<<<<
 *                 timestamps_np = np.ascontiguousarray(
 *                     timestamps, dtype=np.int64)
*/
      goto __pyx_L12;
    }

    /* "nvtx/_lib/writer.pyx":2173
 *                     timestamps, dtype=np.int64)
 *             else:
 *                 timestamps_np = np.fromiter(timestamps, dtype=np.int64)             # <<<<<<<<<<<<<<
 *             if timestamps_np.ndim != 1:
 *                 raise ValueError("timestamps must be one-dimensional")
*/
    /*else*/ {
      __pyx_t_7 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2173, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_fromiter); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2173, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2173, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_int64); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2173, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_5 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_9))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_9);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_9, __pyx__function);
        __pyx_t_5 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_7, __pyx_v_timestamps};
        __pyx_t_1 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2173, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dtype, __pyx_t_4, __pyx_t_1, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 2173, __pyx_L1_error)
        __pyx_t_6 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_9, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_1);
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2173, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_DECREF_SET(__pyx_v_timestamps_np, __pyx_t_6);
      __pyx_t_6 = 0;
    }
    __pyx_L12:;

    /* "nvtx/_lib/writer.pyx":2174
 *             else:
 *                 timestamps_np = np.fromiter(timestamps, dtype=np.int64)
 *             if timestamps_np.ndim != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError("timestamps must be one-dimensional")
 *             if timestamps_np.size != samples.size:
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_timestamps_np, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2174, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_3 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_6, __pyx_mstate_global->__pyx_int_1, 1, 0)); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2174, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_t_3)) {

      /* "nvtx/_lib/writer.pyx":2175
 *                 timestamps_np = np.fromiter(timestamps, dtype=np.int64)
 *             if timestamps_np.ndim != 1:
 *                 raise ValueError("timestamps must be one-dimensional")             # <<<<<<<<<<<<<<
 *             if timestamps_np.size != samples.size:
 *                 raise ValueError(
*/
      __pyx_t_9 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_timestamps_must_be_one_dimension};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2175, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 2175, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":2174
 *             else:
 *                 timestamps_np = np.fromiter(timestamps, dtype=np.int64)
 *             if timestamps_np.ndim != 1:             # <<<<<<<<<<<<<<
 *                 raise ValueError("timestamps must be one-dimensional")
 *             if timestamps_np.size != samples.size:
*/
    }

    /* "nvtx/_lib/writer.pyx":2176
 *             if timestamps_np.ndim != 1:
 *                 raise ValueError("timestamps must be one-dimensional")
 *             if timestamps_np.size != samples.size:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "data and timestamps must have the same length")
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_timestamps_np, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 2176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_samples, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2176, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_6, __pyx_t_9, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2176, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 2176, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(__pyx_t_3)) {

      /* "nvtx/_lib/writer.pyx":2177
 *                 raise ValueError("timestamps must be one-dimensional")
 *             if timestamps_np.size != samples.size:
 *                 raise ValueError(             # <<<<<<<<<<<<<<
 *                     "data and timestamps must have the same length")
 *             batch.timestamps = \
*/
      __pyx_t_9 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_data_and_timestamps_must_have_th};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2177, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 2177, __pyx_L1_error)

      /* "nvtx/_lib/writer.pyx":2176
 *             if timestamps_np.ndim != 1:
 *                 raise ValueError("timestamps must be one-dimensional")
 *             if timestamps_np.size != samples.size:             # <<<<<<<<<<<<<<
 *                 raise ValueError(
 *                     "data and timestamps must have the same length")
*/
    }

    /* "nvtx/_lib/writer.pyx":2180
 *                     "data and timestamps must have the same length")
 *             batch.timestamps = \
 *                 <const int64_t*><size_t>timestamps_np.ctypes.data             # <<<<<<<<<<<<<<
 *             batch.timestampsSize = timestamps_np.nbytes
 *         _check(
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_timestamps_np, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2180, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_9); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2180, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "nvtx/_lib/writer.pyx":2179
 *                 raise ValueError(
 *                     "data and timestamps must have the same length")
 *             batch.timestamps = \             # <<<<<<<<<<<<<<
 *                 <const int64_t*><size_t>timestamps_np.ctypes.data
 *             batch.timestampsSize = timestamps_np.nbytes
*/
    __pyx_v_batch.timestamps = ((int64_t const *)((size_t)__pyx_t_5));

    /* "nvtx/_lib/writer.pyx":2181
 *             batch.timestamps = \
 *                 <const int64_t*><size_t>timestamps_np.ctypes.data
 *             batch.timestampsSize = timestamps_np.nbytes             # <<<<<<<<<<<<<<
 *         _check(
 *             self._backend._iface.CounterBatchWrite(
*/
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_timestamps_np, __pyx_mstate_global->__pyx_n_u_nbytes); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 2181, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_5 = __Pyx_PyLong_As_size_t(__pyx_t_9); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 2181, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_v_batch.timestampsSize = __pyx_t_5;

    /* "nvtx/_lib/writer.pyx":2168
 *         batch.timestamps = NULL
 *         batch.timestampsSize = 0
 *         if timestamps is not None:             # <<<<<<<<<<<<<<
 *             if isinstance(timestamps, np.ndarray):
 *                 timestamps_np = np.ascontiguousarray(
*/
  }

  /* "nvtx/_lib/writer.pyx":2182
 *                 <const int64_t*><size_t>timestamps_np.ctypes.data
 *             batch.timestampsSize = timestamps_np.nbytes
 *         _check(             # <<<<<<<<<<<<<<
 *             self._backend._iface.CounterBatchWrite(
 *                 self._writer.stream, &batch),
*/
  __pyx_f_4nvtx_4_lib_6writer__check(__pyx_v_self->_backend->_iface->CounterBatchWrite(__pyx_v_self->_writer.stream, (&__pyx_v_batch)), __pyx_mstate_global->__pyx_n_u_CounterBatchWrite); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 2182, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":2081
 *             "CounterNoValueWrite")
 * 
 *     def write_counter_batch(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, data, *,
 *         timestamps=None, ordering=BatchOrdering.SORTED,
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.write_counter_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dt);
  __Pyx_XDECREF(__pyx_v_samples);
  __Pyx_XDECREF(__pyx_v_timestamps_np);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_35__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_34__reduce_cython__, "Stream.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_35__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_35__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_34__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_35__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_34__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_34__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_37__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_6Stream_36__setstate_cython__, "Stream.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_6Stream_37__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_37__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_36__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_6Stream_37__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_6Stream_36__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_6Stream_36__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_6writer_Stream *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.writer.Stream.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle__WriterSchemaRegistrar(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7ba2b5b, 0x17fb052, 0x61860e1, b'_array_cache, _domain, _dtype_cache, _fixed_cache')
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_15__pyx_unpickle__WriterSchemaRegistrar(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_6writer_14__pyx_unpickle__WriterSchemaRegistrar, "__pyx_unpickle__WriterSchemaRegistrar(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_6writer_15__pyx_unpickle__WriterSchemaRegistrar = {"__pyx_unpickle__WriterSchemaRegistrar", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_15__pyx_unpickle__WriterSchemaRegistrar, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_14__pyx_unpickle__WriterSchemaRegistrar};
static PyObject *__pyx_pw_4nvtx_4_lib_6writer_15__pyx_unpickle__WriterSchemaRegistrar(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle__WriterSchemaRegistrar (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle__WriterSchemaRegistrar", 0) < (0)) __PYX_ERR(1, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__WriterSchemaRegistrar", 1, 3, 3, i); __PYX_ERR(1, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__WriterSchemaRegistrar", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.writer.__pyx_unpickle__WriterSchemaRegistrar", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(1, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_6writer_14__pyx_unpickle__WriterSchemaRegistrar(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_6writer_14__pyx_unpickle__WriterSchemaRegistrar(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__WriterSchemaRegistrar", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle__WriterSchemaRegistrar(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7ba2b5b, 0x17fb052, 0x61860e1, b'_array_cache, _domain, _dtype_cache, _fixed_cache')             # <<<<<<<<<<<<<<
 *     __pyx_result = _WriterSchemaRegistrar.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0x7ba2b5b, 0x17fb052, 0x61860e1, __pyx_k_array_cache__domain__dtype_cach); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7ba2b5b, 0x17fb052, 0x61860e1, b'_array_cache, _domain, _dtype_cache, _fixed_cache')
 *     __pyx_result = _WriterSchemaRegistrar.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle__WriterSchemaRegistrar__set_state(<_WriterSchemaRegistrar> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7ba2b5b, 0x17fb052, 0x61860e1, b'_array_cache, _domain, _dtype_cache, _fixed_cache')
 *     __pyx_result = _WriterSchemaRegistrar.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__WriterSchemaRegistrar__set_state(<_WriterSchemaRegistrar> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = _WriterSchemaRegistrar.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle__WriterSchemaRegistrar__set_state(<_WriterSchemaRegistrar> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle__WriterSchemaRegistrar__set_state(_WriterSchemaRegistrar __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(1, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_4nvtx_4_lib_6writer___pyx_unpickle__WriterSchemaRegistrar__set_state(((struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7ba2b5b, 0x17fb052, 0x61860e1, b'_array_cache, _domain, _dtype_cache, _fixed_cache')
 *     __pyx_result = _WriterSchemaRegistrar.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__WriterSchemaRegistrar__set_state(<_WriterSchemaRegistrar> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle__WriterSchemaRegistrar__set_state(<_WriterSchemaRegistrar> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle__WriterSchemaRegistrar__set_state(_WriterSchemaRegistrar __pyx_result, __pyx_state: tuple):
 *     __pyx_result._array_cache = __pyx_state[0]; __pyx_result._domain = __pyx_state[1]; __pyx_result._dtype_cache = __pyx_state[2]; __pyx_result._fixed_cache = __pyx_state[3]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle__WriterSchemaRegistrar(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7ba2b5b, 0x17fb052, 0x61860e1, b'_array_cache, _domain, _dtype_cache, _fixed_cache')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.writer.__pyx_unpickle__WriterSchemaRegistrar", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle__WriterSchemaRegistrar__set_state(<_WriterSchemaRegistrar> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__WriterSchemaRegistrar__set_state(_WriterSchemaRegistrar __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._array_cache = __pyx_state[0]; __pyx_result._domain = __pyx_state[1]; __pyx_result._dtype_cache = __pyx_state[2]; __pyx_result._fixed_cache = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

static PyObject *__pyx_f_4nvtx_4_lib_6writer___pyx_unpickle__WriterSchemaRegistrar__set_state(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__WriterSchemaRegistrar__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle__WriterSchemaRegistrar__set_state(_WriterSchemaRegistrar __pyx_result, __pyx_state: tuple):
 *     __pyx_result._array_cache = __pyx_state[0]; __pyx_result._domain = __pyx_state[1]; __pyx_result._dtype_cache = __pyx_state[2]; __pyx_result._fixed_cache = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._array_cache);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._array_cache);
  __pyx_v___pyx_result->__pyx_base._array_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->_domain);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->_domain);
  __pyx_v___pyx_result->_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._dtype_cache);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._dtype_cache);
  __pyx_v___pyx_result->__pyx_base._dtype_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._fixed_cache);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._fixed_cache);
  __pyx_v___pyx_result->__pyx_base._fixed_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle__WriterSchemaRegistrar__set_state(_WriterSchemaRegistrar __pyx_result, __pyx_state: tuple):
 *     __pyx_result._array_cache = __pyx_state[0]; __pyx_result._domain = __pyx_state[1]; __pyx_result._dtype_cache = __pyx_state[2]; __pyx_result._fixed_cache = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 4); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle__WriterSchemaRegistrar__set_state(<_WriterSchemaRegistrar> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__WriterSchemaRegistrar__set_state(_WriterSchemaRegistrar __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._array_cache = __pyx_state[0]; __pyx_result._domain = __pyx_state[1]; __pyx_result._dtype_cache = __pyx_state[2]; __pyx_result._fixed_cache = __pyx_state[3]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 4)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.writer.__pyx_unpickle__WriterSchemaRegistrar__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer__BackendModule __pyx_vtable_4nvtx_4_lib_6writer__BackendModule;

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer__BackendModule(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)o);
  p->__pyx_vtab = __pyx_vtabptr_4nvtx_4_lib_6writer__BackendModule;
  p->_lib = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer__BackendModule(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *p = (struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer__BackendModule) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_lib);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer__BackendModule(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *p = (struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_lib) {
    e = (*v)(p->_lib, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer__BackendModule(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *p = (struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *)o;
  tmp = ((PyObject*)p->_lib);
  p->_lib = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer__BackendModule[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_14_BackendModule_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_14_BackendModule_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_14_BackendModule_4__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer__BackendModule_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer__BackendModule},
  {Py_tp_doc, (void *)PyDoc_STR("One canonical backend module shared by all public wrappers.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer__BackendModule},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer__BackendModule},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer__BackendModule},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer__BackendModule},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer__BackendModule_spec = {
  "nvtx._lib.writer._BackendModule",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer__BackendModule_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer__BackendModule = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""_BackendModule", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer__BackendModule, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("One canonical backend module shared by all public wrappers."), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer__BackendModule, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer__BackendModule, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer__BackendModule, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer__BackendModule, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Backend __pyx_vtable_4nvtx_4_lib_6writer_Backend;

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Backend(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Backend *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)o);
  p->__pyx_vtab = __pyx_vtabptr_4nvtx_4_lib_6writer_Backend;
  p->_module = Py_None; Py_INCREF(Py_None);
  p->_lock = Py_None; Py_INCREF(Py_None);
  p->_active_sessions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4nvtx_4_lib_6writer_7Backend_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer_Backend(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Backend *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer_Backend) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_module);
  Py_CLEAR(p->_lock);
  Py_CLEAR(p->_active_sessions);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer_Backend(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer_Backend *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_module) {
    e = (*v)(p->_module, a); if (e) return e;
  }
  if (p->_lock) {
    e = (*v)(p->_lock, a); if (e) return e;
  }
  if (p->_active_sessions) {
    e = (*v)(p->_active_sessions, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer_Backend(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer_Backend *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)o;
  tmp = ((PyObject*)p->_module);
  p->_module = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_lock);
  p->_lock = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_active_sessions);
  p->_active_sessions = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer_Backend[] = {
  {"close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Backend_3close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Backend_2close},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Backend_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Backend_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Backend_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Backend_6__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer_Backend_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer_Backend},
  {Py_tp_doc, (void *)PyDoc_STR("\n    A loaded NVTXW backend: the module handle plus its interface table.\n\n    Do not construct directly; use :func:`load_backend`.\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer_Backend},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer_Backend},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer_Backend},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer_Backend},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer_Backend_spec = {
  "nvtx._lib.writer.Backend",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Backend),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer_Backend_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer_Backend = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""Backend", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Backend), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer_Backend, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    A loaded NVTXW backend: the module handle plus its interface table.\n\n    Do not construct directly; use :func:`load_backend`.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer_Backend, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer_Backend, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer_Backend, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer_Backend, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Session(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Session *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)o);
  p->_backend = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)Py_None); Py_INCREF(Py_None);
  p->_name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_config = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_lock = Py_None; Py_INCREF(Py_None);
  p->_open_streams = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_domains = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer_Session(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Session *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Session *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer_Session) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_backend);
  Py_CLEAR(p->_name);
  Py_CLEAR(p->_config);
  Py_CLEAR(p->_lock);
  Py_CLEAR(p->_open_streams);
  Py_CLEAR(p->_domains);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer_Session(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer_Session *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Session *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_backend) {
    e = (*v)(((PyObject *)p->_backend), a); if (e) return e;
  }
  if (p->_lock) {
    e = (*v)(p->_lock, a); if (e) return e;
  }
  if (p->_open_streams) {
    e = (*v)(p->_open_streams, a); if (e) return e;
  }
  if (p->_domains) {
    e = (*v)(p->_domains, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer_Session(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer_Session *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Session *)o;
  tmp = ((PyObject*)p->_backend);
  p->_backend = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_lock);
  p->_lock = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_open_streams);
  p->_open_streams = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_domains);
  p->_domains = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer_Session[] = {
  {"begin", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_3begin, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_2begin},
  {"end", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_5end, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_4end},
  {"_register_domain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_7_register_domain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_6_register_domain},
  {"get_domain", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_9get_domain, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_8get_domain},
  {"create_stream", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_11create_stream, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_10create_stream},
  {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_13__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_12__enter__},
  {"__exit__", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_15__exit__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_14__exit__},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_17__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_16__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Session_19__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Session_18__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer_Session_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer_Session},
  {Py_tp_doc, (void *)PyDoc_STR("Session(name, backend, *, config=None)\n\nAn NVTXW session: the top-level container data is written into.\n\nUse as a context manager (recommended) or call :meth:`begin` and\n:meth:`end` explicitly.\n\nParameters\n----------\nname : str or bytes\n    Session name. Tools may display it or use it to name a file or\n    directory representing the session.\nbackend : Backend\n    Backend the session writes to, from :func:`load_backend`.\nconfig : str or bytes, optional\n    Backend-specific configuration options, one ``key=value`` pair per\n    line. Backends use reasonable defaults for options not provided and\n    ignore keys they do not support.\n\nNotes\n-----\nWrites to distinct streams may proceed concurrently. Calls on the same\nstream must not overlap unless the caller synchronizes them. Registration\nand lifecycle operations must not overlap writes: complete setup before\nstarting writer threads, and stop them before closing streams or ending\nthe session.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer_Session},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer_Session},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer_Session},
  {Py_tp_init, (void *)__pyx_pw_4nvtx_4_lib_6writer_7Session_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer_Session},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer_Session_spec = {
  "nvtx._lib.writer.Session",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Session),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer_Session_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer_Session = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""Session", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Session), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer_Session, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("Session(name, backend, *, config=None)\n\nAn NVTXW session: the top-level container data is written into.\n\nUse as a context manager (recommended) or call :meth:`begin` and\n:meth:`end` explicitly.\n\nParameters\n----------\nname : str or bytes\n    Session name. Tools may display it or use it to name a file or\n    directory representing the session.\nbackend : Backend\n    Backend the session writes to, from :func:`load_backend`.\nconfig : str or bytes, optional\n    Backend-specific configuration options, one ``key=value`` pair per\n    line. Backends use reasonable defaults for options not provided and\n    ignore keys they do not support.\n\nNotes\n-----\nWrites to distinct streams may proceed concurrently. Calls on the same\nstream must not overlap unless the caller synchronizes them. Registration\nand lifecycle operations must not overlap writes: complete setup before\nstarting writer threads, and stop them before closing streams or ending\nthe session."), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer_Session, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer_Session, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer_Session, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4nvtx_4_lib_6writer_7Session_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer_Session, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_RegisteredString(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)o);
  p->_string = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_domain = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer_RegisteredString(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *p = (struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer_RegisteredString) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_string);
  Py_CLEAR(p->_domain);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer_RegisteredString(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *p = (struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_domain) {
    e = (*v)(p->_domain, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer_RegisteredString(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *p = (struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString *)o;
  tmp = ((PyObject*)p->_domain);
  p->_domain = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_16RegisteredString_handle(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_6handle_1__get__(o);
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer_RegisteredString[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_16RegisteredString_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_16RegisteredString_6__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4nvtx_4_lib_6writer_RegisteredString[] = {
  {"handle", __pyx_getprop_4nvtx_4_lib_6writer_16RegisteredString_handle, 0, PyDoc_STR("Opaque numeric handle for a registered-string payload field."), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer_RegisteredString_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer_RegisteredString},
  {Py_tp_repr, (void *)__pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_3__repr__},
  {Py_tp_doc, (void *)PyDoc_STR("\n    Wrapper for ``nvtxStringHandle_t``, created by\n    :meth:`Domain.get_registered_string`.\n\n    Pass to the single-event range/mark write methods in place of a ``str``\n    message to emit a registered-string handle instead of an inline UTF-8\n    message. The numeric :attr:`handle` can also be stored in a payload field\n    annotated with :attr:`nvtx.PayloadEntryType.REGISTERED_STRING`. Valid until\n    the owning session ends.\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer_RegisteredString},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer_RegisteredString},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer_RegisteredString},
  {Py_tp_getset, (void *)__pyx_getsets_4nvtx_4_lib_6writer_RegisteredString},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer_RegisteredString},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer_RegisteredString_spec = {
  "nvtx._lib.writer.RegisteredString",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer_RegisteredString_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer_RegisteredString = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""RegisteredString", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_RegisteredString), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer_RegisteredString, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_4nvtx_4_lib_6writer_16RegisteredString_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    Wrapper for ``nvtxStringHandle_t``, created by\n    :meth:`Domain.get_registered_string`.\n\n    Pass to the single-event range/mark write methods in place of a ``str``\n    message to emit a registered-string handle instead of an inline UTF-8\n    message. The numeric :attr:`handle` can also be stored in a payload field\n    annotated with :attr:`nvtx.PayloadEntryType.REGISTERED_STRING`. Valid until\n    the owning session ends.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer_RegisteredString, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer_RegisteredString, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer_RegisteredString, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4nvtx_4_lib_6writer_RegisteredString, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer_RegisteredString, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Scope(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Scope *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)o);
  p->_path = Py_None; Py_INCREF(Py_None);
  p->_domain = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4nvtx_4_lib_6writer_5Scope_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer_Scope(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Scope *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer_Scope) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_path);
  Py_CLEAR(p->_domain);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer_Scope(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer_Scope *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_path) {
    e = (*v)(p->_path, a); if (e) return e;
  }
  if (p->_domain) {
    e = (*v)(p->_domain, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer_Scope(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer_Scope *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Scope *)o;
  tmp = ((PyObject*)p->_path);
  p->_path = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_domain);
  p->_domain = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_5Scope_scope_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_5Scope_8scope_id_1__get__(o);
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_5Scope_path(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_5Scope_4path_1__get__(o);
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer_Scope[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_5Scope_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_5Scope_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_5Scope_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_5Scope_6__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4nvtx_4_lib_6writer_Scope[] = {
  {"scope_id", __pyx_getprop_4nvtx_4_lib_6writer_5Scope_scope_id, 0, PyDoc_STR("Registered scope ID within the domain."), 0},
  {"path", __pyx_getprop_4nvtx_4_lib_6writer_5Scope_path, 0, PyDoc_STR("The path the scope was registered with (``None`` if unnamed)."), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer_Scope_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer_Scope},
  {Py_tp_repr, (void *)__pyx_pw_4nvtx_4_lib_6writer_5Scope_3__repr__},
  {Py_tp_doc, (void *)PyDoc_STR("\n    Wrapper for a registered scope ID.\n    Created by :meth:`Domain.get_scope`.\n    Valid until the owning session ends.\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer_Scope},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer_Scope},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer_Scope},
  {Py_tp_getset, (void *)__pyx_getsets_4nvtx_4_lib_6writer_Scope},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer_Scope},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer_Scope_spec = {
  "nvtx._lib.writer.Scope",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Scope),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer_Scope_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer_Scope = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""Scope", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Scope), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer_Scope, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_4nvtx_4_lib_6writer_5Scope_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    Wrapper for a registered scope ID.\n    Created by :meth:`Domain.get_scope`.\n    Valid until the owning session ends.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer_Scope, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer_Scope, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer_Scope, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4nvtx_4_lib_6writer_Scope, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer_Scope, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Domain __pyx_vtable_4nvtx_4_lib_6writer_Domain;

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Domain(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)o);
  p->__pyx_vtab = __pyx_vtabptr_4nvtx_4_lib_6writer_Domain;
  p->_backend = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)Py_None); Py_INCREF(Py_None);
  p->_session = ((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)Py_None); Py_INCREF(Py_None);
  p->_get_string_cached = Py_None; Py_INCREF(Py_None);
  p->_get_scope_cached = Py_None; Py_INCREF(Py_None);
  p->_categories = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_schemas = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_get_counter_cached = Py_None; Py_INCREF(Py_None);
  p->_category_ids = Py_None; Py_INCREF(Py_None);
  p->_user_category_ids = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_schema_registrar = ((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4nvtx_4_lib_6writer_6Domain_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer_Domain(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer_Domain) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_backend);
  Py_CLEAR(p->_session);
  Py_CLEAR(p->_get_string_cached);
  Py_CLEAR(p->_get_scope_cached);
  Py_CLEAR(p->_categories);
  Py_CLEAR(p->_schemas);
  Py_CLEAR(p->_get_counter_cached);
  Py_CLEAR(p->_category_ids);
  Py_CLEAR(p->_user_category_ids);
  Py_CLEAR(p->_schema_registrar);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer_Domain(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_backend) {
    e = (*v)(((PyObject *)p->_backend), a); if (e) return e;
  }
  if (p->_session) {
    e = (*v)(((PyObject *)p->_session), a); if (e) return e;
  }
  if (p->_get_string_cached) {
    e = (*v)(p->_get_string_cached, a); if (e) return e;
  }
  if (p->_get_scope_cached) {
    e = (*v)(p->_get_scope_cached, a); if (e) return e;
  }
  if (p->_categories) {
    e = (*v)(p->_categories, a); if (e) return e;
  }
  if (p->_schemas) {
    e = (*v)(p->_schemas, a); if (e) return e;
  }
  if (p->_get_counter_cached) {
    e = (*v)(p->_get_counter_cached, a); if (e) return e;
  }
  if (p->_category_ids) {
    e = (*v)(p->_category_ids, a); if (e) return e;
  }
  if (p->_user_category_ids) {
    e = (*v)(p->_user_category_ids, a); if (e) return e;
  }
  if (p->_schema_registrar) {
    e = (*v)(((PyObject *)p->_schema_registrar), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer_Domain(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer_Domain *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)o;
  tmp = ((PyObject*)p->_backend);
  p->_backend = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_session);
  p->_session = ((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_get_string_cached);
  p->_get_string_cached = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_get_scope_cached);
  p->_get_scope_cached = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_categories);
  p->_categories = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_schemas);
  p->_schemas = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_get_counter_cached);
  p->_get_counter_cached = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_category_ids);
  p->_category_ids = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_user_category_ids);
  p->_user_category_ids = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_schema_registrar);
  p->_schema_registrar = ((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer_Domain[] = {
  {"get_registered_string", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_3get_registered_string, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_2get_registered_string},
  {"_register_string", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_5_register_string, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_4_register_string},
  {"get_category_id", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_7get_category_id, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_6get_category_id},
  {"_register_category", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_9_register_category, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_8_register_category},
  {"get_scope", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_11get_scope, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_10get_scope},
  {"_register_scope", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_13_register_scope, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_12_register_scope},
  {"get_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_15get_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_14get_schema},
  {"get_counter", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_17get_counter, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_16get_counter},
  {"_register_counter", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_19_register_counter, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_18_register_counter},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_21__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_20__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Domain_23__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Domain_22__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer_Domain_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer_Domain},
  {Py_tp_doc, (void *)PyDoc_STR("\n    Session-owned NVTX domain. Created by :meth:`Session.get_domain`.\n    Valid until the owning session ends.\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer_Domain},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer_Domain},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer_Domain},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer_Domain},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer_Domain_spec = {
  "nvtx._lib.writer.Domain",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Domain),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer_Domain_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer_Domain = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""Domain", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Domain), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer_Domain, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    Session-owned NVTX domain. Created by :meth:`Session.get_domain`.\n    Valid until the owning session ends.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer_Domain, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer_Domain, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer_Domain, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer_Domain, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Counter(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Counter *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)o);
  p->_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)Py_None); Py_INCREF(Py_None);
  p->_name = Py_None; Py_INCREF(Py_None);
  p->_dtype = Py_None; Py_INCREF(Py_None);
  p->_description = Py_None; Py_INCREF(Py_None);
  p->_semantics = Py_None; Py_INCREF(Py_None);
  p->_timestamp_field = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4nvtx_4_lib_6writer_7Counter_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer_Counter(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Counter *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer_Counter) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_domain);
  Py_CLEAR(p->_name);
  Py_CLEAR(p->_dtype);
  Py_CLEAR(p->_description);
  Py_CLEAR(p->_semantics);
  Py_CLEAR(p->_timestamp_field);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer_Counter(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer_Counter *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_domain) {
    e = (*v)(((PyObject *)p->_domain), a); if (e) return e;
  }
  if (p->_name) {
    e = (*v)(p->_name, a); if (e) return e;
  }
  if (p->_dtype) {
    e = (*v)(p->_dtype, a); if (e) return e;
  }
  if (p->_description) {
    e = (*v)(p->_description, a); if (e) return e;
  }
  if (p->_semantics) {
    e = (*v)(p->_semantics, a); if (e) return e;
  }
  if (p->_timestamp_field) {
    e = (*v)(p->_timestamp_field, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer_Counter(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer_Counter *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Counter *)o;
  tmp = ((PyObject*)p->_domain);
  p->_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_name);
  p->_name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_dtype);
  p->_dtype = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_description);
  p->_description = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_semantics);
  p->_semantics = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_timestamp_field);
  p->_timestamp_field = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_7Counter_counter_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_7Counter_10counter_id_1__get__(o);
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_7Counter_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_7Counter_4name_1__get__(o);
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_7Counter_dtype(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_7Counter_5dtype_1__get__(o);
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_7Counter_description(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_7Counter_11description_1__get__(o);
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_7Counter_semantics(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_7Counter_9semantics_1__get__(o);
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer_Counter[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Counter_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Counter_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_7Counter_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_7Counter_6__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4nvtx_4_lib_6writer_Counter[] = {
  {"counter_id", __pyx_getprop_4nvtx_4_lib_6writer_7Counter_counter_id, 0, PyDoc_STR("Registered counter ID within the domain."), 0},
  {"name", __pyx_getprop_4nvtx_4_lib_6writer_7Counter_name, 0, PyDoc_STR("The counter's display name."), 0},
  {"dtype", __pyx_getprop_4nvtx_4_lib_6writer_7Counter_dtype, 0, PyDoc_STR("The normalized dtype the counter was registered with."), 0},
  {"description", __pyx_getprop_4nvtx_4_lib_6writer_7Counter_description, 0, PyDoc_STR("The counter's description, or ``None``."), 0},
  {"semantics", __pyx_getprop_4nvtx_4_lib_6writer_7Counter_semantics, 0, PyDoc_STR("The counter's :class:`nvtx.CounterSemantics`, or ``None``."), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer_Counter_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer_Counter},
  {Py_tp_repr, (void *)__pyx_pw_4nvtx_4_lib_6writer_7Counter_3__repr__},
  {Py_tp_doc, (void *)PyDoc_STR("\n    Domain-owned counter handle. Created by :meth:`Domain.get_counter`.\n    Pass to the :class:`Stream` counter write methods.\n    Valid until the owning session ends.\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer_Counter},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer_Counter},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer_Counter},
  {Py_tp_getset, (void *)__pyx_getsets_4nvtx_4_lib_6writer_Counter},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer_Counter},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer_Counter_spec = {
  "nvtx._lib.writer.Counter",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Counter),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer_Counter_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer_Counter = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""Counter", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Counter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer_Counter, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_4nvtx_4_lib_6writer_7Counter_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    Domain-owned counter handle. Created by :meth:`Domain.get_counter`.\n    Pass to the :class:`Stream` counter write methods.\n    Valid until the owning session ends.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer_Counter, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer_Counter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer_Counter, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4nvtx_4_lib_6writer_Counter, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer_Counter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Schema(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Schema *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)o);
  p->_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)Py_None); Py_INCREF(Py_None);
  p->_dtype = Py_None; Py_INCREF(Py_None);
  p->_kind = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4nvtx_4_lib_6writer_6Schema_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer_Schema(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Schema *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer_Schema) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_domain);
  Py_CLEAR(p->_dtype);
  Py_CLEAR(p->_kind);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer_Schema(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer_Schema *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_domain) {
    e = (*v)(((PyObject *)p->_domain), a); if (e) return e;
  }
  if (p->_dtype) {
    e = (*v)(p->_dtype, a); if (e) return e;
  }
  if (p->_kind) {
    e = (*v)(p->_kind, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer_Schema(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer_Schema *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Schema *)o;
  tmp = ((PyObject*)p->_domain);
  p->_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_dtype);
  p->_dtype = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_kind);
  p->_kind = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_6Schema_schema_id(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_6Schema_9schema_id_1__get__(o);
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_6Schema_dtype(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_6Schema_5dtype_1__get__(o);
}

static PyObject *__pyx_getprop_4nvtx_4_lib_6writer_6Schema_kind(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_6writer_6Schema_4kind_1__get__(o);
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer_Schema[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Schema_5__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Schema_4__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Schema_7__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Schema_6__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4nvtx_4_lib_6writer_Schema[] = {
  {"schema_id", __pyx_getprop_4nvtx_4_lib_6writer_6Schema_schema_id, 0, PyDoc_STR("Registered schema ID within the domain."), 0},
  {"dtype", __pyx_getprop_4nvtx_4_lib_6writer_6Schema_dtype, 0, PyDoc_STR("The NumPy dtype the schema was registered with."), 0},
  {"kind", __pyx_getprop_4nvtx_4_lib_6writer_6Schema_kind, 0, PyDoc_STR("The schema's :class:`EventKind`, or ``None`` if generic."), 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer_Schema_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer_Schema},
  {Py_tp_repr, (void *)__pyx_pw_4nvtx_4_lib_6writer_6Schema_3__repr__},
  {Py_tp_doc, (void *)PyDoc_STR("\n    Domain-owned payload schema handle. Created by :meth:`Domain.get_schema`.\n    Pass event schemas (created with a ``kind``) to\n    :meth:`Stream.write_event` and :meth:`Stream.write_event_batch`.\n    Valid until the owning session ends.\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer_Schema},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer_Schema},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer_Schema},
  {Py_tp_getset, (void *)__pyx_getsets_4nvtx_4_lib_6writer_Schema},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer_Schema},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer_Schema_spec = {
  "nvtx._lib.writer.Schema",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Schema),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer_Schema_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer_Schema = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""Schema", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Schema), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer_Schema, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_4nvtx_4_lib_6writer_6Schema_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    Domain-owned payload schema handle. Created by :meth:`Domain.get_schema`.\n    Pass event schemas (created with a ``kind``) to\n    :meth:`Stream.write_event` and :meth:`Stream.write_event_batch`.\n    Valid until the owning session ends.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer_Schema, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer_Schema, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer_Schema, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4nvtx_4_lib_6writer_Schema, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer_Schema, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer_Stream __pyx_vtable_4nvtx_4_lib_6writer_Stream;

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer_Stream(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Stream *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)o);
  p->__pyx_vtab = __pyx_vtabptr_4nvtx_4_lib_6writer_Stream;
  p->_backend = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)Py_None); Py_INCREF(Py_None);
  p->_session = ((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)Py_None); Py_INCREF(Py_None);
  p->_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)Py_None); Py_INCREF(Py_None);
  p->_name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4nvtx_4_lib_6writer_6Stream_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer_Stream(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer_Stream *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer_Stream) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_backend);
  Py_CLEAR(p->_session);
  Py_CLEAR(p->_domain);
  Py_CLEAR(p->_name);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer_Stream(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer_Stream *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_backend) {
    e = (*v)(((PyObject *)p->_backend), a); if (e) return e;
  }
  if (p->_session) {
    e = (*v)(((PyObject *)p->_session), a); if (e) return e;
  }
  if (p->_domain) {
    e = (*v)(((PyObject *)p->_domain), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer_Stream(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer_Stream *p = (struct __pyx_obj_4nvtx_4_lib_6writer_Stream *)o;
  tmp = ((PyObject*)p->_backend);
  p->_backend = ((struct __pyx_obj_4nvtx_4_lib_6writer_Backend *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_session);
  p->_session = ((struct __pyx_obj_4nvtx_4_lib_6writer_Session *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_domain);
  p->_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer_Stream[] = {
  {"open", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_3open, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_2open},
  {"close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_5close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_4close},
  {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_7__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_6__enter__},
  {"__exit__", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_9__exit__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_8__exit__},
  {"write_mark", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_11write_mark, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_10write_mark},
  {"write_pushpop", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_13write_pushpop, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_12write_pushpop},
  {"write_startend", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_15write_startend, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_14write_startend},
  {"write_push", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_17write_push, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_16write_push},
  {"write_pop", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_19write_pop, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_18write_pop},
  {"write_start", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_21write_start, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_20write_start},
  {"write_end", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_23write_end, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_22write_end},
  {"write_event", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_25write_event, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_24write_event},
  {"write_event_batch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_27write_event_batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_26write_event_batch},
  {"write_counter_sample", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_29write_counter_sample, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_28write_counter_sample},
  {"write_counter_sample_no_value", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_31write_counter_sample_no_value, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_30write_counter_sample_no_value},
  {"write_counter_batch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_33write_counter_batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_32write_counter_batch},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_35__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_34__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_6Stream_37__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_6Stream_36__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer_Stream_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer_Stream},
  {Py_tp_doc, (void *)PyDoc_STR("\n    NVTXW stream: the object events and counter samples are written to.\n\n    Created by :meth:`Session.create_stream`.  Use as a context manager\n    (recommended) or call :meth:`open` and :meth:`close` explicitly before and\n    after writing data.\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer_Stream},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer_Stream},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer_Stream},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer_Stream},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer_Stream_spec = {
  "nvtx._lib.writer.Stream",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Stream),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer_Stream_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer_Stream = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""Stream", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer_Stream), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer_Stream, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    NVTXW stream: the object events and counter samples are written to.\n\n    Created by :meth:`Session.create_stream`.  Use as a context manager\n    (recommended) or call :meth:`open` and :meth:`close` explicitly before and\n    after writing data.\n    "), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer_Stream, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer_Stream, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer_Stream, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer_Stream, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4nvtx_4_lib_6writer__WriterSchemaRegistrar __pyx_vtable_4nvtx_4_lib_6writer__WriterSchemaRegistrar;

static PyObject *__pyx_tp_new_4nvtx_4_lib_6writer__WriterSchemaRegistrar(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *p;
  PyObject *o = __Pyx_PyType_GetSlot(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, tp_new, newfunc)(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar*)__pyx_vtabptr_4nvtx_4_lib_6writer__WriterSchemaRegistrar;
  p->_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_6writer__WriterSchemaRegistrar(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *p = (struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_6writer__WriterSchemaRegistrar) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_domain);
  if (PyType_IS_GC(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar)) PyObject_GC_Track(o);
  #if !CYTHON_USE_MODULE_STATE
  if (likely(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar)) __Pyx_PyType_GetSlot(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, tp_dealloc, destructor)(o); else
  #endif
  __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_4nvtx_4_lib_6writer__WriterSchemaRegistrar);
}

static int __pyx_tp_traverse_4nvtx_4_lib_6writer__WriterSchemaRegistrar(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *p = (struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *)o;
  #if !CYTHON_USE_MODULE_STATE
  e = 0;
  if (likely(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar)) {
    traverseproc traverse = __Pyx_PyType_GetSlot(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, tp_traverse, traverseproc);
    if (traverse) { e = traverse(o, v, a); }
  } else
  #endif
  { e = __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_4nvtx_4_lib_6writer__WriterSchemaRegistrar); }
  if (e) return e;
  {
    e = __Pyx_call_type_traverse(o, 0, v, a);
    if (e) return e;
  }
  if (p->_domain) {
    e = (*v)(((PyObject *)p->_domain), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_6writer__WriterSchemaRegistrar(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *p = (struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar *)o;
  #if !CYTHON_USE_MODULE_STATE
  if (likely(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar)) {
    inquiry clear = __Pyx_PyType_GetSlot(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, tp_clear, inquiry);
    if (clear) clear(o);
  } else
  #endif
  { __Pyx_call_next_tp_clear(o, __pyx_tp_clear_4nvtx_4_lib_6writer__WriterSchemaRegistrar); }
  tmp = ((PyObject*)p->_domain);
  p->_domain = ((struct __pyx_obj_4nvtx_4_lib_6writer_Domain *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_6writer__WriterSchemaRegistrar[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_1__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar___reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_3__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_2__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_6writer__WriterSchemaRegistrar},
  {Py_tp_doc, (void *)PyDoc_STR("Registers schemas via the backend interface's ``SchemaRegister``.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_6writer__WriterSchemaRegistrar},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_6writer__WriterSchemaRegistrar},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_6writer__WriterSchemaRegistrar},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_6writer__WriterSchemaRegistrar},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar_spec = {
  "nvtx._lib.writer._WriterSchemaRegistrar",
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.writer.""_WriterSchemaRegistrar", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_6writer__WriterSchemaRegistrar), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_6writer__WriterSchemaRegistrar, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("Registers schemas via the backend interface's ``SchemaRegister``."), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_6writer__WriterSchemaRegistrar, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_6writer__WriterSchemaRegistrar, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_6writer__WriterSchemaRegistrar, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_6writer__WriterSchemaRegistrar, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  {
    __pyx_t_1 = __Pyx_ApiExport_GetApiDict(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_export_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_PyObject__as_bytes__as);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_export_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_export_name = __pyx_export_signature + 24;
    void (*const __pyx_export_pointers[])(void) = {(void (*)(void))&__pyx_f_4nvtx_4_lib_6writer__as_bytes, (void (*)(void))&__pyx_f_4nvtx_4_lib_6writer__as_str, (void (*)(void)) NULL};
    void (*const *__pyx_export_pointer)(void) = __pyx_export_pointers;
    const char *__pyx_export_current_signature = __pyx_export_signature;
    while (*__pyx_export_pointer) {
      if (__Pyx_ExportFunction(__pyx_t_1, __pyx_export_name, *__pyx_export_pointer, __pyx_export_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_export_pointer;
      __pyx_export_name = strchr(__pyx_export_name, '\0') + 1;
      __pyx_export_signature = strchr(__pyx_export_signature, '\0') + 1;
      if (*__pyx_export_signature != '\0') __pyx_export_current_signature = __pyx_export_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_4nvtx_4_lib_6writer__BackendModule = &__pyx_vtable_4nvtx_4_lib_6writer__BackendModule;
  __pyx_vtable_4nvtx_4_lib_6writer__BackendModule._initialize = (void (*)(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *))__pyx_f_4nvtx_4_lib_6writer_14_BackendModule__initialize;
  __pyx_vtable_4nvtx_4_lib_6writer__BackendModule._finalize = (void (*)(struct __pyx_obj_4nvtx_4_lib_6writer__BackendModule *))__pyx_f_4nvtx_4_lib_6writer_14_BackendModule__finalize;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer__BackendModule_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule)) __PYX_ERR(0, 398, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer__BackendModule_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule) < (0)) __PYX_ERR(0, 398, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule = &__pyx_type_4nvtx_4_lib_6writer__BackendModule;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule) < (0)) __PYX_ERR(0, 398, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule, __pyx_vtabptr_4nvtx_4_lib_6writer__BackendModule) < (0)) __PYX_ERR(0, 398, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule) < (0)) __PYX_ERR(0, 398, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_BackendModule, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule) < (0)) __PYX_ERR(0, 398, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__BackendModule) < (0)) __PYX_ERR(0, 398, __pyx_L1_error)
  __pyx_vtabptr_4nvtx_4_lib_6writer_Backend = &__pyx_vtable_4nvtx_4_lib_6writer_Backend;
  __pyx_vtable_4nvtx_4_lib_6writer_Backend._ensure_open = (void (*)(struct __pyx_obj_4nvtx_4_lib_6writer_Backend *))__pyx_f_4nvtx_4_lib_6writer_7Backend__ensure_open;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer_Backend_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend)) __PYX_ERR(0, 448, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer_Backend_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend) < (0)) __PYX_ERR(0, 448, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend = &__pyx_type_4nvtx_4_lib_6writer_Backend;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend) < (0)) __PYX_ERR(0, 448, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend, __pyx_vtabptr_4nvtx_4_lib_6writer_Backend) < (0)) __PYX_ERR(0, 448, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend) < (0)) __PYX_ERR(0, 448, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Backend, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend) < (0)) __PYX_ERR(0, 448, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Backend) < (0)) __PYX_ERR(0, 448, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer_Session_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session)) __PYX_ERR(0, 557, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer_Session_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session) < (0)) __PYX_ERR(0, 557, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session = &__pyx_type_4nvtx_4_lib_6writer_Session;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session) < (0)) __PYX_ERR(0, 557, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Session, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session) < (0)) __PYX_ERR(0, 557, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Session) < (0)) __PYX_ERR(0, 557, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer_RegisteredString_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString)) __PYX_ERR(0, 840, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer_RegisteredString_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString) < (0)) __PYX_ERR(0, 840, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString = &__pyx_type_4nvtx_4_lib_6writer_RegisteredString;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString) < (0)) __PYX_ERR(0, 840, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_RegisteredString_2, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString) < (0)) __PYX_ERR(0, 840, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_RegisteredString) < (0)) __PYX_ERR(0, 840, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer_Scope_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope)) __PYX_ERR(0, 867, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer_Scope_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope) < (0)) __PYX_ERR(0, 867, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope = &__pyx_type_4nvtx_4_lib_6writer_Scope;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope) < (0)) __PYX_ERR(0, 867, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Scope, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope) < (0)) __PYX_ERR(0, 867, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Scope) < (0)) __PYX_ERR(0, 867, __pyx_L1_error)
  __pyx_vtabptr_4nvtx_4_lib_6writer_Domain = &__pyx_vtable_4nvtx_4_lib_6writer_Domain;
  __pyx_vtable_4nvtx_4_lib_6writer_Domain._invalidate = (PyObject *(*)(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *))__pyx_f_4nvtx_4_lib_6writer_6Domain__invalidate;
  __pyx_vtable_4nvtx_4_lib_6writer_Domain._ensure_valid = (PyObject *(*)(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *))__pyx_f_4nvtx_4_lib_6writer_6Domain__ensure_valid;
  __pyx_vtable_4nvtx_4_lib_6writer_Domain._get_event_schema_ids = (PyObject *(*)(struct __pyx_obj_4nvtx_4_lib_6writer_Domain *, nvtxwEventHelperSchemaIds_t *))__pyx_f_4nvtx_4_lib_6writer_6Domain__get_event_schema_ids;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer_Domain_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain)) __PYX_ERR(0, 987, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer_Domain_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain) < (0)) __PYX_ERR(0, 987, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain = &__pyx_type_4nvtx_4_lib_6writer_Domain;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain) < (0)) __PYX_ERR(0, 987, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_vtabptr_4nvtx_4_lib_6writer_Domain) < (0)) __PYX_ERR(0, 987, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain) < (0)) __PYX_ERR(0, 987, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Domain, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain) < (0)) __PYX_ERR(0, 987, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Domain) < (0)) __PYX_ERR(0, 987, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer_Counter_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter)) __PYX_ERR(0, 893, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer_Counter_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter) < (0)) __PYX_ERR(0, 893, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter = &__pyx_type_4nvtx_4_lib_6writer_Counter;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter) < (0)) __PYX_ERR(0, 893, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Counter, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter) < (0)) __PYX_ERR(0, 893, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Counter) < (0)) __PYX_ERR(0, 893, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer_Schema_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema)) __PYX_ERR(0, 951, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer_Schema_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema) < (0)) __PYX_ERR(0, 951, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema = &__pyx_type_4nvtx_4_lib_6writer_Schema;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema) < (0)) __PYX_ERR(0, 951, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Schema, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema) < (0)) __PYX_ERR(0, 951, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Schema) < (0)) __PYX_ERR(0, 951, __pyx_L1_error)
  __pyx_vtabptr_4nvtx_4_lib_6writer_Stream = &__pyx_vtable_4nvtx_4_lib_6writer_Stream;
  __pyx_vtable_4nvtx_4_lib_6writer_Stream._ensure_writable = (void (*)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *))__pyx_f_4nvtx_4_lib_6writer_6Stream__ensure_writable;
  __pyx_vtable_4nvtx_4_lib_6writer_Stream._ensure_counter = (void (*)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *, struct __pyx_obj_4nvtx_4_lib_6writer_Counter *))__pyx_f_4nvtx_4_lib_6writer_6Stream__ensure_counter;
  __pyx_vtable_4nvtx_4_lib_6writer_Stream._get_writer = (nvtxwEventWriter_t *(*)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *))__pyx_f_4nvtx_4_lib_6writer_6Stream__get_writer;
  __pyx_vtable_4nvtx_4_lib_6writer_Stream._resolve_event_attrs = (PyObject *(*)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *, PyObject *, PyObject *, PyObject *, nvtxwEventAttributes_t *, nvtxwEventAttributesUtf8_t *))__pyx_f_4nvtx_4_lib_6writer_6Stream__resolve_event_attrs;
  __pyx_vtable_4nvtx_4_lib_6writer_Stream._emit_at = (void (*)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *, int64_t, PyObject *, PyObject *, PyObject *, __pyx_t_4nvtx_4_lib_6writer__at_bin_fn, __pyx_t_4nvtx_4_lib_6writer__at_utf8_fn, PyObject *, PyObject *))__pyx_f_4nvtx_4_lib_6writer_6Stream__emit_at;
  __pyx_vtable_4nvtx_4_lib_6writer_Stream._emit_span = (void (*)(struct __pyx_obj_4nvtx_4_lib_6writer_Stream *, int64_t, int64_t, PyObject *, PyObject *, PyObject *, __pyx_t_4nvtx_4_lib_6writer__span_bin_fn, __pyx_t_4nvtx_4_lib_6writer__span_utf8_fn, PyObject *, PyObject *))__pyx_f_4nvtx_4_lib_6writer_6Stream__emit_span;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer_Stream_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream)) __PYX_ERR(0, 1402, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer_Stream_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream) < (0)) __PYX_ERR(0, 1402, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream = &__pyx_type_4nvtx_4_lib_6writer_Stream;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream) < (0)) __PYX_ERR(0, 1402, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_vtabptr_4nvtx_4_lib_6writer_Stream) < (0)) __PYX_ERR(0, 1402, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream) < (0)) __PYX_ERR(0, 1402, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Stream, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream) < (0)) __PYX_ERR(0, 1402, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer_Stream) < (0)) __PYX_ERR(0, 1402, __pyx_L1_error)
  __pyx_t_1 = PyImport_ImportModule("nvtx._lib.lib"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar = __Pyx_ImportType_3_2_9(__pyx_t_1, "nvtx._lib.lib", "SchemaRegistrar",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar),
  #else
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_9); if (!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_4nvtx_4_lib_3lib_SchemaRegistrar = (struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar); if (unlikely(!__pyx_vtabptr_4nvtx_4_lib_3lib_SchemaRegistrar)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_4nvtx_4_lib_6writer__WriterSchemaRegistrar = &__pyx_vtable_4nvtx_4_lib_6writer__WriterSchemaRegistrar;
  __pyx_vtable_4nvtx_4_lib_6writer__WriterSchemaRegistrar.__pyx_base = *__pyx_vtabptr_4nvtx_4_lib_3lib_SchemaRegistrar;
  __pyx_vtable_4nvtx_4_lib_6writer__WriterSchemaRegistrar.__pyx_base._do_register = (uint64_t (*)(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *, nvtxPayloadSchemaAttr_t const *))__pyx_f_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar__do_register;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_2 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar_spec, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar)) __PYX_ERR(0, 135, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar) < (0)) __PYX_ERR(0, 135, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar = &__pyx_type_4nvtx_4_lib_6writer__WriterSchemaRegistrar;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar->tp_base = __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar) < (0)) __PYX_ERR(0, 135, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar, __pyx_vtabptr_4nvtx_4_lib_6writer__WriterSchemaRegistrar) < (0)) __PYX_ERR(0, 135, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar) < (0)) __PYX_ERR(0, 135, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_WriterSchemaRegistrar, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar) < (0)) __PYX_ERR(0, 135, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar) < (0)) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("nvtx._lib.lib"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes = __Pyx_ImportType_3_2_9(__pyx_t_1, "nvtx._lib.lib", "EventAttributes",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes),
  #else
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_9); if (!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes) __PYX_ERR(2, 250, __pyx_L1_error)
  __pyx_vtabptr_4nvtx_4_lib_3lib_EventAttributes = (struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes); if (unlikely(!__pyx_vtabptr_4nvtx_4_lib_3lib_EventAttributes)) __PYX_ERR(2, 250, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle = __Pyx_ImportType_3_2_9(__pyx_t_1, "nvtx._lib.lib", "DomainHandle",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle),
  #else
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_9); if (!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle) __PYX_ERR(2, 278, __pyx_L1_error)
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle = __Pyx_ImportType_3_2_9(__pyx_t_1, "nvtx._lib.lib", "StringHandle",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle),
  #else
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_9); if (!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle) __PYX_ERR(2, 282, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("nvtx._lib.counters"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_8counters_Counter = __Pyx_ImportType_3_2_9(__pyx_t_1, "nvtx._lib.counters", "Counter",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4nvtx_4_lib_8counters_Counter), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_8counters_Counter),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4nvtx_4_lib_8counters_Counter), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_8counters_Counter),
  #else
  sizeof(struct __pyx_obj_4nvtx_4_lib_8counters_Counter), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_8counters_Counter),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_9); if (!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_8counters_Counter) __PYX_ERR(3, 112, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  {
    __pyx_t_1 = PyImport_ImportModule("nvtx._lib.counters"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_void_nvtxSemanticsCounter_t_PyOb);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 44;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_4nvtx_4_lib_8counters__fill_counter_semantics, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_9(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_writer(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_writer},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, __Pyx_FREETHREADING_COMPATIBLE},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "writer",
      0, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit_writer(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_writer(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
    PY_INT64_T current_id = GraalPyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_writer(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  size_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'writer' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "writer" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_NOT_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
    PyErr_Clear();
    __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
    if (!__Pyx_RefNanny)
        Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  
__Pyx_RefNannySetupContext("PyInit_writer", 0);
  __Pyx_init_runtime_version();
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Library function declarations ---*/
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__pyx_module_is_main_nvtx___lib__writer) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_main) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "nvtx._lib.writer")) {
      if (unlikely((PyDict_SetItemString(modules, "nvtx._lib.writer", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  (void)__Pyx_modinit_variable_export_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_function_export_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_init_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_function_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/

  /* "nvtx/_lib/writer.pyx":19
 * # See https://nvidia.github.io/NVTX/LICENSE.txt for license information.
 * 
 * import ctypes             # <<<<<<<<<<<<<<
 * import enum
 * import functools
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_ctypes, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ctypes, __pyx_t_2) < (0)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":20
 * 
 * import ctypes
 * import enum             # <<<<<<<<<<<<<<
 * import functools
 * import itertools
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_enum, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_enum, __pyx_t_2) < (0)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":21
 * import ctypes
 * import enum
 * import functools             # <<<<<<<<<<<<<<
 * import itertools
 * import os
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_functools, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_functools, __pyx_t_2) < (0)) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":22
 * import enum
 * import functools
 * import itertools             # <<<<<<<<<<<<<<
 * import os
 * import threading
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_itertools, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_itertools, __pyx_t_2) < (0)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":23
 * import functools
 * import itertools
 * import os             # <<<<<<<<<<<<<<
 * import threading
 * from pathlib import Path
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_os, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_os, __pyx_t_2) < (0)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":24
 * import itertools
 * import os
 * import threading             # <<<<<<<<<<<<<<
 * from pathlib import Path
 * 
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_threading, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_threading, __pyx_t_2) < (0)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":25
 * import os
 * import threading
 * from pathlib import Path             # <<<<<<<<<<<<<<
 * 
 * from nvtx import (
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Path};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pathlib, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Path};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 25, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 25, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":27
 * from pathlib import Path
 * 
 * from nvtx import (             # <<<<<<<<<<<<<<
 *     BatchOrdering,
 *     CounterNoValueReason,
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_BatchOrdering,__pyx_mstate_global->__pyx_n_u_CounterNoValueReason,__pyx_mstate_global->__pyx_n_u_EntryKind,__pyx_mstate_global->__pyx_n_u_EventKind,__pyx_mstate_global->__pyx_n_u_PayloadEntryType,__pyx_mstate_global->__pyx_n_u_PredefinedScope,__pyx_mstate_global->__pyx_n_u_TimestampType,__pyx_mstate_global->__pyx_n_u_numpy_dtype};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_nvtx, __pyx_imported_names, 8, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_BatchOrdering,__pyx_mstate_global->__pyx_n_u_CounterNoValueReason,__pyx_mstate_global->__pyx_n_u_EntryKind,__pyx_mstate_global->__pyx_n_u_EventKind,__pyx_mstate_global->__pyx_n_u_PayloadEntryType,__pyx_mstate_global->__pyx_n_u_PredefinedScope,__pyx_mstate_global->__pyx_n_u_TimestampType,__pyx_mstate_global->__pyx_n_u_numpy_dtype};
    for (__pyx_t_3=0; __pyx_t_3 < 8; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":37
 *     numpy_dtype,
 * )
 * from nvtx.colors import color_to_hex             # <<<<<<<<<<<<<<
 * from nvtx._lib.lib import _normalize_counter_dtype
 * from nvtx._metadata import PayloadSchemaKey, _nvtx_metadata_from_dtype
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_color_to_hex};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_nvtx_colors, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_color_to_hex};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 37, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 37, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":38
 * )
 * from nvtx.colors import color_to_hex
 * from nvtx._lib.lib import _normalize_counter_dtype             # <<<<<<<<<<<<<<
 * from nvtx._metadata import PayloadSchemaKey, _nvtx_metadata_from_dtype
 * 
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_normalize_counter_dtype};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_normalize_counter_dtype};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 38, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 38, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":39
 * from nvtx.colors import color_to_hex
 * from nvtx._lib.lib import _normalize_counter_dtype
 * from nvtx._metadata import PayloadSchemaKey, _nvtx_metadata_from_dtype             # <<<<<<<<<<<<<<
 * 
 * from nvtx._lib.counters cimport (
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_PayloadSchemaKey,__pyx_mstate_global->__pyx_n_u_nvtx_metadata_from_dtype};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_nvtx__metadata, __pyx_imported_names, 2, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_PayloadSchemaKey,__pyx_mstate_global->__pyx_n_u_nvtx_metadata_from_dtype};
    for (__pyx_t_3=0; __pyx_t_3 < 2; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 39, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 39, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":51
 * )
 * 
 * try:             # <<<<<<<<<<<<<<
 *     import numpy as np
 * except ImportError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {

      /* "nvtx/_lib/writer.pyx":52
 * 
 * try:
 *     import numpy as np             # <<<<<<<<<<<<<<
 * except ImportError:
 *     np = None
*/
      __pyx_t_7 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_numpy, 0, 0, NULL, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 52, __pyx_L2_error)
      __pyx_t_2 = __pyx_t_7;
      __Pyx_GOTREF(__pyx_t_2);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_np, __pyx_t_2) < (0)) __PYX_ERR(0, 52, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "nvtx/_lib/writer.pyx":51
 * )
 * 
 * try:             # <<<<<<<<<<<<<<
 *     import numpy as np
 * except ImportError:
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L7_try_end;
    __pyx_L2_error:;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/writer.pyx":53
 * try:
 *     import numpy as np
 * except ImportError:             # <<<<<<<<<<<<<<
 *     np = None
 * 
*/
    __pyx_t_8 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_ImportError))));
    if (__pyx_t_8) {
      __Pyx_AddTraceback("nvtx._lib.writer", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_9) < 0) __PYX_ERR(0, 53, __pyx_L4_except_error)
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_9);

      /* "nvtx/_lib/writer.pyx":54
 *     import numpy as np
 * except ImportError:
 *     np = None             # <<<<<<<<<<<<<<
 * 
 * try:
*/
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_np, Py_None) < (0)) __PYX_ERR(0, 54, __pyx_L4_except_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L3_exception_handled;
    }
    goto __pyx_L4_except_error;

    /* "nvtx/_lib/writer.pyx":51
 * )
 * 
 * try:             # <<<<<<<<<<<<<<
 *     import numpy as np
 * except ImportError:
*/
    __pyx_L4_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L3_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_5, __pyx_t_6);
    __pyx_L7_try_end:;
  }

  /* "nvtx/_lib/writer.pyx":56
 *     np = None
 * 
 * try:             # <<<<<<<<<<<<<<
 *     import pandas as pd
 * except ImportError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_5, &__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_1);
    /*try:*/ {

      /* "nvtx/_lib/writer.pyx":57
 * 
 * try:
 *     import pandas as pd             # <<<<<<<<<<<<<<
 * except ImportError:
 *     pd = None
*/
      __pyx_t_7 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_pandas, 0, 0, NULL, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 57, __pyx_L10_error)
      __pyx_t_9 = __pyx_t_7;
      __Pyx_GOTREF(__pyx_t_9);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pd, __pyx_t_9) < (0)) __PYX_ERR(0, 57, __pyx_L10_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "nvtx/_lib/writer.pyx":56
 *     np = None
 * 
 * try:             # <<<<<<<<<<<<<<
 *     import pandas as pd
 * except ImportError:
*/
    }
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L15_try_end;
    __pyx_L10_error:;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "nvtx/_lib/writer.pyx":58
 * try:
 *     import pandas as pd
 * except ImportError:             # <<<<<<<<<<<<<<
 *     pd = None
 * 
*/
    __pyx_t_8 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_ImportError))));
    if (__pyx_t_8) {
      __Pyx_AddTraceback("nvtx._lib.writer", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_9, &__pyx_t_4, &__pyx_t_2) < 0) __PYX_ERR(0, 58, __pyx_L12_except_error)
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_2);

      /* "nvtx/_lib/writer.pyx":59
 *     import pandas as pd
 * except ImportError:
 *     pd = None             # <<<<<<<<<<<<<<
 * 
 * 
*/
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pd, Py_None) < (0)) __PYX_ERR(0, 59, __pyx_L12_except_error)
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L11_exception_handled;
    }
    goto __pyx_L12_except_error;

    /* "nvtx/_lib/writer.pyx":56
 *     np = None
 * 
 * try:             # <<<<<<<<<<<<<<
 *     import pandas as pd
 * except ImportError:
*/
    __pyx_L12_except_error:;
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_5, __pyx_t_1);
    goto __pyx_L1_error;
    __pyx_L11_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_5, __pyx_t_1);
    __pyx_L15_try_end:;
  }

  /* "nvtx/_lib/writer.pyx":62
 * 
 * 
 * _GET_INTERFACE_SYMBOL_NAME = NVTXW_GET_INTERFACE_SYMBOL_NAME.decode()             # <<<<<<<<<<<<<<
 * _FINALIZE_SYMBOL_NAME = NVTXW_FINALIZE_SYMBOL_NAME.decode()
 * _BACKEND_MODULES = {}
*/
  __pyx_t_3 = __Pyx_ssize_strlen(NVTXW_GET_INTERFACE_SYMBOL_NAME); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 62, __pyx_L1_error)
  __pyx_t_2 = __Pyx_decode_c_string(NVTXW_GET_INTERFACE_SYMBOL_NAME, 0, __pyx_t_3, NULL, NULL, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_GET_INTERFACE_SYMBOL_NAME, __pyx_t_2) < (0)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":63
 * 
 * _GET_INTERFACE_SYMBOL_NAME = NVTXW_GET_INTERFACE_SYMBOL_NAME.decode()
 * _FINALIZE_SYMBOL_NAME = NVTXW_FINALIZE_SYMBOL_NAME.decode()             # <<<<<<<<<<<<<<
 * _BACKEND_MODULES = {}
 * _BACKEND_MODULES_LOCK = threading.RLock()
*/
  __pyx_t_3 = __Pyx_ssize_strlen(NVTXW_FINALIZE_SYMBOL_NAME); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 63, __pyx_L1_error)
  __pyx_t_2 = __Pyx_decode_c_string(NVTXW_FINALIZE_SYMBOL_NAME, 0, __pyx_t_3, NULL, NULL, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_FINALIZE_SYMBOL_NAME, __pyx_t_2) < (0)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":64
 * _GET_INTERFACE_SYMBOL_NAME = NVTXW_GET_INTERFACE_SYMBOL_NAME.decode()
 * _FINALIZE_SYMBOL_NAME = NVTXW_FINALIZE_SYMBOL_NAME.decode()
 * _BACKEND_MODULES = {}             # <<<<<<<<<<<<<<
 * _BACKEND_MODULES_LOCK = threading.RLock()
 * 
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_BACKEND_MODULES, __pyx_t_2) < (0)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":65
 * _FINALIZE_SYMBOL_NAME = NVTXW_FINALIZE_SYMBOL_NAME.decode()
 * _BACKEND_MODULES = {}
 * _BACKEND_MODULES_LOCK = threading.RLock()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_threading); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_RLock); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_11 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_11, (1-__pyx_t_11) | (__pyx_t_11*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_BACKEND_MODULES_LOCK, __pyx_t_2) < (0)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":68
 * 
 * 
 * class StreamInterleaving(enum.Enum):             # <<<<<<<<<<<<<<
 *     """Whether ordering guarantees apply across the whole stream or per scope."""
 *     NONE = NVTXW_STREAM_ORDER_INTERLEAVING_NONE
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Enum); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_Pack(1, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PEP560_update_bases(__pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_4 = __Pyx_CalculateMetaclass(NULL, __pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_Py3MetaclassPrepare(__pyx_t_4, __pyx_t_10, __pyx_mstate_global->__pyx_n_u_StreamInterleaving, __pyx_mstate_global->__pyx_n_u_StreamInterleaving, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_kp_u_Whether_ordering_guarantees_appl); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  if (__pyx_t_10 != __pyx_t_2) {
    if (unlikely((PyDict_SetItemString(__pyx_t_9, "__orig_bases__", __pyx_t_2) < 0))) __PYX_ERR(0, 68, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":70
 * class StreamInterleaving(enum.Enum):
 *     """Whether ordering guarantees apply across the whole stream or per scope."""
 *     NONE = NVTXW_STREAM_ORDER_INTERLEAVING_NONE             # <<<<<<<<<<<<<<
 *     SCOPE = NVTXW_STREAM_ORDER_INTERLEAVING_SCOPE
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_STREAM_ORDER_INTERLEAVING_NONE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_NONE, __pyx_t_2) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":71
 *     """Whether ordering guarantees apply across the whole stream or per scope."""
 *     NONE = NVTXW_STREAM_ORDER_INTERLEAVING_NONE
 *     SCOPE = NVTXW_STREAM_ORDER_INTERLEAVING_SCOPE             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_STREAM_ORDER_INTERLEAVING_SCOPE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_SCOPE, __pyx_t_2) < (0)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":68
 * 
 * 
 * class StreamInterleaving(enum.Enum):             # <<<<<<<<<<<<<<
 *     """Whether ordering guarantees apply across the whole stream or per scope."""
 *     NONE = NVTXW_STREAM_ORDER_INTERLEAVING_NONE
*/
  __pyx_t_2 = __Pyx_Py3ClassCreate(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_StreamInterleaving, __pyx_t_10, __pyx_t_9, NULL, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_StreamInterleaving, __pyx_t_2) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/writer.pyx":74
 * 
 * 
 * class StreamOrdering(enum.Enum):             # <<<<<<<<<<<<<<
 *     """How fully events are sorted in the stream."""
 *     UNKNOWN = NVTXW_STREAM_ORDERING_TYPE_UNKNOWN
*/
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_enum); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_Enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyTuple_Pack(1, __pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PEP560_update_bases(__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_CalculateMetaclass(NULL, __pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_9, __pyx_t_4, __pyx_mstate_global->__pyx_n_u_StreamOrdering, __pyx_mstate_global->__pyx_n_u_StreamOrdering, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_kp_u_How_fully_events_are_sorted_in_t); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__pyx_t_4 != __pyx_t_10) {
    if (unlikely((PyDict_SetItemString(__pyx_t_2, "__orig_bases__", __pyx_t_10) < 0))) __PYX_ERR(0, 74, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/writer.pyx":76
 * class StreamOrdering(enum.Enum):
 *     """How fully events are sorted in the stream."""
 *     UNKNOWN = NVTXW_STREAM_ORDERING_TYPE_UNKNOWN             # <<<<<<<<<<<<<<
 *     STRICT = NVTXW_STREAM_ORDERING_TYPE_STRICT
 *     PACKED_RANGE_START = NVTXW_STREAM_ORDERING_TYPE_PACKED_RANGE_START
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(NVTXW_STREAM_ORDERING_TYPE_UNKNOWN); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_UNKNOWN, __pyx_t_10) < (0)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/writer.pyx":77
 *     """How fully events are sorted in the stream."""
 *     UNKNOWN = NVTXW_STREAM_ORDERING_TYPE_UNKNOWN
 *     STRICT = NVTXW_STREAM_ORDERING_TYPE_STRICT             # <<<<<<<<<<<<<<
 *     PACKED_RANGE_START = NVTXW_STREAM_ORDERING_TYPE_PACKED_RANGE_START
 *     PACKED_RANGE_END = NVTXW_STREAM_ORDERING_TYPE_PACKED_RANGE_END
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(NVTXW_STREAM_ORDERING_TYPE_STRICT); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_STRICT, __pyx_t_10) < (0)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/writer.pyx":78
 *     UNKNOWN = NVTXW_STREAM_ORDERING_TYPE_UNKNOWN
 *     STRICT = NVTXW_STREAM_ORDERING_TYPE_STRICT
 *     PACKED_RANGE_START = NVTXW_STREAM_ORDERING_TYPE_PACKED_RANGE_START             # <<<<<<<<<<<<<<
 *     PACKED_RANGE_END = NVTXW_STREAM_ORDERING_TYPE_PACKED_RANGE_END
 * 
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(NVTXW_STREAM_ORDERING_TYPE_PACKED_RANGE_START); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_PACKED_RANGE_START, __pyx_t_10) < (0)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/writer.pyx":79
 *     STRICT = NVTXW_STREAM_ORDERING_TYPE_STRICT
 *     PACKED_RANGE_START = NVTXW_STREAM_ORDERING_TYPE_PACKED_RANGE_START
 *     PACKED_RANGE_END = NVTXW_STREAM_ORDERING_TYPE_PACKED_RANGE_END             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(NVTXW_STREAM_ORDERING_TYPE_PACKED_RANGE_END); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_PACKED_RANGE_END, __pyx_t_10) < (0)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/writer.pyx":74
 * 
 * 
 * class StreamOrdering(enum.Enum):             # <<<<<<<<<<<<<<
 *     """How fully events are sorted in the stream."""
 *     UNKNOWN = NVTXW_STREAM_ORDERING_TYPE_UNKNOWN
*/
  __pyx_t_10 = __Pyx_Py3ClassCreate(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_StreamOrdering, __pyx_t_4, __pyx_t_2, NULL, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_StreamOrdering, __pyx_t_10) < (0)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":82
 * 
 * 
 * class StreamSkid(enum.Enum):             # <<<<<<<<<<<<<<
 *     """How partial-sort "skid" is quantified (paired with ``skid_amount``)."""
 *     NONE = NVTXW_STREAM_ORDERING_SKID_NONE
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_Enum); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(1, __pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PEP560_update_bases(__pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_9, __pyx_mstate_global->__pyx_n_u_StreamSkid, __pyx_mstate_global->__pyx_n_u_StreamSkid, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_kp_u_How_partial_sort_skid_is_quantif); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__pyx_t_9 != __pyx_t_4) {
    if (unlikely((PyDict_SetItemString(__pyx_t_10, "__orig_bases__", __pyx_t_4) < 0))) __PYX_ERR(0, 82, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":84
 * class StreamSkid(enum.Enum):
 *     """How partial-sort "skid" is quantified (paired with ``skid_amount``)."""
 *     NONE = NVTXW_STREAM_ORDERING_SKID_NONE             # <<<<<<<<<<<<<<
 *     TIME_NS = NVTXW_STREAM_ORDERING_SKID_TIME_NS
 *     EVENT_COUNT = NVTXW_STREAM_ORDERING_SKID_EVENT_COUNT
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTXW_STREAM_ORDERING_SKID_NONE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_NONE, __pyx_t_4) < (0)) __PYX_ERR(0, 84, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":85
 *     """How partial-sort "skid" is quantified (paired with ``skid_amount``)."""
 *     NONE = NVTXW_STREAM_ORDERING_SKID_NONE
 *     TIME_NS = NVTXW_STREAM_ORDERING_SKID_TIME_NS             # <<<<<<<<<<<<<<
 *     EVENT_COUNT = NVTXW_STREAM_ORDERING_SKID_EVENT_COUNT
 * 
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTXW_STREAM_ORDERING_SKID_TIME_NS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_TIME_NS, __pyx_t_4) < (0)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":86
 *     NONE = NVTXW_STREAM_ORDERING_SKID_NONE
 *     TIME_NS = NVTXW_STREAM_ORDERING_SKID_TIME_NS
 *     EVENT_COUNT = NVTXW_STREAM_ORDERING_SKID_EVENT_COUNT             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTXW_STREAM_ORDERING_SKID_EVENT_COUNT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_EVENT_COUNT, __pyx_t_4) < (0)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":82
 * 
 * 
 * class StreamSkid(enum.Enum):             # <<<<<<<<<<<<<<
 *     """How partial-sort "skid" is quantified (paired with ``skid_amount``)."""
 *     NONE = NVTXW_STREAM_ORDERING_SKID_NONE
*/
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_StreamSkid, __pyx_t_9, __pyx_t_10, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_StreamSkid, __pyx_t_4) < (0)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "nvtx/_lib/writer.pyx":90
 * 
 * _RESULT_MESSAGES = {
 *     NVTXW_RESULT_SUCCESS: "success",             # <<<<<<<<<<<<<<
 *     NVTXW_RESULT_FAILED: "failed",
 *     NVTXW_RESULT_INVALID_ARGUMENT: "invalid argument",
*/
  __pyx_t_9 = __Pyx_PyDict_NewPresized(8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_RESULT_SUCCESS); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_9, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_success) < (0)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":91
 * _RESULT_MESSAGES = {
 *     NVTXW_RESULT_SUCCESS: "success",
 *     NVTXW_RESULT_FAILED: "failed",             # <<<<<<<<<<<<<<
 *     NVTXW_RESULT_INVALID_ARGUMENT: "invalid argument",
 *     NVTXW_RESULT_LIBRARY_NOT_FOUND: "library not found",
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_RESULT_FAILED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_9, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_failed_2) < (0)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":92
 *     NVTXW_RESULT_SUCCESS: "success",
 *     NVTXW_RESULT_FAILED: "failed",
 *     NVTXW_RESULT_INVALID_ARGUMENT: "invalid argument",             # <<<<<<<<<<<<<<
 *     NVTXW_RESULT_LIBRARY_NOT_FOUND: "library not found",
 *     NVTXW_RESULT_LIBRARY_LOAD_FAILED: "library load failed",
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_RESULT_INVALID_ARGUMENT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_9, __pyx_t_2, __pyx_mstate_global->__pyx_kp_u_invalid_argument) < (0)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":93
 *     NVTXW_RESULT_FAILED: "failed",
 *     NVTXW_RESULT_INVALID_ARGUMENT: "invalid argument",
 *     NVTXW_RESULT_LIBRARY_NOT_FOUND: "library not found",             # <<<<<<<<<<<<<<
 *     NVTXW_RESULT_LIBRARY_LOAD_FAILED: "library load failed",
 *     NVTXW_RESULT_LIBRARY_SYMBOL_MISSING: "library symbol missing",
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_RESULT_LIBRARY_NOT_FOUND); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_9, __pyx_t_2, __pyx_mstate_global->__pyx_kp_u_library_not_found) < (0)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":94
 *     NVTXW_RESULT_INVALID_ARGUMENT: "invalid argument",
 *     NVTXW_RESULT_LIBRARY_NOT_FOUND: "library not found",
 *     NVTXW_RESULT_LIBRARY_LOAD_FAILED: "library load failed",             # <<<<<<<<<<<<<<
 *     NVTXW_RESULT_LIBRARY_SYMBOL_MISSING: "library symbol missing",
 *     NVTXW_RESULT_INTERFACE_VERSION_NOT_SUPPORTED:
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_RESULT_LIBRARY_LOAD_FAILED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_9, __pyx_t_2, __pyx_mstate_global->__pyx_kp_u_library_load_failed) < (0)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":95
 *     NVTXW_RESULT_LIBRARY_NOT_FOUND: "library not found",
 *     NVTXW_RESULT_LIBRARY_LOAD_FAILED: "library load failed",
 *     NVTXW_RESULT_LIBRARY_SYMBOL_MISSING: "library symbol missing",             # <<<<<<<<<<<<<<
 *     NVTXW_RESULT_INTERFACE_VERSION_NOT_SUPPORTED:
 *         "interface version not supported",
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_RESULT_LIBRARY_SYMBOL_MISSING); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_9, __pyx_t_2, __pyx_mstate_global->__pyx_kp_u_library_symbol_missing) < (0)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":96
 *     NVTXW_RESULT_LIBRARY_LOAD_FAILED: "library load failed",
 *     NVTXW_RESULT_LIBRARY_SYMBOL_MISSING: "library symbol missing",
 *     NVTXW_RESULT_INTERFACE_VERSION_NOT_SUPPORTED:             # <<<<<<<<<<<<<<
 *         "interface version not supported",
 *     NVTXW_RESULT_NOT_SUPPORTED: "not supported by this backend",
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_RESULT_INTERFACE_VERSION_NOT_SUPPORTED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_9, __pyx_t_2, __pyx_mstate_global->__pyx_kp_u_interface_version_not_supported) < (0)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":98
 *     NVTXW_RESULT_INTERFACE_VERSION_NOT_SUPPORTED:
 *         "interface version not supported",
 *     NVTXW_RESULT_NOT_SUPPORTED: "not supported by this backend",             # <<<<<<<<<<<<<<
 * }
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTXW_RESULT_NOT_SUPPORTED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_9, __pyx_t_2, __pyx_mstate_global->__pyx_kp_u_not_supported_by_this_backend) < (0)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_RESULT_MESSAGES, __pyx_t_9) < (0)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "nvtx/_lib/writer.pyx":102
 * 
 * 
 * class WriterError(Exception):             # <<<<<<<<<<<<<<
 *     """
 *     Raised when an NVTXW backend call returns a non-success result code.
*/
  __pyx_t_9 = PyTuple_Pack(1, ((PyObject *)(((PyTypeObject*)PyExc_Exception)))); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = __Pyx_PEP560_update_bases(__pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_4 = __Pyx_Py3MetaclassPrepare(__pyx_t_10, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_WriterError, __pyx_mstate_global->__pyx_n_u_WriterError, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_kp_u_Raised_when_an_NVTXW_backend_ca); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_2 != __pyx_t_9) {
    if (unlikely((PyDict_SetItemString(__pyx_t_4, "__orig_bases__", __pyx_t_9) < 0))) __PYX_ERR(0, 102, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);

  /* "nvtx/_lib/writer.pyx":121
 *     """
 * 
 *     def __init__(self, int result_code, operation=None):             # <<<<<<<<<<<<<<
 *         self.result_code = result_code
 *         self.operation = operation
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_11WriterError_1__init__, 0, __pyx_mstate_global->__pyx_n_u_WriterError___init, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  PyList_Append(__pyx_t_9, __pyx_t_12);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_12, __pyx_mstate_global->__pyx_tuple[1]);
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_init, __pyx_t_12) < (0)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":102
 * 
 * 
 * class WriterError(Exception):             # <<<<<<<<<<<<<<
 *     """
 *     Raised when an NVTXW backend call returns a non-success result code.
*/
  __pyx_t_12 = __Pyx_Py3ClassCreate(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_WriterError, __pyx_t_2, __pyx_t_4, NULL, 0, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  if (__Pyx_CyFunction_InitClassCell(__pyx_t_9, __pyx_t_12) < (0)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_WriterError, __pyx_t_12) < (0)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_1__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriterSchemaRegistrar___reduce, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__WriterSchemaRegistrar, (type(self), 0x7ba2b5b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__WriterSchemaRegistrar__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_22_WriterSchemaRegistrar_3__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriterSchemaRegistrar___setstat, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer__WriterSchemaRegistrar, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":241
 * 
 * 
 * def _validate_counter_layout(dtype):             # <<<<<<<<<<<<<<
 *     """
 *     Reject event entry roles in a counter layout; return the name of the
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_1_validate_counter_layout, 0, __pyx_mstate_global->__pyx_n_u_validate_counter_layout, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_validate_counter_layout, __pyx_t_2) < (0)) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":276
 * 
 * 
 * def _batch_rows(data, dt):             # <<<<<<<<<<<<<<
 *     """Return batch data as a flat, contiguous array of ``dt`` rows."""
 *     if isinstance(data, np.ndarray):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_3_batch_rows, 0, __pyx_mstate_global->__pyx_n_u_batch_rows, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[4])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_batch_rows, __pyx_t_2) < (0)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":299
 * 
 * _COMPLETE_EVENT_KINDS = (
 *     EventKind.MARK,             # <<<<<<<<<<<<<<
 *     EventKind.RANGE_PUSHPOP,
 *     EventKind.RANGE_STARTEND,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_MARK); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":300
 * _COMPLETE_EVENT_KINDS = (
 *     EventKind.MARK,
 *     EventKind.RANGE_PUSHPOP,             # <<<<<<<<<<<<<<
 *     EventKind.RANGE_STARTEND,
 * )
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_RANGE_PUSHPOP); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":301
 *     EventKind.MARK,
 *     EventKind.RANGE_PUSHPOP,
 *     EventKind.RANGE_STARTEND,             # <<<<<<<<<<<<<<
 * )
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_EventKind); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_RANGE_STARTEND); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":299
 * 
 * _COMPLETE_EVENT_KINDS = (
 *     EventKind.MARK,             # <<<<<<<<<<<<<<
 *     EventKind.RANGE_PUSHPOP,
 *     EventKind.RANGE_STARTEND,
*/
  __pyx_t_2 = PyTuple_Pack(3, __pyx_t_10, __pyx_t_4, __pyx_t_12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_COMPLETE_EVENT_KINDS, __pyx_t_2) < (0)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":305
 * 
 * 
 * def _validate_native_byte_order(dt, operation):             # <<<<<<<<<<<<<<
 *     subdtype = dt.subdtype
 *     if subdtype is not None:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_5_validate_native_byte_order, 0, __pyx_mstate_global->__pyx_n_u_validate_native_byte_order, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[5])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_validate_native_byte_order, __pyx_t_2) < (0)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":320
 * 
 * 
 * def _validate_event_dtype(kind, dt, operation):             # <<<<<<<<<<<<<<
 *     """Validate the static layout and event roles of an event dtype."""
 *     if dt.fields is None:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7_validate_event_dtype, 0, __pyx_mstate_global->__pyx_n_u_validate_event_dtype, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[6])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_validate_event_dtype, __pyx_t_2) < (0)) __PYX_ERR(0, 320, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":392
 * 
 * 
 * def _canonical_backend_path(path):             # <<<<<<<<<<<<<<
 *     """Return the canonical registry key and filename for a backend path."""
 *     path = Path(os.fsdecode(path)).expanduser().resolve()
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_9_canonical_backend_path, 0, __pyx_mstate_global->__pyx_n_u_canonical_backend_path, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[7])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_canonical_backend_path, __pyx_t_2) < (0)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_14_BackendModule_3__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_BackendModule___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[8])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_14_BackendModule_5__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_BackendModule___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[9])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":437
 * 
 * 
 * def _release_backend_module(module):             # <<<<<<<<<<<<<<
 *     """Drop one wrapper reference and finalize the module at zero."""
 *     cdef _BackendModule loaded = module
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_11_release_backend_module, 0, __pyx_mstate_global->__pyx_n_u_release_backend_module, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[10])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_release_backend_module, __pyx_t_2) < (0)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":470
 *             raise RuntimeError("backend is closed")
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """
 *         Finalize and release the backend.
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Backend_3close, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Backend_close, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[11])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Backend, __pyx_mstate_global->__pyx_n_u_close, __pyx_t_2) < (0)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Backend_5__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Backend___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[12])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Backend_7__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Backend___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[13])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":509
 * 
 * 
 * def load_backend(path):             # <<<<<<<<<<<<<<
 *     """
 *     Load and initialize an NVTXW backend from an explicit library path.
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_13load_backend, 0, __pyx_mstate_global->__pyx_n_u_load_backend, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[14])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_load_backend, __pyx_t_2) < (0)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":596
 *         self._domains = {}
 * 
 *     def begin(self):             # <<<<<<<<<<<<<<
 *         """
 *         Start the session.
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Session_3begin, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Session_begin, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[15])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Session, __pyx_mstate_global->__pyx_n_u_begin, __pyx_t_2) < (0)) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":636
 *                     raise WriterError(rc, "SessionBegin")
 * 
 *     def end(self):             # <<<<<<<<<<<<<<
 *         """
 *         End the session.
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Session_5end, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Session_end, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[16])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Session, __pyx_mstate_global->__pyx_n_u_end, __pyx_t_2) < (0)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":685
 *             raise error
 * 
 *     def _register_domain(self, name):             # <<<<<<<<<<<<<<
 *         cdef nvtxwDomainAttributes_t attr
 *         cdef nvtxDomainHandle_t handle = NULL
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Session_7_register_domain, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Session__register_domain, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[17])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 685, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Session, __pyx_mstate_global->__pyx_n_u_register_domain, __pyx_t_2) < (0)) __PYX_ERR(0, 685, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":700
 *         return Domain(self, <uintptr_t>handle)
 * 
 *     def get_domain(self, name=None):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a domain within this session.
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Session_9get_domain, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Session_get_domain, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[18])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 700, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[1]);
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Session, __pyx_mstate_global->__pyx_n_u_get_domain, __pyx_t_2) < (0)) __PYX_ERR(0, 700, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":742
 *         domain=None,
 *         scope=None,
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,             # <<<<<<<<<<<<<<
 *         interleaving=StreamInterleaving.NONE,
 *         ordering=StreamOrdering.UNKNOWN,
*/
  __pyx_t_2 = __Pyx_PyLong_From_uint64_t(NVTX_TIME_DOMAIN_ID_NONE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_mstate_global->__pyx_k__3 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":743
 *         scope=None,
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
 *         interleaving=StreamInterleaving.NONE,             # <<<<<<<<<<<<<<
 *         ordering=StreamOrdering.UNKNOWN,
 *         skid=StreamSkid.NONE,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_StreamInterleaving); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_NONE); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_mstate_global->__pyx_k__4 = __pyx_t_12;
  __Pyx_GIVEREF(__pyx_t_12);
  __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":744
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
 *         interleaving=StreamInterleaving.NONE,
 *         ordering=StreamOrdering.UNKNOWN,             # <<<<<<<<<<<<<<
 *         skid=StreamSkid.NONE,
 *         skid_amount=0,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_StreamOrdering); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_UNKNOWN); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_mstate_global->__pyx_k__5 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":745
 *         interleaving=StreamInterleaving.NONE,
 *         ordering=StreamOrdering.UNKNOWN,
 *         skid=StreamSkid.NONE,             # <<<<<<<<<<<<<<
 *         skid_amount=0,
 *     ):
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_StreamSkid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_NONE); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_mstate_global->__pyx_k__6 = __pyx_t_12;
  __Pyx_GIVEREF(__pyx_t_12);
  __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":736
 *             return domain
 * 
 *     def create_stream(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/
  __pyx_t_12 = __Pyx_PyDict_NewPresized(7); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);

  /* "nvtx/_lib/writer.pyx":740
 *         name,
 *         *,
 *         domain=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
*/
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_domain, Py_None) < (0)) __PYX_ERR(0, 736, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":741
 *         *,
 *         domain=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
 *         interleaving=StreamInterleaving.NONE,
*/
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_scope, Py_None) < (0)) __PYX_ERR(0, 736, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":742
 *         domain=None,
 *         scope=None,
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,             # <<<<<<<<<<<<<<
 *         interleaving=StreamInterleaving.NONE,
 *         ordering=StreamOrdering.UNKNOWN,
*/
  __pyx_t_2 = __Pyx_PyLong_From_uint64_t(NVTX_TIME_DOMAIN_ID_NONE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_time_domain_id, __pyx_t_2) < (0)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":743
 *         scope=None,
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
 *         interleaving=StreamInterleaving.NONE,             # <<<<<<<<<<<<<<
 *         ordering=StreamOrdering.UNKNOWN,
 *         skid=StreamSkid.NONE,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_StreamInterleaving); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_NONE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_interleaving, __pyx_t_4) < (0)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":744
 *         time_domain_id=NVTX_TIME_DOMAIN_ID_NONE,
 *         interleaving=StreamInterleaving.NONE,
 *         ordering=StreamOrdering.UNKNOWN,             # <<<<<<<<<<<<<<
 *         skid=StreamSkid.NONE,
 *         skid_amount=0,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_StreamOrdering); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_UNKNOWN); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_ordering, __pyx_t_2) < (0)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":745
 *         interleaving=StreamInterleaving.NONE,
 *         ordering=StreamOrdering.UNKNOWN,
 *         skid=StreamSkid.NONE,             # <<<<<<<<<<<<<<
 *         skid_amount=0,
 *     ):
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_StreamSkid); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_NONE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_skid, __pyx_t_4) < (0)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_skid_amount, __pyx_mstate_global->__pyx_int_0) < (0)) __PYX_ERR(0, 736, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":736
 *             return domain
 * 
 *     def create_stream(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Session_11create_stream, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Session_create_stream, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[19])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_4, __pyx_t_12);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Session, __pyx_mstate_global->__pyx_n_u_create_stream, __pyx_t_4) < (0)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":828
 *             )
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.begin()
 *         return self
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Session_13__enter__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Session___enter, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[20])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 828, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Session, __pyx_mstate_global->__pyx_n_u_enter, __pyx_t_4) < (0)) __PYX_ERR(0, 828, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":832
 *         return self
 * 
 *     def __exit__(self, *_):             # <<<<<<<<<<<<<<
 *         # Backend.close() may already have ended this session.
 *         with self._backend._lock:
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Session_15__exit__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Session___exit, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[21])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 832, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Session, __pyx_mstate_global->__pyx_n_u_exit, __pyx_t_4) < (0)) __PYX_ERR(0, 832, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Session_17__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Session___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[22])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._handle cannot be converted to a Python object for pickling"
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Session_19__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Session___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[23])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_16RegisteredString_5__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RegisteredString___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[24])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_16RegisteredString_7__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RegisteredString___setstate_cyth, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[25])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_5Scope_5__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Scope___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[26])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_5Scope_7__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Scope___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[27])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Counter_5__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Counter___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[28])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_7Counter_7__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Counter___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[29])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Schema_5__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Schema___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[30])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Schema_7__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Schema___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[31])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1035
 *             schema_ids_out[0] = self._event_schema_ids
 * 
 *     def get_registered_string(self, string):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a registered string in this domain.
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_3get_registered_string, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain_get_registered_string, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[32])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1035, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_mstate_global->__pyx_n_u_get_registered_string, __pyx_t_4) < (0)) __PYX_ERR(0, 1035, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1065
 *             return self._get_string_cached(_as_str(string))
 * 
 *     def _register_string(self, string):             # <<<<<<<<<<<<<<
 *         cdef bytes encoded = _as_bytes(string)
 *         cdef const char* c_string = encoded
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_5_register_string, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain__register_string, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[33])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1065, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_mstate_global->__pyx_n_u_register_string, __pyx_t_4) < (0)) __PYX_ERR(0, 1065, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1074
 *         return RegisteredString(self, string, <uintptr_t>handle)
 * 
 *     def get_category_id(self, name: str | bytes):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a named category in this domain.
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1074, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_name, __pyx_mstate_global->__pyx_kp_u_str_bytes) < (0)) __PYX_ERR(0, 1074, __pyx_L1_error)
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_7get_category_id, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain_get_category_id, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[34])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1074, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_12, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_mstate_global->__pyx_n_u_get_category_id, __pyx_t_12) < (0)) __PYX_ERR(0, 1074, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":1108
 *         return self._register_category(name)
 * 
 *     def _register_category(self, name):             # <<<<<<<<<<<<<<
 *         cdef uint32_t category_id
 *         cdef bytes encoded
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_9_register_category, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain__register_category, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[35])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_mstate_global->__pyx_n_u_register_category, __pyx_t_12) < (0)) __PYX_ERR(0, 1108, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":1132
 *             return category_id
 * 
 *     def get_scope(self, path, *, parent=None):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a scope in this domain.
*/
  __pyx_t_12 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_parent, Py_None) < (0)) __PYX_ERR(0, 1132, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_11get_scope, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain_get_scope, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[36])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_4, __pyx_t_12);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_mstate_global->__pyx_n_u_get_scope, __pyx_t_4) < (0)) __PYX_ERR(0, 1132, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1194
 *             return self._get_scope_cached(_as_str(path), parent_id)
 * 
 *     def _register_scope(self, path, uint64_t parent_id):             # <<<<<<<<<<<<<<
 *         cdef bytes encoded = _as_bytes(path)
 *         cdef const char* c_path = NULL
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_13_register_scope, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain__register_scope, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[37])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_mstate_global->__pyx_n_u_register_scope, __pyx_t_4) < (0)) __PYX_ERR(0, 1194, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1210
 *         return Scope(scope_id, path, self)
 * 
 *     def get_schema(self, dtype, *, kind=None):             # <<<<<<<<<<<<<<
 *         """
 *         Get or create a payload schema in this domain.
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_kind, Py_None) < (0)) __PYX_ERR(0, 1210, __pyx_L1_error)
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_15get_schema, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain_get_schema, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[38])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_12, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_mstate_global->__pyx_n_u_get_schema, __pyx_t_12) < (0)) __PYX_ERR(0, 1210, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":1272
 *             return schema
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/
  __pyx_t_12 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);

  /* "nvtx/_lib/writer.pyx":1277
 *         dtype,
 *         *,
 *         description=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         semantics=None,
*/
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_description, Py_None) < (0)) __PYX_ERR(0, 1272, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1278
 *         *,
 *         description=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         semantics=None,
 *     ):
*/
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_scope, Py_None) < (0)) __PYX_ERR(0, 1272, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1279
 *         description=None,
 *         scope=None,
 *         semantics=None,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_semantics, Py_None) < (0)) __PYX_ERR(0, 1272, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1272
 *             return schema
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_17get_counter, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain_get_counter, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[39])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_4, __pyx_t_12);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_mstate_global->__pyx_n_u_get_counter, __pyx_t_4) < (0)) __PYX_ERR(0, 1272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1342
 *             )
 * 
 *     def _register_counter(             # <<<<<<<<<<<<<<
 *         self, name, schema_key, description, uint64_t scope_id, semantics
 *     ):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_19_register_counter, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain__register_counter, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[40])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Domain, __pyx_mstate_global->__pyx_n_u_register_counter, __pyx_t_4) < (0)) __PYX_ERR(0, 1342, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_21__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[41])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Domain_23__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Domain___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[42])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1445
 *         self._writer.iface = self._backend._iface
 * 
 *     def open(self):             # <<<<<<<<<<<<<<
 *         """
 *         Open the stream for writing.
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_3open, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_open, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[43])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_open, __pyx_t_4) < (0)) __PYX_ERR(0, 1445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1486
 *             self._session._open_streams.add(self)
 * 
 *     def close(self):             # <<<<<<<<<<<<<<
 *         """
 *         Close the stream without ending its session.
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_5close, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_close, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[44])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_close, __pyx_t_4) < (0)) __PYX_ERR(0, 1486, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1509
 *             _check(rc, "StreamClose")
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         self.open()
 *         return self
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_7__enter__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream___enter, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[45])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_enter, __pyx_t_4) < (0)) __PYX_ERR(0, 1509, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1513
 *         return self
 * 
 *     def __exit__(self, *_):             # <<<<<<<<<<<<<<
 *         with self._session._lock:
 *             if self._writer.stream != NULL:
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_9__exit__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream___exit, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[46])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_exit, __pyx_t_4) < (0)) __PYX_ERR(0, 1513, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1609
 *             _check(utf8_fn(writer, begin, end, uattr), utf8_op)
 * 
 *     def write_mark(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "nvtx/_lib/writer.pyx":1610
 * 
 *     def write_mark(
 *         self, timestamp, *, message=None, color=None, category=None             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_message, Py_None) < (0)) __PYX_ERR(0, 1609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_color, Py_None) < (0)) __PYX_ERR(0, 1609, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_category, Py_None) < (0)) __PYX_ERR(0, 1609, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1609
 *             _check(utf8_fn(writer, begin, end, uattr), utf8_op)
 * 
 *     def write_mark(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_11write_mark, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_mark, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[47])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_12, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_mark, __pyx_t_12) < (0)) __PYX_ERR(0, 1609, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":1642
 *                       "nvtxwMarkWrite", "nvtxwMarkWriteUtf8")
 * 
 *     def write_pushpop(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/
  __pyx_t_12 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1642, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);

  /* "nvtx/_lib/writer.pyx":1643
 * 
 *     def write_pushpop(
 *         self, start, end, *, message=None, color=None, category=None             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_message, Py_None) < (0)) __PYX_ERR(0, 1642, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_color, Py_None) < (0)) __PYX_ERR(0, 1642, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_category, Py_None) < (0)) __PYX_ERR(0, 1642, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1642
 *                       "nvtxwMarkWrite", "nvtxwMarkWriteUtf8")
 * 
 *     def write_pushpop(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_13write_pushpop, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_pushpop, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[48])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1642, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_4, __pyx_t_12);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_pushpop, __pyx_t_4) < (0)) __PYX_ERR(0, 1642, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1678
 *                         "nvtxwRangePushPopWriteUtf8")
 * 
 *     def write_startend(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1678, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "nvtx/_lib/writer.pyx":1679
 * 
 *     def write_startend(
 *         self, start, end, *, message=None, color=None, category=None             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_message, Py_None) < (0)) __PYX_ERR(0, 1678, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_color, Py_None) < (0)) __PYX_ERR(0, 1678, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_category, Py_None) < (0)) __PYX_ERR(0, 1678, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1678
 *                         "nvtxwRangePushPopWriteUtf8")
 * 
 *     def write_startend(             # <<<<<<<<<<<<<<
 *         self, start, end, *, message=None, color=None, category=None
 *     ):
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_15write_startend, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_startend, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[49])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1678, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_12, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_startend, __pyx_t_12) < (0)) __PYX_ERR(0, 1678, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":1714
 *                         "nvtxwRangeStartEndWriteUtf8")
 * 
 *     def write_push(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/
  __pyx_t_12 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1714, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);

  /* "nvtx/_lib/writer.pyx":1715
 * 
 *     def write_push(
 *         self, timestamp, *, message=None, color=None, category=None             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_message, Py_None) < (0)) __PYX_ERR(0, 1714, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_color, Py_None) < (0)) __PYX_ERR(0, 1714, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_category, Py_None) < (0)) __PYX_ERR(0, 1714, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1714
 *                         "nvtxwRangeStartEndWriteUtf8")
 * 
 *     def write_push(             # <<<<<<<<<<<<<<
 *         self, timestamp, *, message=None, color=None, category=None
 *     ):
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_17write_push, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_push, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[50])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1714, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_4, __pyx_t_12);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_push, __pyx_t_4) < (0)) __PYX_ERR(0, 1714, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1749
 *                       "nvtxwRangePushWrite", "nvtxwRangePushWriteUtf8")
 * 
 *     def write_pop(self, timestamp):             # <<<<<<<<<<<<<<
 *         """
 *         Write the end of the most recently pushed range.
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_19write_pop, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_pop, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[51])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1749, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_pop, __pyx_t_4) < (0)) __PYX_ERR(0, 1749, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1769
 *             "nvtxwRangePopWrite")
 * 
 *     def write_start(             # <<<<<<<<<<<<<<
 *         self,
 *         timestamp,
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1769, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "nvtx/_lib/writer.pyx":1774
 *         range_id,
 *         *,
 *         message=None,             # <<<<<<<<<<<<<<
 *         color=None,
 *         category=None,
*/
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_message, Py_None) < (0)) __PYX_ERR(0, 1769, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1775
 *         *,
 *         message=None,
 *         color=None,             # <<<<<<<<<<<<<<
 *         category=None,
 *     ):
*/
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_color, Py_None) < (0)) __PYX_ERR(0, 1769, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1776
 *         message=None,
 *         color=None,
 *         category=None,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_category, Py_None) < (0)) __PYX_ERR(0, 1769, __pyx_L1_error)

  /* "nvtx/_lib/writer.pyx":1769
 *             "nvtxwRangePopWrite")
 * 
 *     def write_start(             # <<<<<<<<<<<<<<
 *         self,
 *         timestamp,
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_21write_start, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_start, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[52])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1769, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_12, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_start, __pyx_t_12) < (0)) __PYX_ERR(0, 1769, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":1827
 *                    "nvtxwRangeStartWriteUtf8")
 * 
 *     def write_end(self, timestamp, range_id):             # <<<<<<<<<<<<<<
 *         """
 *         Write the end of a start/end range.
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_23write_end, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_end, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[53])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1827, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_end, __pyx_t_12) < (0)) __PYX_ERR(0, 1827, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":1851
 *             "nvtxwRangeEndWrite")
 * 
 *     def write_event(self, Schema schema not None, row):             # <<<<<<<<<<<<<<
 *         """
 *         Write an event described by a role-annotated dtype schema.
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_25write_event, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_event, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[54])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_event, __pyx_t_12) < (0)) __PYX_ERR(0, 1851, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":1912
 *     def write_event_batch(
 *         self, Schema schema not None, rows, *,
 *         ordering=BatchOrdering.SORTED,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_BatchOrdering); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_SORTED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_mstate_global->__pyx_k__8 = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":1910
 *         _check(rc, "EventWrite")
 * 
 *     def write_event_batch(             # <<<<<<<<<<<<<<
 *         self, Schema schema not None, rows, *,
 *         ordering=BatchOrdering.SORTED,
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1910, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "nvtx/_lib/writer.pyx":1912
 *     def write_event_batch(
 *         self, Schema schema not None, rows, *,
 *         ordering=BatchOrdering.SORTED,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_BatchOrdering); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_SORTED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ordering, __pyx_t_2) < (0)) __PYX_ERR(0, 1910, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":1910
 *         _check(rc, "EventWrite")
 * 
 *     def write_event_batch(             # <<<<<<<<<<<<<<
 *         self, Schema schema not None, rows, *,
 *         ordering=BatchOrdering.SORTED,
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_27write_event_batch, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_event_batch, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[55])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1910, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_event_batch, __pyx_t_2) < (0)) __PYX_ERR(0, 1910, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":1973
 *         _check(rc, "EventBatchWrite")
 * 
 *     def write_counter_sample(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, int64_t timestamp, value
 *     ):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_29write_counter_sample, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_counter_sample, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[56])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_counter_sample, __pyx_t_2) < (0)) __PYX_ERR(0, 1973, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":2043
 *         _check(rc, "CounterWrite")
 * 
 *     def write_counter_sample_no_value(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, int64_t timestamp, reason
 *     ):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_31write_counter_sample_no_value, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_counter_sample_no_v, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[57])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2043, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_counter_sample_no_value, __pyx_t_2) < (0)) __PYX_ERR(0, 2043, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/writer.pyx":2083
 *     def write_counter_batch(
 *         self, Counter counter not None, data, *,
 *         timestamps=None, ordering=BatchOrdering.SORTED,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_BatchOrdering); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2083, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_SORTED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2083, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_mstate_global->__pyx_k__9 = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "nvtx/_lib/writer.pyx":2081
 *             "CounterNoValueWrite")
 * 
 *     def write_counter_batch(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, data, *,
 *         timestamps=None, ordering=BatchOrdering.SORTED,
*/
  __pyx_t_4 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 2081, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "nvtx/_lib/writer.pyx":2083
 *     def write_counter_batch(
 *         self, Counter counter not None, data, *,
 *         timestamps=None, ordering=BatchOrdering.SORTED,             # <<<<<<<<<<<<<<
 *     ):
 *         """
*/
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_timestamps, Py_None) < (0)) __PYX_ERR(0, 2081, __pyx_L1_error)
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_BatchOrdering); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2083, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_SORTED); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 2083, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ordering, __pyx_t_12) < (0)) __PYX_ERR(0, 2081, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":2081
 *             "CounterNoValueWrite")
 * 
 *     def write_counter_batch(             # <<<<<<<<<<<<<<
 *         self, Counter counter not None, data, *,
 *         timestamps=None, ordering=BatchOrdering.SORTED,
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_33write_counter_batch, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream_write_counter_batch, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[58])); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 2081, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_12, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_6writer_Stream, __pyx_mstate_global->__pyx_n_u_write_counter_batch, __pyx_t_12) < (0)) __PYX_ERR(0, 2081, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_35__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[59])); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_12) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_6Stream_37__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Stream___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[60])); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_12) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle__WriterSchemaRegistrar(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0x7ba2b5b, 0x17fb052, 0x61860e1, b'_array_cache, _domain, _dtype_cache, _fixed_cache')
*/
  __pyx_t_12 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_6writer_15__pyx_unpickle__WriterSchemaRegistrar, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle__WriterSchemaRegi, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_writer, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[61])); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_12);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle__WriterSchemaRegi, __pyx_t_12) < (0)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "nvtx/_lib/writer.pyx":1
 * # SPDX-FileCopyrightText: Copyright (c) 2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.             # <<<<<<<<<<<<<<
 * # SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 * #
*/
  __pyx_t_12 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_12) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init nvtx._lib.writer", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init nvtx._lib.writer");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */
/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_super); if (!__pyx_builtin_super) __PYX_ERR(0, 127, __pyx_L1_error)

  /* Cached unbound methods */
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.method_name = &__pyx_mstate->__pyx_n_u_get;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.method_name = &__pyx_mstate->__pyx_n_u_items;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.method_name = &__pyx_mstate->__pyx_n_u_values;
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "nvtx/_lib/writer.pyx":146
 *         cdef Domain domain = self._domain
 *         cdef nvtxwResultCode_t rc
 *         with domain._session._lock:             # <<<<<<<<<<<<<<
 *             domain._ensure_valid()
 *             rc = domain._backend._iface.SchemaRegister(
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);

  /* "nvtx/_lib/writer.pyx":121
 *     """
 * 
 *     def __init__(self, int result_code, operation=None):             # <<<<<<<<<<<<<<
 *         self.result_code = result_code
 *         self.operation = operation
*/
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(1, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_tuple;
    for (Py_ssize_t i=0; i<2; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  {
    const struct { const unsigned int length: 9; } index[] = {{1},{13},{10},{42},{68},{41},{179},{25},{480},{17},{17},{15},{60},{71},{1},{1},{2},{16},{72},{8},{15},{17},{40},{38},{47},{34},{43},{42},{30},{43},{34},{36},{7},{13},{49},{45},{7},{8},{37},{55},{8},{11},{6},{48},{23},{9},{7},{6},{2},{6},{57},{31},{54},{16},{100},{18},{41},{9},{7},{38},{19},{17},{22},{53},{55},{88},{26},{50},{29},{38},{46},{50},{48},{7},{77},{25},{43},{28},{65},{39},{56},{48},{38},{57},{41},{64},{25},{21},{38},{24},{11},{22},{18},{29},{54},{38},{103},{34},{15},{13},{21},{109},{136},{49},{154},{37},{69},{16},{21},{7},{14},{32},{34},{25},{27},{13},{13},{4},{21},{17},{18},{10},{16},{7},{17},{20},{19},{15},{12},{25},{27},{9},{6},{14},{24},{26},{25},{24},{22},{23},{22},{18},{28},{17},{16},{11},{9},{4},{15},{9},{10},{21},{26},{4},{4},{16},{18},{4},{16},{16},{15},{20},{11},{9},{8},{9},{10},{13},{11},{14},{16},{5},{4},{16},{34},{36},{5},{6},{6},{6},{14},{24},{26},{5},{13},{23},{25},{7},{12},{10},{17},{16},{25},{27},{24},{13},{21},{11},{18},{6},{11},{18},{10},{14},{10},{16},{15},{24},{26},{12},{11},{26},{27},{36},{16},{18},{24},{17},{16},{17},{20},{18},{21},{14},{7},{13},{7},{11},{20},{22},{40},{42},{1},{12},{6},{17},{18},{4},{7},{5},{11},{5},{6},{6},{8},{8},{5},{6},{23},{14},{4},{8},{11},{17},{18},{5},{5},{12},{6},{6},{4},{5},{7},{13},{10},{17},{13},{6},{4},{11},{17},{15},{8},{5},{7},{6},{13},{2},{5},{5},{7},{3},{9},{10},{10},{4},{5},{10},{6},{3},{8},{10},{14},{18},{1},{3},{6},{11},{10},{6},{8},{8},{6},{8},{9},{3},{15},{11},{10},{23},{21},{10},{9},{12},{6},{11},{9},{1},{3},{5},{8},{5},{12},{13},{8},{5},{8},{9},{4},{3},{12},{6},{8},{7},{13},{8},{6},{10},{15},{4},{8},{10},{5},{6},{7},{4},{7},{4},{24},{8},{2},{5},{11},{4},{13},{16},{14},{11},{25},{25},{14},{18},{18},{18},{22},{26},{19},{23},{23},{27},{20},{24},{4},{9},{8},{2},{5},{6},{6},{9},{4},{7},{7},{2},{3},{11},{12},{14},{12},{11},{10},{37},{14},{12},{8},{15},{2},{6},{10},{17},{13},{18},{17},{16},{15},{16},{23},{14},{7},{7},{11},{3},{4},{11},{3},{4},{7},{6},{12},{9},{10},{5},{8},{4},{9},{7},{12},{10},{12},{19},{4},{4},{11},{5},{5},{6},{6},{8},{7},{5},{8},{9},{14},{9},{15},{16},{10},{13},{8},{5},{6},{12},{24},{21},{27},{5},{6},{19},{20},{29},{9},{11},{17},{10},{9},{10},{13},{11},{14},{6},{41},{0},{142},{11},{44},{55},{58},{214},{86},{84},{179},{239},{36},{36},{266},{348},{39},{39},{45},{14},{14},{124},{32},{194},{105},{110},{70},{232},{147},{25},{110},{454},{239},{29},{97},{130},{162},{67},{9},{175},{162},{95},{59},{506},{104},{221},{166},{67}};
    #if (CYTHON_COMPRESS_STRINGS) == 3 && __PYX_LIMITED_VERSION_HEX >= 0x030e0000 /* compression: zstd (6359 bytes) */
const char* const cstring = "(\265/\375`\3178m\306\000\272\274\374'H\360\354\272\001\314\300\304\210\233\006\254N<\020Lg\212\244\014\362\ts\323BF\360\370\300&3\322\253\365\006\262\265\234|B\326\001>t\335\004\351\316F\310\302\307#\003\025\\\340\304\320\276\331\003'\202v\363\342\002\005\366\004\336,A\002R\002\207\002\233\246yy\312l9>\323\351\212\006h\345\304\262\"\302fr%\357\242z\252\360I\245\253\264Tc\223d\301p\351\306\351Y\272\001\353\024\253X\"\307\264\335O\032\236\021i\342F\205vJ\252\267r\327hFsc>\222\245\306a\276g\232\323\314\263\3342\025\274K\231\346\344X\3534s\227\034\247k\235w\333\251\222\232Vp\007\024n\301.\016\024(p\301\246\006\033L\246\\z\322v\327\242I\312l\"\017:\323]\2426\315\365.MRw\223\233\333F\216-\370\234a\323\246\226\234\2236\335\365\210-\234\322\223P\307\231[\246\006\235\345\314T\361\026\315Y\251\216g)bS\357<\216z,7\217N\030\203\255tX\261\374Z\305Z=;)\214;?\265\314\266m\270\353Rc\023\205\323\t-O~\246\242FQ\235j\225\226\356q\327~N(:b\234\323i\252\361\016A\362\226q\013\2213\\\226-\350\031\023a4#\231\371\212\266\355\232\010[I;lwN\322\366\313\014:M <#\261i\233g\013\247'\016\322\351\356\252\254\207\343n\225\332\224n\364<\225\244\236\345d\037>(\310\261\234\036\330\352\225w-\326#\340\334\302S\315\203q\332\343)\342^\252\234\363-\212*o1O\023k\314\315T\355;\0257\220\250e\370\007C\314\247\305I\375\302\030*V^\331\264\302\327)6\352\263\314i\236\323\346\245\256\33182%G\254B3\311l\236d\327\3754M\255\3227\307&\327\350\270\267\211\3253\207g,|\342\033\315\365\222Texvb\2769\327\245(S9\346]\334 v\212GS\343\027\231\227\026\236\025\0176E2W\356\256\323-#\313=\232g\255\347R\270\353\335\317\326\251&v-\313b\336\235L\203\360\346*>;\316\2416\325\304e\251;7Y'\226\017\037l:\177\370\200k\235\014JI\215\256\2203\262\233Z\275aN\342\223\221\004\342'\270\245\324dO5j\323;\355\265\216M\022\217\273\247'\307;\325\230ws\3436\316\225\245:\303A\347\315i4I\234\\\332\315\025\315\266is%\335\335T\317\024K,\003\0237\364%\2437\374\250\357a\375\030\275\332\273|\000\237\262v\214^\327\212`\215\321e\210^\375]\237\364K\240\013""\212^\214\213\327\305\206\254\351\303\005\t@\3412\364\231\340:SP\260\254(\330\311\361\244<\217\250\252\236\03553\037k\250\312e*\237\225;\267\201h\272\234\345\221\347q\013m1\315\245Lw\3346U\233\312\201Ht\247h\316#\212\312\310\222\353t\363\023V\343*z\362\274df\275G\207\007\345\314O\250\271KmKO\241\240m\216\231o\362Lu\3179\r\3019\2319\231;\371\002C\222\\\327\322q\324\231\306\313\363\354\242\251\316r\346\263\031\317/0\263\031O7WQ>\232;\327Q\333\036\373\230\305s\034;\232\311=\212\\Di9\0329\023\332\265|\366\234\236\345)n1\333r\316\235\347R\213\344\250E4\217\344\211\313\242\230\305,fY\332\273\346\333$\273z\246\247x:I\321\314q\315\325S\245\221\\\243\350Nu\232Wi\353\247\325\003k\345\236\326\270\212\236(P\252m\270\234\332\262N\n\345=\232+\013\261\024\353\025\203\251v`+\036 \361\314y\301V6!\214\325b\331\254,\314\366\253\013\355V\356\"\251\022\312\361\344\234\3338<Y\305\223\nkX9\266\262\311+\235\260\225sv,\350d\320\300\312\t[\254\232lS\315U\256\321]\253~R0U\213\205\343\263\312\253\330*\207Gg'\225srT\251\276y\316\342I'\343\244vr`\253\234\036\325\216N\306I\205\026r\356\233vR\251Z\253\330\213g\207\347\006\326\303\203M8\257P\3474\303[v/\321-\212\232\353\031\276wi\252\243\251\323\215M\225e\205\343H\256\226\242\220V;\271\3133\305\261\234\315\021B9\224Af\300@L9l,\201D\022\225\235}\353Z>\231\332\304\307\215+\317\026S\325\346Lz\366\221\014j\231\225\233s\365\265\325R\373\234-\363\344g\272C\350\370\244'\353&\025\352\344\237\233\034k\255^\257\236\0246\305vvV\255\235\026Fij]\243\3304\356\224G\243;\026r\356\244I\212\245&q[\035q\316[Ts\326\263=j\241\235mg:\303:%\367\342\373*\217\322\255\311HrK\340\217y\223\001A\035\004J\377\004\367B\023!O\335\341\2647l\357\343=\330e\264\"\326C\263.\026\367\177\016\270/\276\000\007\321\2273\324\001\276\033\004\336\016\220\002c\250\264\326\334\300u\245w3\022~\367kj\205\20024\"\362\212\374\260\254\257~\274\003-*\301\273\200\204\037\320\003\301\026Urc:\r\003\256\313k\354d\022\270\241\377Y#x\235\333NNO{\003#\315\2539\271\247\022\327\242\247\275\003\357\303\275""\2065@\350=\274\210\366\"\253\207\013\211K\"\002Y\220\275\370\370\243\\P\214X\024\333\277\201\217\002t\201\357d\332\316jV\000{\373\032h>\303JZS@6\366\023\204\356\315\277\370\014\214\017\025\023Z|\334\262\371X\030\004\375\035\310\254\275\257m\360K\027\024\".\266\005d\201\204\217]\371) /\330\252\260#\037hu\255,\335m\267\3216@yA\236\312\317\307\340Ml\353\307\n\216z\245\037\343\013\220\350\025\2044\333\251\210\270\262\025d\324\003Y\226\234\225\244Q\"d!|\031\337\327\020\025\360\213\3364r\355X\255E\213bm>J\207\373\325\254]\007\376\013/B\3612v!\355\005\253\372(\242^i\027\323\332\2607\237\304\212\203\3626\353\215\366\002|\216N4\024\002\362\352\235\200\335\255\215I\251aU\222{\262v\010\030By\000*0\237\nd@\273\341\263\374\rvC\374\2056\031\271\245\277\342]\215\013\n\231\024\017\267\243\202V\t\250q\\AV\275\006\3516D-\335H\244\377\307W\261pK>\304\265\243Q\340\216\013@/\256\016\n\260*\177\304\307\326\217V\2675\366d\342\372|m\316\377\361&\037/\013\216\313\223\374\010G\312o\241\224x|\030_@\204\306\027 \315\274\001\221\367\203\364\362K\177\364\027Jl\007\317ZI\034\357\355\353o\362$\244\027\365\001\374\334\312\361\001~+\210jKm\014[b\025\204\270/\332\023X9Fo\311\217\254\240\306\307>\370,\036\364!F\027\024\r\206B\023\241W\240\275\211\346-\364\350\\\336[+\304\276\330\005\277\306\243<\313Z\261\336t/\272\014\213\336g\002\265\363i\303\366\305\342\375Lh \030\275\034\033/\312\227|\314\3521z/Do\350M\035\001\216\367\373 H\036o\351\227+\010\307\2531zo>\345\013\2540`\357\026\271,\311Y\306\246\024l\005\343V\335\323\354GNd\375z+\222\366\032\026n+\310\312\272\260>\243\312W\003\272)+\270\2068q\013d\270\243\2277\301%%#\032\013v=\036\033dE\020Z\027\244q!\265Z\2540Q\000\336\206\353\322\271\350z\2117\356\\\273\005\025'\357E\354&\007\256\251n#lQ\255\203\022\327\003DI\206[\322\361\033\317\006\306\315\300\226@Y{\002`\027F\272aH\020\322\351\217W\301\344z|\370\021\021\320\250\264\033i'\364\332\334\227\000\322\214:\002\027,>\245Sc~F\213\301\256\250\220\374\017\034\211m\211\320\303%\265\350\214/#\240\023Y\364""\364\032\022`%Y\364*H\021]\252?\270\037^\356\013\202+\032!\302\262x\270\036\026\357\313\345\002\361f\032`\227\325/\026\256\356\371\364\270\264\0233<\233\035j\327\333\230[\005O\305\316N\305M\305\315\027\202\202b(\247\272j\352\2466\3276sg\030\\\01417\346rW'~x,X\033\240\ra'\370\336\023\255p&\006X\2745+j\265\321\215,^\027\217\304c\320\216\007\010\004\370B\326d\001\365J'.\270\270*_\222Bzp\333Y\354@\260\016\330\255\343\272\240\362\352\203\230k]\240\303\000\375\347s\272\227k\027\356\362\210\373y\031\252%\361O:\334\216^O\263A\237\0340\242\321\023\215,\334O{\030\3760\217[CH\324\274\201\0170\245\324\202-\207O\250\3641DHJ\276\306\303\260\004~\210\037\001{$$\035\357\006\375k\021\300\322\216\002\273\200\304M9\3154\300\217W$\344ki>\002\rV\004\206\354\017\002\022\037O\226\303\362{\002>&\235\270\025\037+\373\246\031\017a\270\373ii&\035z3\226\357\271Z\334\211'\251\370)=\227\021\021\341\372\003\256\242\316\301\350\3430tc[\376\211\007o\302\210^\310\373\331\341Z\r,\r\034qGX\220\0207\204\303\005eC\036\035 \037\032\033\030F\254M\2670\t\334\2174\300\201\342IBh\261_\002\256\035\027\337\211\033\031\227\203\366\265q\315\240\001\203\206\253!\352}'Bx0\266\237\"\352\225<_AX\230\020\274eL\030\002\301\035\020[\373\002\376\315\037\330p]?\374\n6^\327\307\205\253\252y.\036n\247w\027n\254\201\336\307q\255\270\033O\363\004\360n;\r-\251\375\332\003\277\203L\006o\"\352\2112v\245\335J\353\022\206\353\250#\030\213\244\324H\315\210\210H\222\244\220\016S0 x\230\220P8!\227\351\371\223\363\343\251\\\210\222H)D\214##\"2IA\nR\224\024:6\177U9[\234\260-\034uE\3141^\\\255\321\275\277\2721uY\211\351\271lN\231\265\033\022i\021\330^q\0045\277\364b\365\214\023~\313zy\272\207\265\242dY\204\372\205y\356\203n_E\034\255^T\332l'\023|Y\320N\"g-4\030\250\225\265S7\035\311\030\367@\212\227/\245\216\227J\332Z\034F\363M1/8\347\031\312\203$\024\316\3017x\214\006\255\360\200c\247\225F\223\315\223[\310\177: 8\374\376\0346\003u\003\347\267\377,e\374\\LN\340\",\253,\224\r\250\201\276\2716^\004\220m\224<\352s\372\250\201\277\037=4""\310\016\215\376\240\"\365`V\370\245?\022\201=\343\347\325\347\340m\325\342\263|\001\000\220W\3347\302\353mJ\302\211\221\367~\025\310\242\2724\235\264M(\215\306\312\344@0V\330-\377\330X|\260`&#\300zO\260\rw\000\305\0100\362\0144\203-\277\302\275\306\001T\n\212\027\302^<\004\207\200H\217\305\005\314\353\252\317H6e\021\034\366\003\355$\376\275\205\344q\331\221\365\034\341%Rh\020\000\337s\325\261\001\235E+-\322$ \r\253\215{\315\304\214\240\243\003\255m\332\\d\020\2362\025\034\372\"\0358{l\205\021\306\224S\033\313E\"\321\341\364\357a\235,S`\274\245J\037\003\322\007l\221sj$\227\246\377\223\357F\362u\034:a\276U\367\221Q\003\276j\244\225\034\257\36037D\232{\214r\370\376\257\354\233&\363\005\314\2567=\021A:\312\033\252C+\335\227\364_k\372\346X2\247\242\001}\303\307\224\270\243\227\253\234\315g`\220\357\235\324\304J\314\341\325\212/\031\267Fw\271m\264\360E\301gM{\022\300\005\022\323(\210\335\265\324\255\216\306C\247k\271;\347k#\250\301\325\362(\351\220\306-3\005\236\317\273TMp\247\037\037\024:\351E\0328\020r\263%\352\215\241\366n\204\260 \236\273Q_\355~u~\346\232\032\334\031<xy\205\333\340\276\206\220\0271<pP\230\305\336*$~i \253{_fc\343\376\0348\233\032\230~\213S\302\230\017g\276\340\027I\3671\350\317\341)a\223pi\217\352FD\321\336\230$\246?\307\216C\rk\311_l\356\022\3540\206\025\014\307t\033\212$\253\004x\376\017%\343(R\276\252/\037 \013\340\230\350\006\310\220G\245\2406\340\027W]\230\027]\230\370k\026\364:_U6\225f\332\030\231\030\006f\r\353&\272\342\264a*Qk\317u\016\315\201\026\3337\313\267H?\302\035\244\243y\243\372\037\231\326\304\364\023\3365\017\340G\263\231\233\357\230\320t_\365H\350\220\234r\027\022\tH%n\022D\311a\032\233\026\272 Xg\307\3760\303Ag<D\307\242\317\014\0334\365\346\3556\326\3276al\023:\306i\245\245\234x\376\307\247i\203\225\rX\335!d\356o\310\250\023\257s)\t-C%\010S%\272\370\033 j\377\177\037\341_\033P\210\257o\270\331mK\341\"u\\\234\327\0168\2509\032\234O\010\330\002\260x\304\342\232sg\356\355v\3002[\272\314\320\244\345\021\3203\210\357\302,P96j\351\251""\207^\002N_\270\254!\014\265]9\240p\305U\300\232\244\334\247\347\351\177L\216\004\003\363~\177\237\260-B\314x\222\004\273\363\007,\355\257\034\310W\177!\030S#r\365-\004\230B\021}\306f\215`0-\347\224\353OQ\353\034\254\333R\354\025\000\220~\341\375V\350\230\032O\316i\275\224\360\321\210\332\245\001\364!\307\"\2010g\031%\241\215\031\033\343\337\376\247\335D\3254\002)\256\330\024\210\0326\214\"\315RS\222j\242\347\034\227{\330wL\312\260Nc\2308\276\342p\342h\017\213\201\005\264\233\004\3476lhW\311\302\233\033o\204q\314\256\346k\031\250\326\204O\255,\372\350D\326B\310\204Q\372\211\244\226\253\347\363\201?'*\236.g\235\206\273\352\322\250Y0\305J\247Rn\361\033\237\010\326_\014x\301\375\035\277\2202\253\035\360\273\367?\320\223l8\272\373\201\263$\320Gz^\374m\020\217\t%z\370y\243k\005\201^/ZT\030\215\302Sgm'\"\341.g@;\301.~\346\242\365MH>{gz\031\000-\200|\263\271\337$m\377x\200\317I*/P\306g\021C4\244\0358C\243E\032\212\002\241\000\361\263\202xi\260\264\235&X\305\231\356\271\007\\Y\222\376\231\241\231oO\\\001\372>\025\240+)\006\023\353\234\264\323\3325\341\023\010o-\313\340\373\360\006\004\036\004\311Kd\3172\021[\321\231\307q\371\363/\327\224\016\226[\020:\024\006\243h7%\345kyR\376\254~,\367jX\000\0218\025\000\342\314\022U\211\256\257\023K\346D\364u,\315\270\345\032\256\242\377~\324H\240\204\324c5\351\313\247\004\214\024\r\252\375\352!\353\316[]@\332\031\246\007?\353\032D\256\232\030\330\2379-S\001:\314\022\034I\3269\037\216\027Q\000\260\264rW(\200q\231\014\203$^\315K]^[\0361k3\265\375,\355Y\013\220\313\365A\013\010a\303\256/B\256<\007\260\rJ\020\326\250\215\364#\313\004W\365\372\266\200\307F\255z\2511\246`hG\326\301\240\306\017i\253\301.\377\347\356f\322\315\363h64x\000T\273\2323\247\326\004X\264\300\034\253\341\305\377\345\220\340\355k\202\235\331`\377\356pu:\344,^\rh\250c\312\330\275\304m\220\364L\355\222j\034f\010T\303}K\324\003\212#\263\260\022R\002\244\272\007\374\027\333\233];\257O\373\234BZP\305\342\273\014_Z\034\254\rC\316\326\314V\332{ \202\215\346\224zC\356E\230R(t""\006\355EA\rjA\245\323\334\230%n\034\242\354\277\005]F\327\3428\301\340\346d(\203\212A\350U\264\212\022\351\t\353\033]j\365\302\203*<\271i(,\177\254\325F\351.\033\361\033\r-\\D\3514\325\222Pu\254\313CtY\216\032\000\251\371\221\204\313#\010UB\020.\235h$\265\225\232Zq\224P|2]}\226|\221\217o\216\252\326\262\233\222\212\374[\223H\003\263f\030w3IQ\324\212\200'\335\212\243\370\035~#M\001\315qJUY\323\257)\2108\0279\331\203Sc\213\233?4b\306\002\237\322\r\251\366\007\212\375*\350\031\235\223\217\032\320N\225\372\377\271\247\264\353\022\366\252\344\363\361&\236B\272\364\rVmx8(\330\364z\226\335J\304\215\t \\\320[\231*\237\013$\206%\031\223{t\250\210;AD\327\013q#]-\266\325\223\225\231\325?\362ur\317H\251D\254\340\271Il\242<\330\301\020\231\246\326\r\270h\312\024\371\242 \342A_A\366hx\320\207HJ\021\335N=a\234E\234\3665\325\330\356h\3758\243\034\230\3421+S\334\262\325\243\027\262\215\246\243\037J\263\354\231,\274\362\364\242\004/xU\237\004\231\027\213\377\004\357\310A\354ER\256\346A\020\211-5x\025n\250f\007*\215\317!\337\035\277\364[\350\375#\336\352\316\333\353\302\241D.\337G`\225\374\216\005y\242\215z\033\361\221\314\r\226|\204\333L\355g[YC\223\256b\204\375\277\000b\326\022\351\023\250\336A]\013\265\227\226\201U{\345A\rD\306m*D\264\025R\205\002\312\227\250a\255\n\324\322QD\024q\244\211\326=W\274A\372|\320H\240\204\035\300.\006[\3077\245TZ*\360\201\234\027\375\340\007a\177n\n\320\005\331\r\353J\202\240\324\013\363U&+\223\036\272\236\334\222\032\017\371\001\301D\352\204N\227\355\277v\310\001T\002\341\200\006j\343\373s\273l\354\2076\262L0\241-H\326\325\231\n\027\336\376\024\223Fs4\227\223\002-\314\235L#\201^:[\344\224\030\255\300\352\243\324\353)h\241\325\342&\005\371\016\376\313Z\033\222\301B\035\022\036\343\277\212\315\374\263B\350%$\367K\231A\376$R\270\361\336tY\267\374\236e\301\340Ka\334\220\267db\304\252TC\241\032\374\212\374\365\033$\003nw\244Q\025\226\200w\031\351\331\330\376H\343\311\317\321\366\003n\246\235\211U\315\243\367\311\375\004\327\215\004m\235\2232\272\275\225""\223\341\217\204ASb\300D\033:\263\245\002Y\304\rZ\nGiT\261\014\375z\255\210}\241\306CC\347\232\"\345\242\300>\223I\310=/\"\256\305e\204\261Xa0Z\210P^\377rn]\"\342\364\\)\205P6\355E\333\004\374q<\021\222\242>\032\030\300T\302\244\177\274\"5\370\300\"\034V\034\371\230\342\220L\016\222\244\315\243\242:\355\223c\344\025\225N\215\267Z\356\232c\357\317x\330\215\354\343S\371oN\324)\250\201\275\261\303;\026\371x\257 \311\333\017\240yO)\303\3466\250\3645\305\321\223\205\010s\301E\357\342ff\004'\260\241\235m \222?'x\030\375:r\001*e\333\252\375\271\235Y\375\r\362%E'\023\273~\033\307\263\022\214\224D\357\266\305\267\326\2061\003\211\017c\304\334\244i.ma%\207\2068\204\201\304\023A\362\276\345\257\205dBY^QK\023\315X\3077\212E\344\033V\377Ld\030\007\006\021\226\353h\267\006\330\213\217sN\353\022.QT\006\346E\235\005@\331)\202\346\313O\346p\006\201\235D#X2\206<\034)\210\312U\236\023\356\361$\203\027\035w\214*S\003\023\223M\252\334\200\"v\346\275\353\234?\230t4\313\334\001\267\334be\017\2536\031\323j\330\344\277\371\3266\\6N\205\343\262Y=\307\337\241\025\006\2277\352\334\244\306r\276IH\274\013:\014l\031\343\231\370\203\271q\263\0064\301!nD\t5\204\343\320\177[{\377\364\250\212\317\215;\017\264\325\010\344\340\253d\030\"SJ\234k=,\335\375\256\327\360\302\217\177\367\263\370A)\363D{d$\326(\246\343\375\212[\326?\233\013Y\376\247<\353(\214Zkh\314\316S\271\004\374\207\204\302\345`[\214s\347DR\215a\035@\227\335\363~\362\353\2340\327\372LB\333\023\227\2173\354\035M\177z9\007\346F\307\231\306X(\333\264X\241\237\2503\206\313\t\036}p\3012\342\207T\207\035\271\257B\275\332';\231\033\211Vp\315\035\232\216\254\030\330\"o^dr\331\207V\225&\307+\001\234sU\304\263\234\262\033\341u\016\323\372[..\360\233\344Te!S\254<fS\352k\250\341w\273\2642\272\234~\000t\010p\337z\326`D\371{X^\2671#x|R\332PW0\331\315\324~\033\265Y\240l\2529\230[6\367\036\375\305B\255N\222\305YS\016\3279\nv\023\246\007u\243\205y:\233c\225y\311\330\3442\034\227$\006\225d\303\334\235\"w\037\321\365J\013Jv\351W\n\327\242\362B%(\213\376""\204:\377\366%\036\353\\\033@!x\257\214\222[r\267\022\236\306\206[L\200\373\204(\033\367\207\374p\232 H\2738d:\177\013U_S\351\375\300\001\252^\254\346%\312\261D\347\3220\037\203\004\245\2449\353\223\262\306j\214\367\357o\021B#\265q\037\366 \303/\304:\334\226\367\363\347U\240\374\013\306\352\311j\247.u8wZ\300\340\366\036Y`wla\212\3659\263~\327\004U\254@J\357H\306\n\007\264\376\233\377\r\014\306N\\{c\365\237\262I\003\014\237Ba(\211\373i\032\260\347\320Tw\034u\326\035V\222\003\007!\251\024M)\177\tAN\324\007\235R*\3750$\3104\214\344(KX\320W\301\203m\3622I\"C0D\266;\207\312s\323;\260D\270a\327\304\205\206\312\261\251\273\351\246:z\2732\343+\374MR\2563\317`xP\\\361\242\364\002\021\030/.;\273\312\263\302/\300'\343\035\017[\347\253\357h\030;>3\177J,\030\352}\217)\007a\314\337\005\223\3462\303\207\004\274\252\014\273\236G\013\251\0212\001_\305sQ\256Q\232\351\267>\202\033[L\336\305Y\250\304\304\005\210\252\366/|\221;\257\373\235\230\273\322\027\346\242\010*5\010\346v\000v\241\362VP\204N\202\243\001\035\002\334$_\232\333\272\303\355h\313b4\321\256Q\007\374\326\024@N$z\352V\373rWh\273\017\214)\366\004\205\220\016\265cD\212\017\342\371Xx\350\033\310|\305\031\250\026h)\253nP\010k\371\256\200\264\"\017(\254\257\321\204\245\372S\231\212\312Hs\023\366\r\007i]Iz\234C\201\270\377\335\235\265\313\033\021CE\324\227\353\245\324rN\\6k\255\016u\227X\332\372[0\213\273S*\274\315Q\227\022\024\315\205q\250\363\202K\"\304\361\017/\246\274\2263p\003\300\032\267\250\226\250\247L\204U\277\242\032\262~\tLf\304\273w\370*\024_\350\212\026\251\262\263\025\265G\237\200\337]\031\272A$\232\027\244\266e\230\367\n)\305}\242S\213~\010\321e6\2104%\207r\032f\010\"'N\314L\024q\335\335*\303\2378b\031C\3538\216\342\273R\2775/\350\306rY\362\024\376I G\266*\265\2320 \271\005\t\306?\201\261[q\nA\006L>\222\377e\360\271H\323\351)\373]\020>\006/l|\255\352\3121p\037C\327\226\314\274\271\261\t\016T\3755\263\3251\300Up9\035\254\365\262\007G\270\277[\275\223\341\251(\241\036\001\371Hq\305\0033\217\321$\362\344\367\3575""\256~k\026\213\211w\201\033l\023-\301%\242a\311\nb\323\210\315\313\344\200\t\242UQ\000\363\000C\237 h;Qq\213Bu#\236N\022\2011r\251\201\006\330-0\252\351\247^\263\355\300\347wE\023\3764K+\343\364Q7";
    PyObject *data = __Pyx_DecompressString(cstring, 6359, 3);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) == 2 /* compression: bz2 (6210 bytes) */
const char* const cstring = "BZh91AY&SY\277q,\326\000\005C\177\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377@@@@@@@@@@@@@\000@\000`\032\217w\276\224\247\335\330\037w\271\356k\327w\035bwl\273Y\273\030\353\215g\247+\263\2737.\306g\024\254\201\252\332\035r\272\323\233v\326\350\353\203(\007\261S\330\326\301@\372,\000\004\241\021\222&M\004\332'\251\344\004d\364\021\202\230\320\320\030\006\221\212{Q\350\244\375(\366\231MCe\006\311\232\247\243)\352=56\243\324\022D&&\200\232Sji\251\350\247\212\037\252?T\364\236$\332\236Q\3456\246\332\240\000\000h\001\240d4\320\000\r\032\000\006\230\202B\232(\364\233SQ\3514\032\0003I\246\200\036\246\206\200\033$= \000\000\000\000\000\000\004\232\221\001M6\2421(=M3)\3524\000i\351\033)\223G\215P4zM\000\3204\000\000hmA\352\032=M44\032\232iF\240\365\000\006\200\006\200h\003#\324\014\200\000\320\032z\206\206\200h\320\000\000\003j\000H\220A2\000\206M4\23212\000I\017\022~\223&\200\321\240\004\032z\230\203\0042\030\010\310=@z4\372\225\326\332i\371\254\315\231\332\231\007\257F]\r\024\221\244g\343d\314\351'\264\230T\342\363\253\317a\320 \222\240\220]\022\320\224\t\000\201+[y\234\\|\302\333\357\215j/\307\310\212\200a\r}\032\233\264\205\030n.\362Ca\021G\375\275R\023\201,A\261!22@\022\325\352H\031\341\224\275^dQ\347\372\032\344j`\375\314q\302\027\017[\326V*\n\210#\033>\027\n\0026\205\365\250\030\254\304\266\242\202\211\371<B=\322v\206{\215\031\022i1\223\031\241\244\233\241&\366\004Y \261@\026H\252\n\021AE\005\202\"\242{\354\346a\231\237\006'\t \306\221DP\241@\216\205x}\003\240\356\234$\273\307\346\377_|\317\22402\345\376\023j\257\020\210\262I\221\221\227)\322\350r\022Nf\nH\0102,\221dY\026\002\204X,Pc\rb0d\340\204\324\340D\217\207\0169\005U\243\033o\215\241\334\217\n/ERV\347\220I\023E\223 \322\362]\014\006ML\311\266z\2068\303'rT\203\024Blmi\330\231\201\302\302I\262\004U\220]b13\220\016\316\026\t\r!n\344\361Y\262^#\034\001\231:\006}h#ToF\241yY\224Y\253\337Dl)\224x\233\251\273e\314n\230\253\267h\262F\030\213\220M\001\304\251)\016UT%'\020\214LV\214Y<\001\2759a6""\000?I@\326\251kpQ\331'L\301\232\314\304\265\241\227\235\200\205a\217\035\326B0e`\252,\032\233\267]*\255\366Xh-\311\202c{\334\323\231\027\344kp\027\241\225\323fj!3\t\250QL\351\371\365\300P\246\342\273\341\013a\004\357\344\202\305\030\230\3577\332$93K69\013\221\2031z\245\375\024\016\371\311\256h\325\213qX\002\310\3425\221y\256\270\227\222\352`\032\205\343\364A\357\273\277\034\221\266\030+\226\r{\276q\0075e*\303\216x\252\027\326\253\247\264\353\233\337SJ\332\275f*_P\322\233\315\347DA\014Q\023\177\333\376\327\237\213\376r\034\361A\375\373\274\253,\017\"r\276tz_\022X(\262\313.\210\251D\005\373\3640\222H\251=\255\277\303\3023~\204?\206\332\311\177\3235\242O\033+/T\322\320\315\336b\007sk\206\227q\036\365\212\273\231!\207n\323\r\r\347\374x\255\tR\276\260\3706L\022\036\016\301~\232&\326\262_Y\347W\223\270\370\372YKC\tte%\n\356\263\022f\224\030\215`r\372O\344!'8\360\235\363\250\335.;\017)\023\255\203h\331\300\255+\025z\254\215\231`\2356i\257.\237z\267\343\220\307\212\361e,\331\207%\264}~\003\300H\353\035.\232\204\013\225\233\022\220\223\376iV\355\252B\036\212\033\260\003/\251\307\002\344&\023|\361\310\363\021e\307\200\326V\272\2076\254_\273\320[\203\"\370G\031>\376\022\201\247\370x\260\\\275v0/\226\372\312\324\376WO\206Hj@\271\306\327^\3440\037\024\3078+\217\004f\t\226\032\002\266\237A\323\223%\034\314\256\274u\371\236\213\030\250\253*\204\367\327\317\315a\031s\2631L\374\356\232(wL\314\3506h\020\014\320\367\273{\220a\366\370X\240\222%\020I\303\022\246\030\205\"g*\002\240e\027\016J>l\351'H\223\\YN\273LW\261\3620\233ZL\365w\236\267\260V\315vq{\321\304\326%\341S3\206\030#\003<:.\265\341\274\347\353c\274\355\234\230\210\342\313\213\307\201\200q\256\244gq\333\320/\324\257\205^7\224\273\364\326\252}\224\250\234z\030\325+\000PB\264\372\334)\316jU\013\304\355\255\276\027`\240\001A\013j\370N'\276\366\206\225\022\363\002\316\357\262\343\325}\200\237\327\014\247\224\222+\245\356\326\350Lg\363\252\005\362\027\325\371\255\201\246\\(X(\347\320N\322\335c\322\031p1\357\337,\222\213{\343~""\2277\351\020#\260\273X\275\326\202\031\030\267\302e\227\331\361\361\332\243jn\325]t\352(F\205;\256\364B\314$X\260$\250>\257\233AE\177~N91\372\347\372\313\351\341\177\001j\346e\327Br\227\010R\027\204<\177\203\017L\245J=)\001T\032\n2u.\207\224\035\323\331\340\356\207WFp\233((\016\355\241\352e\331\274.\360G6\357H\275bYS\251\207\243\333\365UR*\225UO'\341\307\345\334\256\233\317no\357\356\272S\005:*\034u\222$)\022\331\030w~\221\324\"s\0134) \362\331-\220\270\300\2458\214\010!nd[\212\243\337\321\257\215\223\231bI\242%\265<C\230\217-\256\255\311\246UC-\343\360\242\347O\315\251\315\334\362\373\215\003\330|?D\323C\340\202J0{8\341\230\330\3003\222\024\316I\315\311\205)\367II1\320\254L\221J\020\252\224I\272s,Y\001\030\024\202\250,\371\351\016\r\000A/\030\0009S]'\230\014&\265\345E\301\350\366i[\366\277\302r(O\275\234\n\337\245\032\233\rq\335\357\334\306\344\236\353\205\227e\335\375KR\016\234\347s\365\262\036\215\204\020<\335\2711\221(\313\307\333(\251\340\246\234\3072y\027{\230\352\335,\304\310\377\236\261\263\232\331\212?\332\321Q\177\202\226[\341YX\033\003\223\376\263\341\305n\203\310\263S\nf\026\014\262\221\362\303\007\007\305\303\206^s'V\214J\251[ \244\354i\252\240\230sV\345R\2479\347{\177W\252v\226&f\010\300VV\363\366\236\315\312.\3577\250{\031{9\345\3273Y\205\304\214\347\243\216e\032(P\247\363h\330\214cv\376\257\032\032c\301\030\030d\347\367\317\017_\321\312+\213\270i\201\206\335Z\034;>\271\333\312^b\035\234ta\316s\253\261X\322\205\351\331E\205\231\004\245[\004\222\n\361 $\207\t\374u\010t\322A.h\211D\251\217\371_g\031\346&!T\277r\2317\303\204\223\0211\264UV\222P\266\010\336\230n\273{c\323W\014\227Y4\341C5G\006h\0136\375\312\022\366\216\316\303\231s\"IH\362zU)\014\223\361\023 \223\253\016\306 -\005a\324 \032l\002\303&J\005\315\371{\366z{\000\370>\032\352\267~\014\225\037\214`s?\346(V\220 \205\355l\201\n\247J\376\377C.N\305\274\312\3277=\230\006\016\006\017\307\345{5[\324\274\001\300\352\316G\ns\356\335\233E\301J\236\214\205\331$/\025\366\225\202\366U\310{\017q(=\300\317""\256\303\347;\003\3672\2722\342s\246fV\231\022\2461\316+\327n\257B\252\016\372\336-\357b9&R\276_\342\016\317Ar\275\303=C]\3347v\247\2606\013V\303\277\327\324^\267\353\026\260\361-sF\005\332\316CX}\225\2343\254\374\275\237\335\237\312A\311\315\323\241\365m\216\216\242q2q:\350\322\033\r\"\322i6\032\217?\243\300\033n-%\274\220^\246\253\340\214Q\024F\376r\030.i\223\207\255\010Hw\315\006\007\227\217Z+\215\270\271\247\000\222\243\220\306W\325#\244\260\246\326\036K\250X8&&\013\302\213NY$d\361$@F\003\335BHH\222\000A\261\334!\222\370}]\0366\333g\303\361\005\322\016}\032\330I\233\341\371\004\221\277\223~\225\251\034\036Z\334@\344\246\344@v\347\201~HO\350*\260\017BU\317\262\276\372\354\025\2748\311W\020\210\215\2361\020\025\305\"\003\303\271\031F\031\\\\\274\266t\014\027>\206\\\204\214\261{i\030^\365\233]\200\356\2719\"\216G\260\233\315\207\362\274\004w\346OA\034\036\273\266\010\224\371\337:\253\256\223\"\314\320\020)0W7\213\305\325\327!\233\033\265Z*\025d1\253\320\244\330U\251U\341\030\312\243\311f\021\026\334\362\227\317\253\322\217t\263\215v\317|\222\233\033\023\353\352\206\005B\230\n\016\356&\375\223\213\204\373`|U\221\026g\214\333\211)-z\344KT5\030\231\0369I\034\231\222\225F\332Bm\033\364Y\301\275$|\000V\311\243\361\232O\225\351,mgI\006{\375A\350`<\232\206\212 i\\^\034\254\354#&5\21055\020\301\241\360\2757!@\335h\343k\203\303t\025\204\324\350\031\363\343\262W1U\235\323,4\364\306\347\010Ut\302\004\272\367\"\252a\340h\346\003\200\3060\267(M\2442\024\\\036\226\214\322\246:\230\2451\233\202\335\277\225i\200]\2633^\027\276\311{\216\352\016\352\363\021\004\216`\357x<\037\026\224'\352\264\266\214v\265YqA\262K\213J\034\205s\263\266\005\271\311rl\333^\257b\321\301y\267G\243\272\26599\306\251\02656\266\252\t\034a\346\345\226\023\364\257\031\224&M\205\033\013\206\014\341\006\265\260\342\212\350\363\344\362\334i\347\247\tZ`\2265yc\241\217\251\201\342\247W\243\314o\252\024#\355%\300+\007\245\344Dd`)r\363\332\344\221\241\203\313\230l\217\026\356\227D\210+V\034\036\222\233\0270z\031""\273%\242\031\2064\305\311}\264\326\375\266)\201\204m\222\016\033 \245-/\rZ\303@q\302\204\331$\3645=\327\211\035{\322\357\034\361\320d\270c!\253\323\3112\005)\271e\022\312\n\262\231(\005\006\255b\242\336T\033\312,\250\004\220\021H0\204\271]\372r\004\260:\"\371\371\220\310\351\326\007#\302\021\212K\327X\340\r*\353\031P\002\314\2566\340m\031\330\031\222=\r@d\\\277\032\366\274\246R\302\263\313}l\3478*(\312\252\034\244\314\331\n\360\261$\255\020Z\226q\215\346\345\275o\017dv\036{j\235\367f\355\322\362\241\264N\014\013\366\304\363wO\211/\202\207\003\252\307\014vH\264\336W\003\226`\221|\267\2469\306\312P\332\235\317!\341s\325\3170\3320\200F\271\270\232#\001\034\222\251\270l\323J\007\001\004\005\\\251\250\016W\022\3071r$\254\235\t\244\036\270u\003\240\2004I*\242\242\261,$+\034\363E\335\204\270\311sJ\002\265\246B\r\353\306\032\020C7M\372;\365\365C\202:\024WOw\036Z\026\310P\2553\201\035\326\204\347\221\331\032m\005!K)\020S`\215\321b\311\242\222\236D\243\254\325\322\322\010\272\235\326\230\246\200a0\310\210\212W=\256}P\017\212\265\327\345x\320\212t\262\003\200WMI\232\n\326\376\236\264\227Ov\351ee\314\265\224\246\232\027\354 \323\200\306\301\251\020%\026&\313\344\262\324\271*\346\333\000H\014\374\225\2244\0274\nLIj\205\000\036`\315\204\363R\3064\014T\342\307N\262\245G\243,\226\222!\0137e\307\201\036\026z&\215L\036\315C\035\315\024\326\016\010L\235J\002enh\337]Bm\005B.\211ksl\025U\266Y\247f\247\007T\005\216]1\253%\267N\271\036\022\013.\302\207\360O\334\335\325\335\361\362\256V|}\0076(\342\316y\257A\224B\345\264\202+\264\335k\2356\360sg\200@{\3740J\025\2167J\206\225\3405|\214\200,\036W\312\230\206\301\337\035:\235^\\\"\306odg\231\026\351\024 U\024A\330n\227e\263\340\332m\r\275\315u\370\247RDTw\224(\225C\303\2757n\274)\223\313`\263\031f\004\361\254||\250\226h\025\303\241\031L\026S.n\256\313\271\004g\215E\010\017_\220\247=\201\243\241\270\200\206\307)(\t\2653Lp\020Og=\313%\031\271(\352\234\323\036\321\240\245\211\241\212!\001\3144\257\214;\302\032d\340~\014\330I\253H\33530l{6B\t6""\323\244*<\371N[\203:$\377\256\232\345\246\314\246D\201\203\033w\255v\231!+W}\215\2415\205)>~\323\000A\34556d\020\214+\214\201\0269m\013\252U\2056\022\324\317B\350:6\243$\311\241w8\257M$o\251\332d\336nj1\221\272/:\206'f\335\376\006}\215\301\001\206f\332\301\2023h\330\200\333\316on\361\250}\307\306\351J\025\264\322\022\351\265\203\261\272[i\256\303\310\357X\246\247\264\366\020\201\230\032di\004\237\0272\356\005\372B\261{\364V<,m\033\324\214\211`\344\224\212\263>\235g\013\236\344\253j\022\271\240\245\362\234\220\206\204ka9\266\270\\B\220\277\026\031\3036DD\317\250\332\270u\300\"\027\324\254\313\340JL\356\222\223\001\031#\311\345\306#\\!c\313\313\277\013\023C\2522\354X\256\256\026\022\314\240\023\013\001S\006R\250\242\010\260Eb\330j\020PAF\213\324\327\336lr\241\342^\250\343\323NWu\350\227G\200\252\001\025\203;Lb\0317\275G\r\331\340\027\276\030Z\256\226\222\314^b\241\300\332o\255\313j\2012\222U\021L\2320\251\231e@`\300a\014\251S\327\004\000\314\004\027\2065O}\241\216\322Cd\024\010\343E\014\222\254\016\267\240\205\343\010R\340\265xaX\033\3102\256NO%\330\356\213-\240w\371\361\302h\366\366]\202\257\322\200m\267\332n\347\026\301\371Y\237\342+\244P\345\021\227wO\311\351A\300\366\206\332\235Q\3468\373q\013V\341\231\t\026\207\034\221$\275\2360\311\305\252\332X%%\241\n\252@\343\334E\030W9a:M-\007\243y\355'0\027\207\222\323\264\276\204\252G\\\2011\274\353\034\006\211\257=yf9R{Xf&\"\010k\272s\01030\226\315\330\312\373\231\007\205\014hi\003j-\352\252P|\266\362\\c\236\201n\344\306\014\356\313\255\325\343r\345\322\240\362N\2377\205n\201m\375\001\202\240\221DE\024X\314\020\324{<\221&3%\222R\324\341\266u\353\254\265\210\266xe\306\"ai\220<\364\265.\341\203\276z\251\001B\023f\362H\341\031\252\007_\211\370\342 \016\304\016<q\345\016\334\335]1!\331G\310Q\362\\\340Fa\004)0A\016[\204\302\3020\206\302\205\024U\325\325\032\320\236\201\021\2711\036\030\005\003\004\245\210S\312\254\tUE\232d\222\326N\267>\033{\277c\310_0\360\346\241\227yO\273\205\"\003\2416\371n\016~\202\324\010\247Z*w/J\030""\233:<\0138.\251] !\222a\232\345\212f'\0300\250.\366\221\214\352B\013\027Ev\261dTAAc\035\363\260C\256\0002$E\006\330\205\002\303:\203\275|_\206\325\304\246\361\354\274\267;\267-\270\247\352\rX\331$2\031\022\223c'\307\275\360\210\336E\214Uz\307Vq\343\262s{j(\350\312\014\340\256\362\017i\034{\207\2145g\203$\3145\001\313\\\363b0R6\235\267\305Q\315\265\247{L\314t\370\000\230)uk\224\325\014\365\263\315T\013T\373\362\t\3503\031W\207\266\316!\323lC\227\202\302\017\243W\034\360/\272\2243)\266\017\006(\205\310\332\330\312\240M\0332K!\270\243\345=p\372~\376\320\263j\306\2540WM\241\265\013\232\003\263E\307\\7o\204N\225\210!\033\267 U\270\216\314\013r\275\230fB\312R\250\001^L\265\027\221\252\003XY\n\366\307k\224\221\262%)\301\233L\205\353[@\203c\312\271y\316p\300\344\351q,\353\307.\031\0003\"\221\201\005\001\344R\016R2\215\\\301\265\345\341RaA\201\276\345\243\014\306\034\211B\343\311\021\232\232a3-\022\221\034\230\032[Za\343\334(\270eHG\260R0\334\200-\210\027\303\245og\215\367\020\2700R2\010\013\033\321\n\014\260\341\336\010\350a!\214\260\311`.\320\"&1(b\325\002T\004\210DDFH\323\006\263\254\367 l\350e\356\"\313\003\n-\202k\222\001Q\242V\200\235\330\305\342\305\343u\267\356\363\304\212\021\2579y`\330\362\274\214aj\014i,o1`N\220\033\330\215@%\214\352Z\261\275 \210\207M/p\224\350g\321\253\033;\3117o\371\317\026TU\257\030\222\277JZ\032l1zn\211\357\336\200\336\033\327\233\312\275\0058q\374\0077\014\350\312%\250{R\362\330\004\220un\260m>H\276-\032(qc\374\221\305!$b\355qr3\004\252\033\261\217!\271\215e\353@\254KK#\305\312qr\362\234\276\246\236<\301\2604\316:6\265\241\320\017\"M\260%e\264U\017v\322bK1\231fY\263s\234\3722k]\250Z#9\324\302\001\311\366so \237\236\"\352\311\276\025\212\277\013#\204\024\210G\261X\310\0336 \330-w$\272\2632\032 7\rI\212L\204*\323\213\227\007\244g2\034\234w\026\211tL\006\262@Lb\261\306I#\210r\342\204\246\177\033\00664\225\227\202\365\357\354\022.\206\006\275\006\260\341\253\301\350\343J\253\030\240\227-\027\023\r3\014\311\351\311\374\272\026\024\246\262_I\224x""_$\035\374\274\273\362\221\204[\203<\260\266x\325\253\032\364\340z\014\031\270\360\347\245\336\236#\350\244Ub\262,\024PD\007\246\201\250\325\024\n\023\354\327\220\317\342\331\263B\005\244\362\277%\007\337i\201G\232\300\366\0220\224\300\243\352x\267\272E\020\3745\n}\307\t\227\372\262!\"\202Z\275\321y\354\3513/\341\3100l,\034\365\007\030\252J\244[\035yEJ\264\235\324\236\241\324\371\231X\271\013~\n\016\310\220/\350\316\260\277\215\2312Y\225\030POu\241\342\016\306\335\314\361\243\321|\347\236I3$d\003)Q2\270O\237\261>j\212\250\252\361Q8\375\343#{\332(\360\320i\237\235\351\345O\346\357\177U\275\322u\224\220yA\014\325]\217+\375c\301\177\260\201\014\213\006#^\323\341?\2769@#\363v\374\273\031\333\030\031\371\364\030\336\262\360\re\202\025\252y_\267\350\376\253\325\357\362\356v\276 \263\267\340[\371\225q\327\\9\362\230&\206Nuo/\271!\371\317\003\316v\320\310\307?\220\311d\357\234\370\260\255\023\237\265\361\273\255~\311\334o}e0\371%\213\332=\371\362\217K\376\230\342_\n\225Qs\231\310\340\236\316_B\007*\376\337\266\317\254\332Y\177\357\233j\001\t\202S|\234<T\360\244\t\\\327\315\357\372q\2301\363-\335\217\353\363b\3178\271\336\261\234\236rVZO\241\374\004\277\322\303\377\014)Z\331\241\271\321\356\367;\236i\372?\327\344\022\t'\365\025e\266\022\017\316\t+lU\006`\221\217\211\267\362>\236\262\321j\312\245\225kNh=\230\230Y\0057\325\366\222\003\235jQ\377\255\255\326-/\212\313Y_\027\"\332t\032+\005z\272\014\016G\257\000A\0344\030\235h\364*\356\357\365\007o\026d\227\342\240A\362\234\037r2@X[\264^\222p=\0346\337\217\332\313F\327\337e\316\265j\324\250\371\026a\234H\274\023\022\2717\275Hn\035\237\036\351\276x\275\366\270>\351\323O\210\333\344\006\234N\231\236Mm:1\336\363\026\270\030\274\354\331\030-/\276X\277}Imo\244G^e\013\033\337xy\275\316\357;\371\221Q\362\233\221\336\312\231\242\001 \252\363\202\032\356w|\3173\354\n\355\267\234\237[\227H\351w\376q\204a(\272\372>\004\2726\307Xs\244\033k\355\263\275\005\035\030\331&g=\321R2\222x6\272\233Ka\310xTs\325\317>\331\017\003n\272\t\330>Jl)m\311""\371\266\342\371\376L\322\236\004~\352;z\307\204\3166\215\246z\233*(\253aU\177\363\252\032e\216\271\263\260\365\332\032\303\265\324\331\260w\264\273\024uu5\325\347\336\366\327\344w\242H\272]n\242q(\315\311\261\255\306l\355J\370\023r\330_V\272=MR\306\263\263\373\232\215\337\322\363q9\362j\266\263zt\276\034F\233]\3529T\343\342\356jw\274\332\337MU\\\t8\2444\345O/\027p\322\325\234\341t\364\336\232o4\353\367\233\271\335\025\3564\277\027\271\355/Z\230\257I\350K\3425\342f\334\220\202V\341!\207\210a\344\024\30468\033Ze2\340;\261\321\006 BEt?\033fe\225\361\322\317\323r\330\305\333\304(\342n2\374\260\214.\200\272\375|\035|M/V\301\253[\332y0\276\247\017\223L\303\321\301\354\353\344hs\272\306\345Zu=y\225/\257\216\332\324\275\346\320\352\344\326\267\201\016~5\364\377\305\347\330\312\250\250\224\022\034\250E9U9r\214\231\307\340\264\331\177\246\354\373\276\33267\036q\314\267\362N\313\241B\374\351\337Q\307\245\310\275|\235\215\2277\321\356\244\231\306\206\274\323\327P\037\257\276\326\273\372\247r\363\001\303\222\244\250L\242\223@rf\370\326Zq\261)e\032ik\034\342\024O\307\267\223i\267\2362\323\221\266\014\326\\N\004\336\014 \226\t\245\237\031\263k5d\267\216\265yRd\214\"l\232\244\356\206E\231\272\255\310\313\312\221y&\330kAe\n\"\215:\233\261X\026Qg\016U\250\270qB\204\311\312tdX\301$\3250\2159\304\323\247\002f\343\351M\352\\d\263\216\265TQ\267%N\241\334\320BLb\274\244\007+\310\321\250\253$\3260\314\255\245\235\253,\022\273FA\004\3540\214\355x\267\301\007\005\333\230\002V\310\004]ks\0319\"\362\315\254\010L\315M\207\202\304\331\233\3655\225\200\313\321d\367\333m\222v\330^\206\252\356\302\311\246\005\334\353<y\213U3\230L[As\241\265\355\242\t\r\334\245Y=\355\221:\337\305\031'\203\270\303\003\030\336\323;\335;\346C*^\017\201\201^\271\023\2134\220\\\372\025\027qc\004\327\027UC\250|\336WC\233\230\355\301\336\256\320\336H\020K\341\026\033G\245O\247\327\377\027rE8P\220\277q,\326";
    PyObject *data = __Pyx_DecompressString(cstring, 6210, 2);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) != 0 /* compression: zlib (6435 bytes) */
const char* const cstring = "x\332\255ZK{\323\326\326\306\234\000\0168\200IB\022n\225)%\001\022\267\001\n\264\224\236c\222\000\201\334\023H\257G(\266\234\250\261%\307\222\223\270\337\351y\030z\250\241\206\032j\350\241\207\031v\250\241\207\376\t\374\204\357]{K\262d;\234\313\367\361\200\264\265\265\264\367\272\257wmsgF\253\250\206\\\236P\245\242\374lN5\312\3257\212\232K\277\322\016\204|\245P\250\n\362\276\254\032\272 \225eA\327\312\206\234\023\024U0v\360d\224e\251\310(KR\331P\244\302\024\021\0107\365]%wSPta\257\"\251\206\222W\360\315DIR\312\270\037(\306\216\360\376=\221\210R\221\366~\377\376Nz^\325\r\251P\020\324J\261T\025\014M\250\350\262pPV\014Y\314r\376\304-\311\310\356\244\2274C\306\346\222!\314T\215\035M\245]rrA\331\222\313\222!\203[0\245dAOD\252\2602\2672\365\360\311CARsBY\376M\316B\020\275\262\225-H\272.\353\202\226\027\266*J\301 \201\252%YO\013\363y\241\252U\004U\006\253`\243\004\272\360\007\020[\025t\331`\362\217K\252\252\031\222\241h\252\210\317\025u{\\\310A\310\254\241\354\313\364\365\013\251\240\313\351\025\251Z\320\244\034S\355\006vI\257e\226^\316\211\363\263g\005\374Y\223\024\235\324B\013\203\341\245w\033?l\n[RvW\006\313Y\322IY6*e\025\006\020TM\235\322+\331\254\014\246\312\262^)\030BV\313\311\351\263l\245\025\251\014\023Bt\235=N\005\177\330#\247\027\211^\370\026\0264\330,\375\331\200(\357\337\263\215\305\265\271\365\267\013\033\342\335\367\357\331\312\336\336\340o\253\312D\316KJ\001r2\306\322l\005\255D\232\207\n\260*t?\211\tz\222\n\301\372K\340\2124M\337G\004cf\244\025\345\234'B\306\200\365\266*\206\374_\210P\226\016\376\377\304\020\264\262\260\244\251\362\177$\304\244\240\344\205]U;P\371\232k\362\266\242\303\036rn\035b\251\333\023\353\331\035\271(M\350\354&*\271g\353Y\354\213g\\\351qC+M\025\020n\005!\257\034\312\271)]\371]F\334\225\245\252\340E\201\220c~\312\202\021\276\007\327,\225XL\2467wd\360F\214\347d\332L\330\256\300\037\360\rQ\227J\210\014)[\326t\346\302\3606\255\340\307/\211\n\341\005\306E:\375\327I8\232\277[AB0\030\235\317B\021\014\311\305-\350\023\262\0275\335\020\240*""\241\235;f\226\337.m\314\255\211\033\363\213s\353\033\231\305\025\310#\027rR.'\202i\031n\316r\212\300\025\341\253\023\221\234-h\010\006\177\242\250\345*\0052^\036:T\2632gC\250\250\2200_\320\016\002\272\n8\330\222)|\324}\3430\315\362F9\375\234\277fyC\310I\020\003;\024\025]'\355\220L\205JQ\025\362\232\247T\316\243\020\"\367\327\205pS9\245(\253:s\354N\212\254\246\032\0222\010tQ\220%O\031\272T,\025dNZ\326\016\364O\220r]dY2\001EqKQef\245\266B\263\310m\333Z\271\332\026\225\307\332V\025\2412I\2410\351{l\027)\266\373\371\253I\341\376\335\273\017\356\013S\302\364\257\001\005e\374\350\212\264\010[S\230`q3\351[\236\274\323w\002h\2218-\007\336\315\353\001\246\271C\215##kE\210\311TD\231\327\200\362\220\337\213%O\r;\322>\227P'\016\n\262\272m\354\344\024]\332B\020\261/\205q\357\276%\0274u[\247d*!\267\346\231'\300q\220\001a\013\217\2101$\020!\270\333\227\nJ\016\331\001\211\036\201H\246\366\210\261\253.\300\037(N\231\301\237\371v\207\320\262J\233sK\354\310\005\304\003e\330\275\n\26296\016\211\352\305\014\337Y>,!\331S\030D\225\307\323\301\267\202\347R\374\266\235}*lCo0\226\274\355\307[\330J\244\020J_X\013\231(\353\021\314\317\"\263A2\341g\372\260\234\227\020\007\373`\217$\212d\000\366\032^\265O2\037\023\022\353\214\373\371\020\245\2422\205!\241lW\340\341\306\370\247\314\313\025\231\026f5\201\373*\252v\271\2225<\255\350^\365+T\237\262\372\275\356\321g\361\022\205\234\023M\334I\373[\360\235I\375\321\031\n\220\316\2243\301U\211Z\311,\005\033\356\302\301\236\321\245K\33092#\305\215\037\023\200\007e\t\016Oe\3303\216?E\273\346\261U0\241W\213[Z\301\317\023\360[]\202\362\025*\277\235\371\\\310\227\265b\304/\271[\370\037\365\016\326\316U\202\310\365\322]\373+b\227jV$\035r\325\206\215Jd\242O\204@\341\352\306\367\317\303\371\321O=\362\241D\026b\231\007Q\223\223\363\022\341\010Q\004C\225\254,\212B\256\302\300\013\201\r0\270\017X\207\267YEU\014Q\214x\034\257\246P\215\267yPz:\r\362\234\322\340\262\367\366X\252\210\217\372\324\000\226\244\330\266RV\300\247\234G\354\345X\361\244j\345\r<5z_\360\350Q""\364\016G\356\266\326$@\236\261\363\254\024\254+DV\360w^Z^\232\203\361\226\2277&\205\231\267kksK\033\342\253Mq13\363j\236\336`w\177\372\335\"\213X\344\314\340kR\346\357rY\203h:\002\267St\017\203/i\357\244BE^\343D^\376\321\271\377 \314*$\003\317Y\301;\026\026:G\325\272\\\310Oyv\246\010\341 \342@\311\321;C+\303'\237\372))X@\225\030Z%\347\344\330\201g)o\033\260\205\334=\005B\255\260O\351\217t\322\206\037^\356\320\203\264\310\275I\347\205\375\337\257\024\201\261\216!\017S\005\006\201\255'\377\225C@\301\310\213\374\303\220>\217K\350\244\301\264\210\306!\207@\364*\362\026\341\016\025)\327\340-\001v\344\235\207\266E\375\004\313_%%\273K\200\322/5\224.\nX7G\250\213\364\033zAkz\223\273J\356\323Q\260\016\n\275\234\375\222\336|)\"E}\351\275.U\017\251\342\374\203'\025O\206\320\256\220Wm\317\322\226\336L$\360\"U\213\014-r5\210=\370\332\360k8\2650m\325\266k;`\016\224\306\034\261\253\343\t\001\000\302`\025\243\"\261\316\3620[\250\350\344~\354\263\256\244\317\2605\003\232:m\367/\001f'\316\350\306n\023\320\t*>B\201\212\335\235\212\312\340\272 \227\021\232\224X\361\241\244\"Nr~\300\364\350A\333\216\024\332\257\242\026d\017\\w\010\361\037\360\037\335\214CHx\246\254\207+\375\377]MO\217\353\256aPD\205\324\233\017\016v\364\020\272\245~\210\347\t\376\001\203O\336J\022\272\324\022\243.P\363B\344\206<\025\206\375\372S\336]\263:\036\224\354\364bf\355\315\244\320~\346\315\362\312\333\365W+\313+,\246;\337A\262\265\215\271\245Y\n\316Y\346\277\351m\331\020\371.!\316\"\022\000\223\367d?\224%\242]J\230\333\236;\211\31733o\300\206\270\270<\373van\275\353yay\346\215W\222E\357\276\310Jz\364)\035\224\342,\3133\242\330\365^\227\r\302\210m\n\217\240\373\323\366\213c\277a=W\244B\317\314.,\2103\313\213+\013s\033s\342\334;\252io\346\227f\327\273\374\251\273\020\266k\340\214\327f\370x\307\253q\336\215m\270Iz\357U\373\242sa\262\216\325\302\257\272\245o\277\350\222~\026\275\311\013:/\341v\344W\177q\317\266]\353\005\363\335\313yo\374\302\"\372MV\367\013\316S\327<\253Q\335\263,c\207|\315_\027):<\033Y\224f\332""\025\256{\r\356\257\221\t\354\315\355\314M\354g\2209\265Rd\341\3266W\020}l\300\246\304\027\363K\231\205\371\237\020\212?.>_^\020\2272\213s\342\313\271\rq\236\334\345Ef&\362\206\"\234\320\324\n\305\307\254\310\203\030\201\022yfA\275\002`\326YK\274g~\224\362F\256v\224\177Q\\\251\036\342\337\254\2225\304%\371\320X\223\363|\305\347s/\347\227\202\315\37438/\275,\257\264\363L$\343\204\270\211f\033\377\214\t\221\260\236y9\267\276\266\240ew\t\036v\342\373\316\347n\267\352A\321\345`\210\275\225\271\365el=\273\276\2616?\263\301\025\300\257\376\002\374\251{\203`\276{Y\322\031\273\264\327\240c\240\036K\360\351\356\0258\262\361n\317\261\212?\236Ss~\007(\2122+%bh\342\220\372\211\366s\347~\301\213\343v\014E\tG-\376\374V\210\207h\357\351O\312m\316\310\375\275\3179\t\273\316PN\354n\226\275\326\004H*\332\244\264\301\032\037\205\344\r\236\271\270\376c\247\264\376|\267\260\374M\266\315PZ\013\366O\367\250\341=\337\360z\375\211Wh\354\304}\312\265\021\032\322S\344\231B\276{\246\307\306E\251\274\033\231(i\245\350sE\337\351\232\350$\202*\312F\367\014g+\024ZT\217\304\245\365\010D}\273\204$\263\271\304\022Ty\216\360]h\010M\363~V\344\223\3410*K\345\336\263=\n\363qt]f\304\037\357\307\002d(\251\004\013\346$\235Z5e\273\242Utv\306+\351U5\253h\351\254V\006\252CF\323%\303(o\205N1\271\242E:OdN\236\025\351\010/+R\003\233\365\362|V\334\327\224\234X\312J\340\212]r(\032\350?\225\254T\360\217\t\370\027\301,\177\322\r\277\270\204\212\014:\177\372\351CD\234@\324\"=\2023hO\204\250Y\231\326c\276\231\325\nZ\231]DC\023w\200\354\331\361*D\314+\333\310\035U\346o\236\323\371\276\267\rAK\355\023\306\300%\241Mh&\253G\2027\313N\276\351L1'\343\215\302~f\010\rE\326\305\204'J\006\"0G\205\300\277\345\264\254\350\205\273\327\351\360^/g0 (\027KFUV\351\3503GZ\362\243X\246\342#\022\364\343#F\213\322\310\332\006\036\002\304\027\377\241\014\242\373\341\016\300\211\345\001\266\313\376I\241\027v\376\223\177N\240\347\363\217\036\362\203)\206\320E\306\016\037\222\211y\227O\307M\344oyHI<\346u2\234(\346+\360\004v54\255\240\303T\035H""!\004\021\332\371\216L*\262\337\333\370n\036,\350\t\036\332\250!\200\013\"}\317\235\334S\"\277\212R.W\336\001\210c\335\261\242<z\250\320\231\245\037ph\0311\023\312\251\242\242\213\201\307+:?\216 W\323\371\345wz(3\301\310\000\350\201\303\347]\374\210L\024\231@\242w\360F\003C\342~\313\206d\034~z\206g\357\216QYc\226Ud\220\221\226#W\346M4\322U>\314\261 U\321L\202B>\240\313!\024\250\225\213R\001\\\006\t\225)\223\246\021Q\262Zb\032\016\251\231\232i\326PS;\237\306\277\366\003\357\355\371\263\3176{bq\245\2134\016^\210\344\r\355%\017\030\034\343\231H\367\363\"{\261\210T\314\362T\364\351\255\221\177\302f\326\310\007Q\253\3334lfE+u\316 C\177z6\272&\275\351A\334\203r\235\262z7\013\221\351\036_\364\"\017h\251J\006?\352\371g\216\232\216n_.STJ:?\346\343W\004\t\305\022\375\203%J\034c\226r(H\242X*\2633Er\231\022\260eV*)\301\030)vW\257\024\371\223\367\373$\033{\221AC\026\\lTQ\331A\221|\\\331aD\373\006\035p\323\016{\025\251\300=\322\317\023\376]\344\t\241\234\345\347\211\355\303\333\356\022\345O\310\207l\334\321\236t\365%\235\240\252\243A\351\354L\360L?d\311Aa\341\341\3455\321HpZA\326\275\323C\334H\025\241_q\313\220\010\004\364\317\227H;\240\352\306A\211w\214\350\265\327\371\202\264\255\007?\235z\203]\271\312\217$\275\237P\351\000/(\037\336\241\033+\307^hc\344\235Q\206\212\264\330\243`S\342\241\363\271\320\377S`\330\203Qy?w0\r\350\225-\376\233,\377]^\257\224\250b\020\r\256;t\032\007\242\350\331Zpl\024\014\304\350\361\225\247\214\366\361R{$\252%\324W\226O*\004\017*%$\003\031%\246-\002;\224\225B\020\217\037\023\265\347\275\212\305\3742\230\344\211\227\245=\221\205\013\203\203\354\242\367\200\231\275@\344'\201e\200(C\300\261\013C\266\301c\200\032\333\3500\202\023Cp0\212\014y\022]\251.\363\323\331\273\023\355\341\235\023'DI\347\231\235\215`\301\017\261\217\367O\234\272\354\016O:\031g\265\331\177\266\031O\272\311\224=d\257\332R3~\256\366\330\374\334\\m&.\231\267\255S\326\252%5\023\303\346\337\355i;\323L\\\2632\356\365/\235\275\026\377$I\344\217\314\253\326\214\265\327L\0144/^6\013v\25298D""\243\337\354X\363\"(\316\327T+\323\214\3677\023\203\346c+eM7/\\l&>\347\013\016|\2105\373\036\3263\365w\215\351\017\261V\337\271\332\303\332\236y\322\274cM[/\354{\316\232sX\337kb\372~\355-\233|nm\023\237DK\373\023\367!\026.\231\223\226\204\201\277i\344\325]+\363!\363\361\253\023\375\tl\362\273\225\264\356\330_;1zC\013g\232\003Is\314J5\023\027\315\363$\364\240\371\320\035\271\353\244\234\307\365T}\232\177\2314c>Yr\030J\272\010\"\263j\377\305\236n\016]\261&m\311\336k^\0342\347\255_\235\024\r^[9;e?\304\354\320ps\364\206\225\207\314CW\255g\330v\364\252\365\310\036\304\363\265[\270\200v\311\276\215\351\213c\326\220\265J\317o\254=\177\211;\304Cs\370\272\265i/\324c|\207w\330\223\006\356\365\264\263S\227\240\245\213\303\346\2065f\217;W\241O\306\333\0033k%\211\263{\320Y\316\275\373\254\261\312u2b1\271Sn\0376\207;\004*\031\267\207\341\004\201J\342\211\332\022d\215_\250\035\230\222\t\275^6s\0203\007\255L\323J\337\231\007\\U\323Dc\270\203\267I\005\320\325\223\023\375\003.\271\213\305v\344\213O\3007BK\273\211\353\364\366|M\"\235\367[#\256\000O \241^Y\363\366\317\3653$\323\230u\231\371\026\326\234\215\021\237\217\240\375qrV\317\244\314\307\270L\177\201o\005\223\037O3\026\330\026\227\314\233\356\020\261Vq^@\221\314\307oG\310\037\327\367\032\247\033R\363\362\010I\3641\216ok\013\326i\010\007\025\2742_\202\024\2037\346\037\366+\347e\375q#\325\212\337\262g\235\376\372H\343\334\3214\361\037\267\372\355$3!IT3Lr\231\027\261\260\036|\211\261\275{\351\226\375\322\t\263\213-\247\353}\365\371\306*\r\023\340p\322\312\302\303\311L\315\004\302m\234|k\3044\254'\3667N\206\206\377\204\022\006\206\314\005\346\177\001\335\035'\331M\327J\\6\267\334Q\356\316\343\r,K&\315\263\320\360b\025\036:t\031V&\213^\271j=\266S\255D\322\0346\213\334\321B6\271n\375`o25\214\037%C\3426o}U\277\320\330c\306\257\315\233\253\346\256=\346\214\327\223\315\313\320}\363\362\204\023k\323|\327Es\017\352\270<\345\254Bm\243'\372S\340(\376U=\326\362\243\022\356e>\201L\017\260\024\3173$4\315~\003\316\326)t:f_\372\214/Q0\nv\037l""\227\341/\037\303\205<-M\007\001O6\214\321\355,\317\002\347\254\207\344z\027]\366m\206\215\276\2607\240\336\350\\{t\233\"\003F0\207\314\037\221\247~\366c\365'{\3179\355H\216Q\177\322`\354\351,\3052\346^syv\341<\267l/\022n\330\2063\355P\266J\234\350\277\0051\342\202\335\357$\035x\035\264D\236\235dL\306\257\270W\356Sv\342\0210h\202\215Q\3100@\241/\2611\331p\324\032\264\210A\363\224\371\223\245\333)>|g=\340R{\263\255\377\212|\324\352s\257\245\335\364\213#\311\013\272\204\237\025<\236\233\211\033\226\324\3623I\002a\025\343A\305\322\313\262\037Y{t[\264X0\177kn\022\007\254F\270_<i0\253\014X\253\356\r\324\236\200n\240\266A\271\300MM\327\223\236\331\220R\223`\330w\311\026O\\\274\226\235\261\376 \363\223\223\334#/k~1]\377\354(\306=\021\373\203\240b\317;\233HA\243p\274\346\350\030\376~\232\2146\366\310>\336\"\311\2771gL\277\326\004\271\r\342[\206\375\035\254\377\024\031\206R\031\2110g&)g\356\361\247\027f\312{\372x\366D\362R\363\374\205f\362\322\307\205\330q\213\306\311\307~\241\002|\325z\350~Fy\023Uz\223\274\311\336wV\035)\344\021\224\347\007\221=\234x\375\\\343\353\243\030\352s\213O\302\231\234W\3657\215\312Q\006\223\0372\255\343j\033\254\314\212\247\340\005\346M;\005S\237\277\364\357VBzy@\225\233\252\037E\225G~\206*\343uk\203\202\221*`\212\307\004\025\001w\310\253\213\254\302\345\230\227\007\025\356z]B\276\202ON[\263\0043\202R7j\365\273\251\373\365\211F\252\301\330>\307\371a\365\317IA\3028\322c+~\335\372\221qH\202\220\246\236\0202!\rR\226FZ\372\325N5\333\036\313\\\325\3636\373\263z\254>T\377\251\261wt\372H:\332\013|\215\314\267\217|\226\267\336\330\314\250\341=Zq\200\225V\327F\341\365\243\241\340\357v\303\331CQx\335\220\032\373G\253\010\261\360n\0253\203P\334\267W#\273\221\326H\310fP\240.\034\355\361\360\270kg\354\325\360\022\356\205\317\254=\373\264\275\343\3546\222PN\022\256\372\361\3736\032\270kgB\005\373\334\247j\357\231\000cy\305\314\257\275I\000\260\014\320\312#\307\243\275l\352V*p\347V\374\266]DZ|t4\362g\254\025\037p\0078J\202\006~ \373\322\340'\256\221\037\201=R|""\260C\205y\240\366\013K\003\356\000\333\010\367\317\254\275@l\367\311\253?c\355\302\234\n\227\345\263'\372/!\374:\000\0379\3275\312\357N\212\227\307m\002b#\024{W\254[,\210)\367\"\271Re&-\\\274\021\340;\344\025\005\272\214q\333\372%\300\375\342\021<\225\241\270\"\257\366)\370=3G\002\322\377@\232\034d,!\262.D\223\3054G=gO\014_\246tpi\360\343\026\322A\322\275\210:\n\037\356`~\304\035\241\371\313\310\217\2111w\214\201\200$1Hpy\333\371\251\276\337\330<\312\2648\252\245RK\202\301j\377`\360\366\373#\"\207\007\014\336\264\247\234\275\217\277\304\216s\202D\0174\363\240\2265\223!\360=P\33343Q\230s\241\366?\274\344\372X\203\322\307X\007bc\270\347>\354\316\262?\257\211\000T-|M\214\334\266OR\202\270b\335D\305A5\335\257\257\266\350\t\345\177\325\3163d\327\205jx\213\022E{\344t\210\0228[\355W\306\024\363\244\353\210\202\355\372f#\343y\324&w\302\300\327\376\316|\321\363\261s\265\247\314;<\316&\355-\302\037\202}\322\275I\251xH@Pa\252ES@\357\323\316\002\320\345\026R/>\370\003\215\315\347\344<>\203\301\334&\301\265\000\214\301\207\222\264f\334\031`l\021\266\373\334~V\217E!3\352\277\017\317\250\177c>\023?\001\333\303gFF\233\303\004\033\223\304\364\000y2\024\237\017\3316\035\364?^(\367\216\373\207\236\376c\335\263\356\225{\316_\033\251\320*\003\304\267\007\301<C\204`\331\267\350(w\010\362P\237\3633yr\363\322\025X\341\nkq|\235\264\222c\326\210}\336a\275\305#\213\373r\357\277\037\332\215[\004\"\016\231/,$\374\217W\3742u\014\202$5^\267\275\360\037\265\223<e\234\345\260\207\240\330!Rp\254\r\352~\251'\353\251 \222\261\301m\336\026\306\223f\017\331\223\001\220d \021\317\3037 ?\301D\004\340\037\324\016\216\230\373\326:T;*\330\314?\366\315u+\334\007\276\266$\277\035\214\364n\3158%\357\230\333\207\270g\215F\302\357\334\031\004\244\360\216\363\025\t\002]$\027\210^Z\341`\034\030\203\314\324\275\264?\274h\236A\321V\354\003G\n\273%Y\006u\273\025X\274\325\253qF\002\"XO\275\006\3430T\340\030x\273\344^\272I\200\327\376\243\316\234i\232\002~\313\212\271ct,!62\324\252\016\324\326jU+f%\351\224\340l\355""\236\271\312\245\335`\322~\r\376*\250\377}\366k\236\r\216i\345\t\323^e9+\310\214C\346\"\032]\331y\340\374\326\350k -r`q\213\225P\2461B\023\200\025_\263\3026D\322\334F!\206\005\257Y\263\010:\367\376\353?SPG\2137\250\010\367]\240\202U\016yV-\225\220&%\366\030b\361\324y\024\363\317\030\233\271\372\227GCG\253\315\276D\355\2459\r\017\005Pj\366\235\371\260\217\2149\307\014\036g\226\205\rZq\352N\276wF\352\247\352\353`r\376h\363\3179wu\303\335\370\301\375!\357\346\213n\261\354\226+n\245\372\361\304\211\337c\257N\342\366\352\3442\335\226O\256\234\244U\367X\211\177\\7\032\323\215\227GO\377\334v\337\276kE\247$\267\217j\376C\n\222c\242\226\237\332\360\263\212c\364\004_%\220&9{\276\002(\304\271\002Z}p5W\370\n\330\363\351\321\256\273\272\326\354\033t\007\3578\243,\216\300\345?\030$\212\003\345\273\337\254\271k\357\335\367\022\035\370\260O\317|(\327F\031\212E\316S\340\230'\331\001\022-I\361\324\352c\332\0322\203\223\n^}#\0278\367\251\021Sj\365\305k\347\311\303\350\020c\213\314j\237\243&\213's)H`\024\330\270M\021\210\246vf\310\035\344\007eq\302\327\031\364r\377\254w>\320\nY\356Z\t\3269\333\323\336\244\225\354\256}h\262y\261NR\241\210!\365M7\307'\234\263u\364T\024|\347\354\307t\276B\322Wj3\024\000h\364\254;X\263\377|-\007.\237\001\337$;\346\276\263\253u\237\256\305n\337\327\307\032)\222:\201\024\262\313\220A|\314\032\306p\320\235\374\033j\021\253\334R\355\220\343\261A\363;\250\344,\307>{Qf\271\002\206\310+\275\022\272\356\ntd\002\023\235\2420}k\336\002\374\371\247\363\243\373\350\345\321\036q\256\327R\356\271k\314\225\250\257\361B\334+\365\254\371\340\032@\214\201\337\363\265\337\020\346\203\356\265\007\365\351z\206\242\375B\255\014\207\304BUD9>\247^\020\355\273\263A\222\3677=\317\030\341\275\344\022@\000\234\223z\234\223,\353\334k\2546rG)\217\322\000\250X\365<\250\231@\361t/@\376#d\025\240\253SqD\337\272\331\207&\213\347a\306.\323\375kV\343\375\357\006\230'\263\226\005o\017\240\271\003\250\350\264\235u\222\3367T`\231\227\311\224\350\006\271\343\274\245\024\330\270\307\321{\267\327""\220\203\014\370\3431\252:\327\350\340\002\276\355+\353\032\373b\217\240\000UF_\267\327\2555~\320\371\033\325\203\001\362\362\204{\356*\235\233\2727\021L<|~7\223\300>'\255\024/\330\274\266\264[Ih\231@\305\264\375\002\351-\306\327\0367\207\001\264\251Wx\347\334w8\021@\373\036!(\260\306p<\365,\317)h\006\274B\205\274\337>\365\035\232#i[\310\2761s\024\351\261\nK\017SK\n-\265\330Y\356\217\246\004m\277\263\247\351\227{a\202~\321Z\367\177\303\360\376\013\233h\010w'\205\360Q\266\230W\n\005\261\353'\363\377\005k\260\372F";
    PyObject *data = __Pyx_DecompressString(cstring, 6435, 1);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #else /* compression: none (14799 bytes) */
const char* const bytes = ")Counter(name=EntryKind.How fully events are sorted in the stream.How partial-sort \"skid\" is quantified (paired with ``skid_amount``).Install numpy to use write_counter_batch.Note that Cython is deliberately stricter than PEP-484 and rejects subclasses of builtin types. If you need to pass subclasses then set the 'annotation_typing' directive to False.PayloadEntryType.RANGE_ID\n    Raised when an NVTXW backend call returns a non-success result code.\n\n    Parameters\n    ----------\n    result_code : int\n        The ``NVTXW_RESULT_*`` code returned by the failing call.\n    operation : str, optional\n        Name of the backend call that failed.\n\n    Attributes\n    ----------\n    result_code : int\n        The raw ``NVTXW_RESULT_*`` code returned by the failing call.\n    operation : str or None\n        Name of the backend call that failed, if known.\n    RegisteredString(Schema(schema_id=Scope(scope_id=Top-level fixed-size array counter dtypes are not supported.Whether ordering guarantees apply across the whole stream or per scope..?, a counter layouta counter layout may embed at most one EntryKind.COUNTER_TIMESTAMP fieldadd_notean event schemabackend is closedbackend module reference count underflowbackend must be an nvtx.writer.Backendbatch data is missing a column for dtype field batch data must be one-dimensionalbatch data must contain at least one samplebatch rows must contain at least one event cannot combine the EntryKind.category must be a str, bytes, int, or Nonecategory must be in [0, 2**32 - 1]category name must be a str or bytes (code , counter_id=counter is not registered in this stream's domaindata and timestamps must have the same lengthdisabledomain 'domain belongs to a different sessiondomain is no longer valid: its owning session has ended, dtype= dtype for enableevent helpers require a registered stream domainexpected a str or bytes failed:  field  fieldgc; got integer scope must be in the NVTX static scope ID range [interface version no""t supportedinterleaving must be an nvtx.writer.StreamInterleavinginvalid argument' is not registered in this session. Do not construct streams directly; use Session.create_stream(). is not valid for  is not valid in a counter layout (field isenabled, kind=kind must be an nvtx.EventKind or Nonelibrary load failedlibrary not foundlibrary symbol missingmessage is a RegisteredString from a different domainmessage must be a str, bytes, RegisteredString, or Nonemodule must be a loaded backend module; use nvtx.writer.load_backend to create a Backend must contain exactly one no default __reduce__ due to non-trivial __cinit__not supported by this backendordering must be an nvtx.BatchOrderingordering must be an nvtx.writer.StreamOrderingparent must be a PredefinedScope, a Scope, or Noneparent scope is registered in a different domain, path=predefined parent scope must be NONE, ROOT, CURRENT_HW_MACHINE, or CURRENT_VMrange_id must be non-zeroreason must be an nvtx.CounterNoValueReason requires a structured dtype requires fields with self-contained, fixed-width storage; field  requires native byte order; got dtype runtime-resolved scopes are not valid as stream defaultsschema is not registered in this stream's domainscope is not registered in this domainscope must be None, a PredefinedScope, a Scope, or an intscope requires a registered stream domainself._handle cannot be converted to a Python object for picklingsession is already activesession is not activeskid must be an nvtx.writer.StreamSkidsrc/nvtx/_lib/writer.pyxstr | bytesstream is already openstream is not openstring must be a str or bytestime_domain_id must be an nvtx.TimestampType or an int timestamp role with PayloadEntryType.timestamps is mutually exclusive with a counter layout that embeds an EntryKind.COUNTER_TIMESTAMP fieldtimestamps must be one-dimensional(tree fragment)unknown error uses managed storagewrite_counter_batch requires timestamps unless the counter layout embeds an EntryKind.COUNTER_TIMESTAMP fieldwr""ite_counter_sample does not support a counter layout that embeds an EntryKind.COUNTER_TIMESTAMP field; use write_counter_batch insteadwrite_counter_sample expects one sample of dtype write_event_batch accepts only complete-event schemas; pass kind=EventKind.MARK, EventKind.RANGE_PUSHPOP, or EventKind.RANGE_STARTEND to Domain.get_schemawrite_event expects one row of dtype write_event requires an event schema; pass kind= to Domain.get_schema_BACKEND_MODULES_BACKEND_MODULES_LOCKBackend_BackendModule_BackendModule.__reduce_cython___BackendModule.__setstate_cython__Backend.__reduce_cython__Backend.__setstate_cython__Backend.closeBatchOrderingCDLL_COMPLETE_EVENT_KINDSCOUNTER_TIMESTAMPCURRENT_HW_MACHINECURRENT_VMCategoryRegisterCounterCounterBatchWriteCounterNoValueReasonCounterNoValueWriteCounterRegisterCounterWriteCounter.__reduce_cython__Counter.__setstate_cython__DataFrameDomainDomainRegisterDomain.__reduce_cython__Domain.__setstate_cython__Domain._register_categoryDomain._register_counterDomain._register_scopeDomain._register_stringDomain.get_category_idDomain.get_counterDomain.get_registered_stringDomain.get_schemaDomain.get_scopeEVENT_COUNTEntryKindEnumEventBatchWriteEventKindEventWrite_FINALIZE_SYMBOL_NAME_GET_INTERFACE_SYMBOL_NAMEMARKNONEPACKED_RANGE_ENDPACKED_RANGE_STARTPathPayloadEntryTypePayloadSchemaKeyPredefinedScope__Pyx_PyDict_NextRefRANGE_BEGINRANGE_ENDRANGE_IDRANGE_POPRANGE_PUSHRANGE_PUSHPOPRANGE_STARTRANGE_STARTEND_RESULT_MESSAGESRLockROOTRegisteredStringRegisteredString.__reduce_cython__RegisteredString.__setstate_cython__SCOPESORTEDSTRICTSchemaSchemaRegisterSchema.__reduce_cython__Schema.__setstate_cython__ScopeScopeRegisterScope.__reduce_cython__Scope.__setstate_cython__SessionSessionBeginSessionEndSession.__enter__Session.__exit__Session.__reduce_cython__Session.__setstate_cython__Session._register_domainSession.beginSession.create_streamSession.endSession.get_domainStreamStreamCloseStreamInterleavingStreamOpenStreamOrderingStreamSkidStream.__enter__""Stream.__exit__Stream.__reduce_cython__Stream.__setstate_cython__Stream.closeStream.openStream.write_counter_batchStream.write_counter_sampleStream.write_counter_sample_no_valueStream.write_endStream.write_eventStream.write_event_batchStream.write_markStream.write_popStream.write_pushStream.write_pushpopStream.write_startStream.write_startendStringRegisterTIME_NSTimestampTypeUNKNOWNWriterErrorWriterError.__init___WriterSchemaRegistrar_WriterSchemaRegistrar.__reduce_cython___WriterSchemaRegistrar.__setstate_cython_____annotate__appendascontiguousarrayasyncio.coroutinesattrbackendbatch_batch_rowsbeginc_namec_pathc_stringc_void_pcachecached_canonical_backend_pathcanonical_pathcastcategorycategory_id__class_getitem__cline_in_tracebackclosecolorcolor_to_hexcolumnconfigcopycountcountercounter_groupcounter_idcounter_semanticscreate_streamctypesdatadescriptiondescription_bytesdescription_ptr__dict___dict__doc__domaindomain_handledtdtypeemptyencodedend__enter__entry_kindentry_typeenumerrorevent_dataeventsexc__exit__expanduserexpected_countexpected_range_idsff64failedfield_dtypefield_namefieldsfromiterfsdecodefspath__func__functoolsgetget_category_idget_counterget_domain_get_numpy_dtype_schemaget_registered_stringget_schemaget_scope__getstate__handlehandle_addrhasobjectii64iface__init__int64interleaving_is_coroutineisnativeitemsitemsizeitertoolskindlibload_backendloaded__main__message__metaclass__metadatamodule__module____mro_entries__name__name__name_bytesnamesnbytesndarrayndim__new__next_normalize_counter_dtypenormcasenpnumpynumpy_dtypenvtxnvtx._lib.libnvtx._lib.writernvtx._metadatanvtx.colors_nvtx_metadata_from_dtypenvtxwEventSchemasRegisternvtxwMarkWritenvtxwMarkWriteUtf8nvtxwRangeEndWritenvtxwRangePopWritenvtxwRangePushPopWritenvtxwRangePushPopWriteUtf8nvtxwRangePushWritenvtxwRangePushWriteUtf8nvtxwRangeStartEndWritenvtxwRangeStartEndWriteUtf8nvtxwRangeStartWritenvtxwRangeStartWriteUtf8openoperationorderingosownerpandasparentparent_idpathpathlibpayloadpdpop__prepare____p""yx_capi____pyx_checksum__pyx_result__pyx_state__pyx_type__pyx_unpickle__WriterSchemaRegistrar__pyx_vtable____qualname__range_idrange_id_fieldsrcreason__reduce____reduce_cython____reduce_ex___register_category_register_counter_register_domain_register_scope_register_string_release_backend_modulerequired_rolesresolverestyperesult_coderidrolerole_fieldsrowrowssamplesschemaschema_flagsschema_idschema_keyscopescope_idselfsemanticssession__set_name__setdefault__setstate____setstate_cython__sizeskidskid_amountstartstatestreamstringsubdtypesuccesssuper__test__threadingtime_domain_idtimestamptimestamp_fieldtimestamp_fieldstimestampstimestamps_npto_numpyuattrupdateuse_setstate_validate_counter_layout_validate_event_dtype_validate_native_byte_ordervaluevalueswrite_counter_batchwrite_counter_samplewrite_counter_sample_no_valuewrite_endwrite_eventwrite_event_batchwrite_markwrite_popwrite_pushwrite_pushpopwrite_startwrite_startendwriterPyObject *(PyObject *)\000\000_as_bytes\000_as_str\200\001\3602\000\005\026\320\025,\250A\250Q\330\t\n\330\010\021\320\021!\240\024\240Q\240a\330\010\013\2107\220#\220Q\330\014\022\220&\230\005\230Q\230a\330\014\025\220^\2401\240A\330\014\034\230A\320\035/\250q\340\010\021\320\021!\240\021\330\010\013\2106\220\033\230C\230q\330\014\r\330\020\026\220l\240!\330\023\024\330\020\026\220j\240\001\330\020\021\330\010\016\210n\230A\330\010\t\330\014\023\2207\230!\2301\330\017\020\330\014#\2401\240A\330\014\r\200\001\330\0044\260A\260V\2701\200\001\340\004\013\2104\210q\220\002\220)\2301\230F\240+\250R\250x\260q\330\004\013\2102\210U\220)\2301\230B\230g\240Q\240a\200\001\340\004!\240\021\330\t\n\330\010\013\2106\220\033\230C\230q\330\014\022\220,\230a\230q\330\010\016\210n\230A\330\010\013\2106\220\033\230C\230q\330\014\022\220*\230A\200A\3600\000\t\014\2104\210z\230\021\230)\2405\250\001\330\014\022\220)\2301\230A\330\r\021\220\031\230!\330\014\020\220\016\230a\330\014\023\2204\320\027*\250!\2507\260!\2601\200A\3600\000\t\021\220\001\330\r\021\220\031""\230!\330\021\025\220Q\330\020\023\2204\220y\240\003\2401\330\024\032\230,\240a\240q\330\020\024\220I\230]\250!\330\020\024\220J\230d\240!\2404\240q\330\024\025\330\030\036\230f\240A\330\024\033\230=\250\001\330\030\033\2306\240\023\240A\330\034$\240A\330\020\024\220N\240&\250\001\330\020\031\230\024\230Q\330\020\024\220K\230q\330\020\024\220J\230d\240)\2507\260!\330\025\035\230W\240L\260\001\330\020\024\220I\230V\2401\330\020\024\220I\320\035.\250h\260a\260q\330\020\025\220T\230\031\240'\250\033\260A\260Q\330\020\023\2203\220c\230\021\330\024\032\230+\240Q\240d\320*=\270Q\330\010\013\2106\220\027\230\001\330\014\022\220!\320\004$\240A\3602\000\t\014\2104\210z\230\021\230'\240\025\240a\330\014\022\220)\2301\230A\330\010\014\210N\230!\330\010\017\210w\220a\220q\330\010\026\220d\230,\240d\250!\2501\330\010\013\210<\220w\230a\330\014\023\2201\330\010\017\210t\320\023&\240a\240q\200A\3608\000\t\r\320\014\034\230A\230Q\330\010\013\2104\210z\230\021\230(\240!\330\014\022\220)\2301\230A\330\010\014\320\014\035\230Q\330\010\016\210a\330\014\020\220\t\230\027\320 4\260A\330\020\024\220H\230I\240[\260\007\260q\330\020\031\230\026\230q\330\014\r\200A\360D\001\000\t\014\2106\220\031\230'\240\024\240Q\330\014\022\220*\230A\330\020\021\330\010\013\2106\220\027\230\003\2301\330\014\022\220*\230A\330\020\021\360\006\000\t\r\320\014\035\230Q\330\010\022\220\"\320\024&\240a\240u\250F\260!\330\010\013\2107\220&\230\003\2301\330\014\022\220*\230A\330\0207\260q\270\006\270a\330\026\027\220w\230a\360\010\000\t\r\210L\230\006\230a\330\010\014\210H\220G\2301\330\010\014\210K\220}\240H\250G\2607\270!\340\010$\240D\250\t\260\027\270\013\3001\330\014\020\220\010\230\t\240\021\240&\250\001\330\010\016\210a\210t\2201\200A\360F\001\000\t\r\320\014\034\230A\230Q\330\010\014\320\014\035\230Q\330\010\013\2107\320\022$\240G\2501\330\014\022\220*\230A\330\020\021\360\010\000\t1\260\004\260I\270Q\360\010\000\t\014\2107\220,\230c\240\021\330\014\022\220!\330\014\021\320\021'\240q\330\020\027\220t\2308""\2409\250A\330\020\027\220~\240Q\330\r\024\220L\240\003\2401\330\014\022\220!\330\014\021\320\021)\250\021\330\020\027\220t\2308\2409\250A\330\020\027\220~\240Q\340\014\026\220b\320\030*\250!\2507\260'\270\021\330\014\017\210w\220f\230C\230q\330\020\026\220j\240\001\330\024\025\330\024\026\220a\220w\230a\330\032\033\2307\240!\340\014\021\220\025\220m\2401\330\020\024\220H\230I\240[\260\007\260q\330\020\035\230X\240W\250G\2607\270'\300\021\330\010\016\210a\210t\2201\200A\330$0\260\017\270q\3608\000\t\r\210I\220Q\220k\240\031\250'\260\021\330\026&\240a\330\026(\250\001\200A\330$0\260\017\270q\360<\000\t\r\210I\220Q\220k\240\031\250'\260\021\330\026+\2501\330\026-\250Q\200A\360\030\000\t!\240\001\330\0100\260\001\340\r\021\220\031\230!\330\014\017\210t\2208\2308\2403\240a\330\020\026\220l\240!\2401\330\014\017\210t\2209\230I\240S\250\001\330\020\026\220l\240!\2401\330\014\017\210t\2209\230G\2401\330\020\024\220H\230N\250!\330\020 \240\004\240H\250A\330\014\017\210t\2207\230'\240\021\330\020\027\220t\2301\330\014\020\220\016\230a\330\014\020\220\010\230\001\330\014\020\220\n\230!\330\014\020\220\013\2304\230q\330\014\020\320\020 \240\004\240A\330\014\020\320\020%\240T\250\021\330\014\020\320\020 \240\004\240A\330\014\020\320\020 \240\004\240A\330\014\020\320\020&\240d\250!\330\014\021\220\024\220Y\230g\240[\260\001\330\020\024\220I\230Z\240q\250\006\250a\250t\2608\2701\330\014\017\210s\220#\220Q\330\020\024\220H\230J\240a\330\020\026\220k\240\021\240$\240a\330\014\020\220\t\230\036\240t\2501\250A\200A\360\014\000\t$\2401\330\010 \240\t\250\021\250!\340\010+\2501\360\006\000\t\021\220\n\230!\330\010\032\320\0322\260!\2601\330\010\013\2106\220\023\220A\330\014\030\230\001\330\r\023\2203\220a\330\014\030\230\001\340\014\021\220\030\230\023\230A\330\014\020\220\005\220Z\230s\240!\330\014\020\220\005\220V\2303\230a\330\014\020\220\005\220Z\230s\240!\340\014\030\230\001\340\014\021\220\030\230\023\230A\330\014\020\220\005\220Z\230s\240!\330\014\020\220\005\220V\2303\230a\330""\014\020\220\005\220Z\230s\240!\340\014\030\230\001\340\014\030\230\004\320\034.\320.F\300a\330\020\021\360\006\000\t\014\210<\220w\230a\330\014 \240\t\250\021\250!\330\014\036\230a\340\010\014\210N\230!\330\010\014\210L\230\001\330\010\014\210H\220A\330\010\014\210O\2301\330\010\014\210K\220q\330\010\014\210M\230\021\330\010\013\210:\220W\230A\330\014#\2401\240A\320%8\270\001\330\014\020\220\r\230Q\320\036/\250q\330\010\014\210M\230\021\330\010\r\210T\220\031\230'\320!1\260\021\330\014\020\220\n\230!\2306\240\021\240!\330\010\016\210a\210t\2201\340\010\017\210w\220a\330\014\022\220&\230\007\230}\250A\330\014\026\220l\240+\250Q\200A\330%1\260\037\300\001\360<\000\t\r\210K\220q\230\007\230u\240I\250W\260A\330\0300\260\001\330\030\031\330\030\031\200A\330%1\260\037\300\001\360<\000\t\r\210K\220q\230\007\230u\240I\250W\260A\330\0301\260\021\330\030\031\330\030\031\200A\360$\000\t\014\2109\220C\220q\330\014\022\220*\230A\230Q\330\010\016\210a\330\014\036\230a\230t\240<\250t\260;\270a\330\014\r\200A\330\010\014\210E\220\021\330\010\017\210q\200A\330\010\014\210F\220!\330\010\017\210q\200A\360\n\000\021\022\330\016\017\330\021\022\360L\001\000\t\014\2109\220C\220q\330\014\022\220*\230A\230Q\330\010\016\210a\330\010\021\220\024\220\\\240\021\330\010\033\2304\320\0374\260A\330\014\025\220W\230J\240a\240v\250Q\250a\330\010\013\2106\220\023\220A\330\014\022\220!\320\023'\240q\250\010\260\013\2705\300\001\330\023\024\340\014\022\220!\320\023+\2501\250H\260K\270u\300A\330\023\024\200A\340\r\021\220\031\230!\330\021\025\220Q\330\020\023\2204\220y\240\003\2401\330\024\030\230\004\230A\200A\360 \000\t!\240\001\330\010\"\240!\360\006\000\016\022\220\031\230!\330\021\025\220Q\330\020\023\2204\220y\240\003\2401\330\024\032\230,\240a\240q\330\020\024\220I\230]\250!\330\020\023\2204\220w\230g\240Q\330\024\033\2304\230q\330\020\023\2204\220y\240\007\240q\330\024\035\230T\240\021\330\020\024\220N\240!\330\020\024\220H\230A\330\020\024\320\024$\240A\330\020\024\220I\320\035.\250d\260!""\2601\330\020\025\220T\230\031\240'\250\035\260a\330\024\025\220V\2301\230D\240\001\330\020\023\2203\220c\230\021\330\024\030\230\t\320!2\260(\270!\2701\330\024\030\230\013\2401\330\024\032\230+\240Q\240d\250!\200A\340\010)\250\021\340\010\035\230Y\240a\240q\330\010\"\240!\330\010\013\2108\2207\230!\330\014\025\220Q\330\010\014\210I\220]\240!\330\010\014\210N\230!\330\010\014\210H\220A\330\010\r\210T\220\031\230'\240\037\260\001\260\024\260Z\270q\300\006\300a\300q\330\010\016\210a\210t\2201\330\010\017\210v\220Q\220f\230K\240q\200A\330\010\035\230Y\240a\240q\330\010\"\240!\340\010!\240\021\340\010\013\2108\2207\230!\330\014\025\220Q\330\010\014\210N\230!\330\010\014\210H\220A\330\010\014\210O\2301\330\010\014\210K\220q\330\010\r\210T\220\031\230'\240\036\250q\260\004\260J\270a\270v\300Q\300a\330\010\016\210a\210t\2201\330\010\017\210u\220A\220Z\230v\240Q\200A\330\010\035\230Y\240a\240q\330\010$\240A\330\010)\250\021\330\010$\240D\250\t\260\027\270\017\300q\330\014\020\220\n\230*\240A\240Q\330\010\016\210a\210t\2201\330\010\017\320\017\037\230q\240\006\240h\250k\270\021\200A\340\021\036\230a\360>\000\t\014\2104\210z\230\021\230*\240A\330\014\022\220)\2301\230A\330\010\013\2106\220\031\230'\240\024\240Q\330\014\022\220*\230A\330\020\021\330\010\013\2106\220\027\230\007\230q\330\014\022\220*\230A\330\020\021\360\010\000\t\r\320\014\035\230Q\330\010\021\220\033\230A\230V\2406\250\021\330\010\013\2106\220\026\220s\230!\330\014\022\220*\230A\230Q\340\010&\240m\2608\2706\300\027\310\001\340\010\r\320\r\036\230f\240A\330\010\r\210X\220V\2301\330\010\r\210Z\220q\330\010\r\210Y\220d\230!\330\010\r\210Y\220h\230a\330\010\r\210\\\230\021\330\010\r\320\r\035\230Q\330\010\r\320\r\037\230q\330\010$\240D\250\t\260\027\3208H\310\001\330\014\020\220\010\230\t\240\021\240!\330\010\016\210a\210t\2201\200A\360\n\000\t\022\220\021\330\r\021\220\031\230!\330\014\020\220\016\230a\330\014\025\220T\230\034\240T\250\021\250!\330\014\017\210w\220g\230Q\330\020\027\220q\330\014\032\230$\230a""\230t\2401\360\006\000\r\023\220,\230c\240\024\240Q\330\020\036\230d\240!\2404\240q\330\014\026\220i\230q\240\001\330\014\025\220Q\330\014\021\220\024\220Y\230g\320%6\260a\330\020\024\220J\230m\2501\330\014\022\220!\2204\220q\330\014\020\220\014\230A\230X\240Q\330\014\023\2201\200A\360 \000\t\017\210a\330\014\036\230a\230t\240<\250t\2601\330\014\r\200A\360\n\000\025\026\330\016\017\330\022\023\360b\001\000\t\021\320\020(\250\001\250\021\340\r\021\220\031\230!\330\014\020\220\016\230a\330\014\027\320\027(\250\001\250\026\250q\330\014\031\320\031)\250\021\330\020\021\330\020\036\230g\240Q\240g\250Z\260v\270W\300A\340\014\023\2204\320\027+\2501\330\020\027\220q\230\007\230|\2507\260!\260>\300\021\330\020\021\200A\340\023\"\240-\250q\360\\\001\000\t\014\2104\210z\230\021\230*\240A\330\014\022\220)\2301\230A\330\010\014\320\014\034\230A\230Q\330\010\014\320\014\035\230Q\330\010\013\2103\210c\220\021\330\014\022\220,\230a\230q\330\010\r\210W\220A\330\010\013\2107\320\022$\240G\2501\330\014\017\210{\230'\240\021\330\020\026\220j\240\001\330\024\025\360\006\000\016\031\230\003\2301\330\014\022\220*\230A\330\020\021\360\010\000\t\014\2102\210X\220W\230A\330\014\026\220k\240\021\240&\250\001\340\014\017\210z\230\021\230&\240\002\240!\330\020\032\230\"\320\034.\250a\250v\260Q\340\020\032\230\"\230I\240Q\240f\250F\260!\330\014\017\210w\220f\230C\230q\330\020\026\220j\240\001\240\021\330\010\013\2107\220&\230\003\2301\330\014\022\220*\230A\230Q\340\010$\240A\340\010\r\210]\230'\240\021\330\010\r\210\\\230\035\240h\250g\260W\270A\330\010\r\320\r\035\230W\240A\330\010\r\210Y\220h\230a\330\010\r\210^\2301\330\010\r\320\r\037\230q\330\010\013\210;\220g\230Q\330\014\017\210z\230\021\230,\240b\250\001\330\020 \240\002\320\"4\260A\330\024 \240\006\240b\250\001\340\020 \240\002\240)\2501\250L\270\006\270b\300\001\330\014\017\210}\230F\240#\240Q\330\020\026\220j\240\001\240\021\330\014\017\210}\230F\240#\240W\250A\330\020\026\220j\240\001\330\024\025\330\014\021\220\021\330\020 \240\010""\250\r\260W\270A\330\014\021\320\021#\240=\260\001\330\010\016\210a\330\014\020\220\t\230\027\320 2\260!\330\020\024\220H\230I\240Q\240a\330\014\r\200A\360\010\000\020\021\330\016\017\330\027\030\330\025'\240q\330\021\037\230q\330\r\027\220q\330\024\025\360f\001\000\t\014\2104\210z\230\021\230.\250\001\330\014\022\220)\2301\330\020\021\330\010\013\2104\210z\230\021\230*\240A\330\014\022\220)\2301\230A\330\010\013\2104\210z\230\021\230&\240\001\330\014\022\220)\2301\230A\330\010\013\2104\210z\230\021\320\032+\250?\270!\330\014\022\220)\2301\330\020\021\330\r\021\220\021\330\014\017\210t\2209\230C\230q\330\020\026\220l\240!\2401\330\014\017\210t\220:\230Q\230h\240a\330\020\031\230\024\230[\250\001\250\021\330\022\032\230'\240\032\2507\260!\330\020\026\220j\240\001\240\021\340\021\031\230\027\240\016\250a\330\014\023\2206\230\021\330\020\021\330\020\021\330\020\021\330\020\021\330\020\021\330\020\021\330\020\021\330\020\021\330\020\021\200A\330\r\021\220\031\230!\330\014\017\210t\2208\2308\2403\240a\330\020\024\220F\230!\200A\360\032\000\016\022\220\031\230!\330\014\017\210t\2208\2308\2403\240a\330\020\026\220l\240!\2401\330\014\020\220\t\230\035\240a\330\014\025\220T\230\030\240\021\330\014\020\220\010\230\n\240!\330\014\020\220\t\230\036\240x\250q\260\001\330\014\021\220\024\220Y\230g\240\\\260\021\260!\330\014\022\220!\2204\220q\200A\360&\000\t\021\220\001\330\010\021\220\021\330\r\021\220\021\330\014\017\210t\2209\230C\230q\330\020\021\330\014\020\220\013\2304\230q\240\004\240A\330\020\021\330\025\036\230h\240d\250!\330\020\027\220}\240A\330\024\027\220v\230S\240\001\330\030 \240\001\330\014\017\210v\220S\230\001\330\020\031\230\024\230Q\330\020\024\220J\230a\330\020\024\220K\230q\330\010\013\2106\220\027\230\001\330\014\022\220!\330\010\037\230q\240\001\320\004(\250\001\360F\001\000\t\014\2107\220#\220Q\330\014\030\230\001\330\r\027\220q\230\010\240\001\330\014\017\210w\220a\330\020\037\230q\330\020\037\230q\330\020\037\230q\330\020\037\230q\340\020\026\220j\240""\001\330\024\025\360\006\000\r\031\230\n\240&\250\001\330\r\027\220q\230\010\240\001\330\014\020\220\007\220w\230i\240w\250a\330\020\026\220j\240\001\330\024\025\330\014\031\230\027\240\007\240q\340\014\022\220)\2301\330\020\021\340\r\021\220\031\230!\330\014\020\220\016\230a\330\014\023\2204\320\027)\250\021\250'\260\021\260'\270\021\320\004(\250\001\330\010\014\210O\2301\330\010\014\210M\230\021\330\010\022\320\022\"\240$\240a\240}\260A\330\010\013\2101\330\014\026\220b\230\001\320\031,\250A\250_\270A\270Q\330\010\r\210R\210y\230\001\230\021\200\001\330\004\n\210+\220Q\320\004(\250\001\360T\001\000\t\014\2105\220\007\220u\230D\240\004\240J\250a\250v\260Q\330\014\022\220)\2301\230A\330\r\021\220\031\230!\330\014\020\220\016\230a\340\014\021\220\033\230A\230Q\330\014\031\320\031)\250\021\330\020\024\220M\240\025\240e\2503\250j\270\004\270A\340\014\025\220T\230\031\240$\240a\240q\330\014\017\210w\220c\230\021\330\020\023\2205\230\007\230q\330\024)\250\021\250&\260\004\260A\330\020\034\230D\320 2\3202J\310!\330\024\025\340\020\031\230\026\230q\240\006\240k\260\024\260Q\330\020\024\220I\230Q\230n\250A\330\014\023\2201\200\001\360\010\000\005\016\210T\220\037\240\004\240J\250d\260/\300\024\300Q\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220E\230\024\230Q\330\010\022\220!\330\010\027\220q\340\010\027\220t\230>\250\027\260\005\260S\270\004\270I\300W\310E\320QT\320TX\320Xf\320fm\320mr\320ru\320uy\360\000\000z\001H\002\360\000\000H\002O\002\360\000\000O\002P\002\330\004\007\200q\330\010\017\320\0177\260t\2701\270G\300;\310g\320UV\340\010\017\320\0177\260t\2701\270G\300;\310a\320\004\036\230a\3604\000\016\022\220\021\330\014\017\210t\2209\230C\230q\330\020\026\220l\240!\2401\330\014\023\2207\230!\2306\240\023\240A\330\014\025\220T\230\031\240$\240a\240q\330\014\017\210w\220c\230\021\330\020\031\230\024\320\035.\250a\250q\330\020\024\220I\230Q\230h\240a\330\014\023\2201\200\001\340\004\037\230q\320 0\260\013\270;\300k\320QR\330\004\023\320\023)\250\030\260""\021\260!\330\004\007\200|\2207\230!\330\0108\270\001\3209R\320R`\320`a\330\004\013\2101\200\001\340\004\007\200r\210\030\220\023\220A\330\010\016\210i\220q\230\002\230!\2301\330\004\037\230q\240\004\240A\340\004\022\220!\330\010\024\220A\330\010\014\210N\230!\330\014\025\220Q\330\014\025\220Q\330\014\025\220Q\330\014\025\220Q\360\006\000\005\027\220a\340\004\010\210\016\220b\230\001\330\010\026\220b\230\007\230q\240\013\2501\250A\330\010\013\210;\220a\330\014\022\220)\2301\330\020\022\220!\2201\330\020-\250Q\250a\360\006\000\t\024\320\023,\250A\250Q\330\010\025\220W\230A\230Z\240~\260Q\330\010\025\220W\230A\230Z\240~\260Q\330\010\013\210;\220c\320\031)\250\021\330\014\033\2307\240!\2401\330\010\013\210;\220c\230\021\330\014\017\210{\230'\240\021\330\020\026\220j\240\001\330\024\026\220a\320\027(\250\001\250\021\330\024 \240\001\240\032\2501\330'(\250\n\260!\340\014\027\220q\230\013\2407\250!\2501\340\004\007\200u\210C\210y\230\001\330\010\032\230)\2401\330\t\016\210d\220)\230=\250\t\260\021\330\010\032\230)\2401\330\t\016\210d\220)\230<\240y\260\001\330\010\032\230)\2401\340\010\032\230)\240>\260\031\270!\340\004\010\210\014\220J\230k\250\026\250q\330\010\031\230\025\230k\250\023\320,@\300\001\330\010\013\2103\210a\210x\220s\230!\330\014\023\220<\230q\240\n\250!\330\014\017\210q\330\020\026\220j\240\001\330\024\026\220a\320\027,\250A\250Q\330\024\027\220q\340\014\022\220*\230A\230S\320 7\260q\270\001\340\004\005\330\010\r\210U\220$\220i\230~\250Y\3206G\300q\340\004\007\200s\210!\320\013\034\230C\230q\330\010\017\210q\330\010\013\2101\330\014\022\220*\230A\330\020\022\220!\320\023(\250\001\250\021\330\020\023\2201\340\010\016\210j\230\001\230\023\320\0343\2601\260A\200\001\330\004\017\210r\220\021\330\004\007\200y\220\007\220q\330\010#\2401\240H\250A\250T\260\021\330\010\t\330\004\007\200r\210\030\220\027\230\001\330\010\014\210N\230\"\230A\330\014'\240q\250\002\250'\260\021\260+\270Q\270d\300!\330\010\t\330\004\007\200t\2102\210Q\330\010\016\210i\220q\330\014\016""\210a\320\017@\300\001\300\021\200\001\360\n\000\005\010\200v\210S\220\004\220C\220v\230S\240\001\330\010\017\210q\330\004\007\200u\210J\220g\230Q\330\010\016\210i\220q\330\014\r\340\004\037\230q\240\007\240q\330\004\007\200w\210a\210w\220j\240\006\240c\250\021\330\010\017\210q\330\004\027\220q\330\004\010\210\016\220e\2301\330\010\023\320\023,\250A\250U\260'\270\021\270+\300Q\300a\330\010\025\220W\230A\230Z\240~\260Q\330\010\013\210;\220c\230\021\330\014\r\330\010\013\210;\220c\230\031\240!\330\014\034\230G\2401\240A\340\014\022\220*\230A\330\020\034\230A\230Z\240q\330\020 \240\001\240\021\340\004\007\200s\210!\320\013\035\230R\230q\330\010\016\210j\230\001\330\014\r\360\006\000\005\014\320\013\033\2301\230F\320\"8\270\001\200\001\340\004\007\200z\220\021\220&\230\002\230!\330\010\013\2104\210v\220S\230\001\330\014\022\220*\230A\230Q\330\010\017\210r\320\021#\2401\240F\250&\260\001\340\004\007\200s\210'\220\025\220d\230*\240A\240V\2502\250Q\330\010\017\210r\220\026\220q\230\003\2301\230G\2406\250\021\330\010\014\210H\220B\220a\330\014\r\330\020\031\230\024\230Q\230a\330\023\024\330\020\026\220j\240\001\330\024E\300Q\300a\340\014\020\220\001\220\030\230\026\230y\250\001\250\025\250a\330\010\017\210q\340\004\013\2102\210Y\220a\220v\230V\2401void (nvtxSemanticsCounter_t *, PyObject *)\000_fill_counter_semantics";
    PyObject *data = NULL;
    CYTHON_UNUSED_VAR(__Pyx_DecompressString);
    #endif
    PyObject **stringtab = __pyx_mstate->__pyx_string_tab;
    Py_ssize_t pos = 0;
    for (int i = 0; i < 470; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyUnicode_DecodeUTF8(bytes + pos, bytes_length, NULL);
      if (likely(string) && i >= 107) PyUnicode_InternInPlace(&string);
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
      stringtab[i] = string;
      pos += bytes_length;
    }
    for (int i = 470; i < 518; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyBytes_FromStringAndSize(bytes + pos, bytes_length);
      stringtab[i] = string;
      pos += bytes_length;
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    Py_XDECREF(data);
    for (Py_ssize_t i = 0; i < 518; i++) {
      if (unlikely(PyObject_Hash(stringtab[i]) == -1)) {
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    #if CYTHON_IMMORTAL_CONSTANTS
    {
      PyObject **table = stringtab + 470;
      for (Py_ssize_t i=0; i<48; ++i) {
        #if PY_VERSION_HEX >= 0x030F0000
        PyUnstable_SetImmortal(table[i]);
        #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
        if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
        #if PY_VERSION_HEX < 0x030E0000
        if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
        #else
        if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
        #endif
        {
          Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
        }
        #else
        if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
        #endif
      }
    }
    #endif
  }
  {
    PyObject **numbertab = __pyx_mstate->__pyx_number_tab + 0;
    int8_t const cint_constants_1[] = {0,1,8};
    int32_t const cint_constants_4[] = {129641307L};
    int64_t const cint_constants_8[] = {4294967295LL};
    for (int i = 0; i < 5; i++) {
      numbertab[i] = PyLong_FromLongLong((i < 3 ? cint_constants_1[i - 0] : (i < 4 ? cint_constants_4[i - 3] : cint_constants_8[i - 4])));
      if (unlikely(!numbertab[i])) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_number_tab;
    for (Py_ssize_t i=0; i<5; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */
typedef struct {
    unsigned int argcount : 3;
    unsigned int num_posonly_args : 1;
    unsigned int num_kwonly_args : 3;
    unsigned int nlocals : 5;
    unsigned int flags : 10;
    unsigned int first_line : 12;
} __Pyx_PyCode_New_function_description;
/* NewCodeObj.proto */
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
);


static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  PyObject* tuple_dedup_map = PyDict_New();
  if (unlikely(!tuple_dedup_map)) return -1;
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 121};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_result_code, __pyx_mstate->__pyx_n_u_operation, __pyx_mstate->__pyx_n_u_message};
    __pyx_mstate_global->__pyx_codeobj_tab[0] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_init, __pyx_mstate->__pyx_kp_b_iso88591_O1_M_a_A_1_b_A_AQ_Ry, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[0])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[1] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_T_Jd_Q_G1F_a_vWE_Q_q_t_S_IWEQTT, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[1])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[2] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_4AV1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[2])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 241};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_dtype, __pyx_mstate->__pyx_n_u_timestamp_fields, __pyx_mstate->__pyx_n_u_field_name, __pyx_mstate->__pyx_n_u_metadata, __pyx_mstate->__pyx_n_u_entry_kind};
    __pyx_mstate_global->__pyx_codeobj_tab[3] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_validate_counter_layout, __pyx_mstate->__pyx_kp_b_iso88591_vS_CvS_q_uJgQ_iq_q_q_wawj_c_q_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[3])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 276};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_data, __pyx_mstate->__pyx_n_u_dt, __pyx_mstate->__pyx_n_u_rows, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_column};
    __pyx_mstate_global->__pyx_codeobj_tab[4] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_batch_rows, __pyx_mstate->__pyx_kp_b_iso88591_z_4vS_AQ_r_1F_s_d_AV2Q_r_q_1G6, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[4])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 305};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_dt, __pyx_mstate->__pyx_n_u_operation, __pyx_mstate->__pyx_n_u_subdtype, __pyx_mstate->__pyx_n_u_field_name};
    __pyx_mstate_global->__pyx_codeobj_tab[5] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_validate_native_byte_order, __pyx_mstate->__pyx_kp_b_iso88591_r_y_q_1HAT_r_N_A_q_Qd_t2Q_iq_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[5])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 16, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 320};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_kind, __pyx_mstate->__pyx_n_u_dt, __pyx_mstate->__pyx_n_u_operation, __pyx_mstate->__pyx_n_u_role_fields, __pyx_mstate->__pyx_n_u_range_id_fields, __pyx_mstate->__pyx_n_u_field_name, __pyx_mstate->__pyx_n_u_field_dtype, __pyx_mstate->__pyx_n_u_metadata, __pyx_mstate->__pyx_n_u_entry_kind, __pyx_mstate->__pyx_n_u_entry_type, __pyx_mstate->__pyx_n_u_required_roles, __pyx_mstate->__pyx_n_u_fields, __pyx_mstate->__pyx_n_u_expected_count, __pyx_mstate->__pyx_n_u_role, __pyx_mstate->__pyx_n_u_expected_range_ids, __pyx_mstate->__pyx_n_u_entry_kind};
    __pyx_mstate_global->__pyx_codeobj_tab[6] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_validate_event_dtype, __pyx_mstate->__pyx_kp_b_iso88591_r_A_iq_1_q_A_A_N_Q_Q_Q_Q_a_b_b, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[6])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 392};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_path};
    __pyx_mstate_global->__pyx_codeobj_tab[7] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_canonical_backend_path, __pyx_mstate->__pyx_kp_b_iso88591_4q_1F_Rxq_2U_1BgQa, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[7])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[8] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[8])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[9] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[9])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 437};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_module, __pyx_mstate->__pyx_n_u_loaded};
    __pyx_mstate_global->__pyx_codeobj_tab[10] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_release_backend_module, __pyx_mstate->__pyx_kp_b_iso88591_6_Cq_aq_nA_6_Cq_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[10])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 470};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_error, __pyx_mstate->__pyx_n_u_module, __pyx_mstate->__pyx_n_u_session, __pyx_mstate->__pyx_n_u_exc};
    __pyx_mstate_global->__pyx_codeobj_tab[11] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_close, __pyx_mstate->__pyx_kp_b_iso88591_A_t9Cq_4q_A_hd_A_vS_vS_Q_Ja_Kq_6, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[11])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[12] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[12])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[13] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[13])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 509};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_loaded, __pyx_mstate->__pyx_n_u_canonical_path, __pyx_mstate->__pyx_n_u_module, __pyx_mstate->__pyx_n_u_lib};
    __pyx_mstate_global->__pyx_codeobj_tab[14] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_load_backend, __pyx_mstate->__pyx_kp_b_iso88591_2_AQ_Qa_7_Q_Qa_1A_A_q_6_Cq_l_j, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[14])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 596};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attr, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_config, __pyx_mstate->__pyx_n_u_rc};
    __pyx_mstate_global->__pyx_codeobj_tab[15] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_begin, __pyx_mstate->__pyx_kp_b_iso88591_A_Q_4y_1_aq_I_4wgQ_4q_4y_q_T_N_H, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[15])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 636};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_handle, __pyx_mstate->__pyx_n_u_rc, __pyx_mstate->__pyx_n_u_error, __pyx_mstate->__pyx_n_u_stream, __pyx_mstate->__pyx_n_u_exc, __pyx_mstate->__pyx_n_u_domain};
    __pyx_mstate_global->__pyx_codeobj_tab[16] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_end, __pyx_mstate->__pyx_kp_b_iso88591_A0_Q_4y_1_aq_I_Jd_4q_fA_6_A_A_N, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[16])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 685};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_attr, __pyx_mstate->__pyx_n_u_handle, __pyx_mstate->__pyx_n_u_rc, __pyx_mstate->__pyx_n_u_encoded, __pyx_mstate->__pyx_n_u_c_name};
    __pyx_mstate_global->__pyx_codeobj_tab[17] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_register_domain, __pyx_mstate->__pyx_kp_b_iso88591_A_Yaq_87_Q_I_N_HA_T_Zq_aq_at1_vQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[17])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 700};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_domain};
    __pyx_mstate_global->__pyx_codeobj_tab[18] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_get_domain, __pyx_mstate->__pyx_kp_b_iso88591_a4_t9Cq_l_1_7_6_A_T_aq_wc_aq_IQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[18])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 7, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 736};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_domain, __pyx_mstate->__pyx_n_u_scope, __pyx_mstate->__pyx_n_u_time_domain_id, __pyx_mstate->__pyx_n_u_interleaving, __pyx_mstate->__pyx_n_u_ordering, __pyx_mstate->__pyx_n_u_skid, __pyx_mstate->__pyx_n_u_skid_amount};
    __pyx_mstate_global->__pyx_codeobj_tab[19] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_create_stream, __pyx_mstate->__pyx_kp_b_iso88591_A_q_q_q_f_4z_1_4z_A_1A_4z_1A_4z, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[19])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 828};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[20] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_enter, __pyx_mstate->__pyx_kp_b_iso88591_A_F_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[20])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_VARARGS), 832};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u__12};
    __pyx_mstate_global->__pyx_codeobj_tab[21] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_exit, __pyx_mstate->__pyx_kp_b_iso88591_A_Q_4y_1_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[21])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[22] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[22])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[23] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[23])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[24] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[24])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[25] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[25])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[26] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[26])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[27] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[27])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[28] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[28])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[29] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[29])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[30] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[30])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[31] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[31])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1035};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_string};
    __pyx_mstate_global->__pyx_codeobj_tab[32] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_get_registered_string, __pyx_mstate->__pyx_kp_b_iso88591_A0_4z_5_1A_a_4_7_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[32])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1065};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_string, __pyx_mstate->__pyx_n_u_encoded, __pyx_mstate->__pyx_n_u_c_string, __pyx_mstate->__pyx_n_u_handle, __pyx_mstate->__pyx_n_u_rc};
    __pyx_mstate_global->__pyx_codeobj_tab[33] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_register_string, __pyx_mstate->__pyx_kp_b_iso88591_A_Yaq_A_D_q_AQ_at1_q_hk, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[33])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1074};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_category_id};
    __pyx_mstate_global->__pyx_codeobj_tab[34] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_get_category_id, __pyx_mstate->__pyx_kp_b_iso88591_A2_4z_a_1A_N_waq_d_d_1_wa_1_t_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[34])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1108};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_category_id, __pyx_mstate->__pyx_n_u_encoded, __pyx_mstate->__pyx_n_u_c_name, __pyx_mstate->__pyx_n_u_rc, __pyx_mstate->__pyx_n_u_cached};
    __pyx_mstate_global->__pyx_codeobj_tab[35] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_register_category, __pyx_mstate->__pyx_kp_b_iso88591_A_a_T_T_wgQ_q_at1_c_Q_d_4q_iq_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[35])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 1, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1132};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_parent, __pyx_mstate->__pyx_n_u_parent_id};
    __pyx_mstate_global->__pyx_codeobj_tab[36] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_get_scope, __pyx_mstate->__pyx_kp_b_iso88591_F_7_Q_q_wa_q_q_q_q_j_q_wiwa_j_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[36])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1194};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_parent_id, __pyx_mstate->__pyx_n_u_encoded, __pyx_mstate->__pyx_n_u_c_path, __pyx_mstate->__pyx_n_u_attr, __pyx_mstate->__pyx_n_u_scope_id, __pyx_mstate->__pyx_n_u_rc};
    __pyx_mstate_global->__pyx_codeobj_tab[37] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_register_scope, __pyx_mstate->__pyx_kp_b_iso88591_A_Yaq_87_Q_N_HA_O1_Kq_T_q_JavQa, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[37])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 1, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1210};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_dtype, __pyx_mstate->__pyx_n_u_kind, __pyx_mstate->__pyx_n_u_schema_id, __pyx_mstate->__pyx_n_u_dt, __pyx_mstate->__pyx_n_u_schema_key, __pyx_mstate->__pyx_n_u_schema};
    __pyx_mstate_global->__pyx_codeobj_tab[38] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_get_schema, __pyx_mstate->__pyx_kp_b_iso88591_T_5_uD_JavQ_1A_a_AQ_M_e3j_A_T_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[38])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 3, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1272};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_dtype, __pyx_mstate->__pyx_n_u_description, __pyx_mstate->__pyx_n_u_scope, __pyx_mstate->__pyx_n_u_semantics, __pyx_mstate->__pyx_n_u_scope_id, __pyx_mstate->__pyx_n_u_schema_key};
    __pyx_mstate_global->__pyx_codeobj_tab[39] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_get_counter, __pyx_mstate->__pyx_kp_b_iso88591_A_b_a_q_gQgZvWA_4_1_q_7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[39])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {6, 0, 0, 16, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1342};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_schema_key, __pyx_mstate->__pyx_n_u_description, __pyx_mstate->__pyx_n_u_scope_id, __pyx_mstate->__pyx_n_u_semantics, __pyx_mstate->__pyx_n_u_attr, __pyx_mstate->__pyx_n_u_counter_semantics, __pyx_mstate->__pyx_n_u_schema_id, __pyx_mstate->__pyx_n_u_counter_id, __pyx_mstate->__pyx_n_u_name_bytes, __pyx_mstate->__pyx_n_u_description_bytes, __pyx_mstate->__pyx_n_u_description_ptr, __pyx_mstate->__pyx_n_u_rc, __pyx_mstate->__pyx_n_u_dtype, __pyx_mstate->__pyx_n_u_timestamp_field};
    __pyx_mstate_global->__pyx_codeobj_tab[40] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_register_counter, __pyx_mstate->__pyx_kp_b_iso88591_A_1_1_2_1_6_A_3a_A_Zs_V3a_Zs_A_Z, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[40])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[41] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[41])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[42] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[42])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1445};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attr, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_domain_handle, __pyx_mstate->__pyx_n_u_rc};
    __pyx_mstate_global->__pyx_codeobj_tab[43] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_open, __pyx_mstate->__pyx_kp_b_iso88591_A_0_t883a_l_1_t9IS_l_1_t9G1_HN_H, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[43])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1486};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_handle, __pyx_mstate->__pyx_n_u_rc};
    __pyx_mstate_global->__pyx_codeobj_tab[44] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_close, __pyx_mstate->__pyx_kp_b_iso88591_A_t883a_l_1_a_T_xq_Yg_4q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[44])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1509};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[45] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_enter, __pyx_mstate->__pyx_kp_b_iso88591_A_E_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[45])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_VARARGS), 1513};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u__12};
    __pyx_mstate_global->__pyx_codeobj_tab[46] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_exit, __pyx_mstate->__pyx_kp_b_iso88591_A_t883a_F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[46])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 3, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1609};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_timestamp, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category};
    __pyx_mstate_global->__pyx_codeobj_tab[47] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_mark, __pyx_mstate->__pyx_kp_b_iso88591_A_0_q8_IQk_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[47])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 3, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1642};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_end, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category};
    __pyx_mstate_global->__pyx_codeobj_tab[48] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_pushpop, __pyx_mstate->__pyx_kp_b_iso88591_A_1_Kq_uIWA_0, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[48])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 3, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1678};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_end, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category};
    __pyx_mstate_global->__pyx_codeobj_tab[49] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_startend, __pyx_mstate->__pyx_kp_b_iso88591_A_1_Kq_uIWA_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[49])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 3, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1714};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_timestamp, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category};
    __pyx_mstate_global->__pyx_codeobj_tab[50] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_push, __pyx_mstate->__pyx_kp_b_iso88591_A_0_q_IQk_1_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[50])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1749};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_timestamp};
    __pyx_mstate_global->__pyx_codeobj_tab[51] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_pop, __pyx_mstate->__pyx_kp_b_iso88591_A_a_at_t1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[51])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 3, 11, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1769};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_timestamp, __pyx_mstate->__pyx_n_u_range_id, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category, __pyx_mstate->__pyx_n_u_writer, __pyx_mstate->__pyx_n_u_rid, __pyx_mstate->__pyx_n_u_attr, __pyx_mstate->__pyx_n_u_uattr, __pyx_mstate->__pyx_n_u_owner};
    __pyx_mstate_global->__pyx_codeobj_tab[52] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_start, __pyx_mstate->__pyx_kp_b_iso88591_A_L_9Cq_AQ_a_4_4A_WJavQa_6_A_q_5, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[52])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1827};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_timestamp, __pyx_mstate->__pyx_n_u_range_id};
    __pyx_mstate_global->__pyx_codeobj_tab[53] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_end, __pyx_mstate->__pyx_kp_b_iso88591_A_9Cq_AQ_a_at_t_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[53])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1851};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_schema, __pyx_mstate->__pyx_n_u_row, __pyx_mstate->__pyx_n_u_payload, __pyx_mstate->__pyx_n_u_data, __pyx_mstate->__pyx_n_u_rc};
    __pyx_mstate_global->__pyx_codeobj_tab[54] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_event, __pyx_mstate->__pyx_kp_b_iso88591_AD_6_Q_A_6_1_A_Q_auF_7_1_A_7q_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[54])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 1, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1910};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_schema, __pyx_mstate->__pyx_n_u_rows, __pyx_mstate->__pyx_n_u_ordering, __pyx_mstate->__pyx_n_u_events, __pyx_mstate->__pyx_n_u_event_data, __pyx_mstate->__pyx_n_u_batch, __pyx_mstate->__pyx_n_u_rc};
    __pyx_mstate_global->__pyx_codeobj_tab[55] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_event_batch, __pyx_mstate->__pyx_kp_b_iso88591_A_a_4z_A_1A_6_Q_A_6_q_A_Q_AV6_6, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[55])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1973};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_counter, __pyx_mstate->__pyx_n_u_timestamp, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_iface, __pyx_mstate->__pyx_n_u_i64, __pyx_mstate->__pyx_n_u_f64, __pyx_mstate->__pyx_n_u_rc, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[56] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_counter_sample, __pyx_mstate->__pyx_kp_b_iso88591_AF_AQ_Q_7_G1_A_1_IQ_7_c_q_t89A_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[56])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2043};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_counter, __pyx_mstate->__pyx_n_u_timestamp, __pyx_mstate->__pyx_n_u_reason};
    __pyx_mstate_global->__pyx_codeobj_tab[57] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_counter_sample_no_value, __pyx_mstate->__pyx_kp_b_iso88591_A8_AQ_4z_1A_Q_a_4A_HI_q_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[57])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 2, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 2081};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_counter, __pyx_mstate->__pyx_n_u_data, __pyx_mstate->__pyx_n_u_timestamps, __pyx_mstate->__pyx_n_u_ordering, __pyx_mstate->__pyx_n_u_dt, __pyx_mstate->__pyx_n_u_samples, __pyx_mstate->__pyx_n_u_timestamps_np, __pyx_mstate->__pyx_n_u_batch};
    __pyx_mstate_global->__pyx_codeobj_tab[58] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_writer_pyx, __pyx_mstate->__pyx_n_u_write_counter_batch, __pyx_mstate->__pyx_kp_b_iso88591_A_q_4z_A_1A_AQ_Q_3c_aq_WA_7_G1_j, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[58])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[59] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[59])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[60] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[60])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 4};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[61] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_pyx_unpickle__WriterSchemaRegi, __pyx_mstate->__pyx_kp_b_iso88591_q_0_kQR_7_8_9RR_a_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[61])) goto bad;
  }
  Py_DECREF(tuple_dedup_map);
  return 0;
  bad:
  Py_DECREF(tuple_dedup_map);
  return -1;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CommonTypesMetaclass.init */
  if (likely(__pyx_CommonTypesMetaclass_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CachedMethodType.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
  {
      PyObject *typesModule=NULL;
      typesModule = PyImport_ImportModule("types");
      if (typesModule) {
          __pyx_mstate_global->__Pyx_CachedMethodType = PyObject_GetAttrString(typesModule, "MethodType");
          Py_DECREF(typesModule);
      }
  } // error handling follows
  #endif
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CythonFunctionShared.init */
  if (likely(__pyx_CyFunction_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError (used by GetBuiltinName) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* TupleAndListFromArray (used by fastcall) */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        Py_INCREF(src[i]);
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < (0))) {
            Py_DECREF(res);
            return NULL;
        }
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* PyObjectCall (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall (used by PyObjectCallOneArg) */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED) && CYTHON_METH_FASTCALL
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
          #if CYTHON_COMPILING_IN_LIMITED_API
            return PyObject_Vectorcall(func, args, _nargs, NULL);
          #else
            vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
            if (f) {
                return f(func, args, _nargs, NULL);
            }
          #endif
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectCallOneArg (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* UnpackUnboundCMethod (used by CallUnboundCMethod0) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod0 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        if (likely(cfunc->flag == METH_NOARGS))
            return __Pyx_CallCFunction(cfunc, self, NULL);
        if (likely(cfunc->flag == METH_FASTCALL))
            return __Pyx_CallCFunctionFast(cfunc, self, NULL, 0);
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, NULL, 0, NULL);
        if (likely(cfunc->flag == (METH_VARARGS | METH_KEYWORDS)))
            return __Pyx_CallCFunctionWithKeywords(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple, NULL);
        if (cfunc->flag == METH_VARARGS)
            return __Pyx_CallCFunction(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple);
        return __Pyx__CallUnboundCMethod0(cfunc, self);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod0(&tmp_cfunc, self);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod0(cfunc, self);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *result;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
    result = __Pyx_PyObject_CallOneArg(cfunc->method, self);
    return result;
}

/* py_dict_items (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_items, d);
}

/* py_dict_values (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_values, d);
}

/* OwnedDictNext (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue) {
    PyObject *next = NULL;
    if (!*ppos) {
        if (pvalue) {
            PyObject *dictview = pkey ? __Pyx_PyDict_Items(p) : __Pyx_PyDict_Values(p);
            if (unlikely(!dictview)) goto bad;
            *ppos = PyObject_GetIter(dictview);
            Py_DECREF(dictview);
        } else {
            *ppos = PyObject_GetIter(p);
        }
        if (unlikely(!*ppos)) goto bad;
    }
    next = PyIter_Next(*ppos);
    if (!next) {
        if (PyErr_Occurred()) goto bad;
        return 0;
    }
    if (pkey && pvalue) {
        *pkey = __Pyx_PySequence_ITEM(next, 0);
        if (unlikely(*pkey)) goto bad;
        *pvalue = __Pyx_PySequence_ITEM(next, 1);
        if (unlikely(*pvalue)) goto bad;
        Py_DECREF(next);
    } else if (pkey) {
        *pkey = next;
    } else {
        assert(pvalue);
        *pvalue = next;
    }
    return 1;
  bad:
    Py_XDECREF(next);
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
    PyErr_FormatUnraisable("Exception ignored in __Pyx_PyDict_NextRef");
#else
    PyErr_WriteUnraisable(__pyx_mstate_global->__pyx_n_u_Pyx_PyDict_NextRef);
#endif
    if (pkey) *pkey = NULL;
    if (pvalue) *pvalue = NULL;
    return 0;
}
#else // !CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue) {
    int result = PyDict_Next(p, ppos, pkey, pvalue);
    if (likely(result == 1)) {
        if (pkey) Py_INCREF(*pkey);
        if (pvalue) Py_INCREF(*pvalue);
    }
    return result;
}
#endif

/* RaiseDoubleKeywords (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywordsImpl (used by ParseKeywords) */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos = NULL;
    #else
    Py_ssize_t pos = 0;
    #endif
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (
        #if CYTHON_AVOID_BORROWED_REFS
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL)
        #else
        PyDict_Next(kwds, &pos, &key, NULL)
        #endif
    ) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        #endif
    }
    __Pyx_END_CRITICAL_SECTION();
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(pos);
    #endif
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}

/* ParseKeywords */
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyDictVersioning (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* CIntToDigits (used by CIntToPyUnicode) */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp = PySequence_Repeat(padding, uoffset - prepend_sign);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyUnicode_Concat(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyUnicode_Concat(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* COrdinalToPyUnicode (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value) {
    return value <= 1114111;
}
static PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t ulength, char padding_char) {
    Py_ssize_t padding_length = ulength - 1;
    if (likely((padding_length <= 250) && (value < 0xD800 || value > 0xDFFF))) {
        char chars[256];
        if (value <= 255) {
            memset(chars, padding_char, (size_t) padding_length);
            chars[ulength-1] = (char) value;
            return PyUnicode_DecodeLatin1(chars, ulength, NULL);
        }
        char *cpos = chars + sizeof(chars);
        if (value < 0x800) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xc0 | (value & 0x1f));
        } else if (value < 0x10000) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xe0 | (value & 0x0f));
        } else {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xf0 | (value & 0x07));
        }
        cpos -= padding_length;
        memset(cpos, padding_char, (size_t) padding_length);
        return PyUnicode_DecodeUTF8(cpos, chars + sizeof(chars) - cpos, NULL);
    }
    if (value <= 127 && CYTHON_USE_UNICODE_INTERNALS) {
        const char chars[1] = {(char) value};
        return __Pyx_PyUnicode_BuildFromAscii(ulength, chars, 1, 0, padding_char);
    }
    {
        PyObject *uchar, *padding_uchar, *padding, *result;
        padding_uchar = PyUnicode_FromOrdinal(padding_char);
        if (unlikely(!padding_uchar)) return NULL;
        padding = PySequence_Repeat(padding_uchar, padding_length);
        Py_DECREF(padding_uchar);
        if (unlikely(!padding)) return NULL;
        uchar = PyUnicode_FromOrdinal(value);
        if (unlikely(!uchar)) {
            Py_DECREF(padding);
            return NULL;
        }
        result = PyUnicode_Concat(padding, uchar);
        Py_DECREF(padding);
        Py_DECREF(uchar);
        return result;
    }
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (int) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(int)*3+2];
    char *dpos, *end = digits + sizeof(int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    int remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (int) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (int) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        Py_INCREF(values[i]);
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* RaiseException */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* PyObjectLookupSpecial */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* GetTopmostException (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C0000
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#elif __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    local_value = PyErr_GetRaisedException();
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
#else
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
#endif // __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#elif __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    PyErr_SetHandledException(local_value);
    Py_XDECREF(local_value);
    Py_XDECREF(local_type);
    Py_XDECREF(local_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
#endif
}

/* RejectKeywords */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds) {
    PyObject *key = NULL;
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds))) {
        key = __Pyx_PySequence_ITEM(kwds, 0);
    } else {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *pos = NULL;
#else
        Py_ssize_t pos = 0;
#endif
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
        if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return;
#endif
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL);
#if CYTHON_AVOID_BORROWED_REFS
        Py_XDECREF(pos);
#endif
    }
    if (likely(key)) {
        PyErr_Format(PyExc_TypeError,
            "%s() got an unexpected keyword argument '%U'",
            function_name, key);
        Py_DECREF(key);
    }
}

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* decode_c_bytes (used by decode_bytes) */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    if (unlikely((start < 0) | (stop < 0))) {
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (stop > length)
        stop = length;
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* PyObjectCallNoArg (used by pyfrozenset_new) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* pyfrozenset_new (used by PySetContains) */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it) {
    if (it) {
        PyObject* result;
#if CYTHON_COMPILING_IN_PYPY
        PyObject* args;
        args = PyTuple_Pack(1, it);
        if (unlikely(!args))
            return NULL;
        result = PyObject_Call((PyObject*)&PyFrozenSet_Type, args, NULL);
        Py_DECREF(args);
        return result;
#else
        if (PyFrozenSet_CheckExact(it)) {
            Py_INCREF(it);
            return it;
        }
        result = PyFrozenSet_New(it);
        if (unlikely(!result))
            return NULL;
        if ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
#if CYTHON_COMPILING_IN_LIMITED_API
            || __Pyx_get_runtime_version() >= 0x030A0000
#endif
            )
            return result;
        {
            Py_ssize_t size = __Pyx_PySet_GET_SIZE(result);
            if (likely(size > 0))
                return result;
#if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely(size < 0)) {
                Py_DECREF(result);
                return NULL;
            }
#endif
        }
        Py_DECREF(result);
#endif
    }
    return __Pyx_PyObject_CallNoArg((PyObject*) &PyFrozenSet_Type);
}

/* PySetContains */
static int __Pyx_PySet_ContainsUnhashable(PyObject *set, PyObject *key) {
    int result = -1;
    if (PySet_Check(key) && PyErr_ExceptionMatches(PyExc_TypeError)) {
        PyObject *tmpkey;
        PyErr_Clear();
        tmpkey = __Pyx_PyFrozenSet_New(key);
        if (tmpkey != NULL) {
            result = PySet_Contains(set, tmpkey);
            Py_DECREF(tmpkey);
        }
    }
    return result;
}
static CYTHON_INLINE int __Pyx_PySet_ContainsTF(PyObject* key, PyObject* set, int eq) {
    int result = PySet_Contains(set, key);
    if (unlikely(result < 0)) {
        result = __Pyx_PySet_ContainsUnhashable(set, key);
    }
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_uint64_t(uint64_t value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (uint64_t) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_uint64_t(uint64_t value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(uint64_t)*3+2];
    char *dpos, *end = digits + sizeof(uint64_t)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    uint64_t remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (uint64_t) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (uint64_t) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (uint64_t) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static PyObject *__Pyx_GetItemInt_Generic_size(PyObject *o, Py_ssize_t i) {
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_AVOID_BORROWED_REFS
    CYTHON_UNUSED_VAR(boundscheck);
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        Py_ssize_t size = __Pyx_PyList_GET_SIZE(o);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(size < 0)) return NULL;
        #endif
        wrapped_i += size;
    }
    return __Pyx_PyList_GetItemRef(o, wrapped_i);
#elif CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS
    Py_ssize_t wrapped_i = i;
    Py_ssize_t size = (wraparound | boundscheck) ? PyList_GET_SIZE(o) : -1;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += size;
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, size))) {
        return __Pyx_PyList_GET_ITEM_REF(o, wrapped_i, unsafe_shared);
    }
    return __Pyx_GetItemInt_Generic_size(o, i);
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_AVOID_BORROWED_REFS
    CYTHON_UNUSED_VAR(boundscheck);
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(o);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(size < 0)) return NULL;
        #endif
        wrapped_i += size;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_COMPILING_IN_LIMITED_API
    return PySequence_ITEM(o, wrapped_i);
    #else
    if (unlikely(wrapped_i < 0)) {
        PyErr_SetString(PyExc_IndexError, "tuple index out of range");
        return NULL;
    }
    return PySequence_GetItem(o, wrapped_i);
    #endif
#elif CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS
    Py_ssize_t wrapped_i = i;
    Py_ssize_t size = (wraparound | boundscheck) ? PyTuple_GET_SIZE(o) : -1;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += size;
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, size))) {
        return __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic_size(o, i);
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        return boundscheck ? __Pyx_PyList_GetItemRef(o, n) : __Pyx_PyList_GET_ITEM_REF(o, n, unsafe_shared);
    } else
    #if !CYTHON_AVOID_BORROWED_REFS
    if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            return __Pyx_NewRef(PyTuple_GET_ITEM(o, n));
        }
    } else
    #endif
#else
    if ((!wraparound || i >= 0) & PyList_CheckExact(o)) {
        return boundscheck ? __Pyx_PyList_GetItemRef(o, i) : __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared);
    } else
#endif
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
    {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (!is_list && mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (is_list || likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    (void)wraparound;
    (void)boundscheck;
    return __Pyx_GetItemInt_Generic_size(o, i);
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        #if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        if ((PyTypeObject*)obj == &PyType_Type) {
            return Py_GenericAlias(obj, key);
        }
        #endif
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_mstate_global->__pyx_n_u_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result;
    if (unlikely(!s)) return NULL;
    result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 0;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) != 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 1;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 1;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal != 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a != (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_NE));
}

/* PyObjectVectorCallKwBuilder */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    Py_INCREF(key);
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* PyObjectVectorCallMethodKwBuilder */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames) {
    PyObject *result;
    PyObject *obj = PyObject_GetAttr(args[0], name);
    if (unlikely(!obj))
        return NULL;
    result = __Pyx_Object_Vectorcall_CallFromBuilder(obj, args+1, nargsf-1, kwnames);
    Py_DECREF(obj);
    return result;
}
#endif

/* UnicodeConcatInPlace */
# if CYTHON_COMPILING_IN_CPYTHON
static int
__Pyx_unicode_modifiable(PyObject *unicode, int unsafe_shared)
{
    if (!__Pyx_IS_UNIQUELY_REFERENCED(unicode, unsafe_shared))
        return 0;
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030F0000
    if (PyUnstable_Unicode_GET_CACHED_HASH(unicode) != -1)
        return 0;
#endif
    if (!PyUnicode_CheckExact(unicode))
        return 0;
    if (PyUnicode_CHECK_INTERNED(unicode))
        return 0;
    return 1;
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    ) {
    PyObject *left = *p_left;
    Py_ssize_t left_len, right_len, new_len;
    if (unlikely(__Pyx_PyUnicode_READY(left) == -1))
        return NULL;
    if (unlikely(__Pyx_PyUnicode_READY(right) == -1))
        return NULL;
    left_len = PyUnicode_GET_LENGTH(left);
    if (left_len == 0) {
        Py_INCREF(right);
        return right;
    }
    right_len = PyUnicode_GET_LENGTH(right);
    if (right_len == 0) {
        Py_INCREF(left);
        return left;
    }
    if (unlikely(left_len > PY_SSIZE_T_MAX - right_len)) {
        PyErr_SetString(PyExc_OverflowError,
                        "strings are too large to concat");
        return NULL;
    }
    new_len = left_len + right_len;
    if (left != right
            && __Pyx_unicode_modifiable(left, unsafe_shared)
            && PyUnicode_CheckExact(right)
            && PyUnicode_KIND(right) <= PyUnicode_KIND(left)
            && !(PyUnicode_IS_ASCII(left) && !PyUnicode_IS_ASCII(right))) {
        int ret;
        __Pyx_GIVEREF(*p_left);
        ret = PyUnicode_Resize(p_left, new_len);
        __Pyx_GOTREF(*p_left);
        if (unlikely(ret != 0))
            return NULL;
        #if PY_VERSION_HEX >= 0x030d0000
        if (unlikely(PyUnicode_CopyCharacters(*p_left, left_len, right, 0, right_len) < 0)) return NULL;
        #else
        _PyUnicode_FastCopyCharacters(*p_left, left_len, right, 0, right_len);
        #endif
        __Pyx_INCREF(*p_left);
        __Pyx_GIVEREF(*p_left);
        return *p_left;
    } else {
        return __Pyx_PyUnicode_Concat(left, right);
    }
  }
#endif

/* PyObjectCall2Args (used by PyObjectCallMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod (used by PyObjectCallMethod1) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(tp);
    PyErr_Format(PyExc_AttributeError,
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}
#endif

/* PyObjectCallMethod1 (used by append) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
#endif
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[2] = {obj, arg};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_Call2Args;
    return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
#endif
}

/* append */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_mstate_global->__pyx_n_u_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* DictGetItem */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    if (unlikely(__Pyx_PyDict_GetItemRef(d, key, &value) == 0)) { // no value, no error
        if (unlikely(PyTuple_Check(key))) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args)) {
                PyErr_SetObject(PyExc_KeyError, args);
                Py_DECREF(args);
            }
        } else {
            PyErr_SetObject(PyExc_KeyError, key);
        }
    }
    return value;
}
#endif

/* IterFinish (used by dict_iter) */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* PyObjectCallMethod0 (used by dict_iter) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[1] = {obj};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_CallNoArg;
    return PyObject_VectorcallMethod(method_name, args, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
#endif
}

/* RaiseNeedMoreValuesToUnpack (used by UnpackTuple2) */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack (used by UnpackItemEndCheck) */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck (used by UnpackTuple2) */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* RaiseNoneIterError (used by UnpackTupleError) */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError (used by UnpackTuple2) */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else {
      Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(t);
 #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely(size < 0)) return;
 #endif
      if (size < index) {
        __Pyx_RaiseNeedMoreValuesError(size);
      } else {
        __Pyx_RaiseTooManyValuesError(index);
      }
    }
}

/* UnpackTuple2 (used by dict_iter) */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
        PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple) {
    if (likely(is_tuple || PyTuple_Check(tuple))) {
        Py_ssize_t size;
        if (has_known_size) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        size = __Pyx_PyTuple_GET_SIZE(tuple);
        if (likely(size == 2)) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        if (size >= 0) {
            __Pyx_UnpackTupleError(tuple, 2);
        }
        return -1;
    } else {
        return __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple);
    }
}
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
    value1 = __Pyx_PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = __Pyx_PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = __Pyx_PyObject_GetIterNextFunc(iter);
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
#if CYTHON_AVOID_BORROWED_REFS
#include <string.h>
#endif
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_AVOID_BORROWED_REFS
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#else
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_AVOID_BORROWED_REFS
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
#if !CYTHON_AVOID_BORROWED_REFS
static CYTHON_INLINE int __Pyx_dict_iter_next_source_is_dict(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem) {
    PyObject *key, *value;
    if (unlikely(orig_length != PyDict_Size(iter_obj))) {
        PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
        return -1;
    }
    if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
        return 0;
    }
    if (pitem) {
        PyObject* tuple = PyTuple_New(2);
        if (unlikely(!tuple)) {
            return -1;
        }
        Py_INCREF(key);
        Py_INCREF(value);
        #if CYTHON_ASSUME_SAFE_MACROS
        PyTuple_SET_ITEM(tuple, 0, key);
        PyTuple_SET_ITEM(tuple, 1, value);
        #else
        if (unlikely(PyTuple_SetItem(tuple, 0, key) < 0)) {
            Py_DECREF(value);
            Py_DECREF(tuple);
            return -1;
        }
        if (unlikely(PyTuple_SetItem(tuple, 1, value) < 0)) {
            Py_DECREF(tuple);
            return -1;
        }
        #endif
        *pitem = tuple;
    } else {
        if (pkey) {
            Py_INCREF(key);
            *pkey = key;
        }
        if (pvalue) {
            Py_INCREF(value);
            *pvalue = value;
        }
    }
    return 1;
}
#endif
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_AVOID_BORROWED_REFS
    if (source_is_dict) {
        int result;
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_BEGIN_CRITICAL_SECTION(iter_obj);
#endif
        result = __Pyx_dict_iter_next_source_is_dict(iter_obj, orig_length, ppos, pkey, pvalue, pitem);
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_END_CRITICAL_SECTION();
#endif
        return result;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(tuple_size < 0)) return -1;
        #endif
        if (unlikely(pos >= tuple_size)) return 0;
        *ppos = pos + 1;
        #if CYTHON_ASSUME_SAFE_MACROS
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        #else
        next_item = PyTuple_GetItem(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        #endif
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t list_size = __Pyx_PyList_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(list_size < 0)) return -1;
        #endif
        if (unlikely(pos >= list_size)) return 0;
        *ppos = pos + 1;
        next_item = __Pyx_PyList_GET_ITEM_REF(iter_obj, pos, __Pyx_ReferenceSharing_OwnStrongReference);
        if (unlikely(!next_item)) return -1;
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* RaiseKeywordRequired */
static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name) {
    PyErr_Format(PyExc_TypeError,
        "%s() needs keyword-only argument %U", func_name, kw_name);
}

/* py_set_discard_unhashable (used by py_set_discard) */
static int __Pyx_PySet_DiscardUnhashable(PyObject *set, PyObject *key) {
    PyObject *tmpkey;
    int rv;
    if (likely(!PySet_Check(key) || !PyErr_ExceptionMatches(PyExc_TypeError)))
        return -1;
    PyErr_Clear();
    tmpkey = __Pyx_PyFrozenSet_New(key);
    if (tmpkey == NULL)
        return -1;
    rv = PySet_Discard(set, tmpkey);
    Py_DECREF(tmpkey);
    return rv;
}

/* py_set_discard */
static CYTHON_INLINE int __Pyx_PySet_Discard(PyObject *set, PyObject *key) {
    int found = PySet_Discard(set, key);
    if (unlikely(found < 0)) {
        found = __Pyx_PySet_DiscardUnhashable(set, key);
    }
    return found;
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    int was_initialized =  __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return __Pyx_CallCFunction(cfunc, self, arg);
        } else if (flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, &arg, 1);
        } else if (flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, &arg, 1, NULL);
        }
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod1(&tmp_cfunc, self, arg);
    }
#endif
    PyObject* result = __Pyx__CallUnboundCMethod1(cfunc, self, arg);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *args = PyTuple_New(1);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
    } else
#endif
    {
        result = __Pyx_PyObject_Call2Args(cfunc->method, self, arg);
    }
    return result;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyBytes_CheckExact(key) || PyUnicode_CheckExact(key) || PyLong_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* ArgTypeTestFunc (used by ArgTypeTest) */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    PyObject *extra_info = __pyx_mstate_global->__pyx_empty_unicode;
    int from_annotation_subclass = 0;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (!exact) {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    } else if (exact == 2) {
        if (__Pyx_TypeCheck(obj, type)) {
            from_annotation_subclass = 1;
            extra_info = __pyx_mstate_global->__pyx_kp_u_Note_that_Cython_is_deliberately;
        }
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")"
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        "%s%U"
#endif
        , name, type_name, obj_type_name
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        , (from_annotation_subclass ? ". " : ""), extra_info
#endif
        );
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (exact == 2 && from_annotation_subclass) {
        PyObject *res;
        PyObject *vargs[2];
        vargs[0] = PyErr_GetRaisedException();
        vargs[1] = extra_info;
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_kp_u_add_note, vargs, 2, NULL);
        Py_XDECREF(res);
        PyErr_SetRaisedException(vargs[0]);
    }
#endif
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* IterNextPlain (used by IterNext) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void) {
    if (unlikely(!__pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache))
        __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache = __Pyx_GetBuiltinName(__pyx_mstate_global->__pyx_n_u_next);
    return __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator) {
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    PyObject *result;
    PyObject *next = __Pyx_GetBuiltinNext_LimitedAPI();
    if (unlikely(!next)) return NULL;
    result = PyObject_CallFunctionObjArgs(next, iterator, NULL);
    return result;
#else
    (void)__Pyx_GetBuiltinName; // only for early limited API
    iternextfunc iternext = __Pyx_PyObject_GetIterNextFunc(iterator);
    assert(iternext);
    return iternext(iterator);
#endif
}

/* IterNext */
static PyObject *__Pyx_PyIter_Next2Default(PyObject* defval) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (!defval || unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return NULL;
        __Pyx_PyErr_Clear();
        Py_INCREF(defval);
        return defval;
    }
    if (defval) {
        Py_INCREF(defval);
        return defval;
    }
    __Pyx_PyErr_SetNone(PyExc_StopIteration);
    return NULL;
}
static void __Pyx_PyIter_Next_ErrorNoIterator(PyObject *iterator) {
    __Pyx_TypeName iterator_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(iterator));
    PyErr_Format(PyExc_TypeError,
        __Pyx_FMT_TYPENAME " object is not an iterator", iterator_type_name);
    __Pyx_DECREF_TypeName(iterator_type_name);
}
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject* iterator, PyObject* defval) {
    PyObject* next;
#if !CYTHON_COMPILING_IN_LIMITED_API
    iternextfunc iternext = __Pyx_PyObject_TryGetSlot(iterator, tp_iternext, iternextfunc);
    if (likely(iternext)) {
        next = iternext(iterator);
        if (likely(next))
            return next;
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
        if (unlikely(iternext == &_PyObject_NextNotImplemented))
            return NULL;
    #endif
    } else if (CYTHON_USE_TYPE_SLOTS) {
        __Pyx_PyIter_Next_ErrorNoIterator(iterator);
        return NULL;
    } else
#endif
    if (unlikely(!PyIter_Check(iterator))) {
        __Pyx_PyIter_Next_ErrorNoIterator(iterator);
        return NULL;
    } else {
        next = defval ? PyIter_Next(iterator) : __Pyx_PyIter_Next_Plain(iterator);
        if (likely(next))
            return next;
    }
    return __Pyx_PyIter_Next2Default(defval);
}

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 1;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) == 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 0;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 0;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal == 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a == (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* AllocateExtensionType */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final) {
    if (is_final || likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
        allocfunc alloc_func = __Pyx_PyType_GetSlot(t, tp_alloc, allocfunc);
        return alloc_func(t, 0);
    } else {
        newfunc tp_new = __Pyx_PyType_TryGetSlot(&PyBaseObject_Type, tp_new, newfunc);
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (!tp_new) {
            PyObject *new_str = PyUnicode_FromString("__new__");
            if (likely(new_str)) {
                PyObject *o = PyObject_CallMethodObjArgs((PyObject *)&PyBaseObject_Type, new_str, t, NULL);
                Py_DECREF(new_str);
                return o;
            } else
                return NULL;
        } else
    #endif
        return tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
    }
}

/* CallTypeTraverse */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg) {
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
    if (__Pyx_get_runtime_version() < 0x03090000) return 0;
    #endif
    if (!always_call) {
        PyTypeObject *base = __Pyx_PyObject_GetSlot(o, tp_base, PyTypeObject*);
        unsigned long flags = PyType_GetFlags(base);
        if (flags & Py_TPFLAGS_HEAPTYPE) {
            return 0;
        }
    }
    Py_VISIT((PyObject*)Py_TYPE(o));
    return 0;
}
#endif

/* CallNextTpDealloc */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    destructor tp_dealloc = NULL;
    while (type && __Pyx_PyType_GetSlot(type, tp_dealloc, destructor) != current_tp_dealloc)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    while (type && (tp_dealloc = __Pyx_PyType_GetSlot(type, tp_dealloc, destructor)) == current_tp_dealloc)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    if (type)
        tp_dealloc(obj);
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    traverseproc tp_traverse = NULL;
    while (type && __Pyx_PyType_GetSlot(type, tp_traverse, traverseproc) != current_tp_traverse)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    while (type && (tp_traverse = __Pyx_PyType_GetSlot(type, tp_traverse, traverseproc)) == current_tp_traverse)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    if (type && tp_traverse)
        return tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    inquiry tp_clear = NULL;
    while (type && __Pyx_PyType_GetSlot(type, tp_clear, inquiry) != current_tp_clear)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    while (type && (tp_clear = __Pyx_PyType_GetSlot(type, tp_clear, inquiry)) == current_tp_clear)
        type = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
    if (type && tp_clear)
        tp_clear(obj);
}

/* FunctionExport */
static int __Pyx_ExportFunction(PyObject *api_dict, const char *name, void (*f)(void), const char *sig) {
    PyObject *cobj;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    tmp.fp = f;
    cobj = PyCapsule_New(tmp.p, sig, 0);
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(api_dict, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    return 0;
bad:
    Py_XDECREF(cobj);
    return -1;
}

/* GetApiDict */
static PyObject *__Pyx_ApiExport_GetApiDict(void) {
    PyObject *d;
    if (__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_capi, &d) == -1)
        return NULL;
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_pyx_capi, d) < 0)
            goto bad;
    }
    return d;
bad:
    Py_XDECREF(d);
    return NULL;
}

/* LimitedApiGetTypeDict (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static Py_ssize_t __Pyx_GetTypeDictOffset(void) {
    PyObject *tp_dictoffset_o;
    Py_ssize_t tp_dictoffset;
    tp_dictoffset_o = PyObject_GetAttrString((PyObject*)(&PyType_Type), "__dictoffset__");
    if (unlikely(!tp_dictoffset_o)) return -1;
    tp_dictoffset = PyLong_AsSsize_t(tp_dictoffset_o);
    Py_DECREF(tp_dictoffset_o);
    if (unlikely(tp_dictoffset == 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' doesn't have a dictoffset");
        return -1;
    } else if (unlikely(tp_dictoffset < 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' has an unexpected negative dictoffset. "
            "Please report this as Cython bug");
        return -1;
    }
    return tp_dictoffset;
}
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp) {
    static Py_ssize_t tp_dictoffset = 0;
    if (unlikely(tp_dictoffset == 0)) {
        tp_dictoffset = __Pyx_GetTypeDictOffset();
        if (unlikely(tp_dictoffset == -1 && PyErr_Occurred())) {
            tp_dictoffset = 0; // try again next time?
            return NULL;
        }
    }
    return *(PyObject**)((char*)tp + tp_dictoffset);
}
#endif

/* SetItemOnTypeDict (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_SetItem(tp_dict, k, v);
    if (likely(!result)) {
        PyType_Modified(tp);
        if (unlikely(PyObject_HasAttr(v, __pyx_mstate_global->__pyx_n_u_set_name))) {
            PyObject *setNameResult = PyObject_CallMethodObjArgs(v, __pyx_mstate_global->__pyx_n_u_set_name,  (PyObject *) tp, k, NULL);
            if (!setNameResult) return -1;
            Py_DECREF(setNameResult);
        }
    }
    return result;
}

/* FixUpExtensionType */
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if __PYX_LIMITED_VERSION_HEX > 0x030900B1
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#else
    const PyType_Slot *slot = spec->slots;
    int changed = 0;
#if !CYTHON_COMPILING_IN_LIMITED_API
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
#if !CYTHON_COMPILING_IN_CPYTHON
        const
#endif  // !CYTHON_COMPILING_IN_CPYTHON)
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_vectorcall_offset = memb->offset;
                    changed = 1;
                }
#endif  // CYTHON_METH_FASTCALL
#if !CYTHON_COMPILING_IN_PYPY
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                    Py_DECREF(descr);
                    if (unlikely(set_item_result < 0)) {
                        return -1;
                    }
                    changed = 1;
                }
#endif  // !CYTHON_COMPILING_IN_PYPY
            }
            memb++;
        }
    }
#endif  // !CYTHON_COMPILING_IN_LIMITED_API
#if !CYTHON_COMPILING_IN_PYPY
    slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_getset)
        slot++;
    if (slot && slot->slot == Py_tp_getset) {
        PyGetSetDef *getset = (PyGetSetDef*) slot->pfunc;
        while (getset && getset->name) {
            if (getset->name[0] == '_' && getset->name[1] == '_' && strcmp(getset->name, "__module__") == 0) {
                PyObject *descr = PyDescr_NewGetSet(type, getset);
                if (unlikely(!descr))
                    return -1;
                #if CYTHON_COMPILING_IN_LIMITED_API
                PyObject *pyname = PyUnicode_FromString(getset->name);
                if (unlikely(!pyname)) {
                    Py_DECREF(descr);
                    return -1;
                }
                int set_item_result = __Pyx_SetItemOnTypeDict(type, pyname, descr);
                Py_DECREF(pyname);
                #else
                CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
                int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                #endif
                Py_DECREF(descr);
                if (unlikely(set_item_result < 0)) {
                    return -1;
                }
                changed = 1;
            }
            ++getset;
        }
    }
#else
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#endif  // !CYTHON_COMPILING_IN_PYPY
    if (changed)
        PyType_Modified(type);
#endif  // PY_VERSION_HEX > 0x030900B1
    return 0;
}

/* ValidateBasesTuple (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_SIZE
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (unlikely(n < 0)) return -1;
#endif
    for (i = 1; i < n; i++)
    {
        PyTypeObject *b;
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !CYTHON_USE_TYPE_SLOTS
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
CYTHON_UNUSED static int __Pyx_PyType_HasMultipleInheritance(PyTypeObject *t) {
    while (t) {
        PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
        if (bases) {
            return 1;
        }
        t = __Pyx_PyType_GetSlot(t, tp_base, PyTypeObject*);
    }
    return 0;
}
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !CYTHON_COMPILING_IN_CPYTHON || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    if (!__Pyx_PyType_HasMultipleInheritance(t)) {
        return PyType_Ready(t);
    }
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400) &&\
                !CYTHON_COMPILING_IN_GRAAL
        gc = PyImport_GetModule(__pyx_mstate_global->__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_mstate_global->__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i=0;
    Py_ssize_t size;
    void** base_vtables;
    __Pyx_TypeName tp_base_name = NULL;
    __Pyx_TypeName base_name = NULL;
    void* unknown = (void*)-1;
    PyObject* bases = __Pyx_PyType_GetSlot(type, tp_bases, PyObject*);
    int base_depth = 0;
    {
        PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        while (base) {
            base_depth += 1;
            base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
        }
    }
    base_vtables = (void**) PyMem_Malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
#if CYTHON_COMPILING_IN_LIMITED_API
    size = PyTuple_Size(bases);
    if (size < 0) goto other_failure;
#else
    size = PyTuple_GET_SIZE(bases);
#endif
    for (i = 1; i < size; i++) {
        PyObject *basei;
        void* base_vtable;
#if CYTHON_AVOID_BORROWED_REFS
        basei = PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#else
        basei = PyTuple_GET_ITEM(bases, i);
#endif
        base_vtable = __Pyx_GetVtable((PyTypeObject*)basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
            }
        }
    }
    PyErr_Clear();
    PyMem_Free(base_vtables);
    return 0;
bad:
    {
        PyTypeObject* basei = NULL;
        PyTypeObject* tp_base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        tp_base_name = __Pyx_PyType_GetFullyQualifiedName(tp_base);
#if CYTHON_AVOID_BORROWED_REFS
        basei = (PyTypeObject*)PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = (PyTypeObject*)PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#else
        basei = (PyTypeObject*)PyTuple_GET_ITEM(bases, i);
#endif
        base_name = __Pyx_PyType_GetFullyQualifiedName(basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
    }
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
really_bad: // bad has failed!
#endif
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
other_failure:
#endif
    PyMem_Free(base_vtables);
    return -1;
}

/* DelItemOnTypeDict (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_DelItem(tp_dict, k);
    if (likely(!result)) PyType_Modified(tp);
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_mstate_global->__pyx_n_u_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_mstate_global->__pyx_n_u_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython);
            if (likely(reduce_cython)) {
                ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_mstate_global->__pyx_n_u_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetFullyQualifiedName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_2_9
#define __PYX_HAVE_RT_ImportType_3_2_9
static PyTypeObject *__Pyx_ImportType_3_2_9(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_9 check_size)
{
    PyObject *result = 0;
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API)
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !( defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API) )
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    if (size == 0) {
        return (PyTypeObject *)result;
    }
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_2_9 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_2_9 && (size_t)basicsize > size) {
        if (PyErr_WarnFormat(NULL, 0,
                "%.200s.%.200s size changed, may indicate binary incompatibility. "
                "Expected %zd from C header, got %zd from PyObject",
                module_name, class_name, size, basicsize) < 0) {
            goto bad;
        }
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* PxdImportShared (used by FunctionImport) */
#ifndef __PYX_HAVE_RT_ImportFromPxd_3_2_9
#define __PYX_HAVE_RT_ImportFromPxd_3_2_9
static int __Pyx_ImportFromPxd_3_2_9(PyObject *module, const char *name, void **p, const char *sig, const char *what) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, "__pyx_capi__");
    if (!d)
        goto bad;
#if (defined(Py_LIMITED_API) && Py_LIMITED_API >= 0x030d0000) || (!defined(Py_LIMITED_API) && PY_VERSION_HEX >= 0x030d0000)
    PyDict_GetItemStringRef(d, name, &cobj);
#else
    cobj = PyDict_GetItemString(d, name);
    Py_XINCREF(cobj);
#endif
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C %.8s %.200s",
                PyModule_GetName(module), what, name);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C %.8s %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             what, PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    Py_DECREF(cobj);
    return 0;
bad:
    Py_XDECREF(d);
    Py_XDECREF(cobj);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction_3_2_9
#define __PYX_HAVE_RT_ImportFunction_3_2_9
static int __Pyx_ImportFunction_3_2_9(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    int result = __Pyx_ImportFromPxd_3_2_9(module, funcname, &tmp.p, sig, "function");
    if (result == 0) {
        *f = tmp.fp;
    }
    return result;
}
#endif

/* HasAttr (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!PyUnicode_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_PyObject_GetAttrStrNoError(o, n);
    if (!r) {
        return (unlikely(PyErr_Occurred())) ? -1 : 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}
#endif

/* ImportImpl (used by Import) */
static int __Pyx__Import_GetModule(PyObject *qualname, PyObject **module) {
    PyObject *imported_module = PyImport_GetModule(qualname);
    if (unlikely(!imported_module)) {
        *module = NULL;
        if (PyErr_Occurred()) {
            return -1;
        }
        return 0;
    }
    *module = imported_module;
    return 1;
}
static int __Pyx__Import_Lookup(PyObject *qualname, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject **module) {
    PyObject *imported_module;
    PyObject *top_level_package_name;
    Py_ssize_t i;
    int status, module_found;
    Py_ssize_t dot_index;
    module_found = __Pyx__Import_GetModule(qualname, &imported_module);
    if (unlikely(!module_found || module_found == -1)) {
        *module = NULL;
        return module_found;
    }
    if (imported_names) {
        for (i = 0; i < len_imported_names; i++) {
            PyObject *imported_name = imported_names[i];
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
            int has_imported_attribute = PyObject_HasAttr(imported_module, imported_name);
#else
            int has_imported_attribute = PyObject_HasAttrWithError(imported_module, imported_name);
            if (unlikely(has_imported_attribute == -1)) goto error;
#endif
            if (!has_imported_attribute) {
                goto not_found;
            }
        }
        *module = imported_module;
        return 1;
    }
    dot_index = PyUnicode_FindChar(qualname, '.', 0, PY_SSIZE_T_MAX, 1);
    if (dot_index == -1) {
        *module = imported_module;
        return 1;
    }
    if (unlikely(dot_index == -2)) goto error;
    top_level_package_name = PyUnicode_Substring(qualname, 0, dot_index);
    if (unlikely(!top_level_package_name)) goto error;
    Py_DECREF(imported_module);
    status = __Pyx__Import_GetModule(top_level_package_name, module);
    Py_DECREF(top_level_package_name);
    return status;
error:
    Py_DECREF(imported_module);
    *module = NULL;
    return -1;
not_found:
    Py_DECREF(imported_module);
    *module = NULL;
    return 0;
}
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *from_list = 0;
    int module_found;
    if (!qualname) {
        qualname = name;
    }
    module_found = __Pyx__Import_Lookup(qualname, imported_names, len_imported_names, &module);
    if (likely(module_found == 1)) {
        return module;
    } else if (unlikely(module_found == -1)) {
        return NULL;
    }
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    if (imported_names) {
#if CYTHON_COMPILING_IN_CPYTHON
        from_list = __Pyx_PyList_FromArray(imported_names, len_imported_names);
        if (unlikely(!from_list))
            goto bad;
#else
        from_list = PyList_New(len_imported_names);
        if (unlikely(!from_list)) goto bad;
        for (Py_ssize_t i=0; i<len_imported_names; ++i) {
            if (PyList_SetItem(from_list, i, __Pyx_NewRef(imported_names[i])) < 0) goto bad;
        }
#endif
    }
    if (level == -1) {
        const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
        if (package_sep != (0)) {
            module = PyImport_ImportModuleLevelObject(
                name, moddict, empty_dict, from_list, 1);
            if (unlikely(!module)) {
                if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                    goto bad;
                PyErr_Clear();
            }
        }
        level = 0;
    }
    if (!module) {
        module = PyImport_ImportModuleLevelObject(
            name, moddict, empty_dict, from_list, level);
    }
bad:
    Py_XDECREF(from_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level) {
    return __Pyx__Import(name, imported_names, len_imported_names, qualname, __pyx_mstate_global->__pyx_d, level);
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u__10);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                CYTHON_COMPILING_IN_GRAAL
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
    }
    return value;
}

/* decode_c_string */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* Py3UpdateBases */
static PyObject*
__Pyx_PEP560_update_bases(PyObject *bases)
{
    Py_ssize_t i, j, size_bases;
    PyObject *base = NULL, *meth, *new_base, *result, *new_bases = NULL;
#if CYTHON_ASSUME_SAFE_SIZE
    size_bases = PyTuple_GET_SIZE(bases);
#else
    size_bases = PyTuple_Size(bases);
    if (size_bases < 0) return NULL;
#endif
    for (i = 0; i < size_bases; i++) {
#if CYTHON_AVOID_BORROWED_REFS
        Py_CLEAR(base);
#endif
#if CYTHON_ASSUME_SAFE_MACROS
        base = PyTuple_GET_ITEM(bases, i);
#else
        base = PyTuple_GetItem(bases, i);
        if (!base) goto error;
#endif
#if CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(base);
#endif
        if (PyType_Check(base)) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        meth = __Pyx_PyObject_GetAttrStrNoError(base, __pyx_mstate_global->__pyx_n_u_mro_entries);
        if (!meth && PyErr_Occurred()) {
            goto error;
        }
        if (!meth) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        new_base = __Pyx_PyObject_CallOneArg(meth, bases);
        Py_DECREF(meth);
        if (!new_base) {
            goto error;
        }
        if (!PyTuple_Check(new_base)) {
            PyErr_SetString(PyExc_TypeError,
                            "__mro_entries__ must return a tuple");
            Py_DECREF(new_base);
            goto error;
        }
        if (!new_bases) {
            if (!(new_bases = PyList_New(i))) {
                goto error;
            }
            for (j = 0; j < i; j++) {
                PyObject *base_from_list;
#if CYTHON_ASSUME_SAFE_MACROS
                base_from_list = PyTuple_GET_ITEM(bases, j);
                PyList_SET_ITEM(new_bases, j, base_from_list);
                Py_INCREF(base_from_list);
#else
                base_from_list = PyTuple_GetItem(bases, j);
                if (!base_from_list) goto error;
                Py_INCREF(base_from_list);
                if (PyList_SetItem(new_bases, j, base_from_list) < 0) goto error;
#endif
            }
        }
#if CYTHON_ASSUME_SAFE_SIZE
        j = PyList_GET_SIZE(new_bases);
#else
        j = PyList_Size(new_bases);
        if (j < 0) goto error;
#endif
        if (PyList_SetSlice(new_bases, j, j, new_base) < 0) {
            goto error;
        }
        Py_DECREF(new_base);
    }
    if (!new_bases) {
        Py_INCREF(bases);
        return bases;
    }
    result = PyList_AsTuple(new_bases);
    Py_DECREF(new_bases);
#if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(base);
#endif
    return result;
error:
    Py_XDECREF(new_bases);
#if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(base);
#endif
    return NULL;
}

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases;
#if CYTHON_ASSUME_SAFE_SIZE
    nbases = PyTuple_GET_SIZE(bases);
#else
    nbases = PyTuple_Size(bases);
    if (nbases < 0) return NULL;
#endif
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
#if CYTHON_ASSUME_SAFE_MACROS
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *tmp = PyTuple_GetItem(bases, i);
        if (!tmp) return NULL;
#endif
        tmptype = Py_TYPE(tmp);
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
        metaclass = &PyType_Type;
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStrNoError(metaclass, __pyx_mstate_global->__pyx_n_u_prepare);
        if (prep) {
            PyObject *pargs[3] = {NULL, name, bases};
            ns = __Pyx_PyObject_FastCallDict(prep, pargs+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
            Py_DECREF(prep);
        } else {
            if (unlikely(PyErr_Occurred()))
                return NULL;
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_module_2, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result;
    PyObject *owned_metaclass = NULL;
    PyObject *margs[4] = {NULL, name, bases, dict};
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_mstate_global->__pyx_n_u_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    result = __Pyx_PyObject_FastCallDict(metaclass, margs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
    Py_XDECREF(owned_metaclass);
    return result;
}

/* dict_setdefault (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value) {
    PyObject* value;
#if __PYX_LIMITED_VERSION_HEX >= 0x030F0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4)
    PyDict_SetDefaultRef(d, key, default_value, &value);
#elif CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[] = {d, key, default_value};
    value = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_n_u_setdefault, args, 3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethodObjArgs(d, __pyx_mstate_global->__pyx_n_u_setdefault, key, default_value, NULL);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* AddModuleRef (used by FetchSharedCythonModule) */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  static PyObject *__Pyx_PyImport_AddModuleObjectRef(PyObject *name) {
      PyObject *module_dict = PyImport_GetModuleDict();
      PyObject *m;
      if (PyMapping_GetOptionalItem(module_dict, name, &m) < 0) {
          return NULL;
      }
      if (m != NULL && PyModule_Check(m)) {
          return m;
      }
      Py_XDECREF(m);
      m = PyModule_NewObject(name);
      if (m == NULL)
          return NULL;
      if (PyDict_CheckExact(module_dict)) {
          PyObject *new_m;
          (void)PyDict_SetDefaultRef(module_dict, name, m, &new_m);
          Py_DECREF(m);
          return new_m;
      } else {
           if (PyObject_SetItem(module_dict, name, m) != 0) {
                Py_DECREF(m);
                return NULL;
            }
            return m;
      }
  }
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *py_name = PyUnicode_FromString(name);
      if (!py_name) return NULL;
      PyObject *module = __Pyx_PyImport_AddModuleObjectRef(py_name);
      Py_DECREF(py_name);
      return module;
  }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif

/* FetchSharedCythonModule (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef(__PYX_ABI_MODULE_NAME);
}

/* FetchCommonType (used by CommonTypesMetaclass) */
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject* __Pyx_PyType_FromMetaclass(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *result = __Pyx_PyType_FromModuleAndSpec(module, spec, bases);
    if (result && metaclass) {
        PyObject *old_tp = (PyObject*)Py_TYPE(result);
    Py_INCREF((PyObject*)metaclass);
#if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        Py_SET_TYPE(result, metaclass);
#else
        result->ob_type = metaclass;
#endif
        Py_DECREF(old_tp);
    }
    return result;
}
#else
#define __Pyx_PyType_FromMetaclass(me, mo, s, b) PyType_FromMetaclass(me, mo, s, b)
#endif
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t expected_basicsize) {
    Py_ssize_t basicsize;
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (expected_basicsize == 0) {
        return 0; // size is inherited, nothing useful to check
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
    if (unlikely(!py_basicsize)) return -1;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = NULL;
    if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) return -1;
#else
    basicsize = ((PyTypeObject*) cached_type)->tp_basicsize;
#endif
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module = NULL, *cached_type = NULL, *abi_module_dict, *new_cached_type, *py_object_name;
    int get_item_ref_result;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    py_object_name = PyUnicode_FromString(object_name);
    if (!py_object_name) return NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) goto done;
    abi_module_dict = PyModule_GetDict(abi_module);
    if (!abi_module_dict) goto done;
    get_item_ref_result = __Pyx_PyDict_GetItemRef(abi_module_dict, py_object_name, &cached_type);
    if (get_item_ref_result == 1) {
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else if (unlikely(get_item_ref_result == -1)) {
        goto bad;
    }
    cached_type = __Pyx_PyType_FromMetaclass(
        metaclass,
        CYTHON_USE_MODULE_STATE ? module : abi_module,
        spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    new_cached_type = __Pyx_PyDict_SetDefault(abi_module_dict, py_object_name, cached_type);
    if (unlikely(new_cached_type != cached_type)) {
        if (unlikely(!new_cached_type)) goto bad;
        Py_DECREF(cached_type);
        cached_type = new_cached_type;
        if (__Pyx_VerifyCachedType(
                cached_type,
                object_name,
                spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else {
        Py_DECREF(new_cached_type);
    }
done:
    Py_XDECREF(abi_module);
    Py_DECREF(py_object_name);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CommonTypesMetaclass (used by CythonFunctionShared) */
static PyObject* __pyx_CommonTypesMetaclass_get_module(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED void* context) {
    return PyUnicode_FromString(__PYX_ABI_MODULE_NAME);
}
#if __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject* __pyx_CommonTypesMetaclass_call(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *args, CYTHON_UNUSED PyObject *kwds) {
    PyErr_SetString(PyExc_TypeError, "Cannot instantiate Cython internal types");
    return NULL;
}
static int __pyx_CommonTypesMetaclass_setattr(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *attr, CYTHON_UNUSED PyObject *value) {
    PyErr_SetString(PyExc_TypeError, "Cython internal types are immutable");
    return -1;
}
#endif
static PyGetSetDef __pyx_CommonTypesMetaclass_getset[] = {
    {"__module__", __pyx_CommonTypesMetaclass_get_module, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_CommonTypesMetaclass_slots[] = {
    {Py_tp_getset, (void *)__pyx_CommonTypesMetaclass_getset},
    #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {Py_tp_call, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_new, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_setattro, (void*)__pyx_CommonTypesMetaclass_setattr},
    #endif
    {0, 0}
};
static PyType_Spec __pyx_CommonTypesMetaclass_spec = {
    __PYX_TYPE_MODULE_PREFIX "_common_types_metatype",
    0,
    0,
    Py_TPFLAGS_IMMUTABLETYPE |
    Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT,
    __pyx_CommonTypesMetaclass_slots
};
static int __pyx_CommonTypesMetaclass_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    PyObject *bases = PyTuple_Pack(1, &PyType_Type);
    if (unlikely(!bases)) {
        return -1;
    }
    mstate->__pyx_CommonTypesMetaclassType = __Pyx_FetchCommonTypeFromSpec(NULL, module, &__pyx_CommonTypesMetaclass_spec, bases);
    Py_DECREF(bases);
    if (unlikely(mstate->__pyx_CommonTypesMetaclassType == NULL)) {
        return -1;
    }
    return 0;
}

/* PyMethodNew (used by CythonFunctionShared) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *result;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    #if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    {
        PyObject *args[] = {func, self};
        result = PyObject_Vectorcall(__pyx_mstate_global->__Pyx_CachedMethodType, args, 2, NULL);
    }
    #else
    result = PyObject_CallFunctionObjArgs(__pyx_mstate_global->__Pyx_CachedMethodType, func, self, NULL);
    #endif
    return result;
}
#else
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#endif

/* PyVectorcallFastCallDict (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i;
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos;
    #else
    Py_ssize_t pos;
    #endif
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    #if !CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t nkw = PyDict_Size(kw);
    if (unlikely(nkw == -1)) return NULL;
    #else
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    #endif
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = 0;
    i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (__Pyx_PyDict_NextRef(kw, &pos, &key, &value)) {
        keys_are_strings &=
        #if CYTHON_COMPILING_IN_LIMITED_API
            PyType_GetFlags(Py_TYPE(key));
        #else
            Py_TYPE(key)->tp_flags;
        #endif
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(PyTuple_SetItem(kwnames, i, key) < 0)) goto cleanup;
        #else
        PyTuple_SET_ITEM(kwnames, i, key);
        #endif
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(pos);
    #endif
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    Py_ssize_t kw_size =
        likely(kw == NULL) ?
        0 :
#if !CYTHON_ASSUME_SAFE_SIZE
        PyDict_Size(kw);
#else
        PyDict_GET_SIZE(kw);
#endif
    if (kw_size == 0) {
        return vc(func, args, nargs, NULL);
    }
#if !CYTHON_ASSUME_SAFE_SIZE
    else if (unlikely(kw_size == -1)) {
        return NULL;
    }
#endif
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared (used by CythonFunction) */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunctionNoMethod(PyObject *func, void (*cfunc)(void)) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if ((PyObject*)Py_TYPE(func) == __pyx_mstate_global->__Pyx_CachedMethodType) {
        int result;
        PyObject *newFunc = PyObject_GetAttr(func, __pyx_mstate_global->__pyx_n_u_func);
        if (unlikely(!newFunc)) {
            PyErr_Clear(); // It's only an optimization, so don't throw an error
            return 0;
        }
        result = __Pyx__IsSameCyOrCFunctionNoMethod(newFunc, cfunc);
        Py_DECREF(newFunc);
        return result;
    }
    return __Pyx__IsSameCyOrCFunctionNoMethod(func, cfunc);
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if (PyMethod_Check(func)) {
        func = PyMethod_GET_FUNCTION(func);
    }
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure) {
    PyObject *result;
    CYTHON_UNUSED_VAR(closure);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_doc_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#else
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_name_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    Py_INCREF(op->func_qualname);
    result = op->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
#endif
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_defaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_kwdefaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int __Pyx_CyFunction_set_annotate_in_dict_if_exists(PyObject *op_in, PyObject *value);
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    __Pyx_END_CRITICAL_SECTION();
    if (unlikely(__Pyx_CyFunction_set_annotate_in_dict_if_exists((PyObject*) op, Py_None) < 0)) return -1;
    return 0;
}
static int
__Pyx_CyFunction_get_dict_if_exists(PyObject *op_in, PyObject **dict) {
    /* Return 1 if the function dict exists, 0 otherwise.  This cannot fail:
     * _PyObject_GetDictPtr() may clear errors internally, but never reports them. */
#if CYTHON_COMPILING_IN_PYPY
    *dict = PyObject_GenericGetDict(op_in, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030C0000
    *dict = ((__pyx_CyFunctionObject*) op_in)->func_dict;
#else
    PyObject **dictptr = _PyObject_GetDictPtr(op_in);
    *dict = likely(dictptr) ? *dictptr : NULL;
#endif
    return *dict ? 1 : 0;
}
static int
__Pyx_CyFunction_get_annotate_from_dict_if_exists(PyObject *op_in, PyObject **annotate) {
    PyObject *dict;
    int dict_found;
    *annotate = NULL;
    dict_found = __Pyx_CyFunction_get_dict_if_exists(op_in, &dict);
    if (!dict_found) return 0;
    return __Pyx_PyDict_GetItemRef(dict, __pyx_mstate_global->__pyx_n_u_annotate, annotate);
}
static int
__Pyx_CyFunction_set_annotate_in_dict_if_exists(PyObject *op_in, PyObject *value) {
    PyObject *dict;
    int dict_found;
    dict_found = __Pyx_CyFunction_get_dict_if_exists(op_in, &dict);
    if (!dict_found) return 0;
    return PyDict_SetItem(dict, __pyx_mstate_global->__pyx_n_u_annotate, value);
}
static int
__Pyx_CyFunction_set_annotate_in_dict(PyObject *op_in, PyObject *value) {
    PyObject *dict;
    int result;
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    dict = __Pyx_CyFunction_get_dict((__pyx_CyFunctionObject*) op_in, NULL);
#else
    dict = PyObject_GenericGetDict(op_in, NULL);
#endif
    if (unlikely(!dict)) return -1;
    result = PyDict_SetItem(dict, __pyx_mstate_global->__pyx_n_u_annotate, value);
    Py_DECREF(dict);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations(PyObject *op_in, void *context) {
    PyObject *annotate = NULL;
    PyObject *result = NULL;
    __pyx_CyFunctionObject *op = (__pyx_CyFunctionObject*) op_in;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op_in);
    result = __Pyx_XNewRef(op->func_annotations);
    __Pyx_END_CRITICAL_SECTION();
    if (result) return result;
    if (unlikely(__Pyx_CyFunction_get_annotate_from_dict_if_exists(op_in, &annotate) < 0)) return NULL;
    if (!annotate || annotate == Py_None) {
        Py_XDECREF(annotate);
        __Pyx_BEGIN_CRITICAL_SECTION(op_in);
        result = __Pyx_CyFunction_get_annotations_locked(op);
        __Pyx_END_CRITICAL_SECTION();
        return result;
    }
    PyObject *format = PyLong_FromLong(1L);  // annotationlib.Format.VALUE
    if (likely(format)) {
        result = __Pyx_PyObject_CallOneArg(annotate, format);
        Py_DECREF(format);
    }
    Py_DECREF(annotate);
    if (unlikely(!result)) return NULL;
    if (unlikely(!PyDict_Check(result))) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ must return a dict");
        Py_DECREF(result);
        return NULL;
    }
    __Pyx_BEGIN_CRITICAL_SECTION(op_in);
    __Pyx_Py_XDECREF_SET(op->func_annotations, __Pyx_NewRef(result));
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyObject *__Pyx_CyFunction_annotate_impl(PyObject *self, PyObject *args) {
    CYTHON_UNUSED_VAR(args);
    if (unlikely(!self)) {
        PyErr_SetString(PyExc_SystemError, "cython __annotate__ called without 'self' argument");
    }
    Py_XINCREF(self);
    return self;
}
static PyMethodDef __Pyx_CyFunction_annotate_method = {
    "__annotate__",
    (PyCFunction)(void (*)(void))__Pyx_CyFunction_annotate_impl,
    METH_VARARGS,
    "Placeholder __annotate__ function to allow 'functools.wraps' to work "
    "on Cython functions."
};
static PyObject *
__Pyx_CyFunction_get_annotate(PyObject *op_in, void *context) {
    PyObject *annotate = NULL;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(__Pyx_CyFunction_get_annotate_from_dict_if_exists(op_in, &annotate) < 0)) return NULL;
    if (annotate) return annotate;
    PyObject *annotations = __Pyx_CyFunction_get_annotations(op_in, NULL);
    if (unlikely(!annotations)) return NULL;
    PyObject *method = PyCFunction_New(
        &__Pyx_CyFunction_annotate_method,
        annotations);
    Py_DECREF(annotations);
    return method;
}
static int
__Pyx_CyFunction_set_annotate(PyObject *op_in, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ cannot be deleted");
        return -1;
    }
    if (unlikely(value != Py_None && !PyCallable_Check(value))) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ must be callable or None");
        return -1;
    }
    if (value != Py_None) {
        __pyx_CyFunctionObject *op = (__pyx_CyFunctionObject*) op_in;
        __Pyx_BEGIN_CRITICAL_SECTION(op_in);
        Py_CLEAR(op->func_annotations);
        __Pyx_END_CRITICAL_SECTION();
    }
    return __Pyx_CyFunction_set_annotate_in_dict(op_in, value);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine_value(__pyx_CyFunctionObject *op) {
    int is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
    if (is_coroutine) {
        PyObject *is_coroutine_value, *module, *fromlist, *marker = __pyx_mstate_global->__pyx_n_u_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_mstate_global->__pyx_n_u_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        is_coroutine_value = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(is_coroutine_value)) {
            return is_coroutine_value;
        }
ignore:
        PyErr_Clear();
    }
    return __Pyx_PyBool_FromLong(is_coroutine);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    result = __Pyx_CyFunction_get_is_coroutine_value(op);
    if (unlikely(!result))
        return NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    if (op->func_is_coroutine) {
        Py_DECREF(result);
        result = __Pyx_NewRef(op->func_is_coroutine);
    } else {
        op->func_is_coroutine = __Pyx_NewRef(result);
    }
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static void __Pyx_CyFunction_raise_argument_count_error(__pyx_CyFunctionObject *func, const char* message, Py_ssize_t size) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        py_name, message, size);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        name, message, size);
#endif
}
static void __Pyx_CyFunction_raise_type_error(__pyx_CyFunctionObject *func, const char* message) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s",
        py_name, message);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s",
        name, message);
#endif
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {"func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {"func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
#else
    {"func_dict", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
#endif
    {"func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {"__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {"__annotate__", (getter)__Pyx_CyFunction_get_annotate, (setter)__Pyx_CyFunction_set_annotate, 0, 0},
    {"_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    {"__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#endif
#if CYTHON_METH_FASTCALL
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {"__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(args);
    __Pyx_BEGIN_CRITICAL_SECTION(m);
    Py_INCREF(m->func_qualname);
    result = m->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_dict = NULL;
#endif
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func_dict);
#elif PY_VERSION_HEX < 0x030d0000
    _PyObject_ClearManagedDict((PyObject*)m);
#else
    PyObject_ClearManagedDict((PyObject*)m);
#endif
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    Py_CLEAR(m->defaults);
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    {
        int e = __Pyx_call_type_traverse((PyObject*)m, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func_dict);
#else
    {
        int e =
#if PY_VERSION_HEX < 0x030d0000
            _PyObject_VisitManagedDict
#else
            PyObject_VisitManagedDict
#endif
                ((PyObject*)m, visit, arg);
        if (e != 0) return e;
    }
#endif
    __Pyx_VISIT_CONST(m->func_name);
    __Pyx_VISIT_CONST(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    __Pyx_VISIT_CONST(m->func_code);
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_annotations);
    Py_VISIT(m->func_is_coroutine);
    Py_VISIT(m->defaults);
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
    PyObject *repr;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    repr = PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
    __Pyx_END_CRITICAL_SECTION();
    return repr;
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void(*)(void))meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes no arguments", size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes exactly one argument", size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
    __Pyx_CyFunction_raise_type_error(
        (__pyx_CyFunctionObject*)func, "takes no keyword arguments");
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_SIZE
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(argc < 0)) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            __Pyx_CyFunction_raise_type_error(
                cyfunc, "needs an argument");
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(__Pyx_PyTuple_GET_SIZE(kwnames))) {
        __Pyx_CyFunction_raise_type_error(
            cyfunc, "takes no keyword arguments");
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes no arguments", nargs);
        return NULL;
    }
    return meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes exactly one argument", nargs);
        return NULL;
    }
    return meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    #if PY_VERSION_HEX < 0x030e00A6
    size_t nargs_value = (size_t) nargs;
    #else
    Py_ssize_t nargs_value = nargs;
    #endif
    return ((__Pyx_PyCMethod)(void(*)(void))meth)(self, cls, args, nargs_value, kwnames);
}
#endif
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if CYTHON_METH_FASTCALL
#if defined(Py_TPFLAGS_HAVE_VECTORCALL)
    Py_TPFLAGS_HAVE_VECTORCALL |
#elif defined(_Py_TPFLAGS_HAVE_VECTORCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
#endif // CYTHON_METH_FASTCALL
#if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_TPFLAGS_MANAGED_DICT |
#endif
    Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
static int __pyx_CyFunction_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_CyFunctionType_spec, NULL);
    if (unlikely(mstate->__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func, PyTypeObject *defaults_type) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_CallObject((PyObject*)defaults_type, NULL); // _PyObject_New(defaults_type);
    if (unlikely(!m->defaults))
        return NULL;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* CyFunctionClassCell */
static int __Pyx_CyFunction_InitClassCell(PyObject *cyfunctions, PyObject *classobj) {
    Py_ssize_t i, count = __Pyx_PyList_GET_SIZE(cyfunctions);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(count < 0)) return -1;
    #endif
    for (i = 0; i < count; i++) {
        __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *)
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && !CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
            PyList_GET_ITEM(cyfunctions, i);
#else
            __Pyx_PySequence_ITEM(cyfunctions, i);
        if (unlikely(!m))
            return -1;
#endif
        __Pyx_CyFunction_SetClassObj(m, classobj);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && !CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
        Py_DECREF((PyObject*)m);
#endif
    }
    return 0;
}

/* CLineInTraceback (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
#define __Pyx_PyProbablyModule_GetDict(o) __Pyx_XNewRef(PyModule_GetDict(o))
#elif !CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyProbablyModule_GetDict(o) PyObject_GenericGetDict(o, NULL);
#else
PyObject* __Pyx_PyProbablyModule_GetDict(PyObject *o) {
    PyObject **dict_ptr = _PyObject_GetDictPtr(o);
    return dict_ptr ? __Pyx_XNewRef(*dict_ptr) : NULL;
}
#endif
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline = NULL;
    PyObject *ptype, *pvalue, *ptraceback;
    PyObject *cython_runtime_dict;
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
    cython_runtime_dict = __Pyx_PyProbablyModule_GetDict(__pyx_mstate_global->__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, cython_runtime_dict,
            __Pyx_PyDict_SetDefault(cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False))
    }
    if (use_cline == NULL || use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    Py_XDECREF(use_cline);
    Py_XDECREF(cython_runtime_dict);
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache (used by AddTraceback) */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
    __Pyx_CachedCodeObjectType* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = code_cache->entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__find_code_object;
    return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
    if (old_count < 0) {
        __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
        return NULL;
    }
#endif
    __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
#endif
    return result;
#endif
}
static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
{
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            code_cache->entries = entries;
            code_cache->max_count = 64;
            code_cache->count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
        __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_INCREF(code_object);
        Py_DECREF(tmp);
        return;
    }
    if (code_cache->count == code_cache->max_count) {
        int new_max = code_cache->max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        code_cache->entries = entries;
        code_cache->max_count = new_max;
    }
    for (i=code_cache->count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    code_cache->count++;
    Py_INCREF(code_object);
}
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__insert_code_object;
    return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type expected = 0;
    if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
        return;
    }
#endif
    __pyx__insert_code_object(code_cache, code_line, code_object);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
#endif
#endif
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!code_object) {
        code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
        if (unlikely(!code_object)) goto bad;
        py_py_line = PyLong_FromLong(py_line);
        if (unlikely(!py_py_line)) goto bad;
        if (c_line) {
            py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        } else {
            py_funcname = PyUnicode_FromString(funcname);
        }
        if (unlikely(!py_funcname)) goto bad;
        dict = PyDict_New();
        if (unlikely(!dict)) goto bad;
        {
            PyObject *old_code_object = code_object;
            code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
            Py_DECREF(old_code_object);
        }
        if (unlikely(!code_object)) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
    } else {
        dict = PyDict_New();
    }
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    if (c_line) {
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
    }
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CheckUnpickleChecksum */
static void __Pyx_RaiseUnpickleChecksumError(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
    PyObject *pickle_module = PyImport_ImportModule("pickle");
    if (unlikely(!pickle_module)) return;
    PyObject *pickle_error = PyObject_GetAttrString(pickle_module, "PickleError");
    Py_DECREF(pickle_module);
    if (unlikely(!pickle_error)) return;
    if (checksum2 == checksum1) {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x) = (%s))",
            checksum, checksum1, members);
    } else if (checksum3 == checksum2) {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x) = (%s))",
            checksum, checksum1, checksum2, members);
    } else {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x, 0x%x) = (%s))",
            checksum, checksum1, checksum2, checksum3, members);
    }
    Py_DECREF(pickle_error);
}
static int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
    int found = 0;
    found |= checksum1 == checksum;
    found |= checksum2 == checksum;
    found |= checksum3 == checksum;
    if (likely(found))
        return 0;
    __Pyx_RaiseUnpickleChecksumError(checksum, checksum1, checksum2, checksum3, members);
    return -1;
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyLong_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint64_t(uint64_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint64_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(uint64_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uint64_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE uint64_t __Pyx_PyLong_As_uint64_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        uint64_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (uint64_t) -1;
        val = __Pyx_PyLong_As_uint64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint64_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(uint64_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) >= 2 * PyLong_SHIFT)) {
                            return (uint64_t) (((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint64_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) >= 3 * PyLong_SHIFT)) {
                            return (uint64_t) (((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint64_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) >= 4 * PyLong_SHIFT)) {
                            return (uint64_t) (((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (uint64_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(uint64_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint64_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(uint64_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint64_t) (((uint64_t)-1)*(((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(uint64_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint64_t) ((((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint64_t) (((uint64_t)-1)*(((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint64_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint64_t) ((((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint64_t) (((uint64_t)-1)*(((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint64_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint64_t) ((((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(uint64_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint64_t, long, PyLong_AsLong(x))
        } else if ((sizeof(uint64_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        uint64_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (uint64_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (uint64_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (uint64_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (uint64_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(uint64_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((uint64_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(uint64_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((uint64_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((uint64_t) 1) << (sizeof(uint64_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (uint64_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint64_t");
    return (uint64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint64_t");
    return (uint64_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int64_t __Pyx_PyLong_As_int64_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int64_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int64_t) -1;
        val = __Pyx_PyLong_As_int64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int64_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int64_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) >= 2 * PyLong_SHIFT)) {
                            return (int64_t) (((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int64_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) >= 3 * PyLong_SHIFT)) {
                            return (int64_t) (((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int64_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) >= 4 * PyLong_SHIFT)) {
                            return (int64_t) (((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int64_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int64_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int64_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int64_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (int64_t) (((int64_t)-1)*(((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int64_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (int64_t) ((((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (int64_t) (((int64_t)-1)*(((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int64_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (int64_t) ((((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (int64_t) (((int64_t)-1)*(((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int64_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (int64_t) ((((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int64_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int64_t, long, PyLong_AsLong(x))
        } else if ((sizeof(int64_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int64_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int64_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int64_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int64_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int64_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int64_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int64_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int64_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int64_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int64_t) 1) << (sizeof(int64_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int64_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int64_t");
    return (int64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int64_t");
    return (int64_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyLong_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int32_t(int32_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int32_t neg_one = (int32_t) -1, const_zero = (int32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int32_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(int32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int32_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int32_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int32_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE uint32_t __Pyx_PyLong_As_uint32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyLong_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(uint32_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT)) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint32_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT)) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint32_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT)) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (uint32_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(uint32_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(uint32_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint32_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint32_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(uint32_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
        } else if ((sizeof(uint32_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        uint32_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (uint32_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (uint32_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (uint32_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (uint32_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(uint32_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((uint32_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(uint32_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((uint32_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((uint32_t) 1) << (sizeof(uint32_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (uint32_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyLong_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 2 * PyLong_SHIFT)) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 3 * PyLong_SHIFT)) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 4 * PyLong_SHIFT)) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (size_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(size_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(size_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
        } else if ((sizeof(size_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        size_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (size_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (size_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (size_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (size_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(size_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((size_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(size_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((size_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((size_t) 1) << (sizeof(size_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (size_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint32_t(uint32_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint32_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint32_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(uint32_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uint32_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE int16_t __Pyx_PyLong_As_int16_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int16_t neg_one = (int16_t) -1, const_zero = (int16_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int16_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int16_t) -1;
        val = __Pyx_PyLong_As_int16_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int16_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int16_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int16_t) >= 2 * PyLong_SHIFT)) {
                            return (int16_t) (((((int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int16_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int16_t) >= 3 * PyLong_SHIFT)) {
                            return (int16_t) (((((((int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int16_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int16_t) >= 4 * PyLong_SHIFT)) {
                            return (int16_t) (((((((((int16_t)digits[3]) << PyLong_SHIFT) | (int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int16_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int16_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int16_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int16_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int16_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int16_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int16_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int16_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int16_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (int16_t) (((int16_t)-1)*(((((int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int16_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int16_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (int16_t) ((((((int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int16_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int16_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int16_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (int16_t) (((int16_t)-1)*(((((((int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int16_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int16_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (int16_t) ((((((((int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int16_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int16_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int16_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (int16_t) (((int16_t)-1)*(((((((((int16_t)digits[3]) << PyLong_SHIFT) | (int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int16_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int16_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int16_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (int16_t) ((((((((((int16_t)digits[3]) << PyLong_SHIFT) | (int16_t)digits[2]) << PyLong_SHIFT) | (int16_t)digits[1]) << PyLong_SHIFT) | (int16_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int16_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int16_t, long, PyLong_AsLong(x))
        } else if ((sizeof(int16_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int16_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int16_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int16_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int16_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int16_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int16_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int16_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int16_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int16_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int16_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int16_t) 1) << (sizeof(int16_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int16_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int16_t");
    return (int16_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int16_t");
    return (int16_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int32_t __Pyx_PyLong_As_int32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int32_t neg_one = (int32_t) -1, const_zero = (int32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int32_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int32_t) -1;
        val = __Pyx_PyLong_As_int32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int32_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int32_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int32_t) >= 2 * PyLong_SHIFT)) {
                            return (int32_t) (((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int32_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int32_t) >= 3 * PyLong_SHIFT)) {
                            return (int32_t) (((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int32_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int32_t) >= 4 * PyLong_SHIFT)) {
                            return (int32_t) (((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int32_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int32_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int32_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int32_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int32_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (int32_t) (((int32_t)-1)*(((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int32_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (int32_t) ((((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (int32_t) (((int32_t)-1)*(((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int32_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (int32_t) ((((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (int32_t) (((int32_t)-1)*(((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int32_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (int32_t) ((((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int32_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int32_t, long, PyLong_AsLong(x))
        } else if ((sizeof(int32_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int32_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int32_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int32_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int32_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int32_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int32_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int32_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int32_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int32_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int32_t) 1) << (sizeof(int32_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int32_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int32_t");
    return (int32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int32_t");
    return (int32_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE uint8_t __Pyx_PyLong_As_uint8_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        uint8_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (uint8_t) -1;
        val = __Pyx_PyLong_As_uint8_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint8_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(uint8_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint8_t) >= 2 * PyLong_SHIFT)) {
                            return (uint8_t) (((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint8_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint8_t) >= 3 * PyLong_SHIFT)) {
                            return (uint8_t) (((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint8_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint8_t) >= 4 * PyLong_SHIFT)) {
                            return (uint8_t) (((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (uint8_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(uint8_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint8_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(uint8_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint8_t) (((uint8_t)-1)*(((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(uint8_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint8_t) ((((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint8_t) (((uint8_t)-1)*(((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint8_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint8_t) ((((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint8_t) (((uint8_t)-1)*(((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint8_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint8_t) ((((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(uint8_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint8_t, long, PyLong_AsLong(x))
        } else if ((sizeof(uint8_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint8_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        uint8_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (uint8_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (uint8_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (uint8_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (uint8_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(uint8_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((uint8_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(uint8_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((uint8_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((uint8_t) 1) << (sizeof(uint8_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (uint8_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint8_t");
    return (uint8_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint8_t");
    return (uint8_t) -1;
}

/* UpdateUnpickledDict */
static int __Pyx__UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
    PyObject *state_dict = __Pyx_PySequence_ITEM(state, index);
    if (unlikely(!state_dict)) {
        return -1;
    }
    int non_empty = PyObject_IsTrue(state_dict);
    if (non_empty == 0) {
        Py_DECREF(state_dict);
        return 0;
    } else if (unlikely(non_empty == -1)) {
        return -1;
    }
    PyObject *dict;
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    dict = PyObject_GetAttrString(obj, "__dict__");
    #else
    dict = PyObject_GenericGetDict(obj, NULL);
    #endif
    if (unlikely(!dict)) {
        Py_DECREF(state_dict);
        return -1;
    }
    int result;
    if (likely(PyDict_CheckExact(dict))) {
        result = PyDict_Update(dict, state_dict);
    } else {
        PyObject *obj_result = __Pyx_PyObject_CallMethod1(dict, __pyx_mstate_global->__pyx_n_u_update, state_dict);
        if (likely(obj_result)) {
            Py_DECREF(obj_result);
            result = 0;
        } else {
            result = -1;
        }
    }
    Py_DECREF(state_dict);
    Py_DECREF(dict);
    return result;
}
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
    Py_ssize_t state_size = __Pyx_PyTuple_GET_SIZE(state);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(state_size == -1)) return -1;
    #endif
    if (state_size <= index) {
        return 0;
    }
    return __Pyx__UpdateUnpickledDict(obj, state, index);
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
static __Pyx_TypeName
__Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
{
    PyObject *module = NULL, *name = NULL, *result = NULL;
    #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
    name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_qualname);
    #else
    name = PyType_GetQualName(tp);
    #endif
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
    module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_module_2);
    if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
    if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
        result = name;
        name = NULL;
        goto done;
    }
    result = PyUnicode_FromFormat("%U.%U", module, name);
    if (unlikely(result == NULL)) goto bad;
  done:
    Py_XDECREF(name);
    Py_XDECREF(module);
    return result;
  bad:
    PyErr_Clear();
    if (name) {
        result = name;
        name = NULL;
    } else {
        result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__11);
    }
    goto done;
}
#endif

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* GetRuntimeVersion */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
void __Pyx_init_runtime_version(void) {
    if (__Pyx_cached_runtime_version == 0) {
        const char* rt_version = Py_GetVersion();
        unsigned long version = 0;
        unsigned long factor = 0x01000000UL;
        unsigned int digit = 0;
        int i = 0;
        while (factor) {
            while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                ++i;
            }
            version += factor * digit;
            if (rt_version[i] != '.')
                break;
            digit = 0;
            factor >>= 8;
            ++i;
        }
        __Pyx_cached_runtime_version = version;
    }
}
#endif
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    return Py_Version & ~0xFFUL;
#else
    return __Pyx_cached_runtime_version;
#endif
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* NewCodeObj */
#if CYTHON_COMPILING_IN_LIMITED_API
    static PyObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                       PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                       PyObject *fv, PyObject *cell, PyObject* fn,
                                       PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
#elif PY_VERSION_HEX >= 0x030B0000
  static PyCodeObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                         PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                         PyObject *fv, PyObject *cell, PyObject* fn,
                                         PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, __pyx_mstate_global->__pyx_empty_bytes);
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030c00A1
    if (likely(result))
        result->_co_firsttraceable = 0;
    #endif
    return result;
  }
#elif !CYTHON_COMPILING_IN_PYPY
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
) {
    PyObject *code_obj = NULL, *varnames_tuple_dedup = NULL, *code_bytes = NULL;
    Py_ssize_t var_count = (Py_ssize_t) descr.nlocals;
    PyObject *varnames_tuple = PyTuple_New(var_count);
    if (unlikely(!varnames_tuple)) return NULL;
    for (Py_ssize_t i=0; i < var_count; i++) {
        Py_INCREF(varnames[i]);
        if (__Pyx_PyTuple_SET_ITEM(varnames_tuple, i, varnames[i]) != (0)) goto done;
    }
    #if CYTHON_COMPILING_IN_LIMITED_API
    varnames_tuple_dedup = PyDict_GetItem(tuple_dedup_map, varnames_tuple);
    if (!varnames_tuple_dedup) {
        if (unlikely(PyDict_SetItem(tuple_dedup_map, varnames_tuple, varnames_tuple) < 0)) goto done;
        varnames_tuple_dedup = varnames_tuple;
    }
    #else
    varnames_tuple_dedup = PyDict_SetDefault(tuple_dedup_map, varnames_tuple, varnames_tuple);
    if (unlikely(!varnames_tuple_dedup)) goto done;
    #endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(varnames_tuple_dedup);
    #endif
    if (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table != NULL && !CYTHON_COMPILING_IN_GRAAL) {
        Py_ssize_t line_table_length = __Pyx_PyBytes_GET_SIZE(line_table);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(line_table_length == -1)) goto done;
        #endif
        Py_ssize_t code_len = (line_table_length * 2 + 4) & ~3LL;
        code_bytes = PyBytes_FromStringAndSize(NULL, code_len);
        if (unlikely(!code_bytes)) goto done;
        char* c_code_bytes = PyBytes_AsString(code_bytes);
        if (unlikely(!c_code_bytes)) goto done;
        memset(c_code_bytes, 0, (size_t) code_len);
    }
    code_obj = (PyObject*) __Pyx__PyCode_New(
        (int) descr.argcount,
        (int) descr.num_posonly_args,
        (int) descr.num_kwonly_args,
        (int) descr.nlocals,
        0,
        (int) descr.flags,
        code_bytes ? code_bytes : __pyx_mstate_global->__pyx_empty_bytes,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        varnames_tuple_dedup,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        filename,
        funcname,
        (int) descr.first_line,
        (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table) ? line_table : __pyx_mstate_global->__pyx_empty_bytes
    );
done:
    Py_XDECREF(code_bytes);
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(varnames_tuple_dedup);
    #endif
    Py_DECREF(varnames_tuple);
    return code_obj;
}

/* DecompressString */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo) {
    PyObject *module = NULL, *decompress, *compressed_bytes, *decompressed;
    const char* module_name = algo == 3 ? "compression.zstd" : algo == 2 ? "bz2" : "zlib";
    PyObject *methodname = PyUnicode_FromString("decompress");
    if (unlikely(!methodname)) return NULL;
    #if __PYX_LIMITED_VERSION_HEX >= 0x030e0000
    if (algo == 3) {
        PyObject *fromlist = Py_BuildValue("[O]", methodname);
        if (unlikely(!fromlist)) goto bad;
        module = PyImport_ImportModuleLevel("compression.zstd", NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
    } else
    #endif
        module = PyImport_ImportModule(module_name);
    if (unlikely(!module)) goto import_failed;
    decompress = PyObject_GetAttr(module, methodname);
    if (unlikely(!decompress)) goto import_failed;
    {
        #ifdef __cplusplus
            char *memview_bytes = const_cast<char*>(s);
        #else
            #if defined(__clang__)
              #pragma clang diagnostic push
              #pragma clang diagnostic ignored "-Wcast-qual"
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic push
              #pragma GCC diagnostic ignored "-Wcast-qual"
            #endif
            char *memview_bytes = (char*) s;
            #if defined(__clang__)
              #pragma clang diagnostic pop
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic pop
            #endif
        #endif
        #if CYTHON_COMPILING_IN_LIMITED_API && !defined(PyBUF_READ)
        int memview_flags = 0x100;
        #else
        int memview_flags = PyBUF_READ;
        #endif
        compressed_bytes = PyMemoryView_FromMemory(memview_bytes, length, memview_flags);
    }
    if (unlikely(!compressed_bytes)) {
        Py_DECREF(decompress);
        goto bad;
    }
    decompressed = PyObject_CallFunctionObjArgs(decompress, compressed_bytes, NULL);
    Py_DECREF(compressed_bytes);
    Py_DECREF(decompress);
    Py_DECREF(module);
    Py_DECREF(methodname);
    return decompressed;
import_failed:
    PyErr_Format(PyExc_ImportError,
        "Failed to import '%.20s.decompress' - cannot initialise module strings. "
        "String compression was configured with the C macro 'CYTHON_COMPRESS_STRINGS=%d'.",
        module_name, algo);
bad:
    Py_XDECREF(module);
    Py_DECREF(methodname);
    return NULL;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return __Pyx_NewRef(b ? Py_True: Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


/* MultiPhaseInitModuleState */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
#ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
#else
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
#endif
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
#error "Module state with PEP489 requires atomics. Currently that's one of\
 C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
#endif
#if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#define __Pyx_ModuleStateLookup_Lock()
#define __Pyx_ModuleStateLookup_Unlock()
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
#define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(__cplusplus) && __cplusplus >= 201103L
#include <mutex>
static std::mutex __Pyx_ModuleStateLookup_mutex;
#define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
#define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
#include <threads.h>
static mtx_t __Pyx_ModuleStateLookup_mutex;
static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
    mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
}
#define __Pyx_ModuleStateLookup_Lock()\
  call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
  mtx_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(HAVE_PTHREAD_H)
#include <pthread.h>
static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
#define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(_WIN32)
#include <Windows.h>  // synchapi.h on its own doesn't work
static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
#define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#else
#error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
 Requires C standard >= C11, or C++ standard >= C++11,\
 or pthreads, or the Windows 32 API, or Python >= 3.13."
#endif
typedef struct {
    int64_t id;
    PyObject *module;
} __Pyx_InterpreterIdAndModule;
typedef struct {
    char interpreter_id_as_index;
    Py_ssize_t count;
    Py_ssize_t allocated;
    __Pyx_InterpreterIdAndModule table[1];
} __Pyx_ModuleStateLookupData;
#define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
#else
static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
#endif
static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
        __Pyx_InterpreterIdAndModule* table,
        Py_ssize_t count,
        int64_t interpreterId) {
    __Pyx_InterpreterIdAndModule* begin = table;
    __Pyx_InterpreterIdAndModule* end = begin + count;
    if (begin->id == interpreterId) {
        return begin;
    }
    while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
        __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
        if (halfway->id == interpreterId) {
            return halfway;
        }
        if (halfway->id < interpreterId) {
            begin = halfway;
        } else {
            end = halfway;
        }
    }
    for (; begin < end; ++begin) {
        if (begin->id >= interpreterId) return begin;
    }
    return begin;
}
static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return NULL;
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
    {
        __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        if (likely(data)) {
            __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
            if (likely(data == new_data)) {
                goto read_finished;
            }
        }
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        __Pyx_ModuleStateLookup_Lock();
        __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
        data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
        __Pyx_ModuleStateLookup_Unlock();
    }
  read_finished:;
#else
    __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_InterpreterIdAndModule* found = NULL;
    if (unlikely(!data)) goto end;
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            found = data->table+interpreter_id;
        }
    } else {
        found = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
    }
  end:
    {
        PyObject *result=NULL;
        if (found && found->id == interpreter_id) {
            result = found->module;
        }
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
#endif
        return result;
    }
}
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
    while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
}
#else
#define __Pyx_ModuleStateLookup_wait_until_no_readers()
#endif
static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
    Py_ssize_t to_allocate = (*old_data)->allocated;
    while (to_allocate <= interpreter_id) {
        if (to_allocate == 0) to_allocate = 1;
        else to_allocate *= 2;
    }
    __Pyx_ModuleStateLookupData *new_data = *old_data;
    if (to_allocate != (*old_data)->allocated) {
         new_data = (__Pyx_ModuleStateLookupData *)realloc(
            *old_data,
            sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
        if (!new_data) {
            PyErr_NoMemory();
            return -1;
        }
        for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
            new_data->table[i].id = i;
            new_data->table[i].module = NULL;
        }
        new_data->allocated = to_allocate;
    }
    new_data->table[interpreter_id].module = module;
    if (new_data->count < interpreter_id+1) {
        new_data->count = interpreter_id+1;
    }
    *old_data = new_data;
    return 0;
}
static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
    __Pyx_InterpreterIdAndModule *read = data->table;
    __Pyx_InterpreterIdAndModule *write = data->table;
    __Pyx_InterpreterIdAndModule *end = read + data->count;
    for (; read<end; ++read) {
        if (read->module) {
            write->id = read->id;
            write->module = read->module;
            ++write;
        }
    }
    data->count = write - data->table;
    for (; write<end; ++write) {
        write->id = 0;
        write->module = NULL;
    }
    data->interpreter_id_as_index = 0;
}
static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    int result = 0;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_ModuleStateLookupData *new_data = old_data;
    if (!new_data) {
        new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
        if (!new_data) {
            result = -1;
            PyErr_NoMemory();
            goto end;
        }
        new_data->allocated = 1;
        new_data->interpreter_id_as_index = 1;
    }
    __Pyx_ModuleStateLookup_wait_until_no_readers();
    if (new_data->interpreter_id_as_index) {
        if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
            result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
            goto end;
        }
        __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
    }
    {
        Py_ssize_t insert_at = 0;
        {
            __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                new_data->table, new_data->count, interpreter_id);
            assert(lower_bound);
            insert_at = lower_bound - new_data->table;
            if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                lower_bound->module = module;
                goto end;  // already in table, nothing more to do
            }
        }
        if (new_data->count+1 >= new_data->allocated) {
            Py_ssize_t to_allocate = (new_data->count+1)*2;
            new_data =
                (__Pyx_ModuleStateLookupData*)realloc(
                    new_data,
                    sizeof(__Pyx_ModuleStateLookupData) +
                    (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
            if (!new_data) {
                result = -1;
                new_data = old_data;
                PyErr_NoMemory();
                goto end;
            }
            new_data->allocated = to_allocate;
        }
        ++new_data->count;
        int64_t last_id = interpreter_id;
        PyObject *last_module = module;
        for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
            int64_t current_id = new_data->table[i].id;
            new_data->table[i].id = last_id;
            last_id = current_id;
            PyObject *current_module = new_data->table[i].module;
            new_data->table[i].module = last_module;
            last_module = current_module;
        }
    }
  end:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
#else
    __Pyx_ModuleStateLookup_data = new_data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return result;
}
static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
#endif
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            data->table[interpreter_id].module = NULL;
        }
        goto done;
    }
    {
        __Pyx_ModuleStateLookup_wait_until_no_readers();
        __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
        if (!lower_bound) goto done;
        if (lower_bound->id != interpreter_id) goto done;
        __Pyx_InterpreterIdAndModule *end = data->table+data->count;
        for (;lower_bound<end-1; ++lower_bound) {
            lower_bound->id = (lower_bound+1)->id;
            lower_bound->module = (lower_bound+1)->module;
        }
    }
    --data->count;
    if (data->count == 0) {
        free(data);
        data = NULL;
    }
  done:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
#else
    __Pyx_ModuleStateLookup_data = data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return 0;
}
#endif

/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
