/* Generated by Cython 3.2.9 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/build/nvtx/src/nvtx/c/include/nvtx3/nvToolsExt.h",
            "/build/nvtx/src/nvtx/c/include/nvtx3/nvToolsExtCounters.h",
            "/build/nvtx/src/nvtx/c/include/nvtx3/nvToolsExtPayload.h",
            "/build/nvtx/src/nvtx/c/include/nvtx3/nvToolsExtSemanticsCounters.h",
            "/build/nvtx/src/nvtx/c/include/nvtx3/nvToolsExtSemanticsTime.h"
        ],
        "include_dirs": [
            "/build/nvtx/src/nvtx/c/include"
        ],
        "name": "nvtx._lib.lib",
        "sources": [
            "src/nvtx/_lib/lib.pyx"
        ]
    },
    "module_name": "nvtx._lib.lib"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API)
  #if !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef _MSC_VER
  #pragma message ("Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.")
  #else
  #warning Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_2_9"
#define CYTHON_HEX_VERSION 0x030209F0
#define CYTHON_FUTURE_DIVISION 1
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #ifndef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 1
  #endif
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
  #if defined(CYTHON_IMMORTAL_CONSTANTS) && PY_VERSION_HEX < 0x030C0000
    #undef CYTHON_IMMORTAL_CONSTANTS
    #define CYTHON_IMMORTAL_CONSTANTS 0  // definitely won't work
  #elif !defined(CYTHON_IMMORTAL_CONSTANTS)
    #define CYTHON_IMMORTAL_CONSTANTS (PY_VERSION_HEX >= 0x030C0000 && !CYTHON_USE_MODULE_STATE && CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_COMPRESS_STRINGS
  #define CYTHON_COMPRESS_STRINGS 1
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL)
#endif
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#include <stdint.h>
typedef uintptr_t  __pyx_uintptr_t;
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CInitCode */
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef Py_TPFLAGS_IMMUTABLETYPE
  #define Py_TPFLAGS_IMMUTABLETYPE (1UL << 8)
#endif
#ifndef Py_TPFLAGS_DISALLOW_INSTANTIATION
  #define Py_TPFLAGS_DISALLOW_INSTANTIATION (1UL << 7)
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if PY_VERSION_HEX < 0x03090000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) GraalPyFrame_SetLineNumber((frame), (lineno))
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && PyUnstable_Object_IsUniquelyReferenced(obj)) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#elif CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
enum __Pyx_ReferenceSharing {
  __Pyx_ReferenceSharing_DefinitelyUnique, // We created it so we know it's unshared - no need to check
  __Pyx_ReferenceSharing_OwnStrongReference,
  __Pyx_ReferenceSharing_FunctionArgument,
  __Pyx_ReferenceSharing_SharedReference, // Never trust it to be unshared because it's a global or similar
};
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && PY_VERSION_HEX >= 0x030E0000
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing)\
    (sharing == __Pyx_ReferenceSharing_DefinitelyUnique ? 1 :\
      (sharing == __Pyx_ReferenceSharing_FunctionArgument ? PyUnstable_Object_IsUniqueReferencedTemporary(o) :\
      (sharing == __Pyx_ReferenceSharing_OwnStrongReference ? PyUnstable_Object_IsUniquelyReferenced(o) : 0)))
#elif (CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING) || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)sharing), Py_REFCNT(o) == 1)
#else
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)o), ((void)sharing), 0)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyList_GetItemRef(o, i) (likely((i) >= 0) ? PySequence_GetItem(o, i) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_ITEM(o, i)
  #endif
#elif CYTHON_COMPILING_IN_LIMITED_API || !(CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE)
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_XNewRef(PyList_GetItem(o, i))
#else
  #define __Pyx_PyList_GetItemRef(o, i) (likely(__Pyx_is_valid_index(i, PyList_GET_SIZE(o))) ?\
    __Pyx_NewRef(PyList_GET_ITEM(o, i)) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
#endif
#if CYTHON_AVOID_BORROWED_REFS || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared)  ((void)(unsafe_shared),\
      __Pyx_PyList_GetItemRef(o, i))
#elif CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared) (\
      __Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
      __Pyx_NewRef(PyList_GET_ITEM(o, i)) : __Pyx_PyList_GetItemRef(o, i))
  #else
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared) (\
      __Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
      __Pyx_XNewRef(PyList_GetItem(o, i)) : __Pyx_PyList_GetItemRef(o, i))
  #endif
#elif CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared)  ((void)(unsafe_shared),\
      __Pyx_NewRef(PyList_GET_ITEM(o, i)))
#else
  #define __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared)  ((void)(unsafe_shared),\
      __Pyx_XNewRef(PyList_GetItem(o, i)))
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__nvtx___lib__lib
#define __PYX_HAVE_API__nvtx___lib__lib
/* Early includes */
#include <stdint.h>
#include <stddef.h>
#include "nvtx3/nvToolsExt.h"
#include "nvtx3/nvToolsExtPayload.h"
#include <string.h>
#include <stdlib.h>
#include "nvtx3/nvToolsExtCounters.h"
#include "nvtx3/nvToolsExtSemanticsCounters.h"
#include "nvtx3/nvToolsExtSemanticsTime.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifdef CYTHON_FREETHREADING_COMPATIBLE
#if CYTHON_FREETHREADING_COMPATIBLE
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_USED
#endif
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#endif
#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#if __cplusplus > 201103L
#include <type_traits>
#endif
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "src/nvtx/_lib/lib.pyx",
  "(tree fragment)",
  "src/nvtx/_lib/counters.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* Atomics.proto (used by UnpackUnboundCMethod) */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_nonatomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_nonatomic_ptr_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #define __pyx_nonatomic_ptr_type void*
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_atomic_ptr_type old = _InterlockedCompareExchangePointer(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif

/* CriticalSectionsDefinition.proto (used by CriticalSections) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_End(cs)
#define __Pyx_PyCriticalSection2_End(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_End PyCriticalSection_End
#define __Pyx_PyCriticalSection2_End PyCriticalSection2_End
#endif

/* CriticalSections.proto (used by ParseKeywordsImpl) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection_Begin(cs, arg) (void)(cs)
#define __Pyx_PyCriticalSection2_Begin(cs, arg1, arg2) (void)(cs)
#else
#define __Pyx_PyCriticalSection_Begin PyCriticalSection_Begin
#define __Pyx_PyCriticalSection2_Begin PyCriticalSection2_Begin
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* IncludeStructmemberH.proto (used by FixUpExtensionType) */
#include <structmember.h>

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_4nvtx_4_lib_8counters_Counter;
struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes;
struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar;
struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle;
struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle;
struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar;
struct __pyx_defaults;
struct __pyx_defaults1;
struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter;

/* "nvtx/_lib/counters.pxd":112
 *         nvtxCounterLimit_t max
 * 
 * cdef class Counter:             # <<<<<<<<<<<<<<
 *     cdef readonly object domain
 *     cdef readonly object name
*/
struct __pyx_obj_4nvtx_4_lib_8counters_Counter {
  PyObject_HEAD
  PyObject *domain;
  PyObject *name;
  PyObject *dtype;
  PyObject *description;
  PyObject *scope;
  PyObject *semantics;
  uint64_t time_domain;
  uint64_t _counter_id;
  uint64_t _schema_id;
};


/* "nvtx/_lib/lib.pxd":250
 *     )
 * 
 * cdef class EventAttributes:             # <<<<<<<<<<<<<<
 *     cdef object domain
 *     cdef object _message
*/
struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes {
  PyObject_HEAD
  struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes *__pyx_vtab;
  PyObject *domain;
  PyObject *_message;
  PyObject *_color;
  uint32_t _category;
  PyObject *_payload;
  nvtxStringHandle_t string_handle;
  nvtxEventAttributes_t c_obj;
  nvtxPayloadData_t _payload_data;
  PyObject *_payload_object;
  void *_allocated_payload;
};


/* "nvtx/_lib/lib.pxd":269
 * 
 * 
 * cdef class SchemaRegistrar:             # <<<<<<<<<<<<<<
 *     cdef dict _dtype_cache
 *     cdef dict _array_cache
*/
struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar {
  PyObject_HEAD
  struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_vtab;
  PyObject *_dtype_cache;
  PyObject *_array_cache;
  PyObject *_fixed_cache;
};


/* "nvtx/_lib/lib.pxd":278
 * 
 * 
 * cdef class DomainHandle:             # <<<<<<<<<<<<<<
 *     cdef bytes _name
 *     cdef nvtxDomainHandle_t c_obj
*/
struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle {
  PyObject_HEAD
  PyObject *_name;
  nvtxDomainHandle_t c_obj;
};


/* "nvtx/_lib/lib.pxd":282
 *     cdef nvtxDomainHandle_t c_obj
 * 
 * cdef class StringHandle:             # <<<<<<<<<<<<<<
 *     cdef bytes _string
 *     cdef nvtxStringHandle_t c_obj
*/
struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle {
  PyObject_HEAD
  PyObject *_string;
  nvtxStringHandle_t c_obj;
};


/* "nvtx/_lib/lib.pyx":766
 * 
 * 
 * cdef class _GlobalSchemaRegistrar(SchemaRegistrar):             # <<<<<<<<<<<<<<
 *     """Registers schemas via the global ``nvtxPayloadSchemaRegister``."""
 * 
*/
struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar {
  struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar __pyx_base;
  nvtxDomainHandle_t _domain;
};


/* "nvtx/_lib/lib.pyx":484
 *         pass
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/
struct __pyx_defaults {
  PyObject_HEAD
  PyObject *arg0;
};


/* "nvtx/_lib/lib.pyx":866
 *         return EventAttributes(self, message, color, category, payload)
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,             # <<<<<<<<<<<<<<
 *                              message=DONT_SET, color=DONT_SET, category=DONT_SET, payload=DONT_SET):
 *         """
*/
struct __pyx_defaults1 {
  PyObject_HEAD
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
};


/* "nvtx/_lib/lib.pyx":242
 * 
 * 
 * def payload_setter(type):             # <<<<<<<<<<<<<<
 *     """
 *     A helper decorator to register a payload setter for a given type.
*/
struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter {
  PyObject_HEAD
  PyObject *__pyx_v_type;
};



/* "nvtx/_lib/lib.pyx":252
 * 
 * 
 * cdef class EventAttributes:             # <<<<<<<<<<<<<<
 *     """
 *     A wrapper class for ``nvtxEventAttributes_t`` C struct.
*/

struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes {
  PyObject *(*_set_binary_payload)(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *, void *, uint64_t, size_t, size_t);
  PyObject *(*_clear_payload)(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *);
};
static struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes *__pyx_vtabptr_4nvtx_4_lib_3lib_EventAttributes;


/* "nvtx/_lib/lib.pyx":508
 * DONT_SET = object()
 * 
 * cdef class SchemaRegistrar:             # <<<<<<<<<<<<<<
 *     """
 *     Builds payload schemas from NumPy dtypes and registers them, caching so
*/

struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar {
  uint64_t (*_do_register)(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *, nvtxPayloadSchemaAttr_t const *);
};
static struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_vtabptr_4nvtx_4_lib_3lib_SchemaRegistrar;


/* "nvtx/_lib/lib.pyx":766
 * 
 * 
 * cdef class _GlobalSchemaRegistrar(SchemaRegistrar):             # <<<<<<<<<<<<<<
 *     """Registers schemas via the global ``nvtxPayloadSchemaRegister``."""
 * 
*/

struct __pyx_vtabstruct_4nvtx_4_lib_3lib__GlobalSchemaRegistrar {
  struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar __pyx_base;
};
static struct __pyx_vtabstruct_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *__pyx_vtabptr_4nvtx_4_lib_3lib__GlobalSchemaRegistrar;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto (used by PyErrFetchRestore) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto (used by GetBuiltinName) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PyObjectCall.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto (used by PyObjectFastCallMethod) */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* TupleAndListFromArray.proto (used by fastcall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto (used by BytesEquals) */
#include <string.h>

/* BytesEquals.proto (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL __Pyx_ArgsSlice_VARARGS
#endif

/* py_dict_items.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* CallCFunction.proto (used by CallUnboundCMethod0) */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* PyObjectCallOneArg.proto (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* UnpackUnboundCMethod.proto (used by CallUnboundCMethod0) */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod0.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* py_dict_values.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* OwnedDictNext.proto (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue);
#else
CYTHON_INLINE
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue);
#endif

/* RaiseDoubleKeywords.proto (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywordsImpl.export */
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name
);
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyDictVersioning.proto (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __Pyx_XNewRef(__pyx_dict_cached_value);\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyRuntimeError_Check.proto */
#define __Pyx_PyExc_RuntimeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_RuntimeError)

/* RaiseException.export */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck, unsafe_shared) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck, int unsafe_shared);

/* PyTypeError_Check.proto */
#define __Pyx_PyExc_TypeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_TypeError)

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* JoinPyUnicode.export */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* GetTopmostException.proto (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* RaiseClosureNameError.proto */
static void __Pyx_RaiseClosureNameError(const char *varname);

/* dict_setdefault.proto (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value);

/* LimitedApiGetTypeDict.proto (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp);
#endif

/* SetItemOnTypeDict.proto (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v);
#define __Pyx_SetItemOnTypeDict(tp, k, v) __Pyx__SetItemOnTypeDict((PyTypeObject*)tp, k, v)

/* FixUpExtensionType.proto (used by FetchCommonType) */
static CYTHON_INLINE int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);

/* AddModuleRef.proto (used by FetchSharedCythonModule) */
#if ((CYTHON_COMPILING_IN_CPYTHON_FREETHREADING ) ||\
     __PYX_LIMITED_VERSION_HEX < 0x030d0000)
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name);
#else
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#endif

/* FetchSharedCythonModule.proto (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto (used by CommonTypesMetaclass) */
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases);

/* CommonTypesMetaclass.proto (used by CythonFunctionShared) */
static int __pyx_CommonTypesMetaclass_init(PyObject *module);
#define __Pyx_CommonTypesMetaclass_USED

/* CallTypeTraverse.proto (used by CythonFunctionShared) */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#define __Pyx_call_type_traverse(o, always_call, visit, arg) 0
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg);
#endif

/* PyMethodNew.proto (used by CythonFunctionShared) */
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ);

/* PyVectorcallFastCallDict.proto (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto (used by CythonFunction) */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && CYTHON_METH_FASTCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_dict;
#endif
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    PyObject *defaults;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_mstate_global->__pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_mstate_global->__pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void));
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func,
                                                         PyTypeObject *defaults_type);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyObjectDelAttr.proto (used by PyObjectSetAttrStr) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
#define __Pyx_PyObject_DelAttr(o, n) PyObject_SetAttr(o, n, NULL)
#else
#define __Pyx_PyObject_DelAttr(o, n) PyObject_DelAttr(o, n)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   __Pyx_PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* UnicodeConcatInPlace.proto */
# if CYTHON_COMPILING_IN_CPYTHON
    #if CYTHON_REFNANNY
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared, __pyx_refnanny)
    #else
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared)
    #endif
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_OwnStrongReference)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_FunctionArgument)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_SharedReference)
    static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    );
#else
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace __Pyx_PyUnicode_Concat
#endif
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right))

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyMemoryError_Check.proto */
#define __Pyx_PyExc_MemoryError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_MemoryError)

/* RejectKeywords.export */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds);

/* decode_c_string_utf16.proto (used by decode_c_bytes) */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_bytes.proto (used by decode_bytes) */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* decode_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_bytes(
         PyObject* string, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    char* as_c_string;
    Py_ssize_t size;
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    as_c_string = PyBytes_AS_STRING(string);
    size = PyBytes_GET_SIZE(string);
#else
    if (PyBytes_AsStringAndSize(string, &as_c_string, &size) < 0) {
        return NULL;
    }
#endif
    return __Pyx_decode_c_bytes(
        as_c_string, size,
        start, stop, encoding, errors, decode_func);
}

/* ArgTypeTestFunc.export */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))

/* KeywordStringCheck.proto */
static CYTHON_INLINE int __Pyx_CheckKeywordStrings(const char* function_name, PyObject *kw);

/* PyNotImplementedError_Check.proto */
#define __Pyx_PyExc_NotImplementedError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_NotImplementedError)

/* PyLongBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE PyObject* __Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyLong_FloorDivideObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceFloorDivide(op1, op2) : PyNumber_FloorDivide(op1, op2))
#endif

/* PyLongCompare.proto */
static CYTHON_INLINE int __Pyx_PyLong_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyLongBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE PyObject* __Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyLong_AddObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* PyObjectCall2Args.proto (used by CallUnboundCMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* CallUnboundCMethod1.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* pyint_simplify.proto */
static CYTHON_INLINE int __Pyx_PyInt_FromNumber(PyObject **number_var, const char *argname, int accept_none);

/* PyObjectVectorCallKwBuilder.proto */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* AllocateExtensionType.proto */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final);

/* DeallocKeepAlive.proto */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_DeallocKeepAliveBegin(o) do {\
        _Py_atomic_store_uintptr_relaxed(&(o)->ob_tid, _Py_ThreadId());\
        _Py_atomic_store_uint32_relaxed(&(o)->ob_ref_local, 1);\
        _Py_atomic_store_ssize_relaxed(&(o)->ob_ref_shared, 0);\
    } while (0)
#define __Pyx_DeallocKeepAliveEnd(o)\
        _Py_atomic_store_uint32_relaxed(&(o)->ob_ref_local, 0)
#else
#define __Pyx_DeallocKeepAliveBegin(o) __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1)
#define __Pyx_DeallocKeepAliveEnd(o)   __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1)
#endif

/* CheckTypeForFreelists.proto */
#if CYTHON_USE_FREELISTS
#if CYTHON_USE_TYPE_SPECS
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t) == (expected_tp)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  Py_TPFLAGS_IS_ABSTRACT
#else
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t)->tp_basicsize == (expected_size)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)
#endif
#define __PYX_CHECK_TYPE_FOR_FREELISTS(t, expected_tp, expected_size)\
    (__PYX_CHECK_FINAL_TYPE_FOR_FREELISTS((t), (expected_tp), (expected_size)) &\
     (int) (!__Pyx_PyType_HasFeature((t), __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS)))
#endif

/* FunctionExport.proto */
static int __Pyx_ExportFunction(PyObject *api_dict, const char *name, void (*f)(void), const char *sig);

/* GetApiDict.proto */
static PyObject *__Pyx_ApiExport_GetApiDict(void);

/* PyObjectCallNoArg.proto (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectGetMethod.proto (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);
#endif

/* PyObjectCallMethod0.proto (used by PyType_Ready) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* ValidateBasesTuple.proto (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
static int __Pyx_MergeVtables(PyTypeObject *type);

/* DelItemOnTypeDict.proto (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k);
#define __Pyx_DelItemOnTypeDict(tp, k) __Pyx__DelItemOnTypeDict((PyTypeObject*)tp, k)

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_2_9
#define __PYX_HAVE_RT_ImportType_proto_3_2_9
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_9(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_9(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_2_9 {
   __Pyx_ImportType_CheckSize_Error_3_2_9 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_2_9 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_2_9 = 2
};
static PyTypeObject *__Pyx_ImportType_3_2_9(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_9 check_size);
#endif

/* FunctionImport.proto */
static int __Pyx_ImportFunction_3_2_9(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* HasAttr.proto (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* ImportImpl.export */
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level);

/* Import.proto */
static CYTHON_INLINE PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* PyImportError_Check.proto */
#define __Pyx_PyExc_ImportError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ImportError)

/* PyUserWarning_Check.proto */
#define __Pyx_PyExc_UserWarning_Check(obj)  __Pyx_TypeCheck(obj, PyExc_UserWarning)

/* Py3UpdateBases.proto */
static PyObject* __Pyx_PEP560_update_bases(PyObject *bases);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* PyObjectLookupSpecial.proto (used by Py3ClassCreate) */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* GetNameInClass.proto */
#define __Pyx_GetNameInClass(var, nmspace, name)  (var) = __Pyx__GetNameInClass(nmspace, name)
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name);

/* PyRange_Check.proto */
#if CYTHON_COMPILING_IN_PYPY && !defined(PyRange_Check)
  #define PyRange_Check(obj)  __Pyx_TypeCheck((obj), &PyRange_Type)
#endif

/* SetNewInClass.proto */
static int __Pyx_SetNewInClass(PyObject *ns, PyObject *name, PyObject *value);

/* CyFunctionClassCell.proto */
static int __Pyx_CyFunction_InitClassCell(PyObject *cyfunctions, PyObject *classobj);

/* CLineInTraceback.proto (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto (used by AddTraceback) */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint64_t __Pyx_PyLong_As_uint64_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyLong_As_uint32_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint32_t(uint32_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE int64_t __Pyx_PyLong_As_int64_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint8_t(uint8_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint64_t(uint64_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int64_t(int64_t value);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

/* GetRuntimeVersion.proto */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
static unsigned long __Pyx_cached_runtime_version = 0;
static void __Pyx_init_runtime_version(void);
#else
#define __Pyx_init_runtime_version()
#endif
static unsigned long __Pyx_get_runtime_version(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* DecompressString.proto */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

static PyObject *__pyx_f_4nvtx_4_lib_3lib_15EventAttributes__set_binary_payload(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, void *__pyx_v_payload, uint64_t __pyx_v_schema, size_t __pyx_v_nbytes, size_t __pyx_v_array_length); /* proto*/
static PyObject *__pyx_f_4nvtx_4_lib_3lib_15EventAttributes__clear_payload(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self); /* proto*/
static uint64_t __pyx_f_4nvtx_4_lib_3lib_15SchemaRegistrar__do_register(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, CYTHON_UNUSED nvtxPayloadSchemaAttr_t const *__pyx_v_attr); /* proto*/
static uint64_t __pyx_f_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar__do_register(struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *__pyx_v_self, nvtxPayloadSchemaAttr_t const *__pyx_v_attr); /* proto*/

/* Module declarations from "libc.stdint" */

/* Module declarations from "libc.stddef" */

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdlib" */

/* Module declarations from "nvtx._lib.counters" */
static void (*__pyx_f_4nvtx_4_lib_8counters__fill_counter_semantics)(nvtxSemanticsCounter_t *, PyObject *); /*proto*/

/* Module declarations from "nvtx._lib.time" */

/* Module declarations from "nvtx._lib.lib" */
static PyObject *__pyx_f_4nvtx_4_lib_3lib__to_bytes(PyObject *, int __pyx_skip_dispatch); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "nvtx._lib.lib"
extern int __pyx_module_is_main_nvtx___lib__lib;
int __pyx_module_is_main_nvtx___lib__lib = 0;

/* Implementation of "nvtx._lib.lib" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_super;
/* #### Code section: string_decls ### */
/* #### Code section: decls ### */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib__to_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_2initialize(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_4_validate_counter_group_dtype(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6_normalize_counter_dtype(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_8_counter_semantics_from_metadata(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_metadata); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_10_entry_flags_from_metadata(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_metadata); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12_entry_type_from_metadata(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_metadata); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_14payload_setter_register(PyObject *__pyx_self, PyObject *__pyx_v_func); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_14payload_setter(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_type); /* proto */
static void __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes___dealloc__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self); /* proto */
static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_2__init__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_domain, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7message___get__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self); /* proto */
static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7message_2__set__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_5color___get__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self); /* proto */
static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_5color_2__set__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_8category___get__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self); /* proto */
static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_8category_2__set__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7payload___get__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self); /* proto */
static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7payload_2__set__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_4_set_payload_int(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_6_set_payload_float(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_8_set_payload_numpy(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_10_set_payload_iterable(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle___init__(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_4name___get__(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_2enabled(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self); /* proto */
static void __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_4__dealloc__(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_16RegisteredString___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_domain, PyObject *__pyx_v_string); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_get_registered_string(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_string); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_2get_category_id(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_4get_event_attributes(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_6set_event_attributes(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_8mark(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_10push_range(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_12pop_range(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_14start_range(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_16end_range(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED nvtxRangeId_t __pyx_v_range_id); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_26__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_18get_counter(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, CYTHON_UNUSED PyObject *__pyx_v_description, CYTHON_UNUSED PyObject *__pyx_v_scope, CYTHON_UNUSED PyObject *__pyx_v_semantics, CYTHON_UNUSED PyObject *__pyx_v_time_domain); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_20get_timestamp(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self); /* proto */
static int __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar___cinit__(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_2_register_builtin_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, PyObject *__pyx_v_dt, PyObject *__pyx_v_metadata); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_4_register_structured_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, PyObject *__pyx_v_schema_key); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_6_get_numpy_array_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, uint64_t __pyx_v_scalar_schema); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_8_get_fixed_size_array_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, uint64_t __pyx_v_scalar_schema, size_t __pyx_v_array_length, size_t __pyx_v_size); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_10_get_numpy_dtype_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, PyObject *__pyx_v_schema_key); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar___init__(struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_handle); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain___new__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cls, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_2__init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_4get_registered_string(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_string); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_6get_category_id(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_pf_4nvtx_4_lib_3lib_6Domain_8get_event_attributes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_28__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_10set_event_attributes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, PyObject *__pyx_v_payload); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_12mark(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_14push_range(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_16pop_range(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_18start_range(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_20end_range(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, nvtxRangeId_t __pyx_v_range_id); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_30__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_22get_counter(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, PyObject *__pyx_v_description, PyObject *__pyx_v_scope, PyObject *__pyx_v_semantics, PyObject *__pyx_v_time_domain); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_24_get_scope_id(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_path); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_26_get_counter_cached(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_schema_key, PyObject *__pyx_v_description, PyObject *__pyx_v_scope, PyObject *__pyx_v_semantics, uint64_t __pyx_v_time_domain); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_28get_timestamp(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_30_get_numpy_dtype_schema(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_schema_key); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_32_get_numpy_array_schema(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_scalar_schema); /* proto */
static int __pyx_pf_4nvtx_4_lib_3lib_12StringHandle___init__(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain_handle, PyObject *__pyx_v_string); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12StringHandle_6string___get__(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12StringHandle_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12StringHandle_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_16push_range(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_18pop_range(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_20start_range(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_22end_range(CYTHON_UNUSED PyObject *__pyx_self, nvtxRangeId_t __pyx_v_range_id, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain); /* proto */
static PyObject *__pyx_pf_4nvtx_4_lib_3lib_24mark(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain); /* proto */
static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib_EventAttributes(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib_SchemaRegistrar(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib_DomainHandle(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib_StringHandle(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib__GlobalSchemaRegistrar(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib___pyx_defaults(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib___pyx_defaults1(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_8counters_Counter;
  PyObject *__pyx_type_4nvtx_4_lib_3lib_EventAttributes;
  PyObject *__pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar;
  PyObject *__pyx_type_4nvtx_4_lib_3lib_DomainHandle;
  PyObject *__pyx_type_4nvtx_4_lib_3lib_StringHandle;
  PyObject *__pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar;
  PyObject *__pyx_type_4nvtx_4_lib_3lib___pyx_defaults;
  PyObject *__pyx_type_4nvtx_4_lib_3lib___pyx_defaults1;
  PyObject *__pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib_StringHandle;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1;
  PyTypeObject *__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values;
  PyObject *__pyx_slice[1];
  PyObject *__pyx_tuple[2];
  PyObject *__pyx_codeobj_tab[63];
  PyObject *__pyx_string_tab[391];
  PyObject *__pyx_number_tab[4];
/* #### Code section: module_state_contents ### */
/* CommonTypesMetaclass.module_state_decls */
PyTypeObject *__pyx_CommonTypesMetaclassType;

/* CachedMethodType.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedMethodType;
#endif

/* CythonFunctionShared.module_state_decls */
PyTypeObject *__pyx_CyFunctionType;


#if CYTHON_USE_FREELISTS
struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *__pyx_freelist_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter[8];
int __pyx_freecount_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter;
#endif
/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_ __pyx_string_tab[0]
#define __pyx_kp_u_A_class_that_provides_an_interf __pyx_string_tab[1]
#define __pyx_kp_u_A_replacement_for_class_nvtx_Do __pyx_string_tab[2]
#define __pyx_kp_u_A_wrapper_class_for_nvtxStringH __pyx_string_tab[3]
#define __pyx_kp_u_Counter_dtype_must_be_int_float __pyx_string_tab[4]
#define __pyx_kp_u_Counter_group_array_fields_are_n __pyx_string_tab[5]
#define __pyx_kp_u_Domain_mark_line_902 __pyx_string_tab[6]
#define __pyx_kp_u_Domain_push_range_line_945 __pyx_string_tab[7]
#define __pyx_kp_u_Domain_start_range_line_1004 __pyx_string_tab[8]
#define __pyx_kp_u_Event_kind_for_the_writer_s_wri __pyx_string_tab[9]
#define __pyx_kp_u_Failed_to_allocate_memory_for_pa __pyx_string_tab[10]
#define __pyx_kp_u_Failed_to_allocate_memory_for_sc __pyx_string_tab[11]
#define __pyx_kp_u_Failed_to_allocate_memory_for_sc_2 __pyx_string_tab[12]
#define __pyx_kp_u_Install_numpy_for_extended_payl __pyx_string_tab[13]
#define __pyx_kp_u_Install_numpy_to_use_dtype_based __pyx_string_tab[14]
#define __pyx_kp_u_Mark_an_instantaneous_event_Par __pyx_string_tab[15]
#define __pyx_kp_u_Mark_the_beginning_of_a_code_ra __pyx_string_tab[16]
#define __pyx_kp_u_Mark_the_beginning_of_a_process __pyx_string_tab[17]
#define __pyx_kp_u_NVTX_interpretation_of_a_specia __pyx_string_tab[18]
#define __pyx_kp_u_Nested_counter_groups_are_not_su __pyx_string_tab[19]
#define __pyx_kp_u_Nested_counter_groups_are_not_su_2 __pyx_string_tab[20]
#define __pyx_kp_u_Note_that_Cython_is_deliberately __pyx_string_tab[21]
#define __pyx_kp_u_Optional_str __pyx_string_tab[22]
#define __pyx_kp_u_Predefined_NVTX_execution_scopes __pyx_string_tab[23]
#define __pyx_kp_u_Role_of_a_field_in_a_structured __pyx_string_tab[24]
#define __pyx_kp_u_SchemaRegistrar_subclasses_must __pyx_string_tab[25]
#define __pyx_kp_u_Timestamp_ordering_within_a_batc __pyx_string_tab[26]
#define __pyx_kp_u_Unsupported_payload_type __pyx_string_tab[27]
#define __pyx_kp_u__3 __pyx_string_tab[28]
#define __pyx_kp_u_add_note __pyx_string_tab[29]
#define __pyx_kp_u_disable __pyx_string_tab[30]
#define __pyx_kp_u_enable __pyx_string_tab[31]
#define __pyx_kp_u_gc __pyx_string_tab[32]
#define __pyx_kp_u_isenabled __pyx_string_tab[33]
#define __pyx_kp_u_must_be_a_scalar_dtype __pyx_string_tab[34]
#define __pyx_kp_u_must_not_have_a_structured_dtyp __pyx_string_tab[35]
#define __pyx_kp_u_no_default___reduce___due_to_non __pyx_string_tab[36]
#define __pyx_kp_u_self__allocated_payload_self__pa __pyx_string_tab[37]
#define __pyx_kp_u_self_c_obj_cannot_be_converted_t __pyx_string_tab[38]
#define __pyx_kp_u_src_nvtx__lib_lib_pyx __pyx_string_tab[39]
#define __pyx_kp_u_tree_fragment __pyx_string_tab[40]
#define __pyx_n_u_BatchOrdering __pyx_string_tab[41]
#define __pyx_n_u_CATEGORY __pyx_string_tab[42]
#define __pyx_n_u_COLOR_ARGB __pyx_string_tab[43]
#define __pyx_n_u_COUNTER_TIMESTAMP __pyx_string_tab[44]
#define __pyx_n_u_CURRENT_CONTAINER __pyx_string_tab[45]
#define __pyx_n_u_CURRENT_HW_CPU_LOGICAL __pyx_string_tab[46]
#define __pyx_n_u_CURRENT_HW_CPU_PHYSICAL __pyx_string_tab[47]
#define __pyx_n_u_CURRENT_HW_INNERMOST __pyx_string_tab[48]
#define __pyx_n_u_CURRENT_HW_MACHINE __pyx_string_tab[49]
#define __pyx_n_u_CURRENT_HW_SOCKET __pyx_string_tab[50]
#define __pyx_n_u_CURRENT_HYPERVISOR __pyx_string_tab[51]
#define __pyx_n_u_CURRENT_KERNEL __pyx_string_tab[52]
#define __pyx_n_u_CURRENT_OS __pyx_string_tab[53]
#define __pyx_n_u_CURRENT_SW_INNERMOST __pyx_string_tab[54]
#define __pyx_n_u_CURRENT_SW_PROCESS __pyx_string_tab[55]
#define __pyx_n_u_CURRENT_SW_THREAD __pyx_string_tab[56]
#define __pyx_n_u_CURRENT_VM __pyx_string_tab[57]
#define __pyx_n_u_Counter __pyx_string_tab[58]
#define __pyx_n_u_DONT_SET __pyx_string_tab[59]
#define __pyx_n_u_Domain __pyx_string_tab[60]
#define __pyx_n_u_DomainHandle __pyx_string_tab[61]
#define __pyx_n_u_DomainHandle___reduce_cython __pyx_string_tab[62]
#define __pyx_n_u_DomainHandle___setstate_cython __pyx_string_tab[63]
#define __pyx_n_u_DomainHandle_enabled __pyx_string_tab[64]
#define __pyx_n_u_Domain___init __pyx_string_tab[65]
#define __pyx_n_u_Domain___new __pyx_string_tab[66]
#define __pyx_n_u_Domain__get_counter_cached __pyx_string_tab[67]
#define __pyx_n_u_Domain__get_numpy_array_schema __pyx_string_tab[68]
#define __pyx_n_u_Domain__get_numpy_dtype_schema __pyx_string_tab[69]
#define __pyx_n_u_Domain__get_scope_id __pyx_string_tab[70]
#define __pyx_n_u_Domain_end_range __pyx_string_tab[71]
#define __pyx_n_u_Domain_get_category_id __pyx_string_tab[72]
#define __pyx_n_u_Domain_get_counter __pyx_string_tab[73]
#define __pyx_n_u_Domain_get_event_attributes __pyx_string_tab[74]
#define __pyx_n_u_Domain_get_registered_string __pyx_string_tab[75]
#define __pyx_n_u_Domain_get_timestamp __pyx_string_tab[76]
#define __pyx_n_u_Domain_mark __pyx_string_tab[77]
#define __pyx_n_u_Domain_pop_range __pyx_string_tab[78]
#define __pyx_n_u_Domain_push_range __pyx_string_tab[79]
#define __pyx_n_u_Domain_set_event_attributes __pyx_string_tab[80]
#define __pyx_n_u_Domain_start_range __pyx_string_tab[81]
#define __pyx_n_u_DummyCounter __pyx_string_tab[82]
#define __pyx_n_u_DummyDomain __pyx_string_tab[83]
#define __pyx_n_u_DummyDomain_end_range __pyx_string_tab[84]
#define __pyx_n_u_DummyDomain_get_category_id __pyx_string_tab[85]
#define __pyx_n_u_DummyDomain_get_counter __pyx_string_tab[86]
#define __pyx_n_u_DummyDomain_get_event_attributes __pyx_string_tab[87]
#define __pyx_n_u_DummyDomain_get_registered_strin __pyx_string_tab[88]
#define __pyx_n_u_DummyDomain_get_timestamp __pyx_string_tab[89]
#define __pyx_n_u_DummyDomain_mark __pyx_string_tab[90]
#define __pyx_n_u_DummyDomain_pop_range __pyx_string_tab[91]
#define __pyx_n_u_DummyDomain_push_range __pyx_string_tab[92]
#define __pyx_n_u_DummyDomain_set_event_attributes __pyx_string_tab[93]
#define __pyx_n_u_DummyDomain_start_range __pyx_string_tab[94]
#define __pyx_n_u_EntryKind __pyx_string_tab[95]
#define __pyx_n_u_Enum __pyx_string_tab[96]
#define __pyx_n_u_EventAttributes __pyx_string_tab[97]
#define __pyx_n_u_EventAttributes___reduce_cython __pyx_string_tab[98]
#define __pyx_n_u_EventAttributes___setstate_cytho __pyx_string_tab[99]
#define __pyx_n_u_EventAttributes__set_payload_flo __pyx_string_tab[100]
#define __pyx_n_u_EventAttributes__set_payload_int __pyx_string_tab[101]
#define __pyx_n_u_EventAttributes__set_payload_ite __pyx_string_tab[102]
#define __pyx_n_u_EventAttributes__set_payload_num __pyx_string_tab[103]
#define __pyx_n_u_EventKind __pyx_string_tab[104]
#define __pyx_n_u_GlobalSchemaRegistrar __pyx_string_tab[105]
#define __pyx_n_u_GlobalSchemaRegistrar___reduce __pyx_string_tab[106]
#define __pyx_n_u_GlobalSchemaRegistrar___setstat __pyx_string_tab[107]
#define __pyx_n_u_MARK __pyx_string_tab[108]
#define __pyx_n_u_MESSAGE __pyx_string_tab[109]
#define __pyx_n_u_NONE __pyx_string_tab[110]
#define __pyx_n_u_NvtxWarning __pyx_string_tab[111]
#define __pyx_n_u_Optional __pyx_string_tab[112]
#define __pyx_n_u_PARTIALLY_SORTED __pyx_string_tab[113]
#define __pyx_n_u_PayloadEntryType __pyx_string_tab[114]
#define __pyx_n_u_PayloadSchemaKey __pyx_string_tab[115]
#define __pyx_n_u_PredefinedScope __pyx_string_tab[116]
#define __pyx_n_u_Pyx_PyDict_NextRef __pyx_string_tab[117]
#define __pyx_n_u_RANGE_BEGIN __pyx_string_tab[118]
#define __pyx_n_u_RANGE_END __pyx_string_tab[119]
#define __pyx_n_u_RANGE_ID __pyx_string_tab[120]
#define __pyx_n_u_RANGE_POP __pyx_string_tab[121]
#define __pyx_n_u_RANGE_PUSH __pyx_string_tab[122]
#define __pyx_n_u_RANGE_PUSHPOP __pyx_string_tab[123]
#define __pyx_n_u_RANGE_START __pyx_string_tab[124]
#define __pyx_n_u_RANGE_STARTEND __pyx_string_tab[125]
#define __pyx_n_u_REGISTERED_STRING __pyx_string_tab[126]
#define __pyx_n_u_ROOT __pyx_string_tab[127]
#define __pyx_n_u_RegisteredString __pyx_string_tab[128]
#define __pyx_n_u_RegisteredString___init __pyx_string_tab[129]
#define __pyx_n_u_SCOPE_ID __pyx_string_tab[130]
#define __pyx_n_u_SORTED __pyx_string_tab[131]
#define __pyx_n_u_SORTED_PER_SCOPE __pyx_string_tab[132]
#define __pyx_n_u_SchemaRegistrar __pyx_string_tab[133]
#define __pyx_n_u_SchemaRegistrar___reduce_cython __pyx_string_tab[134]
#define __pyx_n_u_SchemaRegistrar___setstate_cytho __pyx_string_tab[135]
#define __pyx_n_u_SchemaRegistrar__get_fixed_size __pyx_string_tab[136]
#define __pyx_n_u_SchemaRegistrar__get_numpy_array __pyx_string_tab[137]
#define __pyx_n_u_SchemaRegistrar__get_numpy_dtype __pyx_string_tab[138]
#define __pyx_n_u_SchemaRegistrar__register_builti __pyx_string_tab[139]
#define __pyx_n_u_SchemaRegistrar__register_struct __pyx_string_tab[140]
#define __pyx_n_u_StringHandle __pyx_string_tab[141]
#define __pyx_n_u_StringHandle___reduce_cython __pyx_string_tab[142]
#define __pyx_n_u_StringHandle___setstate_cython __pyx_string_tab[143]
#define __pyx_n_u_TOOL_PROVIDED __pyx_string_tab[144]
#define __pyx_n_u_TimestampType __pyx_string_tab[145]
#define __pyx_n_u_UNSORTED __pyx_string_tab[146]
#define __pyx_n_u_annotate __pyx_string_tab[147]
#define __pyx_n_u_array __pyx_string_tab[148]
#define __pyx_n_u_arrayOrUnionDetail __pyx_string_tab[149]
#define __pyx_n_u_array_length __pyx_string_tab[150]
#define __pyx_n_u_ascontiguousarray __pyx_string_tab[151]
#define __pyx_n_u_asyncio_coroutines __pyx_string_tab[152]
#define __pyx_n_u_attr __pyx_string_tab[153]
#define __pyx_n_u_attributes __pyx_string_tab[154]
#define __pyx_n_u_bytes __pyx_string_tab[155]
#define __pyx_n_u_cache_key __pyx_string_tab[156]
#define __pyx_n_u_cached __pyx_string_tab[157]
#define __pyx_n_u_categories __pyx_string_tab[158]
#define __pyx_n_u_category __pyx_string_tab[159]
#define __pyx_n_u_category_id __pyx_string_tab[160]
#define __pyx_n_u_class_getitem __pyx_string_tab[161]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[162]
#define __pyx_n_u_cls __pyx_string_tab[163]
#define __pyx_n_u_color __pyx_string_tab[164]
#define __pyx_n_u_color_to_hex __pyx_string_tab[165]
#define __pyx_n_u_counter_class_from_dtype __pyx_string_tab[166]
#define __pyx_n_u_counter_cls __pyx_string_tab[167]
#define __pyx_n_u_counter_group __pyx_string_tab[168]
#define __pyx_n_u_counter_semantics __pyx_string_tab[169]
#define __pyx_n_u_counter_semantics_from_metadata __pyx_string_tab[170]
#define __pyx_n_u_counter_semantics_obj __pyx_string_tab[171]
#define __pyx_n_u_ctypes __pyx_string_tab[172]
#define __pyx_n_u_data __pyx_string_tab[173]
#define __pyx_n_u_data_entry __pyx_string_tab[174]
#define __pyx_n_u_description __pyx_string_tab[175]
#define __pyx_n_u_dh __pyx_string_tab[176]
#define __pyx_n_u_dict __pyx_string_tab[177]
#define __pyx_n_u_doc __pyx_string_tab[178]
#define __pyx_n_u_domain __pyx_string_tab[179]
#define __pyx_n_u_domain_handle __pyx_string_tab[180]
#define __pyx_n_u_dt __pyx_string_tab[181]
#define __pyx_n_u_dtype __pyx_string_tab[182]
#define __pyx_n_u_dtype_to_entry_type __pyx_string_tab[183]
#define __pyx_n_u_dummy_counter __pyx_string_tab[184]
#define __pyx_n_u_dummy_domain __pyx_string_tab[185]
#define __pyx_n_u_e __pyx_string_tab[186]
#define __pyx_n_u_enabled __pyx_string_tab[187]
#define __pyx_n_u_encode __pyx_string_tab[188]
#define __pyx_n_u_end_range __pyx_string_tab[189]
#define __pyx_n_u_entries __pyx_string_tab[190]
#define __pyx_n_u_entry __pyx_string_tab[191]
#define __pyx_n_u_entry_flags_from_metadata __pyx_string_tab[192]
#define __pyx_n_u_entry_kind __pyx_string_tab[193]
#define __pyx_n_u_entry_type __pyx_string_tab[194]
#define __pyx_n_u_entry_type_from_metadata __pyx_string_tab[195]
#define __pyx_n_u_enum __pyx_string_tab[196]
#define __pyx_n_u_enumerate __pyx_string_tab[197]
#define __pyx_n_u_field_counter_semantics __pyx_string_tab[198]
#define __pyx_n_u_field_name __pyx_string_tab[199]
#define __pyx_n_u_field_names __pyx_string_tab[200]
#define __pyx_n_u_field_schema_key __pyx_string_tab[201]
#define __pyx_n_u_field_type __pyx_string_tab[202]
#define __pyx_n_u_fields __pyx_string_tab[203]
#define __pyx_n_u_flags __pyx_string_tab[204]
#define __pyx_n_u_flat __pyx_string_tab[205]
#define __pyx_n_u_float16 __pyx_string_tab[206]
#define __pyx_n_u_float32 __pyx_string_tab[207]
#define __pyx_n_u_float64 __pyx_string_tab[208]
#define __pyx_n_u_func __pyx_string_tab[209]
#define __pyx_n_u_func_2 __pyx_string_tab[210]
#define __pyx_n_u_functools __pyx_string_tab[211]
#define __pyx_n_u_get __pyx_string_tab[212]
#define __pyx_n_u_get_category_id __pyx_string_tab[213]
#define __pyx_n_u_get_counter __pyx_string_tab[214]
#define __pyx_n_u_get_counter_cached __pyx_string_tab[215]
#define __pyx_n_u_get_event_attributes __pyx_string_tab[216]
#define __pyx_n_u_get_fixed_size_array_schema __pyx_string_tab[217]
#define __pyx_n_u_get_numpy_array_schema __pyx_string_tab[218]
#define __pyx_n_u_get_numpy_dtype_schema __pyx_string_tab[219]
#define __pyx_n_u_get_registered_string __pyx_string_tab[220]
#define __pyx_n_u_get_scope_id __pyx_string_tab[221]
#define __pyx_n_u_get_timestamp __pyx_string_tab[222]
#define __pyx_n_u_getstate __pyx_string_tab[223]
#define __pyx_n_u_handle __pyx_string_tab[224]
#define __pyx_n_u_i __pyx_string_tab[225]
#define __pyx_n_u_init __pyx_string_tab[226]
#define __pyx_n_u_initialize __pyx_string_tab[227]
#define __pyx_n_u_int __pyx_string_tab[228]
#define __pyx_n_u_int16 __pyx_string_tab[229]
#define __pyx_n_u_int32 __pyx_string_tab[230]
#define __pyx_n_u_int64 __pyx_string_tab[231]
#define __pyx_n_u_int8 __pyx_string_tab[232]
#define __pyx_n_u_is_coroutine __pyx_string_tab[233]
#define __pyx_n_u_items __pyx_string_tab[234]
#define __pyx_n_u_itemsize __pyx_string_tab[235]
#define __pyx_n_u_kwargs __pyx_string_tab[236]
#define __pyx_n_u_lru_cache __pyx_string_tab[237]
#define __pyx_n_u_main __pyx_string_tab[238]
#define __pyx_n_u_mark __pyx_string_tab[239]
#define __pyx_n_u_maxsize __pyx_string_tab[240]
#define __pyx_n_u_message __pyx_string_tab[241]
#define __pyx_n_u_metaclass __pyx_string_tab[242]
#define __pyx_n_u_metadata __pyx_string_tab[243]
#define __pyx_n_u_metadata_flags __pyx_string_tab[244]
#define __pyx_n_u_module __pyx_string_tab[245]
#define __pyx_n_u_mro_entries __pyx_string_tab[246]
#define __pyx_n_u_name __pyx_string_tab[247]
#define __pyx_n_u_name_2 __pyx_string_tab[248]
#define __pyx_n_u_names __pyx_string_tab[249]
#define __pyx_n_u_nbytes __pyx_string_tab[250]
#define __pyx_n_u_ndarray __pyx_string_tab[251]
#define __pyx_n_u_ndim __pyx_string_tab[252]
#define __pyx_n_u_new __pyx_string_tab[253]
#define __pyx_n_u_normalize_counter_dtype __pyx_string_tab[254]
#define __pyx_n_u_np __pyx_string_tab[255]
#define __pyx_n_u_numpy __pyx_string_tab[256]
#define __pyx_n_u_nvtx__lib_counters __pyx_string_tab[257]
#define __pyx_n_u_nvtx__lib_lib __pyx_string_tab[258]
#define __pyx_n_u_nvtx__lib_time __pyx_string_tab[259]
#define __pyx_n_u_nvtx__metadata __pyx_string_tab[260]
#define __pyx_n_u_nvtx_colors __pyx_string_tab[261]
#define __pyx_n_u_obj __pyx_string_tab[262]
#define __pyx_n_u_object __pyx_string_tab[263]
#define __pyx_n_u_offset __pyx_string_tab[264]
#define __pyx_n_u_path __pyx_string_tab[265]
#define __pyx_n_u_path_bytes __pyx_string_tab[266]
#define __pyx_n_u_payload __pyx_string_tab[267]
#define __pyx_n_u_payload_setter __pyx_string_tab[268]
#define __pyx_n_u_payload_setter_locals_register __pyx_string_tab[269]
#define __pyx_n_u_payload_setters __pyx_string_tab[270]
#define __pyx_n_u_pop __pyx_string_tab[271]
#define __pyx_n_u_pop_range __pyx_string_tab[272]
#define __pyx_n_u_prepare __pyx_string_tab[273]
#define __pyx_n_u_prod __pyx_string_tab[274]
#define __pyx_n_u_push_range __pyx_string_tab[275]
#define __pyx_n_u_pyx_capi __pyx_string_tab[276]
#define __pyx_n_u_pyx_state __pyx_string_tab[277]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[278]
#define __pyx_n_u_qualname __pyx_string_tab[279]
#define __pyx_n_u_range_id __pyx_string_tab[280]
#define __pyx_n_u_reduce __pyx_string_tab[281]
#define __pyx_n_u_reduce_cython __pyx_string_tab[282]
#define __pyx_n_u_reduce_ex __pyx_string_tab[283]
#define __pyx_n_u_register __pyx_string_tab[284]
#define __pyx_n_u_register_builtin_schema __pyx_string_tab[285]
#define __pyx_n_u_register_structured_schema __pyx_string_tab[286]
#define __pyx_n_u_reserved __pyx_string_tab[287]
#define __pyx_n_u_result __pyx_string_tab[288]
#define __pyx_n_u_return __pyx_string_tab[289]
#define __pyx_n_u_s __pyx_string_tab[290]
#define __pyx_n_u_scalar_schema __pyx_string_tab[291]
#define __pyx_n_u_schema __pyx_string_tab[292]
#define __pyx_n_u_schemaAttr __pyx_string_tab[293]
#define __pyx_n_u_schemaEntries __pyx_string_tab[294]
#define __pyx_n_u_schemaEntry __pyx_string_tab[295]
#define __pyx_n_u_schema_flags __pyx_string_tab[296]
#define __pyx_n_u_schema_key __pyx_string_tab[297]
#define __pyx_n_u_schema_registrar __pyx_string_tab[298]
#define __pyx_n_u_scope __pyx_string_tab[299]
#define __pyx_n_u_self __pyx_string_tab[300]
#define __pyx_n_u_semantics __pyx_string_tab[301]
#define __pyx_n_u_semanticsEntries __pyx_string_tab[302]
#define __pyx_n_u_set_event_attributes __pyx_string_tab[303]
#define __pyx_n_u_set_name __pyx_string_tab[304]
#define __pyx_n_u_set_payload_float __pyx_string_tab[305]
#define __pyx_n_u_set_payload_int __pyx_string_tab[306]
#define __pyx_n_u_set_payload_iterable __pyx_string_tab[307]
#define __pyx_n_u_set_payload_numpy __pyx_string_tab[308]
#define __pyx_n_u_setdefault __pyx_string_tab[309]
#define __pyx_n_u_setstate __pyx_string_tab[310]
#define __pyx_n_u_setstate_cython __pyx_string_tab[311]
#define __pyx_n_u_shape __pyx_string_tab[312]
#define __pyx_n_u_size __pyx_string_tab[313]
#define __pyx_n_u_size_entry __pyx_string_tab[314]
#define __pyx_n_u_start_range __pyx_string_tab[315]
#define __pyx_n_u_str __pyx_string_tab[316]
#define __pyx_n_u_string __pyx_string_tab[317]
#define __pyx_n_u_subdtype __pyx_string_tab[318]
#define __pyx_n_u_subdtype_schema __pyx_string_tab[319]
#define __pyx_n_u_super __pyx_string_tab[320]
#define __pyx_n_u_test __pyx_string_tab[321]
#define __pyx_n_u_time_domain __pyx_string_tab[322]
#define __pyx_n_u_to_bytes __pyx_string_tab[323]
#define __pyx_n_u_type __pyx_string_tab[324]
#define __pyx_n_u_typing __pyx_string_tab[325]
#define __pyx_n_u_uint16 __pyx_string_tab[326]
#define __pyx_n_u_uint32 __pyx_string_tab[327]
#define __pyx_n_u_uint64 __pyx_string_tab[328]
#define __pyx_n_u_uint8 __pyx_string_tab[329]
#define __pyx_n_u_validate_counter_group_dtype __pyx_string_tab[330]
#define __pyx_n_u_value __pyx_string_tab[331]
#define __pyx_n_u_values __pyx_string_tab[332]
#define __pyx_n_u_warn __pyx_string_tab[333]
#define __pyx_n_u_warnings __pyx_string_tab[334]
#define __pyx_kp_b_PyObject_PyObject_int___pyx_skip __pyx_string_tab[335]
#define __pyx_kp_b__2 __pyx_string_tab[336]
#define __pyx_kp_b_iso88591_1 __pyx_string_tab[337]
#define __pyx_kp_b_iso88591_1BfAQ __pyx_string_tab[338]
#define __pyx_kp_b_iso88591_1_2 __pyx_string_tab[339]
#define __pyx_kp_b_iso88591_1_3 __pyx_string_tab[340]
#define __pyx_kp_b_iso88591_1_4 __pyx_string_tab[341]
#define __pyx_kp_b_iso88591_5_3l_7 __pyx_string_tab[342]
#define __pyx_kp_b_iso88591_9A __pyx_string_tab[343]
#define __pyx_kp_b_iso88591_A __pyx_string_tab[344]
#define __pyx_kp_b_iso88591_AV81Ja __pyx_string_tab[345]
#define __pyx_kp_b_iso88591_A_0_q_IQ __pyx_string_tab[346]
#define __pyx_kp_b_iso88591_A_2 __pyx_string_tab[347]
#define __pyx_kp_b_iso88591_A_55HHZZ_87_z_1_4AQ_k_6_iq_9G1_z __pyx_string_tab[348]
#define __pyx_kp_b_iso88591_A_A_Zq_C1_31_7q_1_Zq_1_A_1_nA_iq __pyx_string_tab[349]
#define __pyx_kp_b_iso88591_A_F_A_F __pyx_string_tab[350]
#define __pyx_kp_b_iso88591_A_F_Q_F __pyx_string_tab[351]
#define __pyx_kp_b_iso88591_A_L_Q_q_t_q_q __pyx_string_tab[352]
#define __pyx_kp_b_iso88591_A_N_HA_O1_Kq_HHAQ __pyx_string_tab[353]
#define __pyx_kp_b_iso88591_A_N_ha __pyx_string_tab[354]
#define __pyx_kp_b_iso88591_A_N_hha __pyx_string_tab[355]
#define __pyx_kp_b_iso88591_A_Qj_t_1A_87_d_0_avYgZq __pyx_string_tab[356]
#define __pyx_kp_b_iso88591_A__N_aq_7_1_Fa_q_1_a_Q_C1_325_a __pyx_string_tab[357]
#define __pyx_kp_b_iso88591_A_a __pyx_string_tab[358]
#define __pyx_kp_b_iso88591_A_aq_7_1_Zq_2Q_r_1_d_T_6a_5_9Ba __pyx_string_tab[359]
#define __pyx_kp_b_iso88591_A_aq_7_1_ay_Q_AQ_q_1_a_Q_Q_AQ_q __pyx_string_tab[360]
#define __pyx_kp_b_iso88591_A_q __pyx_string_tab[361]
#define __pyx_kp_b_iso88591_A_q_2 __pyx_string_tab[362]
#define __pyx_kp_b_iso88591_A_q_3 __pyx_string_tab[363]
#define __pyx_kp_b_iso88591_A_q_4 __pyx_string_tab[364]
#define __pyx_kp_b_iso88591_A_q_5 __pyx_string_tab[365]
#define __pyx_kp_b_iso88591_A_r_gQ_6a_LAQ_1A_8_q_aq_c_Qb_c_2 __pyx_string_tab[366]
#define __pyx_kp_b_iso88591_A_t1_c_m2Q_Kq_q_a_Qa_q __pyx_string_tab[367]
#define __pyx_kp_b_iso88591_A_t1_q_A __pyx_string_tab[368]
#define __pyx_kp_b_iso88591_A_t_Q __pyx_string_tab[369]
#define __pyx_kp_b_iso88591_A_t_Qa __pyx_string_tab[370]
#define __pyx_kp_b_iso88591_K1_c_2_Q_QnA_R_T_TU __pyx_string_tab[371]
#define __pyx_kp_b_iso88591_O1J_c_2_Q_QnA_HHAZq __pyx_string_tab[372]
#define __pyx_kp_b_iso88591_O_7_QR __pyx_string_tab[373]
#define __pyx_kp_b_iso88591_Q __pyx_string_tab[374]
#define __pyx_kp_b_iso88591_V1_Qa_6_r_z_1_1 __pyx_string_tab[375]
#define __pyx_kp_b_iso88591_avQ __pyx_string_tab[376]
#define __pyx_kp_b_iso88591_avXQ __pyx_string_tab[377]
#define __pyx_kp_b_iso88591_c_2_Q_QnA_R_T_A __pyx_string_tab[378]
#define __pyx_kp_b_iso88591_nA_1_q __pyx_string_tab[379]
#define __pyx_kp_b_iso88591_q_Ja_Ja_Jl_81 __pyx_string_tab[380]
#define __pyx_kp_b_iso88591_s_6_A_BfARq_iq____1 __pyx_string_tab[381]
#define __pyx_kp_b_iso88591_s_Q_6_D_6_A_l_1_uJgU_e9ARxwa_iq __pyx_string_tab[382]
#define __pyx_kp_b_iso88591_vQ_HA_N_7q_A __pyx_string_tab[383]
#define __pyx_kp_b_iso88591_whc_T_8_q_wa_W_AQ_aq_wa_1_A_xwg __pyx_string_tab[384]
#define __pyx_kp_b_iso88591_y_1_q_7_2 __pyx_string_tab[385]
#define __pyx_kp_b_iso88591_y_1_q_Z_1 __pyx_string_tab[386]
#define __pyx_kp_b_iso88591_y_1_q_Z_1_2 __pyx_string_tab[387]
#define __pyx_kp_b_void_nvtxSemanticsCounter_t_PyOb __pyx_string_tab[388]
#define __pyx_n_b_data __pyx_string_tab[389]
#define __pyx_n_b_size __pyx_string_tab[390]
#define __pyx_int_0 __pyx_number_tab[0]
#define __pyx_int_1 __pyx_number_tab[1]
#define __pyx_int_2 __pyx_number_tab[2]
#define __pyx_int_4 __pyx_number_tab[3]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_8counters_Counter);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_3lib_EventAttributes);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_3lib_DomainHandle);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_3lib_StringHandle);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_3lib___pyx_defaults);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_3lib___pyx_defaults1);
  Py_CLEAR(clear_module_state->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter);
  Py_CLEAR(clear_module_state->__pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter);
  for (int i=0; i<1; ++i) { Py_CLEAR(clear_module_state->__pyx_slice[i]); }
  for (int i=0; i<2; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<63; ++i) { Py_CLEAR(clear_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<391; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<4; ++i) { Py_CLEAR(clear_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_clear_contents ### */
/* CommonTypesMetaclass.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CyFunctionType);

/* #### Code section: module_state_clear_end ### */
return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_8counters_Counter);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_3lib_EventAttributes);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_3lib_DomainHandle);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_3lib_StringHandle);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_3lib___pyx_defaults);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_3lib___pyx_defaults1);
  Py_VISIT(traverse_module_state->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter);
  Py_VISIT(traverse_module_state->__pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter);
  for (int i=0; i<1; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_slice[i]); }
  for (int i=0; i<2; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<63; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<391; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<4; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_traverse_contents ### */
/* CommonTypesMetaclass.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CyFunctionType);

/* #### Code section: module_state_traverse_end ### */
return 0;
}
#endif
/* #### Code section: module_code ### */

/* "nvtx/_lib/lib.pyx":63
 * 
 * 
 * cpdef bytes _to_bytes(object s):             # <<<<<<<<<<<<<<
 *     return s if isinstance(s, bytes) else s.encode()
 * 
*/

static PyObject *__pyx_pw_4nvtx_4_lib_3lib_1_to_bytes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_4nvtx_4_lib_3lib__to_bytes(PyObject *__pyx_v_s, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_to_bytes", 0);

  /* "nvtx/_lib/lib.pyx":64
 * 
 * cpdef bytes _to_bytes(object s):
 *     return s if isinstance(s, bytes) else s.encode()             # <<<<<<<<<<<<<<
 * 
 * def initialize():
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyBytes_Check(__pyx_v_s); 
  if (__pyx_t_2) {
    __pyx_t_3 = __pyx_v_s;
    __Pyx_INCREF(__pyx_t_3);
    if (!(likely(PyBytes_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_3))) __PYX_ERR(0, 64, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __pyx_t_4 = __pyx_v_s;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 64, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    if (!(likely(PyBytes_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_3))) __PYX_ERR(0, 64, __pyx_L1_error)
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":63
 * 
 * 
 * cpdef bytes _to_bytes(object s):             # <<<<<<<<<<<<<<
 *     return s if isinstance(s, bytes) else s.encode()
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.lib._to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_1_to_bytes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib__to_bytes, "_to_bytes(s) -> bytes");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_1_to_bytes = {"_to_bytes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_1_to_bytes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib__to_bytes};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_1_to_bytes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_s = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_to_bytes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_s,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 63, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 63, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_to_bytes", 0) < (0)) __PYX_ERR(0, 63, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_to_bytes", 1, 1, 1, i); __PYX_ERR(0, 63, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 63, __pyx_L3_error)
    }
    __pyx_v_s = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_to_bytes", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 63, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib._to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib__to_bytes(__pyx_self, __pyx_v_s);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib__to_bytes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_to_bytes", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4nvtx_4_lib_3lib__to_bytes(__pyx_v_s, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib._to_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":66
 *     return s if isinstance(s, bytes) else s.encode()
 * 
 * def initialize():             # <<<<<<<<<<<<<<
 *     nvtxInitialize(NULL)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_3initialize(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_2initialize, "initialize()");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_3initialize = {"initialize", (PyCFunction)__pyx_pw_4nvtx_4_lib_3lib_3initialize, METH_NOARGS, __pyx_doc_4nvtx_4_lib_3lib_2initialize};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_3initialize(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("initialize (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_2initialize(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_2initialize(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("initialize", 0);

  /* "nvtx/_lib/lib.pyx":67
 * 
 * def initialize():
 *     nvtxInitialize(NULL)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  nvtxInitialize(NULL);

  /* "nvtx/_lib/lib.pyx":66
 *     return s if isinstance(s, bytes) else s.encode()
 * 
 * def initialize():             # <<<<<<<<<<<<<<
 *     nvtxInitialize(NULL)
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":74
 * 
 * 
 * def _validate_counter_group_dtype(dtype):             # <<<<<<<<<<<<<<
 *     if np is None:
 *         if dtype is int or dtype is float:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_5_validate_counter_group_dtype(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_4_validate_counter_group_dtype, "_validate_counter_group_dtype(dtype)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_5_validate_counter_group_dtype = {"_validate_counter_group_dtype", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_5_validate_counter_group_dtype, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_4_validate_counter_group_dtype};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_5_validate_counter_group_dtype(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_dtype = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_validate_counter_group_dtype (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_dtype,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 74, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 74, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_validate_counter_group_dtype", 0) < (0)) __PYX_ERR(0, 74, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_validate_counter_group_dtype", 1, 1, 1, i); __PYX_ERR(0, 74, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 74, __pyx_L3_error)
    }
    __pyx_v_dtype = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_validate_counter_group_dtype", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 74, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib._validate_counter_group_dtype", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_4_validate_counter_group_dtype(__pyx_self, __pyx_v_dtype);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_4_validate_counter_group_dtype(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_v_field_name = NULL;
  PyObject *__pyx_v_field_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10[3];
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_validate_counter_group_dtype", 0);

  /* "nvtx/_lib/lib.pyx":75
 * 
 * def _validate_counter_group_dtype(dtype):
 *     if np is None:             # <<<<<<<<<<<<<<
 *         if dtype is int or dtype is float:
 *             return
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":76
 * def _validate_counter_group_dtype(dtype):
 *     if np is None:
 *         if dtype is int or dtype is float:             # <<<<<<<<<<<<<<
 *             return
 *         raise RuntimeError("Install numpy to use dtype-based NVTX counters.")
*/
    __pyx_t_3 = (__pyx_v_dtype == ((PyObject *)(&PyLong_Type)));
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_dtype == ((PyObject *)(&PyFloat_Type)));
    __pyx_t_2 = __pyx_t_3;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_2) {

      /* "nvtx/_lib/lib.pyx":77
 *     if np is None:
 *         if dtype is int or dtype is float:
 *             return             # <<<<<<<<<<<<<<
 *         raise RuntimeError("Install numpy to use dtype-based NVTX counters.")
 *     if dtype.subdtype is not None and dtype.subdtype[0].fields is not None:
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "nvtx/_lib/lib.pyx":76
 * def _validate_counter_group_dtype(dtype):
 *     if np is None:
 *         if dtype is int or dtype is float:             # <<<<<<<<<<<<<<
 *             return
 *         raise RuntimeError("Install numpy to use dtype-based NVTX counters.")
*/
    }

    /* "nvtx/_lib/lib.pyx":78
 *         if dtype is int or dtype is float:
 *             return
 *         raise RuntimeError("Install numpy to use dtype-based NVTX counters.")             # <<<<<<<<<<<<<<
 *     if dtype.subdtype is not None and dtype.subdtype[0].fields is not None:
 *         raise TypeError(
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Install_numpy_to_use_dtype_based};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 78, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":75
 * 
 * def _validate_counter_group_dtype(dtype):
 *     if np is None:             # <<<<<<<<<<<<<<
 *         if dtype is int or dtype is float:
 *             return
*/
  }

  /* "nvtx/_lib/lib.pyx":79
 *             return
 *         raise RuntimeError("Install numpy to use dtype-based NVTX counters.")
 *     if dtype.subdtype is not None and dtype.subdtype[0].fields is not None:             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             "Nested counter groups are not supported. "
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_3 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __pyx_t_3;
  __pyx_L8_bool_binop_done:;
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/lib.pyx":80
 *         raise RuntimeError("Install numpy to use dtype-based NVTX counters.")
 *     if dtype.subdtype is not None and dtype.subdtype[0].fields is not None:
 *         raise TypeError(             # <<<<<<<<<<<<<<
 *             "Nested counter groups are not supported. "
 *             "Counter dtype must not be a fixed-size array of a structured dtype."
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Nested_counter_groups_are_not_su};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 80, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":79
 *             return
 *         raise RuntimeError("Install numpy to use dtype-based NVTX counters.")
 *     if dtype.subdtype is not None and dtype.subdtype[0].fields is not None:             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             "Nested counter groups are not supported. "
*/
  }

  /* "nvtx/_lib/lib.pyx":85
 *         )
 * 
 *     if dtype.fields is None:             # <<<<<<<<<<<<<<
 *         return
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":86
 * 
 *     if dtype.fields is None:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     # Iterate dtype.names, not dtype.fields to avoid alias duplicates.
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":85
 *         )
 * 
 *     if dtype.fields is None:             # <<<<<<<<<<<<<<
 *         return
 * 
*/
  }

  /* "nvtx/_lib/lib.pyx":89
 * 
 *     # Iterate dtype.names, not dtype.fields to avoid alias duplicates.
 *     for field_name in dtype.names:             # <<<<<<<<<<<<<<
 *         field_type = dtype.fields[field_name][0]
 *         if field_type.subdtype:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_4 = __pyx_t_1; __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 89, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 89, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        __pyx_t_1 = __Pyx_PyList_GET_ITEM_REF(__pyx_t_4, __pyx_t_6, __Pyx_ReferenceSharing_OwnStrongReference);
        ++__pyx_t_6;
      } else {
        {
          Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 89, __pyx_L1_error)
          #endif
          if (__pyx_t_6 >= __pyx_temp) break;
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_6));
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_6);
        #endif
        ++__pyx_t_6;
      }
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 89, __pyx_L1_error)
    } else {
      __pyx_t_1 = __pyx_t_7(__pyx_t_4);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 89, __pyx_L1_error)
          PyErr_Clear();
        }
        break;
      }
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_field_name, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/lib.pyx":90
 *     # Iterate dtype.names, not dtype.fields to avoid alias duplicates.
 *     for field_name in dtype.names:
 *         field_type = dtype.fields[field_name][0]             # <<<<<<<<<<<<<<
 *         if field_type.subdtype:
 *             raise TypeError(
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_field_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_8, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF_SET(__pyx_v_field_type, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/lib.pyx":91
 *     for field_name in dtype.names:
 *         field_type = dtype.fields[field_name][0]
 *         if field_type.subdtype:             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 "Counter group array fields are not supported. "
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_type, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(__pyx_t_2)) {

      /* "nvtx/_lib/lib.pyx":92
 *         field_type = dtype.fields[field_name][0]
 *         if field_type.subdtype:
 *             raise TypeError(             # <<<<<<<<<<<<<<
 *                 "Counter group array fields are not supported. "
 *                 f"Field {field_name!r} must be a scalar dtype."
*/
      __pyx_t_8 = NULL;

      /* "nvtx/_lib/lib.pyx":94
 *             raise TypeError(
 *                 "Counter group array fields are not supported. "
 *                 f"Field {field_name!r} must be a scalar dtype."             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_9 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_field_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 94, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Counter_group_array_fields_are_n;
      __pyx_t_10[1] = __pyx_t_9;
      __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_must_be_a_scalar_dtype;

      /* "nvtx/_lib/lib.pyx":93
 *         if field_type.subdtype:
 *             raise TypeError(
 *                 "Counter group array fields are not supported. "             # <<<<<<<<<<<<<<
 *                 f"Field {field_name!r} must be a scalar dtype."
 *             )
*/
      __pyx_t_11 = __Pyx_PyUnicode_Join(__pyx_t_10, 3, 52 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9) + 24, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9));
      if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 93, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_11};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 92, __pyx_L1_error)

      /* "nvtx/_lib/lib.pyx":91
 *     for field_name in dtype.names:
 *         field_type = dtype.fields[field_name][0]
 *         if field_type.subdtype:             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 "Counter group array fields are not supported. "
*/
    }

    /* "nvtx/_lib/lib.pyx":97
 *             )
 * 
 *         if field_type.fields is not None:             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 "Nested counter groups are not supported. "
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_type, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = (__pyx_t_1 != Py_None);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(__pyx_t_2)) {

      /* "nvtx/_lib/lib.pyx":98
 * 
 *         if field_type.fields is not None:
 *             raise TypeError(             # <<<<<<<<<<<<<<
 *                 "Nested counter groups are not supported. "
 *                 f"Field {field_name!r} must not have a structured dtype."
*/
      __pyx_t_11 = NULL;

      /* "nvtx/_lib/lib.pyx":100
 *             raise TypeError(
 *                 "Nested counter groups are not supported. "
 *                 f"Field {field_name!r} must not have a structured dtype."             # <<<<<<<<<<<<<<
 *             )
 * 
*/
      __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_field_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 100, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_10[0] = __pyx_mstate_global->__pyx_kp_u_Nested_counter_groups_are_not_su_2;
      __pyx_t_10[1] = __pyx_t_8;
      __pyx_t_10[2] = __pyx_mstate_global->__pyx_kp_u_must_not_have_a_structured_dtyp;

      /* "nvtx/_lib/lib.pyx":99
 *         if field_type.fields is not None:
 *             raise TypeError(
 *                 "Nested counter groups are not supported. "             # <<<<<<<<<<<<<<
 *                 f"Field {field_name!r} must not have a structured dtype."
 *             )
*/
      __pyx_t_9 = __Pyx_PyUnicode_Join(__pyx_t_10, 3, 47 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 34, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8));
      if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 99, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_9};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 98, __pyx_L1_error)

      /* "nvtx/_lib/lib.pyx":97
 *             )
 * 
 *         if field_type.fields is not None:             # <<<<<<<<<<<<<<
 *             raise TypeError(
 *                 "Nested counter groups are not supported. "
*/
    }

    /* "nvtx/_lib/lib.pyx":89
 * 
 *     # Iterate dtype.names, not dtype.fields to avoid alias duplicates.
 *     for field_name in dtype.names:             # <<<<<<<<<<<<<<
 *         field_type = dtype.fields[field_name][0]
 *         if field_type.subdtype:
*/
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":74
 * 
 * 
 * def _validate_counter_group_dtype(dtype):             # <<<<<<<<<<<<<<
 *     if np is None:
 *         if dtype is int or dtype is float:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("nvtx._lib.lib._validate_counter_group_dtype", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_field_name);
  __Pyx_XDECREF(__pyx_v_field_type);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":104
 * 
 * 
 * def _normalize_counter_dtype(dtype):             # <<<<<<<<<<<<<<
 *     """
 *     Normalize dtype-like counter specifications before caching.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_7_normalize_counter_dtype(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6_normalize_counter_dtype, "_normalize_counter_dtype(dtype)\n\nNormalize dtype-like counter specifications before caching.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_7_normalize_counter_dtype = {"_normalize_counter_dtype", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_7_normalize_counter_dtype, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6_normalize_counter_dtype};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_7_normalize_counter_dtype(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_dtype = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_normalize_counter_dtype (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_dtype,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 104, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 104, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_normalize_counter_dtype", 0) < (0)) __PYX_ERR(0, 104, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_normalize_counter_dtype", 1, 1, 1, i); __PYX_ERR(0, 104, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 104, __pyx_L3_error)
    }
    __pyx_v_dtype = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_normalize_counter_dtype", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 104, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib._normalize_counter_dtype", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6_normalize_counter_dtype(__pyx_self, __pyx_v_dtype);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6_normalize_counter_dtype(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_t_12;
  char const *__pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_normalize_counter_dtype", 0);
  __Pyx_INCREF(__pyx_v_dtype);

  /* "nvtx/_lib/lib.pyx":109
 *     """
 * 
 *     if np is not None:             # <<<<<<<<<<<<<<
 *         if dtype is int:
 *             # Force 64-bit integer (64-bit Windows uses int32 for plain ``int``).
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":110
 * 
 *     if np is not None:
 *         if dtype is int:             # <<<<<<<<<<<<<<
 *             # Force 64-bit integer (64-bit Windows uses int32 for plain ``int``).
 *             dtype = np.dtype(np.int64)
*/
    __pyx_t_2 = (__pyx_v_dtype == ((PyObject *)(&PyLong_Type)));
    if (__pyx_t_2) {

      /* "nvtx/_lib/lib.pyx":112
 *         if dtype is int:
 *             # Force 64-bit integer (64-bit Windows uses int32 for plain ``int``).
 *             dtype = np.dtype(np.int64)             # <<<<<<<<<<<<<<
 *         else:
 *             try:
*/
      __pyx_t_3 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int64); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 112, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_7 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
        __pyx_t_7 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 112, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF_SET(__pyx_v_dtype, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "nvtx/_lib/lib.pyx":110
 * 
 *     if np is not None:
 *         if dtype is int:             # <<<<<<<<<<<<<<
 *             # Force 64-bit integer (64-bit Windows uses int32 for plain ``int``).
 *             dtype = np.dtype(np.int64)
*/
      goto __pyx_L4;
    }

    /* "nvtx/_lib/lib.pyx":114
 *             dtype = np.dtype(np.int64)
 *         else:
 *             try:             # <<<<<<<<<<<<<<
 *                 dtype = np.dtype(dtype)
 *             except TypeError as e:
*/
    /*else*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        /*try:*/ {

          /* "nvtx/_lib/lib.pyx":115
 *         else:
 *             try:
 *                 dtype = np.dtype(dtype)             # <<<<<<<<<<<<<<
 *             except TypeError as e:
 *                 raise TypeError("Counter dtype must be int, float, or NumPy dtype-like.") from e
*/
          __pyx_t_5 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 115, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 115, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_7 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
            assert(__pyx_t_5);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
            __pyx_t_7 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_dtype};
            __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 115, __pyx_L5_error)
            __Pyx_GOTREF(__pyx_t_1);
          }
          __Pyx_DECREF_SET(__pyx_v_dtype, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "nvtx/_lib/lib.pyx":114
 *             dtype = np.dtype(np.int64)
 *         else:
 *             try:             # <<<<<<<<<<<<<<
 *                 dtype = np.dtype(dtype)
 *             except TypeError as e:
*/
        }
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        goto __pyx_L10_try_end;
        __pyx_L5_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "nvtx/_lib/lib.pyx":116
 *             try:
 *                 dtype = np.dtype(dtype)
 *             except TypeError as e:             # <<<<<<<<<<<<<<
 *                 raise TypeError("Counter dtype must be int, float, or NumPy dtype-like.") from e
 *     _validate_counter_group_dtype(dtype)
*/
        __pyx_t_11 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_TypeError))));
        if (__pyx_t_11) {
          __Pyx_AddTraceback("nvtx._lib.lib._normalize_counter_dtype", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_3, &__pyx_t_5) < 0) __PYX_ERR(0, 116, __pyx_L7_except_error)
          __Pyx_XGOTREF(__pyx_t_1);
          __Pyx_XGOTREF(__pyx_t_3);
          __Pyx_XGOTREF(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_3);
          __pyx_v_e = __pyx_t_3;
          /*try:*/ {

            /* "nvtx/_lib/lib.pyx":117
 *                 dtype = np.dtype(dtype)
 *             except TypeError as e:
 *                 raise TypeError("Counter dtype must be int, float, or NumPy dtype-like.") from e             # <<<<<<<<<<<<<<
 *     _validate_counter_group_dtype(dtype)
 *     return dtype
*/
            __pyx_t_4 = NULL;
            __pyx_t_7 = 1;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Counter_dtype_must_be_int_float};
              __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 117, __pyx_L16_error)
              __Pyx_GOTREF(__pyx_t_6);
            }
            __Pyx_Raise(__pyx_t_6, 0, 0, __pyx_v_e);
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __PYX_ERR(0, 117, __pyx_L16_error)
          }

          /* "nvtx/_lib/lib.pyx":116
 *             try:
 *                 dtype = np.dtype(dtype)
 *             except TypeError as e:             # <<<<<<<<<<<<<<
 *                 raise TypeError("Counter dtype must be int, float, or NumPy dtype-like.") from e
 *     _validate_counter_group_dtype(dtype)
*/
          /*finally:*/ {
            __pyx_L16_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
               __Pyx_ExceptionSwap(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
              if ( unlikely(__Pyx_GetException(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16) < 0)) __Pyx_ErrFetch(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
              __Pyx_XGOTREF(__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_15);
              __Pyx_XGOTREF(__pyx_t_16);
              __Pyx_XGOTREF(__pyx_t_17);
              __Pyx_XGOTREF(__pyx_t_18);
              __Pyx_XGOTREF(__pyx_t_19);
              __pyx_t_11 = __pyx_lineno; __pyx_t_12 = __pyx_clineno; __pyx_t_13 = __pyx_filename;
              {
                __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
              }
              __Pyx_XGIVEREF(__pyx_t_17);
              __Pyx_XGIVEREF(__pyx_t_18);
              __Pyx_XGIVEREF(__pyx_t_19);
              __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_XGIVEREF(__pyx_t_15);
              __Pyx_XGIVEREF(__pyx_t_16);
              __Pyx_ErrRestore(__pyx_t_14, __pyx_t_15, __pyx_t_16);
              __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
              __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_12; __pyx_filename = __pyx_t_13;
              goto __pyx_L7_except_error;
            }
          }
        }
        goto __pyx_L7_except_error;

        /* "nvtx/_lib/lib.pyx":114
 *             dtype = np.dtype(np.int64)
 *         else:
 *             try:             # <<<<<<<<<<<<<<
 *                 dtype = np.dtype(dtype)
 *             except TypeError as e:
*/
        __pyx_L7_except_error:;
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
        goto __pyx_L1_error;
        __pyx_L10_try_end:;
      }
    }
    __pyx_L4:;

    /* "nvtx/_lib/lib.pyx":109
 *     """
 * 
 *     if np is not None:             # <<<<<<<<<<<<<<
 *         if dtype is int:
 *             # Force 64-bit integer (64-bit Windows uses int32 for plain ``int``).
*/
  }

  /* "nvtx/_lib/lib.pyx":118
 *             except TypeError as e:
 *                 raise TypeError("Counter dtype must be int, float, or NumPy dtype-like.") from e
 *     _validate_counter_group_dtype(dtype)             # <<<<<<<<<<<<<<
 *     return dtype
 * 
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_validate_counter_group_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
    __pyx_t_7 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_dtype};
    __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "nvtx/_lib/lib.pyx":119
 *                 raise TypeError("Counter dtype must be int, float, or NumPy dtype-like.") from e
 *     _validate_counter_group_dtype(dtype)
 *     return dtype             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_dtype);
  __pyx_r = __pyx_v_dtype;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":104
 * 
 * 
 * def _normalize_counter_dtype(dtype):             # <<<<<<<<<<<<<<
 *     """
 *     Normalize dtype-like counter specifications before caching.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("nvtx._lib.lib._normalize_counter_dtype", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":122
 * 
 * 
 * def _counter_semantics_from_metadata(metadata):             # <<<<<<<<<<<<<<
 *     """
 *     Return counter semantics stored in NVTX dtype metadata, if present.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_9_counter_semantics_from_metadata(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_8_counter_semantics_from_metadata, "_counter_semantics_from_metadata(metadata)\n\nReturn counter semantics stored in NVTX dtype metadata, if present.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_9_counter_semantics_from_metadata = {"_counter_semantics_from_metadata", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_9_counter_semantics_from_metadata, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_8_counter_semantics_from_metadata};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_9_counter_semantics_from_metadata(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_metadata = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_counter_semantics_from_metadata (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_metadata,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 122, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 122, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_counter_semantics_from_metadata", 0) < (0)) __PYX_ERR(0, 122, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_counter_semantics_from_metadata", 1, 1, 1, i); __PYX_ERR(0, 122, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 122, __pyx_L3_error)
    }
    __pyx_v_metadata = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_counter_semantics_from_metadata", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 122, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib._counter_semantics_from_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_8_counter_semantics_from_metadata(__pyx_self, __pyx_v_metadata);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_8_counter_semantics_from_metadata(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_metadata) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_counter_semantics_from_metadata", 0);

  /* "nvtx/_lib/lib.pyx":127
 *     """
 * 
 *     if metadata is None:             # <<<<<<<<<<<<<<
 *         return None
 *     return getattr(metadata, "counter_semantics", None)
*/
  __pyx_t_1 = (__pyx_v_metadata == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":128
 * 
 *     if metadata is None:
 *         return None             # <<<<<<<<<<<<<<
 *     return getattr(metadata, "counter_semantics", None)
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":127
 *     """
 * 
 *     if metadata is None:             # <<<<<<<<<<<<<<
 *         return None
 *     return getattr(metadata, "counter_semantics", None)
*/
  }

  /* "nvtx/_lib/lib.pyx":129
 *     if metadata is None:
 *         return None
 *     return getattr(metadata, "counter_semantics", None)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_metadata, __pyx_mstate_global->__pyx_n_u_counter_semantics, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":122
 * 
 * 
 * def _counter_semantics_from_metadata(metadata):             # <<<<<<<<<<<<<<
 *     """
 *     Return counter semantics stored in NVTX dtype metadata, if present.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib._counter_semantics_from_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":152
 * 
 * 
 * def _entry_flags_from_metadata(metadata):             # <<<<<<<<<<<<<<
 *     """
 *     Return the resolved NVTX payload entry flags stored in dtype metadata, if
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11_entry_flags_from_metadata(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_10_entry_flags_from_metadata, "_entry_flags_from_metadata(metadata)\n\nReturn the resolved NVTX payload entry flags stored in dtype metadata, if\npresent (0 when absent).");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11_entry_flags_from_metadata = {"_entry_flags_from_metadata", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11_entry_flags_from_metadata, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_10_entry_flags_from_metadata};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11_entry_flags_from_metadata(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_metadata = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_entry_flags_from_metadata (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_metadata,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 152, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 152, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_entry_flags_from_metadata", 0) < (0)) __PYX_ERR(0, 152, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_entry_flags_from_metadata", 1, 1, 1, i); __PYX_ERR(0, 152, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 152, __pyx_L3_error)
    }
    __pyx_v_metadata = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_entry_flags_from_metadata", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 152, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib._entry_flags_from_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_10_entry_flags_from_metadata(__pyx_self, __pyx_v_metadata);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_10_entry_flags_from_metadata(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_metadata) {
  PyObject *__pyx_v_entry_kind = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_entry_flags_from_metadata", 0);

  /* "nvtx/_lib/lib.pyx":158
 *     """
 * 
 *     if metadata is None:             # <<<<<<<<<<<<<<
 *         return 0
 *     entry_kind = getattr(metadata, "entry_kind", None)
*/
  __pyx_t_1 = (__pyx_v_metadata == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":159
 * 
 *     if metadata is None:
 *         return 0             # <<<<<<<<<<<<<<
 *     entry_kind = getattr(metadata, "entry_kind", None)
 *     return entry_kind.value if entry_kind is not None else 0
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
    __pyx_r = __pyx_mstate_global->__pyx_int_0;
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":158
 *     """
 * 
 *     if metadata is None:             # <<<<<<<<<<<<<<
 *         return 0
 *     entry_kind = getattr(metadata, "entry_kind", None)
*/
  }

  /* "nvtx/_lib/lib.pyx":160
 *     if metadata is None:
 *         return 0
 *     entry_kind = getattr(metadata, "entry_kind", None)             # <<<<<<<<<<<<<<
 *     return entry_kind.value if entry_kind is not None else 0
 * 
*/
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_metadata, __pyx_mstate_global->__pyx_n_u_entry_kind, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_entry_kind = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":161
 *         return 0
 *     entry_kind = getattr(metadata, "entry_kind", None)
 *     return entry_kind.value if entry_kind is not None else 0             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = (__pyx_v_entry_kind != Py_None);
  if (__pyx_t_1) {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_entry_kind, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 161, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
    __pyx_t_2 = __pyx_mstate_global->__pyx_int_0;
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":152
 * 
 * 
 * def _entry_flags_from_metadata(metadata):             # <<<<<<<<<<<<<<
 *     """
 *     Return the resolved NVTX payload entry flags stored in dtype metadata, if
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib._entry_flags_from_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_entry_kind);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":164
 * 
 * 
 * def _entry_type_from_metadata(metadata):             # <<<<<<<<<<<<<<
 *     """Return an explicit NVTX payload entry type, or ``None``."""
 *     if metadata is None:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_13_entry_type_from_metadata(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_12_entry_type_from_metadata, "_entry_type_from_metadata(metadata)\n\nReturn an explicit NVTX payload entry type, or ``None``.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_13_entry_type_from_metadata = {"_entry_type_from_metadata", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_13_entry_type_from_metadata, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12_entry_type_from_metadata};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_13_entry_type_from_metadata(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_metadata = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_entry_type_from_metadata (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_metadata,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 164, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 164, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_entry_type_from_metadata", 0) < (0)) __PYX_ERR(0, 164, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_entry_type_from_metadata", 1, 1, 1, i); __PYX_ERR(0, 164, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 164, __pyx_L3_error)
    }
    __pyx_v_metadata = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_entry_type_from_metadata", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 164, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib._entry_type_from_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12_entry_type_from_metadata(__pyx_self, __pyx_v_metadata);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12_entry_type_from_metadata(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_metadata) {
  PyObject *__pyx_v_entry_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_entry_type_from_metadata", 0);

  /* "nvtx/_lib/lib.pyx":166
 * def _entry_type_from_metadata(metadata):
 *     """Return an explicit NVTX payload entry type, or ``None``."""
 *     if metadata is None:             # <<<<<<<<<<<<<<
 *         return None
 *     entry_type = getattr(metadata, "entry_type", None)
*/
  __pyx_t_1 = (__pyx_v_metadata == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":167
 *     """Return an explicit NVTX payload entry type, or ``None``."""
 *     if metadata is None:
 *         return None             # <<<<<<<<<<<<<<
 *     entry_type = getattr(metadata, "entry_type", None)
 *     return entry_type.value if entry_type is not None else None
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":166
 * def _entry_type_from_metadata(metadata):
 *     """Return an explicit NVTX payload entry type, or ``None``."""
 *     if metadata is None:             # <<<<<<<<<<<<<<
 *         return None
 *     entry_type = getattr(metadata, "entry_type", None)
*/
  }

  /* "nvtx/_lib/lib.pyx":168
 *     if metadata is None:
 *         return None
 *     entry_type = getattr(metadata, "entry_type", None)             # <<<<<<<<<<<<<<
 *     return entry_type.value if entry_type is not None else None
 * 
*/
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_metadata, __pyx_mstate_global->__pyx_n_u_entry_type, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_entry_type = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":169
 *         return None
 *     entry_type = getattr(metadata, "entry_type", None)
 *     return entry_type.value if entry_type is not None else None             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = (__pyx_v_entry_type != Py_None);
  if (__pyx_t_1) {
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_entry_type, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __Pyx_INCREF(Py_None);
    __pyx_t_2 = Py_None;
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":164
 * 
 * 
 * def _entry_type_from_metadata(metadata):             # <<<<<<<<<<<<<<
 *     """Return an explicit NVTX payload entry type, or ``None``."""
 *     if metadata is None:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib._entry_type_from_metadata", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_entry_type);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":242
 * 
 * 
 * def payload_setter(type):             # <<<<<<<<<<<<<<
 *     """
 *     A helper decorator to register a payload setter for a given type.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15payload_setter(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_14payload_setter, "payload_setter(type)\n\nA helper decorator to register a payload setter for a given type.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15payload_setter = {"payload_setter", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15payload_setter, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_14payload_setter};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15payload_setter(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_type = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("payload_setter (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_type,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 242, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 242, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "payload_setter", 0) < (0)) __PYX_ERR(0, 242, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("payload_setter", 1, 1, 1, i); __PYX_ERR(0, 242, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 242, __pyx_L3_error)
    }
    __pyx_v_type = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("payload_setter", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 242, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.payload_setter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_14payload_setter(__pyx_self, __pyx_v_type);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":246
 *     A helper decorator to register a payload setter for a given type.
 *     """
 *     def register(func):             # <<<<<<<<<<<<<<
 *         _payload_setters[type] = func
 *         return func
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_14payload_setter_1register(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_14payload_setter_1register = {"register", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_14payload_setter_1register, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_14payload_setter_1register(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_func = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("register (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_func,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 246, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 246, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "register", 0) < (0)) __PYX_ERR(0, 246, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("register", 1, 1, 1, i); __PYX_ERR(0, 246, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 246, __pyx_L3_error)
    }
    __pyx_v_func = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("register", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 246, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.payload_setter.register", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_14payload_setter_register(__pyx_self, __pyx_v_func);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_14payload_setter_register(PyObject *__pyx_self, PyObject *__pyx_v_func) {
  struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *__pyx_cur_scope;
  struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("register", 0);
  __pyx_outer_scope = (struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;

  /* "nvtx/_lib/lib.pyx":247
 *     """
 *     def register(func):
 *         _payload_setters[type] = func             # <<<<<<<<<<<<<<
 *         return func
 *     return register
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_payload_setters); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(!__pyx_cur_scope->__pyx_v_type)) { __Pyx_RaiseClosureNameError("type"); __PYX_ERR(0, 247, __pyx_L1_error) }
  if (unlikely((PyObject_SetItem(__pyx_t_1, __pyx_cur_scope->__pyx_v_type, __pyx_v_func) < 0))) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":248
 *     def register(func):
 *         _payload_setters[type] = func
 *         return func             # <<<<<<<<<<<<<<
 *     return register
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_func);
  __pyx_r = __pyx_v_func;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":246
 *     A helper decorator to register a payload setter for a given type.
 *     """
 *     def register(func):             # <<<<<<<<<<<<<<
 *         _payload_setters[type] = func
 *         return func
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.payload_setter.register", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":242
 * 
 * 
 * def payload_setter(type):             # <<<<<<<<<<<<<<
 *     """
 *     A helper decorator to register a payload setter for a given type.
*/

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_14payload_setter(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_type) {
  struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *__pyx_cur_scope;
  PyObject *__pyx_v_register = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("payload_setter", 0);
  __pyx_cur_scope = (struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *)__pyx_tp_new_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 242, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_type = __pyx_v_type;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_type);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_type);

  /* "nvtx/_lib/lib.pyx":246
 *     A helper decorator to register a payload setter for a given type.
 *     """
 *     def register(func):             # <<<<<<<<<<<<<<
 *         _payload_setters[type] = func
 *         return func
*/
  __pyx_t_1 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_14payload_setter_1register, 0, __pyx_mstate_global->__pyx_n_u_payload_setter_locals_register, ((PyObject*)__pyx_cur_scope), __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_register = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":249
 *         _payload_setters[type] = func
 *         return func
 *     return register             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_register);
  __pyx_r = __pyx_v_register;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":242
 * 
 * 
 * def payload_setter(type):             # <<<<<<<<<<<<<<
 *     """
 *     A helper decorator to register a payload setter for a given type.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.payload_setter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_register);
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":279
 *     """
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._clear_payload()
 * 
*/

/* Python wrapper */
static void __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_1__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_1__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes___dealloc__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes___dealloc__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "nvtx/_lib/lib.pyx":280
 * 
 *     def __dealloc__(self):
 *         self._clear_payload()             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, object domain, object message=None, color=None, category=None,
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self->__pyx_vtab)->_clear_payload(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":279
 *     """
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self._clear_payload()
 * 
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("nvtx._lib.lib.EventAttributes.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "nvtx/_lib/lib.pyx":282
 *         self._clear_payload()
 * 
 *     def __init__(self, object domain, object message=None, color=None, category=None,             # <<<<<<<<<<<<<<
 *                  payload=None):
 *         self.domain = domain
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_domain = 0;
  PyObject *__pyx_v_message = 0;
  PyObject *__pyx_v_color = 0;
  PyObject *__pyx_v_category = 0;
  PyObject *__pyx_v_payload = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_domain,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 282, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 282, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 282, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 282, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 282, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 282, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 282, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":283
 * 
 *     def __init__(self, object domain, object message=None, color=None, category=None,
 *                  payload=None):             # <<<<<<<<<<<<<<
 *         self.domain = domain
 *         self.c_obj.version = NVTX_VERSION
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 5, i); __PYX_ERR(0, 282, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 282, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 282, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 282, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 282, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 282, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "nvtx/_lib/lib.pyx":282
 *         self._clear_payload()
 * 
 *     def __init__(self, object domain, object message=None, color=None, category=None,             # <<<<<<<<<<<<<<
 *                  payload=None):
 *         self.domain = domain
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":283
 * 
 *     def __init__(self, object domain, object message=None, color=None, category=None,
 *                  payload=None):             # <<<<<<<<<<<<<<
 *         self.domain = domain
 *         self.c_obj.version = NVTX_VERSION
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_domain = values[0];
    __pyx_v_message = values[1];
    __pyx_v_color = values[2];
    __pyx_v_category = values[3];
    __pyx_v_payload = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 5, __pyx_nargs); __PYX_ERR(0, 282, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_2__init__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), __pyx_v_domain, __pyx_v_message, __pyx_v_color, __pyx_v_category, __pyx_v_payload);

  /* "nvtx/_lib/lib.pyx":282
 *         self._clear_payload()
 * 
 *     def __init__(self, object domain, object message=None, color=None, category=None,             # <<<<<<<<<<<<<<
 *                  payload=None):
 *         self.domain = domain
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_2__init__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_domain, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, PyObject *__pyx_v_payload) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "nvtx/_lib/lib.pyx":284
 *     def __init__(self, object domain, object message=None, color=None, category=None,
 *                  payload=None):
 *         self.domain = domain             # <<<<<<<<<<<<<<
 *         self.c_obj.version = NVTX_VERSION
 *         self.c_obj.size = NVTX_EVENT_ATTRIB_STRUCT_SIZE
*/
  __Pyx_INCREF(__pyx_v_domain);
  __Pyx_GIVEREF(__pyx_v_domain);
  __Pyx_GOTREF(__pyx_v_self->domain);
  __Pyx_DECREF(__pyx_v_self->domain);
  __pyx_v_self->domain = __pyx_v_domain;

  /* "nvtx/_lib/lib.pyx":285
 *                  payload=None):
 *         self.domain = domain
 *         self.c_obj.version = NVTX_VERSION             # <<<<<<<<<<<<<<
 *         self.c_obj.size = NVTX_EVENT_ATTRIB_STRUCT_SIZE
 *         self.c_obj.colorType = NVTX_COLOR_ARGB
*/
  __pyx_v_self->c_obj.version = NVTX_VERSION;

  /* "nvtx/_lib/lib.pyx":286
 *         self.domain = domain
 *         self.c_obj.version = NVTX_VERSION
 *         self.c_obj.size = NVTX_EVENT_ATTRIB_STRUCT_SIZE             # <<<<<<<<<<<<<<
 *         self.c_obj.colorType = NVTX_COLOR_ARGB
 * 
*/
  __pyx_v_self->c_obj.size = NVTX_EVENT_ATTRIB_STRUCT_SIZE;

  /* "nvtx/_lib/lib.pyx":287
 *         self.c_obj.version = NVTX_VERSION
 *         self.c_obj.size = NVTX_EVENT_ATTRIB_STRUCT_SIZE
 *         self.c_obj.colorType = NVTX_COLOR_ARGB             # <<<<<<<<<<<<<<
 * 
 *         self.message = message
*/
  __pyx_v_self->c_obj.colorType = NVTX_COLOR_ARGB;

  /* "nvtx/_lib/lib.pyx":289
 *         self.c_obj.colorType = NVTX_COLOR_ARGB
 * 
 *         self.message = message             # <<<<<<<<<<<<<<
 *         self.color = color
 *         self.category = category
*/
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_message, __pyx_v_message) < (0)) __PYX_ERR(0, 289, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":290
 * 
 *         self.message = message
 *         self.color = color             # <<<<<<<<<<<<<<
 *         self.category = category
 *         self.payload = payload
*/
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_color, __pyx_v_color) < (0)) __PYX_ERR(0, 290, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":291
 *         self.message = message
 *         self.color = color
 *         self.category = category             # <<<<<<<<<<<<<<
 *         self.payload = payload
 * 
*/
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_category, __pyx_v_category) < (0)) __PYX_ERR(0, 291, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":292
 *         self.color = color
 *         self.category = category
 *         self.payload = payload             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_payload, __pyx_v_payload) < (0)) __PYX_ERR(0, 292, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":282
 *         self._clear_payload()
 * 
 *     def __init__(self, object domain, object message=None, color=None, category=None,             # <<<<<<<<<<<<<<
 *                  payload=None):
 *         self.domain = domain
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":294
 *         self.payload = payload
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def message(self):
 *         return self._message
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7message_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7message_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7message___get__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7message___get__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/lib.pyx":296
 *     @property
 *     def message(self):
 *         return self._message             # <<<<<<<<<<<<<<
 * 
 *     @message.setter
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_message);
  __pyx_r = __pyx_v_self->_message;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":294
 *         self.payload = payload
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def message(self):
 *         return self._message
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":298
 *         return self._message
 * 
 *     @message.setter             # <<<<<<<<<<<<<<
 *     def message(self, object value):
 *         if value is None:
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7message_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7message_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7message_2__set__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7message_2__set__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  nvtxStringHandle_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "nvtx/_lib/lib.pyx":300
 *     @message.setter
 *     def message(self, object value):
 *         if value is None:             # <<<<<<<<<<<<<<
 *             self.c_obj.messageType = NVTX_MESSAGE_UNKNOWN
 *             self._message = None
*/
  __pyx_t_1 = (__pyx_v_value == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":301
 *     def message(self, object value):
 *         if value is None:
 *             self.c_obj.messageType = NVTX_MESSAGE_UNKNOWN             # <<<<<<<<<<<<<<
 *             self._message = None
 *         else:
*/
    __pyx_v_self->c_obj.messageType = NVTX_MESSAGE_UNKNOWN;

    /* "nvtx/_lib/lib.pyx":302
 *         if value is None:
 *             self.c_obj.messageType = NVTX_MESSAGE_UNKNOWN
 *             self._message = None             # <<<<<<<<<<<<<<
 *         else:
 *             self.c_obj.messageType = NVTX_MESSAGE_TYPE_REGISTERED
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_message);
    __Pyx_DECREF(__pyx_v_self->_message);
    __pyx_v_self->_message = Py_None;

    /* "nvtx/_lib/lib.pyx":300
 *     @message.setter
 *     def message(self, object value):
 *         if value is None:             # <<<<<<<<<<<<<<
 *             self.c_obj.messageType = NVTX_MESSAGE_UNKNOWN
 *             self._message = None
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/lib.pyx":304
 *             self._message = None
 *         else:
 *             self.c_obj.messageType = NVTX_MESSAGE_TYPE_REGISTERED             # <<<<<<<<<<<<<<
 *             self._message = value
 *             self.c_obj.message.registered = (<StringHandle> self._message.handle).c_obj
*/
  /*else*/ {
    __pyx_v_self->c_obj.messageType = NVTX_MESSAGE_TYPE_REGISTERED;

    /* "nvtx/_lib/lib.pyx":305
 *         else:
 *             self.c_obj.messageType = NVTX_MESSAGE_TYPE_REGISTERED
 *             self._message = value             # <<<<<<<<<<<<<<
 *             self.c_obj.message.registered = (<StringHandle> self._message.handle).c_obj
 * 
*/
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    __Pyx_GOTREF(__pyx_v_self->_message);
    __Pyx_DECREF(__pyx_v_self->_message);
    __pyx_v_self->_message = __pyx_v_value;

    /* "nvtx/_lib/lib.pyx":306
 *             self.c_obj.messageType = NVTX_MESSAGE_TYPE_REGISTERED
 *             self._message = value
 *             self.c_obj.message.registered = (<StringHandle> self._message.handle).c_obj             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_message, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 306, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = ((struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *)__pyx_t_2)->c_obj;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->c_obj.message.registered = __pyx_t_3;
  }
  __pyx_L3:;

  /* "nvtx/_lib/lib.pyx":298
 *         return self._message
 * 
 *     @message.setter             # <<<<<<<<<<<<<<
 *     def message(self, object value):
 *         if value is None:
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.message.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":308
 *             self.c_obj.message.registered = (<StringHandle> self._message.handle).c_obj
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def color(self):
 *         return self._color
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5color_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5color_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_5color___get__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_5color___get__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/lib.pyx":310
 *     @property
 *     def color(self):
 *         return self._color             # <<<<<<<<<<<<<<
 * 
 *     @color.setter
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_color);
  __pyx_r = __pyx_v_self->_color;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":308
 *             self.c_obj.message.registered = (<StringHandle> self._message.handle).c_obj
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def color(self):
 *         return self._color
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":312
 *         return self._color
 * 
 *     @color.setter             # <<<<<<<<<<<<<<
 *     def color(self, value):
 *         self._color = value
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5color_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5color_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_5color_2__set__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_5color_2__set__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  uint32_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "nvtx/_lib/lib.pyx":314
 *     @color.setter
 *     def color(self, value):
 *         self._color = value             # <<<<<<<<<<<<<<
 *         self.c_obj.color = color_to_hex(self._color)
 * 
*/
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_color);
  __Pyx_DECREF(__pyx_v_self->_color);
  __pyx_v_self->_color = __pyx_v_value;

  /* "nvtx/_lib/lib.pyx":315
 *     def color(self, value):
 *         self._color = value
 *         self.c_obj.color = color_to_hex(self._color)             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_color_to_hex); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_self->_color};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_PyLong_As_uint32_t(__pyx_t_1); if (unlikely((__pyx_t_5 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 315, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->c_obj.color = __pyx_t_5;

  /* "nvtx/_lib/lib.pyx":312
 *         return self._color
 * 
 *     @color.setter             # <<<<<<<<<<<<<<
 *     def color(self, value):
 *         self._color = value
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.color.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":317
 *         self.c_obj.color = color_to_hex(self._color)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def category(self):
 *         return self._category
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_8category_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_8category_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_8category___get__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_8category___get__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/lib.pyx":319
 *     @property
 *     def category(self):
 *         return self._category             # <<<<<<<<<<<<<<
 * 
 *     @category.setter
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_uint32_t(__pyx_v_self->_category); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":317
 *         self.c_obj.color = color_to_hex(self._color)
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def category(self):
 *         return self._category
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.category.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":321
 *         return self._category
 * 
 *     @category.setter             # <<<<<<<<<<<<<<
 *     def category(self, value):
 *         if value is None:
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_8category_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_8category_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_8category_2__set__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_8category_2__set__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  uint32_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);

  /* "nvtx/_lib/lib.pyx":323
 *     @category.setter
 *     def category(self, value):
 *         if value is None:             # <<<<<<<<<<<<<<
 *             value = 0
 *         self._category = value
*/
  __pyx_t_1 = (__pyx_v_value == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":324
 *     def category(self, value):
 *         if value is None:
 *             value = 0             # <<<<<<<<<<<<<<
 *         self._category = value
 *         self.c_obj.category = value
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_mstate_global->__pyx_int_0);

    /* "nvtx/_lib/lib.pyx":323
 *     @category.setter
 *     def category(self, value):
 *         if value is None:             # <<<<<<<<<<<<<<
 *             value = 0
 *         self._category = value
*/
  }

  /* "nvtx/_lib/lib.pyx":325
 *         if value is None:
 *             value = 0
 *         self._category = value             # <<<<<<<<<<<<<<
 *         self.c_obj.category = value
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_As_uint32_t(__pyx_v_value); if (unlikely((__pyx_t_2 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 325, __pyx_L1_error)
  __pyx_v_self->_category = __pyx_t_2;

  /* "nvtx/_lib/lib.pyx":326
 *             value = 0
 *         self._category = value
 *         self.c_obj.category = value             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
  __pyx_t_2 = __Pyx_PyLong_As_uint32_t(__pyx_v_value); if (unlikely((__pyx_t_2 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 326, __pyx_L1_error)
  __pyx_v_self->c_obj.category = __pyx_t_2;

  /* "nvtx/_lib/lib.pyx":321
 *         return self._category
 * 
 *     @category.setter             # <<<<<<<<<<<<<<
 *     def category(self, value):
 *         if value is None:
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.category.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":328
 *         self.c_obj.category = value
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def payload(self):
 *         return self._payload
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7payload_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7payload_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7payload___get__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7payload___get__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/lib.pyx":330
 *     @property
 *     def payload(self):
 *         return self._payload             # <<<<<<<<<<<<<<
 * 
 *     @payload.setter
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_payload);
  __pyx_r = __pyx_v_self->_payload;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":328
 *         self.c_obj.category = value
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def payload(self):
 *         return self._payload
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":332
 *         return self._payload
 * 
 *     @payload.setter             # <<<<<<<<<<<<<<
 *     def payload(self, value):
 *         self._clear_payload()
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7payload_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7payload_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7payload_2__set__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_7payload_2__set__(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_setter = NULL;
  PyObject *__pyx_v_msg = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7[3];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "nvtx/_lib/lib.pyx":334
 *     @payload.setter
 *     def payload(self, value):
 *         self._clear_payload()             # <<<<<<<<<<<<<<
 *         if value is None:
 *             self.c_obj.payloadType = NVTX_PAYLOAD_UNKNOWN
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self->__pyx_vtab)->_clear_payload(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":335
 *     def payload(self, value):
 *         self._clear_payload()
 *         if value is None:             # <<<<<<<<<<<<<<
 *             self.c_obj.payloadType = NVTX_PAYLOAD_UNKNOWN
 *             return
*/
  __pyx_t_2 = (__pyx_v_value == Py_None);
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":336
 *         self._clear_payload()
 *         if value is None:
 *             self.c_obj.payloadType = NVTX_PAYLOAD_UNKNOWN             # <<<<<<<<<<<<<<
 *             return
 * 
*/
    __pyx_v_self->c_obj.payloadType = NVTX_PAYLOAD_UNKNOWN;

    /* "nvtx/_lib/lib.pyx":337
 *         if value is None:
 *             self.c_obj.payloadType = NVTX_PAYLOAD_UNKNOWN
 *             return             # <<<<<<<<<<<<<<
 * 
 *         self._payload = value
*/
    __pyx_r = 0;
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":335
 *     def payload(self, value):
 *         self._clear_payload()
 *         if value is None:             # <<<<<<<<<<<<<<
 *             self.c_obj.payloadType = NVTX_PAYLOAD_UNKNOWN
 *             return
*/
  }

  /* "nvtx/_lib/lib.pyx":339
 *             return
 * 
 *         self._payload = value             # <<<<<<<<<<<<<<
 *         setter = _payload_setters.get(type(value))
 *         if setter is None:
*/
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_payload);
  __Pyx_DECREF(__pyx_v_self->_payload);
  __pyx_v_self->_payload = __pyx_v_value;

  /* "nvtx/_lib/lib.pyx":340
 * 
 *         self._payload = value
 *         setter = _payload_setters.get(type(value))             # <<<<<<<<<<<<<<
 *         if setter is None:
 *             msg = f"Unsupported payload type: {type(value)}."
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_payload_setters); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_get); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, ((PyObject *)Py_TYPE(__pyx_v_value))};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_setter = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":341
 *         self._payload = value
 *         setter = _payload_setters.get(type(value))
 *         if setter is None:             # <<<<<<<<<<<<<<
 *             msg = f"Unsupported payload type: {type(value)}."
 *             if np is None:
*/
  __pyx_t_2 = (__pyx_v_setter == Py_None);
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":342
 *         setter = _payload_setters.get(type(value))
 *         if setter is None:
 *             msg = f"Unsupported payload type: {type(value)}."             # <<<<<<<<<<<<<<
 *             if np is None:
 *                 msg += " Install numpy for extended payload support."
*/
    __pyx_t_1 = __Pyx_PyObject_FormatSimple(((PyObject *)Py_TYPE(__pyx_v_value)), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7[0] = __pyx_mstate_global->__pyx_kp_u_Unsupported_payload_type;
    __pyx_t_7[1] = __pyx_t_1;
    __pyx_t_7[2] = __pyx_mstate_global->__pyx_kp_u_;
    __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_7, 3, 26 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1));
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 342, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_msg = ((PyObject*)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "nvtx/_lib/lib.pyx":343
 *         if setter is None:
 *             msg = f"Unsupported payload type: {type(value)}."
 *             if np is None:             # <<<<<<<<<<<<<<
 *                 msg += " Install numpy for extended payload support."
 *             warnings.warn(msg, NvtxWarning)
*/
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 343, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = (__pyx_t_5 == Py_None);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_2) {

      /* "nvtx/_lib/lib.pyx":344
 *             msg = f"Unsupported payload type: {type(value)}."
 *             if np is None:
 *                 msg += " Install numpy for extended payload support."             # <<<<<<<<<<<<<<
 *             warnings.warn(msg, NvtxWarning)
 *             return
*/
      __pyx_t_5 = __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(__pyx_v_msg, __pyx_mstate_global->__pyx_kp_u_Install_numpy_for_extended_payl); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 344, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF_SET(__pyx_v_msg, ((PyObject*)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "nvtx/_lib/lib.pyx":343
 *         if setter is None:
 *             msg = f"Unsupported payload type: {type(value)}."
 *             if np is None:             # <<<<<<<<<<<<<<
 *                 msg += " Install numpy for extended payload support."
 *             warnings.warn(msg, NvtxWarning)
*/
    }

    /* "nvtx/_lib/lib.pyx":345
 *             if np is None:
 *                 msg += " Install numpy for extended payload support."
 *             warnings.warn(msg, NvtxWarning)             # <<<<<<<<<<<<<<
 *             return
 *         setter(self, value)
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_warnings); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_warn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_NvtxWarning); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_1, __pyx_v_msg, __pyx_t_3};
      __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 345, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "nvtx/_lib/lib.pyx":346
 *                 msg += " Install numpy for extended payload support."
 *             warnings.warn(msg, NvtxWarning)
 *             return             # <<<<<<<<<<<<<<
 *         setter(self, value)
 * 
*/
    __pyx_r = 0;
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":341
 *         self._payload = value
 *         setter = _payload_setters.get(type(value))
 *         if setter is None:             # <<<<<<<<<<<<<<
 *             msg = f"Unsupported payload type: {type(value)}."
 *             if np is None:
*/
  }

  /* "nvtx/_lib/lib.pyx":347
 *             warnings.warn(msg, NvtxWarning)
 *             return
 *         setter(self, value)             # <<<<<<<<<<<<<<
 * 
 *     @payload_setter(int)
*/
  __pyx_t_4 = NULL;
  __Pyx_INCREF(__pyx_v_setter);
  __pyx_t_3 = __pyx_v_setter; 
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_4, ((PyObject *)__pyx_v_self), __pyx_v_value};
    __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 347, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "nvtx/_lib/lib.pyx":332
 *         return self._payload
 * 
 *     @payload.setter             # <<<<<<<<<<<<<<
 *     def payload(self, value):
 *         self._clear_payload()
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.payload.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_setter);
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":349
 *         setter(self, value)
 * 
 *     @payload_setter(int)             # <<<<<<<<<<<<<<
 *     def _set_payload_int(self, payload):
 *         self.c_obj.payload.llValue = payload
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5_set_payload_int(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_4_set_payload_int, "EventAttributes._set_payload_int(self, payload)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_5_set_payload_int = {"_set_payload_int", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5_set_payload_int, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_4_set_payload_int};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5_set_payload_int(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_set_payload_int (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 349, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 349, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_set_payload_int", 0) < (0)) __PYX_ERR(0, 349, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_set_payload_int", 1, 1, 1, i); __PYX_ERR(0, 349, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 349, __pyx_L3_error)
    }
    __pyx_v_payload = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_set_payload_int", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 349, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes._set_payload_int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_4_set_payload_int(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), __pyx_v_payload);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_4_set_payload_int(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_payload) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int64_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_payload_int", 0);

  /* "nvtx/_lib/lib.pyx":351
 *     @payload_setter(int)
 *     def _set_payload_int(self, payload):
 *         self.c_obj.payload.llValue = payload             # <<<<<<<<<<<<<<
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_INT64
 * 
*/
  __pyx_t_1 = __Pyx_PyLong_As_int64_t(__pyx_v_payload); if (unlikely((__pyx_t_1 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 351, __pyx_L1_error)
  __pyx_v_self->c_obj.payload.llValue = __pyx_t_1;

  /* "nvtx/_lib/lib.pyx":352
 *     def _set_payload_int(self, payload):
 *         self.c_obj.payload.llValue = payload
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_INT64             # <<<<<<<<<<<<<<
 * 
 *     @payload_setter(float)
*/
  __pyx_v_self->c_obj.payloadType = NVTX_PAYLOAD_TYPE_INT64;

  /* "nvtx/_lib/lib.pyx":349
 *         setter(self, value)
 * 
 *     @payload_setter(int)             # <<<<<<<<<<<<<<
 *     def _set_payload_int(self, payload):
 *         self.c_obj.payload.llValue = payload
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes._set_payload_int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":354
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_INT64
 * 
 *     @payload_setter(float)             # <<<<<<<<<<<<<<
 *     def _set_payload_float(self, payload):
 *         self.c_obj.payload.dValue = payload
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7_set_payload_float(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_6_set_payload_float, "EventAttributes._set_payload_float(self, payload)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_7_set_payload_float = {"_set_payload_float", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7_set_payload_float, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_6_set_payload_float};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7_set_payload_float(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_set_payload_float (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 354, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 354, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_set_payload_float", 0) < (0)) __PYX_ERR(0, 354, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_set_payload_float", 1, 1, 1, i); __PYX_ERR(0, 354, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 354, __pyx_L3_error)
    }
    __pyx_v_payload = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_set_payload_float", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 354, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes._set_payload_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_6_set_payload_float(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), __pyx_v_payload);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_6_set_payload_float(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_payload) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_payload_float", 0);

  /* "nvtx/_lib/lib.pyx":356
 *     @payload_setter(float)
 *     def _set_payload_float(self, payload):
 *         self.c_obj.payload.dValue = payload             # <<<<<<<<<<<<<<
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_DOUBLE
 * 
*/
  __pyx_t_1 = __Pyx_PyFloat_AsDouble(__pyx_v_payload); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 356, __pyx_L1_error)
  __pyx_v_self->c_obj.payload.dValue = __pyx_t_1;

  /* "nvtx/_lib/lib.pyx":357
 *     def _set_payload_float(self, payload):
 *         self.c_obj.payload.dValue = payload
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_DOUBLE             # <<<<<<<<<<<<<<
 * 
 *     if np is not None:
*/
  __pyx_v_self->c_obj.payloadType = NVTX_PAYLOAD_TYPE_DOUBLE;

  /* "nvtx/_lib/lib.pyx":354
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_INT64
 * 
 *     @payload_setter(float)             # <<<<<<<<<<<<<<
 *     def _set_payload_float(self, payload):
 *         self.c_obj.payload.dValue = payload
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes._set_payload_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":360
 * 
 *     if np is not None:
 *         @payload_setter(np.ndarray)             # <<<<<<<<<<<<<<
 *         def _set_payload_numpy(self, payload):
 *             if payload.nbytes == 0:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_9_set_payload_numpy(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_8_set_payload_numpy, "EventAttributes._set_payload_numpy(self, payload)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_9_set_payload_numpy = {"_set_payload_numpy", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_9_set_payload_numpy, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_8_set_payload_numpy};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_9_set_payload_numpy(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_set_payload_numpy (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 360, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 360, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_set_payload_numpy", 0) < (0)) __PYX_ERR(0, 360, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_set_payload_numpy", 1, 1, 1, i); __PYX_ERR(0, 360, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 360, __pyx_L3_error)
    }
    __pyx_v_payload = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_set_payload_numpy", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 360, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes._set_payload_numpy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_8_set_payload_numpy(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), __pyx_v_payload);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_8_set_payload_numpy(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_payload) {
  PyObject *__pyx_v_schema = NULL;
  size_t __pyx_v_array_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  size_t __pyx_t_8;
  uint64_t __pyx_t_9;
  size_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_payload_numpy", 0);
  __Pyx_INCREF(__pyx_v_payload);

  /* "nvtx/_lib/lib.pyx":362
 *         @payload_setter(np.ndarray)
 *         def _set_payload_numpy(self, payload):
 *             if payload.nbytes == 0:             # <<<<<<<<<<<<<<
 *                 return
 *             schema = self.domain._get_numpy_dtype_schema(
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_nbytes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PyLong_BoolEqObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":363
 *         def _set_payload_numpy(self, payload):
 *             if payload.nbytes == 0:
 *                 return             # <<<<<<<<<<<<<<
 *             schema = self.domain._get_numpy_dtype_schema(
 *                 PayloadSchemaKey(payload.dtype)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":362
 *         @payload_setter(np.ndarray)
 *         def _set_payload_numpy(self, payload):
 *             if payload.nbytes == 0:             # <<<<<<<<<<<<<<
 *                 return
 *             schema = self.domain._get_numpy_dtype_schema(
*/
  }

  /* "nvtx/_lib/lib.pyx":364
 *             if payload.nbytes == 0:
 *                 return
 *             schema = self.domain._get_numpy_dtype_schema(             # <<<<<<<<<<<<<<
 *                 PayloadSchemaKey(payload.dtype)
 *             )
*/
  __pyx_t_3 = __pyx_v_self->domain;
  __Pyx_INCREF(__pyx_t_3);

  /* "nvtx/_lib/lib.pyx":365
 *                 return
 *             schema = self.domain._get_numpy_dtype_schema(
 *                 PayloadSchemaKey(payload.dtype)             # <<<<<<<<<<<<<<
 *             )
 *             cdef size_t array_length = 0
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_PayloadSchemaKey); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_8 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_7};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_t_8 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_schema = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":367
 *                 PayloadSchemaKey(payload.dtype)
 *             )
 *             cdef size_t array_length = 0             # <<<<<<<<<<<<<<
 *             if payload.ndim:
 *                 schema = self.domain._get_numpy_array_schema(schema)
*/
  __pyx_v_array_length = 0;

  /* "nvtx/_lib/lib.pyx":368
 *             )
 *             cdef size_t array_length = 0
 *             if payload.ndim:             # <<<<<<<<<<<<<<
 *                 schema = self.domain._get_numpy_array_schema(schema)
 *                 payload = np.ascontiguousarray(payload)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_ndim); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 368, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 368, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":369
 *             cdef size_t array_length = 0
 *             if payload.ndim:
 *                 schema = self.domain._get_numpy_array_schema(schema)             # <<<<<<<<<<<<<<
 *                 payload = np.ascontiguousarray(payload)
 *                 array_length = payload.size
*/
    __pyx_t_4 = __pyx_v_self->domain;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_8 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_schema};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_array_schema, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 369, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF_SET(__pyx_v_schema, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/lib.pyx":370
 *             if payload.ndim:
 *                 schema = self.domain._get_numpy_array_schema(schema)
 *                 payload = np.ascontiguousarray(payload)             # <<<<<<<<<<<<<<
 *                 array_length = payload.size
 * 
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 370, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_ascontiguousarray); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 370, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_8 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_payload};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_8, (2-__pyx_t_8) | (__pyx_t_8*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 370, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF_SET(__pyx_v_payload, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "nvtx/_lib/lib.pyx":371
 *                 schema = self.domain._get_numpy_array_schema(schema)
 *                 payload = np.ascontiguousarray(payload)
 *                 array_length = payload.size             # <<<<<<<<<<<<<<
 * 
 *             self._payload_object = payload
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_8 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_array_length = __pyx_t_8;

    /* "nvtx/_lib/lib.pyx":368
 *             )
 *             cdef size_t array_length = 0
 *             if payload.ndim:             # <<<<<<<<<<<<<<
 *                 schema = self.domain._get_numpy_array_schema(schema)
 *                 payload = np.ascontiguousarray(payload)
*/
  }

  /* "nvtx/_lib/lib.pyx":373
 *                 array_length = payload.size
 * 
 *             self._payload_object = payload             # <<<<<<<<<<<<<<
 *             self._set_binary_payload(
 *                 <void*><size_t>payload.ctypes.data,
*/
  __Pyx_INCREF(__pyx_v_payload);
  __Pyx_GIVEREF(__pyx_v_payload);
  __Pyx_GOTREF(__pyx_v_self->_payload_object);
  __Pyx_DECREF(__pyx_v_self->_payload_object);
  __pyx_v_self->_payload_object = __pyx_v_payload;

  /* "nvtx/_lib/lib.pyx":375
 *             self._payload_object = payload
 *             self._set_binary_payload(
 *                 <void*><size_t>payload.ctypes.data,             # <<<<<<<<<<<<<<
 *                 <uint64_t>schema,
 *                 <size_t>payload.nbytes,
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_ctypes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_data); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = __Pyx_PyLong_As_size_t(__pyx_t_6); if (unlikely((__pyx_t_8 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "nvtx/_lib/lib.pyx":376
 *             self._set_binary_payload(
 *                 <void*><size_t>payload.ctypes.data,
 *                 <uint64_t>schema,             # <<<<<<<<<<<<<<
 *                 <size_t>payload.nbytes,
 *                 array_length)
*/
  __pyx_t_9 = __Pyx_PyLong_As_uint64_t(__pyx_v_schema); if (unlikely((__pyx_t_9 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 376, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":377
 *                 <void*><size_t>payload.ctypes.data,
 *                 <uint64_t>schema,
 *                 <size_t>payload.nbytes,             # <<<<<<<<<<<<<<
 *                 array_length)
 * 
*/
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_payload, __pyx_mstate_global->__pyx_n_u_nbytes); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_10 = __Pyx_PyLong_As_size_t(__pyx_t_6); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 377, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "nvtx/_lib/lib.pyx":374
 * 
 *             self._payload_object = payload
 *             self._set_binary_payload(             # <<<<<<<<<<<<<<
 *                 <void*><size_t>payload.ctypes.data,
 *                 <uint64_t>schema,
*/
  __pyx_t_6 = ((struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self->__pyx_vtab)->_set_binary_payload(__pyx_v_self, ((void *)((size_t)__pyx_t_8)), ((uint64_t)__pyx_t_9), ((size_t)__pyx_t_10), __pyx_v_array_length); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "nvtx/_lib/lib.pyx":360
 * 
 *     if np is not None:
 *         @payload_setter(np.ndarray)             # <<<<<<<<<<<<<<
 *         def _set_payload_numpy(self, payload):
 *             if payload.nbytes == 0:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes._set_payload_numpy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_schema);
  __Pyx_XDECREF(__pyx_v_payload);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":381
 * 
 * 
 *         @payload_setter(range)             # <<<<<<<<<<<<<<
 *         @payload_setter(list)
 *         @payload_setter(tuple)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_11_set_payload_iterable(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_10_set_payload_iterable, "EventAttributes._set_payload_iterable(self, payload)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_11_set_payload_iterable = {"_set_payload_iterable", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_11_set_payload_iterable, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_10_set_payload_iterable};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_11_set_payload_iterable(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_set_payload_iterable (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 381, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 381, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_set_payload_iterable", 0) < (0)) __PYX_ERR(0, 381, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_set_payload_iterable", 1, 1, 1, i); __PYX_ERR(0, 381, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 381, __pyx_L3_error)
    }
    __pyx_v_payload = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_set_payload_iterable", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 381, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes._set_payload_iterable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_10_set_payload_iterable(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), __pyx_v_payload);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_10_set_payload_iterable(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, PyObject *__pyx_v_payload) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_payload_iterable", 0);

  /* "nvtx/_lib/lib.pyx":386
 *         @payload_setter(bytes)
 *         def _set_payload_iterable(self, payload):
 *             self._set_payload_numpy(np.array(payload))             # <<<<<<<<<<<<<<
 * 
 *     cdef _set_binary_payload(self,
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_array); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_7 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_7 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_payload};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_7 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_payload_numpy, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":381
 * 
 * 
 *         @payload_setter(range)             # <<<<<<<<<<<<<<
 *         @payload_setter(list)
 *         @payload_setter(tuple)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes._set_payload_iterable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":388
 *             self._set_payload_numpy(np.array(payload))
 * 
 *     cdef _set_binary_payload(self,             # <<<<<<<<<<<<<<
 *                             void* payload, uint64_t schema, size_t nbytes, size_t array_length):
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_EXT
*/

static PyObject *__pyx_f_4nvtx_4_lib_3lib_15EventAttributes__set_binary_payload(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, void *__pyx_v_payload, uint64_t __pyx_v_schema, size_t __pyx_v_nbytes, size_t __pyx_v_array_length) {
  size_t __pyx_v_payload_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_binary_payload", 0);

  /* "nvtx/_lib/lib.pyx":390
 *     cdef _set_binary_payload(self,
 *                             void* payload, uint64_t schema, size_t nbytes, size_t array_length):
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_EXT             # <<<<<<<<<<<<<<
 *         self.c_obj.payload.ullValue = <uint64_t>&self._payload_data
 *         self.c_obj.reserved0 = 1
*/
  __pyx_v_self->c_obj.payloadType = NVTX_PAYLOAD_TYPE_EXT;

  /* "nvtx/_lib/lib.pyx":391
 *                             void* payload, uint64_t schema, size_t nbytes, size_t array_length):
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_EXT
 *         self.c_obj.payload.ullValue = <uint64_t>&self._payload_data             # <<<<<<<<<<<<<<
 *         self.c_obj.reserved0 = 1
 *         self._payload_data.schemaId = schema
*/
  __pyx_v_self->c_obj.payload.ullValue = ((uint64_t)(&__pyx_v_self->_payload_data));

  /* "nvtx/_lib/lib.pyx":392
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_EXT
 *         self.c_obj.payload.ullValue = <uint64_t>&self._payload_data
 *         self.c_obj.reserved0 = 1             # <<<<<<<<<<<<<<
 *         self._payload_data.schemaId = schema
 *         if array_length == 0:
*/
  __pyx_v_self->c_obj.reserved0 = 1;

  /* "nvtx/_lib/lib.pyx":393
 *         self.c_obj.payload.ullValue = <uint64_t>&self._payload_data
 *         self.c_obj.reserved0 = 1
 *         self._payload_data.schemaId = schema             # <<<<<<<<<<<<<<
 *         if array_length == 0:
 *             self._payload_data.size = nbytes
*/
  __pyx_v_self->_payload_data.schemaId = __pyx_v_schema;

  /* "nvtx/_lib/lib.pyx":394
 *         self.c_obj.reserved0 = 1
 *         self._payload_data.schemaId = schema
 *         if array_length == 0:             # <<<<<<<<<<<<<<
 *             self._payload_data.size = nbytes
 *             self._payload_data.payload = payload
*/
  __pyx_t_1 = (__pyx_v_array_length == 0);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":395
 *         self._payload_data.schemaId = schema
 *         if array_length == 0:
 *             self._payload_data.size = nbytes             # <<<<<<<<<<<<<<
 *             self._payload_data.payload = payload
 *         else:
*/
    __pyx_v_self->_payload_data.size = __pyx_v_nbytes;

    /* "nvtx/_lib/lib.pyx":396
 *         if array_length == 0:
 *             self._payload_data.size = nbytes
 *             self._payload_data.payload = payload             # <<<<<<<<<<<<<<
 *         else:
 *             payload_size = sizeof(uint64_t) + nbytes
*/
    __pyx_v_self->_payload_data.payload = __pyx_v_payload;

    /* "nvtx/_lib/lib.pyx":394
 *         self.c_obj.reserved0 = 1
 *         self._payload_data.schemaId = schema
 *         if array_length == 0:             # <<<<<<<<<<<<<<
 *             self._payload_data.size = nbytes
 *             self._payload_data.payload = payload
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/lib.pyx":398
 *             self._payload_data.payload = payload
 *         else:
 *             payload_size = sizeof(uint64_t) + nbytes             # <<<<<<<<<<<<<<
 *             self._payload_data.size = payload_size
 *             self._payload_data.payload = self._allocated_payload = malloc(payload_size)
*/
  /*else*/ {
    __pyx_v_payload_size = ((sizeof(uint64_t)) + __pyx_v_nbytes);

    /* "nvtx/_lib/lib.pyx":399
 *         else:
 *             payload_size = sizeof(uint64_t) + nbytes
 *             self._payload_data.size = payload_size             # <<<<<<<<<<<<<<
 *             self._payload_data.payload = self._allocated_payload = malloc(payload_size)
 *             if self._allocated_payload is NULL:
*/
    __pyx_v_self->_payload_data.size = __pyx_v_payload_size;

    /* "nvtx/_lib/lib.pyx":400
 *             payload_size = sizeof(uint64_t) + nbytes
 *             self._payload_data.size = payload_size
 *             self._payload_data.payload = self._allocated_payload = malloc(payload_size)             # <<<<<<<<<<<<<<
 *             if self._allocated_payload is NULL:
 *                 raise MemoryError("Failed to allocate memory for payload")
*/
    __pyx_t_2 = malloc(__pyx_v_payload_size);
    __pyx_v_self->_payload_data.payload = __pyx_t_2;
    __pyx_v_self->_allocated_payload = __pyx_t_2;

    /* "nvtx/_lib/lib.pyx":401
 *             self._payload_data.size = payload_size
 *             self._payload_data.payload = self._allocated_payload = malloc(payload_size)
 *             if self._allocated_payload is NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("Failed to allocate memory for payload")
 *             memcpy(self._allocated_payload, &array_length, sizeof(uint64_t))
*/
    __pyx_t_1 = (__pyx_v_self->_allocated_payload == NULL);
    if (unlikely(__pyx_t_1)) {

      /* "nvtx/_lib/lib.pyx":402
 *             self._payload_data.payload = self._allocated_payload = malloc(payload_size)
 *             if self._allocated_payload is NULL:
 *                 raise MemoryError("Failed to allocate memory for payload")             # <<<<<<<<<<<<<<
 *             memcpy(self._allocated_payload, &array_length, sizeof(uint64_t))
 *             memcpy(<char*>self._allocated_payload + sizeof(uint64_t), payload, nbytes)
*/
      __pyx_t_4 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Failed_to_allocate_memory_for_pa};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_MemoryError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 402, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 402, __pyx_L1_error)

      /* "nvtx/_lib/lib.pyx":401
 *             self._payload_data.size = payload_size
 *             self._payload_data.payload = self._allocated_payload = malloc(payload_size)
 *             if self._allocated_payload is NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("Failed to allocate memory for payload")
 *             memcpy(self._allocated_payload, &array_length, sizeof(uint64_t))
*/
    }

    /* "nvtx/_lib/lib.pyx":403
 *             if self._allocated_payload is NULL:
 *                 raise MemoryError("Failed to allocate memory for payload")
 *             memcpy(self._allocated_payload, &array_length, sizeof(uint64_t))             # <<<<<<<<<<<<<<
 *             memcpy(<char*>self._allocated_payload + sizeof(uint64_t), payload, nbytes)
 * 
*/
    (void)(memcpy(__pyx_v_self->_allocated_payload, (&__pyx_v_array_length), (sizeof(uint64_t))));

    /* "nvtx/_lib/lib.pyx":404
 *                 raise MemoryError("Failed to allocate memory for payload")
 *             memcpy(self._allocated_payload, &array_length, sizeof(uint64_t))
 *             memcpy(<char*>self._allocated_payload + sizeof(uint64_t), payload, nbytes)             # <<<<<<<<<<<<<<
 * 
 *     cdef _clear_payload(self):
*/
    (void)(memcpy((((char *)__pyx_v_self->_allocated_payload) + (sizeof(uint64_t))), __pyx_v_payload, __pyx_v_nbytes));
  }
  __pyx_L3:;

  /* "nvtx/_lib/lib.pyx":388
 *             self._set_payload_numpy(np.array(payload))
 * 
 *     cdef _set_binary_payload(self,             # <<<<<<<<<<<<<<
 *                             void* payload, uint64_t schema, size_t nbytes, size_t array_length):
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_EXT
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes._set_binary_payload", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":406
 *             memcpy(<char*>self._allocated_payload + sizeof(uint64_t), payload, nbytes)
 * 
 *     cdef _clear_payload(self):             # <<<<<<<<<<<<<<
 *         self._payload = None
 *         self._payload_object = None
*/

static PyObject *__pyx_f_4nvtx_4_lib_3lib_15EventAttributes__clear_payload(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_clear_payload", 0);

  /* "nvtx/_lib/lib.pyx":407
 * 
 *     cdef _clear_payload(self):
 *         self._payload = None             # <<<<<<<<<<<<<<
 *         self._payload_object = None
 *         if self._allocated_payload is not NULL:
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_payload);
  __Pyx_DECREF(__pyx_v_self->_payload);
  __pyx_v_self->_payload = Py_None;

  /* "nvtx/_lib/lib.pyx":408
 *     cdef _clear_payload(self):
 *         self._payload = None
 *         self._payload_object = None             # <<<<<<<<<<<<<<
 *         if self._allocated_payload is not NULL:
 *             free(self._allocated_payload)
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_payload_object);
  __Pyx_DECREF(__pyx_v_self->_payload_object);
  __pyx_v_self->_payload_object = Py_None;

  /* "nvtx/_lib/lib.pyx":409
 *         self._payload = None
 *         self._payload_object = None
 *         if self._allocated_payload is not NULL:             # <<<<<<<<<<<<<<
 *             free(self._allocated_payload)
 *             self._allocated_payload = NULL
*/
  __pyx_t_1 = (__pyx_v_self->_allocated_payload != NULL);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":410
 *         self._payload_object = None
 *         if self._allocated_payload is not NULL:
 *             free(self._allocated_payload)             # <<<<<<<<<<<<<<
 *             self._allocated_payload = NULL
 * 
*/
    free(__pyx_v_self->_allocated_payload);

    /* "nvtx/_lib/lib.pyx":411
 *         if self._allocated_payload is not NULL:
 *             free(self._allocated_payload)
 *             self._allocated_payload = NULL             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_v_self->_allocated_payload = NULL;

    /* "nvtx/_lib/lib.pyx":409
 *         self._payload = None
 *         self._payload_object = None
 *         if self._allocated_payload is not NULL:             # <<<<<<<<<<<<<<
 *             free(self._allocated_payload)
 *             self._allocated_payload = NULL
*/
  }

  /* "nvtx/_lib/lib.pyx":406
 *             memcpy(<char*>self._allocated_payload + sizeof(uint64_t), payload, nbytes)
 * 
 *     cdef _clear_payload(self):             # <<<<<<<<<<<<<<
 *         self._payload = None
 *         self._payload_object = None
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_13__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_12__reduce_cython__, "EventAttributes.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_13__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_13__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_12__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_13__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_12__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__allocated_payload_self__pa, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_15__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_14__setstate_cython__, "EventAttributes.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_15__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_15__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_14__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_15__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_14__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15EventAttributes_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__allocated_payload_self__pa, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.EventAttributes.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":416
 * cdef class DomainHandle:
 * 
 *     def __init__(self, object name=None):             # <<<<<<<<<<<<<<
 *         if name is not None:
 *             self._name = _to_bytes(name)
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 416, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 416, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 416, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    } else {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 416, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, __pyx_nargs); __PYX_ERR(0, 416, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DomainHandle.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle___init__(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle___init__(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self, PyObject *__pyx_v_name) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  char const *__pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "nvtx/_lib/lib.pyx":417
 * 
 *     def __init__(self, object name=None):
 *         if name is not None:             # <<<<<<<<<<<<<<
 *             self._name = _to_bytes(name)
 *             self.c_obj = nvtxDomainCreateA(
*/
  __pyx_t_1 = (__pyx_v_name != Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":418
 *     def __init__(self, object name=None):
 *         if name is not None:
 *             self._name = _to_bytes(name)             # <<<<<<<<<<<<<<
 *             self.c_obj = nvtxDomainCreateA(
 *                 self._name
*/
    __pyx_t_2 = __pyx_f_4nvtx_4_lib_3lib__to_bytes(__pyx_v_name, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 418, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __Pyx_GOTREF(__pyx_v_self->_name);
    __Pyx_DECREF(__pyx_v_self->_name);
    __pyx_v_self->_name = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "nvtx/_lib/lib.pyx":420
 *             self._name = _to_bytes(name)
 *             self.c_obj = nvtxDomainCreateA(
 *                 self._name             # <<<<<<<<<<<<<<
 *             )
 *         else:
*/
    if (unlikely(__pyx_v_self->_name == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 420, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyBytes_AsString(__pyx_v_self->_name); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(0, 420, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":419
 *         if name is not None:
 *             self._name = _to_bytes(name)
 *             self.c_obj = nvtxDomainCreateA(             # <<<<<<<<<<<<<<
 *                 self._name
 *             )
*/
    __pyx_v_self->c_obj = nvtxDomainCreateA(__pyx_t_3);

    /* "nvtx/_lib/lib.pyx":417
 * 
 *     def __init__(self, object name=None):
 *         if name is not None:             # <<<<<<<<<<<<<<
 *             self._name = _to_bytes(name)
 *             self.c_obj = nvtxDomainCreateA(
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/lib.pyx":423
 *             )
 *         else:
 *             self._name = b""             # <<<<<<<<<<<<<<
 *             self.c_obj = NULL
 * 
*/
  /*else*/ {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_b__2);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_b__2);
    __Pyx_GOTREF(__pyx_v_self->_name);
    __Pyx_DECREF(__pyx_v_self->_name);
    __pyx_v_self->_name = __pyx_mstate_global->__pyx_kp_b__2;

    /* "nvtx/_lib/lib.pyx":424
 *         else:
 *             self._name = b""
 *             self.c_obj = NULL             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
    __pyx_v_self->c_obj = NULL;
  }
  __pyx_L3:;

  /* "nvtx/_lib/lib.pyx":416
 * cdef class DomainHandle:
 * 
 *     def __init__(self, object name=None):             # <<<<<<<<<<<<<<
 *         if name is not None:
 *             self._name = _to_bytes(name)
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.DomainHandle.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":426
 *             self.c_obj = NULL
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def name(self):
 *         return self._name.decode()
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_4name_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_4name___get__(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_4name___get__(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/lib.pyx":428
 *     @property
 *     def name(self):
 *         return self._name.decode()             # <<<<<<<<<<<<<<
 * 
 *     def enabled(self):
*/
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->_name == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "decode");
    __PYX_ERR(0, 428, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_decode_bytes(__pyx_v_self->_name, 0, PY_SSIZE_T_MAX, NULL, NULL, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":426
 *             self.c_obj = NULL
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def name(self):
 *         return self._name.decode()
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.DomainHandle.name.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":430
 *         return self._name.decode()
 * 
 *     def enabled(self):             # <<<<<<<<<<<<<<
 *         return bool(nvtxDomainIsEnabled(self.c_obj))
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_3enabled(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_12DomainHandle_2enabled, "DomainHandle.enabled(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_12DomainHandle_3enabled = {"enabled", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_3enabled, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12DomainHandle_2enabled};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_3enabled(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enabled (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("enabled", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("enabled", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_2enabled(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_2enabled(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("enabled", 0);

  /* "nvtx/_lib/lib.pyx":431
 * 
 *     def enabled(self):
 *         return bool(nvtxDomainIsEnabled(self.c_obj))             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((!(!(nvtxDomainIsEnabled(__pyx_v_self->c_obj) != 0)))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":430
 *         return self._name.decode()
 * 
 *     def enabled(self):             # <<<<<<<<<<<<<<
 *         return bool(nvtxDomainIsEnabled(self.c_obj))
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.DomainHandle.enabled", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":433
 *         return bool(nvtxDomainIsEnabled(self.c_obj))
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         nvtxDomainDestroy(self.c_obj)
 * 
*/

/* Python wrapper */
static void __pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_5__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_4__dealloc__(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_4__dealloc__(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self) {

  /* "nvtx/_lib/lib.pyx":434
 * 
 *     def __dealloc__(self):
 *         nvtxDomainDestroy(self.c_obj)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  nvtxDomainDestroy(__pyx_v_self->c_obj);

  /* "nvtx/_lib/lib.pyx":433
 *         return bool(nvtxDomainIsEnabled(self.c_obj))
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         nvtxDomainDestroy(self.c_obj)
 * 
*/

  /* function exit code */
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_7__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_12DomainHandle_6__reduce_cython__, "DomainHandle.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_12DomainHandle_7__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_7__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12DomainHandle_6__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_7__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_6__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_c_obj_cannot_be_converted_t, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.DomainHandle.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_9__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_12DomainHandle_8__setstate_cython__, "DomainHandle.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_12DomainHandle_9__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_9__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12DomainHandle_8__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_9__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DomainHandle.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_8__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12DomainHandle_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_c_obj_cannot_be_converted_t, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.DomainHandle.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":442
 *     Use :func:`nvtx.Domain.get_registered_string` to create an instance.
 *     """
 *     def __init__(self, domain, string=None):             # <<<<<<<<<<<<<<
 *         self.string = string
 *         self.domain = domain
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_16RegisteredString_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_16RegisteredString___init__, "RegisteredString.__init__(self, domain, string=None)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_16RegisteredString_1__init__ = {"__init__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_16RegisteredString_1__init__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_16RegisteredString___init__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_16RegisteredString_1__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_domain = 0;
  PyObject *__pyx_v_string = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_domain,&__pyx_mstate_global->__pyx_n_u_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 442, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 442, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 442, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 442, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 442, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, i); __PYX_ERR(0, 442, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 442, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 442, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 442, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_domain = values[1];
    __pyx_v_string = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 442, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.RegisteredString.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_16RegisteredString___init__(__pyx_self, __pyx_v_self, __pyx_v_domain, __pyx_v_string);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_16RegisteredString___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_domain, PyObject *__pyx_v_string) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "nvtx/_lib/lib.pyx":443
 *     """
 *     def __init__(self, domain, string=None):
 *         self.string = string             # <<<<<<<<<<<<<<
 *         self.domain = domain
 *         self.handle = StringHandle(domain, string)
*/
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_string, __pyx_v_string) < (0)) __PYX_ERR(0, 443, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":444
 *     def __init__(self, domain, string=None):
 *         self.string = string
 *         self.domain = domain             # <<<<<<<<<<<<<<
 *         self.handle = StringHandle(domain, string)
 * 
*/
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_domain, __pyx_v_domain) < (0)) __PYX_ERR(0, 444, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":445
 *         self.string = string
 *         self.domain = domain
 *         self.handle = StringHandle(domain, string)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_v_domain, __pyx_v_string};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle, ((PyObject *)__pyx_t_1)) < (0)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_DECREF((PyObject *)__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":442
 *     Use :func:`nvtx.Domain.get_registered_string` to create an instance.
 *     """
 *     def __init__(self, domain, string=None):             # <<<<<<<<<<<<<<
 *         self.string = string
 *         self.domain = domain
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.RegisteredString.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":453
 *     (e.g., when no tool is attached).
 *     """
 *     def get_registered_string(self, string):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_1get_registered_string(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_get_registered_string, "DummyDomain.get_registered_string(self, string)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_1get_registered_string = {"get_registered_string", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_1get_registered_string, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_get_registered_string};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_1get_registered_string(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED PyObject *__pyx_v_string = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_registered_string (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 453, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 453, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 453, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_registered_string", 0) < (0)) __PYX_ERR(0, 453, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_registered_string", 1, 2, 2, i); __PYX_ERR(0, 453, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 453, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 453, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_string = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_registered_string", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 453, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.get_registered_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_get_registered_string(__pyx_self, __pyx_v_self, __pyx_v_string);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_get_registered_string(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_string) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_registered_string", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":456
 *         pass
 * 
 *     def get_category_id(self, name):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_3get_category_id(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_2get_category_id, "DummyDomain.get_category_id(self, name)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_3get_category_id = {"get_category_id", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_3get_category_id, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_2get_category_id};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_3get_category_id(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_category_id (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 456, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 456, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 456, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_category_id", 0) < (0)) __PYX_ERR(0, 456, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_category_id", 1, 2, 2, i); __PYX_ERR(0, 456, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 456, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 456, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_name = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_category_id", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 456, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.get_category_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_2get_category_id(__pyx_self, __pyx_v_self, __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_2get_category_id(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_category_id", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":459
 *         pass
 * 
 *     def get_event_attributes(self, message=None, color=None, category=None, payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_5get_event_attributes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_4get_event_attributes, "DummyDomain.get_event_attributes(self, message=None, color=None, category=None, payload=None)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_5get_event_attributes = {"get_event_attributes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_5get_event_attributes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_4get_event_attributes};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_5get_event_attributes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED PyObject *__pyx_v_message = 0;
  CYTHON_UNUSED PyObject *__pyx_v_color = 0;
  CYTHON_UNUSED PyObject *__pyx_v_category = 0;
  CYTHON_UNUSED PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_event_attributes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 459, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 459, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 459, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 459, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 459, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 459, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_event_attributes", 0) < (0)) __PYX_ERR(0, 459, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_event_attributes", 0, 1, 5, i); __PYX_ERR(0, 459, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 459, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 459, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 459, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 459, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 459, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_message = values[1];
    __pyx_v_color = values[2];
    __pyx_v_category = values[3];
    __pyx_v_payload = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_event_attributes", 0, 1, 5, __pyx_nargs); __PYX_ERR(0, 459, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.get_event_attributes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_4get_event_attributes(__pyx_self, __pyx_v_self, __pyx_v_message, __pyx_v_color, __pyx_v_category, __pyx_v_payload);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_4get_event_attributes(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_event_attributes", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":462
 *         pass
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,             # <<<<<<<<<<<<<<
 *                              message=None, color=None, category=None, payload=None):
 *         pass
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_7set_event_attributes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_6set_event_attributes, "DummyDomain.set_event_attributes(self, EventAttributes attributes, *, message=None, color=None, category=None, payload=None)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_7set_event_attributes = {"set_event_attributes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_7set_event_attributes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_6set_event_attributes};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_7set_event_attributes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  CYTHON_UNUSED PyObject *__pyx_v_message = 0;
  CYTHON_UNUSED PyObject *__pyx_v_color = 0;
  CYTHON_UNUSED PyObject *__pyx_v_category = 0;
  CYTHON_UNUSED PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[6] = {0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_event_attributes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_attributes,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 462, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 462, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 462, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "set_event_attributes", 0) < (0)) __PYX_ERR(0, 462, __pyx_L3_error)

      /* "nvtx/_lib/lib.pyx":463
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,
 *                              message=None, color=None, category=None, payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("set_event_attributes", 1, 2, 2, i); __PYX_ERR(0, 462, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 462, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 462, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[1]);
    __pyx_v_message = values[2];
    __pyx_v_color = values[3];
    __pyx_v_category = values[4];
    __pyx_v_payload = values[5];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_event_attributes", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 462, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.set_event_attributes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 462, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_6set_event_attributes(__pyx_self, __pyx_v_self, __pyx_v_attributes, __pyx_v_message, __pyx_v_color, __pyx_v_category, __pyx_v_payload);

  /* "nvtx/_lib/lib.pyx":462
 *         pass
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,             # <<<<<<<<<<<<<<
 *                              message=None, color=None, category=None, payload=None):
 *         pass
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_6set_event_attributes(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_event_attributes", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":466
 *         pass
 * 
 *     def mark(self, EventAttributes attributes=None, *, message=None, color=None, category=None,             # <<<<<<<<<<<<<<
 *              payload=None):
 *         pass
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_9mark(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_8mark, "DummyDomain.mark(self, EventAttributes attributes=None, *, message=None, color=None, category=None, payload=None)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_9mark = {"mark", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_9mark, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_8mark};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_9mark(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  CYTHON_UNUSED PyObject *__pyx_v_message = 0;
  CYTHON_UNUSED PyObject *__pyx_v_color = 0;
  CYTHON_UNUSED PyObject *__pyx_v_category = 0;
  CYTHON_UNUSED PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[6] = {0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("mark (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_attributes,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 466, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 466, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 466, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "mark", 0) < (0)) __PYX_ERR(0, 466, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":467
 * 
 *     def mark(self, EventAttributes attributes=None, *, message=None, color=None, category=None,
 *              payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("mark", 0, 1, 2, i); __PYX_ERR(0, 466, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 466, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 466, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "nvtx/_lib/lib.pyx":466
 *         pass
 * 
 *     def mark(self, EventAttributes attributes=None, *, message=None, color=None, category=None,             # <<<<<<<<<<<<<<
 *              payload=None):
 *         pass
*/
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":467
 * 
 *     def mark(self, EventAttributes attributes=None, *, message=None, color=None, category=None,
 *              payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[1]);
    __pyx_v_message = values[2];
    __pyx_v_color = values[3];
    __pyx_v_category = values[4];
    __pyx_v_payload = values[5];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("mark", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 466, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.mark", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 466, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_8mark(__pyx_self, __pyx_v_self, __pyx_v_attributes, __pyx_v_message, __pyx_v_color, __pyx_v_category, __pyx_v_payload);

  /* "nvtx/_lib/lib.pyx":466
 *         pass
 * 
 *     def mark(self, EventAttributes attributes=None, *, message=None, color=None, category=None,             # <<<<<<<<<<<<<<
 *              payload=None):
 *         pass
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_8mark(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("mark", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":470
 *         pass
 * 
 *     def push_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                    category=None, payload=None):
 *         pass
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_11push_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_10push_range, "DummyDomain.push_range(self, EventAttributes attributes=None, *, message=None, color=None, category=None, payload=None)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_11push_range = {"push_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_11push_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_10push_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_11push_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  CYTHON_UNUSED PyObject *__pyx_v_message = 0;
  CYTHON_UNUSED PyObject *__pyx_v_color = 0;
  CYTHON_UNUSED PyObject *__pyx_v_category = 0;
  CYTHON_UNUSED PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[6] = {0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_attributes,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 470, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 470, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 470, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "push_range", 0) < (0)) __PYX_ERR(0, 470, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":471
 * 
 *     def push_range(self, EventAttributes attributes=None, *, message=None, color=None,
 *                    category=None, payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("push_range", 0, 1, 2, i); __PYX_ERR(0, 470, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 470, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 470, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "nvtx/_lib/lib.pyx":470
 *         pass
 * 
 *     def push_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                    category=None, payload=None):
 *         pass
*/
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":471
 * 
 *     def push_range(self, EventAttributes attributes=None, *, message=None, color=None,
 *                    category=None, payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[1]);
    __pyx_v_message = values[2];
    __pyx_v_color = values[3];
    __pyx_v_category = values[4];
    __pyx_v_payload = values[5];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("push_range", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 470, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.push_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 470, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_10push_range(__pyx_self, __pyx_v_self, __pyx_v_attributes, __pyx_v_message, __pyx_v_color, __pyx_v_category, __pyx_v_payload);

  /* "nvtx/_lib/lib.pyx":470
 *         pass
 * 
 *     def push_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                    category=None, payload=None):
 *         pass
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_10push_range(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push_range", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":474
 *         pass
 * 
 *     def pop_range(self):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_13pop_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_12pop_range, "DummyDomain.pop_range(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_13pop_range = {"pop_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_13pop_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_12pop_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_13pop_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("pop_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 474, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 474, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "pop_range", 0) < (0)) __PYX_ERR(0, 474, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("pop_range", 1, 1, 1, i); __PYX_ERR(0, 474, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 474, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("pop_range", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 474, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.pop_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_12pop_range(__pyx_self, __pyx_v_self);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_12pop_range(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("pop_range", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":477
 *         pass
 * 
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                     category=None, payload=None):
 *         return 0
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_15start_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_14start_range, "DummyDomain.start_range(self, EventAttributes attributes=None, *, message=None, color=None, category=None, payload=None)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_15start_range = {"start_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_15start_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_14start_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_15start_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  CYTHON_UNUSED PyObject *__pyx_v_message = 0;
  CYTHON_UNUSED PyObject *__pyx_v_color = 0;
  CYTHON_UNUSED PyObject *__pyx_v_category = 0;
  CYTHON_UNUSED PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[6] = {0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_attributes,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 477, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 477, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 477, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "start_range", 0) < (0)) __PYX_ERR(0, 477, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":478
 * 
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,
 *                     category=None, payload=None):             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("start_range", 0, 1, 2, i); __PYX_ERR(0, 477, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 477, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 477, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "nvtx/_lib/lib.pyx":477
 *         pass
 * 
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                     category=None, payload=None):
 *         return 0
*/
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":478
 * 
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,
 *                     category=None, payload=None):             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[1]);
    __pyx_v_message = values[2];
    __pyx_v_color = values[3];
    __pyx_v_category = values[4];
    __pyx_v_payload = values[5];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("start_range", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 477, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.start_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 477, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_14start_range(__pyx_self, __pyx_v_self, __pyx_v_attributes, __pyx_v_message, __pyx_v_color, __pyx_v_category, __pyx_v_payload);

  /* "nvtx/_lib/lib.pyx":477
 *         pass
 * 
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                     category=None, payload=None):
 *         return 0
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_14start_range(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, CYTHON_UNUSED PyObject *__pyx_v_message, CYTHON_UNUSED PyObject *__pyx_v_color, CYTHON_UNUSED PyObject *__pyx_v_category, CYTHON_UNUSED PyObject *__pyx_v_payload) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_range", 0);

  /* "nvtx/_lib/lib.pyx":479
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,
 *                     category=None, payload=None):
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def end_range(self, nvtxRangeId_t range_id):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
  __pyx_r = __pyx_mstate_global->__pyx_int_0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":477
 *         pass
 * 
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                     category=None, payload=None):
 *         return 0
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":481
 *         return 0
 * 
 *     def end_range(self, nvtxRangeId_t range_id):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_17end_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_16end_range, "DummyDomain.end_range(self, nvtxRangeId_t range_id)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_17end_range = {"end_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_17end_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_16end_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_17end_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED nvtxRangeId_t __pyx_v_range_id;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("end_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_range_id,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 481, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 481, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 481, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "end_range", 0) < (0)) __PYX_ERR(0, 481, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("end_range", 1, 2, 2, i); __PYX_ERR(0, 481, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 481, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 481, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_range_id = __Pyx_PyLong_As_uint64_t(values[1]); if (unlikely((__pyx_v_range_id == ((nvtxRangeId_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 481, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("end_range", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 481, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.end_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_16end_range(__pyx_self, __pyx_v_self, __pyx_v_range_id);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_16end_range(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED nvtxRangeId_t __pyx_v_range_id) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("end_range", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":484
 *         pass
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_26__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "nvtx/_lib/lib.pyx":489
 *         dtype,
 *         *,
 *         description=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         semantics=None,
*/
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_description, Py_None) < (0)) __PYX_ERR(0, 484, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":490
 *         *,
 *         description=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         semantics=None,
 *         time_domain=TimestampType.TOOL_PROVIDED,
*/
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_scope, Py_None) < (0)) __PYX_ERR(0, 484, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":491
 *         description=None,
 *         scope=None,
 *         semantics=None,             # <<<<<<<<<<<<<<
 *         time_domain=TimestampType.TOOL_PROVIDED,
 *     ) -> DummyCounter:
*/
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_semantics, Py_None) < (0)) __PYX_ERR(0, 484, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":484
 *         pass
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_time_domain, __Pyx_CyFunction_Defaults(struct __pyx_defaults, __pyx_self)->arg0) < (0)) __PYX_ERR(0, 484, __pyx_L1_error)
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, Py_None) != (0)) __PYX_ERR(0, 484, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1) != (0)) __PYX_ERR(0, 484, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_19get_counter(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_18get_counter, "DummyDomain.get_counter(self, name, dtype, *, description=None, scope=None, semantics=None, time_domain=TimestampType.TOOL_PROVIDED) -> DummyCounter");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_19get_counter = {"get_counter", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_19get_counter, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_18get_counter};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_19get_counter(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_dtype = 0;
  CYTHON_UNUSED PyObject *__pyx_v_description = 0;
  CYTHON_UNUSED PyObject *__pyx_v_scope = 0;
  CYTHON_UNUSED PyObject *__pyx_v_semantics = 0;
  CYTHON_UNUSED PyObject *__pyx_v_time_domain = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[7] = {0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_counter (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_dtype,&__pyx_mstate_global->__pyx_n_u_description,&__pyx_mstate_global->__pyx_n_u_scope,&__pyx_mstate_global->__pyx_n_u_semantics,&__pyx_mstate_global->__pyx_n_u_time_domain,0};
    struct __pyx_defaults *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(struct __pyx_defaults, __pyx_self);
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 484, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 484, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 484, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 484, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_counter", 0) < (0)) __PYX_ERR(0, 484, __pyx_L3_error)

      /* "nvtx/_lib/lib.pyx":489
 *         dtype,
 *         *,
 *         description=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         semantics=None,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":490
 *         *,
 *         description=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         semantics=None,
 *         time_domain=TimestampType.TOOL_PROVIDED,
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":491
 *         description=None,
 *         scope=None,
 *         semantics=None,             # <<<<<<<<<<<<<<
 *         time_domain=TimestampType.TOOL_PROVIDED,
 *     ) -> DummyCounter:
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[6]) values[6] = __Pyx_NewRef(__pyx_dynamic_args->arg0);
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_counter", 1, 3, 3, i); __PYX_ERR(0, 484, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 484, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 484, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 484, __pyx_L3_error)

      /* "nvtx/_lib/lib.pyx":489
 *         dtype,
 *         *,
 *         description=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         semantics=None,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":490
 *         *,
 *         description=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         semantics=None,
 *         time_domain=TimestampType.TOOL_PROVIDED,
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":491
 *         description=None,
 *         scope=None,
 *         semantics=None,             # <<<<<<<<<<<<<<
 *         time_domain=TimestampType.TOOL_PROVIDED,
 *     ) -> DummyCounter:
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[6]) values[6] = __Pyx_NewRef(__pyx_dynamic_args->arg0);
    }
    __pyx_v_self = values[0];
    __pyx_v_name = values[1];
    __pyx_v_dtype = values[2];
    __pyx_v_description = values[3];
    __pyx_v_scope = values[4];
    __pyx_v_semantics = values[5];
    __pyx_v_time_domain = values[6];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_counter", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 484, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.get_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_18get_counter(__pyx_self, __pyx_v_self, __pyx_v_name, __pyx_v_dtype, __pyx_v_description, __pyx_v_scope, __pyx_v_semantics, __pyx_v_time_domain);

  /* "nvtx/_lib/lib.pyx":484
 *         pass
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_18get_counter(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, CYTHON_UNUSED PyObject *__pyx_v_description, CYTHON_UNUSED PyObject *__pyx_v_scope, CYTHON_UNUSED PyObject *__pyx_v_semantics, CYTHON_UNUSED PyObject *__pyx_v_time_domain) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_counter", 0);

  /* "nvtx/_lib/lib.pyx":494
 *         time_domain=TimestampType.TOOL_PROVIDED,
 *     ) -> DummyCounter:
 *         _normalize_counter_dtype(dtype) # Validate that the user-provided dtype is supported.             # <<<<<<<<<<<<<<
 *         return dummy_counter
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_normalize_counter_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_dtype};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":495
 *     ) -> DummyCounter:
 *         _normalize_counter_dtype(dtype) # Validate that the user-provided dtype is supported.
 *         return dummy_counter             # <<<<<<<<<<<<<<
 * 
 *     def get_timestamp(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_dummy_counter); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":484
 *         pass
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.get_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":497
 *         return dummy_counter
 * 
 *     def get_timestamp(self):             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_21get_timestamp(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_20get_timestamp, "DummyDomain.get_timestamp(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_21get_timestamp = {"get_timestamp", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_21get_timestamp, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_11DummyDomain_20get_timestamp};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_11DummyDomain_21get_timestamp(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_timestamp (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 497, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 497, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_timestamp", 0) < (0)) __PYX_ERR(0, 497, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_timestamp", 1, 1, 1, i); __PYX_ERR(0, 497, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 497, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_timestamp", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 497, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.DummyDomain.get_timestamp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_20get_timestamp(__pyx_self, __pyx_v_self);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_11DummyDomain_20get_timestamp(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_timestamp", 0);

  /* "nvtx/_lib/lib.pyx":498
 * 
 *     def get_timestamp(self):
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
  __pyx_r = __pyx_mstate_global->__pyx_int_0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":497
 *         return dummy_counter
 * 
 *     def get_timestamp(self):             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":516
 *     """
 * 
 *     def __cinit__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         # Per-domain schema dedup caches.
 *         self._dtype_cache = {}
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return -1;
  if (__pyx_kwds_len > 0) {
    if (unlikely(__Pyx_CheckKeywordStrings("__cinit__", __pyx_kwds) == -1)) return -1;
  }
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar___cinit__(((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self), __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_DECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar___cinit__(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "nvtx/_lib/lib.pyx":518
 *     def __cinit__(self, *args, **kwargs):
 *         # Per-domain schema dedup caches.
 *         self._dtype_cache = {}             # <<<<<<<<<<<<<<
 *         self._array_cache = {}
 *         self._fixed_cache = {}
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_dtype_cache);
  __Pyx_DECREF(__pyx_v_self->_dtype_cache);
  __pyx_v_self->_dtype_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":519
 *         # Per-domain schema dedup caches.
 *         self._dtype_cache = {}
 *         self._array_cache = {}             # <<<<<<<<<<<<<<
 *         self._fixed_cache = {}
 * 
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_array_cache);
  __Pyx_DECREF(__pyx_v_self->_array_cache);
  __pyx_v_self->_array_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":520
 *         self._dtype_cache = {}
 *         self._array_cache = {}
 *         self._fixed_cache = {}             # <<<<<<<<<<<<<<
 * 
 *     cdef uint64_t _do_register(
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_fixed_cache);
  __Pyx_DECREF(__pyx_v_self->_fixed_cache);
  __pyx_v_self->_fixed_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":516
 *     """
 * 
 *     def __cinit__(self, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         # Per-domain schema dedup caches.
 *         self._dtype_cache = {}
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":522
 *         self._fixed_cache = {}
 * 
 *     cdef uint64_t _do_register(             # <<<<<<<<<<<<<<
 *         self, const nvtxPayloadSchemaAttr_t* attr
 *     ) except *:
*/

static uint64_t __pyx_f_4nvtx_4_lib_3lib_15SchemaRegistrar__do_register(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, CYTHON_UNUSED nvtxPayloadSchemaAttr_t const *__pyx_v_attr) {
  uint64_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_do_register", 0);

  /* "nvtx/_lib/lib.pyx":525
 *         self, const nvtxPayloadSchemaAttr_t* attr
 *     ) except *:
 *         raise NotImplementedError(             # <<<<<<<<<<<<<<
 *             "SchemaRegistrar subclasses must implement _do_register")
 * 
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_SchemaRegistrar_subclasses_must};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_NotImplementedError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 525, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 525, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":522
 *         self._fixed_cache = {}
 * 
 *     cdef uint64_t _do_register(             # <<<<<<<<<<<<<<
 *         self, const nvtxPayloadSchemaAttr_t* attr
 *     ) except *:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._do_register", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":528
 *             "SchemaRegistrar subclasses must implement _do_register")
 * 
 *     def _register_builtin_schema(self, dt, metadata):             # <<<<<<<<<<<<<<
 *         name = dt.name.encode()
 *         cdef nvtxSemanticsCounter_t counter_semantics
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_3_register_builtin_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_2_register_builtin_schema, "SchemaRegistrar._register_builtin_schema(self, dt, metadata)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_3_register_builtin_schema = {"_register_builtin_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_3_register_builtin_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_2_register_builtin_schema};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_3_register_builtin_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_dt = 0;
  PyObject *__pyx_v_metadata = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_register_builtin_schema (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_dt,&__pyx_mstate_global->__pyx_n_u_metadata,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 528, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 528, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 528, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_register_builtin_schema", 0) < (0)) __PYX_ERR(0, 528, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_register_builtin_schema", 1, 2, 2, i); __PYX_ERR(0, 528, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 528, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 528, __pyx_L3_error)
    }
    __pyx_v_dt = values[0];
    __pyx_v_metadata = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_register_builtin_schema", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 528, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._register_builtin_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_2_register_builtin_schema(((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self), __pyx_v_dt, __pyx_v_metadata);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_2_register_builtin_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, PyObject *__pyx_v_dt, PyObject *__pyx_v_metadata) {
  PyObject *__pyx_v_name = NULL;
  nvtxSemanticsCounter_t __pyx_v_counter_semantics;
  nvtxSemanticsHeader_t const *__pyx_v_semantics;
  PyObject *__pyx_v_counter_semantics_obj = 0;
  PyObject *__pyx_v_array_length = NULL;
  PyObject *__pyx_v_flags = NULL;
  PyObject *__pyx_v_entry_type = NULL;
  nvtxPayloadSchemaEntry_t __pyx_v_schemaEntry;
  nvtxPayloadSchemaAttr_t __pyx_v_schemaAttr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  nvtxPayloadSchemaEntry_t __pyx_t_6;
  uint64_t __pyx_t_7;
  char const *__pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_register_builtin_schema", 0);

  /* "nvtx/_lib/lib.pyx":529
 * 
 *     def _register_builtin_schema(self, dt, metadata):
 *         name = dt.name.encode()             # <<<<<<<<<<<<<<
 *         cdef nvtxSemanticsCounter_t counter_semantics
 *         cdef const nvtxSemanticsHeader_t* semantics = NULL
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_name = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":531
 *         name = dt.name.encode()
 *         cdef nvtxSemanticsCounter_t counter_semantics
 *         cdef const nvtxSemanticsHeader_t* semantics = NULL             # <<<<<<<<<<<<<<
 *         cdef object counter_semantics_obj = _counter_semantics_from_metadata(metadata)
 *         if counter_semantics_obj is not None:
*/
  __pyx_v_semantics = NULL;

  /* "nvtx/_lib/lib.pyx":532
 *         cdef nvtxSemanticsCounter_t counter_semantics
 *         cdef const nvtxSemanticsHeader_t* semantics = NULL
 *         cdef object counter_semantics_obj = _counter_semantics_from_metadata(metadata)             # <<<<<<<<<<<<<<
 *         if counter_semantics_obj is not None:
 *             _fill_counter_semantics(&counter_semantics, counter_semantics_obj)
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_counter_semantics_from_metadata); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_metadata};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_counter_semantics_obj = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":533
 *         cdef const nvtxSemanticsHeader_t* semantics = NULL
 *         cdef object counter_semantics_obj = _counter_semantics_from_metadata(metadata)
 *         if counter_semantics_obj is not None:             # <<<<<<<<<<<<<<
 *             _fill_counter_semantics(&counter_semantics, counter_semantics_obj)
 *             semantics = &counter_semantics.header
*/
  __pyx_t_5 = (__pyx_v_counter_semantics_obj != Py_None);
  if (__pyx_t_5) {

    /* "nvtx/_lib/lib.pyx":534
 *         cdef object counter_semantics_obj = _counter_semantics_from_metadata(metadata)
 *         if counter_semantics_obj is not None:
 *             _fill_counter_semantics(&counter_semantics, counter_semantics_obj)             # <<<<<<<<<<<<<<
 *             semantics = &counter_semantics.header
 * 
*/
    __pyx_f_4nvtx_4_lib_8counters__fill_counter_semantics((&__pyx_v_counter_semantics), __pyx_v_counter_semantics_obj); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 534, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":535
 *         if counter_semantics_obj is not None:
 *             _fill_counter_semantics(&counter_semantics, counter_semantics_obj)
 *             semantics = &counter_semantics.header             # <<<<<<<<<<<<<<
 * 
 *         array_length = 0
*/
    __pyx_v_semantics = (&__pyx_v_counter_semantics.header);

    /* "nvtx/_lib/lib.pyx":533
 *         cdef const nvtxSemanticsHeader_t* semantics = NULL
 *         cdef object counter_semantics_obj = _counter_semantics_from_metadata(metadata)
 *         if counter_semantics_obj is not None:             # <<<<<<<<<<<<<<
 *             _fill_counter_semantics(&counter_semantics, counter_semantics_obj)
 *             semantics = &counter_semantics.header
*/
  }

  /* "nvtx/_lib/lib.pyx":537
 *             semantics = &counter_semantics.header
 * 
 *         array_length = 0             # <<<<<<<<<<<<<<
 *         flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED
 *         entry_type = _entry_type_from_metadata(metadata)
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
  __pyx_v_array_length = __pyx_mstate_global->__pyx_int_0;

  /* "nvtx/_lib/lib.pyx":538
 * 
 *         array_length = 0
 *         flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED             # <<<<<<<<<<<<<<
 *         entry_type = _entry_type_from_metadata(metadata)
 *         if entry_type is None:
*/
  __pyx_t_1 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_FLAG_UNUSED); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_flags = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":539
 *         array_length = 0
 *         flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED
 *         entry_type = _entry_type_from_metadata(metadata)             # <<<<<<<<<<<<<<
 *         if entry_type is None:
 *             entry_type = _dtype_to_entry_type[dt.type]
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_entry_type_from_metadata); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_metadata};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_entry_type = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":540
 *         flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED
 *         entry_type = _entry_type_from_metadata(metadata)
 *         if entry_type is None:             # <<<<<<<<<<<<<<
 *             entry_type = _dtype_to_entry_type[dt.type]
 *         if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
*/
  __pyx_t_5 = (__pyx_v_entry_type == Py_None);
  if (__pyx_t_5) {

    /* "nvtx/_lib/lib.pyx":541
 *         entry_type = _entry_type_from_metadata(metadata)
 *         if entry_type is None:
 *             entry_type = _dtype_to_entry_type[dt.type]             # <<<<<<<<<<<<<<
 *         if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
 *             array_length = dt.itemsize // 4
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_dtype_to_entry_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 541, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_entry_type, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "nvtx/_lib/lib.pyx":540
 *         flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED
 *         entry_type = _entry_type_from_metadata(metadata)
 *         if entry_type is None:             # <<<<<<<<<<<<<<
 *             entry_type = _dtype_to_entry_type[dt.type]
 *         if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
*/
  }

  /* "nvtx/_lib/lib.pyx":542
 *         if entry_type is None:
 *             entry_type = _dtype_to_entry_type[dt.type]
 *         if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:             # <<<<<<<<<<<<<<
 *             array_length = dt.itemsize // 4
 *         elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_entry_type, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "nvtx/_lib/lib.pyx":543
 *             entry_type = _dtype_to_entry_type[dt.type]
 *         if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
 *             array_length = dt.itemsize // 4             # <<<<<<<<<<<<<<
 *         elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
 *             array_length = dt.itemsize
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 543, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyLong_FloorDivideObjC(__pyx_t_3, __pyx_mstate_global->__pyx_int_4, 4, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_array_length, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "nvtx/_lib/lib.pyx":542
 *         if entry_type is None:
 *             entry_type = _dtype_to_entry_type[dt.type]
 *         if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:             # <<<<<<<<<<<<<<
 *             array_length = dt.itemsize // 4
 *         elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
*/
    goto __pyx_L5;
  }

  /* "nvtx/_lib/lib.pyx":544
 *         if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
 *             array_length = dt.itemsize // 4
 *         elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:             # <<<<<<<<<<<<<<
 *             array_length = dt.itemsize
 *             flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_BYTE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 544, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_entry_type, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 544, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 544, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "nvtx/_lib/lib.pyx":545
 *             array_length = dt.itemsize // 4
 *         elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
 *             array_length = dt.itemsize             # <<<<<<<<<<<<<<
 *             flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
 *         flags |= _entry_flags_from_metadata(metadata)  # entry-role flags
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 545, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_array_length, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "nvtx/_lib/lib.pyx":546
 *         elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
 *             array_length = dt.itemsize
 *             flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE             # <<<<<<<<<<<<<<
 *         flags |= _entry_flags_from_metadata(metadata)  # entry-role flags
 * 
*/
    __pyx_t_3 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 546, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_flags, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "nvtx/_lib/lib.pyx":544
 *         if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
 *             array_length = dt.itemsize // 4
 *         elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:             # <<<<<<<<<<<<<<
 *             array_length = dt.itemsize
 *             flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
*/
  }
  __pyx_L5:;

  /* "nvtx/_lib/lib.pyx":547
 *             array_length = dt.itemsize
 *             flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
 *         flags |= _entry_flags_from_metadata(metadata)  # entry-role flags             # <<<<<<<<<<<<<<
 * 
 *         cdef nvtxPayloadSchemaEntry_t schemaEntry = nvtxPayloadSchemaEntry_t(
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_entry_flags_from_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_metadata};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 547, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_1 = PyNumber_InPlaceOr(__pyx_v_flags, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 547, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF_SET(__pyx_v_flags, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":550
 * 
 *         cdef nvtxPayloadSchemaEntry_t schemaEntry = nvtxPayloadSchemaEntry_t(
 *             flags=flags,             # <<<<<<<<<<<<<<
 *             type=entry_type,
 *             name=name,
*/
  __pyx_t_7 = __Pyx_PyLong_As_uint64_t(__pyx_v_flags); if (unlikely((__pyx_t_7 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 550, __pyx_L1_error)
  __pyx_t_6.flags = __pyx_t_7;

  /* "nvtx/_lib/lib.pyx":551
 *         cdef nvtxPayloadSchemaEntry_t schemaEntry = nvtxPayloadSchemaEntry_t(
 *             flags=flags,
 *             type=entry_type,             # <<<<<<<<<<<<<<
 *             name=name,
 *             description=NULL,
*/
  __pyx_t_7 = __Pyx_PyLong_As_uint64_t(__pyx_v_entry_type); if (unlikely((__pyx_t_7 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 551, __pyx_L1_error)
  __pyx_t_6.type = __pyx_t_7;

  /* "nvtx/_lib/lib.pyx":552
 *             flags=flags,
 *             type=entry_type,
 *             name=name,             # <<<<<<<<<<<<<<
 *             description=NULL,
 *             arrayOrUnionDetail=array_length,
*/
  __pyx_t_8 = __Pyx_PyObject_AsString(__pyx_v_name); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 552, __pyx_L1_error)
  __pyx_t_6.name = __pyx_t_8;

  /* "nvtx/_lib/lib.pyx":553
 *             type=entry_type,
 *             name=name,
 *             description=NULL,             # <<<<<<<<<<<<<<
 *             arrayOrUnionDetail=array_length,
 *             offset=0,
*/
  __pyx_t_6.description = NULL;

  /* "nvtx/_lib/lib.pyx":554
 *             name=name,
 *             description=NULL,
 *             arrayOrUnionDetail=array_length,             # <<<<<<<<<<<<<<
 *             offset=0,
 *             semantics=semantics,
*/
  __pyx_t_7 = __Pyx_PyLong_As_uint64_t(__pyx_v_array_length); if (unlikely((__pyx_t_7 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 554, __pyx_L1_error)
  __pyx_t_6.arrayOrUnionDetail = __pyx_t_7;
  __pyx_t_6.offset = 0;

  /* "nvtx/_lib/lib.pyx":556
 *             arrayOrUnionDetail=array_length,
 *             offset=0,
 *             semantics=semantics,             # <<<<<<<<<<<<<<
 *             reserved=NULL,
 *         )
*/
  __pyx_t_6.semantics = __pyx_v_semantics;

  /* "nvtx/_lib/lib.pyx":557
 *             offset=0,
 *             semantics=semantics,
 *             reserved=NULL,             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_6.reserved = NULL;
  __pyx_v_schemaEntry = __pyx_t_6;

  /* "nvtx/_lib/lib.pyx":561
 * 
 *         cdef nvtxPayloadSchemaAttr_t schemaAttr
 *         schemaAttr.fieldMask = NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_TYPE | \             # <<<<<<<<<<<<<<
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES | \
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE
*/
  __pyx_v_schemaAttr.fieldMask = (((NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_TYPE | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES) | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES) | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE);

  /* "nvtx/_lib/lib.pyx":564
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES | \
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE
 *         schemaAttr.flags = 0             # <<<<<<<<<<<<<<
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC
 *         schemaAttr.entries = &schemaEntry
*/
  __pyx_v_schemaAttr.flags = 0;

  /* "nvtx/_lib/lib.pyx":565
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE
 *         schemaAttr.flags = 0
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC             # <<<<<<<<<<<<<<
 *         schemaAttr.entries = &schemaEntry
 *         schemaAttr.numEntries = 1
*/
  __pyx_v_schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC;

  /* "nvtx/_lib/lib.pyx":566
 *         schemaAttr.flags = 0
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC
 *         schemaAttr.entries = &schemaEntry             # <<<<<<<<<<<<<<
 *         schemaAttr.numEntries = 1
 *         schemaAttr.payloadStaticSize = dt.itemsize
*/
  __pyx_v_schemaAttr.entries = (&__pyx_v_schemaEntry);

  /* "nvtx/_lib/lib.pyx":567
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC
 *         schemaAttr.entries = &schemaEntry
 *         schemaAttr.numEntries = 1             # <<<<<<<<<<<<<<
 *         schemaAttr.payloadStaticSize = dt.itemsize
 *         return self._do_register(&schemaAttr)
*/
  __pyx_v_schemaAttr.numEntries = 1;

  /* "nvtx/_lib/lib.pyx":568
 *         schemaAttr.entries = &schemaEntry
 *         schemaAttr.numEntries = 1
 *         schemaAttr.payloadStaticSize = dt.itemsize             # <<<<<<<<<<<<<<
 *         return self._do_register(&schemaAttr)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_schemaAttr.payloadStaticSize = __pyx_t_4;

  /* "nvtx/_lib/lib.pyx":569
 *         schemaAttr.numEntries = 1
 *         schemaAttr.payloadStaticSize = dt.itemsize
 *         return self._do_register(&schemaAttr)             # <<<<<<<<<<<<<<
 * 
 *     def _register_structured_schema(self, schema_key):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_7 = ((struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self->__pyx_vtab)->_do_register(__pyx_v_self, (&__pyx_v_schemaAttr)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 569, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyLong_From_uint64_t(__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":528
 *             "SchemaRegistrar subclasses must implement _do_register")
 * 
 *     def _register_builtin_schema(self, dt, metadata):             # <<<<<<<<<<<<<<
 *         name = dt.name.encode()
 *         cdef nvtxSemanticsCounter_t counter_semantics
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._register_builtin_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_counter_semantics_obj);
  __Pyx_XDECREF(__pyx_v_array_length);
  __Pyx_XDECREF(__pyx_v_flags);
  __Pyx_XDECREF(__pyx_v_entry_type);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":571
 *         return self._do_register(&schemaAttr)
 * 
 *     def _register_structured_schema(self, schema_key):             # <<<<<<<<<<<<<<
 *         cdef bint counter_group = schema_key.counter_group
 *         dt = schema_key.dtype
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_5_register_structured_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_4_register_structured_schema, "SchemaRegistrar._register_structured_schema(self, schema_key)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_5_register_structured_schema = {"_register_structured_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_5_register_structured_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_4_register_structured_schema};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_5_register_structured_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_schema_key = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_register_structured_schema (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_schema_key,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 571, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 571, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_register_structured_schema", 0) < (0)) __PYX_ERR(0, 571, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_register_structured_schema", 1, 1, 1, i); __PYX_ERR(0, 571, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 571, __pyx_L3_error)
    }
    __pyx_v_schema_key = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_register_structured_schema", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 571, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._register_structured_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_4_register_structured_schema(((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self), __pyx_v_schema_key);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_4_register_structured_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, PyObject *__pyx_v_schema_key) {
  int __pyx_v_counter_group;
  PyObject *__pyx_v_dt = NULL;
  PyObject *__pyx_v_names = NULL;
  nvtxSemanticsHeader_t const *__pyx_v_semantics;
  nvtxPayloadSchemaAttr_t __pyx_v_schemaAttr;
  uint64_t __pyx_v_schema_flags;
  int __pyx_v_flat;
  nvtxPayloadSchemaEntry_t *__pyx_v_schemaEntries;
  nvtxSemanticsCounter_t *__pyx_v_semanticsEntries;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_field_name = NULL;
  PyObject *__pyx_v_field_type = NULL;
  PyObject *__pyx_v_offset = NULL;
  PyObject *__pyx_v_array_length = NULL;
  PyObject *__pyx_v_flags = NULL;
  PyObject *__pyx_v_field_schema_key = NULL;
  PyObject *__pyx_v_subdtype = NULL;
  PyObject *__pyx_v_shape = NULL;
  PyObject *__pyx_v_entry_type = NULL;
  PyObject *__pyx_v_field_counter_semantics = NULL;
  PyObject *__pyx_v_metadata_flags = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  uint64_t __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *(*__pyx_t_13)(PyObject *);
  Py_ssize_t __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  nvtxPayloadSchemaEntry_t __pyx_t_17;
  char const *__pyx_t_18;
  int __pyx_t_19;
  int __pyx_t_20;
  char const *__pyx_t_21;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  PyObject *__pyx_t_26 = NULL;
  PyObject *__pyx_t_27 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_register_structured_schema", 0);

  /* "nvtx/_lib/lib.pyx":572
 * 
 *     def _register_structured_schema(self, schema_key):
 *         cdef bint counter_group = schema_key.counter_group             # <<<<<<<<<<<<<<
 *         dt = schema_key.dtype
 *         names = []
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_schema_key, __pyx_mstate_global->__pyx_n_u_counter_group); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 572, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_counter_group = __pyx_t_2;

  /* "nvtx/_lib/lib.pyx":573
 *     def _register_structured_schema(self, schema_key):
 *         cdef bint counter_group = schema_key.counter_group
 *         dt = schema_key.dtype             # <<<<<<<<<<<<<<
 *         names = []
 *         cdef const nvtxSemanticsHeader_t* semantics
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_schema_key, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_dt = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":574
 *         cdef bint counter_group = schema_key.counter_group
 *         dt = schema_key.dtype
 *         names = []             # <<<<<<<<<<<<<<
 *         cdef const nvtxSemanticsHeader_t* semantics
 *         cdef nvtxPayloadSchemaAttr_t schemaAttr
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_names = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":577
 *         cdef const nvtxSemanticsHeader_t* semantics
 *         cdef nvtxPayloadSchemaAttr_t schemaAttr
 *         schemaAttr.fieldMask = NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_TYPE | \             # <<<<<<<<<<<<<<
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES | \
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES | \
*/
  __pyx_v_schemaAttr.fieldMask = (((NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_TYPE | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES) | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES) | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE);

  /* "nvtx/_lib/lib.pyx":581
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES | \
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE
 *         schemaAttr.name = NULL             # <<<<<<<<<<<<<<
 *         schemaAttr.flags = 0
 *         cdef uint64_t schema_flags = schema_key.schema_flags
*/
  __pyx_v_schemaAttr.name = NULL;

  /* "nvtx/_lib/lib.pyx":582
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE
 *         schemaAttr.name = NULL
 *         schemaAttr.flags = 0             # <<<<<<<<<<<<<<
 *         cdef uint64_t schema_flags = schema_key.schema_flags
 *         if counter_group:
*/
  __pyx_v_schemaAttr.flags = 0;

  /* "nvtx/_lib/lib.pyx":583
 *         schemaAttr.name = NULL
 *         schemaAttr.flags = 0
 *         cdef uint64_t schema_flags = schema_key.schema_flags             # <<<<<<<<<<<<<<
 *         if counter_group:
 *             schema_flags |= NVTX_PAYLOAD_SCHEMA_FLAG_COUNTER_GROUP
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_schema_key, __pyx_mstate_global->__pyx_n_u_schema_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyLong_As_uint64_t(__pyx_t_1); if (unlikely((__pyx_t_3 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_schema_flags = __pyx_t_3;

  /* "nvtx/_lib/lib.pyx":584
 *         schemaAttr.flags = 0
 *         cdef uint64_t schema_flags = schema_key.schema_flags
 *         if counter_group:             # <<<<<<<<<<<<<<
 *             schema_flags |= NVTX_PAYLOAD_SCHEMA_FLAG_COUNTER_GROUP
 *         if schema_flags:
*/
  if (__pyx_v_counter_group) {

    /* "nvtx/_lib/lib.pyx":585
 *         cdef uint64_t schema_flags = schema_key.schema_flags
 *         if counter_group:
 *             schema_flags |= NVTX_PAYLOAD_SCHEMA_FLAG_COUNTER_GROUP             # <<<<<<<<<<<<<<
 *         if schema_flags:
 *             schemaAttr.fieldMask |= NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_FLAGS
*/
    __pyx_v_schema_flags = (__pyx_v_schema_flags | NVTX_PAYLOAD_SCHEMA_FLAG_COUNTER_GROUP);

    /* "nvtx/_lib/lib.pyx":584
 *         schemaAttr.flags = 0
 *         cdef uint64_t schema_flags = schema_key.schema_flags
 *         if counter_group:             # <<<<<<<<<<<<<<
 *             schema_flags |= NVTX_PAYLOAD_SCHEMA_FLAG_COUNTER_GROUP
 *         if schema_flags:
*/
  }

  /* "nvtx/_lib/lib.pyx":586
 *         if counter_group:
 *             schema_flags |= NVTX_PAYLOAD_SCHEMA_FLAG_COUNTER_GROUP
 *         if schema_flags:             # <<<<<<<<<<<<<<
 *             schemaAttr.fieldMask |= NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_FLAGS
 *             schemaAttr.flags = schema_flags
*/
  __pyx_t_2 = (__pyx_v_schema_flags != 0);
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":587
 *             schema_flags |= NVTX_PAYLOAD_SCHEMA_FLAG_COUNTER_GROUP
 *         if schema_flags:
 *             schemaAttr.fieldMask |= NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_FLAGS             # <<<<<<<<<<<<<<
 *             schemaAttr.flags = schema_flags
 *         # Counter groups and writer event schemas are flat: scalar fields map
*/
    __pyx_v_schemaAttr.fieldMask = (__pyx_v_schemaAttr.fieldMask | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_FLAGS);

    /* "nvtx/_lib/lib.pyx":588
 *         if schema_flags:
 *             schemaAttr.fieldMask |= NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_FLAGS
 *             schemaAttr.flags = schema_flags             # <<<<<<<<<<<<<<
 *         # Counter groups and writer event schemas are flat: scalar fields map
 *         # to predefined entry types rather than nested per-field sub-schemas.
*/
    __pyx_v_schemaAttr.flags = __pyx_v_schema_flags;

    /* "nvtx/_lib/lib.pyx":586
 *         if counter_group:
 *             schema_flags |= NVTX_PAYLOAD_SCHEMA_FLAG_COUNTER_GROUP
 *         if schema_flags:             # <<<<<<<<<<<<<<
 *             schemaAttr.fieldMask |= NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_FLAGS
 *             schemaAttr.flags = schema_flags
*/
  }

  /* "nvtx/_lib/lib.pyx":591
 *         # Counter groups and writer event schemas are flat: scalar fields map
 *         # to predefined entry types rather than nested per-field sub-schemas.
 *         cdef bint flat = counter_group or schema_key.schema_flags != 0             # <<<<<<<<<<<<<<
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC
 *         # dt.names, not dt.fields: the fields dict also contains title
*/
  if (!__pyx_v_counter_group) {
  } else {
    __pyx_t_2 = __pyx_v_counter_group;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_schema_key, __pyx_mstate_global->__pyx_n_u_schema_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = (__Pyx_PyLong_BoolNeObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_0, 0, 0)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 591, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __pyx_t_4;
  __pyx_L5_bool_binop_done:;
  __pyx_v_flat = __pyx_t_2;

  /* "nvtx/_lib/lib.pyx":592
 *         # to predefined entry types rather than nested per-field sub-schemas.
 *         cdef bint flat = counter_group or schema_key.schema_flags != 0
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC             # <<<<<<<<<<<<<<
 *         # dt.names, not dt.fields: the fields dict also contains title
 *         # aliases, which would duplicate titled fields in the schema.
*/
  __pyx_v_schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC;

  /* "nvtx/_lib/lib.pyx":595
 *         # dt.names, not dt.fields: the fields dict also contains title
 *         # aliases, which would duplicate titled fields in the schema.
 *         schemaAttr.numEntries = len(dt.names)             # <<<<<<<<<<<<<<
 *         schemaAttr.payloadStaticSize = dt.itemsize
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 595, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 595, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_schemaAttr.numEntries = __pyx_t_5;

  /* "nvtx/_lib/lib.pyx":596
 *         # aliases, which would duplicate titled fields in the schema.
 *         schemaAttr.numEntries = len(dt.names)
 *         schemaAttr.payloadStaticSize = dt.itemsize             # <<<<<<<<<<<<<<
 * 
 *         cdef nvtxPayloadSchemaEntry_t* schemaEntries = \
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyLong_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_schemaAttr.payloadStaticSize = __pyx_t_6;

  /* "nvtx/_lib/lib.pyx":599
 * 
 *         cdef nvtxPayloadSchemaEntry_t* schemaEntries = \
 *             <nvtxPayloadSchemaEntry_t*>malloc(len(dt.names) * sizeof(nvtxPayloadSchemaEntry_t))             # <<<<<<<<<<<<<<
 *         if schemaEntries is NULL:
 *             raise MemoryError("Failed to allocate memory for schema entries")
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_schemaEntries = ((nvtxPayloadSchemaEntry_t *)malloc((__pyx_t_5 * (sizeof(nvtxPayloadSchemaEntry_t)))));

  /* "nvtx/_lib/lib.pyx":600
 *         cdef nvtxPayloadSchemaEntry_t* schemaEntries = \
 *             <nvtxPayloadSchemaEntry_t*>malloc(len(dt.names) * sizeof(nvtxPayloadSchemaEntry_t))
 *         if schemaEntries is NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Failed to allocate memory for schema entries")
 *         cdef nvtxSemanticsCounter_t* semanticsEntries = \
*/
  __pyx_t_2 = (__pyx_v_schemaEntries == NULL);
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/lib.pyx":601
 *             <nvtxPayloadSchemaEntry_t*>malloc(len(dt.names) * sizeof(nvtxPayloadSchemaEntry_t))
 *         if schemaEntries is NULL:
 *             raise MemoryError("Failed to allocate memory for schema entries")             # <<<<<<<<<<<<<<
 *         cdef nvtxSemanticsCounter_t* semanticsEntries = \
 *             <nvtxSemanticsCounter_t*>malloc(len(dt.names) * sizeof(nvtxSemanticsCounter_t))
*/
    __pyx_t_7 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_Failed_to_allocate_memory_for_sc};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_MemoryError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 601, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":600
 *         cdef nvtxPayloadSchemaEntry_t* schemaEntries = \
 *             <nvtxPayloadSchemaEntry_t*>malloc(len(dt.names) * sizeof(nvtxPayloadSchemaEntry_t))
 *         if schemaEntries is NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Failed to allocate memory for schema entries")
 *         cdef nvtxSemanticsCounter_t* semanticsEntries = \
*/
  }

  /* "nvtx/_lib/lib.pyx":603
 *             raise MemoryError("Failed to allocate memory for schema entries")
 *         cdef nvtxSemanticsCounter_t* semanticsEntries = \
 *             <nvtxSemanticsCounter_t*>malloc(len(dt.names) * sizeof(nvtxSemanticsCounter_t))             # <<<<<<<<<<<<<<
 *         if semanticsEntries is NULL:
 *             free(schemaEntries)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 603, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_semanticsEntries = ((nvtxSemanticsCounter_t *)malloc((__pyx_t_5 * (sizeof(nvtxSemanticsCounter_t)))));

  /* "nvtx/_lib/lib.pyx":604
 *         cdef nvtxSemanticsCounter_t* semanticsEntries = \
 *             <nvtxSemanticsCounter_t*>malloc(len(dt.names) * sizeof(nvtxSemanticsCounter_t))
 *         if semanticsEntries is NULL:             # <<<<<<<<<<<<<<
 *             free(schemaEntries)
 *             raise MemoryError("Failed to allocate memory for schema semantics")
*/
  __pyx_t_2 = (__pyx_v_semanticsEntries == NULL);
  if (unlikely(__pyx_t_2)) {

    /* "nvtx/_lib/lib.pyx":605
 *             <nvtxSemanticsCounter_t*>malloc(len(dt.names) * sizeof(nvtxSemanticsCounter_t))
 *         if semanticsEntries is NULL:
 *             free(schemaEntries)             # <<<<<<<<<<<<<<
 *             raise MemoryError("Failed to allocate memory for schema semantics")
 *         try:
*/
    free(__pyx_v_schemaEntries);

    /* "nvtx/_lib/lib.pyx":606
 *         if semanticsEntries is NULL:
 *             free(schemaEntries)
 *             raise MemoryError("Failed to allocate memory for schema semantics")             # <<<<<<<<<<<<<<
 *         try:
 *             schemaAttr.entries = schemaEntries
*/
    __pyx_t_7 = NULL;
    __pyx_t_6 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_mstate_global->__pyx_kp_u_Failed_to_allocate_memory_for_sc_2};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_MemoryError)), __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 606, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":604
 *         cdef nvtxSemanticsCounter_t* semanticsEntries = \
 *             <nvtxSemanticsCounter_t*>malloc(len(dt.names) * sizeof(nvtxSemanticsCounter_t))
 *         if semanticsEntries is NULL:             # <<<<<<<<<<<<<<
 *             free(schemaEntries)
 *             raise MemoryError("Failed to allocate memory for schema semantics")
*/
  }

  /* "nvtx/_lib/lib.pyx":607
 *             free(schemaEntries)
 *             raise MemoryError("Failed to allocate memory for schema semantics")
 *         try:             # <<<<<<<<<<<<<<
 *             schemaAttr.entries = schemaEntries
 *             for i, field_name in enumerate(dt.names):
*/
  /*try:*/ {

    /* "nvtx/_lib/lib.pyx":608
 *             raise MemoryError("Failed to allocate memory for schema semantics")
 *         try:
 *             schemaAttr.entries = schemaEntries             # <<<<<<<<<<<<<<
 *             for i, field_name in enumerate(dt.names):
 *                 field_type, offset = dt.fields[field_name][:2]
*/
    __pyx_v_schemaAttr.entries = __pyx_v_schemaEntries;

    /* "nvtx/_lib/lib.pyx":609
 *         try:
 *             schemaAttr.entries = schemaEntries
 *             for i, field_name in enumerate(dt.names):             # <<<<<<<<<<<<<<
 *                 field_type, offset = dt.fields[field_name][:2]
 *                 array_length = 0
*/
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
    __pyx_t_1 = __pyx_mstate_global->__pyx_int_0;
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_names); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 609, __pyx_L10_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (likely(PyList_CheckExact(__pyx_t_7)) || PyTuple_CheckExact(__pyx_t_7)) {
      __pyx_t_8 = __pyx_t_7; __Pyx_INCREF(__pyx_t_8);
      __pyx_t_5 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 609, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 609, __pyx_L10_error)
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_8))) {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_8);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 609, __pyx_L10_error)
            #endif
            if (__pyx_t_5 >= __pyx_temp) break;
          }
          __pyx_t_7 = __Pyx_PyList_GET_ITEM_REF(__pyx_t_8, __pyx_t_5, __Pyx_ReferenceSharing_OwnStrongReference);
          ++__pyx_t_5;
        } else {
          {
            Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_8);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 609, __pyx_L10_error)
            #endif
            if (__pyx_t_5 >= __pyx_temp) break;
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_7 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_5));
          #else
          __pyx_t_7 = __Pyx_PySequence_ITEM(__pyx_t_8, __pyx_t_5);
          #endif
          ++__pyx_t_5;
        }
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 609, __pyx_L10_error)
      } else {
        __pyx_t_7 = __pyx_t_9(__pyx_t_8);
        if (unlikely(!__pyx_t_7)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) __PYX_ERR(0, 609, __pyx_L10_error)
            PyErr_Clear();
          }
          break;
        }
      }
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_XDECREF_SET(__pyx_v_field_name, __pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_1);
      __pyx_t_7 = __Pyx_PyLong_AddObjC(__pyx_t_1, __pyx_mstate_global->__pyx_int_1, 1, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 609, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1);
      __pyx_t_1 = __pyx_t_7;
      __pyx_t_7 = 0;

      /* "nvtx/_lib/lib.pyx":610
 *             schemaAttr.entries = schemaEntries
 *             for i, field_name in enumerate(dt.names):
 *                 field_type, offset = dt.fields[field_name][:2]             # <<<<<<<<<<<<<<
 *                 array_length = 0
 *                 flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED
*/
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_fields); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 610, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_t_7, __pyx_v_field_name); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 610, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetSlice(__pyx_t_10, 0, 2, NULL, NULL, &__pyx_mstate_global->__pyx_slice[0], 0, 1, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 610, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
        PyObject* sequence = __pyx_t_7;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 610, __pyx_L10_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0);
          __Pyx_INCREF(__pyx_t_10);
          __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1);
          __Pyx_INCREF(__pyx_t_11);
        } else {
          __pyx_t_10 = __Pyx_PyList_GET_ITEM_REF(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 610, __pyx_L10_error)
          __Pyx_XGOTREF(__pyx_t_10);
          __pyx_t_11 = __Pyx_PyList_GET_ITEM_REF(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 610, __pyx_L10_error)
          __Pyx_XGOTREF(__pyx_t_11);
        }
        #else
        __pyx_t_10 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 610, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 610, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_11);
        #endif
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_12 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 610, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_13 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_12);
        index = 0; __pyx_t_10 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_10)) goto __pyx_L14_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        index = 1; __pyx_t_11 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L14_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_11);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 2) < (0)) __PYX_ERR(0, 610, __pyx_L10_error)
        __pyx_t_13 = NULL;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        goto __pyx_L15_unpacking_done;
        __pyx_L14_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_13 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 610, __pyx_L10_error)
        __pyx_L15_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_field_type, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_XDECREF_SET(__pyx_v_offset, __pyx_t_11);
      __pyx_t_11 = 0;

      /* "nvtx/_lib/lib.pyx":611
 *             for i, field_name in enumerate(dt.names):
 *                 field_type, offset = dt.fields[field_name][:2]
 *                 array_length = 0             # <<<<<<<<<<<<<<
 *                 flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED
 *                 field_schema_key = PayloadSchemaKey(field_type)
*/
      __Pyx_INCREF(__pyx_mstate_global->__pyx_int_0);
      __Pyx_XDECREF_SET(__pyx_v_array_length, __pyx_mstate_global->__pyx_int_0);

      /* "nvtx/_lib/lib.pyx":612
 *                 field_type, offset = dt.fields[field_name][:2]
 *                 array_length = 0
 *                 flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED             # <<<<<<<<<<<<<<
 *                 field_schema_key = PayloadSchemaKey(field_type)
 *                 semantics = NULL
*/
      __pyx_t_7 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_FLAG_UNUSED); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 612, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_XDECREF_SET(__pyx_v_flags, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "nvtx/_lib/lib.pyx":613
 *                 array_length = 0
 *                 flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED
 *                 field_schema_key = PayloadSchemaKey(field_type)             # <<<<<<<<<<<<<<
 *                 semantics = NULL
 * 
*/
      __pyx_t_11 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_PayloadSchemaKey); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 613, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
        assert(__pyx_t_11);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_v_field_type};
        __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 613, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_7);
      }
      __Pyx_XDECREF_SET(__pyx_v_field_schema_key, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "nvtx/_lib/lib.pyx":614
 *                 flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED
 *                 field_schema_key = PayloadSchemaKey(field_type)
 *                 semantics = NULL             # <<<<<<<<<<<<<<
 * 
 *                 if flat and field_type.subdtype:
*/
      __pyx_v_semantics = NULL;

      /* "nvtx/_lib/lib.pyx":616
 *                 semantics = NULL
 * 
 *                 if flat and field_type.subdtype:             # <<<<<<<<<<<<<<
 *                     subdtype, shape = field_type.subdtype
 *                     if subdtype.type in _dtype_to_entry_type:
*/
      if (__pyx_v_flat) {
      } else {
        __pyx_t_2 = __pyx_v_flat;
        goto __pyx_L17_bool_binop_done;
      }
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_type, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 616, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 616, __pyx_L10_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_2 = __pyx_t_4;
      __pyx_L17_bool_binop_done:;
      if (__pyx_t_2) {

        /* "nvtx/_lib/lib.pyx":617
 * 
 *                 if flat and field_type.subdtype:
 *                     subdtype, shape = field_type.subdtype             # <<<<<<<<<<<<<<
 *                     if subdtype.type in _dtype_to_entry_type:
 *                         entry_type = _dtype_to_entry_type[subdtype.type]
*/
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_type, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 617, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_7);
        if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
          PyObject* sequence = __pyx_t_7;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 617, __pyx_L10_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0);
            __Pyx_INCREF(__pyx_t_10);
            __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1);
            __Pyx_INCREF(__pyx_t_11);
          } else {
            __pyx_t_10 = __Pyx_PyList_GET_ITEM_REF(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 617, __pyx_L10_error)
            __Pyx_XGOTREF(__pyx_t_10);
            __pyx_t_11 = __Pyx_PyList_GET_ITEM_REF(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
            if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 617, __pyx_L10_error)
            __Pyx_XGOTREF(__pyx_t_11);
          }
          #else
          __pyx_t_10 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 617, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 617, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_11);
          #endif
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_12 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 617, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __pyx_t_13 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_12);
          index = 0; __pyx_t_10 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_10)) goto __pyx_L19_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_10);
          index = 1; __pyx_t_11 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L19_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_11);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 2) < (0)) __PYX_ERR(0, 617, __pyx_L10_error)
          __pyx_t_13 = NULL;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          goto __pyx_L20_unpacking_done;
          __pyx_L19_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __pyx_t_13 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 617, __pyx_L10_error)
          __pyx_L20_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_subdtype, __pyx_t_10);
        __pyx_t_10 = 0;
        __Pyx_XDECREF_SET(__pyx_v_shape, __pyx_t_11);
        __pyx_t_11 = 0;

        /* "nvtx/_lib/lib.pyx":618
 *                 if flat and field_type.subdtype:
 *                     subdtype, shape = field_type.subdtype
 *                     if subdtype.type in _dtype_to_entry_type:             # <<<<<<<<<<<<<<
 *                         entry_type = _dtype_to_entry_type[subdtype.type]
 *                         array_length = np.prod(shape)
*/
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_subdtype, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 618, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_dtype_to_entry_type); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 618, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_7, __pyx_t_11, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 618, __pyx_L10_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        if (__pyx_t_2) {

          /* "nvtx/_lib/lib.pyx":619
 *                     subdtype, shape = field_type.subdtype
 *                     if subdtype.type in _dtype_to_entry_type:
 *                         entry_type = _dtype_to_entry_type[subdtype.type]             # <<<<<<<<<<<<<<
 *                         array_length = np.prod(shape)
 *                         flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
*/
          __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_dtype_to_entry_type); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 619, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_subdtype, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 619, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_10 = __Pyx_PyObject_GetItem(__pyx_t_11, __pyx_t_7); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 619, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF_SET(__pyx_v_entry_type, __pyx_t_10);
          __pyx_t_10 = 0;

          /* "nvtx/_lib/lib.pyx":620
 *                     if subdtype.type in _dtype_to_entry_type:
 *                         entry_type = _dtype_to_entry_type[subdtype.type]
 *                         array_length = np.prod(shape)             # <<<<<<<<<<<<<<
 *                         flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
 *                     else:
*/
          __pyx_t_7 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 620, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_prod); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 620, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_12))) {
            __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_12);
            assert(__pyx_t_7);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_v_shape};
            __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 620, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_10);
          }
          __Pyx_DECREF_SET(__pyx_v_array_length, __pyx_t_10);
          __pyx_t_10 = 0;

          /* "nvtx/_lib/lib.pyx":621
 *                         entry_type = _dtype_to_entry_type[subdtype.type]
 *                         array_length = np.prod(shape)
 *                         flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE             # <<<<<<<<<<<<<<
 *                     else:
 *                         entry_type = self._get_numpy_dtype_schema(field_schema_key)
*/
          __pyx_t_10 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 621, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF_SET(__pyx_v_flags, __pyx_t_10);
          __pyx_t_10 = 0;

          /* "nvtx/_lib/lib.pyx":618
 *                 if flat and field_type.subdtype:
 *                     subdtype, shape = field_type.subdtype
 *                     if subdtype.type in _dtype_to_entry_type:             # <<<<<<<<<<<<<<
 *                         entry_type = _dtype_to_entry_type[subdtype.type]
 *                         array_length = np.prod(shape)
*/
          goto __pyx_L21;
        }

        /* "nvtx/_lib/lib.pyx":623
 *                         flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
 *                     else:
 *                         entry_type = self._get_numpy_dtype_schema(field_schema_key)             # <<<<<<<<<<<<<<
 *                 elif flat:
 *                     entry_type = _entry_type_from_metadata(
*/
        /*else*/ {
          __pyx_t_12 = ((PyObject *)__pyx_v_self);
          __Pyx_INCREF(__pyx_t_12);
          __pyx_t_6 = 0;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_v_field_schema_key};
            __pyx_t_10 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 623, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_10);
          }
          __Pyx_XDECREF_SET(__pyx_v_entry_type, __pyx_t_10);
          __pyx_t_10 = 0;
        }
        __pyx_L21:;

        /* "nvtx/_lib/lib.pyx":616
 *                 semantics = NULL
 * 
 *                 if flat and field_type.subdtype:             # <<<<<<<<<<<<<<
 *                     subdtype, shape = field_type.subdtype
 *                     if subdtype.type in _dtype_to_entry_type:
*/
        goto __pyx_L16;
      }

      /* "nvtx/_lib/lib.pyx":624
 *                     else:
 *                         entry_type = self._get_numpy_dtype_schema(field_schema_key)
 *                 elif flat:             # <<<<<<<<<<<<<<
 *                     entry_type = _entry_type_from_metadata(
 *                         field_schema_key.metadata
*/
      if (__pyx_v_flat) {

        /* "nvtx/_lib/lib.pyx":625
 *                         entry_type = self._get_numpy_dtype_schema(field_schema_key)
 *                 elif flat:
 *                     entry_type = _entry_type_from_metadata(             # <<<<<<<<<<<<<<
 *                         field_schema_key.metadata
 *                     )
*/
        __pyx_t_12 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_entry_type_from_metadata); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 625, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_7);

        /* "nvtx/_lib/lib.pyx":626
 *                 elif flat:
 *                     entry_type = _entry_type_from_metadata(
 *                         field_schema_key.metadata             # <<<<<<<<<<<<<<
 *                     )
 *                     if entry_type is None:
*/
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_schema_key, __pyx_mstate_global->__pyx_n_u_metadata); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 626, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_7);
          assert(__pyx_t_12);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_t_11};
          __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 625, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_10);
        }
        __Pyx_XDECREF_SET(__pyx_v_entry_type, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "nvtx/_lib/lib.pyx":628
 *                         field_schema_key.metadata
 *                     )
 *                     if entry_type is None:             # <<<<<<<<<<<<<<
 *                         if field_type.type in _dtype_to_entry_type:
 *                             entry_type = _dtype_to_entry_type[field_type.type]
*/
        __pyx_t_2 = (__pyx_v_entry_type == Py_None);
        if (__pyx_t_2) {

          /* "nvtx/_lib/lib.pyx":629
 *                     )
 *                     if entry_type is None:
 *                         if field_type.type in _dtype_to_entry_type:             # <<<<<<<<<<<<<<
 *                             entry_type = _dtype_to_entry_type[field_type.type]
 *                         else:
*/
          __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_type, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 629, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_dtype_to_entry_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 629, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_10, __pyx_t_7, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 629, __pyx_L10_error)
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (__pyx_t_2) {

            /* "nvtx/_lib/lib.pyx":630
 *                     if entry_type is None:
 *                         if field_type.type in _dtype_to_entry_type:
 *                             entry_type = _dtype_to_entry_type[field_type.type]             # <<<<<<<<<<<<<<
 *                         else:
 *                             entry_type = self._get_numpy_dtype_schema(
*/
            __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_dtype_to_entry_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 630, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_7);
            __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_type, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 630, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_10);
            __pyx_t_11 = __Pyx_PyObject_GetItem(__pyx_t_7, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 630, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_11);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF_SET(__pyx_v_entry_type, __pyx_t_11);
            __pyx_t_11 = 0;

            /* "nvtx/_lib/lib.pyx":629
 *                     )
 *                     if entry_type is None:
 *                         if field_type.type in _dtype_to_entry_type:             # <<<<<<<<<<<<<<
 *                             entry_type = _dtype_to_entry_type[field_type.type]
 *                         else:
*/
            goto __pyx_L23;
          }

          /* "nvtx/_lib/lib.pyx":632
 *                             entry_type = _dtype_to_entry_type[field_type.type]
 *                         else:
 *                             entry_type = self._get_numpy_dtype_schema(             # <<<<<<<<<<<<<<
 *                                 field_schema_key
 *                             )
*/
          /*else*/ {
            __pyx_t_10 = ((PyObject *)__pyx_v_self);
            __Pyx_INCREF(__pyx_t_10);

            /* "nvtx/_lib/lib.pyx":633
 *                         else:
 *                             entry_type = self._get_numpy_dtype_schema(
 *                                 field_schema_key             # <<<<<<<<<<<<<<
 *                             )
 *                 else:
*/
            __pyx_t_6 = 0;
            {
              PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_field_schema_key};
              __pyx_t_11 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 632, __pyx_L10_error)
              __Pyx_GOTREF(__pyx_t_11);
            }
            __Pyx_DECREF_SET(__pyx_v_entry_type, __pyx_t_11);
            __pyx_t_11 = 0;
          }
          __pyx_L23:;

          /* "nvtx/_lib/lib.pyx":628
 *                         field_schema_key.metadata
 *                     )
 *                     if entry_type is None:             # <<<<<<<<<<<<<<
 *                         if field_type.type in _dtype_to_entry_type:
 *                             entry_type = _dtype_to_entry_type[field_type.type]
*/
        }

        /* "nvtx/_lib/lib.pyx":624
 *                     else:
 *                         entry_type = self._get_numpy_dtype_schema(field_schema_key)
 *                 elif flat:             # <<<<<<<<<<<<<<
 *                     entry_type = _entry_type_from_metadata(
 *                         field_schema_key.metadata
*/
        goto __pyx_L16;
      }

      /* "nvtx/_lib/lib.pyx":636
 *                             )
 *                 else:
 *                     entry_type = self._get_numpy_dtype_schema(field_schema_key)             # <<<<<<<<<<<<<<
 * 
 *                 if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
*/
      /*else*/ {
        __pyx_t_10 = ((PyObject *)__pyx_v_self);
        __Pyx_INCREF(__pyx_t_10);
        __pyx_t_6 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_v_field_schema_key};
          __pyx_t_11 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 636, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_11);
        }
        __Pyx_XDECREF_SET(__pyx_v_entry_type, __pyx_t_11);
        __pyx_t_11 = 0;
      }
      __pyx_L16:;

      /* "nvtx/_lib/lib.pyx":638
 *                     entry_type = self._get_numpy_dtype_schema(field_schema_key)
 * 
 *                 if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:             # <<<<<<<<<<<<<<
 *                     array_length = field_type.itemsize // 4
 *                 elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
*/
      __pyx_t_11 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 638, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_10 = PyObject_RichCompare(__pyx_v_entry_type, __pyx_t_11, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 638, __pyx_L10_error)
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 638, __pyx_L10_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (__pyx_t_2) {

        /* "nvtx/_lib/lib.pyx":639
 * 
 *                 if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
 *                     array_length = field_type.itemsize // 4             # <<<<<<<<<<<<<<
 *                 elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
 *                     array_length = field_type.itemsize
*/
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_type, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 639, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = __Pyx_PyLong_FloorDivideObjC(__pyx_t_10, __pyx_mstate_global->__pyx_int_4, 4, 0, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 639, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF_SET(__pyx_v_array_length, __pyx_t_11);
        __pyx_t_11 = 0;

        /* "nvtx/_lib/lib.pyx":638
 *                     entry_type = self._get_numpy_dtype_schema(field_schema_key)
 * 
 *                 if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:             # <<<<<<<<<<<<<<
 *                     array_length = field_type.itemsize // 4
 *                 elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
*/
        goto __pyx_L24;
      }

      /* "nvtx/_lib/lib.pyx":640
 *                 if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
 *                     array_length = field_type.itemsize // 4
 *                 elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:             # <<<<<<<<<<<<<<
 *                     array_length = field_type.itemsize
 *                     flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
*/
      __pyx_t_11 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_BYTE); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 640, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_10 = PyObject_RichCompare(__pyx_v_entry_type, __pyx_t_11, Py_EQ); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 640, __pyx_L10_error)
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 640, __pyx_L10_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (__pyx_t_2) {

        /* "nvtx/_lib/lib.pyx":641
 *                     array_length = field_type.itemsize // 4
 *                 elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
 *                     array_length = field_type.itemsize             # <<<<<<<<<<<<<<
 *                     flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
 * 
*/
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_type, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 641, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF_SET(__pyx_v_array_length, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "nvtx/_lib/lib.pyx":642
 *                 elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:
 *                     array_length = field_type.itemsize
 *                     flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE             # <<<<<<<<<<<<<<
 * 
 *                 field_counter_semantics = _counter_semantics_from_metadata(
*/
        __pyx_t_10 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 642, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF_SET(__pyx_v_flags, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "nvtx/_lib/lib.pyx":640
 *                 if entry_type == NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32:
 *                     array_length = field_type.itemsize // 4
 *                 elif entry_type == NVTX_PAYLOAD_ENTRY_TYPE_BYTE:             # <<<<<<<<<<<<<<
 *                     array_length = field_type.itemsize
 *                     flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
*/
      }
      __pyx_L24:;

      /* "nvtx/_lib/lib.pyx":644
 *                     flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE
 * 
 *                 field_counter_semantics = _counter_semantics_from_metadata(             # <<<<<<<<<<<<<<
 *                     field_schema_key.metadata
 *                 )
*/
      __pyx_t_11 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_counter_semantics_from_metadata); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 644, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_7);

      /* "nvtx/_lib/lib.pyx":645
 * 
 *                 field_counter_semantics = _counter_semantics_from_metadata(
 *                     field_schema_key.metadata             # <<<<<<<<<<<<<<
 *                 )
 *                 if field_counter_semantics is not None:
*/
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_schema_key, __pyx_mstate_global->__pyx_n_u_metadata); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 645, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
        assert(__pyx_t_11);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_12};
        __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 644, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_XDECREF_SET(__pyx_v_field_counter_semantics, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "nvtx/_lib/lib.pyx":647
 *                     field_schema_key.metadata
 *                 )
 *                 if field_counter_semantics is not None:             # <<<<<<<<<<<<<<
 *                     _fill_counter_semantics(
 *                         &semanticsEntries[i], field_counter_semantics
*/
      __pyx_t_2 = (__pyx_v_field_counter_semantics != Py_None);
      if (__pyx_t_2) {

        /* "nvtx/_lib/lib.pyx":649
 *                 if field_counter_semantics is not None:
 *                     _fill_counter_semantics(
 *                         &semanticsEntries[i], field_counter_semantics             # <<<<<<<<<<<<<<
 *                     )
 *                     semantics = &semanticsEntries[i].header
*/
        __pyx_t_14 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_14 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 649, __pyx_L10_error)

        /* "nvtx/_lib/lib.pyx":648
 *                 )
 *                 if field_counter_semantics is not None:
 *                     _fill_counter_semantics(             # <<<<<<<<<<<<<<
 *                         &semanticsEntries[i], field_counter_semantics
 *                     )
*/
        __pyx_f_4nvtx_4_lib_8counters__fill_counter_semantics((&(__pyx_v_semanticsEntries[__pyx_t_14])), __pyx_v_field_counter_semantics); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 648, __pyx_L10_error)

        /* "nvtx/_lib/lib.pyx":651
 *                         &semanticsEntries[i], field_counter_semantics
 *                     )
 *                     semantics = &semanticsEntries[i].header             # <<<<<<<<<<<<<<
 * 
 *                 metadata_flags = _entry_flags_from_metadata(
*/
        __pyx_t_14 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_14 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 651, __pyx_L10_error)
        __pyx_v_semantics = (&(__pyx_v_semanticsEntries[__pyx_t_14]).header);

        /* "nvtx/_lib/lib.pyx":647
 *                     field_schema_key.metadata
 *                 )
 *                 if field_counter_semantics is not None:             # <<<<<<<<<<<<<<
 *                     _fill_counter_semantics(
 *                         &semanticsEntries[i], field_counter_semantics
*/
      }

      /* "nvtx/_lib/lib.pyx":653
 *                     semantics = &semanticsEntries[i].header
 * 
 *                 metadata_flags = _entry_flags_from_metadata(             # <<<<<<<<<<<<<<
 *                     field_schema_key.metadata
 *                 )
*/
      __pyx_t_7 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_entry_flags_from_metadata); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 653, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_12);

      /* "nvtx/_lib/lib.pyx":654
 * 
 *                 metadata_flags = _entry_flags_from_metadata(
 *                     field_schema_key.metadata             # <<<<<<<<<<<<<<
 *                 )
 *                 flags |= metadata_flags
*/
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_field_schema_key, __pyx_mstate_global->__pyx_n_u_metadata); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 654, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_12);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_11};
        __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 653, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_XDECREF_SET(__pyx_v_metadata_flags, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "nvtx/_lib/lib.pyx":656
 *                     field_schema_key.metadata
 *                 )
 *                 flags |= metadata_flags             # <<<<<<<<<<<<<<
 *                 if counter_group and not (
 *                     metadata_flags & NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
*/
      __pyx_t_10 = PyNumber_InPlaceOr(__pyx_v_flags, __pyx_v_metadata_flags); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 656, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF_SET(__pyx_v_flags, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "nvtx/_lib/lib.pyx":657
 *                 )
 *                 flags |= metadata_flags
 *                 if counter_group and not (             # <<<<<<<<<<<<<<
 *                     metadata_flags & NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
 *                 ):
*/
      if (__pyx_v_counter_group) {
      } else {
        __pyx_t_2 = __pyx_v_counter_group;
        goto __pyx_L27_bool_binop_done;
      }

      /* "nvtx/_lib/lib.pyx":658
 *                 flags |= metadata_flags
 *                 if counter_group and not (
 *                     metadata_flags & NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP             # <<<<<<<<<<<<<<
 *                 ):
 *                     flags |= NVTX_PAYLOAD_ENTRY_FLAG_COUNTER
*/
      __pyx_t_10 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 658, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_12 = PyNumber_And(__pyx_v_metadata_flags, __pyx_t_10); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 658, __pyx_L10_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 658, __pyx_L10_error)
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

      /* "nvtx/_lib/lib.pyx":657
 *                 )
 *                 flags |= metadata_flags
 *                 if counter_group and not (             # <<<<<<<<<<<<<<
 *                     metadata_flags & NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
 *                 ):
*/
      __pyx_t_15 = (!__pyx_t_4);
      __pyx_t_2 = __pyx_t_15;
      __pyx_L27_bool_binop_done:;
      if (__pyx_t_2) {

        /* "nvtx/_lib/lib.pyx":660
 *                     metadata_flags & NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
 *                 ):
 *                     flags |= NVTX_PAYLOAD_ENTRY_FLAG_COUNTER             # <<<<<<<<<<<<<<
 * 
 *                 name = field_name.encode()
*/
        __pyx_t_12 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_FLAG_COUNTER); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 660, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_10 = PyNumber_InPlaceOr(__pyx_v_flags, __pyx_t_12); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 660, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF_SET(__pyx_v_flags, __pyx_t_10);
        __pyx_t_10 = 0;

        /* "nvtx/_lib/lib.pyx":657
 *                 )
 *                 flags |= metadata_flags
 *                 if counter_group and not (             # <<<<<<<<<<<<<<
 *                     metadata_flags & NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
 *                 ):
*/
      }

      /* "nvtx/_lib/lib.pyx":662
 *                     flags |= NVTX_PAYLOAD_ENTRY_FLAG_COUNTER
 * 
 *                 name = field_name.encode()             # <<<<<<<<<<<<<<
 *                 names.append(name)
 *                 schemaEntries[i] = nvtxPayloadSchemaEntry_t(
*/
      __pyx_t_12 = __pyx_v_field_name;
      __Pyx_INCREF(__pyx_t_12);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_12, NULL};
        __pyx_t_10 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 662, __pyx_L10_error)
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "nvtx/_lib/lib.pyx":663
 * 
 *                 name = field_name.encode()
 *                 names.append(name)             # <<<<<<<<<<<<<<
 *                 schemaEntries[i] = nvtxPayloadSchemaEntry_t(
 *                     flags=flags,
*/
      __pyx_t_16 = __Pyx_PyList_Append(__pyx_v_names, __pyx_v_name); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 663, __pyx_L10_error)

      /* "nvtx/_lib/lib.pyx":665
 *                 names.append(name)
 *                 schemaEntries[i] = nvtxPayloadSchemaEntry_t(
 *                     flags=flags,             # <<<<<<<<<<<<<<
 *                     type=entry_type,
 *                     name=name,
*/
      __pyx_t_3 = __Pyx_PyLong_As_uint64_t(__pyx_v_flags); if (unlikely((__pyx_t_3 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 665, __pyx_L10_error)
      __pyx_t_17.flags = __pyx_t_3;

      /* "nvtx/_lib/lib.pyx":666
 *                 schemaEntries[i] = nvtxPayloadSchemaEntry_t(
 *                     flags=flags,
 *                     type=entry_type,             # <<<<<<<<<<<<<<
 *                     name=name,
 *                     description=NULL,
*/
      __pyx_t_3 = __Pyx_PyLong_As_uint64_t(__pyx_v_entry_type); if (unlikely((__pyx_t_3 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 666, __pyx_L10_error)
      __pyx_t_17.type = __pyx_t_3;

      /* "nvtx/_lib/lib.pyx":667
 *                     flags=flags,
 *                     type=entry_type,
 *                     name=name,             # <<<<<<<<<<<<<<
 *                     description=NULL,
 *                     arrayOrUnionDetail=array_length,
*/
      __pyx_t_18 = __Pyx_PyObject_AsString(__pyx_v_name); if (unlikely((!__pyx_t_18) && PyErr_Occurred())) __PYX_ERR(0, 667, __pyx_L10_error)
      __pyx_t_17.name = __pyx_t_18;

      /* "nvtx/_lib/lib.pyx":668
 *                     type=entry_type,
 *                     name=name,
 *                     description=NULL,             # <<<<<<<<<<<<<<
 *                     arrayOrUnionDetail=array_length,
 *                     offset=offset,
*/
      __pyx_t_17.description = NULL;

      /* "nvtx/_lib/lib.pyx":669
 *                     name=name,
 *                     description=NULL,
 *                     arrayOrUnionDetail=array_length,             # <<<<<<<<<<<<<<
 *                     offset=offset,
 *                     semantics=semantics,
*/
      __pyx_t_3 = __Pyx_PyLong_As_uint64_t(__pyx_v_array_length); if (unlikely((__pyx_t_3 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 669, __pyx_L10_error)
      __pyx_t_17.arrayOrUnionDetail = __pyx_t_3;

      /* "nvtx/_lib/lib.pyx":670
 *                     description=NULL,
 *                     arrayOrUnionDetail=array_length,
 *                     offset=offset,             # <<<<<<<<<<<<<<
 *                     semantics=semantics,
 *                     reserved=NULL,
*/
      __pyx_t_3 = __Pyx_PyLong_As_uint64_t(__pyx_v_offset); if (unlikely((__pyx_t_3 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 670, __pyx_L10_error)
      __pyx_t_17.offset = __pyx_t_3;

      /* "nvtx/_lib/lib.pyx":671
 *                     arrayOrUnionDetail=array_length,
 *                     offset=offset,
 *                     semantics=semantics,             # <<<<<<<<<<<<<<
 *                     reserved=NULL,
 *                 )
*/
      __pyx_t_17.semantics = __pyx_v_semantics;

      /* "nvtx/_lib/lib.pyx":672
 *                     offset=offset,
 *                     semantics=semantics,
 *                     reserved=NULL,             # <<<<<<<<<<<<<<
 *                 )
 *             return self._do_register(&schemaAttr)
*/
      __pyx_t_17.reserved = NULL;

      /* "nvtx/_lib/lib.pyx":664
 *                 name = field_name.encode()
 *                 names.append(name)
 *                 schemaEntries[i] = nvtxPayloadSchemaEntry_t(             # <<<<<<<<<<<<<<
 *                     flags=flags,
 *                     type=entry_type,
*/
      __pyx_t_14 = __Pyx_PyIndex_AsSsize_t(__pyx_v_i); if (unlikely((__pyx_t_14 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 664, __pyx_L10_error)
      (__pyx_v_schemaEntries[__pyx_t_14]) = __pyx_t_17;

      /* "nvtx/_lib/lib.pyx":609
 *         try:
 *             schemaAttr.entries = schemaEntries
 *             for i, field_name in enumerate(dt.names):             # <<<<<<<<<<<<<<
 *                 field_type, offset = dt.fields[field_name][:2]
 *                 array_length = 0
*/
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "nvtx/_lib/lib.pyx":674
 *                     reserved=NULL,
 *                 )
 *             return self._do_register(&schemaAttr)             # <<<<<<<<<<<<<<
 *         finally:
 *             free(schemaEntries)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = ((struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self->__pyx_vtab)->_do_register(__pyx_v_self, (&__pyx_v_schemaAttr)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 674, __pyx_L10_error)
    __pyx_t_1 = __Pyx_PyLong_From_uint64_t(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 674, __pyx_L10_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L9_return;
  }

  /* "nvtx/_lib/lib.pyx":676
 *             return self._do_register(&schemaAttr)
 *         finally:
 *             free(schemaEntries)             # <<<<<<<<<<<<<<
 *             free(semanticsEntries)
 * 
*/
  /*finally:*/ {
    __pyx_L10_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0; __pyx_t_26 = 0; __pyx_t_27 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_25, &__pyx_t_26, &__pyx_t_27);
      if ( unlikely(__Pyx_GetException(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24) < 0)) __Pyx_ErrFetch(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24);
      __Pyx_XGOTREF(__pyx_t_22);
      __Pyx_XGOTREF(__pyx_t_23);
      __Pyx_XGOTREF(__pyx_t_24);
      __Pyx_XGOTREF(__pyx_t_25);
      __Pyx_XGOTREF(__pyx_t_26);
      __Pyx_XGOTREF(__pyx_t_27);
      __pyx_t_19 = __pyx_lineno; __pyx_t_20 = __pyx_clineno; __pyx_t_21 = __pyx_filename;
      {
        free(__pyx_v_schemaEntries);

        /* "nvtx/_lib/lib.pyx":677
 *         finally:
 *             free(schemaEntries)
 *             free(semanticsEntries)             # <<<<<<<<<<<<<<
 * 
 *     def _get_numpy_array_schema(self, uint64_t scalar_schema):
*/
        free(__pyx_v_semanticsEntries);
      }
      __Pyx_XGIVEREF(__pyx_t_25);
      __Pyx_XGIVEREF(__pyx_t_26);
      __Pyx_XGIVEREF(__pyx_t_27);
      __Pyx_ExceptionReset(__pyx_t_25, __pyx_t_26, __pyx_t_27);
      __Pyx_XGIVEREF(__pyx_t_22);
      __Pyx_XGIVEREF(__pyx_t_23);
      __Pyx_XGIVEREF(__pyx_t_24);
      __Pyx_ErrRestore(__pyx_t_22, __pyx_t_23, __pyx_t_24);
      __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0; __pyx_t_26 = 0; __pyx_t_27 = 0;
      __pyx_lineno = __pyx_t_19; __pyx_clineno = __pyx_t_20; __pyx_filename = __pyx_t_21;
      goto __pyx_L1_error;
    }
    __pyx_L9_return: {
      __pyx_t_27 = __pyx_r;
      __pyx_r = 0;

      /* "nvtx/_lib/lib.pyx":676
 *             return self._do_register(&schemaAttr)
 *         finally:
 *             free(schemaEntries)             # <<<<<<<<<<<<<<
 *             free(semanticsEntries)
 * 
*/
      free(__pyx_v_schemaEntries);

      /* "nvtx/_lib/lib.pyx":677
 *         finally:
 *             free(schemaEntries)
 *             free(semanticsEntries)             # <<<<<<<<<<<<<<
 * 
 *     def _get_numpy_array_schema(self, uint64_t scalar_schema):
*/
      free(__pyx_v_semanticsEntries);
      __pyx_r = __pyx_t_27;
      __pyx_t_27 = 0;
      goto __pyx_L0;
    }
  }

  /* "nvtx/_lib/lib.pyx":571
 *         return self._do_register(&schemaAttr)
 * 
 *     def _register_structured_schema(self, schema_key):             # <<<<<<<<<<<<<<
 *         cdef bint counter_group = schema_key.counter_group
 *         dt = schema_key.dtype
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._register_structured_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dt);
  __Pyx_XDECREF(__pyx_v_names);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_field_name);
  __Pyx_XDECREF(__pyx_v_field_type);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_array_length);
  __Pyx_XDECREF(__pyx_v_flags);
  __Pyx_XDECREF(__pyx_v_field_schema_key);
  __Pyx_XDECREF(__pyx_v_subdtype);
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XDECREF(__pyx_v_entry_type);
  __Pyx_XDECREF(__pyx_v_field_counter_semantics);
  __Pyx_XDECREF(__pyx_v_metadata_flags);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":679
 *             free(semanticsEntries)
 * 
 *     def _get_numpy_array_schema(self, uint64_t scalar_schema):             # <<<<<<<<<<<<<<
 *         cached = self._array_cache.get(scalar_schema)
 *         if cached is not None:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_7_get_numpy_array_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_6_get_numpy_array_schema, "SchemaRegistrar._get_numpy_array_schema(self, uint64_t scalar_schema)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_7_get_numpy_array_schema = {"_get_numpy_array_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_7_get_numpy_array_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_6_get_numpy_array_schema};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_7_get_numpy_array_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  uint64_t __pyx_v_scalar_schema;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_numpy_array_schema (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_scalar_schema,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 679, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 679, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_get_numpy_array_schema", 0) < (0)) __PYX_ERR(0, 679, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_get_numpy_array_schema", 1, 1, 1, i); __PYX_ERR(0, 679, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 679, __pyx_L3_error)
    }
    __pyx_v_scalar_schema = __Pyx_PyLong_As_uint64_t(values[0]); if (unlikely((__pyx_v_scalar_schema == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 679, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_numpy_array_schema", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 679, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._get_numpy_array_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_6_get_numpy_array_schema(((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self), __pyx_v_scalar_schema);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_6_get_numpy_array_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, uint64_t __pyx_v_scalar_schema) {
  PyObject *__pyx_v_cached = NULL;
  PyObject *__pyx_v_field_names = NULL;
  nvtxPayloadSchemaEntry_t __pyx_v_size_entry;
  nvtxPayloadSchemaEntry_t __pyx_v_data_entry;
  nvtxPayloadSchemaEntry_t __pyx_v_entries[2];
  nvtxPayloadSchemaAttr_t __pyx_v_schemaAttr;
  uint64_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  nvtxPayloadSchemaEntry_t __pyx_t_4;
  char const *__pyx_t_5;
  nvtxPayloadSchemaEntry_t __pyx_t_6[2];
  uint64_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_numpy_array_schema", 0);

  /* "nvtx/_lib/lib.pyx":680
 * 
 *     def _get_numpy_array_schema(self, uint64_t scalar_schema):
 *         cached = self._array_cache.get(scalar_schema)             # <<<<<<<<<<<<<<
 *         if cached is not None:
 *             return cached
*/
  if (unlikely(__pyx_v_self->_array_cache == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 680, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyLong_From_uint64_t(__pyx_v_scalar_schema); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_array_cache, __pyx_t_1, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_cached = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":681
 *     def _get_numpy_array_schema(self, uint64_t scalar_schema):
 *         cached = self._array_cache.get(scalar_schema)
 *         if cached is not None:             # <<<<<<<<<<<<<<
 *             return cached
 *         field_names = [b'size', b'data']
*/
  __pyx_t_3 = (__pyx_v_cached != Py_None);
  if (__pyx_t_3) {

    /* "nvtx/_lib/lib.pyx":682
 *         cached = self._array_cache.get(scalar_schema)
 *         if cached is not None:
 *             return cached             # <<<<<<<<<<<<<<
 *         field_names = [b'size', b'data']
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_cached);
    __pyx_r = __pyx_v_cached;
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":681
 *     def _get_numpy_array_schema(self, uint64_t scalar_schema):
 *         cached = self._array_cache.get(scalar_schema)
 *         if cached is not None:             # <<<<<<<<<<<<<<
 *             return cached
 *         field_names = [b'size', b'data']
*/
  }

  /* "nvtx/_lib/lib.pyx":683
 *         if cached is not None:
 *             return cached
 *         field_names = [b'size', b'data']             # <<<<<<<<<<<<<<
 * 
 *         size_entry = nvtxPayloadSchemaEntry_t(
*/
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 683, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_n_b_size);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_b_size);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 0, __pyx_mstate_global->__pyx_n_b_size) != (0)) __PYX_ERR(0, 683, __pyx_L1_error);
  __Pyx_INCREF(__pyx_mstate_global->__pyx_n_b_data);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_n_b_data);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_2, 1, __pyx_mstate_global->__pyx_n_b_data) != (0)) __PYX_ERR(0, 683, __pyx_L1_error);
  __pyx_v_field_names = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":686
 * 
 *         size_entry = nvtxPayloadSchemaEntry_t(
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_UNUSED,             # <<<<<<<<<<<<<<
 *             type=NVTX_PAYLOAD_ENTRY_TYPE_UINT64,
 *             name=field_names[0],
*/
  __pyx_t_4.flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED;

  /* "nvtx/_lib/lib.pyx":687
 *         size_entry = nvtxPayloadSchemaEntry_t(
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_UNUSED,
 *             type=NVTX_PAYLOAD_ENTRY_TYPE_UINT64,             # <<<<<<<<<<<<<<
 *             name=field_names[0],
 *             description=NULL,
*/
  __pyx_t_4.type = NVTX_PAYLOAD_ENTRY_TYPE_UINT64;

  /* "nvtx/_lib/lib.pyx":688
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_UNUSED,
 *             type=NVTX_PAYLOAD_ENTRY_TYPE_UINT64,
 *             name=field_names[0],             # <<<<<<<<<<<<<<
 *             description=NULL,
 *             arrayOrUnionDetail=0,
*/
  __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_field_names, 0, long, 1, __Pyx_PyLong_From_long, 1, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_AsString(__pyx_t_2); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 688, __pyx_L1_error)
  __pyx_t_4.name = __pyx_t_5;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":689
 *             type=NVTX_PAYLOAD_ENTRY_TYPE_UINT64,
 *             name=field_names[0],
 *             description=NULL,             # <<<<<<<<<<<<<<
 *             arrayOrUnionDetail=0,
 *             offset=0,
*/
  __pyx_t_4.description = NULL;
  __pyx_t_4.arrayOrUnionDetail = 0;
  __pyx_t_4.offset = 0;

  /* "nvtx/_lib/lib.pyx":692
 *             arrayOrUnionDetail=0,
 *             offset=0,
 *             semantics=NULL,             # <<<<<<<<<<<<<<
 *             reserved=NULL,
 *         )
*/
  __pyx_t_4.semantics = NULL;

  /* "nvtx/_lib/lib.pyx":693
 *             offset=0,
 *             semantics=NULL,
 *             reserved=NULL,             # <<<<<<<<<<<<<<
 *         )
 *         data_entry = nvtxPayloadSchemaEntry_t(
*/
  __pyx_t_4.reserved = NULL;
  __pyx_v_size_entry = __pyx_t_4;

  /* "nvtx/_lib/lib.pyx":696
 *         )
 *         data_entry = nvtxPayloadSchemaEntry_t(
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_LENGTH_INDEX,             # <<<<<<<<<<<<<<
 *             type=scalar_schema,
 *             name=field_names[1],
*/
  __pyx_t_4.flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_LENGTH_INDEX;

  /* "nvtx/_lib/lib.pyx":697
 *         data_entry = nvtxPayloadSchemaEntry_t(
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_LENGTH_INDEX,
 *             type=scalar_schema,             # <<<<<<<<<<<<<<
 *             name=field_names[1],
 *             description=NULL,
*/
  __pyx_t_4.type = __pyx_v_scalar_schema;

  /* "nvtx/_lib/lib.pyx":698
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_LENGTH_INDEX,
 *             type=scalar_schema,
 *             name=field_names[1],             # <<<<<<<<<<<<<<
 *             description=NULL,
 *             arrayOrUnionDetail=0,
*/
  __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_field_names, 1, long, 1, __Pyx_PyLong_From_long, 1, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_AsString(__pyx_t_2); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 698, __pyx_L1_error)
  __pyx_t_4.name = __pyx_t_5;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":699
 *             type=scalar_schema,
 *             name=field_names[1],
 *             description=NULL,             # <<<<<<<<<<<<<<
 *             arrayOrUnionDetail=0,
 *             offset=0,
*/
  __pyx_t_4.description = NULL;
  __pyx_t_4.arrayOrUnionDetail = 0;
  __pyx_t_4.offset = 0;

  /* "nvtx/_lib/lib.pyx":702
 *             arrayOrUnionDetail=0,
 *             offset=0,
 *             semantics=NULL,             # <<<<<<<<<<<<<<
 *             reserved=NULL,
 *         )
*/
  __pyx_t_4.semantics = NULL;

  /* "nvtx/_lib/lib.pyx":703
 *             offset=0,
 *             semantics=NULL,
 *             reserved=NULL,             # <<<<<<<<<<<<<<
 *         )
 *         cdef nvtxPayloadSchemaEntry_t[2] entries = [size_entry, data_entry]
*/
  __pyx_t_4.reserved = NULL;
  __pyx_v_data_entry = __pyx_t_4;

  /* "nvtx/_lib/lib.pyx":705
 *             reserved=NULL,
 *         )
 *         cdef nvtxPayloadSchemaEntry_t[2] entries = [size_entry, data_entry]             # <<<<<<<<<<<<<<
 * 
 *         cdef nvtxPayloadSchemaAttr_t schemaAttr
*/
  __pyx_t_6[0] = __pyx_v_size_entry;
  __pyx_t_6[1] = __pyx_v_data_entry;
  memcpy(&(__pyx_v_entries[0]), __pyx_t_6, sizeof(__pyx_v_entries[0]) * (2));

  /* "nvtx/_lib/lib.pyx":708
 * 
 *         cdef nvtxPayloadSchemaAttr_t schemaAttr
 *         schemaAttr.fieldMask = NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_TYPE | \             # <<<<<<<<<<<<<<
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_DYNAMIC
*/
  __pyx_v_schemaAttr.fieldMask = ((NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_TYPE | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES) | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES);

  /* "nvtx/_lib/lib.pyx":710
 *         schemaAttr.fieldMask = NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_TYPE | \
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_DYNAMIC             # <<<<<<<<<<<<<<
 *         schemaAttr.entries = entries
 *         schemaAttr.numEntries = 2
*/
  __pyx_v_schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_DYNAMIC;

  /* "nvtx/_lib/lib.pyx":711
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_DYNAMIC
 *         schemaAttr.entries = entries             # <<<<<<<<<<<<<<
 *         schemaAttr.numEntries = 2
 *         result = self._do_register(&schemaAttr)
*/
  __pyx_v_schemaAttr.entries = __pyx_v_entries;

  /* "nvtx/_lib/lib.pyx":712
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_DYNAMIC
 *         schemaAttr.entries = entries
 *         schemaAttr.numEntries = 2             # <<<<<<<<<<<<<<
 *         result = self._do_register(&schemaAttr)
 *         self._array_cache[scalar_schema] = result
*/
  __pyx_v_schemaAttr.numEntries = 2;

  /* "nvtx/_lib/lib.pyx":713
 *         schemaAttr.entries = entries
 *         schemaAttr.numEntries = 2
 *         result = self._do_register(&schemaAttr)             # <<<<<<<<<<<<<<
 *         self._array_cache[scalar_schema] = result
 *         return result
*/
  __pyx_t_7 = ((struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self->__pyx_vtab)->_do_register(__pyx_v_self, (&__pyx_v_schemaAttr)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 713, __pyx_L1_error)
  __pyx_v_result = __pyx_t_7;

  /* "nvtx/_lib/lib.pyx":714
 *         schemaAttr.numEntries = 2
 *         result = self._do_register(&schemaAttr)
 *         self._array_cache[scalar_schema] = result             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_From_uint64_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 714, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(__pyx_v_self->_array_cache == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 714, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyLong_From_uint64_t(__pyx_v_scalar_schema); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 714, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely((PyDict_SetItem(__pyx_v_self->_array_cache, __pyx_t_1, __pyx_t_2) < 0))) __PYX_ERR(0, 714, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":715
 *         result = self._do_register(&schemaAttr)
 *         self._array_cache[scalar_schema] = result
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def _get_fixed_size_array_schema(self, uint64_t scalar_schema, size_t array_length, size_t size):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyLong_From_uint64_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":679
 *             free(semanticsEntries)
 * 
 *     def _get_numpy_array_schema(self, uint64_t scalar_schema):             # <<<<<<<<<<<<<<
 *         cached = self._array_cache.get(scalar_schema)
 *         if cached is not None:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._get_numpy_array_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cached);
  __Pyx_XDECREF(__pyx_v_field_names);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":717
 *         return result
 * 
 *     def _get_fixed_size_array_schema(self, uint64_t scalar_schema, size_t array_length, size_t size):             # <<<<<<<<<<<<<<
 *         cache_key = (scalar_schema, array_length, size)
 *         cached = self._fixed_cache.get(cache_key)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_9_get_fixed_size_array_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_8_get_fixed_size_array_schema, "SchemaRegistrar._get_fixed_size_array_schema(self, uint64_t scalar_schema, size_t array_length, size_t size)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_9_get_fixed_size_array_schema = {"_get_fixed_size_array_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_9_get_fixed_size_array_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_8_get_fixed_size_array_schema};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_9_get_fixed_size_array_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  uint64_t __pyx_v_scalar_schema;
  size_t __pyx_v_array_length;
  size_t __pyx_v_size;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_fixed_size_array_schema (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_scalar_schema,&__pyx_mstate_global->__pyx_n_u_array_length,&__pyx_mstate_global->__pyx_n_u_size,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 717, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 717, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 717, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 717, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_get_fixed_size_array_schema", 0) < (0)) __PYX_ERR(0, 717, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_get_fixed_size_array_schema", 1, 3, 3, i); __PYX_ERR(0, 717, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 717, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 717, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 717, __pyx_L3_error)
    }
    __pyx_v_scalar_schema = __Pyx_PyLong_As_uint64_t(values[0]); if (unlikely((__pyx_v_scalar_schema == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 717, __pyx_L3_error)
    __pyx_v_array_length = __Pyx_PyLong_As_size_t(values[1]); if (unlikely((__pyx_v_array_length == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 717, __pyx_L3_error)
    __pyx_v_size = __Pyx_PyLong_As_size_t(values[2]); if (unlikely((__pyx_v_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 717, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_fixed_size_array_schema", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 717, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._get_fixed_size_array_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_8_get_fixed_size_array_schema(((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self), __pyx_v_scalar_schema, __pyx_v_array_length, __pyx_v_size);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_8_get_fixed_size_array_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, uint64_t __pyx_v_scalar_schema, size_t __pyx_v_array_length, size_t __pyx_v_size) {
  PyObject *__pyx_v_cache_key = NULL;
  PyObject *__pyx_v_cached = NULL;
  nvtxPayloadSchemaEntry_t __pyx_v_entry;
  nvtxPayloadSchemaAttr_t __pyx_v_schemaAttr;
  uint64_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  nvtxPayloadSchemaEntry_t __pyx_t_6;
  uint64_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_fixed_size_array_schema", 0);

  /* "nvtx/_lib/lib.pyx":718
 * 
 *     def _get_fixed_size_array_schema(self, uint64_t scalar_schema, size_t array_length, size_t size):
 *         cache_key = (scalar_schema, array_length, size)             # <<<<<<<<<<<<<<
 *         cached = self._fixed_cache.get(cache_key)
 *         if cached is not None:
*/
  __pyx_t_1 = __Pyx_PyLong_From_uint64_t(__pyx_v_scalar_schema); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_v_array_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyLong_FromSize_t(__pyx_v_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1) != (0)) __PYX_ERR(0, 718, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 718, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3) != (0)) __PYX_ERR(0, 718, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_cache_key = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":719
 *     def _get_fixed_size_array_schema(self, uint64_t scalar_schema, size_t array_length, size_t size):
 *         cache_key = (scalar_schema, array_length, size)
 *         cached = self._fixed_cache.get(cache_key)             # <<<<<<<<<<<<<<
 *         if cached is not None:
 *             return cached
*/
  if (unlikely(__pyx_v_self->_fixed_cache == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 719, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_fixed_cache, __pyx_v_cache_key, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_cached = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":720
 *         cache_key = (scalar_schema, array_length, size)
 *         cached = self._fixed_cache.get(cache_key)
 *         if cached is not None:             # <<<<<<<<<<<<<<
 *             return cached
 *         cdef nvtxPayloadSchemaEntry_t entry = nvtxPayloadSchemaEntry_t(
*/
  __pyx_t_5 = (__pyx_v_cached != Py_None);
  if (__pyx_t_5) {

    /* "nvtx/_lib/lib.pyx":721
 *         cached = self._fixed_cache.get(cache_key)
 *         if cached is not None:
 *             return cached             # <<<<<<<<<<<<<<
 *         cdef nvtxPayloadSchemaEntry_t entry = nvtxPayloadSchemaEntry_t(
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE,
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_cached);
    __pyx_r = __pyx_v_cached;
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":720
 *         cache_key = (scalar_schema, array_length, size)
 *         cached = self._fixed_cache.get(cache_key)
 *         if cached is not None:             # <<<<<<<<<<<<<<
 *             return cached
 *         cdef nvtxPayloadSchemaEntry_t entry = nvtxPayloadSchemaEntry_t(
*/
  }

  /* "nvtx/_lib/lib.pyx":723
 *             return cached
 *         cdef nvtxPayloadSchemaEntry_t entry = nvtxPayloadSchemaEntry_t(
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE,             # <<<<<<<<<<<<<<
 *             type=scalar_schema,
 *             name=NULL,
*/
  __pyx_t_6.flags = NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE;

  /* "nvtx/_lib/lib.pyx":724
 *         cdef nvtxPayloadSchemaEntry_t entry = nvtxPayloadSchemaEntry_t(
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE,
 *             type=scalar_schema,             # <<<<<<<<<<<<<<
 *             name=NULL,
 *             description=NULL,
*/
  __pyx_t_6.type = __pyx_v_scalar_schema;

  /* "nvtx/_lib/lib.pyx":725
 *             flags=NVTX_PAYLOAD_ENTRY_FLAG_ARRAY_FIXED_SIZE,
 *             type=scalar_schema,
 *             name=NULL,             # <<<<<<<<<<<<<<
 *             description=NULL,
 *             arrayOrUnionDetail=array_length,
*/
  __pyx_t_6.name = NULL;

  /* "nvtx/_lib/lib.pyx":726
 *             type=scalar_schema,
 *             name=NULL,
 *             description=NULL,             # <<<<<<<<<<<<<<
 *             arrayOrUnionDetail=array_length,
 *             offset=0,
*/
  __pyx_t_6.description = NULL;

  /* "nvtx/_lib/lib.pyx":727
 *             name=NULL,
 *             description=NULL,
 *             arrayOrUnionDetail=array_length,             # <<<<<<<<<<<<<<
 *             offset=0,
 *             semantics=NULL,
*/
  __pyx_t_6.arrayOrUnionDetail = __pyx_v_array_length;
  __pyx_t_6.offset = 0;

  /* "nvtx/_lib/lib.pyx":729
 *             arrayOrUnionDetail=array_length,
 *             offset=0,
 *             semantics=NULL,             # <<<<<<<<<<<<<<
 *             reserved=NULL,
 *         )
*/
  __pyx_t_6.semantics = NULL;

  /* "nvtx/_lib/lib.pyx":730
 *             offset=0,
 *             semantics=NULL,
 *             reserved=NULL,             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_6.reserved = NULL;
  __pyx_v_entry = __pyx_t_6;

  /* "nvtx/_lib/lib.pyx":734
 * 
 *         cdef nvtxPayloadSchemaAttr_t schemaAttr
 *         schemaAttr.fieldMask = NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_TYPE | \             # <<<<<<<<<<<<<<
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES | \
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE
*/
  __pyx_v_schemaAttr.fieldMask = (((NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_TYPE | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES) | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES) | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE);

  /* "nvtx/_lib/lib.pyx":737
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_ENTRIES | NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_NUM_ENTRIES | \
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC             # <<<<<<<<<<<<<<
 *         schemaAttr.payloadStaticSize = size
 *         schemaAttr.entries = &entry
*/
  __pyx_v_schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC;

  /* "nvtx/_lib/lib.pyx":738
 *             NVTX_PAYLOAD_SCHEMA_ATTR_FIELD_STATIC_SIZE
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC
 *         schemaAttr.payloadStaticSize = size             # <<<<<<<<<<<<<<
 *         schemaAttr.entries = &entry
 *         schemaAttr.numEntries = 1
*/
  __pyx_v_schemaAttr.payloadStaticSize = __pyx_v_size;

  /* "nvtx/_lib/lib.pyx":739
 *         schemaAttr.type = NVTX_PAYLOAD_SCHEMA_TYPE_STATIC
 *         schemaAttr.payloadStaticSize = size
 *         schemaAttr.entries = &entry             # <<<<<<<<<<<<<<
 *         schemaAttr.numEntries = 1
 *         result = self._do_register(&schemaAttr)
*/
  __pyx_v_schemaAttr.entries = (&__pyx_v_entry);

  /* "nvtx/_lib/lib.pyx":740
 *         schemaAttr.payloadStaticSize = size
 *         schemaAttr.entries = &entry
 *         schemaAttr.numEntries = 1             # <<<<<<<<<<<<<<
 *         result = self._do_register(&schemaAttr)
 *         self._fixed_cache[cache_key] = result
*/
  __pyx_v_schemaAttr.numEntries = 1;

  /* "nvtx/_lib/lib.pyx":741
 *         schemaAttr.entries = &entry
 *         schemaAttr.numEntries = 1
 *         result = self._do_register(&schemaAttr)             # <<<<<<<<<<<<<<
 *         self._fixed_cache[cache_key] = result
 *         return result
*/
  __pyx_t_7 = ((struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self->__pyx_vtab)->_do_register(__pyx_v_self, (&__pyx_v_schemaAttr)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 741, __pyx_L1_error)
  __pyx_v_result = __pyx_t_7;

  /* "nvtx/_lib/lib.pyx":742
 *         schemaAttr.numEntries = 1
 *         result = self._do_register(&schemaAttr)
 *         self._fixed_cache[cache_key] = result             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
  __pyx_t_4 = __Pyx_PyLong_From_uint64_t(__pyx_v_result); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (unlikely(__pyx_v_self->_fixed_cache == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 742, __pyx_L1_error)
  }
  if (unlikely((PyDict_SetItem(__pyx_v_self->_fixed_cache, __pyx_v_cache_key, __pyx_t_4) < 0))) __PYX_ERR(0, 742, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":743
 *         result = self._do_register(&schemaAttr)
 *         self._fixed_cache[cache_key] = result
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def _get_numpy_dtype_schema(self, schema_key):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyLong_From_uint64_t(__pyx_v_result); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":717
 *         return result
 * 
 *     def _get_fixed_size_array_schema(self, uint64_t scalar_schema, size_t array_length, size_t size):             # <<<<<<<<<<<<<<
 *         cache_key = (scalar_schema, array_length, size)
 *         cached = self._fixed_cache.get(cache_key)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._get_fixed_size_array_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cache_key);
  __Pyx_XDECREF(__pyx_v_cached);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":745
 *         return result
 * 
 *     def _get_numpy_dtype_schema(self, schema_key):             # <<<<<<<<<<<<<<
 *         cached = self._dtype_cache.get(schema_key)
 *         if cached is not None:
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_11_get_numpy_dtype_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_10_get_numpy_dtype_schema, "SchemaRegistrar._get_numpy_dtype_schema(self, schema_key)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_11_get_numpy_dtype_schema = {"_get_numpy_dtype_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_11_get_numpy_dtype_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_10_get_numpy_dtype_schema};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_11_get_numpy_dtype_schema(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_schema_key = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_numpy_dtype_schema (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_schema_key,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 745, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 745, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_get_numpy_dtype_schema", 0) < (0)) __PYX_ERR(0, 745, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_get_numpy_dtype_schema", 1, 1, 1, i); __PYX_ERR(0, 745, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 745, __pyx_L3_error)
    }
    __pyx_v_schema_key = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_numpy_dtype_schema", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 745, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._get_numpy_dtype_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_10_get_numpy_dtype_schema(((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self), __pyx_v_schema_key);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_10_get_numpy_dtype_schema(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, PyObject *__pyx_v_schema_key) {
  PyObject *__pyx_v_cached = NULL;
  PyObject *__pyx_v_dt = NULL;
  PyObject *__pyx_v_subdtype = NULL;
  PyObject *__pyx_v_subdtype_schema = NULL;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_numpy_dtype_schema", 0);

  /* "nvtx/_lib/lib.pyx":746
 * 
 *     def _get_numpy_dtype_schema(self, schema_key):
 *         cached = self._dtype_cache.get(schema_key)             # <<<<<<<<<<<<<<
 *         if cached is not None:
 *             return cached
*/
  if (unlikely(__pyx_v_self->_dtype_cache == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 746, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_dtype_cache, __pyx_v_schema_key, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_cached = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":747
 *     def _get_numpy_dtype_schema(self, schema_key):
 *         cached = self._dtype_cache.get(schema_key)
 *         if cached is not None:             # <<<<<<<<<<<<<<
 *             return cached
 *         dt = schema_key.dtype
*/
  __pyx_t_2 = (__pyx_v_cached != Py_None);
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":748
 *         cached = self._dtype_cache.get(schema_key)
 *         if cached is not None:
 *             return cached             # <<<<<<<<<<<<<<
 *         dt = schema_key.dtype
 *         if dt.subdtype:
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_cached);
    __pyx_r = __pyx_v_cached;
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":747
 *     def _get_numpy_dtype_schema(self, schema_key):
 *         cached = self._dtype_cache.get(schema_key)
 *         if cached is not None:             # <<<<<<<<<<<<<<
 *             return cached
 *         dt = schema_key.dtype
*/
  }

  /* "nvtx/_lib/lib.pyx":749
 *         if cached is not None:
 *             return cached
 *         dt = schema_key.dtype             # <<<<<<<<<<<<<<
 *         if dt.subdtype:
 *             subdtype = dt.subdtype[0]
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_schema_key, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 749, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_dt = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":750
 *             return cached
 *         dt = schema_key.dtype
 *         if dt.subdtype:             # <<<<<<<<<<<<<<
 *             subdtype = dt.subdtype[0]
 *             subdtype_schema = self._get_numpy_dtype_schema(
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 750, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 750, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":751
 *         dt = schema_key.dtype
 *         if dt.subdtype:
 *             subdtype = dt.subdtype[0]             # <<<<<<<<<<<<<<
 *             subdtype_schema = self._get_numpy_dtype_schema(
 *                 PayloadSchemaKey(subdtype)
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_subdtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 751, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 751, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_subdtype = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "nvtx/_lib/lib.pyx":752
 *         if dt.subdtype:
 *             subdtype = dt.subdtype[0]
 *             subdtype_schema = self._get_numpy_dtype_schema(             # <<<<<<<<<<<<<<
 *                 PayloadSchemaKey(subdtype)
 *             )
*/
    __pyx_t_1 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_1);

    /* "nvtx/_lib/lib.pyx":753
 *             subdtype = dt.subdtype[0]
 *             subdtype_schema = self._get_numpy_dtype_schema(
 *                 PayloadSchemaKey(subdtype)             # <<<<<<<<<<<<<<
 *             )
 *             result = self._get_fixed_size_array_schema(
*/
    __pyx_t_5 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_PayloadSchemaKey); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 753, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_v_subdtype};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 753, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_t_4};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_v_subdtype_schema = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "nvtx/_lib/lib.pyx":755
 *                 PayloadSchemaKey(subdtype)
 *             )
 *             result = self._get_fixed_size_array_schema(             # <<<<<<<<<<<<<<
 *                 subdtype_schema, np.prod(dt.shape), dt.itemsize
 *             )
*/
    __pyx_t_4 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_4);

    /* "nvtx/_lib/lib.pyx":756
 *             )
 *             result = self._get_fixed_size_array_schema(
 *                 subdtype_schema, np.prod(dt.shape), dt.itemsize             # <<<<<<<<<<<<<<
 *             )
 *         elif dt.type in _dtype_to_entry_type:
*/
    __pyx_t_6 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 756, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_prod); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 756, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_shape); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 756, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_8);
      assert(__pyx_t_6);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 756, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_itemsize); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 756, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_v_subdtype_schema, __pyx_t_1, __pyx_t_8};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_fixed_size_array_schema, __pyx_callargs+__pyx_t_7, (4-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 755, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __pyx_v_result = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "nvtx/_lib/lib.pyx":750
 *             return cached
 *         dt = schema_key.dtype
 *         if dt.subdtype:             # <<<<<<<<<<<<<<
 *             subdtype = dt.subdtype[0]
 *             subdtype_schema = self._get_numpy_dtype_schema(
*/
    goto __pyx_L4;
  }

  /* "nvtx/_lib/lib.pyx":758
 *                 subdtype_schema, np.prod(dt.shape), dt.itemsize
 *             )
 *         elif dt.type in _dtype_to_entry_type:             # <<<<<<<<<<<<<<
 *             result = self._register_builtin_schema(dt, schema_key.metadata)
 *         else:
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dt, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_dtype_to_entry_type); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_3, __pyx_t_8, Py_EQ)); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":759
 *             )
 *         elif dt.type in _dtype_to_entry_type:
 *             result = self._register_builtin_schema(dt, schema_key.metadata)             # <<<<<<<<<<<<<<
 *         else:
 *             result = self._register_structured_schema(schema_key)
*/
    __pyx_t_3 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_schema_key, __pyx_mstate_global->__pyx_n_u_metadata); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 759, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_dt, __pyx_t_1};
      __pyx_t_8 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_register_builtin_schema, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 759, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
    }
    __pyx_v_result = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "nvtx/_lib/lib.pyx":758
 *                 subdtype_schema, np.prod(dt.shape), dt.itemsize
 *             )
 *         elif dt.type in _dtype_to_entry_type:             # <<<<<<<<<<<<<<
 *             result = self._register_builtin_schema(dt, schema_key.metadata)
 *         else:
*/
    goto __pyx_L4;
  }

  /* "nvtx/_lib/lib.pyx":761
 *             result = self._register_builtin_schema(dt, schema_key.metadata)
 *         else:
 *             result = self._register_structured_schema(schema_key)             # <<<<<<<<<<<<<<
 *         self._dtype_cache[schema_key] = result
 *         return result
*/
  /*else*/ {
    __pyx_t_1 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_7 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_schema_key};
      __pyx_t_8 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_register_structured_schema, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 761, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
    }
    __pyx_v_result = __pyx_t_8;
    __pyx_t_8 = 0;
  }
  __pyx_L4:;

  /* "nvtx/_lib/lib.pyx":762
 *         else:
 *             result = self._register_structured_schema(schema_key)
 *         self._dtype_cache[schema_key] = result             # <<<<<<<<<<<<<<
 *         return result
 * 
*/
  if (unlikely(__pyx_v_self->_dtype_cache == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 762, __pyx_L1_error)
  }
  if (unlikely((PyDict_SetItem(__pyx_v_self->_dtype_cache, __pyx_v_schema_key, __pyx_v_result) < 0))) __PYX_ERR(0, 762, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":763
 *             result = self._register_structured_schema(schema_key)
 *         self._dtype_cache[schema_key] = result
 *         return result             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":745
 *         return result
 * 
 *     def _get_numpy_dtype_schema(self, schema_key):             # <<<<<<<<<<<<<<
 *         cached = self._dtype_cache.get(schema_key)
 *         if cached is not None:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar._get_numpy_dtype_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cached);
  __Pyx_XDECREF(__pyx_v_dt);
  __Pyx_XDECREF(__pyx_v_subdtype);
  __Pyx_XDECREF(__pyx_v_subdtype_schema);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_13__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_12__reduce_cython__, "SchemaRegistrar.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_13__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_13__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_12__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_13__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_12__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_15__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_14__setstate_cython__, "SchemaRegistrar.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_15__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_15__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_14__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_15__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_14__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_15SchemaRegistrar_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.SchemaRegistrar.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":771
 *     cdef nvtxDomainHandle_t _domain
 * 
 *     def __init__(self, DomainHandle handle):             # <<<<<<<<<<<<<<
 *         self._domain = handle.c_obj
 * 
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_handle = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_handle,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 771, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 771, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 771, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, i); __PYX_ERR(0, 771, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 771, __pyx_L3_error)
    }
    __pyx_v_handle = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 771, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib._GlobalSchemaRegistrar.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_handle), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle, 1, "handle", 0))) __PYX_ERR(0, 771, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar___init__(((struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *)__pyx_v_self), __pyx_v_handle);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar___init__(struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_handle) {
  int __pyx_r;
  nvtxDomainHandle_t __pyx_t_1;

  /* "nvtx/_lib/lib.pyx":772
 * 
 *     def __init__(self, DomainHandle handle):
 *         self._domain = handle.c_obj             # <<<<<<<<<<<<<<
 * 
 *     cdef uint64_t _do_register(
*/
  __pyx_t_1 = __pyx_v_handle->c_obj;
  __pyx_v_self->_domain = __pyx_t_1;

  /* "nvtx/_lib/lib.pyx":771
 *     cdef nvtxDomainHandle_t _domain
 * 
 *     def __init__(self, DomainHandle handle):             # <<<<<<<<<<<<<<
 *         self._domain = handle.c_obj
 * 
*/

  /* function exit code */
  __pyx_r = 0;
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":774
 *         self._domain = handle.c_obj
 * 
 *     cdef uint64_t _do_register(             # <<<<<<<<<<<<<<
 *         self, const nvtxPayloadSchemaAttr_t* attr
 *     ) except *:
*/

static uint64_t __pyx_f_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar__do_register(struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *__pyx_v_self, nvtxPayloadSchemaAttr_t const *__pyx_v_attr) {
  uint64_t __pyx_r;

  /* "nvtx/_lib/lib.pyx":777
 *         self, const nvtxPayloadSchemaAttr_t* attr
 *     ) except *:
 *         return nvtxPayloadSchemaRegister(self._domain, attr)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_r = nvtxPayloadSchemaRegister(__pyx_v_self->_domain, __pyx_v_attr);
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":774
 *         self._domain = handle.c_obj
 * 
 *     cdef uint64_t _do_register(             # <<<<<<<<<<<<<<
 *         self, const nvtxPayloadSchemaAttr_t* attr
 *     ) except *:
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_2__reduce_cython__, "_GlobalSchemaRegistrar.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_3__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_2__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_2__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib._GlobalSchemaRegistrar.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_4__setstate_cython__, "_GlobalSchemaRegistrar.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_5__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_4__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib._GlobalSchemaRegistrar.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_4__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_no_default___reduce___due_to_non, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib._GlobalSchemaRegistrar.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":792
 *     """
 * 
 *     def __new__(cls, name: Optional[str] = None):             # <<<<<<<<<<<<<<
 *         handle = DomainHandle(name)
 *         if handle.enabled():
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_1__new__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain___new__, "Domain.__new__(cls, str name: Optional[str] = None)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_1__new__ = {"__new__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_1__new__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain___new__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_1__new__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_cls = 0;
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__new__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_cls,&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 792, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 792, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 792, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__new__", 0) < (0)) __PYX_ERR(0, 792, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__new__", 0, 1, 2, i); __PYX_ERR(0, 792, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 792, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 792, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)Py_None));
    }
    __pyx_v_cls = values[0];
    __pyx_v_name = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__new__", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 792, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.__new__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 2))) __PYX_ERR(0, 792, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain___new__(__pyx_self, __pyx_v_cls, __pyx_v_name);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain___new__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cls, PyObject *__pyx_v_name) {
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_handle = NULL;
  PyObject *__pyx_v_obj = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__new__", 0);

  /* "nvtx/_lib/lib.pyx":793
 * 
 *     def __new__(cls, name: Optional[str] = None):
 *         handle = DomainHandle(name)             # <<<<<<<<<<<<<<
 *         if handle.enabled():
 *             obj = super().__new__(cls)
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_name};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 793, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_v_handle = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":794
 *     def __new__(cls, name: Optional[str] = None):
 *         handle = DomainHandle(name)
 *         if handle.enabled():             # <<<<<<<<<<<<<<
 *             obj = super().__new__(cls)
 *             obj.handle = handle
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_handle);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_enabled, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 794, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 794, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_4) {

    /* "nvtx/_lib/lib.pyx":795
 *         handle = DomainHandle(name)
 *         if handle.enabled():
 *             obj = super().__new__(cls)             # <<<<<<<<<<<<<<
 *             obj.handle = handle
 *             return obj
*/
    __pyx_t_6 = NULL;
    __pyx_t_7 = __Pyx_CyFunction_GetClassObj(__pyx_self);
    if (!__pyx_t_7) { PyErr_SetString(PyExc_RuntimeError, "super(): empty __class__ cell"); __PYX_ERR(0, 795, __pyx_L1_error) }
    __Pyx_INCREF(__pyx_t_7);
    __pyx_t_3 = 1;
    {
      PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_t_7, __pyx_v_cls};
      __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_super, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 795, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __pyx_t_2 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_cls};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 795, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_v_obj = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "nvtx/_lib/lib.pyx":796
 *         if handle.enabled():
 *             obj = super().__new__(cls)
 *             obj.handle = handle             # <<<<<<<<<<<<<<
 *             return obj
 *         else:
*/
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_obj, __pyx_mstate_global->__pyx_n_u_handle, ((PyObject *)__pyx_v_handle)) < (0)) __PYX_ERR(0, 796, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":797
 *             obj = super().__new__(cls)
 *             obj.handle = handle
 *             return obj             # <<<<<<<<<<<<<<
 *         else:
 *             return dummy_domain
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_obj);
    __pyx_r = __pyx_v_obj;
    goto __pyx_L0;

    /* "nvtx/_lib/lib.pyx":794
 *     def __new__(cls, name: Optional[str] = None):
 *         handle = DomainHandle(name)
 *         if handle.enabled():             # <<<<<<<<<<<<<<
 *             obj = super().__new__(cls)
 *             obj.handle = handle
*/
  }

  /* "nvtx/_lib/lib.pyx":799
 *             return obj
 *         else:
 *             return dummy_domain             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, name: Optional[str] = None):
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_dummy_domain); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "nvtx/_lib/lib.pyx":792
 *     """
 * 
 *     def __new__(cls, name: Optional[str] = None):             # <<<<<<<<<<<<<<
 *         handle = DomainHandle(name)
 *         if handle.enabled():
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.__new__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_handle);
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":801
 *             return dummy_domain
 * 
 *     def __init__(self, name: Optional[str] = None):             # <<<<<<<<<<<<<<
 *         self.name = name
 *         self.categories = {}
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_3__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_2__init__, "Domain.__init__(self, str name: Optional[str] = None)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_3__init__ = {"__init__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_3__init__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_2__init__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_3__init__(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 801, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 801, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 801, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 801, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, i); __PYX_ERR(0, 801, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 801, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 801, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_name = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 801, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 2))) __PYX_ERR(0, 801, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_2__init__(__pyx_self, __pyx_v_self, __pyx_v_name);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_2__init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "nvtx/_lib/lib.pyx":802
 * 
 *     def __init__(self, name: Optional[str] = None):
 *         self.name = name             # <<<<<<<<<<<<<<
 *         self.categories = {}
 *         self._schema_registrar = _GlobalSchemaRegistrar(self.handle)
*/
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_name, __pyx_v_name) < (0)) __PYX_ERR(0, 802, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":803
 *     def __init__(self, name: Optional[str] = None):
 *         self.name = name
 *         self.categories = {}             # <<<<<<<<<<<<<<
 *         self._schema_registrar = _GlobalSchemaRegistrar(self.handle)
 * 
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_categories, __pyx_t_1) < (0)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":804
 *         self.name = name
 *         self.categories = {}
 *         self._schema_registrar = _GlobalSchemaRegistrar(self.handle)             # <<<<<<<<<<<<<<
 * 
 *     @lru_cache(maxsize=None)
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_schema_registrar, ((PyObject *)__pyx_t_1)) < (0)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_DECREF((PyObject *)__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":801
 *             return dummy_domain
 * 
 *     def __init__(self, name: Optional[str] = None):             # <<<<<<<<<<<<<<
 *         self.name = name
 *         self.categories = {}
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":806
 *         self._schema_registrar = _GlobalSchemaRegistrar(self.handle)
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def get_registered_string(self, string) -> RegisteredString:
 *         """
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_5get_registered_string(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_4get_registered_string, "Domain.get_registered_string(self, string) -> RegisteredString\n\nRegister a given string under this domain (on first use), and return the handle.\n\nParameters\n----------\nstring : str\n    The string to be registered.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_5get_registered_string = {"get_registered_string", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_5get_registered_string, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_4get_registered_string};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_5get_registered_string(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_string = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_registered_string (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 806, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 806, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 806, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_registered_string", 0) < (0)) __PYX_ERR(0, 806, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_registered_string", 1, 2, 2, i); __PYX_ERR(0, 806, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 806, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 806, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_string = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_registered_string", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 806, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_registered_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_4get_registered_string(__pyx_self, __pyx_v_self, __pyx_v_string);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_4get_registered_string(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_string) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_registered_string", 0);

  /* "nvtx/_lib/lib.pyx":816
 *             The string to be registered.
 *         """
 *         return RegisteredString(self.handle, string)             # <<<<<<<<<<<<<<
 * 
 *     @lru_cache(maxsize=None)
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_RegisteredString); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 816, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 816, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_4, __pyx_v_string};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 816, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":806
 *         self._schema_registrar = _GlobalSchemaRegistrar(self.handle)
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def get_registered_string(self, string) -> RegisteredString:
 *         """
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_registered_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":818
 *         return RegisteredString(self.handle, string)
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def get_category_id(self, name) -> int:
 *         """
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_7get_category_id(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_6get_category_id, "Domain.get_category_id(self, name) -> int\n\nReturns the category ID corresponding to the category `name`.\nOn first use with a specific `name`, a new ID is assigned with the given name.\n\nParameters\n----------\nname : str\n    The name of the category.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_7get_category_id = {"get_category_id", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_7get_category_id, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_6get_category_id};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_7get_category_id(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_category_id (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 818, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 818, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 818, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_category_id", 0) < (0)) __PYX_ERR(0, 818, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_category_id", 1, 2, 2, i); __PYX_ERR(0, 818, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 818, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 818, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_name = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_category_id", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 818, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_category_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_6get_category_id(__pyx_self, __pyx_v_self, __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_6get_category_id(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name) {
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_dh = 0;
  Py_ssize_t __pyx_v_category_id;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char const *__pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_category_id", 0);

  /* "nvtx/_lib/lib.pyx":829
 *             The name of the category.
 *         """
 *         cdef DomainHandle dh = self.handle             # <<<<<<<<<<<<<<
 *         category_id = len(self.categories) + 1
 *         self.categories[name] = category_id
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 829, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle))))) __PYX_ERR(0, 829, __pyx_L1_error)
  __pyx_v_dh = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":830
 *         """
 *         cdef DomainHandle dh = self.handle
 *         category_id = len(self.categories) + 1             # <<<<<<<<<<<<<<
 *         self.categories[name] = category_id
 *         nvtxDomainNameCategoryA(
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_categories); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 830, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 830, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_category_id = (__pyx_t_2 + 1);

  /* "nvtx/_lib/lib.pyx":831
 *         cdef DomainHandle dh = self.handle
 *         category_id = len(self.categories) + 1
 *         self.categories[name] = category_id             # <<<<<<<<<<<<<<
 *         nvtxDomainNameCategoryA(
 *             dh.c_obj,
*/
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_category_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_categories); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely((PyObject_SetItem(__pyx_t_3, __pyx_v_name, __pyx_t_1) < 0))) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":835
 *             dh.c_obj,
 *             category_id,
 *             _to_bytes(name)             # <<<<<<<<<<<<<<
 *         )
 *         return category_id
*/
  __pyx_t_1 = __pyx_f_4nvtx_4_lib_3lib__to_bytes(__pyx_v_name, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 835, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 835, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_AsString(__pyx_t_1); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) __PYX_ERR(0, 835, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":832
 *         category_id = len(self.categories) + 1
 *         self.categories[name] = category_id
 *         nvtxDomainNameCategoryA(             # <<<<<<<<<<<<<<
 *             dh.c_obj,
 *             category_id,
*/
  nvtxDomainNameCategoryA(__pyx_v_dh->c_obj, __pyx_v_category_id, __pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":837
 *             _to_bytes(name)
 *         )
 *         return category_id             # <<<<<<<<<<<<<<
 * 
 *     def get_event_attributes(self, message=None, color=None, category=None, payload=None
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyLong_FromSsize_t(__pyx_v_category_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_PyInt_FromNumber(&__pyx_t_1, NULL, 0) < (0)) __PYX_ERR(0, 837, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":818
 *         return RegisteredString(self.handle, string)
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def get_category_id(self, name) -> int:
 *         """
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_category_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_dh);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":839
 *         return category_id
 * 
 *     def get_event_attributes(self, message=None, color=None, category=None, payload=None             # <<<<<<<<<<<<<<
 *             )-> EventAttributes:
 *         """
*/

/* Python wrapper */
static struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_pw_4nvtx_4_lib_3lib_6Domain_9get_event_attributes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_8get_event_attributes, "Domain.get_event_attributes(self, message=None, color=None, category=None, payload=None) -> EventAttributes\n\nCreate an :class:`nvtx._lib.lib.EventAttributes` object.\n\nParameters\n----------\nmessage : str\n    A message associated with the event.\n    If the given message was not registered then it will be registered under this domain.\ncolor : str, int, optional\n    A color associated with the event.\n    Supports `matplotlib` colors if it is available.\ncategory : str, int, optional\n    A string or an integer specifying the category within the domain under which the event\n    is scoped. If unspecified, the event is not associated with a category.\npayload : int, float, list, tuple, range, bytes, numpy.ndarray, optional\n    A value associated with the event.\n    Using payloads provides a separation between the message and the data of the event,\n    which is often useful for analysis.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_9get_event_attributes = {"get_event_attributes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_9get_event_attributes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_8get_event_attributes};
static struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_pw_4nvtx_4_lib_3lib_6Domain_9get_event_attributes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_message = 0;
  PyObject *__pyx_v_color = 0;
  PyObject *__pyx_v_category = 0;
  PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_event_attributes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,&__pyx_mstate_global->__pyx_n_u_payload,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 839, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 839, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 839, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 839, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 839, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 839, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_event_attributes", 0) < (0)) __PYX_ERR(0, 839, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_event_attributes", 0, 1, 5, i); __PYX_ERR(0, 839, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 839, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 839, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 839, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 839, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 839, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_message = values[1];
    __pyx_v_color = values[2];
    __pyx_v_category = values[3];
    __pyx_v_payload = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_event_attributes", 0, 1, 5, __pyx_nargs); __PYX_ERR(0, 839, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_event_attributes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_8get_event_attributes(__pyx_self, __pyx_v_self, __pyx_v_message, __pyx_v_color, __pyx_v_category, __pyx_v_payload);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_pf_4nvtx_4_lib_3lib_6Domain_8get_event_attributes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, PyObject *__pyx_v_payload) {
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_event_attributes", 0);
  __Pyx_INCREF(__pyx_v_message);
  __Pyx_INCREF(__pyx_v_category);

  /* "nvtx/_lib/lib.pyx":860
 *             which is often useful for analysis.
 *         """
 *         if isinstance(category, str):             # <<<<<<<<<<<<<<
 *             category = self.get_category_id(category)
 *         if message is not None:
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_category); 
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":861
 *         """
 *         if isinstance(category, str):
 *             category = self.get_category_id(category)             # <<<<<<<<<<<<<<
 *         if message is not None:
 *             message = self.get_registered_string(message)
*/
    __pyx_t_3 = __pyx_v_self;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_category};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_category_id, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 861, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF_SET(__pyx_v_category, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "nvtx/_lib/lib.pyx":860
 *             which is often useful for analysis.
 *         """
 *         if isinstance(category, str):             # <<<<<<<<<<<<<<
 *             category = self.get_category_id(category)
 *         if message is not None:
*/
  }

  /* "nvtx/_lib/lib.pyx":862
 *         if isinstance(category, str):
 *             category = self.get_category_id(category)
 *         if message is not None:             # <<<<<<<<<<<<<<
 *             message = self.get_registered_string(message)
 *         return EventAttributes(self, message, color, category, payload)
*/
  __pyx_t_1 = (__pyx_v_message != Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":863
 *             category = self.get_category_id(category)
 *         if message is not None:
 *             message = self.get_registered_string(message)             # <<<<<<<<<<<<<<
 *         return EventAttributes(self, message, color, category, payload)
 * 
*/
    __pyx_t_3 = __pyx_v_self;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_message};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_registered_string, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 863, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF_SET(__pyx_v_message, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "nvtx/_lib/lib.pyx":862
 *         if isinstance(category, str):
 *             category = self.get_category_id(category)
 *         if message is not None:             # <<<<<<<<<<<<<<
 *             message = self.get_registered_string(message)
 *         return EventAttributes(self, message, color, category, payload)
*/
  }

  /* "nvtx/_lib/lib.pyx":864
 *         if message is not None:
 *             message = self.get_registered_string(message)
 *         return EventAttributes(self, message, color, category, payload)             # <<<<<<<<<<<<<<
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,
*/
  __Pyx_XDECREF((PyObject *)__pyx_r);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[6] = {__pyx_t_3, __pyx_v_self, __pyx_v_message, __pyx_v_color, __pyx_v_category, __pyx_v_payload};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_callargs+__pyx_t_4, (6-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 864, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_2);
  }
  __pyx_r = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":839
 *         return category_id
 * 
 *     def get_event_attributes(self, message=None, color=None, category=None, payload=None             # <<<<<<<<<<<<<<
 *             )-> EventAttributes:
 *         """
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_event_attributes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_message);
  __Pyx_XDECREF(__pyx_v_category);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":866
 *         return EventAttributes(self, message, color, category, payload)
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,             # <<<<<<<<<<<<<<
 *                              message=DONT_SET, color=DONT_SET, category=DONT_SET, payload=DONT_SET):
 *         """
*/

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_28__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_message, __Pyx_CyFunction_Defaults(struct __pyx_defaults1, __pyx_self)->arg0) < (0)) __PYX_ERR(0, 866, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_color, __Pyx_CyFunction_Defaults(struct __pyx_defaults1, __pyx_self)->arg1) < (0)) __PYX_ERR(0, 866, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_category, __Pyx_CyFunction_Defaults(struct __pyx_defaults1, __pyx_self)->arg2) < (0)) __PYX_ERR(0, 866, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_payload, __Pyx_CyFunction_Defaults(struct __pyx_defaults1, __pyx_self)->arg3) < (0)) __PYX_ERR(0, 866, __pyx_L1_error)
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, Py_None) != (0)) __PYX_ERR(0, 866, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1) != (0)) __PYX_ERR(0, 866, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_11set_event_attributes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_10set_event_attributes, "Domain.set_event_attributes(self, EventAttributes attributes, *, message=DONT_SET, color=DONT_SET, category=DONT_SET, payload=DONT_SET)\n\nSet the attributes of an :class:`nvtx._lib.lib.EventAttributes` object.\n\nParameters\n----------\nattributes : EventAttributes, optional\n    The event attributes to be set.\nmessage : str, RegisteredString, optional\n    A message associated with the event.\n    If the given message was not registered then it will be registered under this domain.\ncolor : str, int, optional\n    A color associated with the event.\n    Supports `matplotlib` colors if it is available.\ncategory : str, int, optional\n    A string or an integer specifying the category within the domain under which the event\n    is scoped. If unspecified, the event is not associated with a category.\npayload : int, float, list, tuple, range, bytes, numpy.ndarray, optional\n    A value associated with the event.\n    Using payloads provides a separation between the message and the data of the event,\n    which is often useful for analysis.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_11set_event_attributes = {"set_event_attributes", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_11set_event_attributes, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_10set_event_attributes};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_11set_event_attributes(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  PyObject *__pyx_v_message = 0;
  PyObject *__pyx_v_color = 0;
  PyObject *__pyx_v_category = 0;
  PyObject *__pyx_v_payload = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[6] = {0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_event_attributes (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_attributes,&__pyx_mstate_global->__pyx_n_u_message,&__pyx_mstate_global->__pyx_n_u_color,&__pyx_mstate_global->__pyx_n_u_category,&__pyx_mstate_global->__pyx_n_u_payload,0};
    struct __pyx_defaults1 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(struct __pyx_defaults1, __pyx_self);
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 866, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 866, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 866, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "set_event_attributes", 0) < (0)) __PYX_ERR(0, 866, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(__pyx_dynamic_args->arg0);
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_dynamic_args->arg1);
      if (!values[4]) values[4] = __Pyx_NewRef(__pyx_dynamic_args->arg2);
      if (!values[5]) values[5] = __Pyx_NewRef(__pyx_dynamic_args->arg3);
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("set_event_attributes", 1, 2, 2, i); __PYX_ERR(0, 866, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 866, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 866, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(__pyx_dynamic_args->arg0);
      if (!values[3]) values[3] = __Pyx_NewRef(__pyx_dynamic_args->arg1);
      if (!values[4]) values[4] = __Pyx_NewRef(__pyx_dynamic_args->arg2);
      if (!values[5]) values[5] = __Pyx_NewRef(__pyx_dynamic_args->arg3);
    }
    __pyx_v_self = values[0];
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[1]);
    __pyx_v_message = values[2];
    __pyx_v_color = values[3];
    __pyx_v_category = values[4];
    __pyx_v_payload = values[5];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_event_attributes", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 866, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.set_event_attributes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 866, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_10set_event_attributes(__pyx_self, __pyx_v_self, __pyx_v_attributes, __pyx_v_message, __pyx_v_color, __pyx_v_category, __pyx_v_payload);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_10set_event_attributes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, PyObject *__pyx_v_message, PyObject *__pyx_v_color, PyObject *__pyx_v_category, PyObject *__pyx_v_payload) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_event_attributes", 0);
  __Pyx_INCREF(__pyx_v_message);
  __Pyx_INCREF(__pyx_v_category);

  /* "nvtx/_lib/lib.pyx":889
 *             which is often useful for analysis.
 *         """
 *         if message is not DONT_SET:             # <<<<<<<<<<<<<<
 *             if isinstance(message, str):
 *                 message = self.get_registered_string(message)
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_DONT_SET); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 889, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_v_message != __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":890
 *         """
 *         if message is not DONT_SET:
 *             if isinstance(message, str):             # <<<<<<<<<<<<<<
 *                 message = self.get_registered_string(message)
 *             attributes.message = message
*/
    __pyx_t_2 = PyUnicode_Check(__pyx_v_message); 
    if (__pyx_t_2) {

      /* "nvtx/_lib/lib.pyx":891
 *         if message is not DONT_SET:
 *             if isinstance(message, str):
 *                 message = self.get_registered_string(message)             # <<<<<<<<<<<<<<
 *             attributes.message = message
 *         if color is not DONT_SET:
*/
      __pyx_t_3 = __pyx_v_self;
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_message};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_registered_string, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 891, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF_SET(__pyx_v_message, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "nvtx/_lib/lib.pyx":890
 *         """
 *         if message is not DONT_SET:
 *             if isinstance(message, str):             # <<<<<<<<<<<<<<
 *                 message = self.get_registered_string(message)
 *             attributes.message = message
*/
    }

    /* "nvtx/_lib/lib.pyx":892
 *             if isinstance(message, str):
 *                 message = self.get_registered_string(message)
 *             attributes.message = message             # <<<<<<<<<<<<<<
 *         if color is not DONT_SET:
 *             attributes.color = color
*/
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_n_u_message, __pyx_v_message) < (0)) __PYX_ERR(0, 892, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":889
 *             which is often useful for analysis.
 *         """
 *         if message is not DONT_SET:             # <<<<<<<<<<<<<<
 *             if isinstance(message, str):
 *                 message = self.get_registered_string(message)
*/
  }

  /* "nvtx/_lib/lib.pyx":893
 *                 message = self.get_registered_string(message)
 *             attributes.message = message
 *         if color is not DONT_SET:             # <<<<<<<<<<<<<<
 *             attributes.color = color
 *         if category is not DONT_SET:
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_DONT_SET); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_v_color != __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":894
 *             attributes.message = message
 *         if color is not DONT_SET:
 *             attributes.color = color             # <<<<<<<<<<<<<<
 *         if category is not DONT_SET:
 *             if isinstance(category, str):
*/
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_n_u_color, __pyx_v_color) < (0)) __PYX_ERR(0, 894, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":893
 *                 message = self.get_registered_string(message)
 *             attributes.message = message
 *         if color is not DONT_SET:             # <<<<<<<<<<<<<<
 *             attributes.color = color
 *         if category is not DONT_SET:
*/
  }

  /* "nvtx/_lib/lib.pyx":895
 *         if color is not DONT_SET:
 *             attributes.color = color
 *         if category is not DONT_SET:             # <<<<<<<<<<<<<<
 *             if isinstance(category, str):
 *                 category = self.get_category_id(category)
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_DONT_SET); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_v_category != __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":896
 *             attributes.color = color
 *         if category is not DONT_SET:
 *             if isinstance(category, str):             # <<<<<<<<<<<<<<
 *                 category = self.get_category_id(category)
 *             attributes.category = category
*/
    __pyx_t_2 = PyUnicode_Check(__pyx_v_category); 
    if (__pyx_t_2) {

      /* "nvtx/_lib/lib.pyx":897
 *         if category is not DONT_SET:
 *             if isinstance(category, str):
 *                 category = self.get_category_id(category)             # <<<<<<<<<<<<<<
 *             attributes.category = category
 *         if payload is not DONT_SET:
*/
      __pyx_t_3 = __pyx_v_self;
      __Pyx_INCREF(__pyx_t_3);
      __pyx_t_4 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_category};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_category_id, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 897, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF_SET(__pyx_v_category, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "nvtx/_lib/lib.pyx":896
 *             attributes.color = color
 *         if category is not DONT_SET:
 *             if isinstance(category, str):             # <<<<<<<<<<<<<<
 *                 category = self.get_category_id(category)
 *             attributes.category = category
*/
    }

    /* "nvtx/_lib/lib.pyx":898
 *             if isinstance(category, str):
 *                 category = self.get_category_id(category)
 *             attributes.category = category             # <<<<<<<<<<<<<<
 *         if payload is not DONT_SET:
 *             attributes.payload = payload
*/
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_n_u_category, __pyx_v_category) < (0)) __PYX_ERR(0, 898, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":895
 *         if color is not DONT_SET:
 *             attributes.color = color
 *         if category is not DONT_SET:             # <<<<<<<<<<<<<<
 *             if isinstance(category, str):
 *                 category = self.get_category_id(category)
*/
  }

  /* "nvtx/_lib/lib.pyx":899
 *                 category = self.get_category_id(category)
 *             attributes.category = category
 *         if payload is not DONT_SET:             # <<<<<<<<<<<<<<
 *             attributes.payload = payload
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_DONT_SET); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_v_payload != __pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "nvtx/_lib/lib.pyx":900
 *             attributes.category = category
 *         if payload is not DONT_SET:
 *             attributes.payload = payload             # <<<<<<<<<<<<<<
 * 
 *     def mark(self, EventAttributes attributes=None, *, **kwargs):
*/
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_n_u_payload, __pyx_v_payload) < (0)) __PYX_ERR(0, 900, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":899
 *                 category = self.get_category_id(category)
 *             attributes.category = category
 *         if payload is not DONT_SET:             # <<<<<<<<<<<<<<
 *             attributes.payload = payload
 * 
*/
  }

  /* "nvtx/_lib/lib.pyx":866
 *         return EventAttributes(self, message, color, category, payload)
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,             # <<<<<<<<<<<<<<
 *                              message=DONT_SET, color=DONT_SET, category=DONT_SET, payload=DONT_SET):
 *         """
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.set_event_attributes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_message);
  __Pyx_XDECREF(__pyx_v_category);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":902
 *             attributes.payload = payload
 * 
 *     def mark(self, EventAttributes attributes=None, *, **kwargs):             # <<<<<<<<<<<<<<
 *         """
 *         Mark an instantaneous event.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_13mark(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_12mark, "Domain.mark(self, EventAttributes attributes=None, **kwargs)\n\nMark an instantaneous event.\n\nParameters\n----------\nattributes : EventAttributes, optional\n    The event attributes to be associated with the event.\n    If not provided, a new :class:`EventAttributes` object is created\n    with the given keyword arguments.\n    Otherwise, this method mutates the attributes object if `kwargs` are provided.\nmessage : str, optional\n    A message associated with the event.\n    If the given message was not registered then it will be registered under this domain.\ncolor : str, int, optional\n    A color associated with the event.\n    Supports `matplotlib` colors if it is available.\ncategory : str, int, optional\n    A string or an integer specifying the category within the domain under which the event\n    is scoped. If unspecified, the event is not associated with a category.\npayload : int, float, list, tuple, range, bytes, numpy.ndarray, optional\n    A value associated with the event.\n    Using payloads provides a separation between the message and the data of the event,\n    which is often useful for analysis.\n\nExamples\n--------\n>>> import nvtx\n>>> domain = nvtx.Domain('my_domain')\n>>> domain.mark(message='my_marker')\n\nAlternatively, an EventAttributes object can be reused:\n\n>>> attributes = domain.get_event_attributes(message='my_marker')\n>>> domain.mark(attributes)\n>>> domain.mark(attributes, message='my_marker_2')");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_13mark = {"mark", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_13mark, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_12mark};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_13mark(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  PyObject *__pyx_v_kwargs = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("mark (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_attributes,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 902, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 902, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 902, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, __pyx_v_kwargs, values, kwd_pos_args, __pyx_kwds_len, "mark", 1) < (0)) __PYX_ERR(0, 902, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("mark", 0, 1, 2, i); __PYX_ERR(0, 902, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 902, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 902, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("mark", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 902, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.mark", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 902, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_12mark(__pyx_self, __pyx_v_self, __pyx_v_attributes, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_DECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_12mark(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("mark", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_attributes);

  /* "nvtx/_lib/lib.pyx":939
 *         >>> domain.mark(attributes, message='my_marker_2')
 *         """
 *         if attributes is None:             # <<<<<<<<<<<<<<
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:
*/
  __pyx_t_1 = (((PyObject *)__pyx_v_attributes) == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":940
 *         """
 *         if attributes is None:
 *             attributes = self.get_event_attributes(**kwargs)             # <<<<<<<<<<<<<<
 *         elif kwargs:
 *             self.set_event_attributes(attributes, **kwargs)
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_get_event_attributes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 940, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 940, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 940, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes))))) __PYX_ERR(0, 940, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_attributes, ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "nvtx/_lib/lib.pyx":939
 *         >>> domain.mark(attributes, message='my_marker_2')
 *         """
 *         if attributes is None:             # <<<<<<<<<<<<<<
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/lib.pyx":941
 *         if attributes is None:
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:             # <<<<<<<<<<<<<<
 *             self.set_event_attributes(attributes, **kwargs)
 *         nvtxDomainMarkEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 941, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":942
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:
 *             self.set_event_attributes(attributes, **kwargs)             # <<<<<<<<<<<<<<
 *         nvtxDomainMarkEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
*/
    __pyx_t_3 = NULL;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_set_event_attributes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 942, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 942, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, ((PyObject *)__pyx_v_attributes)};
      __pyx_t_4 = __Pyx_PyObject_FastCallDict((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 942, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/lib.pyx":941
 *         if attributes is None:
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:             # <<<<<<<<<<<<<<
 *             self.set_event_attributes(attributes, **kwargs)
 *         nvtxDomainMarkEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
*/
  }
  __pyx_L3:;

  /* "nvtx/_lib/lib.pyx":943
 *         elif kwargs:
 *             self.set_event_attributes(attributes, **kwargs)
 *         nvtxDomainMarkEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)             # <<<<<<<<<<<<<<
 * 
 *     def push_range(self, EventAttributes attributes=None, *, **kwargs):
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 943, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  nvtxDomainMarkEx(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_t_4)->c_obj, (&__pyx_v_attributes->c_obj));
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":902
 *             attributes.payload = payload
 * 
 *     def mark(self, EventAttributes attributes=None, *, **kwargs):             # <<<<<<<<<<<<<<
 *         """
 *         Mark an instantaneous event.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.mark", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_attributes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":945
 *         nvtxDomainMarkEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
 *     def push_range(self, EventAttributes attributes=None, *, **kwargs):             # <<<<<<<<<<<<<<
 *         """
 *         Mark the beginning of a code range.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_15push_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_14push_range, "Domain.push_range(self, EventAttributes attributes=None, **kwargs)\n\nMark the beginning of a code range.\n\nParameters\n----------\nattributes : EventAttributes, optional\n    The event attributes to be associated with the event.\n    If not provided, a new :class:`EventAttributes` object is created\n    with the given keyword arguments.\n    Otherwise, this method mutates the attributes object if `kwargs` are provided.\nmessage : str, optional\n    A message associated with the event.\n    If the given message was not registered then it will be registered under this domain.\ncolor : str, int, optional\n    A color associated with the event.\n    Supports `matplotlib` colors if it is available.\ncategory : str, int, optional\n    A string or an integer specifying the category within the domain under which the event\n    is scoped. If unspecified, the event is not associated with a category.\npayload : int, float, list, tuple, range, bytes, numpy.ndarray, optional\n    A value associated with the event.\n    Using payloads provides a separation between the message and the data of the event,\n    which is often useful for analysis.\n\nNotes\n-----\nWhen applicable, prefer to use :class:`annotate`.\n\nExamples\n--------\n>>> import time\n>>> import nvtx\n>>> domain = nvtx.Domain('my_domain')\n>>> domain.push_range(message='my_code_range')\n>>> time.sleep(1)\n>>> domain.pop_range()\n\nAlternatively, an EventAttributes object can be reused:\n\n>>> attributes = domain.get_event_attributes(message='my_code_range')\n>>> domain.push_range(attributes)\n>>> domain.push_range(attributes, message='my_code_range_2')\n>>> time.sleep(1)\n>>> domain.pop_range()\n>>> domain.pop_range()");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_15push_range = {"push_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_15push_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_14push_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_15push_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  PyObject *__pyx_v_kwargs = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_attributes,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 945, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 945, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 945, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, __pyx_v_kwargs, values, kwd_pos_args, __pyx_kwds_len, "push_range", 1) < (0)) __PYX_ERR(0, 945, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("push_range", 0, 1, 2, i); __PYX_ERR(0, 945, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 945, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 945, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("push_range", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 945, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.push_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 945, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_14push_range(__pyx_self, __pyx_v_self, __pyx_v_attributes, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_DECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_14push_range(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("push_range", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_attributes);

  /* "nvtx/_lib/lib.pyx":992
 *         >>> domain.pop_range()
 *         """
 *         if attributes is None:             # <<<<<<<<<<<<<<
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:
*/
  __pyx_t_1 = (((PyObject *)__pyx_v_attributes) == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":993
 *         """
 *         if attributes is None:
 *             attributes = self.get_event_attributes(**kwargs)             # <<<<<<<<<<<<<<
 *         elif kwargs:
 *             self.set_event_attributes(attributes, **kwargs)
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_get_event_attributes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 993, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 993, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 993, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes))))) __PYX_ERR(0, 993, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_attributes, ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "nvtx/_lib/lib.pyx":992
 *         >>> domain.pop_range()
 *         """
 *         if attributes is None:             # <<<<<<<<<<<<<<
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/lib.pyx":994
 *         if attributes is None:
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:             # <<<<<<<<<<<<<<
 *             self.set_event_attributes(attributes, **kwargs)
 *         nvtxDomainRangePushEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 994, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":995
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:
 *             self.set_event_attributes(attributes, **kwargs)             # <<<<<<<<<<<<<<
 *         nvtxDomainRangePushEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
*/
    __pyx_t_3 = NULL;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_set_event_attributes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 995, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 995, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, ((PyObject *)__pyx_v_attributes)};
      __pyx_t_4 = __Pyx_PyObject_FastCallDict((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 995, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/lib.pyx":994
 *         if attributes is None:
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:             # <<<<<<<<<<<<<<
 *             self.set_event_attributes(attributes, **kwargs)
 *         nvtxDomainRangePushEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
*/
  }
  __pyx_L3:;

  /* "nvtx/_lib/lib.pyx":996
 *         elif kwargs:
 *             self.set_event_attributes(attributes, **kwargs)
 *         nvtxDomainRangePushEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)             # <<<<<<<<<<<<<<
 * 
 *     def pop_range(self):
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 996, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  (void)(nvtxDomainRangePushEx(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_t_4)->c_obj, (&__pyx_v_attributes->c_obj)));
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":945
 *         nvtxDomainMarkEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
 *     def push_range(self, EventAttributes attributes=None, *, **kwargs):             # <<<<<<<<<<<<<<
 *         """
 *         Mark the beginning of a code range.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.push_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_attributes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":998
 *         nvtxDomainRangePushEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
 *     def pop_range(self):             # <<<<<<<<<<<<<<
 *         """
 *         Mark the end of a code range that was started with :func:`Domain.push_range`.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_17pop_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_16pop_range, "Domain.pop_range(self)\n\nMark the end of a code range that was started with :func:`Domain.push_range`.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_17pop_range = {"pop_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_17pop_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_16pop_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_17pop_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("pop_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 998, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 998, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "pop_range", 0) < (0)) __PYX_ERR(0, 998, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("pop_range", 1, 1, 1, i); __PYX_ERR(0, 998, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 998, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("pop_range", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 998, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.pop_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_16pop_range(__pyx_self, __pyx_v_self);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_16pop_range(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pop_range", 0);

  /* "nvtx/_lib/lib.pyx":1002
 *         Mark the end of a code range that was started with :func:`Domain.push_range`.
 *         """
 *         nvtxDomainRangePop((<DomainHandle>self.handle).c_obj)             # <<<<<<<<<<<<<<
 * 
 *     def start_range(self, EventAttributes attributes=None, *, **kwargs) -> int:
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1002, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  (void)(nvtxDomainRangePop(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_t_1)->c_obj));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":998
 *         nvtxDomainRangePushEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
 *     def pop_range(self):             # <<<<<<<<<<<<<<
 *         """
 *         Mark the end of a code range that was started with :func:`Domain.push_range`.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.pop_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1004
 *         nvtxDomainRangePop((<DomainHandle>self.handle).c_obj)
 * 
 *     def start_range(self, EventAttributes attributes=None, *, **kwargs) -> int:             # <<<<<<<<<<<<<<
 *         """
 *         Mark the beginning of a process range.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_19start_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_18start_range, "Domain.start_range(self, EventAttributes attributes=None, **kwargs) -> int\n\nMark the beginning of a process range.\n\nParameters\n----------\nattributes : EventAttributes, optional\n    The event attributes to be associated with the event.\n    If not provided, a new :class:`EventAttributes` object is created\n    with the given keyword arguments.\n    Otherwise, this method mutates the attributes object if `kwargs` are provided.\nmessage : str, optional\n    A message associated with the event.\n    If the given message was not registered then it will be registered under this domain.\ncolor : str, int, optional\n    A color associated with the event.\n    Supports `matplotlib` colors if it is available.\ncategory : str, int, optional\n    A string or an integer specifying the category within the domain under which the event\n    is scoped. If unspecified, the event is not associated with a category.\npayload : int, float, list, tuple, range, bytes, numpy.ndarray, optional\n    A value associated with the event.\n    Using payloads provides a separation between the message and the data of the event,\n    which is often useful for analysis.\n\nReturns\n-------\nA numeric value that must be passed to :func:`Domain.end_range`.\n\nExamples\n--------\n>>> import time\n>>> import nvtx\n>>> domain = nvtx.Domain('my_domain')\n>>> range_id = domain.start_range(message='my_code_range')\n>>> time.sleep(1)\n>>> domain.end_range(range_id)\n\nAlternatively, an EventAttributes object can be reused:\n\n>>> attributes = domain.get_event_attributes(message='my_code_range')\n>>> range_id = domain.start_range(attributes)\n>>> time.sleep(1)\n>>> domain.end_range(range_id)\n>>> range_id = domain.start_range(attributes, message='my_code_range_2')\n>>> time.sleep(1)\n>>> domain.end_range(range_id)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_19start_range = {"start_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_19start_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_18start_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_19start_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  PyObject *__pyx_v_kwargs = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_attributes,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1004, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, __pyx_v_kwargs, values, kwd_pos_args, __pyx_kwds_len, "start_range", 1) < (0)) __PYX_ERR(0, 1004, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("start_range", 0, 1, 2, i); __PYX_ERR(0, 1004, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1004, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1004, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)Py_None));
    }
    __pyx_v_self = values[0];
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("start_range", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 1004, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.start_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 1004, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_18start_range(__pyx_self, __pyx_v_self, __pyx_v_attributes, __pyx_v_kwargs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_DECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_18start_range(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("start_range", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_attributes);

  /* "nvtx/_lib/lib.pyx":1052
 *         >>> domain.end_range(range_id)
 *         """
 *         if attributes is None:             # <<<<<<<<<<<<<<
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:
*/
  __pyx_t_1 = (((PyObject *)__pyx_v_attributes) == Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":1053
 *         """
 *         if attributes is None:
 *             attributes = self.get_event_attributes(**kwargs)             # <<<<<<<<<<<<<<
 *         elif kwargs:
 *             self.set_event_attributes(attributes, **kwargs)
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_get_event_attributes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes))))) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_attributes, ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "nvtx/_lib/lib.pyx":1052
 *         >>> domain.end_range(range_id)
 *         """
 *         if attributes is None:             # <<<<<<<<<<<<<<
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/lib.pyx":1054
 *         if attributes is None:
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:             # <<<<<<<<<<<<<<
 *             self.set_event_attributes(attributes, **kwargs)
 *         return nvtxDomainRangeStartEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_kwargs); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1054, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":1055
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:
 *             self.set_event_attributes(attributes, **kwargs)             # <<<<<<<<<<<<<<
 *         return nvtxDomainRangeStartEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
*/
    __pyx_t_3 = NULL;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_set_event_attributes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1055, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1055, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, ((PyObject *)__pyx_v_attributes)};
      __pyx_t_4 = __Pyx_PyObject_FastCallDict((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1055, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "nvtx/_lib/lib.pyx":1054
 *         if attributes is None:
 *             attributes = self.get_event_attributes(**kwargs)
 *         elif kwargs:             # <<<<<<<<<<<<<<
 *             self.set_event_attributes(attributes, **kwargs)
 *         return nvtxDomainRangeStartEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
*/
  }
  __pyx_L3:;

  /* "nvtx/_lib/lib.pyx":1056
 *         elif kwargs:
 *             self.set_event_attributes(attributes, **kwargs)
 *         return nvtxDomainRangeStartEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)             # <<<<<<<<<<<<<<
 * 
 *     def end_range(self, nvtxRangeId_t range_id):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1056, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyLong_From_uint64_t(nvtxDomainRangeStartEx(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_t_4)->c_obj, (&__pyx_v_attributes->c_obj))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1056, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyInt_FromNumber(&__pyx_t_2, NULL, 0) < (0)) __PYX_ERR(0, 1056, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":1004
 *         nvtxDomainRangePop((<DomainHandle>self.handle).c_obj)
 * 
 *     def start_range(self, EventAttributes attributes=None, *, **kwargs) -> int:             # <<<<<<<<<<<<<<
 *         """
 *         Mark the beginning of a process range.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.start_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_attributes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1058
 *         return nvtxDomainRangeStartEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
 *     def end_range(self, nvtxRangeId_t range_id):             # <<<<<<<<<<<<<<
 *         """
 *         Mark the end of a process range that was started with :func:`Domain.start_range`.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_21end_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_20end_range, "Domain.end_range(self, nvtxRangeId_t range_id)\n\nMark the end of a process range that was started with :func:`Domain.start_range`.\n\nParameters\n----------\nrange_id : int\n    The value returned by :func:`Domain.start_range`.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_21end_range = {"end_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_21end_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_20end_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_21end_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  nvtxRangeId_t __pyx_v_range_id;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("end_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_range_id,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1058, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1058, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1058, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "end_range", 0) < (0)) __PYX_ERR(0, 1058, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("end_range", 1, 2, 2, i); __PYX_ERR(0, 1058, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1058, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1058, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_range_id = __Pyx_PyLong_As_uint64_t(values[1]); if (unlikely((__pyx_v_range_id == ((nvtxRangeId_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1058, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("end_range", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1058, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.end_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_20end_range(__pyx_self, __pyx_v_self, __pyx_v_range_id);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_20end_range(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, nvtxRangeId_t __pyx_v_range_id) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("end_range", 0);

  /* "nvtx/_lib/lib.pyx":1067
 *             The value returned by :func:`Domain.start_range`.
 *         """
 *         nvtxDomainRangeEnd((<DomainHandle>self.handle).c_obj, range_id)             # <<<<<<<<<<<<<<
 * 
 *     def get_counter(
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1067, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  nvtxDomainRangeEnd(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_t_1)->c_obj, __pyx_v_range_id);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":1058
 *         return nvtxDomainRangeStartEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
 *     def end_range(self, nvtxRangeId_t range_id):             # <<<<<<<<<<<<<<
 *         """
 *         Mark the end of a process range that was started with :func:`Domain.start_range`.
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.end_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1069
 *         nvtxDomainRangeEnd((<DomainHandle>self.handle).c_obj, range_id)
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_30__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "nvtx/_lib/lib.pyx":1074
 *         dtype,
 *         *,
 *         description=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         semantics=None,
*/
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_description, Py_None) < (0)) __PYX_ERR(0, 1069, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":1075
 *         *,
 *         description=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         semantics=None,
 *         time_domain=TimestampType.TOOL_PROVIDED,
*/
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_scope, Py_None) < (0)) __PYX_ERR(0, 1069, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":1076
 *         description=None,
 *         scope=None,
 *         semantics=None,             # <<<<<<<<<<<<<<
 *         time_domain=TimestampType.TOOL_PROVIDED,
 *     ) -> Counter:
*/
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_semantics, Py_None) < (0)) __PYX_ERR(0, 1069, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":1069
 *         nvtxDomainRangeEnd((<DomainHandle>self.handle).c_obj, range_id)
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/
  if (PyDict_SetItem(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_time_domain, __Pyx_CyFunction_Defaults(struct __pyx_defaults, __pyx_self)->arg0) < (0)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, Py_None) != (0)) __PYX_ERR(0, 1069, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1) != (0)) __PYX_ERR(0, 1069, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_23get_counter(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_22get_counter, "Domain.get_counter(self, name, dtype, *, description=None, scope=None, semantics=None, time_domain=TimestampType.TOOL_PROVIDED) -> Counter\n\nGet an NVTX counter in this domain.\n\nFor enabled domains, the counter is created on first use and cached.\nSubsequent calls with the same arguments return the same counter\nobject.\n\nParameters\n----------\nname : str or bytes\n    Display name for the counter.\ndtype : int, float, or numpy.dtype\n    Counter value type. ``int`` records signed 64-bit integer\n    samples, ``float`` records double-precision samples, and a\n    NumPy dtype-like object records samples with that dtype.\n    Structured dtypes represent flat counter groups.\ndescription : str or bytes, optional\n    Longer description for the counter.\nscope : str or bytes, optional\n    Scope path for the counter.\nsemantics : CounterSemantics, optional\n    Semantics for the counter as a whole. For per-field semantics\n    in a structured dtype, build the field dtype with\n    :func:`nvtx.numpy_dtype`. Dtype metadata applies to schema entries,\n    so top-level scalar counters should use this argument instead\n    of ``nvtx.numpy_dtype(..., counter_semantics=...)``.\ntime_domain : TimestampType, optional\n    Timestamp domain used for batched samples.\n\nReturns\n-------\nCounter\n    A counter whose concrete sample type is determined by ``dtype``,\n    one of :class:`Int64Counter`, :class:`Float64Counter`, or\n    :class:`ExtCounter`.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_23get_counter = {"get_counter", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_23get_counter, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_22get_counter};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_23get_counter(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_v_description = 0;
  PyObject *__pyx_v_scope = 0;
  PyObject *__pyx_v_semantics = 0;
  PyObject *__pyx_v_time_domain = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[7] = {0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_counter (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_dtype,&__pyx_mstate_global->__pyx_n_u_description,&__pyx_mstate_global->__pyx_n_u_scope,&__pyx_mstate_global->__pyx_n_u_semantics,&__pyx_mstate_global->__pyx_n_u_time_domain,0};
    struct __pyx_defaults *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(struct __pyx_defaults, __pyx_self);
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1069, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1069, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1069, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1069, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_counter", 0) < (0)) __PYX_ERR(0, 1069, __pyx_L3_error)

      /* "nvtx/_lib/lib.pyx":1074
 *         dtype,
 *         *,
 *         description=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         semantics=None,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":1075
 *         *,
 *         description=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         semantics=None,
 *         time_domain=TimestampType.TOOL_PROVIDED,
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":1076
 *         description=None,
 *         scope=None,
 *         semantics=None,             # <<<<<<<<<<<<<<
 *         time_domain=TimestampType.TOOL_PROVIDED,
 *     ) -> Counter:
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[6]) values[6] = __Pyx_NewRef(__pyx_dynamic_args->arg0);
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_counter", 1, 3, 3, i); __PYX_ERR(0, 1069, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1069, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1069, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1069, __pyx_L3_error)

      /* "nvtx/_lib/lib.pyx":1074
 *         dtype,
 *         *,
 *         description=None,             # <<<<<<<<<<<<<<
 *         scope=None,
 *         semantics=None,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":1075
 *         *,
 *         description=None,
 *         scope=None,             # <<<<<<<<<<<<<<
 *         semantics=None,
 *         time_domain=TimestampType.TOOL_PROVIDED,
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "nvtx/_lib/lib.pyx":1076
 *         description=None,
 *         scope=None,
 *         semantics=None,             # <<<<<<<<<<<<<<
 *         time_domain=TimestampType.TOOL_PROVIDED,
 *     ) -> Counter:
*/
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[6]) values[6] = __Pyx_NewRef(__pyx_dynamic_args->arg0);
    }
    __pyx_v_self = values[0];
    __pyx_v_name = values[1];
    __pyx_v_dtype = values[2];
    __pyx_v_description = values[3];
    __pyx_v_scope = values[4];
    __pyx_v_semantics = values[5];
    __pyx_v_time_domain = values[6];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_counter", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 1069, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_22get_counter(__pyx_self, __pyx_v_self, __pyx_v_name, __pyx_v_dtype, __pyx_v_description, __pyx_v_scope, __pyx_v_semantics, __pyx_v_time_domain);

  /* "nvtx/_lib/lib.pyx":1069
 *         nvtxDomainRangeEnd((<DomainHandle>self.handle).c_obj, range_id)
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_22get_counter(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_dtype, PyObject *__pyx_v_description, PyObject *__pyx_v_scope, PyObject *__pyx_v_semantics, PyObject *__pyx_v_time_domain) {
  PyObject *__pyx_v_schema_key = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_counter", 0);
  __Pyx_INCREF(__pyx_v_dtype);

  /* "nvtx/_lib/lib.pyx":1116
 *         """
 * 
 *         dtype = _normalize_counter_dtype(dtype)             # <<<<<<<<<<<<<<
 *         schema_key = PayloadSchemaKey(
 *             dtype, counter_group=getattr(dtype, "fields", None) is not None
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_normalize_counter_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_dtype};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF_SET(__pyx_v_dtype, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":1117
 * 
 *         dtype = _normalize_counter_dtype(dtype)
 *         schema_key = PayloadSchemaKey(             # <<<<<<<<<<<<<<
 *             dtype, counter_group=getattr(dtype, "fields", None) is not None
 *         )
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_PayloadSchemaKey); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "nvtx/_lib/lib.pyx":1118
 *         dtype = _normalize_counter_dtype(dtype)
 *         schema_key = PayloadSchemaKey(
 *             dtype, counter_group=getattr(dtype, "fields", None) is not None             # <<<<<<<<<<<<<<
 *         )
 *         return self._get_counter_cached(
*/
  __pyx_t_5 = __Pyx_GetAttr3(__pyx_v_dtype, __pyx_mstate_global->__pyx_n_u_fields, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = (__pyx_t_5 != Py_None);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_v_dtype};
    __pyx_t_7 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_counter_group, __pyx_t_5, __pyx_t_7, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_schema_key = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":1120
 *             dtype, counter_group=getattr(dtype, "fields", None) is not None
 *         )
 *         return self._get_counter_cached(             # <<<<<<<<<<<<<<
 *             name,
 *             schema_key,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_v_self;
  __Pyx_INCREF(__pyx_t_2);

  /* "nvtx/_lib/lib.pyx":1126
 *             scope,
 *             semantics,
 *             time_domain.value,             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_time_domain, __pyx_mstate_global->__pyx_n_u_value); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[7] = {__pyx_t_2, __pyx_v_name, __pyx_v_schema_key, __pyx_v_description, __pyx_v_scope, __pyx_v_semantics, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_counter_cached, __pyx_callargs+__pyx_t_4, (7-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1120, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":1069
 *         nvtxDomainRangeEnd((<DomainHandle>self.handle).c_obj, range_id)
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_counter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_schema_key);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1129
 *         )
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def _get_scope_id(self, path) -> int:
 *         cdef bytes path_bytes = _to_bytes(path)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_25_get_scope_id(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_24_get_scope_id, "Domain._get_scope_id(self, path) -> int");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_25_get_scope_id = {"_get_scope_id", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_25_get_scope_id, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_24_get_scope_id};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_25_get_scope_id(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_path = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_scope_id (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_path,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1129, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1129, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1129, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_get_scope_id", 0) < (0)) __PYX_ERR(0, 1129, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_get_scope_id", 1, 2, 2, i); __PYX_ERR(0, 1129, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1129, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1129, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_path = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_scope_id", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1129, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain._get_scope_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_24_get_scope_id(__pyx_self, __pyx_v_self, __pyx_v_path);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_24_get_scope_id(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_path) {
  PyObject *__pyx_v_path_bytes = 0;
  nvtxScopeAttr_t __pyx_v_attr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  char const *__pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_scope_id", 0);

  /* "nvtx/_lib/lib.pyx":1131
 *     @lru_cache(maxsize=None)
 *     def _get_scope_id(self, path) -> int:
 *         cdef bytes path_bytes = _to_bytes(path)             # <<<<<<<<<<<<<<
 *         cdef nvtxScopeAttr_t attr
 *         attr.structSize = sizeof(nvtxScopeAttr_t)
*/
  __pyx_t_1 = __pyx_f_4nvtx_4_lib_3lib__to_bytes(__pyx_v_path, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_path_bytes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":1133
 *         cdef bytes path_bytes = _to_bytes(path)
 *         cdef nvtxScopeAttr_t attr
 *         attr.structSize = sizeof(nvtxScopeAttr_t)             # <<<<<<<<<<<<<<
 *         attr.path = path_bytes
 *         attr.parentScope = NVTX_SCOPE_NONE
*/
  __pyx_v_attr.structSize = (sizeof(nvtxScopeAttr_t));

  /* "nvtx/_lib/lib.pyx":1134
 *         cdef nvtxScopeAttr_t attr
 *         attr.structSize = sizeof(nvtxScopeAttr_t)
 *         attr.path = path_bytes             # <<<<<<<<<<<<<<
 *         attr.parentScope = NVTX_SCOPE_NONE
 *         attr.scopeId = NVTX_SCOPE_NONE
*/
  if (unlikely(__pyx_v_path_bytes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 1134, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyBytes_AsString(__pyx_v_path_bytes); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 1134, __pyx_L1_error)
  __pyx_v_attr.path = __pyx_t_2;

  /* "nvtx/_lib/lib.pyx":1135
 *         attr.structSize = sizeof(nvtxScopeAttr_t)
 *         attr.path = path_bytes
 *         attr.parentScope = NVTX_SCOPE_NONE             # <<<<<<<<<<<<<<
 *         attr.scopeId = NVTX_SCOPE_NONE
 *         return nvtxScopeRegister((<DomainHandle>self.handle).c_obj, &attr)
*/
  __pyx_v_attr.parentScope = NVTX_SCOPE_NONE;

  /* "nvtx/_lib/lib.pyx":1136
 *         attr.path = path_bytes
 *         attr.parentScope = NVTX_SCOPE_NONE
 *         attr.scopeId = NVTX_SCOPE_NONE             # <<<<<<<<<<<<<<
 *         return nvtxScopeRegister((<DomainHandle>self.handle).c_obj, &attr)
 * 
*/
  __pyx_v_attr.scopeId = NVTX_SCOPE_NONE;

  /* "nvtx/_lib/lib.pyx":1137
 *         attr.parentScope = NVTX_SCOPE_NONE
 *         attr.scopeId = NVTX_SCOPE_NONE
 *         return nvtxScopeRegister((<DomainHandle>self.handle).c_obj, &attr)             # <<<<<<<<<<<<<<
 * 
 *     @lru_cache(maxsize=None)
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_handle); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyLong_From_uint64_t(nvtxScopeRegister(((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)__pyx_t_1)->c_obj, (&__pyx_v_attr))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyInt_FromNumber(&__pyx_t_3, NULL, 0) < (0)) __PYX_ERR(0, 1137, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":1129
 *         )
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def _get_scope_id(self, path) -> int:
 *         cdef bytes path_bytes = _to_bytes(path)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain._get_scope_id", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_path_bytes);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1139
 *         return nvtxScopeRegister((<DomainHandle>self.handle).c_obj, &attr)
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def _get_counter_cached(
 *         self,
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_27_get_counter_cached(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_26_get_counter_cached, "Domain._get_counter_cached(self, name, schema_key, description, scope, semantics, uint64_t time_domain)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_27_get_counter_cached = {"_get_counter_cached", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_27_get_counter_cached, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_26_get_counter_cached};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_27_get_counter_cached(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_schema_key = 0;
  PyObject *__pyx_v_description = 0;
  PyObject *__pyx_v_scope = 0;
  PyObject *__pyx_v_semantics = 0;
  uint64_t __pyx_v_time_domain;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[7] = {0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_counter_cached (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_schema_key,&__pyx_mstate_global->__pyx_n_u_description,&__pyx_mstate_global->__pyx_n_u_scope,&__pyx_mstate_global->__pyx_n_u_semantics,&__pyx_mstate_global->__pyx_n_u_time_domain,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1139, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  7:
        values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1139, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1139, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1139, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1139, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1139, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1139, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1139, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_get_counter_cached", 0) < (0)) __PYX_ERR(0, 1139, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 7; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_get_counter_cached", 1, 7, 7, i); __PYX_ERR(0, 1139, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 7)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1139, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1139, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1139, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 1139, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 1139, __pyx_L3_error)
      values[5] = __Pyx_ArgRef_FASTCALL(__pyx_args, 5);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 1139, __pyx_L3_error)
      values[6] = __Pyx_ArgRef_FASTCALL(__pyx_args, 6);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 1139, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_name = values[1];
    __pyx_v_schema_key = values[2];
    __pyx_v_description = values[3];
    __pyx_v_scope = values[4];
    __pyx_v_semantics = values[5];
    __pyx_v_time_domain = __Pyx_PyLong_As_uint64_t(values[6]); if (unlikely((__pyx_v_time_domain == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1147, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_counter_cached", 1, 7, 7, __pyx_nargs); __PYX_ERR(0, 1139, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain._get_counter_cached", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_26_get_counter_cached(__pyx_self, __pyx_v_self, __pyx_v_name, __pyx_v_schema_key, __pyx_v_description, __pyx_v_scope, __pyx_v_semantics, __pyx_v_time_domain);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_26_get_counter_cached(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_schema_key, PyObject *__pyx_v_description, PyObject *__pyx_v_scope, PyObject *__pyx_v_semantics, uint64_t __pyx_v_time_domain) {
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_counter_cls = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_counter_cached", 0);

  /* "nvtx/_lib/lib.pyx":1149
 *         uint64_t time_domain,
 *     ):
 *         dtype = schema_key.dtype             # <<<<<<<<<<<<<<
 *         counter_cls = _counter_class_from_dtype(dtype)
 *         return counter_cls(
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_schema_key, __pyx_mstate_global->__pyx_n_u_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_dtype = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":1150
 *     ):
 *         dtype = schema_key.dtype
 *         counter_cls = _counter_class_from_dtype(dtype)             # <<<<<<<<<<<<<<
 *         return counter_cls(
 *             self,
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_counter_class_from_dtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_dtype};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_counter_cls = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "nvtx/_lib/lib.pyx":1151
 *         dtype = schema_key.dtype
 *         counter_cls = _counter_class_from_dtype(dtype)
 *         return counter_cls(             # <<<<<<<<<<<<<<
 *             self,
 *             name,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = NULL;
  __Pyx_INCREF(__pyx_v_counter_cls);
  __pyx_t_2 = __pyx_v_counter_cls; 

  /* "nvtx/_lib/lib.pyx":1158
 *             scope,
 *             semantics,
 *             time_domain,             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_5 = __Pyx_PyLong_From_uint64_t(__pyx_v_time_domain); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[8] = {__pyx_t_3, __pyx_v_self, __pyx_v_name, __pyx_v_dtype, __pyx_v_description, __pyx_v_scope, __pyx_v_semantics, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_4, (8-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":1139
 *         return nvtxScopeRegister((<DomainHandle>self.handle).c_obj, &attr)
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def _get_counter_cached(
 *         self,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain._get_counter_cached", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_counter_cls);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1161
 *         )
 * 
 *     def get_timestamp(self):             # <<<<<<<<<<<<<<
 *         """
 *         Return an NVTX timestamp for use with batched counter samples.
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_29get_timestamp(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_28get_timestamp, "Domain.get_timestamp(self)\n\nReturn an NVTX timestamp for use with batched counter samples.");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_29get_timestamp = {"get_timestamp", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_29get_timestamp, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_28get_timestamp};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_29get_timestamp(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_timestamp (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1161, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1161, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_timestamp", 0) < (0)) __PYX_ERR(0, 1161, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_timestamp", 1, 1, 1, i); __PYX_ERR(0, 1161, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1161, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_timestamp", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1161, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_timestamp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_28get_timestamp(__pyx_self, __pyx_v_self);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_28get_timestamp(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_timestamp", 0);

  /* "nvtx/_lib/lib.pyx":1166
 *         """
 * 
 *         return nvtxTimestampGet()             # <<<<<<<<<<<<<<
 * 
 *     def _get_numpy_dtype_schema(self, schema_key):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_int64_t(nvtxTimestampGet()); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":1161
 *         )
 * 
 *     def get_timestamp(self):             # <<<<<<<<<<<<<<
 *         """
 *         Return an NVTX timestamp for use with batched counter samples.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain.get_timestamp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1168
 *         return nvtxTimestampGet()
 * 
 *     def _get_numpy_dtype_schema(self, schema_key):             # <<<<<<<<<<<<<<
 *         # Schema building/caching lives on the domain's SchemaRegistrar.
 *         return self._schema_registrar._get_numpy_dtype_schema(schema_key)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_31_get_numpy_dtype_schema(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_30_get_numpy_dtype_schema, "Domain._get_numpy_dtype_schema(self, schema_key)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_31_get_numpy_dtype_schema = {"_get_numpy_dtype_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_31_get_numpy_dtype_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_30_get_numpy_dtype_schema};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_31_get_numpy_dtype_schema(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_schema_key = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_numpy_dtype_schema (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_schema_key,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1168, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1168, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1168, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_get_numpy_dtype_schema", 0) < (0)) __PYX_ERR(0, 1168, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_get_numpy_dtype_schema", 1, 2, 2, i); __PYX_ERR(0, 1168, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1168, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1168, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_schema_key = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_numpy_dtype_schema", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1168, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain._get_numpy_dtype_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_30_get_numpy_dtype_schema(__pyx_self, __pyx_v_self, __pyx_v_schema_key);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_30_get_numpy_dtype_schema(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_schema_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_numpy_dtype_schema", 0);

  /* "nvtx/_lib/lib.pyx":1170
 *     def _get_numpy_dtype_schema(self, schema_key):
 *         # Schema building/caching lives on the domain's SchemaRegistrar.
 *         return self._schema_registrar._get_numpy_dtype_schema(schema_key)             # <<<<<<<<<<<<<<
 * 
 *     def _get_numpy_array_schema(self, scalar_schema):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_schema_registrar); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_schema_key};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":1168
 *         return nvtxTimestampGet()
 * 
 *     def _get_numpy_dtype_schema(self, schema_key):             # <<<<<<<<<<<<<<
 *         # Schema building/caching lives on the domain's SchemaRegistrar.
 *         return self._schema_registrar._get_numpy_dtype_schema(schema_key)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain._get_numpy_dtype_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1172
 *         return self._schema_registrar._get_numpy_dtype_schema(schema_key)
 * 
 *     def _get_numpy_array_schema(self, scalar_schema):             # <<<<<<<<<<<<<<
 *         return self._schema_registrar._get_numpy_array_schema(
 *             scalar_schema
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_33_get_numpy_array_schema(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_6Domain_32_get_numpy_array_schema, "Domain._get_numpy_array_schema(self, scalar_schema)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_6Domain_33_get_numpy_array_schema = {"_get_numpy_array_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_6Domain_33_get_numpy_array_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_6Domain_32_get_numpy_array_schema};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_6Domain_33_get_numpy_array_schema(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_scalar_schema = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_numpy_array_schema (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_self,&__pyx_mstate_global->__pyx_n_u_scalar_schema,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1172, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1172, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1172, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_get_numpy_array_schema", 0) < (0)) __PYX_ERR(0, 1172, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_get_numpy_array_schema", 1, 2, 2, i); __PYX_ERR(0, 1172, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1172, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1172, __pyx_L3_error)
    }
    __pyx_v_self = values[0];
    __pyx_v_scalar_schema = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_numpy_array_schema", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1172, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.Domain._get_numpy_array_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_6Domain_32_get_numpy_array_schema(__pyx_self, __pyx_v_self, __pyx_v_scalar_schema);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_6Domain_32_get_numpy_array_schema(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_scalar_schema) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_numpy_array_schema", 0);

  /* "nvtx/_lib/lib.pyx":1173
 * 
 *     def _get_numpy_array_schema(self, scalar_schema):
 *         return self._schema_registrar._get_numpy_array_schema(             # <<<<<<<<<<<<<<
 *             scalar_schema
 *         )
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_mstate_global->__pyx_n_u_schema_registrar); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);

  /* "nvtx/_lib/lib.pyx":1174
 *     def _get_numpy_array_schema(self, scalar_schema):
 *         return self._schema_registrar._get_numpy_array_schema(
 *             scalar_schema             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_scalar_schema};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_get_numpy_array_schema, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":1172
 *         return self._schema_registrar._get_numpy_dtype_schema(schema_key)
 * 
 *     def _get_numpy_array_schema(self, scalar_schema):             # <<<<<<<<<<<<<<
 *         return self._schema_registrar._get_numpy_array_schema(
 *             scalar_schema
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("nvtx._lib.lib.Domain._get_numpy_array_schema", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1180
 * cdef class StringHandle:
 * 
 *     def __init__(self, DomainHandle domain_handle, object string=None):             # <<<<<<<<<<<<<<
 *         if string is not None:
 *             self._string = _to_bytes(string)
*/

/* Python wrapper */
static int __pyx_pw_4nvtx_4_lib_3lib_12StringHandle_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_4nvtx_4_lib_3lib_12StringHandle_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain_handle = 0;
  PyObject *__pyx_v_string = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_domain_handle,&__pyx_mstate_global->__pyx_n_u_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1180, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1180, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1180, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 1180, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, i); __PYX_ERR(0, 1180, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1180, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1180, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
    }
    __pyx_v_domain_handle = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)values[0]);
    __pyx_v_string = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 1180, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.StringHandle.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain_handle), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle, 1, "domain_handle", 0))) __PYX_ERR(0, 1180, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12StringHandle___init__(((struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *)__pyx_v_self), __pyx_v_domain_handle, __pyx_v_string);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_4nvtx_4_lib_3lib_12StringHandle___init__(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *__pyx_v_self, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain_handle, PyObject *__pyx_v_string) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  char const *__pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "nvtx/_lib/lib.pyx":1181
 * 
 *     def __init__(self, DomainHandle domain_handle, object string=None):
 *         if string is not None:             # <<<<<<<<<<<<<<
 *             self._string = _to_bytes(string)
 *             self.c_obj = nvtxDomainRegisterStringA(
*/
  __pyx_t_1 = (__pyx_v_string != Py_None);
  if (__pyx_t_1) {

    /* "nvtx/_lib/lib.pyx":1182
 *     def __init__(self, DomainHandle domain_handle, object string=None):
 *         if string is not None:
 *             self._string = _to_bytes(string)             # <<<<<<<<<<<<<<
 *             self.c_obj = nvtxDomainRegisterStringA(
 *                 domain_handle.c_obj, self._string
*/
    __pyx_t_2 = __pyx_f_4nvtx_4_lib_3lib__to_bytes(__pyx_v_string, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1182, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_2);
    __Pyx_GOTREF(__pyx_v_self->_string);
    __Pyx_DECREF(__pyx_v_self->_string);
    __pyx_v_self->_string = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "nvtx/_lib/lib.pyx":1184
 *             self._string = _to_bytes(string)
 *             self.c_obj = nvtxDomainRegisterStringA(
 *                 domain_handle.c_obj, self._string             # <<<<<<<<<<<<<<
 *             )
 *         else:
*/
    if (unlikely(__pyx_v_self->_string == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 1184, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyBytes_AsString(__pyx_v_self->_string); if (unlikely((!__pyx_t_3) && PyErr_Occurred())) __PYX_ERR(0, 1184, __pyx_L1_error)

    /* "nvtx/_lib/lib.pyx":1183
 *         if string is not None:
 *             self._string = _to_bytes(string)
 *             self.c_obj = nvtxDomainRegisterStringA(             # <<<<<<<<<<<<<<
 *                 domain_handle.c_obj, self._string
 *             )
*/
    __pyx_v_self->c_obj = nvtxDomainRegisterStringA(__pyx_v_domain_handle->c_obj, __pyx_t_3);

    /* "nvtx/_lib/lib.pyx":1181
 * 
 *     def __init__(self, DomainHandle domain_handle, object string=None):
 *         if string is not None:             # <<<<<<<<<<<<<<
 *             self._string = _to_bytes(string)
 *             self.c_obj = nvtxDomainRegisterStringA(
*/
    goto __pyx_L3;
  }

  /* "nvtx/_lib/lib.pyx":1187
 *             )
 *         else:
 *             self._string = b""             # <<<<<<<<<<<<<<
 *             self.c_obj = NULL
 * 
*/
  /*else*/ {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_b__2);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_b__2);
    __Pyx_GOTREF(__pyx_v_self->_string);
    __Pyx_DECREF(__pyx_v_self->_string);
    __pyx_v_self->_string = __pyx_mstate_global->__pyx_kp_b__2;

    /* "nvtx/_lib/lib.pyx":1188
 *         else:
 *             self._string = b""
 *             self.c_obj = NULL             # <<<<<<<<<<<<<<
 * 
 *     @property
*/
    __pyx_v_self->c_obj = NULL;
  }
  __pyx_L3:;

  /* "nvtx/_lib/lib.pyx":1180
 * cdef class StringHandle:
 * 
 *     def __init__(self, DomainHandle domain_handle, object string=None):             # <<<<<<<<<<<<<<
 *         if string is not None:
 *             self._string = _to_bytes(string)
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.StringHandle.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1190
 *             self.c_obj = NULL
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def string(self):
 *         return self._string.decode()
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_6string_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_6string_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12StringHandle_6string___get__(((struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12StringHandle_6string___get__(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "nvtx/_lib/lib.pyx":1192
 *     @property
 *     def string(self):
 *         return self._string.decode()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->_string == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "decode");
    __PYX_ERR(0, 1192, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_decode_bytes(__pyx_v_self->_string, 0, PY_SSIZE_T_MAX, NULL, NULL, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":1190
 *             self.c_obj = NULL
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def string(self):
 *         return self._string.decode()
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("nvtx._lib.lib.StringHandle.string.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_12StringHandle_2__reduce_cython__, "StringHandle.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_12StringHandle_3__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12StringHandle_2__reduce_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_3__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12StringHandle_2__reduce_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12StringHandle_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_c_obj_cannot_be_converted_t, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.StringHandle.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_12StringHandle_4__setstate_cython__, "StringHandle.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_12StringHandle_5__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12StringHandle_4__setstate_cython__};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_5__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.StringHandle.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_12StringHandle_4__setstate_cython__(((struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_12StringHandle_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self_c_obj_cannot_be_converted_t, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("nvtx._lib.lib.StringHandle.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1195
 * 
 * 
 * def push_range(EventAttributes attributes, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainRangePushEx(domain.c_obj, &attributes.c_obj)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_17push_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_16push_range, "push_range(EventAttributes attributes, DomainHandle domain)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_17push_range = {"push_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_17push_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_16push_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_17push_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_attributes,&__pyx_mstate_global->__pyx_n_u_domain,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1195, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1195, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1195, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "push_range", 0) < (0)) __PYX_ERR(0, 1195, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("push_range", 1, 2, 2, i); __PYX_ERR(0, 1195, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1195, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1195, __pyx_L3_error)
    }
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[0]);
    __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("push_range", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1195, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.push_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 1195, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle, 1, "domain", 0))) __PYX_ERR(0, 1195, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_16push_range(__pyx_self, __pyx_v_attributes, __pyx_v_domain);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_16push_range(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("push_range", 0);

  /* "nvtx/_lib/lib.pyx":1196
 * 
 * def push_range(EventAttributes attributes, DomainHandle domain):
 *     nvtxDomainRangePushEx(domain.c_obj, &attributes.c_obj)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  (void)(nvtxDomainRangePushEx(__pyx_v_domain->c_obj, (&__pyx_v_attributes->c_obj)));

  /* "nvtx/_lib/lib.pyx":1195
 * 
 * 
 * def push_range(EventAttributes attributes, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainRangePushEx(domain.c_obj, &attributes.c_obj)
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1199
 * 
 * 
 * def pop_range(DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainRangePop(domain.c_obj)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_19pop_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_18pop_range, "pop_range(DomainHandle domain)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_19pop_range = {"pop_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_19pop_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_18pop_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_19pop_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("pop_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_domain,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1199, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1199, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "pop_range", 0) < (0)) __PYX_ERR(0, 1199, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("pop_range", 1, 1, 1, i); __PYX_ERR(0, 1199, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1199, __pyx_L3_error)
    }
    __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("pop_range", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1199, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.pop_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle, 1, "domain", 0))) __PYX_ERR(0, 1199, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_18pop_range(__pyx_self, __pyx_v_domain);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_18pop_range(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("pop_range", 0);

  /* "nvtx/_lib/lib.pyx":1200
 * 
 * def pop_range(DomainHandle domain):
 *     nvtxDomainRangePop(domain.c_obj)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  (void)(nvtxDomainRangePop(__pyx_v_domain->c_obj));

  /* "nvtx/_lib/lib.pyx":1199
 * 
 * 
 * def pop_range(DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainRangePop(domain.c_obj)
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1203
 * 
 * 
 * def start_range(EventAttributes attributes, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     return nvtxDomainRangeStartEx(domain.c_obj, &attributes.c_obj), domain
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_21start_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_20start_range, "start_range(EventAttributes attributes, DomainHandle domain)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_21start_range = {"start_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_21start_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_20start_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_21start_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_attributes,&__pyx_mstate_global->__pyx_n_u_domain,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1203, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1203, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1203, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "start_range", 0) < (0)) __PYX_ERR(0, 1203, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("start_range", 1, 2, 2, i); __PYX_ERR(0, 1203, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1203, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1203, __pyx_L3_error)
    }
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[0]);
    __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("start_range", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1203, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.start_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 1203, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle, 1, "domain", 0))) __PYX_ERR(0, 1203, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_20start_range(__pyx_self, __pyx_v_attributes, __pyx_v_domain);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_20start_range(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("start_range", 0);

  /* "nvtx/_lib/lib.pyx":1204
 * 
 * def start_range(EventAttributes attributes, DomainHandle domain):
 *     return nvtxDomainRangeStartEx(domain.c_obj, &attributes.c_obj), domain             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyLong_From_uint64_t(nvtxDomainRangeStartEx(__pyx_v_domain->c_obj, (&__pyx_v_attributes->c_obj))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1) != (0)) __PYX_ERR(0, 1204, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_domain);
  __Pyx_GIVEREF((PyObject *)__pyx_v_domain);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_domain)) != (0)) __PYX_ERR(0, 1204, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "nvtx/_lib/lib.pyx":1203
 * 
 * 
 * def start_range(EventAttributes attributes, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     return nvtxDomainRangeStartEx(domain.c_obj, &attributes.c_obj), domain
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("nvtx._lib.lib.start_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1207
 * 
 * 
 * def end_range(nvtxRangeId_t range_id, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainRangeEnd(domain.c_obj, range_id)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_23end_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_22end_range, "end_range(nvtxRangeId_t range_id, DomainHandle domain)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_23end_range = {"end_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_23end_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_22end_range};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_23end_range(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  nvtxRangeId_t __pyx_v_range_id;
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("end_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_range_id,&__pyx_mstate_global->__pyx_n_u_domain,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1207, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1207, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1207, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "end_range", 0) < (0)) __PYX_ERR(0, 1207, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("end_range", 1, 2, 2, i); __PYX_ERR(0, 1207, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1207, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1207, __pyx_L3_error)
    }
    __pyx_v_range_id = __Pyx_PyLong_As_uint64_t(values[0]); if (unlikely((__pyx_v_range_id == ((nvtxRangeId_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1207, __pyx_L3_error)
    __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("end_range", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1207, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.end_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle, 1, "domain", 0))) __PYX_ERR(0, 1207, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_22end_range(__pyx_self, __pyx_v_range_id, __pyx_v_domain);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_22end_range(CYTHON_UNUSED PyObject *__pyx_self, nvtxRangeId_t __pyx_v_range_id, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("end_range", 0);

  /* "nvtx/_lib/lib.pyx":1208
 * 
 * def end_range(nvtxRangeId_t range_id, DomainHandle domain):
 *     nvtxDomainRangeEnd(domain.c_obj, range_id)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  nvtxDomainRangeEnd(__pyx_v_domain->c_obj, __pyx_v_range_id);

  /* "nvtx/_lib/lib.pyx":1207
 * 
 * 
 * def end_range(nvtxRangeId_t range_id, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainRangeEnd(domain.c_obj, range_id)
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "nvtx/_lib/lib.pyx":1211
 * 
 * 
 * def mark(EventAttributes attributes, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainMarkEx(domain.c_obj, &attributes.c_obj)
*/

/* Python wrapper */
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_25mark(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_4nvtx_4_lib_3lib_24mark, "mark(EventAttributes attributes, DomainHandle domain)");
static PyMethodDef __pyx_mdef_4nvtx_4_lib_3lib_25mark = {"mark", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_25mark, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_24mark};
static PyObject *__pyx_pw_4nvtx_4_lib_3lib_25mark(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes = 0;
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("mark (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_attributes,&__pyx_mstate_global->__pyx_n_u_domain,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len < 0)) __PYX_ERR(0, 1211, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1211, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "mark", 0) < (0)) __PYX_ERR(0, 1211, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("mark", 1, 2, 2, i); __PYX_ERR(0, 1211, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1211, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1211, __pyx_L3_error)
    }
    __pyx_v_attributes = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)values[0]);
    __pyx_v_domain = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("mark", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1211, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("nvtx._lib.lib.mark", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_attributes), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, 1, "attributes", 0))) __PYX_ERR(0, 1211, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_domain), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle, 1, "domain", 0))) __PYX_ERR(0, 1211, __pyx_L1_error)
  __pyx_r = __pyx_pf_4nvtx_4_lib_3lib_24mark(__pyx_self, __pyx_v_attributes, __pyx_v_domain);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4nvtx_4_lib_3lib_24mark(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *__pyx_v_attributes, struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *__pyx_v_domain) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("mark", 0);

  /* "nvtx/_lib/lib.pyx":1212
 * 
 * def mark(EventAttributes attributes, DomainHandle domain):
 *     nvtxDomainMarkEx(domain.c_obj, &attributes.c_obj)             # <<<<<<<<<<<<<<
*/
  nvtxDomainMarkEx(__pyx_v_domain->c_obj, (&__pyx_v_attributes->c_obj));

  /* "nvtx/_lib/lib.pyx":1211
 * 
 * 
 * def mark(EventAttributes attributes, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainMarkEx(domain.c_obj, &attributes.c_obj)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */
static struct __pyx_vtabstruct_4nvtx_4_lib_3lib_EventAttributes __pyx_vtable_4nvtx_4_lib_3lib_EventAttributes;

static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib_EventAttributes(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)o);
  p->__pyx_vtab = __pyx_vtabptr_4nvtx_4_lib_3lib_EventAttributes;
  p->domain = Py_None; Py_INCREF(Py_None);
  p->_message = Py_None; Py_INCREF(Py_None);
  p->_color = Py_None; Py_INCREF(Py_None);
  p->_payload = Py_None; Py_INCREF(Py_None);
  p->_payload_object = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_3lib_EventAttributes(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *p = (struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_3lib_EventAttributes) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_DeallocKeepAliveBegin(o);
    __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_1__dealloc__(o);
    __Pyx_DeallocKeepAliveEnd(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->domain);
  Py_CLEAR(p->_message);
  Py_CLEAR(p->_color);
  Py_CLEAR(p->_payload);
  Py_CLEAR(p->_payload_object);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_3lib_EventAttributes(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *p = (struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->domain) {
    e = (*v)(p->domain, a); if (e) return e;
  }
  if (p->_message) {
    e = (*v)(p->_message, a); if (e) return e;
  }
  if (p->_color) {
    e = (*v)(p->_color, a); if (e) return e;
  }
  if (p->_payload) {
    e = (*v)(p->_payload, a); if (e) return e;
  }
  if (p->_payload_object) {
    e = (*v)(p->_payload_object, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_3lib_EventAttributes(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *p = (struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *)o;
  tmp = ((PyObject*)p->domain);
  p->domain = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_message);
  p->_message = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_color);
  p->_color = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_payload);
  p->_payload = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_payload_object);
  p->_payload_object = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_4nvtx_4_lib_3lib_15EventAttributes_message(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7message_1__get__(o);
}

static int __pyx_setprop_4nvtx_4_lib_3lib_15EventAttributes_message(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7message_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_4nvtx_4_lib_3lib_15EventAttributes_color(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5color_1__get__(o);
}

static int __pyx_setprop_4nvtx_4_lib_3lib_15EventAttributes_color(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5color_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_4nvtx_4_lib_3lib_15EventAttributes_category(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_8category_1__get__(o);
}

static int __pyx_setprop_4nvtx_4_lib_3lib_15EventAttributes_category(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_8category_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_4nvtx_4_lib_3lib_15EventAttributes_payload(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7payload_1__get__(o);
}

static int __pyx_setprop_4nvtx_4_lib_3lib_15EventAttributes_payload(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7payload_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_3lib_EventAttributes[] = {
  {"_set_payload_int", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_5_set_payload_int, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_4_set_payload_int},
  {"_set_payload_float", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_7_set_payload_float, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_6_set_payload_float},
  {"_set_payload_numpy", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_9_set_payload_numpy, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_8_set_payload_numpy},
  {"_set_payload_iterable", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_11_set_payload_iterable, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_10_set_payload_iterable},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_13__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_12__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_15__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15EventAttributes_14__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4nvtx_4_lib_3lib_EventAttributes[] = {
  {"message", __pyx_getprop_4nvtx_4_lib_3lib_15EventAttributes_message, __pyx_setprop_4nvtx_4_lib_3lib_15EventAttributes_message, 0, 0},
  {"color", __pyx_getprop_4nvtx_4_lib_3lib_15EventAttributes_color, __pyx_setprop_4nvtx_4_lib_3lib_15EventAttributes_color, 0, 0},
  {"category", __pyx_getprop_4nvtx_4_lib_3lib_15EventAttributes_category, __pyx_setprop_4nvtx_4_lib_3lib_15EventAttributes_category, 0, 0},
  {"payload", __pyx_getprop_4nvtx_4_lib_3lib_15EventAttributes_payload, __pyx_setprop_4nvtx_4_lib_3lib_15EventAttributes_payload, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_3lib_EventAttributes_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_3lib_EventAttributes},
  {Py_tp_doc, (void *)PyDoc_STR("EventAttributes(domain, message=None, color=None, category=None, payload=None)\n\nA wrapper class for ``nvtxEventAttributes_t`` C struct.\nUse :func:`nvtx.Domain.get_event_attributes` to create an instance.\n\nAttributes\n----------\nmessage : RegisteredString\n    A message associated with the event.\n    Retrieved by :func:`nvtx.Domain.get_registered_string`.\ncolor : str or int\n    A color associated with the event.\n    Supports `matplotlib` colors if it is available.\ncategory : int\n    An integer specifying the category within the domain\n    under which the event is scoped.\n    If not set, the event is not associated with a category.\n    Retrieved by :func:`nvtx.Domain.get_category_id`.\npayload : int, float, numpy.ndarray, list, tuple, range, or bytes\n    A value associated with this event. Using payload for large data\n    is more efficient than embedding data in messages.\n    It also produces richer information for analysis by profiling tools.\n\n    .. note:: payloads of type other than ``int`` or ``float`` requires\n              NumPy to be installed (not installed with ``nvtx`` package).")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_3lib_EventAttributes},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_3lib_EventAttributes},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_3lib_EventAttributes},
  {Py_tp_getset, (void *)__pyx_getsets_4nvtx_4_lib_3lib_EventAttributes},
  {Py_tp_init, (void *)__pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_3__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_3lib_EventAttributes},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_3lib_EventAttributes_spec = {
  "nvtx._lib.lib.EventAttributes",
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_3lib_EventAttributes_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_3lib_EventAttributes = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.lib.""EventAttributes", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_3lib_EventAttributes, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("EventAttributes(domain, message=None, color=None, category=None, payload=None)\n\nA wrapper class for ``nvtxEventAttributes_t`` C struct.\nUse :func:`nvtx.Domain.get_event_attributes` to create an instance.\n\nAttributes\n----------\nmessage : RegisteredString\n    A message associated with the event.\n    Retrieved by :func:`nvtx.Domain.get_registered_string`.\ncolor : str or int\n    A color associated with the event.\n    Supports `matplotlib` colors if it is available.\ncategory : int\n    An integer specifying the category within the domain\n    under which the event is scoped.\n    If not set, the event is not associated with a category.\n    Retrieved by :func:`nvtx.Domain.get_category_id`.\npayload : int, float, numpy.ndarray, list, tuple, range, or bytes\n    A value associated with this event. Using payload for large data\n    is more efficient than embedding data in messages.\n    It also produces richer information for analysis by profiling tools.\n\n    .. note:: payloads of type other than ``int`` or ``float`` requires\n              NumPy to be installed (not installed with ``nvtx`` package)."), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_3lib_EventAttributes, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_3lib_EventAttributes, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_3lib_EventAttributes, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4nvtx_4_lib_3lib_EventAttributes, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4nvtx_4_lib_3lib_15EventAttributes_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_3lib_EventAttributes, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar __pyx_vtable_4nvtx_4_lib_3lib_SchemaRegistrar;

static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib_SchemaRegistrar(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)o);
  p->__pyx_vtab = __pyx_vtabptr_4nvtx_4_lib_3lib_SchemaRegistrar;
  p->_dtype_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_array_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_fixed_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_3lib_SchemaRegistrar(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *p = (struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_3lib_SchemaRegistrar) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_dtype_cache);
  Py_CLEAR(p->_array_cache);
  Py_CLEAR(p->_fixed_cache);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_3lib_SchemaRegistrar(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *p = (struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_dtype_cache) {
    e = (*v)(p->_dtype_cache, a); if (e) return e;
  }
  if (p->_array_cache) {
    e = (*v)(p->_array_cache, a); if (e) return e;
  }
  if (p->_fixed_cache) {
    e = (*v)(p->_fixed_cache, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_3lib_SchemaRegistrar(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *p = (struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *)o;
  tmp = ((PyObject*)p->_dtype_cache);
  p->_dtype_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_array_cache);
  p->_array_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_fixed_cache);
  p->_fixed_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_3lib_SchemaRegistrar[] = {
  {"_register_builtin_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_3_register_builtin_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_2_register_builtin_schema},
  {"_register_structured_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_5_register_structured_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_4_register_structured_schema},
  {"_get_numpy_array_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_7_get_numpy_array_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_6_get_numpy_array_schema},
  {"_get_fixed_size_array_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_9_get_fixed_size_array_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_8_get_fixed_size_array_schema},
  {"_get_numpy_dtype_schema", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_11_get_numpy_dtype_schema, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_10_get_numpy_dtype_schema},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_13__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_12__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_15SchemaRegistrar_15__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_15SchemaRegistrar_14__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_3lib_SchemaRegistrar},
  {Py_tp_doc, (void *)PyDoc_STR("\n    Builds payload schemas from NumPy dtypes and registers them, caching so\n    each distinct schema registers once. Subclasses override :meth:`_do_register`\n    to supply the register target (global ``nvtxPayloadSchemaRegister`` for\n    instrumentation, ``SchemaRegister`` for the NVTXW writer).\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_3lib_SchemaRegistrar},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_3lib_SchemaRegistrar},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_3lib_SchemaRegistrar},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_3lib_SchemaRegistrar},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar_spec = {
  "nvtx._lib.lib.SchemaRegistrar",
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.lib.""SchemaRegistrar", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_3lib_SchemaRegistrar, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("\n    Builds payload schemas from NumPy dtypes and registers them, caching so\n    each distinct schema registers once. Subclasses override :meth:`_do_register`\n    to supply the register target (global ``nvtxPayloadSchemaRegister`` for\n    instrumentation, ``SchemaRegister`` for the NVTXW writer).\n    "), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_3lib_SchemaRegistrar, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_3lib_SchemaRegistrar, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_3lib_SchemaRegistrar, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_3lib_SchemaRegistrar, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib_DomainHandle(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)o);
  p->_name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_3lib_DomainHandle(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *p = (struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_3lib_DomainHandle) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_DeallocKeepAliveBegin(o);
    __pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_5__dealloc__(o);
    __Pyx_DeallocKeepAliveEnd(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->_name);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static PyObject *__pyx_getprop_4nvtx_4_lib_3lib_12DomainHandle_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_4name_1__get__(o);
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_3lib_DomainHandle[] = {
  {"enabled", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_3enabled, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12DomainHandle_2enabled},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_7__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12DomainHandle_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_9__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12DomainHandle_8__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4nvtx_4_lib_3lib_DomainHandle[] = {
  {"name", __pyx_getprop_4nvtx_4_lib_3lib_12DomainHandle_name, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_3lib_DomainHandle_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_3lib_DomainHandle},
  {Py_tp_doc, (void *)PyDoc_STR("DomainHandle(name=None)")},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_3lib_DomainHandle},
  {Py_tp_getset, (void *)__pyx_getsets_4nvtx_4_lib_3lib_DomainHandle},
  {Py_tp_init, (void *)__pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_3lib_DomainHandle},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_3lib_DomainHandle_spec = {
  "nvtx._lib.lib.DomainHandle",
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_4nvtx_4_lib_3lib_DomainHandle_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_3lib_DomainHandle = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.lib.""DomainHandle", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_DomainHandle), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_3lib_DomainHandle, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("DomainHandle(name=None)"), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_3lib_DomainHandle, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4nvtx_4_lib_3lib_DomainHandle, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4nvtx_4_lib_3lib_12DomainHandle_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_3lib_DomainHandle, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib_StringHandle(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *)o);
  p->_string = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_3lib_StringHandle(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *p = (struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && (!PyType_IS_GC(Py_TYPE(o)) || !__Pyx_PyObject_GC_IsFinalized(o))) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_3lib_StringHandle) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  Py_CLEAR(p->_string);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static PyObject *__pyx_getprop_4nvtx_4_lib_3lib_12StringHandle_string(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_4nvtx_4_lib_3lib_12StringHandle_6string_1__get__(o);
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_3lib_StringHandle[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12StringHandle_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_12StringHandle_4__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_4nvtx_4_lib_3lib_StringHandle[] = {
  {"string", __pyx_getprop_4nvtx_4_lib_3lib_12StringHandle_string, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_3lib_StringHandle_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_3lib_StringHandle},
  {Py_tp_doc, (void *)PyDoc_STR("StringHandle(DomainHandle domain_handle, string=None)")},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_3lib_StringHandle},
  {Py_tp_getset, (void *)__pyx_getsets_4nvtx_4_lib_3lib_StringHandle},
  {Py_tp_init, (void *)__pyx_pw_4nvtx_4_lib_3lib_12StringHandle_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_3lib_StringHandle},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_3lib_StringHandle_spec = {
  "nvtx._lib.lib.StringHandle",
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE,
  __pyx_type_4nvtx_4_lib_3lib_StringHandle_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_3lib_StringHandle = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.lib.""StringHandle", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib_StringHandle), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_3lib_StringHandle, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  PyDoc_STR("StringHandle(DomainHandle domain_handle, string=None)"), /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_3lib_StringHandle, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_4nvtx_4_lib_3lib_StringHandle, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4nvtx_4_lib_3lib_12StringHandle_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_3lib_StringHandle, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_4nvtx_4_lib_3lib__GlobalSchemaRegistrar __pyx_vtable_4nvtx_4_lib_3lib__GlobalSchemaRegistrar;

static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib__GlobalSchemaRegistrar(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *p;
  PyObject *o = __pyx_tp_new_4nvtx_4_lib_3lib_SchemaRegistrar(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_4nvtx_4_lib_3lib_SchemaRegistrar*)__pyx_vtabptr_4nvtx_4_lib_3lib__GlobalSchemaRegistrar;
  return o;
}

static PyMethodDef __pyx_methods_4nvtx_4_lib_3lib__GlobalSchemaRegistrar[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_3__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_5__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_4__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_3lib_SchemaRegistrar},
  {Py_tp_doc, (void *)PyDoc_STR("_GlobalSchemaRegistrar(DomainHandle handle)\n\nRegisters schemas via the global ``nvtxPayloadSchemaRegister``.")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_3lib_SchemaRegistrar},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_3lib_SchemaRegistrar},
  {Py_tp_methods, (void *)__pyx_methods_4nvtx_4_lib_3lib__GlobalSchemaRegistrar},
  {Py_tp_init, (void *)__pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_3lib__GlobalSchemaRegistrar},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar_spec = {
  "nvtx._lib.lib._GlobalSchemaRegistrar",
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.lib.""_GlobalSchemaRegistrar", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib__GlobalSchemaRegistrar), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_3lib_SchemaRegistrar, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("_GlobalSchemaRegistrar(DomainHandle handle)\n\nRegisters schemas via the global ``nvtxPayloadSchemaRegister``."), /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_3lib_SchemaRegistrar, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_3lib_SchemaRegistrar, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4nvtx_4_lib_3lib__GlobalSchemaRegistrar, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_3lib__GlobalSchemaRegistrar, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib___pyx_defaults(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 1);
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_defaults(PyObject *o) {
  struct __pyx_defaults *p = (struct __pyx_defaults *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_defaults) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->arg0);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_3lib___pyx_defaults(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_defaults *p = (struct __pyx_defaults *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->arg0) {
    e = (*v)(p->arg0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_3lib___pyx_defaults(PyObject *o) {
  PyObject* tmp;
  struct __pyx_defaults *p = (struct __pyx_defaults *)o;
  tmp = ((PyObject*)p->arg0);
  p->arg0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_3lib___pyx_defaults_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_defaults},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_3lib___pyx_defaults},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_3lib___pyx_defaults},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_3lib___pyx_defaults},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_3lib___pyx_defaults_spec = {
  "nvtx._lib.lib.__pyx_defaults",
  sizeof(struct __pyx_defaults),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_3lib___pyx_defaults_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_3lib___pyx_defaults = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.lib.""__pyx_defaults", /*tp_name*/
  sizeof(struct __pyx_defaults), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_defaults, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_3lib___pyx_defaults, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_3lib___pyx_defaults, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_3lib___pyx_defaults, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib___pyx_defaults1(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 1);
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_defaults1(PyObject *o) {
  struct __pyx_defaults1 *p = (struct __pyx_defaults1 *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_defaults1) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->arg0);
  Py_CLEAR(p->arg1);
  Py_CLEAR(p->arg2);
  Py_CLEAR(p->arg3);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_4nvtx_4_lib_3lib___pyx_defaults1(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_defaults1 *p = (struct __pyx_defaults1 *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->arg0) {
    e = (*v)(p->arg0, a); if (e) return e;
  }
  if (p->arg1) {
    e = (*v)(p->arg1, a); if (e) return e;
  }
  if (p->arg2) {
    e = (*v)(p->arg2, a); if (e) return e;
  }
  if (p->arg3) {
    e = (*v)(p->arg3, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_3lib___pyx_defaults1(PyObject *o) {
  PyObject* tmp;
  struct __pyx_defaults1 *p = (struct __pyx_defaults1 *)o;
  tmp = ((PyObject*)p->arg0);
  p->arg0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->arg1);
  p->arg1 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->arg2);
  p->arg2 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->arg3);
  p->arg3 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_3lib___pyx_defaults1_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_defaults1},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_3lib___pyx_defaults1},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_3lib___pyx_defaults1},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_3lib___pyx_defaults1},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_3lib___pyx_defaults1_spec = {
  "nvtx._lib.lib.__pyx_defaults1",
  sizeof(struct __pyx_defaults1),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_3lib___pyx_defaults1_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_3lib___pyx_defaults1 = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.lib.""__pyx_defaults1", /*tp_name*/
  sizeof(struct __pyx_defaults1), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_defaults1, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_3lib___pyx_defaults1, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_3lib___pyx_defaults1, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_3lib___pyx_defaults1, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter, sizeof(struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter[--__pyx_mstate_global->__pyx_freecount_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter(PyObject *o) {
  struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *p = (struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_type);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter, sizeof(struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter))))
  {
    __pyx_mstate_global->__pyx_freelist_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter[__pyx_mstate_global->__pyx_freecount_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter++] = ((struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *p = (struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_type) {
    e = (*v)(p->__pyx_v_type, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *p = (struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter *)o;
  tmp = ((PyObject*)p->__pyx_v_type);
  p->__pyx_v_type = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter},
  {Py_tp_clear, (void *)__pyx_tp_clear_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter},
  {Py_tp_new, (void *)__pyx_tp_new_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter},
  {0, 0},
};
static PyType_Spec __pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter_spec = {
  "nvtx._lib.lib.__pyx_scope_struct__payload_setter",
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter_slots,
};
#else

static PyTypeObject __pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter = {
  PyVarObject_HEAD_INIT(0, 0)
  "nvtx._lib.lib.""__pyx_scope_struct__payload_setter", /*tp_name*/
  sizeof(struct __pyx_obj_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter, /*tp_traverse*/
  __pyx_tp_clear_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  {
    __pyx_t_1 = __Pyx_ApiExport_GetApiDict(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_export_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_PyObject_int___pyx_skip);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_export_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_export_name = __pyx_export_signature + 48;
    void (*const __pyx_export_pointers[])(void) = {(void (*)(void))&__pyx_f_4nvtx_4_lib_3lib__to_bytes, (void (*)(void)) NULL};
    void (*const *__pyx_export_pointer)(void) = __pyx_export_pointers;
    const char *__pyx_export_current_signature = __pyx_export_signature;
    while (*__pyx_export_pointer) {
      if (__Pyx_ExportFunction(__pyx_t_1, __pyx_export_name, *__pyx_export_pointer, __pyx_export_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_export_pointer;
      __pyx_export_name = strchr(__pyx_export_name, '\0') + 1;
      __pyx_export_signature = strchr(__pyx_export_signature, '\0') + 1;
      if (*__pyx_export_signature != '\0') __pyx_export_current_signature = __pyx_export_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_4nvtx_4_lib_3lib_EventAttributes = &__pyx_vtable_4nvtx_4_lib_3lib_EventAttributes;
  __pyx_vtable_4nvtx_4_lib_3lib_EventAttributes._set_binary_payload = (PyObject *(*)(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *, void *, uint64_t, size_t, size_t))__pyx_f_4nvtx_4_lib_3lib_15EventAttributes__set_binary_payload;
  __pyx_vtable_4nvtx_4_lib_3lib_EventAttributes._clear_payload = (PyObject *(*)(struct __pyx_obj_4nvtx_4_lib_3lib_EventAttributes *))__pyx_f_4nvtx_4_lib_3lib_15EventAttributes__clear_payload;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_3lib_EventAttributes_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes)) __PYX_ERR(0, 252, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_3lib_EventAttributes_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes) < (0)) __PYX_ERR(0, 252, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes = &__pyx_type_4nvtx_4_lib_3lib_EventAttributes;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes) < (0)) __PYX_ERR(0, 252, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_vtabptr_4nvtx_4_lib_3lib_EventAttributes) < (0)) __PYX_ERR(0, 252, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes) < (0)) __PYX_ERR(0, 252, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_EventAttributes, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes) < (0)) __PYX_ERR(0, 252, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes) < (0)) __PYX_ERR(0, 252, __pyx_L1_error)
  __pyx_vtabptr_4nvtx_4_lib_3lib_SchemaRegistrar = &__pyx_vtable_4nvtx_4_lib_3lib_SchemaRegistrar;
  __pyx_vtable_4nvtx_4_lib_3lib_SchemaRegistrar._do_register = (uint64_t (*)(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *, nvtxPayloadSchemaAttr_t const *))__pyx_f_4nvtx_4_lib_3lib_15SchemaRegistrar__do_register;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar)) __PYX_ERR(0, 508, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar) < (0)) __PYX_ERR(0, 508, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar = &__pyx_type_4nvtx_4_lib_3lib_SchemaRegistrar;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar) < (0)) __PYX_ERR(0, 508, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, __pyx_vtabptr_4nvtx_4_lib_3lib_SchemaRegistrar) < (0)) __PYX_ERR(0, 508, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar) < (0)) __PYX_ERR(0, 508, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_SchemaRegistrar, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar) < (0)) __PYX_ERR(0, 508, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar) < (0)) __PYX_ERR(0, 508, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_3lib_DomainHandle_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle)) __PYX_ERR(0, 414, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_3lib_DomainHandle_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle) < (0)) __PYX_ERR(0, 414, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle = &__pyx_type_4nvtx_4_lib_3lib_DomainHandle;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle) < (0)) __PYX_ERR(0, 414, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_DomainHandle, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle) < (0)) __PYX_ERR(0, 414, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle) < (0)) __PYX_ERR(0, 414, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_3lib_StringHandle_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle)) __PYX_ERR(0, 1178, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_3lib_StringHandle_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle = &__pyx_type_4nvtx_4_lib_3lib_StringHandle;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_StringHandle, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib_StringHandle) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
  __pyx_vtabptr_4nvtx_4_lib_3lib__GlobalSchemaRegistrar = &__pyx_vtable_4nvtx_4_lib_3lib__GlobalSchemaRegistrar;
  __pyx_vtable_4nvtx_4_lib_3lib__GlobalSchemaRegistrar.__pyx_base = *__pyx_vtabptr_4nvtx_4_lib_3lib_SchemaRegistrar;
  __pyx_vtable_4nvtx_4_lib_3lib__GlobalSchemaRegistrar.__pyx_base._do_register = (uint64_t (*)(struct __pyx_obj_4nvtx_4_lib_3lib_SchemaRegistrar *, nvtxPayloadSchemaAttr_t const *))__pyx_f_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar__do_register;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 766, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar)) __PYX_ERR(0, 766, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar) < (0)) __PYX_ERR(0, 766, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar = &__pyx_type_4nvtx_4_lib_3lib__GlobalSchemaRegistrar;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar->tp_base = __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar) < (0)) __PYX_ERR(0, 766, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar, __pyx_vtabptr_4nvtx_4_lib_3lib__GlobalSchemaRegistrar) < (0)) __PYX_ERR(0, 766, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar) < (0)) __PYX_ERR(0, 766, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_GlobalSchemaRegistrar, (PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar) < (0)) __PYX_ERR(0, 766, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib__GlobalSchemaRegistrar) < (0)) __PYX_ERR(0, 766, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_3lib___pyx_defaults_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults)) __PYX_ERR(0, 484, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_3lib___pyx_defaults_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults) < (0)) __PYX_ERR(0, 484, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults = &__pyx_type_4nvtx_4_lib_3lib___pyx_defaults;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults) < (0)) __PYX_ERR(0, 484, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1 = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_3lib___pyx_defaults1_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1)) __PYX_ERR(0, 866, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_3lib___pyx_defaults1_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1) < (0)) __PYX_ERR(0, 866, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1 = &__pyx_type_4nvtx_4_lib_3lib___pyx_defaults1;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1) < (0)) __PYX_ERR(0, 866, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter)) __PYX_ERR(0, 242, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter_spec, __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter) < (0)) __PYX_ERR(0, 242, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter = &__pyx_type_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter) < (0)) __PYX_ERR(0, 242, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter->tp_dictoffset && __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_4nvtx_4_lib_3lib___pyx_scope_struct__payload_setter->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("nvtx._lib.counters"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 112, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_4nvtx_4_lib_8counters_Counter = __Pyx_ImportType_3_2_9(__pyx_t_1, "nvtx._lib.counters", "Counter",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_4nvtx_4_lib_8counters_Counter), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_8counters_Counter),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_4nvtx_4_lib_8counters_Counter), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_8counters_Counter),
  #else
  sizeof(struct __pyx_obj_4nvtx_4_lib_8counters_Counter), __PYX_GET_STRUCT_ALIGNMENT_3_2_9(struct __pyx_obj_4nvtx_4_lib_8counters_Counter),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_9); if (!__pyx_mstate->__pyx_ptype_4nvtx_4_lib_8counters_Counter) __PYX_ERR(2, 112, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  {
    __pyx_t_1 = PyImport_ImportModule("nvtx._lib.counters"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_void_nvtxSemanticsCounter_t_PyOb);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 44;
    void (**const __pyx_import_pointers[])(void) = {(void (**)(void))&__pyx_f_4nvtx_4_lib_8counters__fill_counter_semantics, (void (**)(void)) NULL};
    void (**const *__pyx_import_pointer)(void) = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportFunction_3_2_9(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_lib(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_lib},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, __Pyx_FREETHREADING_COMPATIBLE},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "lib",
      0, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit_lib(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_lib(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
    PY_INT64_T current_id = GraalPyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_lib(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  size_t __pyx_t_12;
  int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'lib' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "lib" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_NOT_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
    PyErr_Clear();
    __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
    if (!__Pyx_RefNanny)
        Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  
__Pyx_RefNannySetupContext("PyInit_lib", 0);
  __Pyx_init_runtime_version();
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Library function declarations ---*/
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__pyx_module_is_main_nvtx___lib__lib) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_main) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "nvtx._lib.lib")) {
      if (unlikely((PyDict_SetItemString(modules, "nvtx._lib.lib", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  (void)__Pyx_modinit_variable_export_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_function_export_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_init_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_function_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/

  /* "nvtx/_lib/lib.pyx":19
 * # See https://nvidia.github.io/NVTX/LICENSE.txt for license information.
 * 
 * import enum             # <<<<<<<<<<<<<<
 * import warnings
 * from nvtx._lib.counters import (
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_enum, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_enum, __pyx_t_2) < (0)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":20
 * 
 * import enum
 * import warnings             # <<<<<<<<<<<<<<
 * from nvtx._lib.counters import (
 *     Counter,
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_warnings, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_warnings, __pyx_t_2) < (0)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":21
 * import enum
 * import warnings
 * from nvtx._lib.counters import (             # <<<<<<<<<<<<<<
 *     Counter,
 *     DummyCounter,
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Counter,__pyx_mstate_global->__pyx_n_u_DummyCounter,__pyx_mstate_global->__pyx_n_u_counter_class_from_dtype,__pyx_mstate_global->__pyx_n_u_dummy_counter};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_nvtx__lib_counters, __pyx_imported_names, 4, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Counter,__pyx_mstate_global->__pyx_n_u_DummyCounter,__pyx_mstate_global->__pyx_n_u_counter_class_from_dtype,__pyx_mstate_global->__pyx_n_u_dummy_counter};
    for (__pyx_t_3=0; __pyx_t_3 < 4; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 21, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 21, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":27
 *     dummy_counter,
 * )
 * from functools import lru_cache             # <<<<<<<<<<<<<<
 * from libc.stdlib cimport malloc, free
 * from libc.string cimport memcpy
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_lru_cache};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_functools, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_lru_cache};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 27, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":36
 * from nvtx._lib.time cimport nvtxTimestampGet
 * from nvtx._lib.lib cimport *
 * from nvtx.colors import color_to_hex             # <<<<<<<<<<<<<<
 * from nvtx._lib.time import TimestampType
 * from nvtx._metadata import PayloadSchemaKey
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_color_to_hex};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_nvtx_colors, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_color_to_hex};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 36, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 36, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":37
 * from nvtx._lib.lib cimport *
 * from nvtx.colors import color_to_hex
 * from nvtx._lib.time import TimestampType             # <<<<<<<<<<<<<<
 * from nvtx._metadata import PayloadSchemaKey
 * 
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_TimestampType};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_nvtx__lib_time, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_TimestampType};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 37, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 37, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":38
 * from nvtx.colors import color_to_hex
 * from nvtx._lib.time import TimestampType
 * from nvtx._metadata import PayloadSchemaKey             # <<<<<<<<<<<<<<
 * 
 * from typing import Optional
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_PayloadSchemaKey};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_nvtx__metadata, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_PayloadSchemaKey};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 38, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 38, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":40
 * from nvtx._metadata import PayloadSchemaKey
 * 
 * from typing import Optional             # <<<<<<<<<<<<<<
 * 
 * try:
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Optional};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_typing, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Optional};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 40, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 40, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":42
 * from typing import Optional
 * 
 * try:             # <<<<<<<<<<<<<<
 *     import numpy as np
 *     _dtype_to_entry_type = {
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {

      /* "nvtx/_lib/lib.pyx":43
 * 
 * try:
 *     import numpy as np             # <<<<<<<<<<<<<<
 *     _dtype_to_entry_type = {
 *         np.int8: NVTX_PAYLOAD_ENTRY_TYPE_INT8,
*/
      __pyx_t_7 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_numpy, 0, 0, NULL, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 43, __pyx_L2_error)
      __pyx_t_2 = __pyx_t_7;
      __Pyx_GOTREF(__pyx_t_2);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_np, __pyx_t_2) < (0)) __PYX_ERR(0, 43, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "nvtx/_lib/lib.pyx":45
 *     import numpy as np
 *     _dtype_to_entry_type = {
 *         np.int8: NVTX_PAYLOAD_ENTRY_TYPE_INT8,             # <<<<<<<<<<<<<<
 *         np.int16: NVTX_PAYLOAD_ENTRY_TYPE_INT16,
 *         np.int32: NVTX_PAYLOAD_ENTRY_TYPE_INT32,
*/
      __pyx_t_2 = __Pyx_PyDict_NewPresized(13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_INT8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":46
 *     _dtype_to_entry_type = {
 *         np.int8: NVTX_PAYLOAD_ENTRY_TYPE_INT8,
 *         np.int16: NVTX_PAYLOAD_ENTRY_TYPE_INT16,             # <<<<<<<<<<<<<<
 *         np.int32: NVTX_PAYLOAD_ENTRY_TYPE_INT32,
 *         np.int64: NVTX_PAYLOAD_ENTRY_TYPE_INT64,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 46, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int16); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 46, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_INT16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 46, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":47
 *         np.int8: NVTX_PAYLOAD_ENTRY_TYPE_INT8,
 *         np.int16: NVTX_PAYLOAD_ENTRY_TYPE_INT16,
 *         np.int32: NVTX_PAYLOAD_ENTRY_TYPE_INT32,             # <<<<<<<<<<<<<<
 *         np.int64: NVTX_PAYLOAD_ENTRY_TYPE_INT64,
 *         np.uint8: NVTX_PAYLOAD_ENTRY_TYPE_UINT8,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int32); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 47, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_INT32); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 47, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":48
 *         np.int16: NVTX_PAYLOAD_ENTRY_TYPE_INT16,
 *         np.int32: NVTX_PAYLOAD_ENTRY_TYPE_INT32,
 *         np.int64: NVTX_PAYLOAD_ENTRY_TYPE_INT64,             # <<<<<<<<<<<<<<
 *         np.uint8: NVTX_PAYLOAD_ENTRY_TYPE_UINT8,
 *         np.uint16: NVTX_PAYLOAD_ENTRY_TYPE_UINT16,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_int64); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 48, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_INT64); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 48, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":49
 *         np.int32: NVTX_PAYLOAD_ENTRY_TYPE_INT32,
 *         np.int64: NVTX_PAYLOAD_ENTRY_TYPE_INT64,
 *         np.uint8: NVTX_PAYLOAD_ENTRY_TYPE_UINT8,             # <<<<<<<<<<<<<<
 *         np.uint16: NVTX_PAYLOAD_ENTRY_TYPE_UINT16,
 *         np.uint32: NVTX_PAYLOAD_ENTRY_TYPE_UINT32,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 49, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_uint8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 49, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_UINT8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 49, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":50
 *         np.int64: NVTX_PAYLOAD_ENTRY_TYPE_INT64,
 *         np.uint8: NVTX_PAYLOAD_ENTRY_TYPE_UINT8,
 *         np.uint16: NVTX_PAYLOAD_ENTRY_TYPE_UINT16,             # <<<<<<<<<<<<<<
 *         np.uint32: NVTX_PAYLOAD_ENTRY_TYPE_UINT32,
 *         np.uint64: NVTX_PAYLOAD_ENTRY_TYPE_UINT64,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 50, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_uint16); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 50, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_UINT16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 50, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":51
 *         np.uint8: NVTX_PAYLOAD_ENTRY_TYPE_UINT8,
 *         np.uint16: NVTX_PAYLOAD_ENTRY_TYPE_UINT16,
 *         np.uint32: NVTX_PAYLOAD_ENTRY_TYPE_UINT32,             # <<<<<<<<<<<<<<
 *         np.uint64: NVTX_PAYLOAD_ENTRY_TYPE_UINT64,
 *         np.float16: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT16,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 51, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_uint32); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 51, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_UINT32); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 51, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":52
 *         np.uint16: NVTX_PAYLOAD_ENTRY_TYPE_UINT16,
 *         np.uint32: NVTX_PAYLOAD_ENTRY_TYPE_UINT32,
 *         np.uint64: NVTX_PAYLOAD_ENTRY_TYPE_UINT64,             # <<<<<<<<<<<<<<
 *         np.float16: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT16,
 *         np.float32: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT32,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 52, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_uint64); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 52, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_UINT64); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 52, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":53
 *         np.uint32: NVTX_PAYLOAD_ENTRY_TYPE_UINT32,
 *         np.uint64: NVTX_PAYLOAD_ENTRY_TYPE_UINT64,
 *         np.float16: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT16,             # <<<<<<<<<<<<<<
 *         np.float32: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT32,
 *         np.float64: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 53, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_float16); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 53, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_FLOAT16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 53, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":54
 *         np.uint64: NVTX_PAYLOAD_ENTRY_TYPE_UINT64,
 *         np.float16: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT16,
 *         np.float32: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT32,             # <<<<<<<<<<<<<<
 *         np.float64: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64,
 *         np.str_: NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 54, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_float32); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 54, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_FLOAT32); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 54, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":55
 *         np.float16: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT16,
 *         np.float32: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT32,
 *         np.float64: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64,             # <<<<<<<<<<<<<<
 *         np.str_: NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32,
 *         np.bytes_: NVTX_PAYLOAD_ENTRY_TYPE_BYTE,
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 55, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_float64); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 55, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 55, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":56
 *         np.float32: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT32,
 *         np.float64: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64,
 *         np.str_: NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32,             # <<<<<<<<<<<<<<
 *         np.bytes_: NVTX_PAYLOAD_ENTRY_TYPE_BYTE,
 *     }
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 56, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_str); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 56, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 56, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "nvtx/_lib/lib.pyx":57
 *         np.float64: NVTX_PAYLOAD_ENTRY_TYPE_FLOAT64,
 *         np.str_: NVTX_PAYLOAD_ENTRY_TYPE_CSTRING_UTF32,
 *         np.bytes_: NVTX_PAYLOAD_ENTRY_TYPE_BYTE,             # <<<<<<<<<<<<<<
 *     }
 * except ImportError:
*/
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 57, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_bytes); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 57, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_BYTE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 57, __pyx_L2_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_t_2, __pyx_t_8, __pyx_t_4) < (0)) __PYX_ERR(0, 45, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_dtype_to_entry_type, __pyx_t_2) < (0)) __PYX_ERR(0, 44, __pyx_L2_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "nvtx/_lib/lib.pyx":42
 * from typing import Optional
 * 
 * try:             # <<<<<<<<<<<<<<
 *     import numpy as np
 *     _dtype_to_entry_type = {
*/
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L7_try_end;
    __pyx_L2_error:;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "nvtx/_lib/lib.pyx":59
 *         np.bytes_: NVTX_PAYLOAD_ENTRY_TYPE_BYTE,
 *     }
 * except ImportError:             # <<<<<<<<<<<<<<
 *     np = None
 * 
*/
    __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_ImportError))));
    if (__pyx_t_9) {
      __Pyx_AddTraceback("nvtx._lib.lib", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_8) < 0) __PYX_ERR(0, 59, __pyx_L4_except_error)
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_8);

      /* "nvtx/_lib/lib.pyx":60
 *     }
 * except ImportError:
 *     np = None             # <<<<<<<<<<<<<<
 * 
 * 
*/
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_np, Py_None) < (0)) __PYX_ERR(0, 60, __pyx_L4_except_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L3_exception_handled;
    }
    goto __pyx_L4_except_error;

    /* "nvtx/_lib/lib.pyx":42
 * from typing import Optional
 * 
 * try:             # <<<<<<<<<<<<<<
 *     import numpy as np
 *     _dtype_to_entry_type = {
*/
    __pyx_L4_except_error:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L3_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_5, __pyx_t_6);
    __pyx_L7_try_end:;
  }

  /* "nvtx/_lib/lib.pyx":63
 * 
 * 
 * cpdef bytes _to_bytes(object s):             # <<<<<<<<<<<<<<
 *     return s if isinstance(s, bytes) else s.encode()
 * 
*/
  __pyx_t_8 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_1_to_bytes, 0, __pyx_mstate_global->__pyx_n_u_to_bytes, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_8);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_to_bytes, __pyx_t_8) < (0)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":66
 *     return s if isinstance(s, bytes) else s.encode()
 * 
 * def initialize():             # <<<<<<<<<<<<<<
 *     nvtxInitialize(NULL)
 * 
*/
  __pyx_t_8 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_3initialize, 0, __pyx_mstate_global->__pyx_n_u_initialize, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_8);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_initialize, __pyx_t_8) < (0)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":70
 * 
 * 
 * class NvtxWarning(UserWarning):             # <<<<<<<<<<<<<<
 *     pass
 * 
*/
  __pyx_t_8 = PyTuple_Pack(1, ((PyObject *)(((PyTypeObject*)PyExc_UserWarning)))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PEP560_update_bases(__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_4, __pyx_mstate_global->__pyx_n_u_NvtxWarning, __pyx_mstate_global->__pyx_n_u_NvtxWarning, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, (PyObject *) NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__pyx_t_4 != __pyx_t_8) {
    if (unlikely((PyDict_SetItemString(__pyx_t_10, "__orig_bases__", __pyx_t_8) < 0))) __PYX_ERR(0, 70, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_NvtxWarning, __pyx_t_4, __pyx_t_10, NULL, 0, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_8);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_NvtxWarning, __pyx_t_8) < (0)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":74
 * 
 * 
 * def _validate_counter_group_dtype(dtype):             # <<<<<<<<<<<<<<
 *     if np is None:
 *         if dtype is int or dtype is float:
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_5_validate_counter_group_dtype, 0, __pyx_mstate_global->__pyx_n_u_validate_counter_group_dtype, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_validate_counter_group_dtype, __pyx_t_4) < (0)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":104
 * 
 * 
 * def _normalize_counter_dtype(dtype):             # <<<<<<<<<<<<<<
 *     """
 *     Normalize dtype-like counter specifications before caching.
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_7_normalize_counter_dtype, 0, __pyx_mstate_global->__pyx_n_u_normalize_counter_dtype, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[4])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_normalize_counter_dtype, __pyx_t_4) < (0)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":122
 * 
 * 
 * def _counter_semantics_from_metadata(metadata):             # <<<<<<<<<<<<<<
 *     """
 *     Return counter semantics stored in NVTX dtype metadata, if present.
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_9_counter_semantics_from_metadata, 0, __pyx_mstate_global->__pyx_n_u_counter_semantics_from_metadata, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[5])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_counter_semantics_from_metadata, __pyx_t_4) < (0)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":132
 * 
 * 
 * class PredefinedScope(enum.Enum):             # <<<<<<<<<<<<<<
 *     """Predefined NVTX execution scopes."""
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PEP560_update_bases(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_8 = __Pyx_Py3MetaclassPrepare(__pyx_t_10, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_PredefinedScope, __pyx_mstate_global->__pyx_n_u_PredefinedScope, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_kp_u_Predefined_NVTX_execution_scopes); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (__pyx_t_2 != __pyx_t_4) {
    if (unlikely((PyDict_SetItemString(__pyx_t_8, "__orig_bases__", __pyx_t_4) < 0))) __PYX_ERR(0, 132, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":135
 *     """Predefined NVTX execution scopes."""
 * 
 *     NONE = NVTX_SCOPE_NONE             # <<<<<<<<<<<<<<
 *     ROOT = NVTX_SCOPE_ROOT
 *     CURRENT_HW_MACHINE = NVTX_SCOPE_CURRENT_HW_MACHINE
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_NONE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_NONE, __pyx_t_4) < (0)) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":136
 * 
 *     NONE = NVTX_SCOPE_NONE
 *     ROOT = NVTX_SCOPE_ROOT             # <<<<<<<<<<<<<<
 *     CURRENT_HW_MACHINE = NVTX_SCOPE_CURRENT_HW_MACHINE
 *     CURRENT_HW_SOCKET = NVTX_SCOPE_CURRENT_HW_SOCKET
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_ROOT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_ROOT, __pyx_t_4) < (0)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":137
 *     NONE = NVTX_SCOPE_NONE
 *     ROOT = NVTX_SCOPE_ROOT
 *     CURRENT_HW_MACHINE = NVTX_SCOPE_CURRENT_HW_MACHINE             # <<<<<<<<<<<<<<
 *     CURRENT_HW_SOCKET = NVTX_SCOPE_CURRENT_HW_SOCKET
 *     CURRENT_HW_CPU_PHYSICAL = NVTX_SCOPE_CURRENT_HW_CPU_PHYSICAL
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_HW_MACHINE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_HW_MACHINE, __pyx_t_4) < (0)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":138
 *     ROOT = NVTX_SCOPE_ROOT
 *     CURRENT_HW_MACHINE = NVTX_SCOPE_CURRENT_HW_MACHINE
 *     CURRENT_HW_SOCKET = NVTX_SCOPE_CURRENT_HW_SOCKET             # <<<<<<<<<<<<<<
 *     CURRENT_HW_CPU_PHYSICAL = NVTX_SCOPE_CURRENT_HW_CPU_PHYSICAL
 *     CURRENT_HW_CPU_LOGICAL = NVTX_SCOPE_CURRENT_HW_CPU_LOGICAL
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_HW_SOCKET); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_HW_SOCKET, __pyx_t_4) < (0)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":139
 *     CURRENT_HW_MACHINE = NVTX_SCOPE_CURRENT_HW_MACHINE
 *     CURRENT_HW_SOCKET = NVTX_SCOPE_CURRENT_HW_SOCKET
 *     CURRENT_HW_CPU_PHYSICAL = NVTX_SCOPE_CURRENT_HW_CPU_PHYSICAL             # <<<<<<<<<<<<<<
 *     CURRENT_HW_CPU_LOGICAL = NVTX_SCOPE_CURRENT_HW_CPU_LOGICAL
 *     CURRENT_HW_INNERMOST = NVTX_SCOPE_CURRENT_HW_INNERMOST
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_HW_CPU_PHYSICAL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_HW_CPU_PHYSICAL, __pyx_t_4) < (0)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":140
 *     CURRENT_HW_SOCKET = NVTX_SCOPE_CURRENT_HW_SOCKET
 *     CURRENT_HW_CPU_PHYSICAL = NVTX_SCOPE_CURRENT_HW_CPU_PHYSICAL
 *     CURRENT_HW_CPU_LOGICAL = NVTX_SCOPE_CURRENT_HW_CPU_LOGICAL             # <<<<<<<<<<<<<<
 *     CURRENT_HW_INNERMOST = NVTX_SCOPE_CURRENT_HW_INNERMOST
 *     CURRENT_HYPERVISOR = NVTX_SCOPE_CURRENT_HYPERVISOR
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_HW_CPU_LOGICAL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_HW_CPU_LOGICAL, __pyx_t_4) < (0)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":141
 *     CURRENT_HW_CPU_PHYSICAL = NVTX_SCOPE_CURRENT_HW_CPU_PHYSICAL
 *     CURRENT_HW_CPU_LOGICAL = NVTX_SCOPE_CURRENT_HW_CPU_LOGICAL
 *     CURRENT_HW_INNERMOST = NVTX_SCOPE_CURRENT_HW_INNERMOST             # <<<<<<<<<<<<<<
 *     CURRENT_HYPERVISOR = NVTX_SCOPE_CURRENT_HYPERVISOR
 *     CURRENT_VM = NVTX_SCOPE_CURRENT_VM
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_HW_INNERMOST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_HW_INNERMOST, __pyx_t_4) < (0)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":142
 *     CURRENT_HW_CPU_LOGICAL = NVTX_SCOPE_CURRENT_HW_CPU_LOGICAL
 *     CURRENT_HW_INNERMOST = NVTX_SCOPE_CURRENT_HW_INNERMOST
 *     CURRENT_HYPERVISOR = NVTX_SCOPE_CURRENT_HYPERVISOR             # <<<<<<<<<<<<<<
 *     CURRENT_VM = NVTX_SCOPE_CURRENT_VM
 *     CURRENT_KERNEL = NVTX_SCOPE_CURRENT_KERNEL
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_HYPERVISOR); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_HYPERVISOR, __pyx_t_4) < (0)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":143
 *     CURRENT_HW_INNERMOST = NVTX_SCOPE_CURRENT_HW_INNERMOST
 *     CURRENT_HYPERVISOR = NVTX_SCOPE_CURRENT_HYPERVISOR
 *     CURRENT_VM = NVTX_SCOPE_CURRENT_VM             # <<<<<<<<<<<<<<
 *     CURRENT_KERNEL = NVTX_SCOPE_CURRENT_KERNEL
 *     CURRENT_CONTAINER = NVTX_SCOPE_CURRENT_CONTAINER
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_VM); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_VM, __pyx_t_4) < (0)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":144
 *     CURRENT_HYPERVISOR = NVTX_SCOPE_CURRENT_HYPERVISOR
 *     CURRENT_VM = NVTX_SCOPE_CURRENT_VM
 *     CURRENT_KERNEL = NVTX_SCOPE_CURRENT_KERNEL             # <<<<<<<<<<<<<<
 *     CURRENT_CONTAINER = NVTX_SCOPE_CURRENT_CONTAINER
 *     CURRENT_OS = NVTX_SCOPE_CURRENT_OS
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_KERNEL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_KERNEL, __pyx_t_4) < (0)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":145
 *     CURRENT_VM = NVTX_SCOPE_CURRENT_VM
 *     CURRENT_KERNEL = NVTX_SCOPE_CURRENT_KERNEL
 *     CURRENT_CONTAINER = NVTX_SCOPE_CURRENT_CONTAINER             # <<<<<<<<<<<<<<
 *     CURRENT_OS = NVTX_SCOPE_CURRENT_OS
 *     CURRENT_SW_PROCESS = NVTX_SCOPE_CURRENT_SW_PROCESS
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_CONTAINER); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_CONTAINER, __pyx_t_4) < (0)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":146
 *     CURRENT_KERNEL = NVTX_SCOPE_CURRENT_KERNEL
 *     CURRENT_CONTAINER = NVTX_SCOPE_CURRENT_CONTAINER
 *     CURRENT_OS = NVTX_SCOPE_CURRENT_OS             # <<<<<<<<<<<<<<
 *     CURRENT_SW_PROCESS = NVTX_SCOPE_CURRENT_SW_PROCESS
 *     CURRENT_SW_THREAD = NVTX_SCOPE_CURRENT_SW_THREAD
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_OS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_OS, __pyx_t_4) < (0)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":147
 *     CURRENT_CONTAINER = NVTX_SCOPE_CURRENT_CONTAINER
 *     CURRENT_OS = NVTX_SCOPE_CURRENT_OS
 *     CURRENT_SW_PROCESS = NVTX_SCOPE_CURRENT_SW_PROCESS             # <<<<<<<<<<<<<<
 *     CURRENT_SW_THREAD = NVTX_SCOPE_CURRENT_SW_THREAD
 *     CURRENT_SW_INNERMOST = NVTX_SCOPE_CURRENT_SW_INNERMOST
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_SW_PROCESS); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_SW_PROCESS, __pyx_t_4) < (0)) __PYX_ERR(0, 147, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":148
 *     CURRENT_OS = NVTX_SCOPE_CURRENT_OS
 *     CURRENT_SW_PROCESS = NVTX_SCOPE_CURRENT_SW_PROCESS
 *     CURRENT_SW_THREAD = NVTX_SCOPE_CURRENT_SW_THREAD             # <<<<<<<<<<<<<<
 *     CURRENT_SW_INNERMOST = NVTX_SCOPE_CURRENT_SW_INNERMOST
 * 
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_SW_THREAD); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_SW_THREAD, __pyx_t_4) < (0)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":149
 *     CURRENT_SW_PROCESS = NVTX_SCOPE_CURRENT_SW_PROCESS
 *     CURRENT_SW_THREAD = NVTX_SCOPE_CURRENT_SW_THREAD
 *     CURRENT_SW_INNERMOST = NVTX_SCOPE_CURRENT_SW_INNERMOST             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_SCOPE_CURRENT_SW_INNERMOST); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CURRENT_SW_INNERMOST, __pyx_t_4) < (0)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":132
 * 
 * 
 * class PredefinedScope(enum.Enum):             # <<<<<<<<<<<<<<
 *     """Predefined NVTX execution scopes."""
 * 
*/
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_PredefinedScope, __pyx_t_2, __pyx_t_8, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_PredefinedScope, __pyx_t_4) < (0)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":152
 * 
 * 
 * def _entry_flags_from_metadata(metadata):             # <<<<<<<<<<<<<<
 *     """
 *     Return the resolved NVTX payload entry flags stored in dtype metadata, if
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11_entry_flags_from_metadata, 0, __pyx_mstate_global->__pyx_n_u_entry_flags_from_metadata, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[6])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_entry_flags_from_metadata, __pyx_t_2) < (0)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":164
 * 
 * 
 * def _entry_type_from_metadata(metadata):             # <<<<<<<<<<<<<<
 *     """Return an explicit NVTX payload entry type, or ``None``."""
 *     if metadata is None:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_13_entry_type_from_metadata, 0, __pyx_mstate_global->__pyx_n_u_entry_type_from_metadata, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[7])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_entry_type_from_metadata, __pyx_t_2) < (0)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":172
 * 
 * 
 * class EntryKind(enum.Enum):             # <<<<<<<<<<<<<<
 *     """
 *     Role of a field in a structured payload dtype.
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Enum); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_Pack(1, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PEP560_update_bases(__pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_8 = __Pyx_CalculateMetaclass(NULL, __pyx_t_10); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_Py3MetaclassPrepare(__pyx_t_8, __pyx_t_10, __pyx_mstate_global->__pyx_n_u_EntryKind, __pyx_mstate_global->__pyx_n_u_EntryKind, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_kp_u_Role_of_a_field_in_a_structured); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_10 != __pyx_t_2) {
    if (unlikely((PyDict_SetItemString(__pyx_t_4, "__orig_bases__", __pyx_t_2) < 0))) __PYX_ERR(0, 172, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":185
 *     RANGE_BEGIN = (
 *         NVTX_PAYLOAD_ENTRY_FLAG_RANGE_BEGIN
 *         | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP             # <<<<<<<<<<<<<<
 *     )
 *     RANGE_END = (
*/
  __pyx_t_2 = __Pyx_PyLong_From_int((NVTX_PAYLOAD_ENTRY_FLAG_RANGE_BEGIN | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_RANGE_BEGIN, __pyx_t_2) < (0)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":188
 *     )
 *     RANGE_END = (
 *         NVTX_PAYLOAD_ENTRY_FLAG_RANGE_END | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP             # <<<<<<<<<<<<<<
 *     )
 *     MARK = NVTX_PAYLOAD_ENTRY_FLAG_MARK | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
*/
  __pyx_t_2 = __Pyx_PyLong_From_int((NVTX_PAYLOAD_ENTRY_FLAG_RANGE_END | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_RANGE_END, __pyx_t_2) < (0)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":190
 *         NVTX_PAYLOAD_ENTRY_FLAG_RANGE_END | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
 *     )
 *     MARK = NVTX_PAYLOAD_ENTRY_FLAG_MARK | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP             # <<<<<<<<<<<<<<
 *     MESSAGE = NVTX_PAYLOAD_ENTRY_FLAG_EVENT_MESSAGE
 *     COUNTER_TIMESTAMP = NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
*/
  __pyx_t_2 = __Pyx_PyLong_From_int((NVTX_PAYLOAD_ENTRY_FLAG_MARK | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_MARK, __pyx_t_2) < (0)) __PYX_ERR(0, 190, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":191
 *     )
 *     MARK = NVTX_PAYLOAD_ENTRY_FLAG_MARK | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
 *     MESSAGE = NVTX_PAYLOAD_ENTRY_FLAG_EVENT_MESSAGE             # <<<<<<<<<<<<<<
 *     COUNTER_TIMESTAMP = NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_FLAG_EVENT_MESSAGE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_MESSAGE, __pyx_t_2) < (0)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":192
 *     MARK = NVTX_PAYLOAD_ENTRY_FLAG_MARK | NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP
 *     MESSAGE = NVTX_PAYLOAD_ENTRY_FLAG_EVENT_MESSAGE
 *     COUNTER_TIMESTAMP = NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_FLAG_TIMESTAMP); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__Pyx_SetNameInClass(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_COUNTER_TIMESTAMP, __pyx_t_2) < (0)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":172
 * 
 * 
 * class EntryKind(enum.Enum):             # <<<<<<<<<<<<<<
 *     """
 *     Role of a field in a structured payload dtype.
*/
  __pyx_t_2 = __Pyx_Py3ClassCreate(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_EntryKind, __pyx_t_10, __pyx_t_4, NULL, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_EntryKind, __pyx_t_2) < (0)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":195
 * 
 * 
 * class PayloadEntryType(enum.Enum):             # <<<<<<<<<<<<<<
 *     """
 *     NVTX interpretation of a specially typed payload field.
*/
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_enum); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_Enum); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyTuple_Pack(1, __pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PEP560_update_bases(__pyx_t_10); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_CalculateMetaclass(NULL, __pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_4, __pyx_t_8, __pyx_mstate_global->__pyx_n_u_PayloadEntryType, __pyx_mstate_global->__pyx_n_u_PayloadEntryType, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_kp_u_NVTX_interpretation_of_a_specia); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (__pyx_t_8 != __pyx_t_10) {
    if (unlikely((PyDict_SetItemString(__pyx_t_2, "__orig_bases__", __pyx_t_10) < 0))) __PYX_ERR(0, 195, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":204
 *     """
 * 
 *     RANGE_ID = NVTX_PAYLOAD_ENTRY_TYPE_RANGE_ID             # <<<<<<<<<<<<<<
 *     CATEGORY = NVTX_PAYLOAD_ENTRY_TYPE_CATEGORY
 *     COLOR_ARGB = NVTX_PAYLOAD_ENTRY_TYPE_COLOR_ARGB
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_RANGE_ID); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_RANGE_ID, __pyx_t_10) < (0)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":205
 * 
 *     RANGE_ID = NVTX_PAYLOAD_ENTRY_TYPE_RANGE_ID
 *     CATEGORY = NVTX_PAYLOAD_ENTRY_TYPE_CATEGORY             # <<<<<<<<<<<<<<
 *     COLOR_ARGB = NVTX_PAYLOAD_ENTRY_TYPE_COLOR_ARGB
 *     SCOPE_ID = NVTX_PAYLOAD_ENTRY_TYPE_SCOPE_ID
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_CATEGORY); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_CATEGORY, __pyx_t_10) < (0)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":206
 *     RANGE_ID = NVTX_PAYLOAD_ENTRY_TYPE_RANGE_ID
 *     CATEGORY = NVTX_PAYLOAD_ENTRY_TYPE_CATEGORY
 *     COLOR_ARGB = NVTX_PAYLOAD_ENTRY_TYPE_COLOR_ARGB             # <<<<<<<<<<<<<<
 *     SCOPE_ID = NVTX_PAYLOAD_ENTRY_TYPE_SCOPE_ID
 *     REGISTERED_STRING = NVTX_PAYLOAD_ENTRY_TYPE_NVTX_REGISTERED_STRING_HANDLE
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_COLOR_ARGB); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_COLOR_ARGB, __pyx_t_10) < (0)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":207
 *     CATEGORY = NVTX_PAYLOAD_ENTRY_TYPE_CATEGORY
 *     COLOR_ARGB = NVTX_PAYLOAD_ENTRY_TYPE_COLOR_ARGB
 *     SCOPE_ID = NVTX_PAYLOAD_ENTRY_TYPE_SCOPE_ID             # <<<<<<<<<<<<<<
 *     REGISTERED_STRING = NVTX_PAYLOAD_ENTRY_TYPE_NVTX_REGISTERED_STRING_HANDLE
 * 
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_SCOPE_ID); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_SCOPE_ID, __pyx_t_10) < (0)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":208
 *     COLOR_ARGB = NVTX_PAYLOAD_ENTRY_TYPE_COLOR_ARGB
 *     SCOPE_ID = NVTX_PAYLOAD_ENTRY_TYPE_SCOPE_ID
 *     REGISTERED_STRING = NVTX_PAYLOAD_ENTRY_TYPE_NVTX_REGISTERED_STRING_HANDLE             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_10 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_ENTRY_TYPE_NVTX_REGISTERED_STRING_HANDLE); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_REGISTERED_STRING, __pyx_t_10) < (0)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":195
 * 
 * 
 * class PayloadEntryType(enum.Enum):             # <<<<<<<<<<<<<<
 *     """
 *     NVTX interpretation of a specially typed payload field.
*/
  __pyx_t_10 = __Pyx_Py3ClassCreate(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_PayloadEntryType, __pyx_t_8, __pyx_t_2, NULL, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_PayloadEntryType, __pyx_t_10) < (0)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":211
 * 
 * 
 * class EventKind(enum.Enum):             # <<<<<<<<<<<<<<
 *     """
 *     Event kind for the writer's ``write_event`` / ``write_event_batch``,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_enum); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_Enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyTuple_Pack(1, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PEP560_update_bases(__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_4, __pyx_mstate_global->__pyx_n_u_EventKind, __pyx_mstate_global->__pyx_n_u_EventKind, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_kp_u_Event_kind_for_the_writer_s_wri); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__pyx_t_4 != __pyx_t_8) {
    if (unlikely((PyDict_SetItemString(__pyx_t_10, "__orig_bases__", __pyx_t_8) < 0))) __PYX_ERR(0, 211, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":221
 *     """
 * 
 *     MARK = NVTX_PAYLOAD_SCHEMA_FLAG_MARK             # <<<<<<<<<<<<<<
 *     RANGE_PUSHPOP = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSHPOP
 *     RANGE_STARTEND = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_STARTEND
*/
  __pyx_t_8 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_SCHEMA_FLAG_MARK); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_MARK, __pyx_t_8) < (0)) __PYX_ERR(0, 221, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":222
 * 
 *     MARK = NVTX_PAYLOAD_SCHEMA_FLAG_MARK
 *     RANGE_PUSHPOP = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSHPOP             # <<<<<<<<<<<<<<
 *     RANGE_STARTEND = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_STARTEND
 *     RANGE_PUSH = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSH
*/
  __pyx_t_8 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSHPOP); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_RANGE_PUSHPOP, __pyx_t_8) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":223
 *     MARK = NVTX_PAYLOAD_SCHEMA_FLAG_MARK
 *     RANGE_PUSHPOP = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSHPOP
 *     RANGE_STARTEND = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_STARTEND             # <<<<<<<<<<<<<<
 *     RANGE_PUSH = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSH
 *     RANGE_POP = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_POP
*/
  __pyx_t_8 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_STARTEND); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_RANGE_STARTEND, __pyx_t_8) < (0)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":224
 *     RANGE_PUSHPOP = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSHPOP
 *     RANGE_STARTEND = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_STARTEND
 *     RANGE_PUSH = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSH             # <<<<<<<<<<<<<<
 *     RANGE_POP = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_POP
 *     RANGE_START = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_START
*/
  __pyx_t_8 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSH); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_RANGE_PUSH, __pyx_t_8) < (0)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":225
 *     RANGE_STARTEND = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_STARTEND
 *     RANGE_PUSH = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSH
 *     RANGE_POP = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_POP             # <<<<<<<<<<<<<<
 *     RANGE_START = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_START
 *     RANGE_END = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_END
*/
  __pyx_t_8 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_POP); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_RANGE_POP, __pyx_t_8) < (0)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":226
 *     RANGE_PUSH = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_PUSH
 *     RANGE_POP = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_POP
 *     RANGE_START = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_START             # <<<<<<<<<<<<<<
 *     RANGE_END = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_END
 * 
*/
  __pyx_t_8 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_START); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_RANGE_START, __pyx_t_8) < (0)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":227
 *     RANGE_POP = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_POP
 *     RANGE_START = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_START
 *     RANGE_END = NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_END             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_8 = __Pyx_PyLong_From_int(NVTX_PAYLOAD_SCHEMA_FLAG_RANGE_END); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_RANGE_END, __pyx_t_8) < (0)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":211
 * 
 * 
 * class EventKind(enum.Enum):             # <<<<<<<<<<<<<<
 *     """
 *     Event kind for the writer's ``write_event`` / ``write_event_batch``,
*/
  __pyx_t_8 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_EventKind, __pyx_t_4, __pyx_t_10, NULL, 0, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_8);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_EventKind, __pyx_t_8) < (0)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":230
 * 
 * 
 * class BatchOrdering(enum.Enum):             # <<<<<<<<<<<<<<
 *     """Timestamp ordering within a batch of deferred events or counters."""
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_Enum); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PEP560_update_bases(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_8 = __Pyx_Py3MetaclassPrepare(__pyx_t_10, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_BatchOrdering, __pyx_mstate_global->__pyx_n_u_BatchOrdering, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_kp_u_Timestamp_ordering_within_a_batc); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (__pyx_t_2 != __pyx_t_4) {
    if (unlikely((PyDict_SetItemString(__pyx_t_8, "__orig_bases__", __pyx_t_4) < 0))) __PYX_ERR(0, 230, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":233
 *     """Timestamp ordering within a batch of deferred events or counters."""
 * 
 *     SORTED = NVTX_BATCH_FLAG_TIME_SORTED             # <<<<<<<<<<<<<<
 *     PARTIALLY_SORTED = NVTX_BATCH_FLAG_TIME_SORTED_PARTIALLY
 *     SORTED_PER_SCOPE = NVTX_BATCH_FLAG_TIME_SORTED_PER_SCOPE
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_BATCH_FLAG_TIME_SORTED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_SORTED, __pyx_t_4) < (0)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":234
 * 
 *     SORTED = NVTX_BATCH_FLAG_TIME_SORTED
 *     PARTIALLY_SORTED = NVTX_BATCH_FLAG_TIME_SORTED_PARTIALLY             # <<<<<<<<<<<<<<
 *     SORTED_PER_SCOPE = NVTX_BATCH_FLAG_TIME_SORTED_PER_SCOPE
 *     UNSORTED = NVTX_BATCH_FLAG_UNSORTED
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_BATCH_FLAG_TIME_SORTED_PARTIALLY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_PARTIALLY_SORTED, __pyx_t_4) < (0)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":235
 *     SORTED = NVTX_BATCH_FLAG_TIME_SORTED
 *     PARTIALLY_SORTED = NVTX_BATCH_FLAG_TIME_SORTED_PARTIALLY
 *     SORTED_PER_SCOPE = NVTX_BATCH_FLAG_TIME_SORTED_PER_SCOPE             # <<<<<<<<<<<<<<
 *     UNSORTED = NVTX_BATCH_FLAG_UNSORTED
 * 
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_BATCH_FLAG_TIME_SORTED_PER_SCOPE); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_SORTED_PER_SCOPE, __pyx_t_4) < (0)) __PYX_ERR(0, 235, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":236
 *     PARTIALLY_SORTED = NVTX_BATCH_FLAG_TIME_SORTED_PARTIALLY
 *     SORTED_PER_SCOPE = NVTX_BATCH_FLAG_TIME_SORTED_PER_SCOPE
 *     UNSORTED = NVTX_BATCH_FLAG_UNSORTED             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_4 = __Pyx_PyLong_From_int(NVTX_BATCH_FLAG_UNSORTED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__Pyx_SetNameInClass(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_UNSORTED, __pyx_t_4) < (0)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "nvtx/_lib/lib.pyx":230
 * 
 * 
 * class BatchOrdering(enum.Enum):             # <<<<<<<<<<<<<<
 *     """Timestamp ordering within a batch of deferred events or counters."""
 * 
*/
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_BatchOrdering, __pyx_t_2, __pyx_t_8, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_BatchOrdering, __pyx_t_4) < (0)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":239
 * 
 * 
 * _payload_setters = {}             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_payload_setters, __pyx_t_2) < (0)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":242
 * 
 * 
 * def payload_setter(type):             # <<<<<<<<<<<<<<
 *     """
 *     A helper decorator to register a payload setter for a given type.
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15payload_setter, 0, __pyx_mstate_global->__pyx_n_u_payload_setter, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[8])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_payload_setter, __pyx_t_2) < (0)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":349
 *         setter(self, value)
 * 
 *     @payload_setter(int)             # <<<<<<<<<<<<<<
 *     def _set_payload_int(self, payload):
 *         self.c_obj.payload.llValue = payload
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_5_set_payload_int, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_EventAttributes__set_payload_int, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[9])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_int, __pyx_t_2) < (0)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_10 = NULL;
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_payload_setter); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, ((PyObject *)(&PyLong_Type))};
    __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 349, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __Pyx_GetNameInClass(__pyx_t_11, (PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_int); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_10, __pyx_t_11};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 349, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_int, __pyx_t_2) < (0)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":354
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_INT64
 * 
 *     @payload_setter(float)             # <<<<<<<<<<<<<<
 *     def _set_payload_float(self, payload):
 *         self.c_obj.payload.dValue = payload
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_7_set_payload_float, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_EventAttributes__set_payload_flo, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[10])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_float, __pyx_t_2) < (0)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = NULL;
  __pyx_t_10 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_payload_setter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_10, ((PyObject *)(&PyFloat_Type))};
    __pyx_t_11 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 354, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
  }
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_float); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_4};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 354, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_float, __pyx_t_2) < (0)) __PYX_ERR(0, 354, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":359
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_DOUBLE
 * 
 *     if np is not None:             # <<<<<<<<<<<<<<
 *         @payload_setter(np.ndarray)
 *         def _set_payload_numpy(self, payload):
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 359, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_13 = (__pyx_t_2 != Py_None);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_13) {

    /* "nvtx/_lib/lib.pyx":360
 * 
 *     if np is not None:
 *         @payload_setter(np.ndarray)             # <<<<<<<<<<<<<<
 *         def _set_payload_numpy(self, payload):
 *             if payload.nbytes == 0:
*/
    __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_9_set_payload_numpy, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_EventAttributes__set_payload_num, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[11])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
    PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
    #endif
    if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_numpy, __pyx_t_2) < (0)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_11 = NULL;
    __pyx_t_8 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_payload_setter); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_np); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_ndarray); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_15};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_GetNameInClass(__pyx_t_10, (PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_numpy); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_10};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_numpy, __pyx_t_2) < (0)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "nvtx/_lib/lib.pyx":381
 * 
 * 
 *         @payload_setter(range)             # <<<<<<<<<<<<<<
 *         @payload_setter(list)
 *         @payload_setter(tuple)
*/
    __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_11_set_payload_iterable, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_EventAttributes__set_payload_ite, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[12])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
    PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
    #endif
    if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_iterable, __pyx_t_2) < (0)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = NULL;
    __pyx_t_11 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_payload_setter); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_11, ((PyObject *)(&PyRange_Type))};
      __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_15, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 381, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
    }

    /* "nvtx/_lib/lib.pyx":382
 * 
 *         @payload_setter(range)
 *         @payload_setter(list)             # <<<<<<<<<<<<<<
 *         @payload_setter(tuple)
 *         @payload_setter(bytes)
*/
    __pyx_t_11 = NULL;
    __pyx_t_14 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_payload_setter); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 382, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_14, ((PyObject *)(&PyList_Type))};
      __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_16, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 382, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
    }

    /* "nvtx/_lib/lib.pyx":383
 *         @payload_setter(range)
 *         @payload_setter(list)
 *         @payload_setter(tuple)             # <<<<<<<<<<<<<<
 *         @payload_setter(bytes)
 *         def _set_payload_iterable(self, payload):
*/
    __pyx_t_14 = NULL;
    __pyx_t_18 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_19, __pyx_mstate_global->__pyx_n_u_payload_setter); if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 383, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_19);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_18, ((PyObject *)(&PyTuple_Type))};
      __pyx_t_17 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_19, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_17);
    }

    /* "nvtx/_lib/lib.pyx":384
 *         @payload_setter(list)
 *         @payload_setter(tuple)
 *         @payload_setter(bytes)             # <<<<<<<<<<<<<<
 *         def _set_payload_iterable(self, payload):
 *             self._set_payload_numpy(np.array(payload))
*/
    __pyx_t_18 = NULL;
    __pyx_t_21 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_22, __pyx_mstate_global->__pyx_n_u_payload_setter); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 384, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_22);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_21, ((PyObject *)(&PyBytes_Type))};
      __pyx_t_20 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_22, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
      __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
      if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 384, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_20);
    }

    /* "nvtx/_lib/lib.pyx":381
 * 
 * 
 *         @payload_setter(range)             # <<<<<<<<<<<<<<
 *         @payload_setter(list)
 *         @payload_setter(tuple)
*/
    __Pyx_GetNameInClass(__pyx_t_22, (PyObject*)__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_iterable); if (unlikely(!__pyx_t_22)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_22);
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_18, __pyx_t_22};
      __pyx_t_19 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_20, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;
      __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
      if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 384, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_19);
    }
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_19};
      __pyx_t_16 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_17, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
      __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
      if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_16);
    }
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_16};
      __pyx_t_15 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 382, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_15);
    }
    __pyx_t_12 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_15};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 381, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_EventAttributes, __pyx_mstate_global->__pyx_n_u_set_payload_iterable, __pyx_t_2) < (0)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "nvtx/_lib/lib.pyx":359
 *         self.c_obj.payloadType = NVTX_PAYLOAD_TYPE_DOUBLE
 * 
 *     if np is not None:             # <<<<<<<<<<<<<<
 *         @payload_setter(np.ndarray)
 *         def _set_payload_numpy(self, payload):
*/
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_13__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_EventAttributes___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[13])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15EventAttributes_15__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_EventAttributes___setstate_cytho, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[14])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":430
 *         return self._name.decode()
 * 
 *     def enabled(self):             # <<<<<<<<<<<<<<
 *         return bool(nvtxDomainIsEnabled(self.c_obj))
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_12DomainHandle_3enabled, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DomainHandle_enabled, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[15])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_DomainHandle, __pyx_mstate_global->__pyx_n_u_enabled, __pyx_t_2) < (0)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_12DomainHandle_7__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DomainHandle___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[16])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_12DomainHandle_9__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DomainHandle___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[17])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":437
 * 
 * 
 * class RegisteredString:             # <<<<<<<<<<<<<<
 *     """
 *     A wrapper class for ``nvtxStringHandle_t`` C struct.
*/
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_mstate_global->__pyx_empty_tuple, __pyx_mstate_global->__pyx_n_u_RegisteredString, __pyx_mstate_global->__pyx_n_u_RegisteredString, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_kp_u_A_wrapper_class_for_nvtxStringH); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "nvtx/_lib/lib.pyx":442
 *     Use :func:`nvtx.Domain.get_registered_string` to create an instance.
 *     """
 *     def __init__(self, domain, string=None):             # <<<<<<<<<<<<<<
 *         self.string = string
 *         self.domain = domain
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_16RegisteredString_1__init__, 0, __pyx_mstate_global->__pyx_n_u_RegisteredString___init, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[18])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_10, __pyx_mstate_global->__pyx_tuple[0]);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_init, __pyx_t_10) < (0)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":437
 * 
 * 
 * class RegisteredString:             # <<<<<<<<<<<<<<
 *     """
 *     A wrapper class for ``nvtxStringHandle_t`` C struct.
*/
  __pyx_t_10 = __Pyx_Py3ClassCreate(((PyObject*)&PyType_Type), __pyx_mstate_global->__pyx_n_u_RegisteredString, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_2, NULL, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_RegisteredString, __pyx_t_10) < (0)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":448
 * 
 * 
 * class DummyDomain:             # <<<<<<<<<<<<<<
 *     """
 *     A replacement for :class:`nvtx.Domain` when the domain is disabled.
*/
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_mstate_global->__pyx_empty_tuple, __pyx_mstate_global->__pyx_n_u_DummyDomain, __pyx_mstate_global->__pyx_n_u_DummyDomain, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_kp_u_A_replacement_for_class_nvtx_Do); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "nvtx/_lib/lib.pyx":453
 *     (e.g., when no tool is attached).
 *     """
 *     def get_registered_string(self, string):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_1get_registered_string, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_get_registered_strin, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[19])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_registered_string, __pyx_t_10) < (0)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":456
 *         pass
 * 
 *     def get_category_id(self, name):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_3get_category_id, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_get_category_id, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[20])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_category_id, __pyx_t_10) < (0)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":459
 *         pass
 * 
 *     def get_event_attributes(self, message=None, color=None, category=None, payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_5get_event_attributes, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_get_event_attributes, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[21])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_10, __pyx_mstate_global->__pyx_tuple[1]);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_event_attributes, __pyx_t_10) < (0)) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":462
 *         pass
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,             # <<<<<<<<<<<<<<
 *                              message=None, color=None, category=None, payload=None):
 *         pass
*/
  __pyx_t_10 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);

  /* "nvtx/_lib/lib.pyx":463
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,
 *                              message=None, color=None, category=None, payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_message, Py_None) < (0)) __PYX_ERR(0, 462, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_color, Py_None) < (0)) __PYX_ERR(0, 462, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_category, Py_None) < (0)) __PYX_ERR(0, 462, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_payload, Py_None) < (0)) __PYX_ERR(0, 462, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":462
 *         pass
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,             # <<<<<<<<<<<<<<
 *                              message=None, color=None, category=None, payload=None):
 *         pass
*/
  __pyx_t_15 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_7set_event_attributes, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_set_event_attributes, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[22])); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_15);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_15, __pyx_t_10);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_set_event_attributes, __pyx_t_15) < (0)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":466
 *         pass
 * 
 *     def mark(self, EventAttributes attributes=None, *, message=None, color=None, category=None,             # <<<<<<<<<<<<<<
 *              payload=None):
 *         pass
*/
  __pyx_t_15 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_message, Py_None) < (0)) __PYX_ERR(0, 466, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_color, Py_None) < (0)) __PYX_ERR(0, 466, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_category, Py_None) < (0)) __PYX_ERR(0, 466, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":467
 * 
 *     def mark(self, EventAttributes attributes=None, *, message=None, color=None, category=None,
 *              payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_payload, Py_None) < (0)) __PYX_ERR(0, 466, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":466
 *         pass
 * 
 *     def mark(self, EventAttributes attributes=None, *, message=None, color=None, category=None,             # <<<<<<<<<<<<<<
 *              payload=None):
 *         pass
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_9mark, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_mark, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[23])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_10, __pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_10, __pyx_t_15);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_mark, __pyx_t_10) < (0)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":470
 *         pass
 * 
 *     def push_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                    category=None, payload=None):
 *         pass
*/
  __pyx_t_10 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_message, Py_None) < (0)) __PYX_ERR(0, 470, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_color, Py_None) < (0)) __PYX_ERR(0, 470, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":471
 * 
 *     def push_range(self, EventAttributes attributes=None, *, message=None, color=None,
 *                    category=None, payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_category, Py_None) < (0)) __PYX_ERR(0, 470, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_payload, Py_None) < (0)) __PYX_ERR(0, 470, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":470
 *         pass
 * 
 *     def push_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                    category=None, payload=None):
 *         pass
*/
  __pyx_t_15 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_11push_range, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_push_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[24])); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_15);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_15, __pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_15, __pyx_t_10);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_push_range, __pyx_t_15) < (0)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":474
 *         pass
 * 
 *     def pop_range(self):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  __pyx_t_15 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_13pop_range, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_pop_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[25])); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_15);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pop_range, __pyx_t_15) < (0)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":477
 *         pass
 * 
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                     category=None, payload=None):
 *         return 0
*/
  __pyx_t_15 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_message, Py_None) < (0)) __PYX_ERR(0, 477, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_color, Py_None) < (0)) __PYX_ERR(0, 477, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":478
 * 
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,
 *                     category=None, payload=None):             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_category, Py_None) < (0)) __PYX_ERR(0, 477, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_payload, Py_None) < (0)) __PYX_ERR(0, 477, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":477
 *         pass
 * 
 *     def start_range(self, EventAttributes attributes=None, *, message=None, color=None,             # <<<<<<<<<<<<<<
 *                     category=None, payload=None):
 *         return 0
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_15start_range, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_start_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[26])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_10, __pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_10, __pyx_t_15);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_start_range, __pyx_t_10) < (0)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":481
 *         return 0
 * 
 *     def end_range(self, nvtxRangeId_t range_id):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_17end_range, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_end_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[27])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_end_range, __pyx_t_10) < (0)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":484
 *         pass
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/
  __pyx_t_10 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_DummyCounter) < (0)) __PYX_ERR(0, 484, __pyx_L1_error)
  __pyx_t_15 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_19get_counter, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_get_counter, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[28])); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_15);
  #endif
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_15, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults)) __PYX_ERR(0, 484, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":492
 *         scope=None,
 *         semantics=None,
 *         time_domain=TimestampType.TOOL_PROVIDED,             # <<<<<<<<<<<<<<
 *     ) -> DummyCounter:
 *         _normalize_counter_dtype(dtype) # Validate that the user-provided dtype is supported.
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_TimestampType); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_TOOL_PROVIDED); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_CyFunction_Defaults(struct __pyx_defaults, __pyx_t_15)->arg0 = __pyx_t_8;
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_15, __pyx_pf_4nvtx_4_lib_3lib_26__defaults__);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_15, __pyx_t_10);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_counter, __pyx_t_15) < (0)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":497
 *         return dummy_counter
 * 
 *     def get_timestamp(self):             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
  __pyx_t_15 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_11DummyDomain_21get_timestamp, 0, __pyx_mstate_global->__pyx_n_u_DummyDomain_get_timestamp, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[29])); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_15);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_get_timestamp, __pyx_t_15) < (0)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":448
 * 
 * 
 * class DummyDomain:             # <<<<<<<<<<<<<<
 *     """
 *     A replacement for :class:`nvtx.Domain` when the domain is disabled.
*/
  __pyx_t_15 = __Pyx_Py3ClassCreate(((PyObject*)&PyType_Type), __pyx_mstate_global->__pyx_n_u_DummyDomain, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_2, NULL, 0, 0); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_15);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DummyDomain, __pyx_t_15) < (0)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":501
 * 
 * 
 * dummy_domain = DummyDomain()             # <<<<<<<<<<<<<<
 * dummy_counter.domain = dummy_domain
 * 
*/
  __pyx_t_15 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_DummyDomain); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_15, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 501, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_dummy_domain, __pyx_t_2) < (0)) __PYX_ERR(0, 501, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "nvtx/_lib/lib.pyx":502
 * 
 * dummy_domain = DummyDomain()
 * dummy_counter.domain = dummy_domain             # <<<<<<<<<<<<<<
 * 
 * # A sentinel value to indicate that the argument should not be set.
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_dummy_domain); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_dummy_counter); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_domain, __pyx_t_2) < (0)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":506
 * # A sentinel value to indicate that the argument should not be set.
 * # Used in `Domain.set_event_attributes` to allow setting fields to None.
 * DONT_SET = object()             # <<<<<<<<<<<<<<
 * 
 * cdef class SchemaRegistrar:
*/
  __pyx_t_2 = NULL;
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_object, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 506, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DONT_SET, __pyx_t_10) < (0)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":528
 *             "SchemaRegistrar subclasses must implement _do_register")
 * 
 *     def _register_builtin_schema(self, dt, metadata):             # <<<<<<<<<<<<<<
 *         name = dt.name.encode()
 *         cdef nvtxSemanticsCounter_t counter_semantics
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_3_register_builtin_schema, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SchemaRegistrar__register_builti, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[30])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, __pyx_mstate_global->__pyx_n_u_register_builtin_schema, __pyx_t_10) < (0)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":571
 *         return self._do_register(&schemaAttr)
 * 
 *     def _register_structured_schema(self, schema_key):             # <<<<<<<<<<<<<<
 *         cdef bint counter_group = schema_key.counter_group
 *         dt = schema_key.dtype
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_5_register_structured_schema, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SchemaRegistrar__register_struct, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[31])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, __pyx_mstate_global->__pyx_n_u_register_structured_schema, __pyx_t_10) < (0)) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":679
 *             free(semanticsEntries)
 * 
 *     def _get_numpy_array_schema(self, uint64_t scalar_schema):             # <<<<<<<<<<<<<<
 *         cached = self._array_cache.get(scalar_schema)
 *         if cached is not None:
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_7_get_numpy_array_schema, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SchemaRegistrar__get_numpy_array, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[32])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, __pyx_mstate_global->__pyx_n_u_get_numpy_array_schema, __pyx_t_10) < (0)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":717
 *         return result
 * 
 *     def _get_fixed_size_array_schema(self, uint64_t scalar_schema, size_t array_length, size_t size):             # <<<<<<<<<<<<<<
 *         cache_key = (scalar_schema, array_length, size)
 *         cached = self._fixed_cache.get(cache_key)
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_9_get_fixed_size_array_schema, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SchemaRegistrar__get_fixed_size, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[33])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, __pyx_mstate_global->__pyx_n_u_get_fixed_size_array_schema, __pyx_t_10) < (0)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":745
 *         return result
 * 
 *     def _get_numpy_dtype_schema(self, schema_key):             # <<<<<<<<<<<<<<
 *         cached = self._dtype_cache.get(schema_key)
 *         if cached is not None:
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_11_get_numpy_dtype_schema, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SchemaRegistrar__get_numpy_dtype, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[34])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib_SchemaRegistrar, __pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_t_10) < (0)) __PYX_ERR(0, 745, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_13__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SchemaRegistrar___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[35])); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_10) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_15SchemaRegistrar_15__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_SchemaRegistrar___setstate_cytho, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[36])); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_10) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_3__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_GlobalSchemaRegistrar___reduce, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[37])); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_10) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "no default __reduce__ due to non-trivial __cinit__"
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_22_GlobalSchemaRegistrar_5__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_GlobalSchemaRegistrar___setstat, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[38])); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_10) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":780
 * 
 * 
 * class Domain:             # <<<<<<<<<<<<<<
 *     """
 *     A class that provides an interface to NVTX API per domain,
*/
  __pyx_t_10 = __Pyx_Py3MetaclassPrepare((PyObject *) NULL, __pyx_mstate_global->__pyx_empty_tuple, __pyx_mstate_global->__pyx_n_u_Domain, __pyx_mstate_global->__pyx_n_u_Domain, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_kp_u_A_class_that_provides_an_interf); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "nvtx/_lib/lib.pyx":792
 *     """
 * 
 *     def __new__(cls, name: Optional[str] = None):             # <<<<<<<<<<<<<<
 *         handle = DomainHandle(name)
 *         if handle.enabled():
*/
  __pyx_t_15 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_name, __pyx_mstate_global->__pyx_kp_u_Optional_str) < (0)) __PYX_ERR(0, 792, __pyx_L1_error)
  __pyx_t_8 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_1__new__, __Pyx_CYFUNCTION_STATICMETHOD, __pyx_mstate_global->__pyx_n_u_Domain___new, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[39])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_8);
  #endif
  PyList_Append(__pyx_t_2, __pyx_t_8);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_8, __pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_8, __pyx_t_15);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  if (__Pyx_SetNewInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_new, __pyx_t_8) < (0)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "nvtx/_lib/lib.pyx":801
 *             return dummy_domain
 * 
 *     def __init__(self, name: Optional[str] = None):             # <<<<<<<<<<<<<<
 *         self.name = name
 *         self.categories = {}
*/
  __pyx_t_8 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 801, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name, __pyx_mstate_global->__pyx_kp_u_Optional_str) < (0)) __PYX_ERR(0, 801, __pyx_L1_error)
  __pyx_t_15 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_3__init__, 0, __pyx_mstate_global->__pyx_n_u_Domain___init, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[40])); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 801, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_15);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_15, __pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_15, __pyx_t_8);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_init, __pyx_t_15) < (0)) __PYX_ERR(0, 801, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":806
 *         self._schema_registrar = _GlobalSchemaRegistrar(self.handle)
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def get_registered_string(self, string) -> RegisteredString:
 *         """
*/
  __pyx_t_8 = NULL;
  __pyx_t_16 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_lru_cache); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_16, NULL};
    __pyx_t_17 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 806, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_maxsize, Py_None, __pyx_t_17, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 806, __pyx_L1_error)
    __pyx_t_4 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_17);
    __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 806, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_RegisteredString) < (0)) __PYX_ERR(0, 806, __pyx_L1_error)
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_5get_registered_string, 0, __pyx_mstate_global->__pyx_n_u_Domain_get_registered_string, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[41])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_17, __pyx_t_11);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_17};
    __pyx_t_15 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 806, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
  }
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_get_registered_string, __pyx_t_15) < (0)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":818
 *         return RegisteredString(self.handle, string)
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def get_category_id(self, name) -> int:
 *         """
*/
  __pyx_t_4 = NULL;
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_lru_cache); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_8, NULL};
    __pyx_t_16 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_maxsize, Py_None, __pyx_t_16, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 818, __pyx_L1_error)
    __pyx_t_17 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_16);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
  }
  __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_int) < (0)) __PYX_ERR(0, 818, __pyx_L1_error)
  __pyx_t_16 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_7get_category_id, 0, __pyx_mstate_global->__pyx_n_u_Domain_get_category_id, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[42])); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_16);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_16, __pyx_t_11);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_16};
    __pyx_t_15 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_17, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
  }
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_get_category_id, __pyx_t_15) < (0)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":839
 *         return category_id
 * 
 *     def get_event_attributes(self, message=None, color=None, category=None, payload=None             # <<<<<<<<<<<<<<
 *             )-> EventAttributes:
 *         """
*/
  __pyx_t_15 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 839, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_EventAttributes) < (0)) __PYX_ERR(0, 839, __pyx_L1_error)
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_9get_event_attributes, 0, __pyx_mstate_global->__pyx_n_u_Domain_get_event_attributes, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[43])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 839, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_17, __pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_17, __pyx_t_15);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_get_event_attributes, __pyx_t_17) < (0)) __PYX_ERR(0, 839, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":866
 *         return EventAttributes(self, message, color, category, payload)
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,             # <<<<<<<<<<<<<<
 *                              message=DONT_SET, color=DONT_SET, category=DONT_SET, payload=DONT_SET):
 *         """
*/
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_11set_event_attributes, 0, __pyx_mstate_global->__pyx_n_u_Domain_set_event_attributes, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[44])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_17, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults1)) __PYX_ERR(0, 866, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":867
 * 
 *     def set_event_attributes(self, EventAttributes attributes, *,
 *                              message=DONT_SET, color=DONT_SET, category=DONT_SET, payload=DONT_SET):             # <<<<<<<<<<<<<<
 *         """
 *         Set the attributes of an :class:`nvtx._lib.lib.EventAttributes` object.
*/
  __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_DONT_SET); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_CyFunction_Defaults(struct __pyx_defaults1, __pyx_t_17)->arg0 = __pyx_t_15;
  __Pyx_GIVEREF(__pyx_t_15);
  __pyx_t_15 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_DONT_SET); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_CyFunction_Defaults(struct __pyx_defaults1, __pyx_t_17)->arg1 = __pyx_t_15;
  __Pyx_GIVEREF(__pyx_t_15);
  __pyx_t_15 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_DONT_SET); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_CyFunction_Defaults(struct __pyx_defaults1, __pyx_t_17)->arg2 = __pyx_t_15;
  __Pyx_GIVEREF(__pyx_t_15);
  __pyx_t_15 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_DONT_SET); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_CyFunction_Defaults(struct __pyx_defaults1, __pyx_t_17)->arg3 = __pyx_t_15;
  __Pyx_GIVEREF(__pyx_t_15);
  __pyx_t_15 = 0;
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_17, __pyx_pf_4nvtx_4_lib_3lib_28__defaults__);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_set_event_attributes, __pyx_t_17) < (0)) __PYX_ERR(0, 866, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":902
 *             attributes.payload = payload
 * 
 *     def mark(self, EventAttributes attributes=None, *, **kwargs):             # <<<<<<<<<<<<<<
 *         """
 *         Mark an instantaneous event.
*/
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_13mark, 0, __pyx_mstate_global->__pyx_n_u_Domain_mark, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[45])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_17, __pyx_mstate_global->__pyx_tuple[0]);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_mark, __pyx_t_17) < (0)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":945
 *         nvtxDomainMarkEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
 *     def push_range(self, EventAttributes attributes=None, *, **kwargs):             # <<<<<<<<<<<<<<
 *         """
 *         Mark the beginning of a code range.
*/
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_15push_range, 0, __pyx_mstate_global->__pyx_n_u_Domain_push_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[46])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_17, __pyx_mstate_global->__pyx_tuple[0]);
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_push_range, __pyx_t_17) < (0)) __PYX_ERR(0, 945, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":998
 *         nvtxDomainRangePushEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
 *     def pop_range(self):             # <<<<<<<<<<<<<<
 *         """
 *         Mark the end of a code range that was started with :func:`Domain.push_range`.
*/
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_17pop_range, 0, __pyx_mstate_global->__pyx_n_u_Domain_pop_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[47])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 998, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_pop_range, __pyx_t_17) < (0)) __PYX_ERR(0, 998, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":1004
 *         nvtxDomainRangePop((<DomainHandle>self.handle).c_obj)
 * 
 *     def start_range(self, EventAttributes attributes=None, *, **kwargs) -> int:             # <<<<<<<<<<<<<<
 *         """
 *         Mark the beginning of a process range.
*/
  __pyx_t_17 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1004, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  if (PyDict_SetItem(__pyx_t_17, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_int) < (0)) __PYX_ERR(0, 1004, __pyx_L1_error)
  __pyx_t_15 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_19start_range, 0, __pyx_mstate_global->__pyx_n_u_Domain_start_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[48])); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1004, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_15);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_15, __pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_15, __pyx_t_17);
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_start_range, __pyx_t_15) < (0)) __PYX_ERR(0, 1004, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":1058
 *         return nvtxDomainRangeStartEx((<DomainHandle>self.handle).c_obj, &attributes.c_obj)
 * 
 *     def end_range(self, nvtxRangeId_t range_id):             # <<<<<<<<<<<<<<
 *         """
 *         Mark the end of a process range that was started with :func:`Domain.start_range`.
*/
  __pyx_t_15 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_21end_range, 0, __pyx_mstate_global->__pyx_n_u_Domain_end_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[49])); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1058, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_15);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_end_range, __pyx_t_15) < (0)) __PYX_ERR(0, 1058, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

  /* "nvtx/_lib/lib.pyx":1069
 *         nvtxDomainRangeEnd((<DomainHandle>self.handle).c_obj, range_id)
 * 
 *     def get_counter(             # <<<<<<<<<<<<<<
 *         self,
 *         name,
*/
  __pyx_t_15 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  if (PyDict_SetItem(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_Counter) < (0)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_23get_counter, 0, __pyx_mstate_global->__pyx_n_u_Domain_get_counter, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[50])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_17, __pyx_mstate_global->__pyx_ptype_4nvtx_4_lib_3lib___pyx_defaults)) __PYX_ERR(0, 1069, __pyx_L1_error)

  /* "nvtx/_lib/lib.pyx":1077
 *         scope=None,
 *         semantics=None,
 *         time_domain=TimestampType.TOOL_PROVIDED,             # <<<<<<<<<<<<<<
 *     ) -> Counter:
 *         """
*/
  __Pyx_GetModuleGlobalName(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_TimestampType); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1077, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_mstate_global->__pyx_n_u_TOOL_PROVIDED); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1077, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __Pyx_CyFunction_Defaults(struct __pyx_defaults, __pyx_t_17)->arg0 = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_17, __pyx_pf_4nvtx_4_lib_3lib_30__defaults__);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_17, __pyx_t_15);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_get_counter, __pyx_t_17) < (0)) __PYX_ERR(0, 1069, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":1129
 *         )
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def _get_scope_id(self, path) -> int:
 *         cdef bytes path_bytes = _to_bytes(path)
*/
  __pyx_t_15 = NULL;
  __pyx_t_16 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_lru_cache); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_16, NULL};
    __pyx_t_8 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_maxsize, Py_None, __pyx_t_8, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1129, __pyx_L1_error)
    __pyx_t_4 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_int) < (0)) __PYX_ERR(0, 1129, __pyx_L1_error)
  __pyx_t_8 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_25_get_scope_id, 0, __pyx_mstate_global->__pyx_n_u_Domain__get_scope_id, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[51])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_8);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_8, __pyx_t_11);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_15, __pyx_t_8};
    __pyx_t_17 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
  }
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_get_scope_id, __pyx_t_17) < (0)) __PYX_ERR(0, 1129, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":1139
 *         return nvtxScopeRegister((<DomainHandle>self.handle).c_obj, &attr)
 * 
 *     @lru_cache(maxsize=None)             # <<<<<<<<<<<<<<
 *     def _get_counter_cached(
 *         self,
*/
  __pyx_t_4 = NULL;
  __pyx_t_15 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_mstate_global->__pyx_n_u_lru_cache); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_15, NULL};
    __pyx_t_16 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_maxsize, Py_None, __pyx_t_16, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1139, __pyx_L1_error)
    __pyx_t_8 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_12, (1-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_16);
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_t_11 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_27_get_counter_cached, 0, __pyx_mstate_global->__pyx_n_u_Domain__get_counter_cached, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[52])); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_11);
  #endif
  __pyx_t_12 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_11};
    __pyx_t_17 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_12, (2-__pyx_t_12) | (__pyx_t_12*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_17);
  }
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_get_counter_cached, __pyx_t_17) < (0)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":1161
 *         )
 * 
 *     def get_timestamp(self):             # <<<<<<<<<<<<<<
 *         """
 *         Return an NVTX timestamp for use with batched counter samples.
*/
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_29get_timestamp, 0, __pyx_mstate_global->__pyx_n_u_Domain_get_timestamp, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[53])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_get_timestamp, __pyx_t_17) < (0)) __PYX_ERR(0, 1161, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":1168
 *         return nvtxTimestampGet()
 * 
 *     def _get_numpy_dtype_schema(self, schema_key):             # <<<<<<<<<<<<<<
 *         # Schema building/caching lives on the domain's SchemaRegistrar.
 *         return self._schema_registrar._get_numpy_dtype_schema(schema_key)
*/
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_31_get_numpy_dtype_schema, 0, __pyx_mstate_global->__pyx_n_u_Domain__get_numpy_dtype_schema, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[54])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_get_numpy_dtype_schema, __pyx_t_17) < (0)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":1172
 *         return self._schema_registrar._get_numpy_dtype_schema(schema_key)
 * 
 *     def _get_numpy_array_schema(self, scalar_schema):             # <<<<<<<<<<<<<<
 *         return self._schema_registrar._get_numpy_array_schema(
 *             scalar_schema
*/
  __pyx_t_17 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_6Domain_33_get_numpy_array_schema, 0, __pyx_mstate_global->__pyx_n_u_Domain__get_numpy_array_schema, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[55])); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 1172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  if (__Pyx_SetNameInClass(__pyx_t_10, __pyx_mstate_global->__pyx_n_u_get_numpy_array_schema, __pyx_t_17) < (0)) __PYX_ERR(0, 1172, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "nvtx/_lib/lib.pyx":780
 * 
 * 
 * class Domain:             # <<<<<<<<<<<<<<
 *     """
 *     A class that provides an interface to NVTX API per domain,
*/
  __pyx_t_17 = __Pyx_Py3ClassCreate(((PyObject*)&PyType_Type), __pyx_mstate_global->__pyx_n_u_Domain, __pyx_mstate_global->__pyx_empty_tuple, __pyx_t_10, NULL, 0, 0); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_17);
  #endif
  if (__Pyx_CyFunction_InitClassCell(__pyx_t_2, __pyx_t_17) < (0)) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Domain, __pyx_t_17) < (0)) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_12StringHandle_3__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_StringHandle___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[56])); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_10) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self.c_obj cannot be converted to a Python object for pickling"
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_12StringHandle_5__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_StringHandle___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[57])); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_10) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":1195
 * 
 * 
 * def push_range(EventAttributes attributes, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainRangePushEx(domain.c_obj, &attributes.c_obj)
 * 
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_17push_range, 0, __pyx_mstate_global->__pyx_n_u_push_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[58])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_push_range, __pyx_t_10) < (0)) __PYX_ERR(0, 1195, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":1199
 * 
 * 
 * def pop_range(DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainRangePop(domain.c_obj)
 * 
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_19pop_range, 0, __pyx_mstate_global->__pyx_n_u_pop_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[59])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pop_range, __pyx_t_10) < (0)) __PYX_ERR(0, 1199, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":1203
 * 
 * 
 * def start_range(EventAttributes attributes, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     return nvtxDomainRangeStartEx(domain.c_obj, &attributes.c_obj), domain
 * 
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_21start_range, 0, __pyx_mstate_global->__pyx_n_u_start_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[60])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1203, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_start_range, __pyx_t_10) < (0)) __PYX_ERR(0, 1203, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":1207
 * 
 * 
 * def end_range(nvtxRangeId_t range_id, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainRangeEnd(domain.c_obj, range_id)
 * 
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_23end_range, 0, __pyx_mstate_global->__pyx_n_u_end_range, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[61])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_end_range, __pyx_t_10) < (0)) __PYX_ERR(0, 1207, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":1211
 * 
 * 
 * def mark(EventAttributes attributes, DomainHandle domain):             # <<<<<<<<<<<<<<
 *     nvtxDomainMarkEx(domain.c_obj, &attributes.c_obj)
*/
  __pyx_t_10 = __Pyx_CyFunction_New(&__pyx_mdef_4nvtx_4_lib_3lib_25mark, 0, __pyx_mstate_global->__pyx_n_u_mark, NULL, __pyx_mstate_global->__pyx_n_u_nvtx__lib_lib, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[62])); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_10);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_mark, __pyx_t_10) < (0)) __PYX_ERR(0, 1211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "nvtx/_lib/lib.pyx":1
 * # SPDX-FileCopyrightText: Copyright (c) 2020-2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.             # <<<<<<<<<<<<<<
 * # SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 * #
*/
  __pyx_t_10 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_kp_u_Domain_mark_line_902, __pyx_mstate_global->__pyx_kp_u_Mark_an_instantaneous_event_Par) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_kp_u_Domain_push_range_line_945, __pyx_mstate_global->__pyx_kp_u_Mark_the_beginning_of_a_code_ra) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_10, __pyx_mstate_global->__pyx_kp_u_Domain_start_range_line_1004, __pyx_mstate_global->__pyx_kp_u_Mark_the_beginning_of_a_process) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_10) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_XDECREF(__pyx_t_21);
  __Pyx_XDECREF(__pyx_t_22);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init nvtx._lib.lib", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init nvtx._lib.lib");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */
/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_object); if (!__pyx_builtin_object) __PYX_ERR(0, 506, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 609, __pyx_L1_error)
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_super); if (!__pyx_builtin_super) __PYX_ERR(0, 795, __pyx_L1_error)

  /* Cached unbound methods */
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.method_name = &__pyx_mstate->__pyx_n_u_get;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.method_name = &__pyx_mstate->__pyx_n_u_items;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.method_name = &__pyx_mstate->__pyx_n_u_values;
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "nvtx/_lib/lib.pyx":610
 *             schemaAttr.entries = schemaEntries
 *             for i, field_name in enumerate(dt.names):
 *                 field_type, offset = dt.fields[field_name][:2]             # <<<<<<<<<<<<<<
 *                 array_length = 0
 *                 flags = NVTX_PAYLOAD_ENTRY_FLAG_UNUSED
*/
  __pyx_mstate_global->__pyx_slice[0] = PySlice_New(Py_None, __pyx_mstate_global->__pyx_int_2, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_slice[0])) __PYX_ERR(0, 610, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_slice[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_slice[0]);

  /* "nvtx/_lib/lib.pyx":442
 *     Use :func:`nvtx.Domain.get_registered_string` to create an instance.
 *     """
 *     def __init__(self, domain, string=None):             # <<<<<<<<<<<<<<
 *         self.string = string
 *         self.domain = domain
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_Pack(1, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);

  /* "nvtx/_lib/lib.pyx":459
 *         pass
 * 
 *     def get_event_attributes(self, message=None, color=None, category=None, payload=None):             # <<<<<<<<<<<<<<
 *         pass
 * 
*/
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(4, Py_None, Py_None, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(0, 459, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_tuple;
    for (Py_ssize_t i=0; i<2; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_slice;
    for (Py_ssize_t i=0; i<1; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  {
    const struct { const unsigned int length: 11; } index[] = {{1},{366},{115},{135},{54},{52},{22},{28},{30},{356},{37},{44},{46},{44},{47},{1613},{1914},{2014},{279},{108},{47},{179},{13},{33},{400},{54},{65},{26},{1},{8},{7},{6},{2},{9},{24},{34},{50},{124},{62},{21},{15},{13},{8},{10},{17},{17},{22},{23},{20},{18},{17},{18},{14},{10},{20},{18},{17},{10},{7},{8},{6},{12},{30},{32},{20},{15},{14},{26},{30},{30},{20},{16},{22},{18},{27},{28},{20},{11},{16},{17},{27},{18},{12},{11},{21},{27},{23},{32},{33},{25},{16},{21},{22},{32},{23},{9},{4},{15},{33},{35},{34},{32},{37},{34},{9},{22},{40},{42},{4},{7},{4},{11},{8},{16},{16},{16},{15},{20},{11},{9},{8},{9},{10},{13},{11},{14},{17},{4},{16},{25},{8},{6},{16},{15},{33},{35},{44},{39},{39},{40},{43},{12},{30},{32},{13},{13},{8},{12},{5},{18},{12},{17},{18},{4},{10},{6},{9},{6},{10},{8},{11},{17},{18},{3},{5},{12},{25},{11},{13},{17},{32},{21},{6},{4},{10},{11},{2},{8},{7},{6},{13},{2},{5},{20},{13},{12},{1},{7},{6},{9},{7},{5},{26},{10},{10},{25},{4},{9},{23},{10},{11},{16},{10},{6},{5},{4},{7},{7},{7},{4},{8},{9},{3},{15},{11},{19},{20},{28},{23},{23},{21},{13},{13},{12},{6},{1},{8},{10},{3},{5},{5},{5},{4},{13},{5},{8},{6},{9},{8},{4},{7},{7},{13},{8},{14},{10},{15},{4},{8},{5},{6},{7},{4},{7},{24},{2},{5},{18},{13},{14},{14},{11},{3},{6},{6},{4},{10},{7},{14},{32},{16},{3},{9},{11},{4},{10},{12},{11},{14},{12},{8},{10},{17},{13},{8},{24},{27},{8},{6},{6},{1},{13},{6},{10},{13},{11},{12},{10},{17},{5},{4},{9},{16},{20},{12},{18},{16},{21},{18},{10},{12},{19},{5},{4},{10},{11},{4},{6},{8},{15},{5},{8},{11},{9},{4},{6},{6},{6},{6},{5},{29},{5},{6},{4},{8},{57},{0},{9},{20},{14},{15},{17},{21},{22},{2},{17},{21},{11},{126},{673},{22},{22},{91},{62},{17},{19},{77},{169},{11},{151},{215},{16},{33},{7},{49},{12},{274},{65},{18},{18},{17},{70},{62},{15},{9},{55},{11},{13},{63},{20},{32},{81},{173},{36},{124},{32},{46},{44},{67},{4},{4}};
    #if (CYTHON_COMPRESS_STRINGS) == 3 && __PYX_LIMITED_VERSION_HEX >= 0x030e0000 /* compression: zstd (5331 bytes) */
const char* const cstring = "(\265/\375`\007=M\246\000\032\246\014#G\360\316\352\314\300\304\210\255\034\326\354<\020L\317\0247\035\344\220\302M\013\t\245\250\201zSA\275\364\361\303&i{\271\266\300\340`Y\226\205\241M\221\333\027\202x$p\317C\266CL9\263\233\010L\274+\220\342\251\355\t\362\001\016\0024\002\242\306X\247_\034\267)\307\233\023\365\3114\342\312\321\345f\034\031\2069\353h9\3048^\313\335\314e\235\034q\375\336`\233\034\311p\227\343\330\257\016X\273\234\313\270\016\216\254\322n\027I\233\t\363\254\263Y\274\254\211\233\324\321a\234\232k=\304)fd\225\351`]o5\211\244\266\365\254\223Cr\2748\320\245r\334\313Y\277*\370v\367mU^\222\270\206\337\331I\304\242\306\035w+u\235\251<\314]s\2165\256\242\356\342t\233\274\341[H\216\371z\270{\227\343xu\307\034\333b\365$\223\241\224t\263*\252\204\345\r\361\014\304\345$n\007\363\334\262a\275b\354K\025[\331\260R\265+\223\245\330\030\220\306\203\005kT2)\230\205\001Y4$\300\226\035G\225{\343\342M&\263\205\375\316\254\004l\371\311\306=\213:\343\344T(,\023&\240\372na\3015}\267\265\361\022\207\2622\001C\305:\217\245\212\263\357\255z\250}\2238\\\311k\307\\\034q\246U\335\335\373V\\I\356\332\t\343\344QE\327\272\225\253K\216\264j\302\263\332yy\253\030\247\353]\227d\212[\021o\334y\256\341\335\303rw^S\335\342Q\324Wfw\030\356n\235M\322j\327\3706\342\360\206\321\335n%\314i\226G\353RE\254w\331\361\256\025E\304\266\360\366~G\222\324Uc\233N5\235j'\\\252\"\036g\227g\261r\332\314\t\207\372v\216\373\305;\224\331\216k'\027\227\212\271MU\341\270\223+\215\023\036oU&\333\3519-\223\325\033\271i\225k2\367P\227\264:\336\0319\326*\256\351\314\313\252\305\213m\033\347H<\206\245(\232t\2767R\254\033\343\352L\013\254#b!V\225\354\025\207\227\266q\327\271\352[9\026s\216\334\270\332\257\255W\316\357\030V\260j\234\00261o\321\2067\313w\260\315\222\335\3621\304\t\357\014\207\366f-\307\331\362^\222\233\014Od\016\265\216\363\222,k\271\353k\3535\215\307\021F\333l\226\265+\033\030\025\314L\010\032\026+f\255m\261R\260W\214\305z\305\254f^0\252\227*\006\213\301d`6-Vk\206\206_}\007\257l\0173\314\216\256ea:""\014\"\326\202Q\305^$hh\300\231\330\013l\355+b\245Z\261V1v\343\256\263J\3059\214\230\255\rm\2676<\305T,\230\232\0200\240\352%\023K\261^-\030\214\245Z\275T X\260\231\020/U\213\204\nD\212\005\223I\20151\030\253f5\003#!\323\242\001S31,\032k\323JY\0212/\325j\006\214\255b+\325\353e\223\352\020\363\002\355\213E\303J\221x\261bT10\305\352\200\202w/\355\350p\022$H\010\367\210\023\346\026\316i\233\265\2415Y\321\236h#\346\326\366\254s8\336\234S\321\245X\255\035\313Kj\234\273\266}\307{E\236\303\232\306\332\212\342\345P]\264\215\210cy\277`\342H\332\256*\006\204\211X\024\361\354\306k\300\030LU3\023\313\320\371M&\003\361\002k`V\252j\222\311V31\252\016V\025\263\002_2Y\251\362\2553\036\311W\031\342\035\2543\r\267N\224`\2350\037\257\207\020\357,\326\223\223\2169\326\265$u\\\347>\346r\306\361\214\343\315r\201E\245\245j\215\270\337\2151i\353\272\324I\304\273\304*2\0179>m1W\216\307Mv.f\035\025\021\357\350\366\325\271r\235eV'\3132\321\202\035o\234\230E\215qa\336\370\025\256\330j\302\221\250\022\016\343\221\237\360-w\031\251\365\335\230\206Ma\036E`\221\244i=[\261\325$j\270\210\372\206\372\346>\202\033o(\241\234X\335\355\244T7\324[\256///\267\273\243\342\231\353\356X\322x6\321\310z\373\r>\022\317\207\2244eTy\034\2759]>\311(\364\031\256I:\372#\036U\245\222\216N\024\221\337\207'\005\317\305\333\200\222\242\210\370\356I\301\3053\000\361\271>\311_\361\373\234%\236\013)%\215|\n<D=\020%\215}\023\220\337\347W\220{z\200\370H\275\022O\304\303`\222\232\200\370>'\354\347\374\216\3711\362}\320\024\337\346\035\214>x\014\337bv\204~(IJ\022\002\t=\010\2558\357\251\363_\177\306\213\204|\256se\301\1771\005\210L\237\362\"\274\021^\t\r\220:Qg\210o\342]>\202\311\341;\374\313\345u\340\311\240#\250\010\rO\327\231\373'\033\276\014\320Ot\356\177\302\221^\001\377\344\352\221\364Vh\312\311\277\003\037\242A/r\336|\023\020J\311\344\000\371\016l\370rP\013(\205^\352A\252bvd|\314\220y\263;p\246\374\021\020\351\217h\316)\341A\312\342\031\204|\340OA\371\030\023C\216\317\037\377aV\330\031\000\371D\037\307T\260\341s\241 ""\312\342 -\361!!\037\352\243\250\330%\330\360\221\274\005\363\302n#\003\200\251\247\304Mz#\223\312\264\244-& \217\345\211\240/\237\004#u\2353\037\343\347\370\007\t\322P\006\000\216\236\t\022|\340\213{~\270\340C\230\2409\002_\200=\014\274\020\364\203\202N\010_\364I\006R\022j\341$\235\243\247\340I^\303l\331\031@\241!\347\347\004}\010^\344\027\220\376H\207\017\305\203\202\267aA\272\000#E\3556\006\000 \320C`\000\000P\317\347\205\353\004O\332\357\360Q\237$\224\212\016\000\320\350\371\354P&\311\344\300\360}^\000@\301\361\234qj)\205\245ki^\212\271\262,K\333-\256\304\270k\343V\247\332\"\316m\314\361\036o\2313\273=\207\273\241Hn\234\265\374\364\302G\r'k\353\027\3271\247\321H\316q\372\266\311[\026\265\346\326\222\333\332\235{\263\345\036\271\345\331^\320Yg\235\307\233\263\265\226\245\302\n\205\n\250\316\361\355\352[\2511\026\273hC\322\212\326\006XE\346\343\230i\266T\305L\346~'\347\n\177#\n\361\242\r\222\213\005\235\310\027 \235\265\022\035\221t\304\245\241$\025\341k\372\341m\246\316\357\263\003y\030\274\215\364\003#\005\277\010?\202@\204\023o\203M\004\273\214\214Ti\346\333\323\341\355\240/\024\304\277\344\333|\330\225$?\356\020\200\277\343W\200!)@\302\360\311ljk\n!\364\313\001=\315\354\360\003\315\016\363d\307_\200\006\234\027\306k~\300HA\036\315{\001yRz\"\0204v\266\360/\337\000E\317\037\341\362\271\337\337\246\211\007\021Ay\230\021v\032/Wj\242\327\002\265\320\221\262^\344\364nx\240\314\311\007\370\320\2032Y/lbi\n\217\210\007\244\007\302\213\241-L@T\377P\212\340\000\t\272\322 \035\371\217\254\007\305\003\"$\r1\001O\2143E\375\256\271C\000\3627_\342\177\367\265tB\372\033\314.JQ^\200\0079\340#\335\370\025\317`v\206f\206{\236>R\026\244\037\336\005\223\224d\203\007\344\2411\245&\220\257\231\036\346>\260BA:\022}\020\037\222\242+=\032\006*\240\337\347\353\314\223\013m\351\251@YP\022\272\0023s\273\004\007~\027\355\200b89Z>\307'\205|\027\255\260 \215\375D?4\222\212\336\301\206o\201\337\256\335J\317\005\252\341\214\320@\245\006\365I\037\216+Ke-\351\212\313\364\330\020\335\236\007\221?1""-\374\373K\376\tsd\212\320\325\267&\357\006\255\235$\256w\341\023M\010IE\360d\220\366\317Oa\272L\227R\217\350\362l\034\027\265\255\374\210g\220\001\276p\365\300\236\014\006z\372\001~M\220&\000\3012eV\244\214_A>\0109\346\307?\351H\373.\001\350\227\323+\320\203\300\323@?\316\027\023\217\324\343\314\000\360\241\351\321\310\320\310\210\360%\224L\017\273\r\320\357w\277\353\203\237\205\311bvl\370.\3123\030\371r\376\211\017q\220z\320\2130=\316\217\257\302\374\010\375\\\276\301\350\347\372+\243\377\342C\3612\240H\351\350w\317\222St\242\200\370Pg\204\010\035\351/\031\005\205\202\250S\210\302\034\226\214!\022\021I\222$\215\001c@@`\210\\0\035\322h\225\356\363\223*u\234\343A\214Hb\210ADF$\030\021\231@\202\t\222\224$Ik\036`\006\360\tg\231\372\222Sil\252+\234\t\3018lB\336$\365+z\366\204\277\3752\377<\350\036}O\266\235M\3452\2262|\271\320q\366\237FC\220jB\323\210$P\2661\311\327\315\010\362\227PZ\\>ep\240\211\215v\006\310/\261\326\201H\334\3019\2135BY2\257\200\255\234\211\353\233\223E\036\206\277\220\306pZI\333\244\240l\331\314\307\027 >6\205\260O\371\250\027FtL\261\333\372=R\250\243\357\302)\351\035\273\321\304\303\r\302\252M\215\360\232|\026b\212x\013\316S'>7\300Z\251\367\233\376\362\021\227\266\021\362~\024\244\352\313\324'\001\\\257\333\004\":\237@'\235\025T\261\320C\336\302\224\035\317\010\274\232\264\236\264C\370\255\014&\340\247\314#\3533J\367\336\336\202\366\325\005\250\024\313g \t'\364\3751yP\033 PA\203\204y\265\222\234_d\007\366\241\267`#@I\245G\322v\222\322\214\213\213Z\212\3227\020\246n\201\236L[k.\264$\020\214\024S\026\215\314\242\306\365\006\231k\213\251\270x\212:g\256(\263\267\332L\313L\312\030Ulk\016LA\242\n\013\236Vy\252\244\340XuS\001\025\327\027\265\255\334\304\t\335\217\373\214\215p\024\266\204\312\211\277Vo\021T\272s\366Y\325#\177\"Am\020\234\352$\363\000H#\375\374\"\037#\352Xw\327'a\302Fa\"\227Z\304s\307\251\252T\213#\016p\234\303N\023\256\245\177\233\201\211\000\204!\346\236\352X\2314\262\352\013\336~b\3324ZR\332&$\305\270'\325w\310X\375\207\320\276\215\304 (""\026\245\024<t\353\244\006\334O\204\370\232\260\301\325\236+\205\310\211\r\267up\315\230\213\242\307\017\323\206\321k\250\353`5\241\330\240B\0248\270\366\273\300\261r\270\214\025\027J\324\305\355&\204\226G\261\353\235\374\361\346,\203P\014>\024\263\362\322\277\3012H\341g\004\345\272\002\324OI\277\346L\2024\364\372\265(\243M\215\214\276E2\2778\375\200\020\361\275bC*T~\201(N\271-Xj\301\021\371w-\2609\230?0c\321|\374I\3572\234\024\324\205\001\370\311%\341\324\347\255\025\226\035\035\256E[\261\2609\371\266\\\013\3207\370\231\361\342u\177\356\361+;2\007f\277!\220\232I\n\246\342|o\360\007\332\"\2315\350\261\023\215\215R\201{\265b\247*\312\211\031\313mK'\363\026{y\356\2434\226\350\336\010\\\r\002\344\t\364\306\252D\262\223G\204\267\213@\226S\211\006^q\025[\r8;\335\347\317\215\256\256f\330\217r[\336\n\021X\"PA7\003\305\201\247\274\300\305\313\033\301\271r\004\304\034\335~\373\364\233\022\223g\214\274LTAJ\306\350\312\226\272Xsl\353\346\023#LX\363\232\303p;\001\315\212\337\347R\254H\361\365\236\275M\023\374\372\376\004\376\245\254\033\201\311\037\212\017\350)\376a\252Z\206\252\276\320K.H\342\321\027/0\335\306\251\317\356\326\035-F):\311\377\332q\215p\343\376\200\n\247!zK\327\n\367\3748\200\336\343a/\233\217\033\240\037\267)\263\351\002\013\306%\226\354\241\330U\256\257\262\276'5\237O\357\250'.\261\263\177\364\244\267a\277\207\"_\334\001\031\277%F\253\253Z\361\236\014#\020\306\257\\2\034\373\370\317\211\003\000\200MI}\037b=\020U\270\020\365\235\210l\013\233n\036\034C\272t\306\204\005\210\314\216\351i\242\225.\034u^2\342>\223\001=\352\301m\306(\245:\235e\025\351e@9&\302u{\212Id\025S\251\177DV\032\277\204w\351f\264tyt\207\350\236\2610\215\230\367\r\374I\223G\343}\361\347\277\244e>m\033\003\276\350\312\213\274\022\223\026V\370\364\246\301p\252\227\"\016\224W\000-.\367\203;X\250\030/O\276\177\362-m(7\334\001U;\307\324\t\363\tc\037\202\2524F\214\260/\363s\304\314\"\255eZ\263g@I\"}\026\024\314\336\316\205\207\231\020\2128\017\350[\367\021\221\232D\202OP\302\377\013]\267P\311\254_#""\346\236\220\362\017c;\222\005\242\223!\220C\n\241t\364Q\211G\226d\372\301\237\027\355\213\207\254\357\253\\ \244>*\366\247\377\006\362\327\301[X\007\271B\341Q\272\340c\277,\364-\000\314\220{\344K\264\200\310m\205Xpqt\367H\302N\353\177\241\332\034\272Y\233\256{W\230\334\370\3474\2164\364\300\350\343\205\333\255X\252=\346zgC?\tb<\203L\317\266r\273\002\306\267\240\362.\260\207\215\217\270\216[]\021\035\332wS\312\343\210+D\230;,\256\013\257E\240\337\330\326zH\305\306Tqt\025MF\350\242\341\237J\302mR\314*\316I\345Z\252\337\232\014qp+\255r\347L\252\001`\350Y\030/\210\003%7.\223\244\311-W{\211\034fN\255N\230P\362\365\\.L\333\262WeG\333\220Q\002\r)+\210\347Y\345\374e#\232,\303\300C\267+\310\"\206^@\311\234\207\305\311\254\327\244\023\350\034Dql\010\032\260\341h\242\223\272kC\260k~\030\035\274\204\267\334D\177J\037\233Z\030\257F3\236\373R\351\224F\242\357\\\263\352\304\310\217/P\203\313e\342c\215\355\020&}\306@`4\316\244\230\241\353\225\223\217\315\207L\226\206\304\353|\223\361\246W\237\r\"\325\375\002\321\3501\344Jh\206<?x<\350eK\344\353l\376\300M\226V;\243X\237\246\004E\377i\203H\316\n^\327\342\200\210\375\017/#\330\323|\272\2050\"\340-t.\021\365D\356\242Nc\375\217\214\305\222\014\256\017\220rU\034\315\002\301\216\223\263\003F\242\330m\361\3609\262\346\t\334XF\327\322\02501\245\320\310>\024\363`n(3\034\270l\024P2\226\207\005\303P\200\240:g\027\024\200\001\251\356\363\333\230\251\336\235\267t\261\242\225\274\310\266:\236D\361\316\201\206f\301\340\346!\016\250\353\340\365\265\tU\341Z\246\232\236$\207\222\327\342\030\220\203\267f(W\205\031]\3147\220\227\341\264\033\340CE>\277\266\237\344A\240\314\321\243\031TZ\377\225\376\234P\034^~h>\301\270\233L\224\202\240\241\n\242sF\264\333g\311\004n\3227\275d-V\227\257`GV\250:d&\274w\221\207@\r\277\346\033\3621\307\020d\305\272\354\206\247\007\014\246(hW\022\264H\340\366\330XW\210\0050\342\342\326L54\025\246~\321\231M\356\213\000c\336P\320\225Z\324\027\313\340~d\331\352\263\316\024\272'\006\246\036\020~1/\364QD)Ka\267\336\234\367\"qx""\267\226\271F\250\357\266\375\007\010;\001\276\020h\277\355/\316?\241\264.l\232\217%\215^%\025\244]\300\254N\3506v\322a8\235\025RV)\336y\177z\013\351\035}\220\316I\247\370\354\361\354\242\301\350esd\270\370\367\276Y\350\375G\263\177JS7\235p\005\363\234\204\220\347\275j\027ttl\365\261L\353P\307\255\274@\224VXG\224\361\371\220\304\316\2377\252u\374\236,\352\035\263\232\207\254\264\357Vf\372?j\364q<\311\r\376R\357\2023\325;+J\361\203\034y-3\0322\273\365\\E\326\3440\"j\004+\037\026W@\002\t\221\324N\357\311\214\255q\006\311\372\256\036\236\016\244\020`\354zY\274\313\262N\343\206=RC\254\375o5\246\010w\\\223\323Z\316\0203\227g\351\257\035\002.\233\331S\3243X\310]3\314\345\240\022\017\27276d\315\221Pc\030\032=\335*\257\"\303\365gZ_\321\344pN\264V|\231\275\200\003<\277\031_:.\210\271|4\210+\322X\376\264\267w\326 \254\332\001\n\270\237\265\370\031\272\022\235\205B\003\\Z\261\310\311/G\233\252LU\357S\276\320F\323E\317\240\232\235\031\363_Z\003\023\255\032z-\007\376\tc|\022{@\321:\314\031\nxim\256\232\021GTX\334\364\352\032\266I5\214\367\234-\236\tk\346\335b\246\266T\314\375\370wtz\321Yan\304\303\013\344\306|\036}\254#\003\220\233^\016\261}%s\355IF\326:\026\247b\250\253\025\224\361\0223;\033\256\272\027\340\212$\301\342\235\177\005\021P\256\326\223\200\222-\034\235v\024\300\267\213N\244\207\321\267\243\013\346\376;[L\230\266nD\025\013\032\000\307l\234\037q\016\323Xu\250N\004\211\372E\275\267U!Y\342\317\216k\344l\350P[\261\005\236\225a\275\007\016\313&\266\0138O\235\265i\217\202\\b,\177\325\326\350\254\223+_\270\304\361X\213Z\3158\224jnPZ\t\342a\200c\376\341o#\230G\242\307\326al\374\304\336^H\n\204\010N\031\233\356\303\026\020\005w\320\"!\257\211\264a2\021bU\365N20\317+M\371\254\024\335\020\357[\023l/\003\264vf\000TOR\321i>\031\253?\242P\270{\267\026d\200\346\244'\312 \246\324p\340o%\013\322\217\345d\314\030\004\323Q\237`\240\0064<\215\313?\231\353\023\205h\013\206X\235]\220\350;\214\021\034#\321H\033\252+u\341\252]\376y\300b\362\273\304!H\321\001:\374\312\376I&\207xWn\323\001k""\245<e\210\267\217q\370\300\211\330\341\024f\221\000E\313\270Zao\275,\305\007^\311\2435\343UH\352\204\356\314\252\274\274_2\221Q\252\260\3523\231\340V\342\255\232Q4\007\366zC4\327e\014\300\370\333\326>\\8L\212\230\347\216\200\201<q\032zq\214\237e\313\010W\024\225\220\310S\277\007\202rPe\242W\246\207R\354\273Jp\337\221\026\271\250zE)\341\233\230\252\252n\370F\215\374\243\032\005e\233N~\276\265\026\215\367A)\t\034\247\350\325\277\244UD\232(\343\002vsjN#\301\027\nL\304M\260$@\001\036fN\310\356r\032\227\300\024\364\365\211'\323\321Om\030\300\223\005\272/\246\225mU\tM\205\242q7)?\340\027#^\232<\331\233\rmau\200$\323\316\3165\371\006 \244\261f#3#\276B\204G\242W\341\021\232/@\2002\233\340\215\270@\212\030\353\244\214W\210-\r\020\364H6\260]we\216\220|\220\362\302\313Q: (\001\212\333\365\006\023e\213\n]\016\357\231ja\325]\203\017A\355#\336O\2474myu\233\313\036\301\025*\265\262\0323)\226\315*\035\330[<\212x\260\022z`\231E\211\016\215]\310\244U\3114\327\375a|\035F\341\021A(#\236c\004\361\316\026i\\\323\330n\010\030\247<i\017Z,y\225\306\267\310E\335,ch\276\303\035\204\333\022\"\310\023\006\252\224\323\337\000*\016\r\201\342\363\376pW\273\307.\000\373\204wa\2109l\264\t\025T'\000!\020\026r\334MA\005\240\355\323 \026\014\206\206\263\235\314\354\tk\211f\315\220\020%\271 =\023\005!\"\327C\037\247\023a?\340\341\264\r&G0\270\325N\314\316\204\216\201\356\274U\363\230\267\343:\217,x\244B\267gF=\003";
    PyObject *data = __Pyx_DecompressString(cstring, 5331, 3);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) == 2 /* compression: bz2 (5477 bytes) */
const char* const cstring = "BZh91AY&SYAf; \000\006B\377\377\377\377\377\377\377\377\377\377\277\377\377\377\377\377\377\373\300@@@@@@@@@@@@\000@\000`\027\235\337\032\"\361]\366\327G_D\347\276\355\347\335\323\257\2402\272\237K\257\037V\306\267\231\331\276\201\311\350\320\210\000U4\017<\334\250\000u\367\010\0008z\206\232B\0216\246\214\21514\323\020i\243\nc&\2322\004\324\311\247\244\323\t\351\350A\220\2314\014 \364\206M\246\220\006\203\"\032\004\002\020S\364S\324\032z!\351\210\324h\364#F\200\r\000\000\000h4\000\000\003@\001\211\242\022h\231\024\030\203\324\0004\006\215\032\000\000\003@\000\000h\001\240\320\0000\200\t\021\0212$\364\233\"\217S\321<\246\324\364\365OQ\352=@\r\250\332\231\000\000\r\032\007\250\000\000\000\336\244\032\007\250\001\264\220\246\020\320jS\332S\321\250\323\324\323\010\332\215\014\207\250\320\320\001\240\014\004\006 `\232\032\r\000\300\206\200\211BhL\204\320'\244\302Le\024\3754LH\364\364S\322?J\036\240hz\200\000\000\000\000h\0004mF\214\027\267\212\342\r\315\271\265S.\244\320\3001\322\234\202\312\300\203\255\030\270\0301?_\306\366x}x\375\317\360G\034\216@\034\220\337 \200\3453J\203\216S\002\203n\\\3329U)\364e\364]\274\337\231\216zd\026,\177\351\366\261\273\032\251\366\241$I\n\n\005\357\264\317wr\377\360\033\037?\235\317\206\t\2647AH@d\002@d\220\020\221\010\033A\244\032DLB\373v\350bE\014-\332\005\302\346\207\220\265\224$lt\242\343\\\212\276+\226\270\177)\332\330\035\260|s*\333\300\273\366[\003\263\353\220\240\036\234\007\0334\r\336\313`\177\320\030\014c\366\035_\357r\363\036_|<\355\272\231\002E\204\t:\273\004=\210\310\022\022\"H\004\214\213\010\000a\nA\355\304n!P\267]>\221b\010;7\006iW\004\240\"\210\200\252-\033i\2646\204\352CDQPx\006\212v\335V6V\242\261\034B\010\331\025Q\262qL\334\223n\315\0347M\246+\325\362\340\226\302\215\204hI(\227#\"\227\304\321\3451\254x\321\205&\020q\316\217\013\246\324\270\233\t&\246\231\333C.\020d$\0238k\230\231\023\232\024\261\206i\010\220-\235\213I(h\005`\021\031\t^\325U\003\367\240\022F\240N\313\r\250b)B\024.K8\214\263\214\330\023|o\n\2652\277\230""\350D\014\361\222\t\014l1\021\210\214\204\247h\307\0313G\263X\324F =\206N\013$\271\304z\215\327\270b{\212\n>d\0046x{]\324\377YO({u\335\013\270?a\367W\237\344\243\276\337\331\303\363\0222<\203\311\344\333\271{1cH\016\004\002u|M\266O\217p\231\246\356\243\271\353\335Lm\014\370\376s\375\212\003`\332\326d\261\267\221qc\tE\250\315\025T7\226\223f\222\344\320\262\211L\322;\024\214\351B+\031\201K\230H\202F\234+\206\226F\246\261\nn\255\346&\035446\364U\r\013\303\244\027\250kE\305\340-\216\363\220`h\2132\317\020\242\002,N\356\206\207|\347J;\210\000\221\311\000m\253o.>4EDh\367+\361\316\004\353\240\260pA8\302\nM@h*\230\024\300\207\302\343?#Z\rB\370\357\307aa\301\216\275O%y\334\034H\327=\216\203\322\340\341g\003H\306\017]\247\237\3431\230\326\211\222%\205L\315\227\213K\035R\306a\r\306\373\250\260\320\303\033\312\232\200\003\025\216\242\214\035@5\030\374\212%8^bfV\005]\252^T\315\262G\000\371L9\326L\336\300*\253\033\365H\010\352\036\222\230DV\205\313P\367\241\254\261\2244\361\245K\314\306\n\022\001\221B\372\360\330Y\005\205\2538\251\211\264\325\201\014Np\367\tcQB\367X\245\r\306\001@\304\364\346d)B\353\215\010\003\010\311\021\321\254\350g\213\002\315\215gT\003J\3316E\221f\326\000\214\257\001lA\250\363\316\020\007\024E\230\n\"\234)\3337-\350\220\020\341\271F\036.3\022\020nv\027\340\210r\030\314\337\tMb\030\013\313\371a\210\312'\014\372.\305r\024\026e\351\354]\265\354\247\023e\021\021\310\232\373\300\364\n\026\323\240\335\255\320\307;\2343;\\wX\034\r\234\202=lw\240\002/v\207t<=\\\343\340\344N\325\364\247k\306\321\233U\267\2541\215\361\"\341\025aV\020\322\212B\212c\241G\307\006\030\024/\361\014`\"m8:\245\n\323,S\214\202\016+\374U3\373\220\324\251\221\331\232\224X'%\304\246\216\001\332\001\025(\204)F85i\200]\215\037\230\354\267\257\360>\037\254\207\274A\t\361\201\365\362\323vt\331\313:\374\332\235\241|'\253\227\240\346\261Z\364\336\316\233o\\hQ\263\203\320\34080j4\201\213d1\034\0029\026b\372.\233\274%\020#\271\226\222K\0046(\010\017\275\272\333D\306\2121\0221bT\241)\024\357\240\242\030""\255\2460\3227\3675\367.\306\203\243\020wx\355}\237N\207Z\322\010q\323_F\354,\324Y!\025<.\017;\030\027<\250\304*\252b\000\226\237\010\013H}\226M\3060+\263;>\023\236C,\016\016\375\035'Y\265q:\236\370\016\225\272\334\036\371\315\321\266\316\177\001\314\331\263x\031\216\321`'\2238\t\034\343N~m\325\0214\266\335\222\372\007lp\035\306$\365\333\340\"\004*H\221\024\240\3670\307\225\337\304\207k\303\233\217\273a\002\362\025K:\331z\233\340\013\330\007\224\001uFt\267\230x\032\310\252!\017\360\373\177%\372/\241z\357\216\025\036\260\2026\224P\337\210\276\241\203\005!\031\200\001C\313\251?S:^h\023\035\\8P\202\001k\336\314\340<\200\311\006\204\0057WP\204\335\320\002\367\275\305\342\201\373\271a\200\373\341\370\270\222\207Jl\321\304\274\321\232\017/\310\242nD\361?$\261\2200\372\2620\263\350u)\266\\\354\215\260%K\336\221\353a*\215\0043z6\nZ\306\0070\026\314\260\252\305\214d\030'\254\2355\3419x\nJ\311t\243\356\027\325\357r[l\255\262\306\017\204\365vr\270iG&\320U\177\3341rm\300\203\n%\304z\330\302FH\254C\033J\004\014!1\360\344Ja\245\216\021\321\305P\377\222zd\032\351\265\211\263M\232?L>\017\027\177\021'\247\300+\245\210\367x\336']\207\253\327\341\341$!\275\364WJ\374\020E\241\033\333\010h\313\303\305h|<e&1B\202\235}\035\371 1\233\361e\306\024\362\351\210\202\336\007\260H\024\r\241\032\225\243\221\364Jp\317\210&Ve\241\210!\025RB@\216t+\234\037\010\025\034}\256(\304\001H\031\000U\300\017,\370\356d*\037\204O\023\3611F\035%\237\205%t\262\237`\313\020 \312\211#D`>\220\nIRR\361\317K\230\246W\323\010\206]&G\007$\324`P0\303\3642\022e\237\336M\241\023N?`\206i\345ilZ\014A\232\003S6\3129\255t\355\206\261\257\300\310\032 y\257@\241Q\317\026\027\007\337\rF(\341\253\236qF=L\363F{U\324\320%\007\310j?P\034\224\t\020\007#\224\0204\352Ad\004\245[\020b\t|\325\304\360!U\2163\226\301\305\022\344 `\220\202\201\303w7p<d\004\234\336jv,\317~\361\352\263\335\221\025D\331\003J9\346\256B\216\307o\262%\2015<c\226\215\316v\034]\2059j\\\006\371\007\255\002\274\032n\274\365\225\352\351\251\253\234\227\010\3435%\222L""\354\247E\030\010\212\261q\304\002\314Y\325D\002\006\314\231V%.z\323b\240\270\374\240\006\3446\206n \321\017\324D\007\204\032\201\244\355\271\002\302!{\005\207\035\377\010\3434c\245F\314\3544\306W\352\256-&\215XCD\354\323\305\356\300\2253\0002T>J\rR\311m\343\332s<C\272\333\202\266:\304\341(!;\234l@\226@\214\363\224U\344OY\334W\303~\250\212(\021a\263\332\034\\\344i\261\273A`\032\222+(O\243\332\264\200c\033\233(\326)\200\320\333\316\372\246\243\203\030 #\300\200\351f5\033f6\363R\205\220K\204\352\211\3138\260%zm\222\027\234\366a\243\252\302\036\255\262\231\242\235\037\021\230\006\014@+z5\305H\003]\210\216gH\033\206\030I\\\007\r\266\3447T*\t\327\2148\332\240\026\255\010-\2310\242\033\304Xa\220\260\223IK\006\212F\230\302\300\245\013\026\014\224\254\334\026\306\225\331\271&\246m\341a2\250\340MB\277U\310\034\025G\234\224(\220\267/VHoM\230[\263r\265z\262f\002\207qQ0\263\006\030`F\004P\361\016=\020l\264\264\223!\201\020\2011\245\263\264\266(\372\215V\306\373\331\255K\220\217e\016\361\354\210\024_\371\240P\271\260\034\242\232R\241\366\\>\263\200\"Sj\276\235i\031\264\306\370\\\336\362coN[3\034\004\313V/k\260;2h\205#f\035P+\361\001\013b\212\\\333\342\376<9\020\230\336\270\347\313D\207\2667\204\301\323\301J\3452^\260\322<J)\255\301\261ndj\317\036P\356+Xma\303\307}9 \2228\253}\270\275hj\216y|\364\356:\035B\361j\035\240O+\001\314\272\305\"M\345.2,\317\316\030\225\346\335\r\025\367\213\025\013\354\250\226 5[`\021\335o\357\366K\231\314T\016\320\225\021B\266T\262y\0076\266\"\227\276V\241l\255\304N\256\320\371\371z=\t@\016hc\033\2773\021\370\305\265\261(\212P:\373,\353\207E\223(|e\256\371\346E\363J\320<\276F\303b\362\3746m\207\"\024&\315\037b6(\305\305\017\231\030!\225\325\203\236\177\0266,K\303D\260.\370~\333\243EyI\027\343\202\370\205\222\002\251\314\206\305s'\266\246\261\300\274\0347!\210\274\264u\251%S\367\251\220=\243\334q\260e\253\034\322=\025\001F\236\032\023\003\324S\001$P\205 \241\251\032J\304\300\032\r\022Yr\2030\304\311(o\240U\022\2452\201\206\016\263\246\216\264\002\305""\332\310\233\345@\274oU\202Sbs\020\006\371J\234\325i\271\265QI\356X\n0R\264\rt\276%YL\361\003L^1$\301\014\260\2054\257\016\035\207\207a\275\2173\277i\357\3570~\2134x\271\2461\261\226\352\201H\211\221\244\247a\006\252\300U:H\321\006\351\225R\330\307IbJl\022\261,F\022\030\266\245\014\316\352\263#~\036\331\n\300\031D(J\005\341\210\323Hh\263\013Z\227\231\213$\202I \236\004\261.\201Im\343`,\r\001\033\332+@\345\311`;\005\264>\014\234\367\353*\335\255\251\216\270U\245\352`\363^\355\225\324\211\3274\022q\201\211\231|i0\310$\020\007\330\241\235\014\356y\002i.9G\327\n'\313R\363>l\352nq\301\332b\216D\010\014*\255\271\264\2656\362 f;2\2637\250\026\362\305\242x\364\350\331\310\344\\3\030\034\355\\\374\2655\240\322\3339rs\201\"\310\001\010\213\216\275\271l\247\247\311z\3268\344\341\016l\223'\230\320\332=,\325\367c\330U\351\203K\320SN4\027uq*\271E\261\247;S\230 \334:p\250\310\310!\260\212jA\344\224\275\212o\327\246\324\271\263eU1-E\370\216\025\307>\352\355\254\300Y!\031\006\001\021T\0333S\22168M\211Kh\357,\202\310\307\334\2105\001<\342\213\344\262?'B5\3664\215\270\2238I@\303\020\266\271ZS\241hJ\366\3672R\236HR(B\030I@\343\372\337\311\201a`i\344\321\315\367\023\365/\214\325\341\303\247\315\000'yDvB\250\304H$\022E(\312\234\221\310\363Pe\000\202\307\003'JKR\003\nVBJ7]\006]SMDDCP\215\r\003o\273\270|\353\025\201\272\233\226\2645k@\326\"|\314\t\025\204\027\003H\212z\252\373d\221\312\213Dq\200\032\375-O\267\363s\373x\352\001\326\016\341Ry\307\023\004=\034\327\354\340\246[\007\222\005N\035\257\310@\014\206\350\270\304\307f\252Y\031a^\340\272\240Z\032n\324TW\020\240\224\t\t2m\215\373\374\t%u\r\nbn\355#\326\316\367ek\323\000\302\004\314\245uyg\240\323\221]y|'\204\357\231\027^\275\231\016\350\t\315\273\206 \364\014_\305g3\246\017\027Js\274\310\006+\26446\356\322\272\032\000\337\023\020\310o\322\031\006\311#\352\362\276\330\206\331N\377\251\230\234=\232<\035\341\r0,\373\316\233\036\260\n\\j\022\327RP'\210\326\316\211\251aA*\320T\311\350\226z\313`\232\033\210P\325\006C\270N)\221\225e\335Z\215""\336\003\322kP\234\353\313V\370\315\335,%\026Q\246\303\\\021x\226\323\204vt%zE\306\234Y\2158\320\275\203\031\023Ew\266C\335\343=\334\361o\225B\212\004\005\0161{j\207\305:\251\023\000\241P\356x\323%\332\022qjt\3528C\003\343n'\214!\t\215\327K\304\215\234@%pl\270F&\000\032X\272\037\010\276\261\221\221\341\226L\010^\217\006!*UJ\004(\306\004kE\251\025\031\006)$I\032\205\0208Y\321\347\215\354\370\266\255\336\314\352^\313\n\2549\333\326\270\016\314qR\303\224\335\n\312;\n\n\325\024\201\224\357.~%W\004\327'(\030\221H\323,\266e\243_\204\250iv\221\002\t\025\200@\t\001\223J7Q\201\032!\\mG\360B!\"Hh;\027S\266\230\226\036\243|\344\360\317[=\374hr\366{O\265\354\253\342\214!\317\013\303\332\352\0019\236U>\257.\030&5W\322\370\001S\310d\016\310\375>&?\0369\003b\211\242\237p\300\203\030\004`\021\204c\030\221\200F\020\336\342\303\222\024\006\002b\350\035c@&\252\357}4\214d\021\r\242(\366\t\255\315\265\252%X\211\231\206\n(\220\266[T<\237;\222HH\214\201\301|\236z\032\301)\253,\222\201\311Nq\260]h\277>\354\232Z\255\211a\323\212\246[\220\026\0264\322S\006\257\221\004R\223\231<\264e\212\033pw\206;^\177:WL\022\200}\326\216\271\201\256\335\345)\024\251e~\354\346\324xj\001L\217L\236\357\246\036hw\303\327Kx|\217\311{G\013L\032K\211N\256&\032T\207d\230\322Z\255\254\261\000\304\3248i\032\250#T I\313ZV#\"R6\205)\347\263\274C\241 1\3644\016\374Q\310\273C\212\275{\023I{\201\354\017W\214\312`\004 |\246\253\304\327\266\257\216<\006$+5\354\255d\274s\301\tTRGt]+I\024\335\301\245\300/\200^\244'fn|\351\340\271\026\023'\025`U\206u\245d\032,\241\007\276\303P\253h\205B\004\031\036\336\303\261\271\354G\245\213\303\023\337\217\033?\006\245&\356\211\200\274\375z\352\360\316\354\201\233@\344\231\316\370\242\244\t\201G4H\022\022\312\213\204\r\003\226\246\002CrT\033\314\251\226\331fL\222F\2269w\020\006\255\340\321PGD\204\"*\240UT8\006|Eg\276\020T\302z\225\2005\362\256A[Z\004kV\020)\002\243HV a\025\007a\016\022\317f\027\022\330\225\241\323\312\314\302\217\333W`E\\4\014I9\235\315\305\216\027\323\002)\0250\225\257,k""\025\33695\271\026\006\301\337\3424\030\252X\220b\021\374\314\211J*\255d\003)\t\342xf\366\246_8\343\231\254X\313\235\263\250\361k(\007\221\214\261D\301\rN\032\251UR\313jdqk\033\021\302\306-\200\301\203.\323\030.\026\241! $\217=,\033.\266\2420\002\014\t1`\230N\371\026\356@\254\214Le\244A/)\026M`\023\241\302\226\nuQ-j)\251\022\346\215\025\214\264\311\200\030=n\373\016\373P\326\006l\002\021!*J\003\r\355\250mcM\206\240\350\300|LVD\207$+\216\303|\020\2579\321\004\037\366\202x8\005\007\027%q\266\3250]\234hQ\336\3550\304)\017`C\227IW\221\314\0230\265\016\352sc\213E/\005\346\366v\321\024\256\324\201\363B$\017%\2506G\201\324\323X\352\007\304m(\300(\305b\020$#&\367\004\260\365\212u\001i\310E\222\224)L)\244\265\375\223\321\037k\243w\321\232f%G\234)\017S\335\353\365\372\374\356'A\263\247\3413J?P[\224\000\305\016\2249\005o\023/\033\023\261\251T\3124/\244\351c\344Y\355\241\312\212\\\361\372\014\236\274C2\275\334kD \237$P\202\020\034\371\213PO\333\316\300\2353\267\265\356=;\023\336\013\361\344zX\374\254\016\370\362\261\002\254\017C>\2517\033M\276\255\2160Bn\2224\350k\320\023\246\033\203\255\346\266\036\307\245\273\335F&eIFL\026wi\020a\263994HDP\013U\211j\205\nP\236f\252>)~}\350}\351\0252\266\2155\001\257\010\242\002B\217\260\201E0\344\004\022\321d\252\313\027\025f9\306\217\274\362cWg\003UO\236i\272\236\264*2iJ\035\033l\231\244\357?\333hm\273\257\350b:9\376&\t\264\333cM1o1\264\037/q\214\310\263\240_z7\023\347\351\376\236\013\200l\345\215\033l\201\254\374-\r4y\343A\364\003N\007\257\273\037\252ixL\366\225b\255W?\000\345\304}y\375n\027\001\3431unV\004GA\177\003\016w_\035$=\177n\333m\252|\243\010\376|;\036eb\353r\260Z$\202G\317\356E\007\264\252\000\337\316]]\337W\224}B\251z\2625\344\017#\307\324@\336\260\322G/W_\266;\200/\371\246\217\006\363\205\264y\027\244\\I\014\323\265\313\351\354\2375\265\021\221\362\307\225<\367\371\356\020>\277\330\\\326\r#\231\213\323\272\236G\005+\372^}\357\205\304\034\336\215\245\034\376\357p\326\004^ p\037\305\277G\247a\313\264\036\375\336\377\n~\t\337\3659""\275\262\034v\237_G\346.hm\235\2675\255\014f\320,\277\267P\263q\314m\277x\350\355\260\235\226\245\346<n\367c\013\213|\337-\272\346p\265F\340#\376\361s\021{g\361\260\342\0346O\345\261\332K\333K\261\203{\330\335\222;\252\260\247s\207o\211\255]\306\245{\251\312\250\232\373\213tu\267\357\354\335Q\300]y\267\274\272\035\275RUT[\267\345\263P\337\027\003mS\035\222?\013,,\337\324]\036\326\226\t\014\321u-\n\325\330;\013c\340ld\216J\365\273\032*=\352:\203\263\243\244\007\363c\226\260\353\230\375\t\333\335A[\277l+\007{\034VZ\341\211\361\364\246\204\320\360x\376$\302\336g\221j\266\266\3045\367P\347\350ngu\306\3531\206oG\021\236\355a\227+\265KJ\351\335\330\360\345R\035\320g\340\320\240\010\340\002p<c\036d\325\031\330\374_\307\311\026_\001\223*\252\333\002t\2103?L\336K\337\\\217\006\256\026\334\177\373\352\277\215\244V\314\211\320\033,\332\254\325\364\247\243\314<\342\"\240#\254\234\211\337\373!\352\2175\375\323q\014_m\241\201\247s\215>i[=\2007\246\307\351\2445\274\023\371\004'\275\213\227\316\355\355\341g\340\367B\2470\221D\3118\223\302\250DW\2027\343\333\025\314!~qS3\2245\224M\222\025R\036\212h\323\207\306\306+\016\314\004!LKA\210\027\250\264b\022\233!\211Qx\265\361,T\227\243\272uuq\215\025\242\301\221\241\301j\013\272\273O\214d\006\005\257\200\342\"\021\270\0101\212\001\347\220\030\214E\364\315bXB\340\312A\221B\010\341\321\275#\0015\220U\017\233RBQ\n\021\241&FGI\320\2139\037A\305\2334$\263S\023\224\310\312\364\200\233\300\252\213:\261\231t/UqT\322\224\\8\031\177\314\3419\242\326Bn\010Py\3529\362\351\333\223\227\377\027rE8P\220Af; ";
    PyObject *data = __Pyx_DecompressString(cstring, 5477, 2);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) != 0 /* compression: zlib (5402 bytes) */
const char* const cstring = "x\332\355[\315W\333\326\266\017-mIB\032\234@C>\332\212$\024H\300)4_\245m\356R\300\t4\t\306\006\222&\367\335+\013K\0065\262l$\031p\273^o\206\0145\324PC\017=d\3100C\r=\344O\310\237\360~\373\034\311\222m \274\256\276\265\372\326\212\027\321\3079\373\354\263\277\367>\347(\311S\002~\242\220\327e\313\022\354u\331\026\312fiSSTK\220\rA3l\325,\310yU\260K\302\302\363\345_\004qq^(\253\246\240\224\212\262f\214\263\341\262\241\320(\245\222\307(]\005\242\322\246j\256\253\262B\030\r\241bi\306\032\036UaM/\255\312\272P\250\030y[+\031\226P0KE!\22736\355\355\\N(\002\207\256&O1\254\013%[\265\330\323\004\375\370\223\260b\251\3024\215\237f\203\222k\252-qZrDc\336Te[\345\244[\266l\344\201\215\017\234/0\n8\254\240Y\202\242Y\362\252\256*\302\250\232\\K\216\013[\353\252!\030%`)\351\324/\333\266\234_W\2251\316\244\300\206\207X\005S\265+\246\201\321\004)L3\371\005$I\272\266\232\244\177\263\225b\261:\313\211\343d\004\3226\325\262\016\241\026U\303\026\n%\263ux0\200\323s0\311\034\331\373\350n\231r\313\224\313\2447\256h\232\224K}\3116\241\2349\250PW%\033:\230\021,\333\254\344m>\272]\334\2348&uS]\323,\230\207\252H\026Cr\244\002fJ\025\262%A\261\253eU(V,[X%q\332\343BA/\311\270\201\244\205Jq\261\312A&t\355\265\232\014G\255\231\245JY\220MS\256\n\005M\325\025\260i\252\340\332\026\254J\271\\2m\010ExD=B@bQ6_\013\243\272f\250\302\367\337N\215\005\255\345\212\265.\231\262\261\246\206}\267\357\204} \327\264[:'\277\375\366\366\030#?\265I\272z\255\301\322Iv\244\226-S\003i#\026\004\311\036%\225` \301[\255-\322\252l\347\327s9nFER\203B\222\"\034\026\024U\224't\000\352\030E\036&-\212/\237\246\305Yiif.\365L\224\036=\025\037K7\200\266\240\313k\201k\034\0019\223^YXNe\245\307\331\364\312\"F\301 HJ\352v\271da^\231\334:\300\301xz\002\226\000\266\252\346\345\nt\235\347\002\237\340\002\347\344qYs\305*\302j5\030\0363\205`\324\350X.\227\024\226\301\027\321JS[\252\315c\210!\353:4[!;\021\270\345\2302\205\000n\035\217dM\347R\001\\)O\026TT\213%\263\312\244]\226\2530\021\345h N\253\000\226LM\265\216\005k\341b\330Z\336\022\346""\311Vu]0*\3052\007Q\267m\325P\200\"\230<\264\263d+(& \261q\223]\225I\304,L\006\022\261\222A\354\020\204gd\217M\267\300\237Z\252X\002\263\220@\253\364[\224M\271\250\322\310f\323D\363\327l\202\213\233\332j\005\361Q\230\346\246)6[\340He\022\254\2547\301\351GZa\223\305\007\203z8!\002B)\2571\345ni\366:3\314\200\2568\006\004P\262\244 =(\343\010{\206\272\325\214]md\344\204\322\352\257j\336&+\010L\247\005[s\2465\r\003\205\327ju\253d\302>\315\265\n\205E\253u\3564 \315-\315R\3071\006\030!\242\365\222\2020b\313\214\017\340\211\261\025\316\\\020r\257\267\200\021\304\220\005\207\224G\250\213\310U2\274}\232b\336!\202\023\233P\307\227S\304W8vK\346~\030\005M\002\2025\330\300\005kZU\343]\025X\236\3119\345\301?\232!_\322)ap\202Y\370<\204j\016xL\232\227\270q#\226\025e\273\254\227l\244\257\034Ga\221\0345\246Gy\023^E\t(F\016P\257\221[\035\203\"\236%(\316\007\225\305\032\230\264\312j^+T\303\002\241\211\217\250\325Z\362\037\027\312\326\272\226\217\261\3212\005\005\234|\251L\251\000J\250\030\034\267F\266\332\034\020\006\304v\301\310\315\251#\346B\327\237n\311S:\264\004\204\225\262\016kd\331b\034A\221y\036\213\tICa\211\352P9l\312z\345\330\326\264\302j\247\200\022+V\234!z\225e\036Ca=\366\226\032T\013Mk5\024.=\331\226\205R!\302?\336\352\207L\236\020J\251\200\200G\301\254P\321Y\004\224Ay\325\322\254XtJm\313E\260\335\031\233\232\r\017\036<\020\264\"\331\222@\365BK{\240\307\237\204X%1:R\254\006\365\333\310\330\001\320,\215\217\006<\375D\300\324\200\264;\026Q%\352,\301\330p8\035R\207q\265\305\2420 \344e\203;\032\270T\246O\265L\027\013\037?\205sSj\343)<\352=\230\226\303\010\217\306\035\007f\\\350D.M\305\320\267&\023\322\350*\202\206a0\267*\220\r\227\024\225\333\344\207\234\362!\247|\310)\037r\312\3375\247t\204\264h\271\337\272\344\247\337\0132k,\234t-O\2462\016\222\325\002Y3\257\276\303p!\033\320\002\270\317\375\331\224ekE\365\377.\225Ek\317\226$B1\2337\267\r#j\222\226\256\252\345\321\311\203\021\226\312\001\276\277E6<\224\221N\376\337\223\027\017\204l\315\216\321d-\031\362\177-\267cv\2777\363""\302\211\362\264\363\365!\371~H\276\037\222\357\207\344\373\377cA\227e\273\330\035\241)J&$\001\325\324\362\001\217\354\200 \334\274-\203c\276\307\026\354\020\007;\202\252\241\360\370\371w\313\303<]hJ\224\323b[\276\177QNn2?\032\316\366\367\316\315G\313\344\260<\375\247\304\360g\246\375\353\222\376Q\024\361\363&\332\267e\233\325\250.m\356\207,\265\263\250\303\366\257i\2277\332\021f\007\021\201\211\213\354\320\205\3557\363v\036\003\342G',\226Hl\2478\307\234Y\243\360\234\243\r\353\252\304Zs\3744-\310\230\301N:CF\004i\006\314\303T4C\246]lm[U&\2664\005\223\360S\023\313.\231,1\255k\272\312\230a\330\350T\310\212\330\242\300\317\316\000\332k\007\212\342\210C\206M\361\325\344!kA\265(\232\345\343\2070\007\036\274\034p\270C\020T\212\004\224Z\332ojp~\303e\312\316\230*\224\033\331\240\344\261\347\342\207<\264Z\340\321h\246\212\312\201\237\216\251Hr*\"(\234\213\345\246\274\315\222.\230]L-N\334\276\177\233\211\335T\311\305\220Y*\253\254\322!\257+\010\253\025M\267)?\201\030\213\245\233j\251\202z\210\0078\nu\361\001,\325\323\341\006\005\341\221`\335\001\203!=\"\274\217\010\212fb\022\3709\215~$\353\226\232L\007\311\343\237 \355_\213\340\\-hFx^\240n\253\371\n39\226\360\202j)[\202*\231\274\270\031\200\276\026\321\205\226\310E\370\327X\342kv t\200%r\2031A\222%\214\346rYq\341qJz\230z<\277\220\313\215\013aCja\226\277>\023\263Or\271\261\340\320+(\232\311K\341\350\305rx\212G\314\205\266\310Oe~\300\330\360\020ky\376YjiY|\266\030\020Dxxq\250\026WU\205\216g\3320\206\333`1+\"\204@\263$>N\265\240\221\303\372\205\017\\/\351JX\264pz\202\310\303U\261\304\210\313\006'Wf\334\030\230\275k\224\335\330\201.rO\363lt\271I\035<We\263\005u\220,\260cA\242W\241\265,\251\223MKN\036\035\037\255\030M\323oj\233\3646-\374CV\024\tv\247\006\207\302\252A\327\265<jj\203\237k\207y\032\214\346e]\016\3343\031\371\347\272\274\251\036\344\213F\211h\222+:\230\001+t\256/I\202Ra\306l\224\214\t\310m\023\021\021\275y\315\320lI\262T\275\220\224\302\2436E\nH\035\347\355\301\233D\225\013o\312KHt\374\221+AZg'\320\224\370\202\310""\221/\031\233*\343\233\231\362\"w\363 ?\262SA-\377Z\307\320\010\341\237\032m\346o\221S\334\242S\373[tj_\256n\217\332\246\212\320\213xJ\032\035{H\232J\007\n\234\021\227S\217\323\331\2273\351\247\351\254$f\037?\354\260\326\231\225l6\265\260,\315\244\027\226\305\371\205T6l\230{!\315,\256HO\323\217\347g\304\247m\255\213s/\227\332\232\347\0270\370Yzi9\326\366L\234\231\003\316X\313Rz\346I*\002y\271\230\312>\237_J7g}\222\312.\244\232h\323K\341\323\322\001\023\240m1\233\236\201\273\304Z\226\347\262)q6lx\376,\010\367\263i\352N-\363\262\213_\371\207\004\361\347d\323\204\362L\t\222\324\326\2130j\321\252\357\340\376\300\230\203\322R\222\270\2755_\261`\215\336bG\321R\236}\003\021\357\341Q\217\345 \211\207\232\316^f\377\007\364\262\240\214\222\241\275\302\215\237\201\007\253\215\010*F\316\354\341e\332Q_T\304\372\232q.\366y\303l\333~E\3077\016\341\207\r\207O\033\253\272\330\327*\241j\243/Wb\2171\276c\215\355\314\267wu`<X\014m\375\235\262h\003\210\004\022\353`R\211\275G\242\2117F\362\211\265Z\357!*&\251\024%J\372p\"\005\253i\253\340\333^;\255\277\023\240\303\001:@\210\2660\210\262E\353\221\020(\366\216\356\207X\311\251\216\004b\376\320\374BDz\314>\334jK\202\007\267v\262|(\\\007\347T4\004\271z!\275\220Z@`~!\233\264\313\026VO\213bvy^|\372\364%\302^v95\273\310\351e*Y\206\363\006\357|\256'j5*\264\226\230\017K\213\325m\374\233Eq(-\250\333vV-\304*\231f\r\303\037\346\203\373bz1xXY\232\213\236\232\315\210\372\331\345\330#!\000\272%d\205\324,\232\262\363\013\217\263\351\364r\266i\324\374\263\253\366\367f|[\232I/\322\364\234G~\225\020\327yG\233 \337+\377c\010\276\003\204\\\214\225\356\022\225\356-a\363@\330\316\350z\004X<\314v\200\205\236/\005%\371{\341\242\372%\004\215}\323\026\177>@2\255\275\035bYN\247\237RB|>?\233\232m\026sdg+\013\201V\244\360\324A\342\334\263K\332\\1`\256\263\264l\323\271Pt\325X\303Z\004\231\004k\254\265J\251bqp\253j\344\265R2_B\255\nnU\213\302O\024\202\330F\024\317f\322k\265\312\323Z\020n5\325\n\003o,\000\243(\243\302\224\304\rO/\322+\320\302\260$\340\315\253\253r\376u^""\267\330V \273HvIZW\267\243\314\311F\323\207\231\\OQ\273\025>\262\272:|i~E%u\2660,EH\201J\277\316n\324cy\266\340\242n\372'\2615\210\002\276L\2159\274\262.I\n\371*~J)/\005\033;\374\032\024\215\212\315\355\211_\301M\264\244V(\200\207t\361\027>4(\227\261\340\245=\205f^\013\276\036\343\010\370\225>ekc$Z(E3\305&m\007\2465\024\355\242AEl\251\321)'\336l\310E5z\n\032\271E\223\352\371;M\300\327N\2140\\l\226\021&\357\262\333wS\354v\3676-\367$)\272\262}\000\230D[\266\216\227L\235\345\323A\211\372\250\310pH$8\304\363\017L\363-\345VK\236\227\250\213\273g\240x-\014\227t\305z\004\344 \367\341o\362..\337M\341r\3676.\367%\215\2143p0r\n\213_~S\371q\202nV\002\027\223\230]\261o/\212\3626A\004\253@\211)\224\373FS\267\341=0\022\211\177\277L\266Z4\271\031\302\230$\246\316\370\3252\230W\007\333\312\206\242\025\203B\026\3539\263\310\370h*\202\t\314(3\351E_\027\207\353\303\226\357\215\243\027\222\031\177\013)do|\373\037N\307\327A\245B\001\001\257,\333\353\364Ob4\005\351?\254\002\320O\033G-o\311\037i\225\247[\017\222\315\010\334\n`\241\352j\026^\222T6i_\233=\224\224\250\366\302;2q^.k$/z\016tK\217\2336y'ulTd\235K.\334\266\213\226\244\235\205F\330\200p\026\345\207C\022\312\021\t\304D\01007U\005w,\202\371\247\336\026_D\007\020\361+\225P\374)\3055\036\275T\203\251\230}D\256\334J\001\262\031\337\364\301B\266\031\022\232\017!\316\203\\\221Ul\\:\2355b{Mx`\r\330Y\363\241!X\374\307\362\241t@n\264\326e\320\014[ea\200\205\277X\215\014\266\002\217\266*\253\314\210\303{(\301J\031\242\227\300\006&\"\203\r\"3\005pf\212\004\313\267\323*\314\245+\314\247+\314\251+\314\2537\341*\n#)\236\227\270\313\260\363\nv\261\340\342\306\026\257\"\255\305j\232\357\001\334\030\215\036\331)\227\020\030\341k\r\03048\002\326\374c'\232\324\274\351jt\237s\206\234\311\235\256w\247N\234I\370}\327\274I\357\241W\250\211\265\314\233\256w='>\371\364\335\247'>\351\335\231\364\273/\372\027\177\332\375\367\333\311\306\371\253\336\020\215\274\350&\334o\274\236ZW\355R}\222\032N\357\334qN\271C\356w\256^\033\252\335\2533\250\323\376\351!/""\341}S\353\251w\325/\355~\277'\276\021\251\275\337\021\235\347\356}\314\367sM~#6n}\373\356\334\211\223}\376\331\257\335\r\257\333\233\257e\374\356k\336\203\372\225]\351-\2064\206\357\370w\346\374\271W\376\253\177\276\033?q\262w\347\276s\317\035j\364\236\335\371\rt\214y\223\215\276K\356u\377\253\333u\261\236i\364~\341\274\366\022\215\236\323;w\235\013n\027\275k\356\006\275\177\357<v'\303a7<\261\321w\331\275\355\177}\253\266Qg`\2727D`\001v\206\006\263\367\\\365n\324p;\263\363\312\001\232>\247\013r9y\336\231\360\277\236\331\003\272\357\352\270\334\253o4z\3174z\3169\243\030\213\333\030f\352\031\366^\325\330\314\200\270\342\212\301\323\027\216\201\2719Y\204\352\222\373\225g\325n\324\027\366\256\355e\002\024|\212\244\3671S\212\214V\377\334\210g\326\022\373='\033\275#\336f-S\313\327\023\365\253\365\271\335\207\2732a~\340\236\307\024\275\347\234\233n\306E\013\340\206\275\347\265\311\332\014\370\373\250>\272{u\227(\360O_qg \217^0\342$Z\341\271\344\320\363\261\363\300\273\010EN\3252\215\276+\356\274\227\365\266jr\355w\206\243\017jj\364\rB\264}\327\374k`\277.\022\220\270\337w\336\271\343v\273?\203\336\376\257\334u\357W\210\265\377\202\263\355nzK\265\256\306\340\260?\374\375nbwto\2501H\314\014\000\345FcP\360\272\366\007\207\275e\177L\334K\240o\340\213F\377\220?4\275\013\260\321Z\327>P\374\356]\3632\215\301\313\35648\372\256&7\256\214\371c?\355fv\177}\333\265\177e\254\326\357O\244 ;\201\3446p\325\353\367\207\251S&\212~p\363\340\251\377\2327]{U\267\200s@ \251\362\226L\243\237\021~\303\277\361\363\333\241F\377u\217\270\360\317\337$\023n\364\337\254M6\006a\374c\265D\355zM\336\357\007\251\376\320$\223;\000A\344\017\273\223\301(\270D\003\323=\360\372A)\220Oy\231\375\376/\335\314~\337\005\3477\357\002\370\357\033p^\270\242\013\211\242\331\377\212\r\275\004k\351\247\241\374\002\374\215\376\021o\243\321\177\031\366\003!B\222\204\243\367\274s\333\371oO\3642\236\274\037j\216\335\336\210\373=\275;\217`3d\320\354\361\226\027k\275\tj\242V\270\372\311S\244j\374\365\3037\033'O\275{\332u\342$""\334\037\222\256\301k\006\374\201a\014\351%N@t\355B\375\324\3567{\237\355m\354\367\234\335\261\375\363D\034\014\275\345\357\202\263A^\372M\243G\360NBTC4\371\002\371A\357\316\234\303\250J\223;\364\356<!\017:\353\237\025\274\217\274d\275\033\306;\267'\356e@X\317\211\223\227\335\253\356\002\344\274^GP\240\210\0205\254\357\312\261\220\320\350K\274\033\24580\355d\234_!2P`\373\227\241\255\232\330\342\300\212\177\351[\204\036\260uv\347?\256\014;|Y[\253\277\332%r{\006\034\311[\200\226{\022N\277\363/\357\272'{\314W\3579#P\005\004\000\212\037\355\311\344!@\226 \211\263\013\031~/\271\000\003\301,\000\031p2\304A<(L\371wr~Nn\211\n\344\303\033\364\032\270\3349xxWH\300\020\\\235\311\352\031\242\323\004\301\235\335!B\277\274Y\373\261\376\237=\n\224G\221\032\204\247\323;S;\031\022\210\351^$\224\215\336\257\\\305\277J\316\324\007\353\362\022\373 v\331\277x\267.7\372\206\274s\336\035(\371\\\355\373\372\303\272\274\177\346\354\316\246\263D\3741\230\251\372P\375v\375\267\275p\314\235z\006\243\002\n\223\260\350\200\302\243\310\202x\234*\034\274g\002\316\326\"\312\313\314\027\016\023\347\237\030@\221\350\351^Gt\236\252gbq\031\222?B\356\376%r\371\220-\314\344\016z\211\340\375\000\337!\203\357j\002\204p\375\360'J\206`\336\377\202'\027\364\374N&y\246\363\217#\346\311\217\3430\235\001g\rN\337C:\352\031\367\307\237\276\025\337fX\350\036b\201\244\227R\265\350\017\337\337\355b\342\360/\336 \237\273\020\244'\346\304I\026[\241\214\037\234<)\003Q\014\002\255\255\0221\315\306\313\356\224\373\n\271g\250qf\320\375\230r#\265@\306\375\344\265\t?\301\\j\277\007\004\3745n\300\262b\\\0271/\0103\034\211\300v~\"\301\242\330\020\033#\243$Q\301\265=fN \334\275\356\026Yj\342\021\305\355\241\341\214\224\317w\330LD\210#\357Gj\261\235I\237\342j\255\273&\006-\376\371a\226%\230\n\366\343-\024k.\371\227\236\274\235|\227\353\2420\023\212\320\355\367\277\234\252_\203=\235\371\034\341\327\357C\244\364\214\272\310B\3320r\344\037\365\345\335\301\275\301\267\211\2677\374\345\2257ig\362\335\317\307\3040\350~\004\227\352""\256\315!&\212\273\257\3666\336\244\375\363\367\352\177\354\375\227\237\311\002\232\352\246S;7\035\224F\003(\236&\311z\257p\371Q\2413\350\262T>\014\267\037$\271\005U\016S\016e\017V\244\221/n\272\231\350\361\027\017\350\220x\336\375\373\230T~\351f\335?j\313\365A\260\231\330\273\361Vd\222z\260k\300@\220\363\270\353\370\335\244\013\350\346gGn\271\241\276\362\356\327@\n,\376\223\2367\326\316\210\023\324i\347\035q\037F\367\320)\300\307\263\356\306>\224\330G\"\371\206\371\027\030\200=4\372X\311\344\013\222/\345\032\335C\214OT\236\214\267\317\200\356\332N&D7\013s\276\353\235\247:\013\206\367\371\216\216\332t\222\240*\240c\315]A\260R\021\364\304z\266\276\275\273\205(\017\030\315\241\324F\225o\017\300\346\234\031\362\247\223T\004\237\334\371\334QI\346\003\316\212;\202yo\302\225\230\344\221\201H\356d\327}\tV\024A\232\237\362\334\364\213\373\302\023;z\375\356\013\244\204Xn\344\231\322\357E9\2012r\267{W\334\351\332\207\376\266\234uV\273`(\305_\367\202/\220\307\2630\2160\260\337\313r1\301!\224Ci\356\013\377\372\2170\236\014\225\303W\375+<\004\240Z\333bE\003\253\360\231:\257S\351Ku\331\226\267Vc\320CD\334gT\326%B\355TQ\004r}\222\214\357a\2510MV\201\302\376]o{\177\202\nm\367\022\362s\202\200\247\235W\356\357\265\021\224\374P\315>\204~<\330\315\222\246\010\243\354?\361\206\213\305\340\370Fb\213\232h\201\203uLA\323\365\316\215'\332\032\240\345\333\377\000\206\333\3470";
    PyObject *data = __Pyx_DecompressString(cstring, 5402, 1);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #else /* compression: none (15879 bytes) */
const char* const bytes = ".\n    A class that provides an interface to NVTX API per domain,\n    and produces less overhead than using the global functions from ``nvtx`` module.\n\n    Notes\n    -----\n    - Use :func:`nvtx.get_domain` to create an instance.\n    - If the domain is disabled (e.g., when no tool is attached),\n      the instance returned is a :class:`nvtx._lib.lib.DummyDomain`.\n    \n    A replacement for :class:`nvtx.Domain` when the domain is disabled.\n    (e.g., when no tool is attached).\n    \n    A wrapper class for ``nvtxStringHandle_t`` C struct.\n    Use :func:`nvtx.Domain.get_registered_string` to create an instance.\n    Counter dtype must be int, float, or NumPy dtype-like.Counter group array fields are not supported. Field Domain.mark (line 902)Domain.push_range (line 945)Domain.start_range (line 1004)\n    Event kind for the writer's ``write_event`` / ``write_event_batch``,\n    mapped to the schema-level ``NVTX_PAYLOAD_SCHEMA_FLAG_*`` flag.\n\n    ``NVTX_PAYLOAD_SCHEMA_FLAG_COUNTER_GROUP`` is not exposed as an\n    ``EventKind`` because counter-group schemas are created by\n    ``Domain.get_counter()``. The flag is set internally during registration.\n    Failed to allocate memory for payloadFailed to allocate memory for schema entriesFailed to allocate memory for schema semantics Install numpy for extended payload support.Install numpy to use dtype-based NVTX counters.\n        Mark an instantaneous event.\n\n        Parameters\n        ----------\n        attributes : EventAttributes, optional\n            The event attributes to be associated with the event.\n            If not provided, a new :class:`EventAttributes` object is created\n            with the given keyword arguments.\n            Otherwise, this method mutates the attributes object if `kwargs` are provided.\n        message : str, optional\n            A message associated with the event.\n            If the given message was not registered then it will be registered under this domain.\n    ""    color : str, int, optional\n            A color associated with the event.\n            Supports `matplotlib` colors if it is available.\n        category : str, int, optional\n            A string or an integer specifying the category within the domain under which the event\n            is scoped. If unspecified, the event is not associated with a category.\n        payload : int, float, list, tuple, range, bytes, numpy.ndarray, optional\n            A value associated with the event.\n            Using payloads provides a separation between the message and the data of the event,\n            which is often useful for analysis.\n\n        Examples\n        --------\n        >>> import nvtx\n        >>> domain = nvtx.Domain('my_domain')\n        >>> domain.mark(message='my_marker')\n\n        Alternatively, an EventAttributes object can be reused:\n\n        >>> attributes = domain.get_event_attributes(message='my_marker')\n        >>> domain.mark(attributes)\n        >>> domain.mark(attributes, message='my_marker_2')\n        \n        Mark the beginning of a code range.\n\n        Parameters\n        ----------\n        attributes : EventAttributes, optional\n            The event attributes to be associated with the event.\n            If not provided, a new :class:`EventAttributes` object is created\n            with the given keyword arguments.\n            Otherwise, this method mutates the attributes object if `kwargs` are provided.\n        message : str, optional\n            A message associated with the event.\n            If the given message was not registered then it will be registered under this domain.\n        color : str, int, optional\n            A color associated with the event.\n            Supports `matplotlib` colors if it is available.\n        category : str, int, optional\n            A string or an integer specifying the category within the domain under which the event\n            is scoped. If unspecified, the event is not associat""ed with a category.\n        payload : int, float, list, tuple, range, bytes, numpy.ndarray, optional\n            A value associated with the event.\n            Using payloads provides a separation between the message and the data of the event,\n            which is often useful for analysis.\n        \n        Notes\n        -----\n        When applicable, prefer to use :class:`annotate`.\n\n        Examples\n        --------\n        >>> import time\n        >>> import nvtx\n        >>> domain = nvtx.Domain('my_domain')\n        >>> domain.push_range(message='my_code_range')\n        >>> time.sleep(1)\n        >>> domain.pop_range()\n\n        Alternatively, an EventAttributes object can be reused:\n\n        >>> attributes = domain.get_event_attributes(message='my_code_range')\n        >>> domain.push_range(attributes)\n        >>> domain.push_range(attributes, message='my_code_range_2')\n        >>> time.sleep(1)\n        >>> domain.pop_range()\n        >>> domain.pop_range()\n        \n        Mark the beginning of a process range.\n\n        Parameters\n        ----------\n        attributes : EventAttributes, optional\n            The event attributes to be associated with the event.\n            If not provided, a new :class:`EventAttributes` object is created\n            with the given keyword arguments.\n            Otherwise, this method mutates the attributes object if `kwargs` are provided.\n        message : str, optional\n            A message associated with the event.\n            If the given message was not registered then it will be registered under this domain.\n        color : str, int, optional\n            A color associated with the event.\n            Supports `matplotlib` colors if it is available.\n        category : str, int, optional\n            A string or an integer specifying the category within the domain under which the event\n            is scoped. If unspecified, the event is not associated with a category.\n        payload :"" int, float, list, tuple, range, bytes, numpy.ndarray, optional\n            A value associated with the event.\n            Using payloads provides a separation between the message and the data of the event,\n            which is often useful for analysis.\n\n        Returns\n        -------\n        A numeric value that must be passed to :func:`Domain.end_range`.\n\n        Examples\n        --------\n        >>> import time\n        >>> import nvtx\n        >>> domain = nvtx.Domain('my_domain')\n        >>> range_id = domain.start_range(message='my_code_range')\n        >>> time.sleep(1)\n        >>> domain.end_range(range_id)\n\n        Alternatively, an EventAttributes object can be reused:\n\n        >>> attributes = domain.get_event_attributes(message='my_code_range')\n        >>> range_id = domain.start_range(attributes)\n        >>> time.sleep(1)\n        >>> domain.end_range(range_id)\n        >>> range_id = domain.start_range(attributes, message='my_code_range_2')\n        >>> time.sleep(1)\n        >>> domain.end_range(range_id)\n        \n    NVTX interpretation of a specially typed payload field.\n\n    Attach to a field with :func:`nvtx.numpy_dtype` and its ``entry_type``\n    argument. The field retains ordinary fixed-width NumPy storage while NVTX\n    tools interpret it as an event attribute or identifier.\n    Nested counter groups are not supported. Counter dtype must not be a fixed-size array of a structured dtype.Nested counter groups are not supported. Field Note that Cython is deliberately stricter than PEP-484 and rejects subclasses of builtin types. If you need to pass subclasses then set the 'annotation_typing' directive to False.Optional[str]Predefined NVTX execution scopes.\n    Role of a field in a structured payload dtype.\n\n    Attach to a field with :func:`nvtx.numpy_dtype` and its ``entry_kind``\n    argument. The event roles (``RANGE_BEGIN``, ``RANGE_END``, ``MARK``)\n    mark the timestamp fields of an event schema; ``COUNTER_TIM""ESTAMP``\n    marks the embedded timestamp field of a counter group; ``MESSAGE``\n    marks a string field holding the event message.\n    SchemaRegistrar subclasses must implement _do_registerTimestamp ordering within a batch of deferred events or counters.Unsupported payload type: ?add_notedisableenablegcisenabled must be a scalar dtype. must not have a structured dtype.no default __reduce__ due to non-trivial __cinit__self._allocated_payload,self._payload_data,self.c_obj,self.string_handle cannot be converted to a Python object for picklingself.c_obj cannot be converted to a Python object for picklingsrc/nvtx/_lib/lib.pyx(tree fragment)BatchOrderingCATEGORYCOLOR_ARGBCOUNTER_TIMESTAMPCURRENT_CONTAINERCURRENT_HW_CPU_LOGICALCURRENT_HW_CPU_PHYSICALCURRENT_HW_INNERMOSTCURRENT_HW_MACHINECURRENT_HW_SOCKETCURRENT_HYPERVISORCURRENT_KERNELCURRENT_OSCURRENT_SW_INNERMOSTCURRENT_SW_PROCESSCURRENT_SW_THREADCURRENT_VMCounterDONT_SETDomainDomainHandleDomainHandle.__reduce_cython__DomainHandle.__setstate_cython__DomainHandle.enabledDomain.__init__Domain.__new__Domain._get_counter_cachedDomain._get_numpy_array_schemaDomain._get_numpy_dtype_schemaDomain._get_scope_idDomain.end_rangeDomain.get_category_idDomain.get_counterDomain.get_event_attributesDomain.get_registered_stringDomain.get_timestampDomain.markDomain.pop_rangeDomain.push_rangeDomain.set_event_attributesDomain.start_rangeDummyCounterDummyDomainDummyDomain.end_rangeDummyDomain.get_category_idDummyDomain.get_counterDummyDomain.get_event_attributesDummyDomain.get_registered_stringDummyDomain.get_timestampDummyDomain.markDummyDomain.pop_rangeDummyDomain.push_rangeDummyDomain.set_event_attributesDummyDomain.start_rangeEntryKindEnumEventAttributesEventAttributes.__reduce_cython__EventAttributes.__setstate_cython__EventAttributes._set_payload_floatEventAttributes._set_payload_intEventAttributes._set_payload_iterableEventAttributes._set_payload_numpyEventKind_GlobalSchemaRegistrar_GlobalSchemaRegistrar.__reduce_cython___GlobalSc""hemaRegistrar.__setstate_cython__MARKMESSAGENONENvtxWarningOptionalPARTIALLY_SORTEDPayloadEntryTypePayloadSchemaKeyPredefinedScope__Pyx_PyDict_NextRefRANGE_BEGINRANGE_ENDRANGE_IDRANGE_POPRANGE_PUSHRANGE_PUSHPOPRANGE_STARTRANGE_STARTENDREGISTERED_STRINGROOTRegisteredStringRegisteredString.__init__SCOPE_IDSORTEDSORTED_PER_SCOPESchemaRegistrarSchemaRegistrar.__reduce_cython__SchemaRegistrar.__setstate_cython__SchemaRegistrar._get_fixed_size_array_schemaSchemaRegistrar._get_numpy_array_schemaSchemaRegistrar._get_numpy_dtype_schemaSchemaRegistrar._register_builtin_schemaSchemaRegistrar._register_structured_schemaStringHandleStringHandle.__reduce_cython__StringHandle.__setstate_cython__TOOL_PROVIDEDTimestampTypeUNSORTED__annotate__arrayarrayOrUnionDetailarray_lengthascontiguousarrayasyncio.coroutinesattrattributesbytes_cache_keycachedcategoriescategorycategory_id__class_getitem__cline_in_tracebackclscolorcolor_to_hex_counter_class_from_dtypecounter_clscounter_groupcounter_semantics_counter_semantics_from_metadatacounter_semantics_objctypesdatadata_entrydescriptiondh__dict____doc__domaindomain_handledtdtype_dtype_to_entry_typedummy_counterdummy_domaineenabledencodeend_rangeentriesentry_entry_flags_from_metadataentry_kindentry_type_entry_type_from_metadataenumenumeratefield_counter_semanticsfield_namefield_namesfield_schema_keyfield_typefieldsflagsflatfloat16float32float64func__func__functoolsgetget_category_idget_counter_get_counter_cachedget_event_attributes_get_fixed_size_array_schema_get_numpy_array_schema_get_numpy_dtype_schemaget_registered_string_get_scope_idget_timestamp__getstate__handlei__init__initializeintint16int32int64int8_is_coroutineitemsitemsizekwargslru_cache__main__markmaxsizemessage__metaclass__metadatametadata_flags__module____mro_entries__name__name__namesnbytesndarrayndim__new___normalize_counter_dtypenpnumpynvtx._lib.countersnvtx._lib.libnvtx._lib.timenvtx._metadatanvtx.colorsobjobjectoffsetpathpath_bytespayloadpayload_setterpayload_setter.<locals>.r""egister_payload_setterspoppop_range__prepare__prodpush_range__pyx_capi____pyx_state__pyx_vtable____qualname__range_id__reduce____reduce_cython____reduce_ex__register_register_builtin_schema_register_structured_schemareservedresultreturnsscalar_schemaschemaschemaAttrschemaEntriesschemaEntryschema_flagsschema_key_schema_registrarscopeselfsemanticssemanticsEntriesset_event_attributes__set_name___set_payload_float_set_payload_int_set_payload_iterable_set_payload_numpysetdefault__setstate____setstate_cython__shapesizesize_entrystart_rangestr_stringsubdtypesubdtype_schemasuper__test__time_domain_to_bytestypetypinguint16uint32uint64uint8_validate_counter_group_dtypevaluevalueswarnwarningsPyObject *(PyObject *, int __pyx_skip_dispatch)\000_to_bytes\200\001\330\004\022\220!\2201\210\001\360\n\000\r\021\320\020#\2401\240B\240f\250A\250Q\200\001\360\010\000\005\006\360\006\000\005\014\2101\320\004\031\320\031=\270^\3101\330\023\"\240!\200\001\330\004\031\230\021\230&\240\010\250\001\250\032\2601\200\001\330\004\013\2105\220\n\230!\2303\230l\250!\2507\260!\200\001\330\004\013\320\013!\240\021\240&\250\010\260\001\260\032\2709\300A\200A\200\001\330\004\024\220A\220V\2308\2401\240J\250a\200A\330/0\360\022\000\t\020\320\017\037\230q\240\004\240I\250Q\320\004#\240>\260\034\270_\310A\200A\330%5\3205H\320HZ\320Z[\360,\000\t\014\2108\2207\230!\330\014\017\210z\230\021\230)\2401\330\020\032\230$\320\0364\260A\260Q\330\014\026\220k\240\021\330\010\013\2106\220\027\230\001\330\014\026\220i\230q\330\010\013\2109\220G\2301\330\014\017\210z\230\021\230*\240A\330\020\033\2304\320\037/\250q\260\001\330\014\026\220l\240!\330\010\013\2108\2207\230!\330\014\026\220k\240\021\200A\330\010\"\240*\250A\330\010\r\210Z\220q\330\010\020\220\001\360\006\000\t\023\220-\320\037C\3001\330\0143\2601\330\0147\260q\330\014\r\330\010\022\220(\230!\330\010\022\220)\2301\330\010%\240Z\250q\330\010\013\2101\330\014\034\230A\330\010\013\2101\330\014\026\220n\240A\330\014\026\220i\230q\360\006\000\t\032\230\036""\240s\250*\260N\300#\300Q\330\010\022\220(\230!\360\006\000\t\023\220.\240\003\2401\240B\240a\330\010\022\320\022'\240r\250\021\340\010\t\330\014'\240v\250Q\250c\260\021\260\"\260H\270B\270a\330\010\013\210>\230\023\230A\330\014\022\220+\230Q\230a\330\010\t\330\014%\240V\2501\250C\250q\260\002\260(\270\"\270A\330\010\013\320\013\034\230C\230q\330\014\020\220\001\220\021\330\014\022\220+\230Q\230a\330\010\t\330\014\026\220k\240\021\330\014\020\220\003\220>\240\031\250!\2502\250Q\330\020\034\230I\240R\240w\250a\250{\270\"\270A\330\020\037\230q\330\020\030\230\001\330\020#\320#3\2601\260A\330\020\034\230A\340\020\023\2205\230\004\230J\240a\330\024\036\230h\240j\260\001\330\024\027\220x\230v\240S\250\001\330\030%\320%9\270\021\270(\300!\330\030'\240r\250\025\250a\250q\330\030 \240\001\340\030%\240T\320)A\300\021\300!\330\025\026\330\024!\320!:\270!\330\030(\250\001\340\024\027\220{\240#\240Q\330\030\033\230:\240V\2503\250a\330\034)\320)=\270Q\270j\310\001\340\034)\250\024\320-E\300Q\330 !\360\006\000\025\"\240\024\320%=\270Q\270a\340\020\023\220;\230c\240\021\330\024#\240:\250Z\260s\270!\330\025 \240\003\2401\330\024#\240:\250Q\330\024\034\230A\340\020*\320*J\310!\330\024$\240A\340\020\023\320\023+\2507\260!\330\024+\2501\330\030\031\320\031)\250\021\250$\250a\340\024 \240\001\320!1\260\021\260\"\260A\340\020!\320!;\2701\330\024$\240A\340\020\031\230\021\330\020\023\220>\240\024\240Q\330\024#\2402\240Q\340\024\035\230Q\340\020\027\220z\240\027\250\001\330\020\025\220W\230A\230Q\330\020\035\230Q\320\036;\2701\330\024\032\230!\330\024\031\230\021\330\024\031\230\021\330\024 \240\001\330\024'\240q\330\024\033\2301\330\024\036\230a\330\024\035\230Q\340\014\023\2204\220}\240A\240Q\240a\340\014\020\220\001\220\021\330\014\020\220\001\220\021\200A\340\010\014\210F\220(\230*\240A\330\010\014\210F\220/\240\021\200A\340\010\014\210F\220(\230+\240Q\330\010\014\210F\220/\240\021\200A\360\n\000\t\n\330\010\t\330\010\t\330\024!\240\021\330\t\n\360L\001\000\t\021\320\020(\250\001\250""\021\330\010\025\320\025%\240Q\330\014\023\220>\240\027\250\001\250\027\260\n\270&\300\007\300q\340\010\017\210t\320\023'\240q\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\027\220q\200A\330%&\330\010 \240\t\250\021\250!\340\010\014\210N\230!\330\010\014\210H\220A\330\010\014\210O\2301\330\010\014\210K\220q\330\010\017\320\017 \240\002\240.\260\004\260H\270H\300A\300Q\200A\360\010\000\t\033\230\"\230N\250$\250h\260a\200A\360\022\000\t\033\230\"\230N\250$\250h\260h\270a\320\004#\240>\260\034\270_\310A\330\020\021\360(\000\t\014\210:\220Q\220j\240\001\330\014\027\220t\320\033+\2501\250A\330\010\013\2108\2207\230!\330\014\026\220d\320\0320\260\001\260\021\330\010\017\210\177\230a\230v\240Y\250g\260Z\270q\200A\330\010\025\220_\240N\260!\330\010\021\220\024\220]\240$\240a\240q\330\010\013\2107\220'\230\021\330\014\023\2201\330\010F\300a\330\014\022\220!\330\014\021\220\021\330\014\021\220\021\330\014\030\230\001\330\014\037\230q\330\014\023\2201\330\014\026\220a\330\014\025\220Q\360\010\000\t\023\220-\320\037C\3001\330\0143\2602\3205`\320`a\330\014\r\330\010\022\220(\230!\330\010\022\320\022'\240q\330\010\022\220+\230Q\230a\330\010\022\220.\240\001\330\010\021\220\024\220]\240!\2401\240A\330\010\014\210M\230\021\230-\240q\330\010\017\210q\200A\330\035+\250<\260\177\300a\200A\330\010\021\220\024\220]\240$\240a\240q\330\010\013\2107\220'\230\021\330\014\023\2201\330\010\r\210Z\220q\330\010\013\2102\210Q\330\014\027\220r\230\031\240!\2401\330\014\036\230d\320\":\270!\330\020 \240\001\240\021\340\014\025\220T\320\0316\260a\330\020!\240\022\2405\250\001\250\022\2509\260B\260a\340\r\017\210v\220S\230\001\330\014\025\220T\320\0312\260!\2604\260z\300\021\340\014\025\220T\320\0315\260Q\260a\330\010\014\210M\230\021\230.\250\001\330\010\017\210q\200A\330\010\021\220\024\220]\240$\240a\240q\330\010\013\2107\220'\230\021\330\014\023\2201\330\010\026\220a\220y\240\001\340\010-\250Q\330\014\022\220!\330\014\021\220\021\330\014\021\220\033\230A\230Q\330\014\030\230\001\330""\014\037\230q\330\014\023\2201\330\014\026\220a\330\014\025\220Q\340\010-\250Q\330\014\022\220!\330\014\021\220\021\330\014\021\220\033\230A\230Q\330\014\030\230\001\330\014\037\230q\330\014\023\2201\330\014\026\220a\330\014\025\220Q\340\0103\2601\260L\300\001\360\006\000\t\023\220-\320\037C\3001\330\0143\2602\260Q\330\010\022\220(\230!\330\010\022\220+\230Q\330\010\022\220.\240\001\330\010\021\220\024\220]\240!\2401\240A\330\010\014\210M\230\021\320\032+\2501\330\010\017\210q\200A\330\010\030\230\001\230\030\240\021\330\010\017\210q\200A\360\n\000\t\n\330\010\t\330\010\t\330\024!\240\021\330\t\n\330\010 \240\001\240\021\330\010\017\210q\200A\330\010\017\210q\200A\360\024\000\t\021\220\n\230!\330\010\026\320\026/\250q\260\001\330\010\017\210{\230!\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\200A\360\n\000\t\020\320\017\037\230q\200A\330\010\017\210r\220\025\220g\230Q\340\0106\260a\330\010,\320,L\310A\310Q\330\010\013\320\013!\240\027\250\001\330\014#\2401\240A\320%8\270\001\330\014\030\230\001\320\031*\250!\340\010\027\220q\330\010\020\220\001\330\010\025\320\025.\250a\250q\330\010\013\210;\220c\230\021\330\014\031\320\031-\250Q\250b\260\001\330\010\013\210;\220c\230\021\330\014\033\2302\230Z\240s\250!\330\r\030\230\003\2301\330\014\033\2302\230Q\330\014\024\220A\330\010\021\320\021+\2501\250A\340\010L\310A\330\014\022\220!\330\014\021\220\021\330\014\021\220\021\330\014\030\230\001\330\014\037\230q\330\014\023\2201\330\014\026\220a\330\014\025\220Q\360\010\000\t\023\220-\320\037C\3001\330\0143\2602\3205`\320`a\330\014\r\330\010\022\220)\2301\330\010\022\220(\230!\330\010\022\220+\230Q\230a\330\010\022\220.\240\001\330\010\022\320\022'\240r\250\021\330\010\017\210t\220=\240\001\240\021\240!\200A\330'(\360\024\000\t \230t\2401\330\010\026\220c\230\021\230$\230m\2502\250Q\330\010\014\210K\220q\230\010\240\001\330\010\037\230q\330\014\016\210a\330\014\r\330\014\025\220Q\220a\340\010\017\210q\200A\330\010\017\210t\2201\320\024'\240q\250\004""\250A\200A\330\010\017\210t\320\023%\320%=\270Q\330\014\r\200A\340\010\017\210t\320\023%\320%=\270Q\270a\320\004\032\320\032K\3101\360`\001\000\t\014\210;\220c\230\021\330\014\031\230\024\320\0352\260#\260Q\330\r\016\330\014\020\320\020%\240Q\240n\260A\330\010\017\320\017%\240R\240~\260T\270\030\300\030\310\021\310*\320TU\200O\2201\360J\001\000\t\014\210;\220c\230\021\330\014\031\230\024\320\0352\260#\260Q\330\r\016\330\014\020\320\020%\240Q\240n\260A\330\010\030\230\002\230.\250\004\250H\260H\270A\270Z\300q\200O\320\0237\260~\300\\\320QR\330\r\016\200\001\330\004\n\210+\220Q\320\004\025\220V\2301\330\010\021\220\034\230Q\230a\330\010\013\2106\220\030\230\021\330\014\022\220%\220r\230\030\240\021\240!\330\014\017\210z\230\021\330\014\023\2201\340\014\023\2201\200\001\330\004\026\220a\220v\230Q\200\001\330\004\026\220a\220v\230X\240Q\320\004\031\230\021\360^\001\000\t\014\210;\220c\230\021\330\014\031\230\024\320\0352\260#\260Q\330\r\016\330\014\020\320\020%\240Q\240n\260A\330\010\035\230R\230~\250T\260\030\270\030\300\021\300*\310A\320\004\032\320\032>\270n\310A\330\024#\2401\330\010\017\210q\320\004\037\230q\330\010\014\210J\220a\330\010\014\210J\220a\330\010\014\210J\220l\240!\2408\2501\200\001\360\n\000\005\010\200s\210'\220\021\330\010\013\2106\220\023\220A\340\014\024\220B\220f\230A\230R\230q\340\014\r\330\020\030\230\002\230&\240\001\240\021\330\014\023\220=\240\001\330\020\026\220i\230q\320 _\320_`\330\004!\240\021\240!\330\004\013\2101\200\001\330\004\007\200s\210#\210Q\330\010\013\2106\220\023\220D\230\003\2306\240\023\240A\330\014\r\330\010\016\210l\230!\2301\330\004\007\200u\210J\220g\230U\240$\240e\2509\260A\260R\260x\270w\300a\330\010\016\210i\220q\330\014\r\360\010\000\005\010\200u\210H\220C\220q\330\010\t\360\006\000\005\t\210\016\220e\2301\330\010\025\220U\230'\240\021\240+\250Q\250a\330\010\013\210:\220Q\330\014\022\220)\2301\330\020\021\330\020\030\230\001\230\021\360\006\000\t\014\210:\220X\230W\240A\330\014\022\220)\2301\330\020\021\330\020\030""\230\001\230\021\320\004\027\220v\230Q\330\010\014\210H\220A\330\010\014\210N\230!\330\010\014\320\014!\320!7\260q\270\004\270A\210\001\340\014\017\210w\220h\230c\240\021\330\020\021\330\014\025\220T\230\027\320 8\270\001\330\020 \240\001\240\027\250\001\340\014'\240q\330\014\017\210w\220a\330\020\031\230\024\230W\320$<\270A\270Q\330\020\032\230\"\320\034.\250a\250q\330\020\037\230w\240a\340\014\020\320\020#\2401\330\014\020\320\020$\240A\330\020\027\220x\230w\240g\250Q\330\020\032\230!\330\020\030\230\007\230q\330\020\021\200\001\360\n\000\005\010\200y\220\003\2201\330\010\017\210q\330\004\013\2107\220!\220:\320\0352\260!\200\001\360\014\000\005\010\200y\220\003\2201\330\010\017\210q\330\004\021\220\027\230\001\230\032\240>\260\021\330\004\013\210:\220Z\230{\250'\260\032\2701\200\001\340\004\007\200y\220\003\2201\330\010\017\210q\330\004\021\220\027\230\001\230\032\240>\260\021\330\004\013\210:\220Z\230{\250'\260\032\2701void (nvtxSemanticsCounter_t *, PyObject *)\000_fill_counter_semanticsdatasize";
    PyObject *data = NULL;
    CYTHON_UNUSED_VAR(__Pyx_DecompressString);
    #endif
    PyObject **stringtab = __pyx_mstate->__pyx_string_tab;
    Py_ssize_t pos = 0;
    for (int i = 0; i < 335; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyUnicode_DecodeUTF8(bytes + pos, bytes_length, NULL);
      if (likely(string) && i >= 41) PyUnicode_InternInPlace(&string);
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
      stringtab[i] = string;
      pos += bytes_length;
    }
    for (int i = 335; i < 391; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyBytes_FromStringAndSize(bytes + pos, bytes_length);
      stringtab[i] = string;
      pos += bytes_length;
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    Py_XDECREF(data);
    for (Py_ssize_t i = 0; i < 391; i++) {
      if (unlikely(PyObject_Hash(stringtab[i]) == -1)) {
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    #if CYTHON_IMMORTAL_CONSTANTS
    {
      PyObject **table = stringtab + 335;
      for (Py_ssize_t i=0; i<56; ++i) {
        #if PY_VERSION_HEX >= 0x030F0000
        PyUnstable_SetImmortal(table[i]);
        #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
        if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
        #if PY_VERSION_HEX < 0x030E0000
        if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
        #else
        if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
        #endif
        {
          Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
        }
        #else
        if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
        #endif
      }
    }
    #endif
  }
  {
    PyObject **numbertab = __pyx_mstate->__pyx_number_tab + 0;
    int8_t const cint_constants_1[] = {0,1,2,4};
    for (int i = 0; i < 4; i++) {
      numbertab[i] = PyLong_FromLong(cint_constants_1[i - 0]);
      if (unlikely(!numbertab[i])) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_number_tab;
    for (Py_ssize_t i=0; i<4; ++i) {
      #if PY_VERSION_HEX >= 0x030F0000
      PyUnstable_SetImmortal(table[i]);
      #elif CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      if ((PY_SSIZE_T_MAX <= _Py_IMMORTAL_REFCNT_LOCAL)) break;
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], ((Py_ssize_t)_Py_IMMORTAL_REFCNT_LOCAL + 1));
      }
      #else
      if ((PY_SSIZE_T_MAX < _Py_IMMORTAL_INITIAL_REFCNT)) break;
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */
typedef struct {
    unsigned int argcount : 3;
    unsigned int num_posonly_args : 1;
    unsigned int num_kwonly_args : 3;
    unsigned int nlocals : 5;
    unsigned int flags : 10;
    unsigned int first_line : 11;
} __Pyx_PyCode_New_function_description;
/* NewCodeObj.proto */
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
);


static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  PyObject* tuple_dedup_map = PyDict_New();
  if (unlikely(!tuple_dedup_map)) return -1;
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 246};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_func};
    __pyx_mstate_global->__pyx_codeobj_tab[0] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_register, __pyx_mstate->__pyx_kp_b_iso88591_A_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[0])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 63};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_s};
    __pyx_mstate_global->__pyx_codeobj_tab[1] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_to_bytes, __pyx_mstate->__pyx_kp_b_iso88591_5_3l_7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[1])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {0, 0, 0, 0, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 66};
    PyObject* const varnames[] = {0};
    __pyx_mstate_global->__pyx_codeobj_tab[2] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_initialize, __pyx_mstate->__pyx_kp_b_iso88591_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[2])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 74};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_dtype, __pyx_mstate->__pyx_n_u_field_name, __pyx_mstate->__pyx_n_u_field_type};
    __pyx_mstate_global->__pyx_codeobj_tab[3] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_validate_counter_group_dtype, __pyx_mstate->__pyx_kp_b_iso88591_s_Q_6_D_6_A_l_1_uJgU_e9ARxwa_iq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[3])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 104};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_dtype, __pyx_mstate->__pyx_n_u_e};
    __pyx_mstate_global->__pyx_codeobj_tab[4] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_normalize_counter_dtype, __pyx_mstate->__pyx_kp_b_iso88591_s_6_A_BfARq_iq____1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[4])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 122};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_metadata};
    __pyx_mstate_global->__pyx_codeobj_tab[5] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_counter_semantics_from_metadata, __pyx_mstate->__pyx_kp_b_iso88591_y_1_q_7_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[5])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 152};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_metadata, __pyx_mstate->__pyx_n_u_entry_kind};
    __pyx_mstate_global->__pyx_codeobj_tab[6] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_entry_flags_from_metadata, __pyx_mstate->__pyx_kp_b_iso88591_y_1_q_Z_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[6])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 164};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_metadata, __pyx_mstate->__pyx_n_u_entry_type};
    __pyx_mstate_global->__pyx_codeobj_tab[7] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_entry_type_from_metadata, __pyx_mstate->__pyx_kp_b_iso88591_y_1_q_Z_1_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[7])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 242};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_type, __pyx_mstate->__pyx_n_u_register, __pyx_mstate->__pyx_n_u_register};
    __pyx_mstate_global->__pyx_codeobj_tab[8] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_payload_setter, __pyx_mstate->__pyx_kp_b_iso88591_1_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[8])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 349};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[9] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_set_payload_int, __pyx_mstate->__pyx_kp_b_iso88591_A_F_Q_F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[9])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 354};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[10] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_set_payload_float, __pyx_mstate->__pyx_kp_b_iso88591_A_F_A_F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[10])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 360};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_payload, __pyx_mstate->__pyx_n_u_schema, __pyx_mstate->__pyx_n_u_array_length};
    __pyx_mstate_global->__pyx_codeobj_tab[11] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_set_payload_numpy, __pyx_mstate->__pyx_kp_b_iso88591_whc_T_8_q_wa_W_AQ_aq_wa_1_A_xwg, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[11])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 381};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[12] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_set_payload_iterable, __pyx_mstate->__pyx_kp_b_iso88591_1BfAQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[12])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[13] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[13])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[14] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[14])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 430};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[15] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_enabled, __pyx_mstate->__pyx_kp_b_iso88591_A_t1_q_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[15])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[16] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[16])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[17] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[17])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 442};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_domain, __pyx_mstate->__pyx_n_u_string};
    __pyx_mstate_global->__pyx_codeobj_tab[18] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_init, __pyx_mstate->__pyx_kp_b_iso88591_q_Ja_Ja_Jl_81, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[18])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 453};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_string};
    __pyx_mstate_global->__pyx_codeobj_tab[19] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_registered_string, __pyx_mstate->__pyx_kp_b_iso88591_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[19])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 456};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name};
    __pyx_mstate_global->__pyx_codeobj_tab[20] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_category_id, __pyx_mstate->__pyx_kp_b_iso88591_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[20])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 459};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[21] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_event_attributes, __pyx_mstate->__pyx_kp_b_iso88591_A_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[21])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 4, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 462};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[22] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_set_event_attributes, __pyx_mstate->__pyx_kp_b_iso88591_A_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[22])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 4, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 466};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[23] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_mark, __pyx_mstate->__pyx_kp_b_iso88591_O_7_QR, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[23])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 4, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 470};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[24] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_push_range, __pyx_mstate->__pyx_kp_b_iso88591_1_3, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[24])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 474};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[25] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_pop_range, __pyx_mstate->__pyx_kp_b_iso88591_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[25])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 4, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 477};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[26] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_start_range, __pyx_mstate->__pyx_kp_b_iso88591_nA_1_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[26])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 481};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_range_id};
    __pyx_mstate_global->__pyx_codeobj_tab[27] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_end_range, __pyx_mstate->__pyx_kp_b_iso88591_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[27])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 4, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 484};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_dtype, __pyx_mstate->__pyx_n_u_description, __pyx_mstate->__pyx_n_u_scope, __pyx_mstate->__pyx_n_u_semantics, __pyx_mstate->__pyx_n_u_time_domain};
    __pyx_mstate_global->__pyx_codeobj_tab[28] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_counter, __pyx_mstate->__pyx_kp_b_iso88591_A_q_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[28])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 497};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[29] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_timestamp, __pyx_mstate->__pyx_kp_b_iso88591_A_q_3, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[29])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 12, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 528};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_dt, __pyx_mstate->__pyx_n_u_metadata, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_counter_semantics, __pyx_mstate->__pyx_n_u_semantics, __pyx_mstate->__pyx_n_u_counter_semantics_obj, __pyx_mstate->__pyx_n_u_array_length, __pyx_mstate->__pyx_n_u_flags, __pyx_mstate->__pyx_n_u_entry_type, __pyx_mstate->__pyx_n_u_schemaEntry, __pyx_mstate->__pyx_n_u_schemaAttr};
    __pyx_mstate_global->__pyx_codeobj_tab[30] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_register_builtin_schema, __pyx_mstate->__pyx_kp_b_iso88591_A_r_gQ_6a_LAQ_1A_8_q_aq_c_Qb_c_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[30])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 24, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 571};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_schema_key, __pyx_mstate->__pyx_n_u_counter_group, __pyx_mstate->__pyx_n_u_dt, __pyx_mstate->__pyx_n_u_names, __pyx_mstate->__pyx_n_u_semantics, __pyx_mstate->__pyx_n_u_schemaAttr, __pyx_mstate->__pyx_n_u_schema_flags, __pyx_mstate->__pyx_n_u_flat, __pyx_mstate->__pyx_n_u_schemaEntries, __pyx_mstate->__pyx_n_u_semanticsEntries, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_field_name, __pyx_mstate->__pyx_n_u_field_type, __pyx_mstate->__pyx_n_u_offset, __pyx_mstate->__pyx_n_u_array_length, __pyx_mstate->__pyx_n_u_flags, __pyx_mstate->__pyx_n_u_field_schema_key, __pyx_mstate->__pyx_n_u_subdtype, __pyx_mstate->__pyx_n_u_shape, __pyx_mstate->__pyx_n_u_entry_type, __pyx_mstate->__pyx_n_u_field_counter_semantics, __pyx_mstate->__pyx_n_u_metadata_flags, __pyx_mstate->__pyx_n_u_name};
    __pyx_mstate_global->__pyx_codeobj_tab[31] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_register_structured_schema, __pyx_mstate->__pyx_kp_b_iso88591_A_A_Zq_C1_31_7q_1_Zq_1_A_1_nA_iq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[31])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 679};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_scalar_schema, __pyx_mstate->__pyx_n_u_cached, __pyx_mstate->__pyx_n_u_field_names, __pyx_mstate->__pyx_n_u_size_entry, __pyx_mstate->__pyx_n_u_data_entry, __pyx_mstate->__pyx_n_u_entries, __pyx_mstate->__pyx_n_u_schemaAttr, __pyx_mstate->__pyx_n_u_result};
    __pyx_mstate_global->__pyx_codeobj_tab[32] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_numpy_array_schema, __pyx_mstate->__pyx_kp_b_iso88591_A_aq_7_1_ay_Q_AQ_q_1_a_Q_Q_AQ_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[32])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 717};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_scalar_schema, __pyx_mstate->__pyx_n_u_array_length, __pyx_mstate->__pyx_n_u_size, __pyx_mstate->__pyx_n_u_cache_key, __pyx_mstate->__pyx_n_u_cached, __pyx_mstate->__pyx_n_u_entry, __pyx_mstate->__pyx_n_u_schemaAttr, __pyx_mstate->__pyx_n_u_result};
    __pyx_mstate_global->__pyx_codeobj_tab[33] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_fixed_size_array_schema, __pyx_mstate->__pyx_kp_b_iso88591_A__N_aq_7_1_Fa_q_1_a_Q_C1_325_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[33])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 745};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_schema_key, __pyx_mstate->__pyx_n_u_cached, __pyx_mstate->__pyx_n_u_dt, __pyx_mstate->__pyx_n_u_subdtype, __pyx_mstate->__pyx_n_u_subdtype_schema, __pyx_mstate->__pyx_n_u_result};
    __pyx_mstate_global->__pyx_codeobj_tab[34] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_numpy_dtype_schema, __pyx_mstate->__pyx_kp_b_iso88591_A_aq_7_1_Zq_2Q_r_1_d_T_6a_5_9Ba, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[34])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[35] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[35])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[36] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[36])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[37] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[37])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[38] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[38])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 792};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_cls, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_handle, __pyx_mstate->__pyx_n_u_obj};
    __pyx_mstate_global->__pyx_codeobj_tab[39] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_new, __pyx_mstate->__pyx_kp_b_iso88591_V1_Qa_6_r_z_1_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[39])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 801};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name};
    __pyx_mstate_global->__pyx_codeobj_tab[40] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_init, __pyx_mstate->__pyx_kp_b_iso88591_vQ_HA_N_7q_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[40])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 806};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_string};
    __pyx_mstate_global->__pyx_codeobj_tab[41] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_registered_string, __pyx_mstate->__pyx_kp_b_iso88591_A_0_q_IQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[41])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 818};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_dh, __pyx_mstate->__pyx_n_u_category_id};
    __pyx_mstate_global->__pyx_codeobj_tab[42] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_category_id, __pyx_mstate->__pyx_kp_b_iso88591_A_t1_c_m2Q_Kq_q_a_Qa_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[42])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 839};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[43] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_event_attributes, __pyx_mstate->__pyx_kp_b_iso88591_A_Qj_t_1A_87_d_0_avYgZq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[43])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 4, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 866};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_message, __pyx_mstate->__pyx_n_u_color, __pyx_mstate->__pyx_n_u_category, __pyx_mstate->__pyx_n_u_payload};
    __pyx_mstate_global->__pyx_codeobj_tab[44] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_set_event_attributes, __pyx_mstate->__pyx_kp_b_iso88591_A_55HHZZ_87_z_1_4AQ_k_6_iq_9G1_z, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[44])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS), 902};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_kwargs};
    __pyx_mstate_global->__pyx_codeobj_tab[45] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_mark, __pyx_mstate->__pyx_kp_b_iso88591_O1J_c_2_Q_QnA_HHAZq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[45])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS), 945};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_kwargs};
    __pyx_mstate_global->__pyx_codeobj_tab[46] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_push_range, __pyx_mstate->__pyx_kp_b_iso88591_c_2_Q_QnA_R_T_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[46])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 998};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[47] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_pop_range, __pyx_mstate->__pyx_kp_b_iso88591_A_N_ha, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[47])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS), 1004};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_kwargs};
    __pyx_mstate_global->__pyx_codeobj_tab[48] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_start_range, __pyx_mstate->__pyx_kp_b_iso88591_K1_c_2_Q_QnA_R_T_TU, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[48])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1058};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_range_id};
    __pyx_mstate_global->__pyx_codeobj_tab[49] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_end_range, __pyx_mstate->__pyx_kp_b_iso88591_A_N_hha, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[49])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 4, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1069};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_dtype, __pyx_mstate->__pyx_n_u_description, __pyx_mstate->__pyx_n_u_scope, __pyx_mstate->__pyx_n_u_semantics, __pyx_mstate->__pyx_n_u_time_domain, __pyx_mstate->__pyx_n_u_schema_key};
    __pyx_mstate_global->__pyx_codeobj_tab[50] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_counter, __pyx_mstate->__pyx_kp_b_iso88591_A_L_Q_q_t_q_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[50])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1129};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_path, __pyx_mstate->__pyx_n_u_path_bytes, __pyx_mstate->__pyx_n_u_attr};
    __pyx_mstate_global->__pyx_codeobj_tab[51] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_scope_id, __pyx_mstate->__pyx_kp_b_iso88591_A_N_HA_O1_Kq_HHAQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[51])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {7, 0, 0, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1139};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_schema_key, __pyx_mstate->__pyx_n_u_description, __pyx_mstate->__pyx_n_u_scope, __pyx_mstate->__pyx_n_u_semantics, __pyx_mstate->__pyx_n_u_time_domain, __pyx_mstate->__pyx_n_u_dtype, __pyx_mstate->__pyx_n_u_counter_cls};
    __pyx_mstate_global->__pyx_codeobj_tab[52] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_counter_cached, __pyx_mstate->__pyx_kp_b_iso88591_A_q_4, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[52])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1161};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[53] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_timestamp, __pyx_mstate->__pyx_kp_b_iso88591_A_q_5, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[53])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1168};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_schema_key};
    __pyx_mstate_global->__pyx_codeobj_tab[54] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_numpy_dtype_schema, __pyx_mstate->__pyx_kp_b_iso88591_A_t_Qa, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[54])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1172};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_scalar_schema};
    __pyx_mstate_global->__pyx_codeobj_tab[55] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_get_numpy_array_schema, __pyx_mstate->__pyx_kp_b_iso88591_A_t_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[55])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[56] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[56])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[57] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[57])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1195};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_domain};
    __pyx_mstate_global->__pyx_codeobj_tab[58] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_push_range, __pyx_mstate->__pyx_kp_b_iso88591_1_4, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[58])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1199};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_domain};
    __pyx_mstate_global->__pyx_codeobj_tab[59] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_pop_range, __pyx_mstate->__pyx_kp_b_iso88591_avQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[59])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1203};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_domain};
    __pyx_mstate_global->__pyx_codeobj_tab[60] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_start_range, __pyx_mstate->__pyx_kp_b_iso88591_9A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[60])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1207};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_range_id, __pyx_mstate->__pyx_n_u_domain};
    __pyx_mstate_global->__pyx_codeobj_tab[61] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_end_range, __pyx_mstate->__pyx_kp_b_iso88591_avXQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[61])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1211};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_attributes, __pyx_mstate->__pyx_n_u_domain};
    __pyx_mstate_global->__pyx_codeobj_tab[62] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_src_nvtx__lib_lib_pyx, __pyx_mstate->__pyx_n_u_mark, __pyx_mstate->__pyx_kp_b_iso88591_AV81Ja, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[62])) goto bad;
  }
  Py_DECREF(tuple_dedup_map);
  return 0;
  bad:
  Py_DECREF(tuple_dedup_map);
  return -1;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CommonTypesMetaclass.init */
  if (likely(__pyx_CommonTypesMetaclass_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CachedMethodType.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
  {
      PyObject *typesModule=NULL;
      typesModule = PyImport_ImportModule("types");
      if (typesModule) {
          __pyx_mstate_global->__Pyx_CachedMethodType = PyObject_GetAttrString(typesModule, "MethodType");
          Py_DECREF(typesModule);
      }
  } // error handling follows
  #endif
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CythonFunctionShared.init */
  if (likely(__pyx_CyFunction_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError (used by GetBuiltinName) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyObjectCall (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall (used by PyObjectFastCallMethod) */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED) && CYTHON_METH_FASTCALL
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
          #if CYTHON_COMPILING_IN_LIMITED_API
            return PyObject_Vectorcall(func, args, _nargs, NULL);
          #else
            vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
            if (f) {
                return f(func, args, _nargs, NULL);
            }
          #endif
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* TupleAndListFromArray (used by fastcall) */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        Py_INCREF(src[i]);
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < (0))) {
            Py_DECREF(res);
            return NULL;
        }
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* PyObjectCallOneArg (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* UnpackUnboundCMethod (used by CallUnboundCMethod0) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod0 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        if (likely(cfunc->flag == METH_NOARGS))
            return __Pyx_CallCFunction(cfunc, self, NULL);
        if (likely(cfunc->flag == METH_FASTCALL))
            return __Pyx_CallCFunctionFast(cfunc, self, NULL, 0);
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, NULL, 0, NULL);
        if (likely(cfunc->flag == (METH_VARARGS | METH_KEYWORDS)))
            return __Pyx_CallCFunctionWithKeywords(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple, NULL);
        if (cfunc->flag == METH_VARARGS)
            return __Pyx_CallCFunction(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple);
        return __Pyx__CallUnboundCMethod0(cfunc, self);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod0(&tmp_cfunc, self);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod0(cfunc, self);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *result;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
    result = __Pyx_PyObject_CallOneArg(cfunc->method, self);
    return result;
}

/* py_dict_items (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_items, d);
}

/* py_dict_values (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_values, d);
}

/* OwnedDictNext (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue) {
    PyObject *next = NULL;
    if (!*ppos) {
        if (pvalue) {
            PyObject *dictview = pkey ? __Pyx_PyDict_Items(p) : __Pyx_PyDict_Values(p);
            if (unlikely(!dictview)) goto bad;
            *ppos = PyObject_GetIter(dictview);
            Py_DECREF(dictview);
        } else {
            *ppos = PyObject_GetIter(p);
        }
        if (unlikely(!*ppos)) goto bad;
    }
    next = PyIter_Next(*ppos);
    if (!next) {
        if (PyErr_Occurred()) goto bad;
        return 0;
    }
    if (pkey && pvalue) {
        *pkey = __Pyx_PySequence_ITEM(next, 0);
        if (unlikely(*pkey)) goto bad;
        *pvalue = __Pyx_PySequence_ITEM(next, 1);
        if (unlikely(*pvalue)) goto bad;
        Py_DECREF(next);
    } else if (pkey) {
        *pkey = next;
    } else {
        assert(pvalue);
        *pvalue = next;
    }
    return 1;
  bad:
    Py_XDECREF(next);
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
    PyErr_FormatUnraisable("Exception ignored in __Pyx_PyDict_NextRef");
#else
    PyErr_WriteUnraisable(__pyx_mstate_global->__pyx_n_u_Pyx_PyDict_NextRef);
#endif
    if (pkey) *pkey = NULL;
    if (pvalue) *pvalue = NULL;
    return 0;
}
#else // !CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue) {
    int result = PyDict_Next(p, ppos, pkey, pvalue);
    if (likely(result == 1)) {
        if (pkey) Py_INCREF(*pkey);
        if (pvalue) Py_INCREF(*pvalue);
    }
    return result;
}
#endif

/* RaiseDoubleKeywords (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywordsImpl (used by ParseKeywords) */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos = NULL;
    #else
    Py_ssize_t pos = 0;
    #endif
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (
        #if CYTHON_AVOID_BORROWED_REFS
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL)
        #else
        PyDict_Next(kwds, &pos, &key, NULL)
        #endif
    ) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        #endif
    }
    __Pyx_END_CRITICAL_SECTION();
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(pos);
    #endif
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}

/* ParseKeywords */
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyDictVersioning (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* RaiseException */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static PyObject *__Pyx_GetItemInt_Generic_size(PyObject *o, Py_ssize_t i) {
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_AVOID_BORROWED_REFS
    CYTHON_UNUSED_VAR(boundscheck);
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        Py_ssize_t size = __Pyx_PyList_GET_SIZE(o);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(size < 0)) return NULL;
        #endif
        wrapped_i += size;
    }
    return __Pyx_PyList_GetItemRef(o, wrapped_i);
#elif CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS
    Py_ssize_t wrapped_i = i;
    Py_ssize_t size = (wraparound | boundscheck) ? PyList_GET_SIZE(o) : -1;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += size;
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, size))) {
        return __Pyx_PyList_GET_ITEM_REF(o, wrapped_i, unsafe_shared);
    }
    return __Pyx_GetItemInt_Generic_size(o, i);
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_AVOID_BORROWED_REFS
    CYTHON_UNUSED_VAR(boundscheck);
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(o);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(size < 0)) return NULL;
        #endif
        wrapped_i += size;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_COMPILING_IN_LIMITED_API
    return PySequence_ITEM(o, wrapped_i);
    #else
    if (unlikely(wrapped_i < 0)) {
        PyErr_SetString(PyExc_IndexError, "tuple index out of range");
        return NULL;
    }
    return PySequence_GetItem(o, wrapped_i);
    #endif
#elif CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS
    Py_ssize_t wrapped_i = i;
    Py_ssize_t size = (wraparound | boundscheck) ? PyTuple_GET_SIZE(o) : -1;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += size;
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, size))) {
        return __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic_size(o, i);
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        return boundscheck ? __Pyx_PyList_GetItemRef(o, n) : __Pyx_PyList_GET_ITEM_REF(o, n, unsafe_shared);
    } else
    #if !CYTHON_AVOID_BORROWED_REFS
    if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            return __Pyx_NewRef(PyTuple_GET_ITEM(o, n));
        }
    } else
    #endif
#else
    if ((!wraparound || i >= 0) & PyList_CheckExact(o)) {
        return boundscheck ? __Pyx_PyList_GetItemRef(o, i) : __Pyx_PyList_GET_ITEM_REF(o, i, unsafe_shared);
    } else
#endif
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
    {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (!is_list && mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (is_list || likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    (void)wraparound;
    (void)boundscheck;
    return __Pyx_GetItemInt_Generic_size(o, i);
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        #if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        if ((PyTypeObject*)obj == &PyType_Type) {
            return Py_GenericAlias(obj, key);
        }
        #endif
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_mstate_global->__pyx_n_u_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result;
    if (unlikely(!s)) return NULL;
    result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        Py_INCREF(values[i]);
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* GetTopmostException (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C0000
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#elif __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    local_value = PyErr_GetRaisedException();
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
#else
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
#endif // __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#elif __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    PyErr_SetHandledException(local_value);
    Py_XDECREF(local_value);
    Py_XDECREF(local_type);
    Py_XDECREF(local_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
#endif
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* RaiseClosureNameError */
static void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* dict_setdefault (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value) {
    PyObject* value;
#if __PYX_LIMITED_VERSION_HEX >= 0x030F0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4)
    PyDict_SetDefaultRef(d, key, default_value, &value);
#elif CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[] = {d, key, default_value};
    value = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_n_u_setdefault, args, 3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethodObjArgs(d, __pyx_mstate_global->__pyx_n_u_setdefault, key, default_value, NULL);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* LimitedApiGetTypeDict (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static Py_ssize_t __Pyx_GetTypeDictOffset(void) {
    PyObject *tp_dictoffset_o;
    Py_ssize_t tp_dictoffset;
    tp_dictoffset_o = PyObject_GetAttrString((PyObject*)(&PyType_Type), "__dictoffset__");
    if (unlikely(!tp_dictoffset_o)) return -1;
    tp_dictoffset = PyLong_AsSsize_t(tp_dictoffset_o);
    Py_DECREF(tp_dictoffset_o);
    if (unlikely(tp_dictoffset == 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' doesn't have a dictoffset");
        return -1;
    } else if (unlikely(tp_dictoffset < 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' has an unexpected negative dictoffset. "
            "Please report this as Cython bug");
        return -1;
    }
    return tp_dictoffset;
}
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp) {
    static Py_ssize_t tp_dictoffset = 0;
    if (unlikely(tp_dictoffset == 0)) {
        tp_dictoffset = __Pyx_GetTypeDictOffset();
        if (unlikely(tp_dictoffset == -1 && PyErr_Occurred())) {
            tp_dictoffset = 0; // try again next time?
            return NULL;
        }
    }
    return *(PyObject**)((char*)tp + tp_dictoffset);
}
#endif

/* SetItemOnTypeDict (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_SetItem(tp_dict, k, v);
    if (likely(!result)) {
        PyType_Modified(tp);
        if (unlikely(PyObject_HasAttr(v, __pyx_mstate_global->__pyx_n_u_set_name))) {
            PyObject *setNameResult = PyObject_CallMethodObjArgs(v, __pyx_mstate_global->__pyx_n_u_set_name,  (PyObject *) tp, k, NULL);
            if (!setNameResult) return -1;
            Py_DECREF(setNameResult);
        }
    }
    return result;
}

/* FixUpExtensionType (used by FetchCommonType) */
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if __PYX_LIMITED_VERSION_HEX > 0x030900B1
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#else
    const PyType_Slot *slot = spec->slots;
    int changed = 0;
#if !CYTHON_COMPILING_IN_LIMITED_API
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
#if !CYTHON_COMPILING_IN_CPYTHON
        const
#endif  // !CYTHON_COMPILING_IN_CPYTHON)
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_vectorcall_offset = memb->offset;
                    changed = 1;
                }
#endif  // CYTHON_METH_FASTCALL
#if !CYTHON_COMPILING_IN_PYPY
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                    Py_DECREF(descr);
                    if (unlikely(set_item_result < 0)) {
                        return -1;
                    }
                    changed = 1;
                }
#endif  // !CYTHON_COMPILING_IN_PYPY
            }
            memb++;
        }
    }
#endif  // !CYTHON_COMPILING_IN_LIMITED_API
#if !CYTHON_COMPILING_IN_PYPY
    slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_getset)
        slot++;
    if (slot && slot->slot == Py_tp_getset) {
        PyGetSetDef *getset = (PyGetSetDef*) slot->pfunc;
        while (getset && getset->name) {
            if (getset->name[0] == '_' && getset->name[1] == '_' && strcmp(getset->name, "__module__") == 0) {
                PyObject *descr = PyDescr_NewGetSet(type, getset);
                if (unlikely(!descr))
                    return -1;
                #if CYTHON_COMPILING_IN_LIMITED_API
                PyObject *pyname = PyUnicode_FromString(getset->name);
                if (unlikely(!pyname)) {
                    Py_DECREF(descr);
                    return -1;
                }
                int set_item_result = __Pyx_SetItemOnTypeDict(type, pyname, descr);
                Py_DECREF(pyname);
                #else
                CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
                int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                #endif
                Py_DECREF(descr);
                if (unlikely(set_item_result < 0)) {
                    return -1;
                }
                changed = 1;
            }
            ++getset;
        }
    }
#else
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#endif  // !CYTHON_COMPILING_IN_PYPY
    if (changed)
        PyType_Modified(type);
#endif  // PY_VERSION_HEX > 0x030900B1
    return 0;
}

/* AddModuleRef (used by FetchSharedCythonModule) */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  static PyObject *__Pyx_PyImport_AddModuleObjectRef(PyObject *name) {
      PyObject *module_dict = PyImport_GetModuleDict();
      PyObject *m;
      if (PyMapping_GetOptionalItem(module_dict, name, &m) < 0) {
          return NULL;
      }
      if (m != NULL && PyModule_Check(m)) {
          return m;
      }
      Py_XDECREF(m);
      m = PyModule_NewObject(name);
      if (m == NULL)
          return NULL;
      if (PyDict_CheckExact(module_dict)) {
          PyObject *new_m;
          (void)PyDict_SetDefaultRef(module_dict, name, m, &new_m);
          Py_DECREF(m);
          return new_m;
      } else {
           if (PyObject_SetItem(module_dict, name, m) != 0) {
                Py_DECREF(m);
                return NULL;
            }
            return m;
      }
  }
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *py_name = PyUnicode_FromString(name);
      if (!py_name) return NULL;
      PyObject *module = __Pyx_PyImport_AddModuleObjectRef(py_name);
      Py_DECREF(py_name);
      return module;
  }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif

/* FetchSharedCythonModule (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef(__PYX_ABI_MODULE_NAME);
}

/* FetchCommonType (used by CommonTypesMetaclass) */
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject* __Pyx_PyType_FromMetaclass(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *result = __Pyx_PyType_FromModuleAndSpec(module, spec, bases);
    if (result && metaclass) {
        PyObject *old_tp = (PyObject*)Py_TYPE(result);
    Py_INCREF((PyObject*)metaclass);
#if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        Py_SET_TYPE(result, metaclass);
#else
        result->ob_type = metaclass;
#endif
        Py_DECREF(old_tp);
    }
    return result;
}
#else
#define __Pyx_PyType_FromMetaclass(me, mo, s, b) PyType_FromMetaclass(me, mo, s, b)
#endif
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t expected_basicsize) {
    Py_ssize_t basicsize;
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (expected_basicsize == 0) {
        return 0; // size is inherited, nothing useful to check
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
    if (unlikely(!py_basicsize)) return -1;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = NULL;
    if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) return -1;
#else
    basicsize = ((PyTypeObject*) cached_type)->tp_basicsize;
#endif
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module = NULL, *cached_type = NULL, *abi_module_dict, *new_cached_type, *py_object_name;
    int get_item_ref_result;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    py_object_name = PyUnicode_FromString(object_name);
    if (!py_object_name) return NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) goto done;
    abi_module_dict = PyModule_GetDict(abi_module);
    if (!abi_module_dict) goto done;
    get_item_ref_result = __Pyx_PyDict_GetItemRef(abi_module_dict, py_object_name, &cached_type);
    if (get_item_ref_result == 1) {
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else if (unlikely(get_item_ref_result == -1)) {
        goto bad;
    }
    cached_type = __Pyx_PyType_FromMetaclass(
        metaclass,
        CYTHON_USE_MODULE_STATE ? module : abi_module,
        spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    new_cached_type = __Pyx_PyDict_SetDefault(abi_module_dict, py_object_name, cached_type);
    if (unlikely(new_cached_type != cached_type)) {
        if (unlikely(!new_cached_type)) goto bad;
        Py_DECREF(cached_type);
        cached_type = new_cached_type;
        if (__Pyx_VerifyCachedType(
                cached_type,
                object_name,
                spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else {
        Py_DECREF(new_cached_type);
    }
done:
    Py_XDECREF(abi_module);
    Py_DECREF(py_object_name);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CommonTypesMetaclass (used by CythonFunctionShared) */
static PyObject* __pyx_CommonTypesMetaclass_get_module(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED void* context) {
    return PyUnicode_FromString(__PYX_ABI_MODULE_NAME);
}
#if __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject* __pyx_CommonTypesMetaclass_call(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *args, CYTHON_UNUSED PyObject *kwds) {
    PyErr_SetString(PyExc_TypeError, "Cannot instantiate Cython internal types");
    return NULL;
}
static int __pyx_CommonTypesMetaclass_setattr(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *attr, CYTHON_UNUSED PyObject *value) {
    PyErr_SetString(PyExc_TypeError, "Cython internal types are immutable");
    return -1;
}
#endif
static PyGetSetDef __pyx_CommonTypesMetaclass_getset[] = {
    {"__module__", __pyx_CommonTypesMetaclass_get_module, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_CommonTypesMetaclass_slots[] = {
    {Py_tp_getset, (void *)__pyx_CommonTypesMetaclass_getset},
    #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {Py_tp_call, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_new, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_setattro, (void*)__pyx_CommonTypesMetaclass_setattr},
    #endif
    {0, 0}
};
static PyType_Spec __pyx_CommonTypesMetaclass_spec = {
    __PYX_TYPE_MODULE_PREFIX "_common_types_metatype",
    0,
    0,
    Py_TPFLAGS_IMMUTABLETYPE |
    Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT,
    __pyx_CommonTypesMetaclass_slots
};
static int __pyx_CommonTypesMetaclass_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    PyObject *bases = PyTuple_Pack(1, &PyType_Type);
    if (unlikely(!bases)) {
        return -1;
    }
    mstate->__pyx_CommonTypesMetaclassType = __Pyx_FetchCommonTypeFromSpec(NULL, module, &__pyx_CommonTypesMetaclass_spec, bases);
    Py_DECREF(bases);
    if (unlikely(mstate->__pyx_CommonTypesMetaclassType == NULL)) {
        return -1;
    }
    return 0;
}

/* CallTypeTraverse (used by CythonFunctionShared) */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg) {
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
    if (__Pyx_get_runtime_version() < 0x03090000) return 0;
    #endif
    if (!always_call) {
        PyTypeObject *base = __Pyx_PyObject_GetSlot(o, tp_base, PyTypeObject*);
        unsigned long flags = PyType_GetFlags(base);
        if (flags & Py_TPFLAGS_HEAPTYPE) {
            return 0;
        }
    }
    Py_VISIT((PyObject*)Py_TYPE(o));
    return 0;
}
#endif

/* PyMethodNew (used by CythonFunctionShared) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *result;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    #if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    {
        PyObject *args[] = {func, self};
        result = PyObject_Vectorcall(__pyx_mstate_global->__Pyx_CachedMethodType, args, 2, NULL);
    }
    #else
    result = PyObject_CallFunctionObjArgs(__pyx_mstate_global->__Pyx_CachedMethodType, func, self, NULL);
    #endif
    return result;
}
#else
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#endif

/* PyVectorcallFastCallDict (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i;
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos;
    #else
    Py_ssize_t pos;
    #endif
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    #if !CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t nkw = PyDict_Size(kw);
    if (unlikely(nkw == -1)) return NULL;
    #else
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    #endif
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = 0;
    i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (__Pyx_PyDict_NextRef(kw, &pos, &key, &value)) {
        keys_are_strings &=
        #if CYTHON_COMPILING_IN_LIMITED_API
            PyType_GetFlags(Py_TYPE(key));
        #else
            Py_TYPE(key)->tp_flags;
        #endif
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(PyTuple_SetItem(kwnames, i, key) < 0)) goto cleanup;
        #else
        PyTuple_SET_ITEM(kwnames, i, key);
        #endif
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(pos);
    #endif
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    Py_ssize_t kw_size =
        likely(kw == NULL) ?
        0 :
#if !CYTHON_ASSUME_SAFE_SIZE
        PyDict_Size(kw);
#else
        PyDict_GET_SIZE(kw);
#endif
    if (kw_size == 0) {
        return vc(func, args, nargs, NULL);
    }
#if !CYTHON_ASSUME_SAFE_SIZE
    else if (unlikely(kw_size == -1)) {
        return NULL;
    }
#endif
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared (used by CythonFunction) */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunctionNoMethod(PyObject *func, void (*cfunc)(void)) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if ((PyObject*)Py_TYPE(func) == __pyx_mstate_global->__Pyx_CachedMethodType) {
        int result;
        PyObject *newFunc = PyObject_GetAttr(func, __pyx_mstate_global->__pyx_n_u_func_2);
        if (unlikely(!newFunc)) {
            PyErr_Clear(); // It's only an optimization, so don't throw an error
            return 0;
        }
        result = __Pyx__IsSameCyOrCFunctionNoMethod(newFunc, cfunc);
        Py_DECREF(newFunc);
        return result;
    }
    return __Pyx__IsSameCyOrCFunctionNoMethod(func, cfunc);
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if (PyMethod_Check(func)) {
        func = PyMethod_GET_FUNCTION(func);
    }
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure) {
    PyObject *result;
    CYTHON_UNUSED_VAR(closure);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_doc_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#else
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_name_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    Py_INCREF(op->func_qualname);
    result = op->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
#endif
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_defaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_kwdefaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int __Pyx_CyFunction_set_annotate_in_dict_if_exists(PyObject *op_in, PyObject *value);
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    __Pyx_END_CRITICAL_SECTION();
    if (unlikely(__Pyx_CyFunction_set_annotate_in_dict_if_exists((PyObject*) op, Py_None) < 0)) return -1;
    return 0;
}
static int
__Pyx_CyFunction_get_dict_if_exists(PyObject *op_in, PyObject **dict) {
    /* Return 1 if the function dict exists, 0 otherwise.  This cannot fail:
     * _PyObject_GetDictPtr() may clear errors internally, but never reports them. */
#if CYTHON_COMPILING_IN_PYPY
    *dict = PyObject_GenericGetDict(op_in, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030C0000
    *dict = ((__pyx_CyFunctionObject*) op_in)->func_dict;
#else
    PyObject **dictptr = _PyObject_GetDictPtr(op_in);
    *dict = likely(dictptr) ? *dictptr : NULL;
#endif
    return *dict ? 1 : 0;
}
static int
__Pyx_CyFunction_get_annotate_from_dict_if_exists(PyObject *op_in, PyObject **annotate) {
    PyObject *dict;
    int dict_found;
    *annotate = NULL;
    dict_found = __Pyx_CyFunction_get_dict_if_exists(op_in, &dict);
    if (!dict_found) return 0;
    return __Pyx_PyDict_GetItemRef(dict, __pyx_mstate_global->__pyx_n_u_annotate, annotate);
}
static int
__Pyx_CyFunction_set_annotate_in_dict_if_exists(PyObject *op_in, PyObject *value) {
    PyObject *dict;
    int dict_found;
    dict_found = __Pyx_CyFunction_get_dict_if_exists(op_in, &dict);
    if (!dict_found) return 0;
    return PyDict_SetItem(dict, __pyx_mstate_global->__pyx_n_u_annotate, value);
}
static int
__Pyx_CyFunction_set_annotate_in_dict(PyObject *op_in, PyObject *value) {
    PyObject *dict;
    int result;
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    dict = __Pyx_CyFunction_get_dict((__pyx_CyFunctionObject*) op_in, NULL);
#else
    dict = PyObject_GenericGetDict(op_in, NULL);
#endif
    if (unlikely(!dict)) return -1;
    result = PyDict_SetItem(dict, __pyx_mstate_global->__pyx_n_u_annotate, value);
    Py_DECREF(dict);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations(PyObject *op_in, void *context) {
    PyObject *annotate = NULL;
    PyObject *result = NULL;
    __pyx_CyFunctionObject *op = (__pyx_CyFunctionObject*) op_in;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op_in);
    result = __Pyx_XNewRef(op->func_annotations);
    __Pyx_END_CRITICAL_SECTION();
    if (result) return result;
    if (unlikely(__Pyx_CyFunction_get_annotate_from_dict_if_exists(op_in, &annotate) < 0)) return NULL;
    if (!annotate || annotate == Py_None) {
        Py_XDECREF(annotate);
        __Pyx_BEGIN_CRITICAL_SECTION(op_in);
        result = __Pyx_CyFunction_get_annotations_locked(op);
        __Pyx_END_CRITICAL_SECTION();
        return result;
    }
    PyObject *format = PyLong_FromLong(1L);  // annotationlib.Format.VALUE
    if (likely(format)) {
        result = __Pyx_PyObject_CallOneArg(annotate, format);
        Py_DECREF(format);
    }
    Py_DECREF(annotate);
    if (unlikely(!result)) return NULL;
    if (unlikely(!PyDict_Check(result))) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ must return a dict");
        Py_DECREF(result);
        return NULL;
    }
    __Pyx_BEGIN_CRITICAL_SECTION(op_in);
    __Pyx_Py_XDECREF_SET(op->func_annotations, __Pyx_NewRef(result));
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyObject *__Pyx_CyFunction_annotate_impl(PyObject *self, PyObject *args) {
    CYTHON_UNUSED_VAR(args);
    if (unlikely(!self)) {
        PyErr_SetString(PyExc_SystemError, "cython __annotate__ called without 'self' argument");
    }
    Py_XINCREF(self);
    return self;
}
static PyMethodDef __Pyx_CyFunction_annotate_method = {
    "__annotate__",
    (PyCFunction)(void (*)(void))__Pyx_CyFunction_annotate_impl,
    METH_VARARGS,
    "Placeholder __annotate__ function to allow 'functools.wraps' to work "
    "on Cython functions."
};
static PyObject *
__Pyx_CyFunction_get_annotate(PyObject *op_in, void *context) {
    PyObject *annotate = NULL;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(__Pyx_CyFunction_get_annotate_from_dict_if_exists(op_in, &annotate) < 0)) return NULL;
    if (annotate) return annotate;
    PyObject *annotations = __Pyx_CyFunction_get_annotations(op_in, NULL);
    if (unlikely(!annotations)) return NULL;
    PyObject *method = PyCFunction_New(
        &__Pyx_CyFunction_annotate_method,
        annotations);
    Py_DECREF(annotations);
    return method;
}
static int
__Pyx_CyFunction_set_annotate(PyObject *op_in, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ cannot be deleted");
        return -1;
    }
    if (unlikely(value != Py_None && !PyCallable_Check(value))) {
        PyErr_SetString(PyExc_TypeError, "__annotate__ must be callable or None");
        return -1;
    }
    if (value != Py_None) {
        __pyx_CyFunctionObject *op = (__pyx_CyFunctionObject*) op_in;
        __Pyx_BEGIN_CRITICAL_SECTION(op_in);
        Py_CLEAR(op->func_annotations);
        __Pyx_END_CRITICAL_SECTION();
    }
    return __Pyx_CyFunction_set_annotate_in_dict(op_in, value);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine_value(__pyx_CyFunctionObject *op) {
    int is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
    if (is_coroutine) {
        PyObject *is_coroutine_value, *module, *fromlist, *marker = __pyx_mstate_global->__pyx_n_u_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_mstate_global->__pyx_n_u_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        is_coroutine_value = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(is_coroutine_value)) {
            return is_coroutine_value;
        }
ignore:
        PyErr_Clear();
    }
    return __Pyx_PyBool_FromLong(is_coroutine);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    result = __Pyx_CyFunction_get_is_coroutine_value(op);
    if (unlikely(!result))
        return NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    if (op->func_is_coroutine) {
        Py_DECREF(result);
        result = __Pyx_NewRef(op->func_is_coroutine);
    } else {
        op->func_is_coroutine = __Pyx_NewRef(result);
    }
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static void __Pyx_CyFunction_raise_argument_count_error(__pyx_CyFunctionObject *func, const char* message, Py_ssize_t size) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        py_name, message, size);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        name, message, size);
#endif
}
static void __Pyx_CyFunction_raise_type_error(__pyx_CyFunctionObject *func, const char* message) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s",
        py_name, message);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s",
        name, message);
#endif
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {"func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {"func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
#else
    {"func_dict", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
#endif
    {"func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {"__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {"__annotate__", (getter)__Pyx_CyFunction_get_annotate, (setter)__Pyx_CyFunction_set_annotate, 0, 0},
    {"_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    {"__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#endif
#if CYTHON_METH_FASTCALL
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {"__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(args);
    __Pyx_BEGIN_CRITICAL_SECTION(m);
    Py_INCREF(m->func_qualname);
    result = m->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_dict = NULL;
#endif
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func_dict);
#elif PY_VERSION_HEX < 0x030d0000
    _PyObject_ClearManagedDict((PyObject*)m);
#else
    PyObject_ClearManagedDict((PyObject*)m);
#endif
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    Py_CLEAR(m->defaults);
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    {
        int e = __Pyx_call_type_traverse((PyObject*)m, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func_dict);
#else
    {
        int e =
#if PY_VERSION_HEX < 0x030d0000
            _PyObject_VisitManagedDict
#else
            PyObject_VisitManagedDict
#endif
                ((PyObject*)m, visit, arg);
        if (e != 0) return e;
    }
#endif
    __Pyx_VISIT_CONST(m->func_name);
    __Pyx_VISIT_CONST(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    __Pyx_VISIT_CONST(m->func_code);
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_annotations);
    Py_VISIT(m->func_is_coroutine);
    Py_VISIT(m->defaults);
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
    PyObject *repr;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    repr = PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
    __Pyx_END_CRITICAL_SECTION();
    return repr;
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void(*)(void))meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes no arguments", size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes exactly one argument", size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
    __Pyx_CyFunction_raise_type_error(
        (__pyx_CyFunctionObject*)func, "takes no keyword arguments");
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_SIZE
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(argc < 0)) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            __Pyx_CyFunction_raise_type_error(
                cyfunc, "needs an argument");
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(__Pyx_PyTuple_GET_SIZE(kwnames))) {
        __Pyx_CyFunction_raise_type_error(
            cyfunc, "takes no keyword arguments");
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes no arguments", nargs);
        return NULL;
    }
    return meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes exactly one argument", nargs);
        return NULL;
    }
    return meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    #if PY_VERSION_HEX < 0x030e00A6
    size_t nargs_value = (size_t) nargs;
    #else
    Py_ssize_t nargs_value = nargs;
    #endif
    return ((__Pyx_PyCMethod)(void(*)(void))meth)(self, cls, args, nargs_value, kwnames);
}
#endif
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if CYTHON_METH_FASTCALL
#if defined(Py_TPFLAGS_HAVE_VECTORCALL)
    Py_TPFLAGS_HAVE_VECTORCALL |
#elif defined(_Py_TPFLAGS_HAVE_VECTORCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
#endif // CYTHON_METH_FASTCALL
#if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_TPFLAGS_MANAGED_DICT |
#endif
    Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
static int __pyx_CyFunction_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_CyFunctionType_spec, NULL);
    if (unlikely(mstate->__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func, PyTypeObject *defaults_type) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_CallObject((PyObject*)defaults_type, NULL); // _PyObject_New(defaults_type);
    if (unlikely(!m->defaults))
        return NULL;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
    else state = (PyGILState_STATE)0;
    CYTHON_UNUSED_VAR(clineno);
    CYTHON_UNUSED_VAR(lineno);
    CYTHON_UNUSED_VAR(filename);
    CYTHON_MAYBE_UNUSED_VAR(nogil);
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(0);
    }
    ctx = PyUnicode_FromString(name);
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
    if (nogil)
        PyGILState_Release(state);
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* UnicodeConcatInPlace */
# if CYTHON_COMPILING_IN_CPYTHON
static int
__Pyx_unicode_modifiable(PyObject *unicode, int unsafe_shared)
{
    if (!__Pyx_IS_UNIQUELY_REFERENCED(unicode, unsafe_shared))
        return 0;
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030F0000
    if (PyUnstable_Unicode_GET_CACHED_HASH(unicode) != -1)
        return 0;
#endif
    if (!PyUnicode_CheckExact(unicode))
        return 0;
    if (PyUnicode_CHECK_INTERNED(unicode))
        return 0;
    return 1;
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    ) {
    PyObject *left = *p_left;
    Py_ssize_t left_len, right_len, new_len;
    if (unlikely(__Pyx_PyUnicode_READY(left) == -1))
        return NULL;
    if (unlikely(__Pyx_PyUnicode_READY(right) == -1))
        return NULL;
    left_len = PyUnicode_GET_LENGTH(left);
    if (left_len == 0) {
        Py_INCREF(right);
        return right;
    }
    right_len = PyUnicode_GET_LENGTH(right);
    if (right_len == 0) {
        Py_INCREF(left);
        return left;
    }
    if (unlikely(left_len > PY_SSIZE_T_MAX - right_len)) {
        PyErr_SetString(PyExc_OverflowError,
                        "strings are too large to concat");
        return NULL;
    }
    new_len = left_len + right_len;
    if (left != right
            && __Pyx_unicode_modifiable(left, unsafe_shared)
            && PyUnicode_CheckExact(right)
            && PyUnicode_KIND(right) <= PyUnicode_KIND(left)
            && !(PyUnicode_IS_ASCII(left) && !PyUnicode_IS_ASCII(right))) {
        int ret;
        __Pyx_GIVEREF(*p_left);
        ret = PyUnicode_Resize(p_left, new_len);
        __Pyx_GOTREF(*p_left);
        if (unlikely(ret != 0))
            return NULL;
        #if PY_VERSION_HEX >= 0x030d0000
        if (unlikely(PyUnicode_CopyCharacters(*p_left, left_len, right, 0, right_len) < 0)) return NULL;
        #else
        _PyUnicode_FastCopyCharacters(*p_left, left_len, right, 0, right_len);
        #endif
        __Pyx_INCREF(*p_left);
        __Pyx_GIVEREF(*p_left);
        return *p_left;
    } else {
        return __Pyx_PyUnicode_Concat(left, right);
    }
  }
#endif

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolEqObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 1;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) == 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 0;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 0;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal == 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a == (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* RejectKeywords */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds) {
    PyObject *key = NULL;
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds))) {
        key = __Pyx_PySequence_ITEM(kwds, 0);
    } else {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *pos = NULL;
#else
        Py_ssize_t pos = 0;
#endif
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
        if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return;
#endif
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL);
#if CYTHON_AVOID_BORROWED_REFS
        Py_XDECREF(pos);
#endif
    }
    if (likely(key)) {
        PyErr_Format(PyExc_TypeError,
            "%s() got an unexpected keyword argument '%U'",
            function_name, key);
        Py_DECREF(key);
    }
}

/* decode_c_bytes (used by decode_bytes) */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    if (unlikely((start < 0) | (stop < 0))) {
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (stop > length)
        stop = length;
    if (unlikely(stop <= start))
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_unicode);
    length = stop - start;
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* ArgTypeTestFunc (used by ArgTypeTest) */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    PyObject *extra_info = __pyx_mstate_global->__pyx_empty_unicode;
    int from_annotation_subclass = 0;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (!exact) {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    } else if (exact == 2) {
        if (__Pyx_TypeCheck(obj, type)) {
            from_annotation_subclass = 1;
            extra_info = __pyx_mstate_global->__pyx_kp_u_Note_that_Cython_is_deliberately;
        }
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")"
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        "%s%U"
#endif
        , name, type_name, obj_type_name
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        , (from_annotation_subclass ? ". " : ""), extra_info
#endif
        );
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (exact == 2 && from_annotation_subclass) {
        PyObject *res;
        PyObject *vargs[2];
        vargs[0] = PyErr_GetRaisedException();
        vargs[1] = extra_info;
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_kp_u_add_note, vargs, 2, NULL);
        Py_XDECREF(res);
        PyErr_SetRaisedException(vargs[0]);
    }
#endif
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    const char* function_name,
    PyObject *kw)
{
#if CYTHON_COMPILING_IN_PYPY && !defined(PyArg_ValidateKeywordArguments)
    CYTHON_UNUSED_VAR(function_name);
    CYTHON_UNUSED_VAR(kw);
    return 0;
#else
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kw))) {
#if PY_VERSION_HEX >= 0x03090000
        CYTHON_UNUSED_VAR(function_name);
#else
        Py_ssize_t kwsize;
        #if CYTHON_ASSUME_SAFE_SIZE
        kwsize = PyTuple_GET_SIZE(kw);
        #else
        kwsize = PyTuple_Size(kw);
        if (unlikely(kwsize < 0)) return -1;
        #endif
        for (Py_ssize_t pos = 0; pos < kwsize; pos++) {
            PyObject* key = NULL;
            #if CYTHON_ASSUME_SAFE_MACROS
            key = PyTuple_GET_ITEM(kw, pos);
            #else
            key = PyTuple_GetItem(kw, pos);
            if (unlikely(!key)) return -1;
            #endif
            if (unlikely(!PyUnicode_Check(key))) {
                PyErr_Format(PyExc_TypeError,
                    "%.200s() keywords must be strings", function_name);
                return -1;
            }
        }
#endif
    } else {
        if (unlikely(!PyArg_ValidateKeywordArguments(kw))) return -1;
    }
    return 0;
#endif
}

/* PyLongBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_Fallback___Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, int inplace) {
    return (inplace ? PyNumber_InPlaceFloorDivide : PyNumber_FloorDivide)(op1, op2);
}
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject* __Pyx_Unpacked___Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    long a;
    const PY_LONG_LONG llb = intval;
    PY_LONG_LONG lla;
    if (unlikely(__Pyx_PyLong_IsZero(op1))) {
        return __Pyx_NewRef(op1);
    }
    const int is_positive = __Pyx_PyLong_IsPos(op1);
    const digit* digits = __Pyx_PyLong_Digits(op1);
    const Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
    if (likely(size == 1)) {
        a = (long) digits[0];
        if (!is_positive) a *= -1;
    } else {
        switch (size) {
            case 2:
                if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                    a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 3:
                if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                    a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 4:
                if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                    a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
        }
        return PyLong_Type.tp_as_number->nb_floor_divide(op1, op2);
    }
    calculate_long:
        {
            long q, r;
            q = a / b;
            r = a - q*b;
            q -= ((r != 0) & ((r ^ b) < 0));
            return PyLong_FromLong(q);
        }
    calculate_long_long:
        {
            PY_LONG_LONG q, r;
            q = lla / llb;
            r = lla - q*llb;
            q -= ((r != 0) & ((r ^ llb) < 0));
            return PyLong_FromLongLong(q);
        }
    
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        return __Pyx_Unpacked___Pyx_PyLong_FloorDivideObjC(op1, op2, intval, inplace, zerodivision_check);
    }
    #endif
    return __Pyx_Fallback___Pyx_PyLong_FloorDivideObjC(op1, op2, inplace);
}
#endif

/* PyLongCompare */
static CYTHON_INLINE int __Pyx_PyLong_BoolNeObjC(PyObject *op1, PyObject *op2, long intval, long inplace) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(inplace);
    if (op1 == op2) {
        return 0;
    }
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
        const digit* digits = __Pyx_PyLong_Digits(op1);
        if (intval == 0) {
            return (__Pyx_PyLong_IsZero(op1) != 1);
        } else if (intval < 0) {
            if (__Pyx_PyLong_IsNonNeg(op1))
                return 1;
            intval = -intval;
        } else {
            if (__Pyx_PyLong_IsNeg(op1))
                return 1;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        return (unequal != 0);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = __Pyx_PyFloat_AS_DOUBLE(op1);
        return ((double)a != (double)b);
    }
    return __Pyx_PyObject_IsTrueAndDecref(
        PyObject_RichCompare(op1, op2, Py_NE));
}

/* PyLongBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_Fallback___Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, int inplace) {
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject* __Pyx_Unpacked___Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(inplace);
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    long a;
    const PY_LONG_LONG llb = intval;
    PY_LONG_LONG lla;
    if (unlikely(__Pyx_PyLong_IsZero(op1))) {
        return __Pyx_NewRef(op2);
    }
    const int is_positive = __Pyx_PyLong_IsPos(op1);
    const digit* digits = __Pyx_PyLong_Digits(op1);
    const Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
    if (likely(size == 1)) {
        a = (long) digits[0];
        if (!is_positive) a *= -1;
    } else {
        switch (size) {
            case 2:
                if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                    a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 3:
                if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                    a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
            case 4:
                if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                    a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if (!is_positive) a *= -1;
                    goto calculate_long;
                } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                    lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                    if (!is_positive) lla *= -1;
                    goto calculate_long_long;
                }
                break;
        }
        return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    calculate_long:
        {
            long x;
            x = a + b;
            return PyLong_FromLong(x);
        }
    calculate_long_long:
        {
            PY_LONG_LONG llx;
            llx = lla + llb;
            return PyLong_FromLongLong(llx);
        }
    
}
#endif
static PyObject* __Pyx_Float___Pyx_PyLong_AddObjC(PyObject *float_val, long intval, int zerodivision_check) {
    CYTHON_UNUSED_VAR(zerodivision_check);
    const long b = intval;
    double a = __Pyx_PyFloat_AS_DOUBLE(float_val);
        double result;
        
        result = ((double)a) + (double)b;
        return PyFloat_FromDouble(result);
}
static CYTHON_INLINE PyObject* __Pyx_PyLong_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
    CYTHON_MAYBE_UNUSED_VAR(intval);
    CYTHON_UNUSED_VAR(zerodivision_check);
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        return __Pyx_Unpacked___Pyx_PyLong_AddObjC(op1, op2, intval, inplace, zerodivision_check);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        return __Pyx_Float___Pyx_PyLong_AddObjC(op1, intval, zerodivision_check);
    }
    return __Pyx_Fallback___Pyx_PyLong_AddObjC(op1, op2, inplace);
}
#endif

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
    __Pyx_TypeName obj_type_name;
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyLong_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyLong_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is unsliceable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
bad:
    return NULL;
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* PyObjectCall2Args (used by CallUnboundCMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    int was_initialized =  __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return __Pyx_CallCFunction(cfunc, self, arg);
        } else if (flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, &arg, 1);
        } else if (flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, &arg, 1, NULL);
        }
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod1(&tmp_cfunc, self, arg);
    }
#endif
    PyObject* result = __Pyx__CallUnboundCMethod1(cfunc, self, arg);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *args = PyTuple_New(1);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
    } else
#endif
    {
        result = __Pyx_PyObject_Call2Args(cfunc->method, self, arg);
    }
    return result;
}

/* dict_getitem_default */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyBytes_CheckExact(key) || PyUnicode_CheckExact(key) || PyLong_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* pybuiltin_invalid (used by pyint_simplify) */
static void __Pyx_PyBuiltin_Invalid(PyObject *obj, const char *type_name, const char *argname) {
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    if (argname) {
        PyErr_Format(PyExc_TypeError,
            "Argument '%.200s' has incorrect type (expected %.200s, got " __Pyx_FMT_TYPENAME ")",
            argname, type_name, obj_type_name
        );
    } else {
        PyErr_Format(PyExc_TypeError,
            "Expected %.200s, got " __Pyx_FMT_TYPENAME,
            type_name, obj_type_name
        );
    }
    __Pyx_DECREF_TypeName(obj_type_name);
}

/* pyint_simplify */
static CYTHON_INLINE int __Pyx_PyInt_FromNumber(PyObject **number_var, const char *argname, int accept_none) {
    PyObject *number = *number_var;
    if (likely((accept_none && number == Py_None) || PyLong_CheckExact(number))) {
        return 0;
    }
    PyObject *int_object;
    if (likely(PyNumber_Check(number))) {
        int_object = PyNumber_Long(number);
        if (unlikely(!int_object)) goto bad;
    } else {
        __Pyx_PyBuiltin_Invalid(number, "int", argname);
        goto bad;
    }
    *number_var = int_object;
    Py_DECREF(number);
    return 0;
bad:
    *number_var = NULL;
    Py_DECREF(number);
    return -1;
}

/* PyObjectVectorCallKwBuilder */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    Py_INCREF(key);
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* AllocateExtensionType */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final) {
    if (is_final || likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
        allocfunc alloc_func = __Pyx_PyType_GetSlot(t, tp_alloc, allocfunc);
        return alloc_func(t, 0);
    } else {
        newfunc tp_new = __Pyx_PyType_TryGetSlot(&PyBaseObject_Type, tp_new, newfunc);
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (!tp_new) {
            PyObject *new_str = PyUnicode_FromString("__new__");
            if (likely(new_str)) {
                PyObject *o = PyObject_CallMethodObjArgs((PyObject *)&PyBaseObject_Type, new_str, t, NULL);
                Py_DECREF(new_str);
                return o;
            } else
                return NULL;
        } else
    #endif
        return tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
    }
}

/* FunctionExport */
static int __Pyx_ExportFunction(PyObject *api_dict, const char *name, void (*f)(void), const char *sig) {
    PyObject *cobj;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    tmp.fp = f;
    cobj = PyCapsule_New(tmp.p, sig, 0);
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(api_dict, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    return 0;
bad:
    Py_XDECREF(cobj);
    return -1;
}

/* GetApiDict */
static PyObject *__Pyx_ApiExport_GetApiDict(void) {
    PyObject *d;
    if (__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_capi, &d) == -1)
        return NULL;
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_pyx_capi, d) < 0)
            goto bad;
    }
    return d;
bad:
    Py_XDECREF(d);
    return NULL;
}

/* PyObjectCallNoArg (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(tp);
    PyErr_Format(PyExc_AttributeError,
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}
#endif

/* PyObjectCallMethod0 (used by PyType_Ready) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[1] = {obj};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_CallNoArg;
    return PyObject_VectorcallMethod(method_name, args, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
#endif
}

/* ValidateBasesTuple (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_SIZE
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (unlikely(n < 0)) return -1;
#endif
    for (i = 1; i < n; i++)
    {
        PyTypeObject *b;
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !CYTHON_USE_TYPE_SLOTS
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
CYTHON_UNUSED static int __Pyx_PyType_HasMultipleInheritance(PyTypeObject *t) {
    while (t) {
        PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
        if (bases) {
            return 1;
        }
        t = __Pyx_PyType_GetSlot(t, tp_base, PyTypeObject*);
    }
    return 0;
}
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !CYTHON_COMPILING_IN_CPYTHON || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    if (!__Pyx_PyType_HasMultipleInheritance(t)) {
        return PyType_Ready(t);
    }
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400) &&\
                !CYTHON_COMPILING_IN_GRAAL
        gc = PyImport_GetModule(__pyx_mstate_global->__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_mstate_global->__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i=0;
    Py_ssize_t size;
    void** base_vtables;
    __Pyx_TypeName tp_base_name = NULL;
    __Pyx_TypeName base_name = NULL;
    void* unknown = (void*)-1;
    PyObject* bases = __Pyx_PyType_GetSlot(type, tp_bases, PyObject*);
    int base_depth = 0;
    {
        PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        while (base) {
            base_depth += 1;
            base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
        }
    }
    base_vtables = (void**) PyMem_Malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
#if CYTHON_COMPILING_IN_LIMITED_API
    size = PyTuple_Size(bases);
    if (size < 0) goto other_failure;
#else
    size = PyTuple_GET_SIZE(bases);
#endif
    for (i = 1; i < size; i++) {
        PyObject *basei;
        void* base_vtable;
#if CYTHON_AVOID_BORROWED_REFS
        basei = PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#else
        basei = PyTuple_GET_ITEM(bases, i);
#endif
        base_vtable = __Pyx_GetVtable((PyTypeObject*)basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
            }
        }
    }
    PyErr_Clear();
    PyMem_Free(base_vtables);
    return 0;
bad:
    {
        PyTypeObject* basei = NULL;
        PyTypeObject* tp_base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        tp_base_name = __Pyx_PyType_GetFullyQualifiedName(tp_base);
#if CYTHON_AVOID_BORROWED_REFS
        basei = (PyTypeObject*)PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = (PyTypeObject*)PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#else
        basei = (PyTypeObject*)PyTuple_GET_ITEM(bases, i);
#endif
        base_name = __Pyx_PyType_GetFullyQualifiedName(basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
    }
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
really_bad: // bad has failed!
#endif
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
other_failure:
#endif
    PyMem_Free(base_vtables);
    return -1;
}

/* DelItemOnTypeDict (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_DelItem(tp_dict, k);
    if (likely(!result)) PyType_Modified(tp);
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_mstate_global->__pyx_n_u_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_mstate_global->__pyx_n_u_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython);
            if (likely(reduce_cython)) {
                ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_mstate_global->__pyx_n_u_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetFullyQualifiedName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_2_9
#define __PYX_HAVE_RT_ImportType_3_2_9
static PyTypeObject *__Pyx_ImportType_3_2_9(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_9 check_size)
{
    PyObject *result = 0;
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API)
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !( defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API) )
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    if (size == 0) {
        return (PyTypeObject *)result;
    }
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_2_9 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_2_9 && (size_t)basicsize > size) {
        if (PyErr_WarnFormat(NULL, 0,
                "%.200s.%.200s size changed, may indicate binary incompatibility. "
                "Expected %zd from C header, got %zd from PyObject",
                module_name, class_name, size, basicsize) < 0) {
            goto bad;
        }
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* PxdImportShared (used by FunctionImport) */
#ifndef __PYX_HAVE_RT_ImportFromPxd_3_2_9
#define __PYX_HAVE_RT_ImportFromPxd_3_2_9
static int __Pyx_ImportFromPxd_3_2_9(PyObject *module, const char *name, void **p, const char *sig, const char *what) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, "__pyx_capi__");
    if (!d)
        goto bad;
#if (defined(Py_LIMITED_API) && Py_LIMITED_API >= 0x030d0000) || (!defined(Py_LIMITED_API) && PY_VERSION_HEX >= 0x030d0000)
    PyDict_GetItemStringRef(d, name, &cobj);
#else
    cobj = PyDict_GetItemString(d, name);
    Py_XINCREF(cobj);
#endif
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C %.8s %.200s",
                PyModule_GetName(module), what, name);
        goto bad;
    }
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C %.8s %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             what, PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    Py_DECREF(cobj);
    return 0;
bad:
    Py_XDECREF(d);
    Py_XDECREF(cobj);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction_3_2_9
#define __PYX_HAVE_RT_ImportFunction_3_2_9
static int __Pyx_ImportFunction_3_2_9(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    int result = __Pyx_ImportFromPxd_3_2_9(module, funcname, &tmp.p, sig, "function");
    if (result == 0) {
        *f = tmp.fp;
    }
    return result;
}
#endif

/* HasAttr (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!PyUnicode_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_PyObject_GetAttrStrNoError(o, n);
    if (!r) {
        return (unlikely(PyErr_Occurred())) ? -1 : 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}
#endif

/* ImportImpl (used by Import) */
static int __Pyx__Import_GetModule(PyObject *qualname, PyObject **module) {
    PyObject *imported_module = PyImport_GetModule(qualname);
    if (unlikely(!imported_module)) {
        *module = NULL;
        if (PyErr_Occurred()) {
            return -1;
        }
        return 0;
    }
    *module = imported_module;
    return 1;
}
static int __Pyx__Import_Lookup(PyObject *qualname, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject **module) {
    PyObject *imported_module;
    PyObject *top_level_package_name;
    Py_ssize_t i;
    int status, module_found;
    Py_ssize_t dot_index;
    module_found = __Pyx__Import_GetModule(qualname, &imported_module);
    if (unlikely(!module_found || module_found == -1)) {
        *module = NULL;
        return module_found;
    }
    if (imported_names) {
        for (i = 0; i < len_imported_names; i++) {
            PyObject *imported_name = imported_names[i];
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
            int has_imported_attribute = PyObject_HasAttr(imported_module, imported_name);
#else
            int has_imported_attribute = PyObject_HasAttrWithError(imported_module, imported_name);
            if (unlikely(has_imported_attribute == -1)) goto error;
#endif
            if (!has_imported_attribute) {
                goto not_found;
            }
        }
        *module = imported_module;
        return 1;
    }
    dot_index = PyUnicode_FindChar(qualname, '.', 0, PY_SSIZE_T_MAX, 1);
    if (dot_index == -1) {
        *module = imported_module;
        return 1;
    }
    if (unlikely(dot_index == -2)) goto error;
    top_level_package_name = PyUnicode_Substring(qualname, 0, dot_index);
    if (unlikely(!top_level_package_name)) goto error;
    Py_DECREF(imported_module);
    status = __Pyx__Import_GetModule(top_level_package_name, module);
    Py_DECREF(top_level_package_name);
    return status;
error:
    Py_DECREF(imported_module);
    *module = NULL;
    return -1;
not_found:
    Py_DECREF(imported_module);
    *module = NULL;
    return 0;
}
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *from_list = 0;
    int module_found;
    if (!qualname) {
        qualname = name;
    }
    module_found = __Pyx__Import_Lookup(qualname, imported_names, len_imported_names, &module);
    if (likely(module_found == 1)) {
        return module;
    } else if (unlikely(module_found == -1)) {
        return NULL;
    }
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    if (imported_names) {
#if CYTHON_COMPILING_IN_CPYTHON
        from_list = __Pyx_PyList_FromArray(imported_names, len_imported_names);
        if (unlikely(!from_list))
            goto bad;
#else
        from_list = PyList_New(len_imported_names);
        if (unlikely(!from_list)) goto bad;
        for (Py_ssize_t i=0; i<len_imported_names; ++i) {
            if (PyList_SetItem(from_list, i, __Pyx_NewRef(imported_names[i])) < 0) goto bad;
        }
#endif
    }
    if (level == -1) {
        const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
        if (package_sep != (0)) {
            module = PyImport_ImportModuleLevelObject(
                name, moddict, empty_dict, from_list, 1);
            if (unlikely(!module)) {
                if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                    goto bad;
                PyErr_Clear();
            }
        }
        level = 0;
    }
    if (!module) {
        module = PyImport_ImportModuleLevelObject(
            name, moddict, empty_dict, from_list, level);
    }
bad:
    Py_XDECREF(from_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level) {
    return __Pyx__Import(name, imported_names, len_imported_names, qualname, __pyx_mstate_global->__pyx_d, level);
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u_);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                CYTHON_COMPILING_IN_GRAAL
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
    }
    return value;
}

/* Py3UpdateBases */
static PyObject*
__Pyx_PEP560_update_bases(PyObject *bases)
{
    Py_ssize_t i, j, size_bases;
    PyObject *base = NULL, *meth, *new_base, *result, *new_bases = NULL;
#if CYTHON_ASSUME_SAFE_SIZE
    size_bases = PyTuple_GET_SIZE(bases);
#else
    size_bases = PyTuple_Size(bases);
    if (size_bases < 0) return NULL;
#endif
    for (i = 0; i < size_bases; i++) {
#if CYTHON_AVOID_BORROWED_REFS
        Py_CLEAR(base);
#endif
#if CYTHON_ASSUME_SAFE_MACROS
        base = PyTuple_GET_ITEM(bases, i);
#else
        base = PyTuple_GetItem(bases, i);
        if (!base) goto error;
#endif
#if CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(base);
#endif
        if (PyType_Check(base)) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        meth = __Pyx_PyObject_GetAttrStrNoError(base, __pyx_mstate_global->__pyx_n_u_mro_entries);
        if (!meth && PyErr_Occurred()) {
            goto error;
        }
        if (!meth) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        new_base = __Pyx_PyObject_CallOneArg(meth, bases);
        Py_DECREF(meth);
        if (!new_base) {
            goto error;
        }
        if (!PyTuple_Check(new_base)) {
            PyErr_SetString(PyExc_TypeError,
                            "__mro_entries__ must return a tuple");
            Py_DECREF(new_base);
            goto error;
        }
        if (!new_bases) {
            if (!(new_bases = PyList_New(i))) {
                goto error;
            }
            for (j = 0; j < i; j++) {
                PyObject *base_from_list;
#if CYTHON_ASSUME_SAFE_MACROS
                base_from_list = PyTuple_GET_ITEM(bases, j);
                PyList_SET_ITEM(new_bases, j, base_from_list);
                Py_INCREF(base_from_list);
#else
                base_from_list = PyTuple_GetItem(bases, j);
                if (!base_from_list) goto error;
                Py_INCREF(base_from_list);
                if (PyList_SetItem(new_bases, j, base_from_list) < 0) goto error;
#endif
            }
        }
#if CYTHON_ASSUME_SAFE_SIZE
        j = PyList_GET_SIZE(new_bases);
#else
        j = PyList_Size(new_bases);
        if (j < 0) goto error;
#endif
        if (PyList_SetSlice(new_bases, j, j, new_base) < 0) {
            goto error;
        }
        Py_DECREF(new_base);
    }
    if (!new_bases) {
        Py_INCREF(bases);
        return bases;
    }
    result = PyList_AsTuple(new_bases);
    Py_DECREF(new_bases);
#if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(base);
#endif
    return result;
error:
    Py_XDECREF(new_bases);
#if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(base);
#endif
    return NULL;
}

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases;
#if CYTHON_ASSUME_SAFE_SIZE
    nbases = PyTuple_GET_SIZE(bases);
#else
    nbases = PyTuple_Size(bases);
    if (nbases < 0) return NULL;
#endif
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
#if CYTHON_ASSUME_SAFE_MACROS
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *tmp = PyTuple_GetItem(bases, i);
        if (!tmp) return NULL;
#endif
        tmptype = Py_TYPE(tmp);
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
        metaclass = &PyType_Type;
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* PyObjectLookupSpecial (used by Py3ClassCreate) */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStrNoError(metaclass, __pyx_mstate_global->__pyx_n_u_prepare);
        if (prep) {
            PyObject *pargs[3] = {NULL, name, bases};
            ns = __Pyx_PyObject_FastCallDict(prep, pargs+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
            Py_DECREF(prep);
        } else {
            if (unlikely(PyErr_Occurred()))
                return NULL;
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result;
    PyObject *owned_metaclass = NULL;
    PyObject *margs[4] = {NULL, name, bases, dict};
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_mstate_global->__pyx_n_u_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    result = __Pyx_PyObject_FastCallDict(metaclass, margs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
    Py_XDECREF(owned_metaclass);
    return result;
}

/* GetNameInClass */
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name) {
    PyObject *result;
    PyObject *dict;
    assert(PyType_Check(nmspace));
#if CYTHON_USE_TYPE_SLOTS
    dict = ((PyTypeObject*)nmspace)->tp_dict;
    Py_XINCREF(dict);
#else
    dict = PyObject_GetAttr(nmspace, __pyx_mstate_global->__pyx_n_u_dict);
#endif
    if (likely(dict)) {
        result = PyObject_GetItem(dict, name);
        Py_DECREF(dict);
        if (result) {
            return result;
        }
    }
    PyErr_Clear();
    __Pyx_GetModuleGlobalNameUncached(result, name);
    return result;
}

/* SetNewInClass */
static int __Pyx_SetNewInClass(PyObject *ns, PyObject *name, PyObject *value) {
#ifdef __Pyx_CyFunction_USED
    int ret;
    if (__Pyx_CyFunction_Check(value)) {
        PyObject *staticnew;
#if !CYTHON_COMPILING_IN_LIMITED_API
        staticnew = PyStaticMethod_New(value);
#else
        PyObject *builtins, *staticmethod_str, *staticmethod;
        builtins = PyEval_GetBuiltins(); // borrowed
        if (!builtins) return -1;
        staticmethod_str = PyUnicode_FromStringAndSize("staticmethod", 12);
        if (!staticmethod_str) return -1;
        staticmethod = PyObject_GetItem(builtins, staticmethod_str);
        Py_DECREF(staticmethod_str);
        if (!staticmethod) return -1;
        staticnew = PyObject_CallFunctionObjArgs(staticmethod, value, NULL);
        Py_DECREF(staticmethod);
#endif
        if (unlikely(!staticnew)) return -1;
        ret = __Pyx_SetNameInClass(ns, name, staticnew);
        Py_DECREF(staticnew);
        return ret;
    }
#endif
    return __Pyx_SetNameInClass(ns, name, value);
}

/* CyFunctionClassCell */
static int __Pyx_CyFunction_InitClassCell(PyObject *cyfunctions, PyObject *classobj) {
    Py_ssize_t i, count = __Pyx_PyList_GET_SIZE(cyfunctions);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(count < 0)) return -1;
    #endif
    for (i = 0; i < count; i++) {
        __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *)
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && !CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
            PyList_GET_ITEM(cyfunctions, i);
#else
            __Pyx_PySequence_ITEM(cyfunctions, i);
        if (unlikely(!m))
            return -1;
#endif
        __Pyx_CyFunction_SetClassObj(m, classobj);
#if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && !CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
        Py_DECREF((PyObject*)m);
#endif
    }
    return 0;
}

/* CLineInTraceback (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
#define __Pyx_PyProbablyModule_GetDict(o) __Pyx_XNewRef(PyModule_GetDict(o))
#elif !CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyProbablyModule_GetDict(o) PyObject_GenericGetDict(o, NULL);
#else
PyObject* __Pyx_PyProbablyModule_GetDict(PyObject *o) {
    PyObject **dict_ptr = _PyObject_GetDictPtr(o);
    return dict_ptr ? __Pyx_XNewRef(*dict_ptr) : NULL;
}
#endif
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline = NULL;
    PyObject *ptype, *pvalue, *ptraceback;
    PyObject *cython_runtime_dict;
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
    cython_runtime_dict = __Pyx_PyProbablyModule_GetDict(__pyx_mstate_global->__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, cython_runtime_dict,
            __Pyx_PyDict_SetDefault(cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False))
    }
    if (use_cline == NULL || use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    Py_XDECREF(use_cline);
    Py_XDECREF(cython_runtime_dict);
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache (used by AddTraceback) */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
    __Pyx_CachedCodeObjectType* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = code_cache->entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__find_code_object;
    return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
    if (old_count < 0) {
        __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
        return NULL;
    }
#endif
    __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
#endif
    return result;
#endif
}
static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
{
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            code_cache->entries = entries;
            code_cache->max_count = 64;
            code_cache->count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
        __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_INCREF(code_object);
        Py_DECREF(tmp);
        return;
    }
    if (code_cache->count == code_cache->max_count) {
        int new_max = code_cache->max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        code_cache->entries = entries;
        code_cache->max_count = new_max;
    }
    for (i=code_cache->count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    code_cache->count++;
    Py_INCREF(code_object);
}
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__insert_code_object;
    return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type expected = 0;
    if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
        return;
    }
#endif
    __pyx__insert_code_object(code_cache, code_line, code_object);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
#endif
#endif
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!code_object) {
        code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
        if (unlikely(!code_object)) goto bad;
        py_py_line = PyLong_FromLong(py_line);
        if (unlikely(!py_py_line)) goto bad;
        if (c_line) {
            py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        } else {
            py_funcname = PyUnicode_FromString(funcname);
        }
        if (unlikely(!py_funcname)) goto bad;
        dict = PyDict_New();
        if (unlikely(!dict)) goto bad;
        {
            PyObject *old_code_object = code_object;
            code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
            Py_DECREF(old_code_object);
        }
        if (unlikely(!code_object)) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
    } else {
        dict = PyDict_New();
    }
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    if (c_line) {
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
    }
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE uint64_t __Pyx_PyLong_As_uint64_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        uint64_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (uint64_t) -1;
        val = __Pyx_PyLong_As_uint64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint64_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(uint64_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) >= 2 * PyLong_SHIFT)) {
                            return (uint64_t) (((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint64_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) >= 3 * PyLong_SHIFT)) {
                            return (uint64_t) (((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint64_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) >= 4 * PyLong_SHIFT)) {
                            return (uint64_t) (((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (uint64_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(uint64_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint64_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(uint64_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint64_t) (((uint64_t)-1)*(((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(uint64_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint64_t) ((((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint64_t) (((uint64_t)-1)*(((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint64_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint64_t) ((((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint64_t) (((uint64_t)-1)*(((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint64_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint64_t) ((((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(uint64_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint64_t, long, PyLong_AsLong(x))
        } else if ((sizeof(uint64_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        uint64_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (uint64_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (uint64_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (uint64_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (uint64_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(uint64_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((uint64_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(uint64_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((uint64_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((uint64_t) 1) << (sizeof(uint64_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (uint64_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint64_t");
    return (uint64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint64_t");
    return (uint64_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyLong_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 2 * PyLong_SHIFT)) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 3 * PyLong_SHIFT)) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 4 * PyLong_SHIFT)) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (size_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(size_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(size_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
        } else if ((sizeof(size_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        size_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (size_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (size_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (size_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (size_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(size_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((size_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(size_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((size_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((size_t) 1) << (sizeof(size_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (size_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE uint32_t __Pyx_PyLong_As_uint32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyLong_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(uint32_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT)) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint32_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT)) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint32_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT)) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (uint32_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(uint32_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(uint32_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(uint32_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(uint32_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(uint32_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
        } else if ((sizeof(uint32_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        uint32_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (uint32_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (uint32_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (uint32_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (uint32_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(uint32_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((uint32_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(uint32_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((uint32_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((uint32_t) 1) << (sizeof(uint32_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (uint32_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint32_t(uint32_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint32_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint32_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(uint32_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uint32_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE int64_t __Pyx_PyLong_As_int64_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int64_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int64_t) -1;
        val = __Pyx_PyLong_As_int64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int64_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int64_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) >= 2 * PyLong_SHIFT)) {
                            return (int64_t) (((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int64_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) >= 3 * PyLong_SHIFT)) {
                            return (int64_t) (((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int64_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) >= 4 * PyLong_SHIFT)) {
                            return (int64_t) (((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int64_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int64_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int64_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int64_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (int64_t) (((int64_t)-1)*(((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int64_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (int64_t) ((((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (int64_t) (((int64_t)-1)*(((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int64_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (int64_t) ((((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (int64_t) (((int64_t)-1)*(((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int64_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (int64_t) ((((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int64_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int64_t, long, PyLong_AsLong(x))
        } else if ((sizeof(int64_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int64_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int64_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int64_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int64_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int64_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int64_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int64_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int64_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int64_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int64_t) 1) << (sizeof(int64_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int64_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int64_t");
    return (int64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int64_t");
    return (int64_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint8_t(uint8_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint8_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint8_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint8_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(uint8_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uint8_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint64_t(uint64_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint64_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(uint64_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uint64_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int64_t(int64_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int64_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int64_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int64_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int64_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
static __Pyx_TypeName
__Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
{
    PyObject *module = NULL, *name = NULL, *result = NULL;
    #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
    name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_qualname);
    #else
    name = PyType_GetQualName(tp);
    #endif
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
    module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_module);
    if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
    if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
        result = name;
        name = NULL;
        goto done;
    }
    result = PyUnicode_FromFormat("%U.%U", module, name);
    if (unlikely(result == NULL)) goto bad;
  done:
    Py_XDECREF(name);
    Py_XDECREF(module);
    return result;
  bad:
    PyErr_Clear();
    if (name) {
        result = name;
        name = NULL;
    } else {
        result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__3);
    }
    goto done;
}
#endif

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyLong_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyLong_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* GetRuntimeVersion */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
void __Pyx_init_runtime_version(void) {
    if (__Pyx_cached_runtime_version == 0) {
        const char* rt_version = Py_GetVersion();
        unsigned long version = 0;
        unsigned long factor = 0x01000000UL;
        unsigned int digit = 0;
        int i = 0;
        while (factor) {
            while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                ++i;
            }
            version += factor * digit;
            if (rt_version[i] != '.')
                break;
            digit = 0;
            factor >>= 8;
            ++i;
        }
        __Pyx_cached_runtime_version = version;
    }
}
#endif
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    return Py_Version & ~0xFFUL;
#else
    return __Pyx_cached_runtime_version;
#endif
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* NewCodeObj */
#if CYTHON_COMPILING_IN_LIMITED_API
    static PyObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                       PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                       PyObject *fv, PyObject *cell, PyObject* fn,
                                       PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
#elif PY_VERSION_HEX >= 0x030B0000
  static PyCodeObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                         PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                         PyObject *fv, PyObject *cell, PyObject* fn,
                                         PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, __pyx_mstate_global->__pyx_empty_bytes);
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030c00A1
    if (likely(result))
        result->_co_firsttraceable = 0;
    #endif
    return result;
  }
#elif !CYTHON_COMPILING_IN_PYPY
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
) {
    PyObject *code_obj = NULL, *varnames_tuple_dedup = NULL, *code_bytes = NULL;
    Py_ssize_t var_count = (Py_ssize_t) descr.nlocals;
    PyObject *varnames_tuple = PyTuple_New(var_count);
    if (unlikely(!varnames_tuple)) return NULL;
    for (Py_ssize_t i=0; i < var_count; i++) {
        Py_INCREF(varnames[i]);
        if (__Pyx_PyTuple_SET_ITEM(varnames_tuple, i, varnames[i]) != (0)) goto done;
    }
    #if CYTHON_COMPILING_IN_LIMITED_API
    varnames_tuple_dedup = PyDict_GetItem(tuple_dedup_map, varnames_tuple);
    if (!varnames_tuple_dedup) {
        if (unlikely(PyDict_SetItem(tuple_dedup_map, varnames_tuple, varnames_tuple) < 0)) goto done;
        varnames_tuple_dedup = varnames_tuple;
    }
    #else
    varnames_tuple_dedup = PyDict_SetDefault(tuple_dedup_map, varnames_tuple, varnames_tuple);
    if (unlikely(!varnames_tuple_dedup)) goto done;
    #endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(varnames_tuple_dedup);
    #endif
    if (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table != NULL && !CYTHON_COMPILING_IN_GRAAL) {
        Py_ssize_t line_table_length = __Pyx_PyBytes_GET_SIZE(line_table);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(line_table_length == -1)) goto done;
        #endif
        Py_ssize_t code_len = (line_table_length * 2 + 4) & ~3LL;
        code_bytes = PyBytes_FromStringAndSize(NULL, code_len);
        if (unlikely(!code_bytes)) goto done;
        char* c_code_bytes = PyBytes_AsString(code_bytes);
        if (unlikely(!c_code_bytes)) goto done;
        memset(c_code_bytes, 0, (size_t) code_len);
    }
    code_obj = (PyObject*) __Pyx__PyCode_New(
        (int) descr.argcount,
        (int) descr.num_posonly_args,
        (int) descr.num_kwonly_args,
        (int) descr.nlocals,
        0,
        (int) descr.flags,
        code_bytes ? code_bytes : __pyx_mstate_global->__pyx_empty_bytes,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        varnames_tuple_dedup,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        filename,
        funcname,
        (int) descr.first_line,
        (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table) ? line_table : __pyx_mstate_global->__pyx_empty_bytes
    );
done:
    Py_XDECREF(code_bytes);
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(varnames_tuple_dedup);
    #endif
    Py_DECREF(varnames_tuple);
    return code_obj;
}

/* DecompressString */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo) {
    PyObject *module = NULL, *decompress, *compressed_bytes, *decompressed;
    const char* module_name = algo == 3 ? "compression.zstd" : algo == 2 ? "bz2" : "zlib";
    PyObject *methodname = PyUnicode_FromString("decompress");
    if (unlikely(!methodname)) return NULL;
    #if __PYX_LIMITED_VERSION_HEX >= 0x030e0000
    if (algo == 3) {
        PyObject *fromlist = Py_BuildValue("[O]", methodname);
        if (unlikely(!fromlist)) goto bad;
        module = PyImport_ImportModuleLevel("compression.zstd", NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
    } else
    #endif
        module = PyImport_ImportModule(module_name);
    if (unlikely(!module)) goto import_failed;
    decompress = PyObject_GetAttr(module, methodname);
    if (unlikely(!decompress)) goto import_failed;
    {
        #ifdef __cplusplus
            char *memview_bytes = const_cast<char*>(s);
        #else
            #if defined(__clang__)
              #pragma clang diagnostic push
              #pragma clang diagnostic ignored "-Wcast-qual"
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic push
              #pragma GCC diagnostic ignored "-Wcast-qual"
            #endif
            char *memview_bytes = (char*) s;
            #if defined(__clang__)
              #pragma clang diagnostic pop
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic pop
            #endif
        #endif
        #if CYTHON_COMPILING_IN_LIMITED_API && !defined(PyBUF_READ)
        int memview_flags = 0x100;
        #else
        int memview_flags = PyBUF_READ;
        #endif
        compressed_bytes = PyMemoryView_FromMemory(memview_bytes, length, memview_flags);
    }
    if (unlikely(!compressed_bytes)) {
        Py_DECREF(decompress);
        goto bad;
    }
    decompressed = PyObject_CallFunctionObjArgs(decompress, compressed_bytes, NULL);
    Py_DECREF(compressed_bytes);
    Py_DECREF(decompress);
    Py_DECREF(module);
    Py_DECREF(methodname);
    return decompressed;
import_failed:
    PyErr_Format(PyExc_ImportError,
        "Failed to import '%.20s.decompress' - cannot initialise module strings. "
        "String compression was configured with the C macro 'CYTHON_COMPRESS_STRINGS=%d'.",
        module_name, algo);
bad:
    Py_XDECREF(module);
    Py_DECREF(methodname);
    return NULL;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return __Pyx_NewRef(b ? Py_True: Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


/* MultiPhaseInitModuleState */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
#ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
#else
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
#endif
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
#error "Module state with PEP489 requires atomics. Currently that's one of\
 C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
#endif
#if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#define __Pyx_ModuleStateLookup_Lock()
#define __Pyx_ModuleStateLookup_Unlock()
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
#define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(__cplusplus) && __cplusplus >= 201103L
#include <mutex>
static std::mutex __Pyx_ModuleStateLookup_mutex;
#define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
#define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
#include <threads.h>
static mtx_t __Pyx_ModuleStateLookup_mutex;
static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
    mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
}
#define __Pyx_ModuleStateLookup_Lock()\
  call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
  mtx_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(HAVE_PTHREAD_H)
#include <pthread.h>
static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
#define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(_WIN32)
#include <Windows.h>  // synchapi.h on its own doesn't work
static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
#define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#else
#error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
 Requires C standard >= C11, or C++ standard >= C++11,\
 or pthreads, or the Windows 32 API, or Python >= 3.13."
#endif
typedef struct {
    int64_t id;
    PyObject *module;
} __Pyx_InterpreterIdAndModule;
typedef struct {
    char interpreter_id_as_index;
    Py_ssize_t count;
    Py_ssize_t allocated;
    __Pyx_InterpreterIdAndModule table[1];
} __Pyx_ModuleStateLookupData;
#define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
#else
static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
#endif
static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
        __Pyx_InterpreterIdAndModule* table,
        Py_ssize_t count,
        int64_t interpreterId) {
    __Pyx_InterpreterIdAndModule* begin = table;
    __Pyx_InterpreterIdAndModule* end = begin + count;
    if (begin->id == interpreterId) {
        return begin;
    }
    while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
        __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
        if (halfway->id == interpreterId) {
            return halfway;
        }
        if (halfway->id < interpreterId) {
            begin = halfway;
        } else {
            end = halfway;
        }
    }
    for (; begin < end; ++begin) {
        if (begin->id >= interpreterId) return begin;
    }
    return begin;
}
static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return NULL;
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
    {
        __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        if (likely(data)) {
            __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
            if (likely(data == new_data)) {
                goto read_finished;
            }
        }
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        __Pyx_ModuleStateLookup_Lock();
        __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
        data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
        __Pyx_ModuleStateLookup_Unlock();
    }
  read_finished:;
#else
    __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_InterpreterIdAndModule* found = NULL;
    if (unlikely(!data)) goto end;
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            found = data->table+interpreter_id;
        }
    } else {
        found = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
    }
  end:
    {
        PyObject *result=NULL;
        if (found && found->id == interpreter_id) {
            result = found->module;
        }
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
#endif
        return result;
    }
}
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
    while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
}
#else
#define __Pyx_ModuleStateLookup_wait_until_no_readers()
#endif
static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
    Py_ssize_t to_allocate = (*old_data)->allocated;
    while (to_allocate <= interpreter_id) {
        if (to_allocate == 0) to_allocate = 1;
        else to_allocate *= 2;
    }
    __Pyx_ModuleStateLookupData *new_data = *old_data;
    if (to_allocate != (*old_data)->allocated) {
         new_data = (__Pyx_ModuleStateLookupData *)realloc(
            *old_data,
            sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
        if (!new_data) {
            PyErr_NoMemory();
            return -1;
        }
        for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
            new_data->table[i].id = i;
            new_data->table[i].module = NULL;
        }
        new_data->allocated = to_allocate;
    }
    new_data->table[interpreter_id].module = module;
    if (new_data->count < interpreter_id+1) {
        new_data->count = interpreter_id+1;
    }
    *old_data = new_data;
    return 0;
}
static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
    __Pyx_InterpreterIdAndModule *read = data->table;
    __Pyx_InterpreterIdAndModule *write = data->table;
    __Pyx_InterpreterIdAndModule *end = read + data->count;
    for (; read<end; ++read) {
        if (read->module) {
            write->id = read->id;
            write->module = read->module;
            ++write;
        }
    }
    data->count = write - data->table;
    for (; write<end; ++write) {
        write->id = 0;
        write->module = NULL;
    }
    data->interpreter_id_as_index = 0;
}
static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    int result = 0;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_ModuleStateLookupData *new_data = old_data;
    if (!new_data) {
        new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
        if (!new_data) {
            result = -1;
            PyErr_NoMemory();
            goto end;
        }
        new_data->allocated = 1;
        new_data->interpreter_id_as_index = 1;
    }
    __Pyx_ModuleStateLookup_wait_until_no_readers();
    if (new_data->interpreter_id_as_index) {
        if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
            result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
            goto end;
        }
        __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
    }
    {
        Py_ssize_t insert_at = 0;
        {
            __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                new_data->table, new_data->count, interpreter_id);
            assert(lower_bound);
            insert_at = lower_bound - new_data->table;
            if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                lower_bound->module = module;
                goto end;  // already in table, nothing more to do
            }
        }
        if (new_data->count+1 >= new_data->allocated) {
            Py_ssize_t to_allocate = (new_data->count+1)*2;
            new_data =
                (__Pyx_ModuleStateLookupData*)realloc(
                    new_data,
                    sizeof(__Pyx_ModuleStateLookupData) +
                    (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
            if (!new_data) {
                result = -1;
                new_data = old_data;
                PyErr_NoMemory();
                goto end;
            }
            new_data->allocated = to_allocate;
        }
        ++new_data->count;
        int64_t last_id = interpreter_id;
        PyObject *last_module = module;
        for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
            int64_t current_id = new_data->table[i].id;
            new_data->table[i].id = last_id;
            last_id = current_id;
            PyObject *current_module = new_data->table[i].module;
            new_data->table[i].module = last_module;
            last_module = current_module;
        }
    }
  end:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
#else
    __Pyx_ModuleStateLookup_data = new_data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return result;
}
static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
#endif
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            data->table[interpreter_id].module = NULL;
        }
        goto done;
    }
    {
        __Pyx_ModuleStateLookup_wait_until_no_readers();
        __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
        if (!lower_bound) goto done;
        if (lower_bound->id != interpreter_id) goto done;
        __Pyx_InterpreterIdAndModule *end = data->table+data->count;
        for (;lower_bound<end-1; ++lower_bound) {
            lower_bound->id = (lower_bound+1)->id;
            lower_bound->module = (lower_bound+1)->module;
        }
    }
    --data->count;
    if (data->count == 0) {
        free(data);
        data = NULL;
    }
  done:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
#else
    __Pyx_ModuleStateLookup_data = data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return 0;
}
#endif

/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
